/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003-2009    (see file CONTACT for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/**
   @example ExampleConstraints.cpp
   @relates Constraints
   @brief Example for derivation of templates classes
   @ingroup g_examples
   @author MIP
*/


#include <Base/Common/Constraints.hh>

using namespace BIAS;
/** \cond POORLY_NAMED_EXAMPLE_CLASSES **/
/* base */
class Base { 
public:
  Base() {}
}; 

/* nobase */
class NonBase { 
public:
  NonBase() {}
}; 

/* frombase */
class FromBase : public Base { 
public:
  FromBase() {}
}; 

// only Ts derived from Base are allowed,
// the check is done at compile time and nopt at runtime
/* templated */
template<class T>
class MyTemplate
  : public DerivedFrom<T, Base> // accept only Ts derived from  Base
{ 
public:
  MyTemplate() {}
}; 
/** \endcond */

int main()
{ 
  //MyTemplate<NonBase> nonBase2;   // compile time error, 
  MyTemplate<Base>    base2;  // no error
  MyTemplate<FromBase> frombase2; // no error
  
  // success, second template argument type is the same as the first 
  // template argument type, but const
  SameButConst<int, const int> t;
  // this is also allowed
  SameButConst<int, int> t0;
  // compiler error, const in wrong position
  //SameButConst<const int, int> t2;
  // compiler error, different types
  //SameButConst<unsigned, const int> t3;
  

  return 0;
}
