#include "ProgressSemaphorePool.hh"
#include "ProgressSemaphore.hh"

using namespace BIAS;
using namespace std;

ProgressSemaphorePool::ProgressSemaphorePool()
{
}

ProgressSemaphorePool::~ProgressSemaphorePool()
{
}


 BIAS::SharedPtr<ProgressSemaphore> ProgressSemaphorePool::
Register(const std::string &name)
{
  SharedPtr<ProgressSemaphore> sem;
  map<string,  SharedPtr<ProgressSemaphore> >::iterator it;
  it = Semaphores_.find(name);
  if (it != Semaphores_.end())
    sem = it->second;
  else {
    sem = SharedPtr<ProgressSemaphore>(new ProgressSemaphore);
    Semaphores_[name]=sem;
  }
  return sem;
}



void ProgressSemaphorePool::
GetAllSemaphoreNames(std::vector<std::string> &names)
{
  names.clear();
  map<string,  SharedPtr<ProgressSemaphore> >::iterator it;
  it = Semaphores_.begin();
  while (it!=Semaphores_.end()) {
    names.push_back(it->first);
    it++;
  }
}


void ProgressSemaphorePool::Clear()
{
  Semaphores_.clear();
}

void ProgressSemaphorePool::
TerminateAll()
{
  map<string,  SharedPtr<ProgressSemaphore> >::iterator it;
  it = Semaphores_.begin();
  while (it!=Semaphores_.end()) {
    it->second->SetTerminateFlag();
    it++;
  }
}

float ProgressSemaphorePool::
GetOverallProgress()
{
  unsigned int count=0;
  float accu=0.0f;
  map<string,  SharedPtr<ProgressSemaphore> >::iterator it;
  it = Semaphores_.begin();
  while (it!=Semaphores_.end()) {
    float prog = it->second->GetProgress();
    BIASASSERT(prog>=0. && prog<=1.);
    accu += prog;
    count ++;
    it++;
  }
  float result = accu/float(count);
#ifdef BIAS_DEBUG
  BIASASSERT(result>=0. && result <=1.);
#endif
  return result;
}


bool ProgressSemaphorePool::
Exists(const string &name)
{
  map<string,  SharedPtr<ProgressSemaphore> >::iterator it;
  it = Semaphores_.find(name);
  return it != Semaphores_.end();

}
