#ifndef __ProgressSemaphorePool_hh__
#define __ProgressSemaphorePool_hh__

// includes
#include <bias_config.h>

#include <Base/Common/Singleton.hh>
#include <Base/Common/SharedPtr.hh>
#include <map>
#include <vector>

namespace BIAS {
  

  class BIASCommon_EXPORT ProgressSemaphore;

  /** @class ProgressSemaphorePool
      @brief
      @author evers 03/2008 */
  class BIASCommon_EXPORT ProgressSemaphorePool: public Singleton<ProgressSemaphorePool>
  {
  public:

    /// destructor
    ~ProgressSemaphorePool();

    // register to use a named semaphore. A new one is created if
    // necessary. Use the returned pointer to use the semaphore If the
    // pool is cleared (Clear()), the SharedPtr remains valid but
    // useless.
    SharedPtr<ProgressSemaphore> Register(const std::string &name);

    void GetAllSemaphoreNames(std::vector<std::string> &names);

    // set Terminate flag for all contained ProgressSemaphores
    void TerminateAll();

    // accumulate progress over all contained ProgressSemaphores,
    // normalized to bw within standard intervall [0,1]
    float GetOverallProgress();

    // The application can clear the pool when ever appropriate.
    void Clear();

    // ask for the existance of a special ProgressSemaphore
    bool Exists(const std::string &name);

    friend class Singleton<ProgressSemaphorePool>;
protected:
    ProgressSemaphorePool();
//     ProgressSemaphorePool(const &ProgressSemaphorePool);
//     ProgressSemaphorePool& operator=(const &ProgressSemaphorePool);


    std::map<std::string, SharedPtr<ProgressSemaphore> > Semaphores_;

    
  }; // class ProgressSemaphorePool

} // namespace

#endif // __ProgressSemaphorePool_hh__
