/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003-2009    (see file CONTACT for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#ifndef __DualQuaternionOperators_hh__
#define __DualQuaternionOperators_hh__

namespace BIAS {

  // forward declarations to avoid nested includes
  template<class T> class Quaternion;
  template<class T> class DualQuaternion;
  
  ////////////////////////////////////////////////////////////////////////////
  //      dual quaternion operators
  ////////////////////////////////////////////////////////////////////////////

  template<class T>
  inline DualQuaternion<T> operator + (const DualQuaternion<T>& l,
                                       const DualQuaternion<T>& r)
  {
    DualQuaternion<T> d(l);
    d.Add(r);
    return d;
  }

  template<class T>
  inline DualQuaternion<T> operator - (const DualQuaternion<T>& l,
                                       const DualQuaternion<T>& r)
  {
    DualQuaternion<T> d(l);
    d.Sub(r);
    return d;
  }

  template<class T>
  inline DualQuaternion<T> operator * (const DualQuaternion<T>& l,
                                       const T& scalar)
  {
    DualQuaternion<T> d(l);
    d.Mult(scalar);
    return d;
  }

  template<class T>
  inline DualQuaternion<T> operator / (const DualQuaternion<T>& l,
                                       const T& scalar)
  {
    DualQuaternion<T> d(l);
    d.Div(scalar);
    return d;
  }

  /** @relates DualQuaternion
      what it stands for (wraps Mult())
      @author esquivel 02/2011 */
  template<class T>
  inline DualQuaternion<T> operator * (const DualQuaternion<T>& l,
                                       const DualQuaternion<T>& r)
  { 
    DualQuaternion<T> d(l);
    d.Mult(r);
    return d;
  }
    
  /** @relates DualQuaternion
      what it stands for (wraps Mult() and Inverse())
      @author esquivel 02/2011 */
  template<class T>
  inline DualQuaternion<T> operator / (const DualQuaternion<T>& l,
                                       const DualQuaternion<T>& r)
  { 
    DualQuaternion<T> d(l);
    d.Mult(r.Inverse());
    return d;
  }
  
}

#endif // __DualQuaternionOperators_hh__
