/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003-2009    (see file CONTACT for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#ifndef _BIAS_HMatrixBase_hh_
#define _BIAS_HMatrixBase_hh_
#include "bias_config.h" 


#include <Base/Debug/Error.hh>
#include <Base/Debug/Debug.hh>
#include <Base/Math/Matrix3x3.hh>

#include "HomgPoint2D.hh"

#define HMATRIX_TYPE double

#define D_HM_COMP 0x0001
namespace BIAS {

/**
    @class HMatrixBase 
    @ingroup g_geometry
    @brief describes a homography 
    
    A HMatrixBase describes a homography (mapping between two planes) between 
    homogenous 2D coordinates by a 3x3 Matrix.
    It is derived from Matrix3x3.
    Only implementation very specific to HMatrixBase (and not valid for general
    3x3 matrices, e.g. like decomposition) 
    should be implemented here. 
    @author Jan Woetzel
    @status alpha (04/18/2002) now a direct child of Matrix3x3 
    (and no more of Matrix<HMATRIX_TYPE>)
    alpha (02/26/2002)**/

  class BIASGeometryBase_EXPORT HMatrixBase : public Matrix3x3<HMATRIX_TYPE> 
  {
  public:
    /**	default constructor
        @status alpha (02/25/2002) , tested in part **/
    HMatrixBase() : Matrix3x3<HMATRIX_TYPE>() {};

    // DEPRECATED due to instantiation problem JW
    //explicit HMatrixBase(const char *s) : Matrix3x3<HMATRIX_TYPE>(s) {};

    /**	constructor setting identity or zero
        @brief author koeser **/
    explicit HMatrixBase(const MatrixInitType& i) : 
      Matrix3x3<HMATRIX_TYPE>(i) {};

    /** @author Jan Woetzel
        @status untested (02/25/2002) **/
    HMatrixBase(const HMatrixBase& A) : Matrix3x3<HMATRIX_TYPE>(A) {};

    HMatrixBase(const Matrix3x3<HMATRIX_TYPE>& A) : Matrix3x3<HMATRIX_TYPE>(A) 
    {};

    HMatrixBase(const Matrix<HMATRIX_TYPE>& A) : Matrix3x3<HMATRIX_TYPE>(A) {};

    ~HMatrixBase();

    /** assignment operators calls corresponding operator from parent
        @author Felix Woelk
        @status alpha (10/28/2002) **/
    inline HMatrixBase& operator= (const HMatrixBase& mat) {
      Matrix3x3<HMATRIX_TYPE>::operator= (mat); 
      return *this;
    };

  }; // class HMatrixBase
} // end of namespace BIAS
#endif // _BIAS_HMatrixBase_hh_

