/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003-2009    (see file CONTACT for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#ifndef __QuaternionOperators_hh__
#define __QuaternionOperators_hh__

namespace BIAS {

  // forward declarations to avoid nested includes
  template<class T> class Quaternion;
  
  ////////////////////////////////////////////////////////////////////////////
  //      quaternion operators
  ////////////////////////////////////////////////////////////////////////////
  
  /** @relates Quaternion
      what it stands for (wraps Mult())
      @author herzog 2005-07-15 */
  template<class T>
  inline Quaternion<T> operator * (const Quaternion<T>& l,
                                   const Quaternion<T>& r)
  { 
    Quaternion<T> q = l;
    q.Mult(r);
    return q;
  }
    
  /** @relates Quaternion
      what it stands for (wraps Mult())
      @author herzog 2005-07-15 */
  template<class T>
  inline Quaternion<T>& operator *= (Quaternion<T>& l,
                                     const Quaternion<T>& r)
  { 
    l.Mult(r); 
    return l; 
  }
  
  /** @relates Quaternion
      what it stands for (wraps Mult() and Inverse())
      @author herzog 2005-07-15 */
  template<class T>
  inline Quaternion<T> operator / (const Quaternion<T>& l,
                                   const Quaternion<T>& r)
  { 
    Quaternion<T> q = l;
    q.Mult(r.Inverse());
    return q;
  }
  
  /** @relates Quaternion
      what it stands for (wraps Mult() and Inverse())
      @author herzog 2005-07-15 */
  template<class T>
  inline Quaternion<T>& operator /= (Quaternion<T>& l,
                                     const Quaternion<T>& r)
  { 
    l.Mult(r.Inverse()); 
    return l; 
  }
  
} // namespace

#endif // __QuaternionOperators_hh__
