/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003-2009    (see file CONTACT for details)
Multimediale Systeme der Informationsverarbeitung
Institut fuer Informatik
Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __ColourRGB_hh__
#define __ColourRGB_hh__
#include <Base/Debug/Error.hh>
namespace BIAS {

  /** @class ColourRGB
      @brief interface class used to ease handover in function calls 
      @author woelk 09/2007 */
  template <class StorageType>
  class ColourRGB
  {
  public:
    ColourRGB() {};
    
    ColourRGB(const ColourRGB& col)
    { *this = col; }

    explicit ColourRGB(const StorageType& red, const StorageType& green, 
                       const StorageType& blue) 
    { Data_[0]=red; Data_[1]=green; Data_[2]=blue; }
    
    explicit ColourRGB(const StorageType col[3])
    { Data_[0]=col[0]; Data_[1]=col[1]; Data_[2]=col[2]; }

    ~ColourRGB() {}

    StorageType& operator[](const int i) 
    { BIASASSERT(i>=0&&i<3); return Data_[i]; }
    
    const StorageType& operator[](const int i) const
    { BIASASSERT(i>=0&&i<3); return Data_[i]; }

    ColourRGB& operator=(const ColourRGB& col)
    { Data_[0]=col.Data_[0]; Data_[1]=col.Data_[1]; 
      Data_[2]=col.Data_[2]; return *this; }
 
//     StorageType *GetData()
//     { BIASERR("deprectade, do not use"); return Data_; }

    const StorageType *GetData() const
    {  return Data_; }
    
    enum PreDefinedColors {
      COLOR_WHITE=0,
      COLOR_RED,
      COLOR_GREEN,
      COLOR_BLUE,
      COLOR_YELLOW,
      COLOR_MAGENTA,
      COLOR_TURQUOISE,
      COLOR_BLACK,
      COLOR_GREY10,
      COLOR_GREY20,
      COLOR_GREY30,
      COLOR_GREY40,
      COLOR_GREY50,
      COLOR_GREY60,
      COLOR_GREY70,
      COLOR_GREY80,
      COLOR_GREY90
    };
    
    static ColourRGB PreDefinedColor(int colorIdx) {
      switch (colorIdx) {
        case COLOR_WHITE:
          return ColourRGB((StorageType)255,(StorageType)255,(StorageType)255);
          break;
        case COLOR_RED:
          return ColourRGB((StorageType)255,(StorageType)0,(StorageType)0);
          break;
        case COLOR_GREEN:
          return ColourRGB((StorageType)0,(StorageType)255,(StorageType)0);
          break;
        case COLOR_BLUE:
          return ColourRGB((StorageType)0,(StorageType)0,(StorageType)255);
          break;
        case COLOR_YELLOW:
          return ColourRGB((StorageType)255,(StorageType)255,(StorageType)0);
          break;
        case COLOR_MAGENTA:
          return ColourRGB((StorageType)255,(StorageType)0,(StorageType)255);
          break;
        case COLOR_TURQUOISE:
          return ColourRGB((StorageType)0,(StorageType)255,(StorageType)255);
          break;
        case COLOR_BLACK:
          return ColourRGB((StorageType)0,(StorageType)0,(StorageType)0);
          break;
        case COLOR_GREY10:
          return ColourRGB((StorageType)25.5,(StorageType)25.5,(StorageType)25.5);
          break;
        case COLOR_GREY20:
          return ColourRGB((StorageType)51,(StorageType)51,(StorageType)51);
          break;
        case COLOR_GREY30:
          return ColourRGB((StorageType)76.5,(StorageType)76.5,(StorageType)76.5);
          break;
        case COLOR_GREY40:
          return ColourRGB((StorageType)102,(StorageType)102,(StorageType)102);
          break;
        case COLOR_GREY50:
          return ColourRGB((StorageType)127.5,(StorageType)127.5,(StorageType)127.5);
          break;
        case COLOR_GREY60:
          return ColourRGB((StorageType)153,(StorageType)153,(StorageType)153);
          break;
        case COLOR_GREY70:
          return ColourRGB((StorageType)178.5,(StorageType)178.5,(StorageType)178.5);
          break;
        case COLOR_GREY80:
          return ColourRGB((StorageType)204,(StorageType)204,(StorageType)204);
          break;
        case COLOR_GREY90:
          return ColourRGB((StorageType)229.5,(StorageType)229.5,(StorageType)229.5);
          break;
        default:
          BIASWARN("undef color");
          return ColourRGB((StorageType)255,(StorageType)255,(StorageType)255);
          break;
      }
    }
  protected:
    StorageType Data_[3];
  };

#define COLOR_RED    BIAS::ColourRGB<unsigned char>(255,0,0)
#define COLOR_GREEN  BIAS::ColourRGB<unsigned char>(0,255,0)
#define COLOR_BLUE   BIAS::ColourRGB<unsigned char>(0,0,255)
#define COLOR_WHITE  BIAS::ColourRGB<unsigned char>(255,255,255)
#define COLOR_GREY50 BIAS::ColourRGB<unsigned char>(128, 128, 128)
#define COLOR_BLACK  BIAS::ColourRGB<unsigned char>(0,0,0)
#define COLOR_MAGENTA BIAS::ColourRGB<unsigned char>(255, 0, 255)
#define COLOR_CYAN BIAS::ColourRGB<unsigned char>(255, 255, 0)
#define COLOR_YELLOW BIAS::ColourRGB<unsigned char>(0, 255, 255)
  
} // namespace

#endif // __ColourRGB_hh__
