/* 
 This file is part of the BIAS library (Basic ImageAlgorithmS).

 Copyright (C) 2003-2009    (see file CONTACT for details)
 Multimediale Systeme der Informationsverarbeitung
 Institut fuer Informatik
 Christian-Albrechts-Universitaet Kiel


 BIAS is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 (at your option) any later version.

 BIAS is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with BIAS; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/**
   @example ExampleHSVConversion2.cpp
   @relates ExampleHSVConversion2
   @brief Second example converting an image to/from HSV colormodel
   @ingroup g_examples
   @author MIP  
*/

// must be first:
//#include <Base/Common/LeakChecking.h> 

#include <iostream>

#include <Base/Image/ImageBase.hh>
#include <Base/Image/ImageConvert.hh>
#include <Base/Image/ImageIO.hh>

using namespace BIAS;
using namespace std;

int main(int argc, char *argv[]) {

  ImageBase im;
  if (argc<2) {
    cerr << argv[0] << " src image \n";
    return -2;
  }

  if (ImageIO::Load(argv[1], im)!=0) {
    BIASERR("error loading image "<<argv[1]);
    return -1;
  } else {
    cerr << "read "<<argv[1]<<endl;
  }

  int sizeX = im.GetWidth();
  int sizeY = im.GetHeight();

  Image<unsigned char> imageHSV(sizeX, sizeY, 3);

  ImageConvert::Convert(im, imageHSV, ImageBase::CM_HSV);

  Image<unsigned char> imgH(sizeX, sizeY, 1);
  Image<unsigned char> imgS(sizeX, sizeY, 1);
  Image<unsigned char> imgV(sizeX, sizeY, 1);

  for (int x=0; x<sizeX; x++) {
    for (int y=0; y<sizeY; y++) {
      imgH.SetPixel(imageHSV.PixelValue(x, y, 0), x, y);
      imgS.SetPixel(imageHSV.PixelValue(x, y, 1), x, y);
      imgV.SetPixel(imageHSV.PixelValue(x, y, 2), x, y);
    }
  }

  //if (ImageIO::Save("Hue.mip", imgH)!=0) {
  if (ImageIO::Save("Hue.mip", imgH)!=0) {
    BIASERR("error writing image ");
    return -2;
  }
  //if (ImageIO::Save("Satuaration.mip", imgS)!=0) {
  if (ImageIO::Save("Satuaration.mip", imgS)!=0) {
    BIASERR("error writing image ");
    return -2;
  }
  //if (ImageIO::Save("Value.mip", imgV)!=0) {
  if (ImageIO::Save("Value.mip", imgV)!=0) {
    BIASERR("error writing image ");
    return -2;
  }

  cerr<<"image data written to file"<<endl;
  return 0;
}
