/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003-2009    (see file CONTACT for details)
Multimediale Systeme der Informationsverarbeitung
Institut fuer Informatik
Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


/**
	 @example ExampleImageMagickIO.cpp
	 @relates ImageIO
	 @brief How to use the ImageMagick IO routines directly instead of
	 all vailable routines from BIAS::ImageIO::Load
	 @ingroup g_examples
	 @author Jan Woetzel
*/

#include <iostream>

#include <Base/Image/ImageBase.hh>
#include <Base/Image/Image.hh>
#include <Base/Image/ImageIO.hh>

#ifndef BIAS_HAVE_IMAGEMAGICKLIB
#  error BIAS_HAVE_IMAGEMAGICKLIB not defined. Recompile BIAS with ImageMagick
#endif


using namespace std;
using namespace BIAS;

void usage(int argc, char** argv)
{
  cerr<<"usage: ";
  if (argc>0) cerr<<argv[0]<<" <image-filename>"<<endl;
}

int main(int argc, char** argv)
{ 
  if (argc!=2) {
    usage(argc, argv);
    return 0;
  };
  
  BIAS::Image<unsigned char> img;
  BIASASSERT(argc>1);
  string filename=argv[1];
  
  // BIAS::ImageIO::ImportImage(filename, img);
  BIAS::ImageIO::ImportMagickPPAutoconvert(filename, img);
  
  BIAS::ImageIO::ExportMagickPP("out_Magick.png", img);
  BIAS::ImageIO::ExportMagickPP("out_Magick_default.jpg", img);
  // BIAS::ImageIO::WriteMagickPP("out_Magick005.jpg", img, 5);
  // BIAS::ImageIO::WriteMagickPP("out_Magick100.jpg", img, 100);
  
  if (BIAS::ImageIO::Save("out_Magick_ExportImage_FF_png.png", img, BIAS::ImageIO::FF_png ) !=BIAS_IO_SUCCESS){
    BIASERR("an error occurred using ExportImage");
  };
  
  //if (BIAS::ImageIO::Save("out_Magick_Write.mip", img ) !=BIAS_IO_SUCCESS){
  if (BIAS::ImageIO::Save("out_Magick_Write.mip", img ) !=BIAS_IO_SUCCESS){
    BIASERR("an error occurred writing mip");
  };  
  
  cout<<"done"<<endl;
  return 0;
}
