/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003-2009    (see file CONTACT for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/**
   @example TestROI
   @ingroup g_examples
   @brief Roi testing
   @author MIP
*/

#include <fstream>
#include <Base/Image/ROI.hh>

using namespace BIAS;
using namespace std;

void WriteRead(const ROI src, ROI& dst);

void TestIO()
{
  ROI src, dst;
  const unsigned w=100, h=75;
  src.Resize(w, h);
  if (src.GetWidth()!=w || src.GetHeight()!=h) { BIASABORT; }

  // corners
  const unsigned ulx=10, uly=10, lrx=50, lry=50;
  src.SetCorners(ulx, uly, lrx, lry);
  WriteRead(src, dst);
  unsigned mulx, muly, mlrx, mlry;
  dst.GetCorners(mulx, muly, mlrx, mlry);
  if (ulx!=mulx || uly!=muly || lrx!=mlrx || lry!=mlry){ BIASABORT; }

  // rows
  vector<unsigned> start(h, 0), end(h, w);
  start[42] = 42; 
  end[42] = 84;
  src.SetRows(start, end);
  WriteRead(src, dst);
  vector<unsigned> mstart, mend;
  //cout << "src: "<<src << endl;
  //cout << "dst: "<<dst << endl;
  if (!dst.GetRows(mstart, mend)) { BIASABORT; }
  if (mstart.size()!=h || mend.size()!=h) { BIASABORT; }
  for (unsigned y=0; y<h; y++){
    if (mstart[y] != start[y]) { BIASABORT; }
    if (mend[y] != end[y]) { BIASABORT; }
  }

  // points
  vector<Position> pos(2);
  pos[0] = Position(42, 42);
  pos[1] = Position(42, 43);
  src.SetVector(pos);
  WriteRead(src, dst);
  vector<Position> dst_pos;
  if (!dst.GetVector(dst_pos)) { BIASABORT; }
  if (dst_pos.size()!=2) { BIASABORT; }
  for (int i=0; i<2; i++){
    if (dst_pos[i].x!=pos[i].x) { BIASABORT; }
    if (dst_pos[i].y!=pos[i].y) { BIASABORT; }
  }

  // mask
  src.SetMask(42, 42, false);
  WriteRead(src, dst);
  if (dst.Mask(42, 42)!=false) { BIASABORT; }
  if (dst.Mask(10, 10)!=true) { BIASABORT; }
  
}


int main(int argc, char *argv[])
{
  TestIO();
  return 0;
}


void WriteRead(const ROI src, ROI& dst)
{
  const string file("roi.bin");
  ofstream of(file.c_str());
  if (src.WriteBinary(of)!=0){ BIASABORT; }
  of.close();
  ifstream is(file.c_str());
  if (dst.ReadBinary(is)!=0){ BIASABORT; }
  is.close();
}
