/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003-2009    (see file CONTACT for details)
Multimediale Systeme der Informationsverarbeitung
Institut fuer Informatik
Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef _ImageConvertThreaded_hh_
#define _ImageConvertThreaded_hh_

#include <bias_config.h>
#ifndef BIAS_HAVE_PTHREADS
#  error You need pthreads. please recompile BIAS with USE_PTHREADS.
#endif // BIAS_HAVE_PTHREADS

#include "ImageConvert.hh"

namespace BIAS {
	/** \class ImageConvertThreaded
	 *  \brief Converts images between different color models using multiple threads */
  class BIASImageBase_EXPORT ImageConvertThreaded 
    : public ImageConvert
  {
  public:
    ImageConvertThreaded();
    ~ImageConvertThreaded();
    /** \brief Convert using num_threads threads */
    static int ConvertThreaded(BIAS::ImageBase &source, BIAS::ImageBase &dest, 
                               enum BIAS::ImageBase::EColorModel targetColorModel,
                               bool bPlanar=false, const int num_threads = 2);

  protected:
    // splits source image into num non overlapping destination images
    // only num=2 is supported
    static void SplitImage_(const ImageBase &source, const int num,
                            std::vector<ImageBase> &parts);

    static void MergeImage_(const std::vector<ImageBase>& parts,
                            ImageBase& dest);

    static void *ConvertThread_(void *arg);

  };


} // namespace


#endif // _ImageConvertThreaded_hh_
