/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003-2009    (see file CONTACT for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/**
   @example ExampleHSVConversion.cpp
   @relates ImageConvert
   @brief Example converting an image to/from HSV colormodel
   @ingroup g_examples
   @author MIP  
*/

// must be first:
//#include <Base/Common/LeakChecking.h> 


#include <iostream>

#include <Base/Image/ImageBase.hh>
#include <Base/Image/ImageConvert.hh>
#include <Base/Image/ImageIO.hh>

using namespace BIAS;
using namespace std;

int main()
{
  unsigned int size=800;
  Image<unsigned char> im(size, size/4, 3);
  im.SetColorModel(ImageBase::CM_RGB);
  unsigned int block_field_size=size/4;
  unsigned int c=0, r=0;
  unsigned char **ida=im.GetImageDataArray();
  cerr<<"R=255, G=0, B=0-255  |  ";
  float blue=0;
  for (c=0;c<block_field_size;c++) {
    for (r=0; r< (block_field_size / 2) ; r++) {
      ida[r][c*3 + 0]= 255;
      ida[r][c*3 + 1]= 0;
      ida[r][c*3 + 2]= (unsigned char)blue;
    }
    blue+=255.0f / (float)block_field_size;
  }

  cerr<<"R=255, G=0-255, B=255 |  ";
  float green=0;
  for (c=block_field_size+1;c<(2*block_field_size);c++) {
    for (r=0; r< (block_field_size / 2) ; r++) {
      ida[r][c*3 + 0]= 255;
      ida[r][c*3 + 1]= (unsigned char)green;
      ida[r][c*3 + 2]= 255;
    }
    green+=255.0f / (float)block_field_size;
  }

  cerr<<"R=255, G=255, B=255-0 |  ";
  blue=255;
  for (c=2*block_field_size+1;c<(3*block_field_size);c++) {
    for (r=0; r< (block_field_size / 2) ; r++) {
      ida[r][c*3 + 0]= 255;
      ida[r][c*3 + 1]= 255;
      ida[r][c*3 + 2]= (unsigned char)blue;
    }
    blue-=255.0f / (float)block_field_size;
  }

  cerr<<"R=255, G=255, B=255-0"<<endl;
  green=255;
  for (c=3*block_field_size+1; c<size;c++) {
    for (r=0; r< (block_field_size / 2) ; r++) {
      ida[r][c*3 + 0]= 255;
      ida[r][c*3 + 1]= (unsigned char)green;
      ida[r][c*3 + 2]= 0;
    }
    green-=255.0f / (float)block_field_size;
  }
  cerr<<"--------------------------------------------------------------"<<endl;
  cerr<<"R=0, G=0-255, B=255  |  ";
  green=0;
  for (c=0;c<block_field_size;c++) {
    for (r=block_field_size / 2 +1; r< block_field_size ; r++) {
      ida[r][c*3 + 0]= 0;
      ida[r][c*3 + 1]= (unsigned char)green;
      ida[r][c*3 + 2]= 255;
    }
    green+=255.0f / (float)block_field_size;
  }

  cerr<<"R=0, G=255, B=255-0   |  ";
  blue=0;
  for (c=block_field_size+1;c<(2*block_field_size);c++) {
    for (r=block_field_size / 2 +1; r< block_field_size ; r++) {
      ida[r][c*3 + 0]= 0;
      ida[r][c*3 + 1]= 255;
      ida[r][c*3 + 2]= (unsigned char)blue;
    }
    blue-=255.0f / (float)block_field_size;
  }

  cerr<<"R=0-255, G=255-0, B=0 |  ";
  green=255; 
  float red=0;
  for (c=2*block_field_size+1;c<(3*block_field_size);c++) {
    for (r=block_field_size / 2 +1; r< block_field_size ; r++) {
      ida[r][c*3 + 0]= (unsigned char)red;
      ida[r][c*3 + 1]= (unsigned char)green;
      ida[r][c*3 + 2]= 0;
    }
    green-=255.0f / (float)block_field_size;
    red+=255.0f / (float)block_field_size;
  }

  cerr<<"R=255-0, G=0, B=0-255"<<endl;
  red=255; blue=0;
  for (c=3*block_field_size+1; c<size;c++) {
    for (r=block_field_size / 2 +1; r< block_field_size ; r++) {
      ida[r][c*3 + 0]= (unsigned char)red;
      ida[r][c*3 + 1]= 0;
      ida[r][c*3 + 2]= 0;
    }
    red-=255.0f / (float)block_field_size;
    blue+=255.0f / (float)block_field_size;
  }

  
  Image<float> imfl;
  ImageConvert::ConvertST(im, imfl, ImageBase::ST_float);
  
  ImageIO::Save("uchar.mip", im);
  ImageIO::Save("float.mip", imfl);
  
  
  ///////////////// HSV ///////////////
  Image<unsigned char> imhsv_uc(im.GetWidth(), im.GetHeight(), im.GetChannelCount());
  Image<unsigned char> imhsv_uc_back(im.GetWidth(), im.GetHeight(), im.GetChannelCount());

  ImageConvert::Convert(im, imhsv_uc, ImageBase::CM_HSV);
  ImageConvert::Convert(imhsv_uc, imhsv_uc_back, ImageBase::CM_RGB);
  
  ImageIO::Save("uchar_hsv.mip", imhsv_uc);
  ImageIO::Save("uchar_hsv_back.mip", imhsv_uc_back);
  
  Image<float> imhsv_fl(imfl.GetWidth(), imfl.GetHeight(), imfl.GetChannelCount()); 
  Image<float> imhsv_fl_back(imfl.GetWidth(), imfl.GetHeight(), imfl.GetChannelCount());
  ImageConvert::Convert(imfl, imhsv_fl, ImageBase::CM_HSV);
  ImageConvert::Convert(imhsv_fl, imhsv_fl_back, ImageBase::CM_RGB);
  ImageIO::Save("float_hsv.mip", imhsv_fl);
  ImageIO::Save("float_hsv_back.mip", imhsv_fl_back);
  
  
  Image<float> imhsl_fl(imfl.GetWidth(), imfl.GetHeight(), imfl.GetChannelCount());
  Image<float> imhsl_fl_back(imfl.GetWidth(), imfl.GetHeight(), imfl.GetChannelCount());
  
  ImageConvert::Convert(imfl, imhsl_fl, ImageBase::CM_HSL);
  ImageConvert::Convert(imhsl_fl, imhsl_fl_back, ImageBase::CM_RGB);
  
  ImageIO::Save("float_hsl.mip", imhsl_fl);
  ImageIO::Save("float_hsl_back.mip", imhsl_fl_back);
  
  
 
  return 0;
}
