#include <Base/Image/Image.hh>

#include <string>

using namespace BIAS;
using namespace std;

int main(int argc, char *argv[])
{
  const unsigned width=640, height=480, cc=1;
  const bool interleaved = true;
  ImageBase fib(width, height, cc, ImageBase::ST_float, interleaved);
  ImageBase fib2(width, height, cc, ImageBase::ST_float, interleaved);
  ImageBase ucib(width, height, cc, ImageBase::ST_unsignedchar, interleaved);
  ImageBase ucib2(width, height, cc, ImageBase::ST_unsignedchar, interleaved);
  Image<float> fi(width, height, cc, interleaved);
  Image<float> fi2(width, height, cc, interleaved);
  Image<unsigned char> uci(width, height, cc, interleaved);
  Image<unsigned char> uci2(width, height, cc, interleaved);

  //////////////////////////////////////////
  // assign compatible storage types
  //////////////////////////////////////

  // assign ImageBase to ImageBase
  try { 
    fib = fib2; 
  } catch (BaseException &ex) { 
    cerr << "Assigning ImageBase float to ImageBase float failed:"<<ex.What()<<endl;
    BIASABORT;
  } 
  try { 
    ucib = ucib2; 
  } catch (BaseException &ex) { 
    cerr << "Assigning ImageBase unsigned char to ImageBase unsigned char "
         <<"failed:"<<ex.What()<<endl;
    BIASABORT;
  } 
  // assign Image<StorageType> to Image<StorageType>
  try { 
    fi = fi2; 
  } catch (BaseException &ex) { 
    cerr << "Assigning Image<float> to Image<float> failed:"<<ex.What()<<endl;
    BIASABORT;
  } 
  try { 
    uci = uci2; 
  } catch (BaseException &ex) { 
    cerr << "Assigning Image<unsigned char> to Image<unsigned char> failed:"<<ex.What()<<endl;
    BIASABORT;
  } 
  // assign ImageBase to Image<StorageType>
  try { 
    fi = fib; 
  } catch (BaseException &ex) {
    cerr << "Assigning ImageBase float to Image<float> failed:"<<ex.What()<<endl;
    BIASABORT;
  } 
  try { 
    uci = ucib; 
  } catch (BaseException &ex) { 
    cerr << "Assigning ImageBase unsigned char to Image<unsigned char> failed:"<<ex.What()<<endl;
    BIASABORT;
  } 
  // assign Image<StorageType> to ImageBase
  try { 
    fib = fi; 
  } catch (BaseException &ex) { 
    cerr << "Assigning Image<float> to ImageBase float failed:"<<ex.What()<<endl;
    BIASABORT;
  } 
  try { 
    ucib = uci; 
  } catch (BaseException &ex) { 
    cerr << "Assigning Image<unsigned char> to ImageBase unsigned char "
         <<"failed:"<<ex.What()<<endl;
    BIASABORT;
  } 
  
  //////////////////////////////////////////
  // assign incompatible storage types
  //////////////////////////////////////

  // assign ImageBase to ImageBase
  bool caught_exception = false;
  try {  fib = ucib; } 
  catch (BaseException &ex) { caught_exception = true;cout<<"Expected error:"<<ex.What()<<endl; }
  if (!caught_exception){ 
    cerr << "Error: Assigning ImageBase unsigned char to ImageBase float "
         <<"succeeded\n";
    BIASABORT;
  } 
  caught_exception = false;
  try {  ucib = fib; } 
  catch (BaseException &ex) { caught_exception = true;cout<<"Expected error:"<<ex.What()<<endl; }
  if (!caught_exception){ 
    cerr << "Error: Assigning ImageBase float to ImageBase unsigned char "
         <<"succeeded\n";
    BIASABORT;
  }  
  // assign Image<StorageType> to Image<StorageType>
  // actually I would rather have a compiler error here, but this is 
  // impossible as long as we acompiler error 
  caught_exception = false;
  try {  fi = uci;  } 
  catch (BaseException &ex) { caught_exception = true;cout<<"Expected error:"<<ex.What()<<endl; }
  if (!caught_exception){ 
    cerr << "Error: Assigning Image<unsigned char> to Image<float> "
         <<"succeeded\n";
    BIASABORT;
  }  
  caught_exception = false;
  try {  uci = fi;  } 
  catch (BaseException &ex) { caught_exception = true;cout<<"Expected error:"<<ex.What()<<endl; }
  if (!caught_exception){ 
    cerr << "Error: Assigning Image<float> to Image<unsigned char> "
         <<"succeeded\n";
    BIASABORT;
  }  
  
  // assign ImageBase to Image<StorageType>
  caught_exception = false;
  try {  uci = fib; } 
  catch (BaseException &ex) { caught_exception = true; cout<<"Expected error:"<<ex.What()<<endl;}
  if (!caught_exception){ 
    cerr << "Error: Assigning ImageBase float to Image<unsigned char> "
         <<"succeeded\n";
    BIASABORT;
  } 
  caught_exception = false;
  try {  fi = ucib; } 
  catch (BaseException &ex) { caught_exception = true; cout<<"Expected error:"<<ex.What()<<endl;}
  if (!caught_exception){ 
    cerr << "Error: Assigning ImageBase unsigned char to Image<float> "
         <<"succeeded\n";
    BIASABORT;
  } 
   
  // assign Image<StorageType> to ImageBase
  caught_exception = false;
  try {  ucib = fi; } 
  catch (BaseException &ex) { caught_exception = true;cout<<"Expected error:"<<ex.What()<<endl; }
  if (!caught_exception){ 
    cerr << "Error: Assigning Image<float> to ImageBase unsigned char "
         <<"succeeded\n";
    BIASABORT;
  } 
  caught_exception = false;
  try {  fib = uci; } 
  catch (BaseException &ex) { caught_exception = true;cout<<"Expected error:"<<ex.What()<<endl; }
  if (!caught_exception){ 
    cerr << "Error: Assigning Image<unsigned char> to ImageBase float "
         <<"succeeded\n";
    BIASABORT;
  } 
  
  if (argc==1){ cout << "OK\n"; }
  return 0;
}
