#include <Base/Image/ImageIO.hh>

#include <string>

using namespace BIAS;
using namespace std;

int main(int argc, char *argv[])
{
  const unsigned width=640, height=480, cc=1;
  const bool interleaved = true;
  ImageBase fib(width, height, cc, ImageBase::ST_float, interleaved);
  ImageBase ucib(width, height, cc, ImageBase::ST_unsignedchar, interleaved);
  Image<float> fi(width, height, cc, interleaved);
  Image<unsigned char> uci(width, height, cc, interleaved);

  const string fname="float_img.mip", ucname="unsigned_char_image.mip";
  if (ImageIO::Save(fname, fib)!=0){
    cerr << "Error saving float image\n";
    BIASABORT;
  }
  if (ImageIO::Save(ucname, ucib)!=0){
    cerr << "Error saving unsigned char image\n";
    BIASABORT;
  }
  
  // try to re-load images as ImageBase and as template
  if (ImageIO::Load(fname, fi)!=0){
    cerr << "Error reloading float image into Image<float>\n";
    BIASABORT;
  }
  if (ImageIO::Load(ucname, uci)!=0){
    cerr << "Error reloading unsigned char image into Image<unsigned char>\n";
    BIASABORT;
  }
  if (ImageIO::Load(fname, fib)!=0){
    cerr << "Error reloading float image into ImageBase with StorageType "
         << "float\n";
    BIASABORT;
  }
  if (ImageIO::Load(ucname, ucib)!=0){
    cerr << "Error reloading unsigned char image into ImageBase with "
         << "StorageType unsigned char \n";
    BIASABORT;
  }

  // try to load image of wrong StorageType must fail
  if (ImageIO::Load(ucname, fi)==0){
    cerr << "Error: Loaded unsigned char image into Image<float> "
         << "without error\n";
    BIASABORT;
  }
  if (ImageIO::Load(fname, uci)==0){
    cerr << "Error: Loaded float image into Image<unsigned char> without "
         << "error\n";
    BIASABORT;
  }
  if (ImageIO::Load(ucname, fib)==0){
    cerr << "Error: Loaded unsigned char image into ImageBase with StorageType "
         << "float without error\n";
    BIASABORT;
  }
  if (ImageIO::Load(fname, ucib)==0){
    cerr << "Error: Loaded float image into ImageBase with StorageType "
         << "unsigned char without error\n";
    BIASABORT;
  }

  // when the StorageType of image base is not determined, loading must succeed
  ImageBase ib1, ib2;
  if (ImageIO::Load(ucname, ib1)!=0){
    cerr << "Error loading unsigned char image into ImageBase with "
         << "StorageType unknown\n";
    BIASABORT;
  }
  if (ImageIO::Load(fname, ib2)!=0){
    cerr << "Error loadingfloat image into ImageBase with "
         << "StorageType unknown\n";
    BIASABORT;
  }

  if (argc==1){ cout << "OK\n"; }
  return 0;
}
