#ifndef __BogIO_hh__
#define __BogIO_hh__

#include <Base/Math/Matrix3x3.hh>

#include <fstream>
#include <sstream>
#include <vector>
#include <string>

namespace BIAS {

  
  
  // JW 03/2005
  template <class T>
  int LoadBogRC(const std::string & filename, Matrix3x3<T>& mat)
  {
    mat.SetZero();// reset this
    std::ifstream fs( filename.c_str() );
    if (!fs) {
      BIASERR("could not open file "<<filename);
      return -1; // error, could not open file.
    };
    // parse and read in 
    const unsigned int nMax=1024;
    char caLine[nMax];
    // std string for find
    std::string strLine;
    // helper for string to number /atof)
    std::istringstream iss;
    unsigned int pos=0, line=0;
    std::vector<std::string> pattern;
    pattern.clear();
    pattern.push_back("RC11 ="); // 0
    pattern.push_back("RC12 ="); // 1
    pattern.push_back("RC13 ="); // 2

    pattern.push_back("RC21 ="); // 3
    pattern.push_back("RC22 ="); // 4
    pattern.push_back("RC23 ="); // 5

    pattern.push_back("RC31 ="); // 6
    pattern.push_back("RC32 ="); // 7
    pattern.push_back("RC33 ="); // 8

    std::vector< bool > found;
    found.resize( pattern.size(), false);
    BIASASSERT(found.size()>8);
    while (!fs.eof() && (fs) 
           && (
               !found[0] 
               || !found[1] 
               || !found[2] 
               || !found[3] 
               || !found[4] 
               || !found[5] 
               || !found[6] 
               || !found[7] 
               || !found[8]) 
           ) 
      {
        line++;
        fs.getline(&caLine[0], nMax);
        strLine = caLine;
        for (unsigned int i=0; i<pattern.size(); i++){
          pos = (unsigned int)strLine.find(pattern[i]);
          if (pos != std::string::npos){
            found[i]=true;
            iss.clear();
            iss.str(  // after pattern: start, length
                    strLine.substr(pos+pattern[i].size(),  strLine.size()-pos-pattern[i].size()) 
                      );
            BIASASSERT(i<9); // dim
            iss >> mat.GetData()[i];
          };
        };
      };
    // check if we found all patterns    
    BIASASSERT(found.size()==pattern.size());
    bool foundAll=true;
    for (unsigned int j=0; j<found.size(); j++) foundAll=foundAll&&found[j];
    if (!foundAll) return -99; // missed something
    return 0; // OK.
  }



  // JW 03/2005
  template <class T>
  int LoadBogK(const std::string & filename, Matrix3x3<T>& mat)
  {
    mat.SetIdentity();// reset this
    std::ifstream fs( filename.c_str() );
    if (!fs) {
      BIASERR("could not open file "<<filename);
      return -1; // error, could not open file.
    };
    // parse and read in 
    const unsigned int nMax=1024;
    char caLine[nMax];
    // std string for find
    std::string strLine;
    // helper for string to number /atof)
    std::istringstream iss;
    unsigned int pos=0, line=0;
    std::vector<std::string> pattern;
    pattern.clear();

    pattern.push_back("FC_U ="); // 0
    pattern.push_back("SKEW ="); // 1
    pattern.push_back("CC_U ="); // 2
    // unused place holder: 
    pattern.push_back("UNUSED_UNUSED-x82S6jd_UNUSED "); // 3
    pattern.push_back("FC_V ="); // 4
    pattern.push_back("CC_V ="); // 5

    std::vector< bool > found;
    found.resize( pattern.size(), false);
    BIASASSERT(found.size()>5);
    while (!fs.eof() && (fs) 
           && (
               !found[0] 
               || !found[1] 
               || !found[2] 
               //|| !found[3] //unused
               || !found[4] 
               || !found[5] ) 
           ) 
      {
        line++;
        fs.getline(&caLine[0], nMax);
        strLine = caLine;
        for (unsigned int i=0; i<pattern.size(); i++){
          if (i!=3){
            pos = (unsigned int)strLine.find(pattern[i]);
            if (pos != std::string::npos){
              found[i]=true;
              iss.clear();
              iss.str(  // after pattern: start, length
                      strLine.substr(pos+pattern[i].size(),  strLine.size()-pos-pattern[i].size()) 
                        );
              BIASASSERT(i<9); // dim
              iss>> mat.GetData()[i];
            }
          }
        };
      };
    // check if we found all patterns    
    BIASASSERT(found.size()==pattern.size());
    bool foundAll=true;
    for (unsigned int j=0; j<found.size(); j++) {
      if (j!=3) foundAll=foundAll&&found[j];
    }
    if (!foundAll) return -99; // missed something
    return 0; // OK.
  }

}

#endif // __BogIO_hh__
