/*
 * Quadric3.hh
 *
 *  Created on: Aug 19, 2008
 *      Author: bangerer
 */

#ifndef __BIAS_QUADRIC3_HH__
#define __BIAS_QUADRIC3_HH__

#include "bias_config.h"
#include <Base/Math/Matrix3x3.hh>
#include <Base/Math/Matrix4x4.hh>
#include <Base/Math/Vector3.hh>

#include <Base/Debug/Error.hh>

namespace BIAS {

  /** @class Quadric3
      @brief Implements a 3D quadric and quadric operations.
      @author bangerer
      @date 19/08/2008
  */
  class BIASMathBase_EXPORT Quadric3
  {
  public:

    Quadric3(double a, double b, double c, double d, double area = 1.0);

    Quadric3(){Clear();};

    Matrix3x3<double> GetTensor();
    Vector3<double> GetVector();
    double GetOffset(){return d2;};
    double GetArea(){return r;};

    Matrix4x4<double> GetHomogenous();

    double Evaluate(double x, double y, double z);

    void Clear(double val=0.0) { a2=ab=ac=ad=b2=bc=bd=c2=cd=d2=r=val; }
    bool Optimize(Vector3<double>& v);
    // some operations
    Quadric3& operator=(const Quadric3& Q);
    Quadric3& operator+=(const Quadric3& Q);
    Quadric3& operator*=(double s);

  protected:
    // Variables for the ten quadric coeffizients

    double a2, ab, ac, ad;
    double     b2, bc, bd;
    double         c2, cd;
    double             d2;

    // the area for weighting
    double r;

  };


}

#endif /* __BIAS_QUADRIC3_HH__ */
