# -Try to find IMAGEMAGICK(++) library 
# See www.imagemagick.org
# 
# Once run this will define: 
# 
# IMAGEMAGICKLIB_FOUND
# IMAGEMAGICKLIB_LIBRARIES  : contains all available Magick++ C and C++ libraries required for linking
# IMAGEMAGICKLIB_LINK_DIRECTORIES 
# IMAGEMAGICKLIB_INCLUDE_DIR
# IMAGEMAGICKLIB_BIN_DIR : contains all DLLs required for packaging (evers)
# 
# IMAGEMAGICKLIB_CXX_LIBRARY : ImageMagick++ lib for C++ binding
# IMAGEMAGICKLIB_C_LIBRARY   : ImageMagick   lib for C binding
# IMAGEMAGICKLIB_X11_LIBRARY : image.display needs X11, possibly stub only. 
#
# AUTHOR 
# Jan Woetzel 10/2004-10/2006
# www.mip.informatik.uni-kiel.de/~jw
#
# tested with 
# - ImageMagick-6.1.1-Q16 on WinXP, MSVS 7.1  (JW 10/2004)
# - ImageMagick 5.5.4     on Linux Suse 8.2, gcc 3.3 (JW 10/2004)
# - ImageMagcik 6.2.5     on Linux Suse 10.1, gcc 4.1.0 (JW 06/2006)
# - ImageMagick-6.2.5-Q16 on WinXP, MSVS 7.1,
# - ImageMagick-6.2.9-Q16 on WinXP, MSVS 8.0/2005
#     precompiled installer libs crash/fail on image.read DLL entry call
#     in Debug build as binary release is linked against MT Release std libs !!!
#     solution: compile Release or build from source with VisualMagick (JW 10/2006)
# --------------------------------

# determine default setting for requiring both, debug and release libs
IF    (WIN32)
  # You need release and debug libs in windows 32 by default. 
  # disable it by hand if you want to live without debug libs
  # but be sure you know what you are doing.
  SET(IMAGEMAGICKLIB_USE_REL_AND_DBG_DEFAULT ON)
ELSE  (WIN32)
  SET(IMAGEMAGICKLIB_USE_REL_AND_DBG_DEFAULT OFF)
ENDIF (WIN32)
# search for Debug and Release libs, particularly useful for WIN32 MSVS IDE 
# with Debug and Release build within one project
OPTION(IMAGEMAGICKLIB_USE_REL_AND_DBG "Use Release and Debug configurations?" ${IMAGEMAGICKLIB_USE_REL_AND_DBG_DEFAULT})
MARK_AS_ADVANCED(IMAGEMAGICKLIB_USE_REL_AND_DBG)


# typical root/base dirs for installation 
SET(IMAGEMAGICKLIB_POSSIBLE_ROOT_DIRS
  "${IMAGEMAGICKLIB_ROOT_DIR}" # FIND_PACKAGE standard compatible
  "$ENV{IMAGEMAGICKLIB_ROOT_DIR}"
  "${MAGICK_HOME}"
  "$ENV{MAGICK_HOME}" # used by ImageMagick itself, contains decoder modules *MOD*  
  "$ENV{MAGICK_HOME}/../"
  "${IMAGEMAGICKLIB_DIR}" # for backward comp.
  "$ENV{IMAGEMAGICKLIB_DIR}"
  "${MAGICK_DIR}"
  "$ENV{MAGICK_DIR}"
  
  "$ENV{EXTERN_LIBS_DIR}/ImageMagick"
  
  "$ENV{HOME}/ImageMagick-6.2.9-vs2005"  # self compiled (jw)
  "$ENV{ProgramFiles}/ImageMagick-6.2.9-vs2005"
  
  # precompiled/installer settings
  "[HKEY_LOCAL_MACHINE\\SOFTWARE\\ImageMagick\\Current;LibPath]"
  "[HKEY_LOCAL_MACHINE\\SOFTWARE\\ImageMagick\\Current;LibPath]/lib"
  "[HKEY_LOCAL_MACHINE\\SOFTWARE\\ImageMagick\\Current;LibPath]/include"
  "[HKEY_LOCAL_MACHINE\\SOFTWARE\\ImageMagick\\Current;BinPath]"
  "[HKEY_LOCAL_MACHINE\\SOFTWARE\\ImageMagick\\Current;BinPath]/lib"
  "[HKEY_LOCAL_MACHINE\\SOFTWARE\\ImageMagick\\Current;BinPath]/include"
  
  # Linux
  /usr/local
  /usr
  /usr/lib64
  /opt/net/gcc41/ImageMagick
  /opt/net/gcc33/ImageMagick
  )

# find the libraries main include header file
FIND_PATH(IMAGEMAGICKLIB_INCLUDE_DIR 
  NAMES Magick++.h
  PATHS ${IMAGEMAGICKLIB_POSSIBLE_ROOT_DIRS}
  PATH_SUFFIXES include include/ImageMagick include/ImageMagick-6
  DOC "Magick++.h in root include dir"
  )

# check external include dir for basic configuration, seperated i.e. since MINT.18
# !!! untested work-around, please check if "magick-baseconfig.h file not found" compiler error
IF (NOT EXISTS ${IMAGEMAGICKLIB_INCLUDE_DIR}/magick/magick-baseconfig.h)
  IF (EXISTS /usr/include/x86_64-linux-gnu/ImageMagick-6/magick/magick-baseconfig.h)
    SET(IMAGEMAGICKLIB_CONFIG_DIR "/usr/include/x86_64-linux-gnu/ImageMagick-6/")
  ENDIF (EXISTS /usr/include/x86_64-linux-gnu/ImageMagick-6/magick/magick-baseconfig.h)
ELSE (NOT EXISTS ${IMAGEMAGICKLIB_INCLUDE_DIR}/magick/magick-baseconfig.h)
  # use empty dummy if config is in place
  SET(IMAGEMAGICKLIB_CONFIG_DIR "")
ENDIF (NOT EXISTS ${IMAGEMAGICKLIB_INCLUDE_DIR}/magick/magick-baseconfig.h)

# Magick++ "C++" 
FIND_LIBRARY(IMAGEMAGICKLIB_CXX_DEBUG_LIBRARY
  NAMES CORE_DB_Magick++_ Magick++ Magick++-6.Q16
  PATHS ${IMAGEMAGICKLIB_POSSIBLE_ROOT_DIRS}
  PATH_SUFFIXES lib/Debug lib lib/x86 x86/lib
  DOC "Magick++ lib, Debug build" )
FIND_LIBRARY(IMAGEMAGICKLIB_CXX_RELEASE_LIBRARY
  NAMES CORE_RL_Magick++_ Magick++ Magick++-6.Q16
  PATHS ${IMAGEMAGICKLIB_POSSIBLE_ROOT_DIRS}
  PATH_SUFFIXES lib/Release lib lib/x86 x86/lib
  DOC "Magick++ lib, Release build" )


# magick "C", earlier than 6.4
FIND_LIBRARY(IMAGEMAGICKLIB_C_DEBUG_LIBRARY
  NAMES  CORE_DB_magick_ Magick
  PATHS ${IMAGEMAGICKLIB_POSSIBLE_ROOT_DIRS}
  PATH_SUFFIXES lib/Debug lib lib/x86 x86/lib
  DOC "Magick++ lib, Debug build" )
FIND_LIBRARY(IMAGEMAGICKLIB_C_RELEASE_LIBRARY
  NAMES  CORE_RL_magick_ Magick
  PATHS ${IMAGEMAGICKLIB_POSSIBLE_ROOT_DIRS}
  PATH_SUFFIXES lib/Debug lib lib/x86 x86/lib
  DOC "magick lib, Release build" )

# magick "C", 6.4 and later
FIND_LIBRARY(IMAGEMAGICKLIB_CORE_LIBRARY
  NAMES MagickCore MagickCore-6.Q16
  PATHS ${IMAGEMAGICKLIB_POSSIBLE_ROOT_DIRS}
  PATH_SUFFIXES lib/Debug lib lib/x86 x86/lib
  DOC "magick lib, Release build" )

FIND_LIBRARY(IMAGEMAGICKLIB_WAND_LIBRARY
  NAMES MagickWand MagickWand-6.Q16
  PATHS ${IMAGEMAGICKLIB_POSSIBLE_ROOT_DIRS}
  PATH_SUFFIXES lib/Debug lib lib/x86 x86/lib
  DOC "magick lib, Release build" )

IF(WIN32)
  if ( CMAKE_SIZEOF_VOID_P EQUAL 4 ) # magick "C", 6.4 and later
    FIND_LIBRARY(IMAGEMAGICKLIB_WAND_LIBRARY
      NAMES MagickWand
      PATHS ${IMAGEMAGICKLIB_POSSIBLE_ROOT_DIRS}
      PATH_SUFFIXES lib/Debug lib lib/x86 x86/lib
      DOC "magick lib, Release build" )
  elseif ( CMAKE_SIZEOF_VOID_P EQUAL 8 ) #for WIN64/ExterbLibs64, unknown version
    FIND_LIBRARY(IMAGEMAGICKLIB_WAND_LIBRARY
      NAMES CORE_RL_wand_
      PATHS ${IMAGEMAGICKLIB_POSSIBLE_ROOT_DIRS}
      PATH_SUFFIXES lib/Debug lib lib/x86 x86/lib
      DOC "magick lib, Release build" )
  else ( CMAKE_SIZEOF_VOID_P EQUAL 4 )
    MESSAGE(FATAL "Unknown processor architecture!!!")
	  endif ( CMAKE_SIZEOF_VOID_P EQUAL 4 )
	ENDIF(WIN32)

# stubs only, optionally
FIND_LIBRARY(IMAGEMAGICKLIB_X11_LIBRARY
  NAMES X11
  PATHS ${IMAGEMAGICKLIB_POSSIBLE_ROOT_DIRS}
  "$ENV{EXTERN_LIBS_DIR}/ImageMagick/lib"
  /lib
  /usr/X11R6/lib
  /usr/local/lib
  /usr/openwin/lib
  /usr/lib
  )

FIND_PATH(IMAGEMAGICKLIB_BIN_DIR 
  NAMES "CORE_RL_Magick++_.dll"
  PATHS ${IMAGEMAGICKLIB_POSSIBLE_ROOT_DIRS}
  "$ENV{EXTERN_LIBS_DIR}/ImageMagick/bin"
)
  

# --------------------------------
# check _FOUND == all required libs and incdirs set?
SET(IMAGEMAGICKLIB_FOUND TRUE) # start with valid, disabled when one is missing
SET(IMAGEMAGICKLIB_V2 FALSE)   # start with assumed version <=6.3.?

# get the path(=directory) of the main library, useful only for Linux rpath
IF    (IMAGEMAGICKLIB_CXX_RELEASE_LIBRARY)
  GET_FILENAME_COMPONENT(IMAGEMAGICKLIB_LINK_DIRECTORIES ${IMAGEMAGICKLIB_CXX_RELEASE_LIBRARY} PATH)
ENDIF (IMAGEMAGICKLIB_CXX_RELEASE_LIBRARY)

IF    (NOT IMAGEMAGICKLIB_INCLUDE_DIR)
  SET(IMAGEMAGICKLIB_FOUND FALSE)  
ENDIF (NOT IMAGEMAGICKLIB_INCLUDE_DIR)

# magick-baseconfig.h found either in standard include dir or in extern dir?
IF (NOT IMAGEMAGICKLIB_CONFIG_DIR)
  SET(IMAGEMAGICKLIB_CONFIG_DIR "NOT FOUND!" CACHE STRING "Path to magicks config header magick-baseconfig.h (without magick/)")
  SET(IMAGEMAGICKLIB_FOUND FALSE)
ENDIF (NOT IMAGEMAGICKLIB_CONFIG_DIR)

IF    (IMAGEMAGICKLIB_USE_REL_AND_DBG)
  # distinguish Debug and Release libs, typically MSVS IDE
  IF    (NOT IMAGEMAGICKLIB_CXX_DEBUG_LIBRARY OR NOT  IMAGEMAGICKLIB_CXX_RELEASE_LIBRARY)
    SET(IMAGEMAGICKLIB_FOUND FALSE)    
  ENDIF (NOT IMAGEMAGICKLIB_CXX_DEBUG_LIBRARY OR NOT  IMAGEMAGICKLIB_CXX_RELEASE_LIBRARY)
  IF    (NOT IMAGEMAGICKLIB_C_DEBUG_LIBRARY OR NOT  IMAGEMAGICKLIB_C_RELEASE_LIBRARY)
    SET(IMAGEMAGICKLIB_FOUND FALSE)    
  ENDIF (NOT IMAGEMAGICKLIB_C_DEBUG_LIBRARY OR NOT  IMAGEMAGICKLIB_C_RELEASE_LIBRARY)
  IF    (IMAGEMAGICKLIB_FOUND)
    SET(IMAGEMAGICKLIB_LIBRARIES
      debug     ${IMAGEMAGICKLIB_CXX_DEBUG_LIBRARY}
      optimized ${IMAGEMAGICKLIB_CXX_RELEASE_LIBRARY}      
      debug     ${IMAGEMAGICKLIB_C_DEBUG_LIBRARY} 
      optimized ${IMAGEMAGICKLIB_C_RELEASE_LIBRARY} )
    IF   (IMAGEMAGICKLIB_X11_LIBRARY)
      # for display, possibly stub only!
      LIST(APPEND IMAGEMAGICKLIB_LIBRARIES ${IMAGEMAGICKLIB_X11_LIBRARY})
    ENDIF(IMAGEMAGICKLIB_X11_LIBRARY)
  ENDIF (IMAGEMAGICKLIB_FOUND)  

ELSE  (IMAGEMAGICKLIB_USE_REL_AND_DBG)

  # do not distinguish between Debug and Release, just use Release, typically Linux 
  IF   (NOT IMAGEMAGICKLIB_CXX_RELEASE_LIBRARY)
    SET(IMAGEMAGICKLIB_FOUND FALSE)    
  ENDIF(NOT IMAGEMAGICKLIB_CXX_RELEASE_LIBRARY)
  IF    (NOT IMAGEMAGICKLIB_C_RELEASE_LIBRARY)
    # for ImageMagick-6.4 and later
    IF   (IMAGEMAGICKLIB_CORE_LIBRARY AND IMAGEMAGICKLIB_WAND_LIBRARY)
      SET(IMAGEMAGICKLIB_V2 TRUE)
    ELSE (IMAGEMAGICKLIB_CORE_LIBRARY AND IMAGEMAGICKLIB_WAND_LIBRARY)
      SET(IMAGEMAGICKLIB_V2 FALSE)      
    ENDIF(IMAGEMAGICKLIB_CORE_LIBRARY AND IMAGEMAGICKLIB_WAND_LIBRARY)
  ENDIF (NOT IMAGEMAGICKLIB_C_RELEASE_LIBRARY)
  IF   (IMAGEMAGICKLIB_FOUND)
    IF   (IMAGEMAGICKLIB_V2)
      SET(IMAGEMAGICKLIB_LIBRARIES
          ${IMAGEMAGICKLIB_CXX_RELEASE_LIBRARY}
          ${IMAGEMAGICKLIB_CORE_LIBRARY}
          ${IMAGEMAGICKLIB_WAND_LIBRARY}  )
    ELSE (IMAGEMAGICKLIB_V2)
      SET(IMAGEMAGICKLIB_LIBRARIES
          ${IMAGEMAGICKLIB_CXX_RELEASE_LIBRARY}
          ${IMAGEMAGICKLIB_C_RELEASE_LIBRARY}  )
    ENDIF(IMAGEMAGICKLIB_V2)
    IF   (IMAGEMAGICKLIB_X11_LIBRARY)
      LIST(APPEND IMAGEMAGICKLIB_LIBRARIES ${IMAGEMAGICKLIB_X11_LIBRARY})
    ENDIF(IMAGEMAGICKLIB_X11_LIBRARY)
  ENDIF(IMAGEMAGICKLIB_FOUND)
  
ENDIF (IMAGEMAGICKLIB_USE_REL_AND_DBG)


SET(IMAGEMAGICKLIB_INCLUDE_DIR ${IMAGEMAGICKLIB_INCLUDE_DIR} ${IMAGEMAGICKLIB_CONFIG_DIR})

MARK_AS_ADVANCED(  
  IMAGEMAGICKLIB_INCLUDE_DIR
  IMAGEMAGICKLIB_CONFIG_DIR
  IMAGEMAGICKLIB_LIBRARIES
  IMAGEMAGICKLIB_WAND_LIBRARY
  IMAGEMAGICKLIB_CORE_LIBRARY
  IMAGEMAGICKLIB_CXX_DEBUG_LIBRARY
  IMAGEMAGICKLIB_CXX_RELEASE_LIBRARY
  IMAGEMAGICKLIB_C_DEBUG_LIBRARY
  IMAGEMAGICKLIB_C_RELEASE_LIBRARY
  IMAGEMAGICKLIB_X11_LIBRARY
  IMAGEMAGICKLIB_USE_REL_AND_DBG
  IMAGEMAGICKLIB_BIN_DIR
  IMAGEMAGICKLIB_HDRI_ENABLE
  IMAGEMAGICKLIB_QUANTUM_DEPTH )

# make FIND_PACKAGE case sensitive compatible
SET(ImageMagickLib_V2          ${IMAGEMAGICKLIB_V2})
SET(ImageMagickLib_FOUND       ${IMAGEMAGICKLIB_FOUND})
SET(ImageMagickLib_LIBRARIES   ${IMAGEMAGICKLIB_LIBRARIES})
SET(ImageMagickLib_INCLUDE_DIR ${IMAGEMAGICKLIB_INCLUDE_DIR})


# ==========================================
IF(NOT IMAGEMAGICKLIB_FOUND)
  # make FIND_PACKAGE friendly
  IF(NOT IMAGEMAGICKLIB_FIND_QUIETLY)
    IF(IMAGEMAGICKLIB_FIND_REQUIRED)
      MESSAGE(FATAL_ERROR "IMAGEMAGICKLIB required, please specify it's location.")
    ELSE(IMAGEMAGICKLIB_FIND_REQUIRED)
      MESSAGE(STATUS       "ERROR: IMAGEMAGICKLIB was not found.")
    ENDIF(IMAGEMAGICKLIB_FIND_REQUIRED)
  ENDIF(NOT IMAGEMAGICKLIB_FIND_QUIETLY)
ENDIF(NOT IMAGEMAGICKLIB_FOUND)

