# - Try to find libfreenect / kinect camera driver
# Once done this will define
#  
#  KINECT_FOUND        - system has libfreenect
#  KINECT_INCLUDE_DIRS - the include directory
#  LIBFREENECT_LIBRARY    - Link these to use libfreenect
#  LIBFREENECT_FAKE_LIBRARY    - Link these to use libfreenect demo (without kinect)
#  KINECT_LIBRARIES    - real device library
# 
# author: fkellner 02/11

FIND_PATH( LIBFREENECT_INCLUDE_DIR
 libfreenect.h
PATHS
 $ENV{EXTERN_LIBS_DIR}
 "$ENV{EXTERN_LIBS_DIR}/libfreenect"
 "$ENV{EXTERN_LIBS_DIR}/libfreenect/include"
 "$ENV{EXTERN_LIBS_DIR}/libfreenect/include/libfreenect"
 /usr/include/libfreenect
 /usr/local/include/libfreenect
 /opt/net64/gcc44/libfreenect/include
)

FIND_LIBRARY( LIBFREENECT_LIBRARY
  freenect
PATHS
$ENV{EXTERN_LIBS_DIR}
 "$ENV{EXTERN_LIBS_DIR}/libfreenect"
 "$ENV{EXTERN_LIBS_DIR}/libfreenect/lib"
  /usr/lib/
  /usr/lib64/
  /usr/local/lib/
  /usr/local/lib64/
  /opt/net64/gcc44/libfreenect/lib/
)

FIND_LIBRARY( LIBFREENECT_FAKE_LIBRARY
  freenect
PATHS
$ENV{EXTERN_LIBS_DIR}
 "$ENV{EXTERN_LIBS_DIR}/libfreenect"
 "$ENV{EXTERN_LIBS_DIR}/libfreenect/lib"
  /usr/lib/fakenect
  /usr/lib64/fakenect
  /usr/local/lib/fakenect
  /usr/local/lib64/fakenect
  /opt/net64/gcc44/libfreenect/lib/fakenect
NO_DEFAULT_PATH
)

IF(WIN32)
FIND_LIBRARY( LIBUSB_WIN32_LIBRARY 
  libusb
PATHS
 $ENV{EXTERN_LIBS_DIR}
 "$ENV{EXTERN_LIBS_DIR}/libusb-win32-bin-1.2.3.0/lib/msvc"
 "$ENV{EXTERN_LIBS_DIR}/libusb-win32-bin-1.2.3.0/lib"
 "$ENV{EXTERN_LIBS_DIR}/libusb-win32/lib/msvc"
 "$ENV{EXTERN_LIBS_DIR}/libusb-win32/lib"
 DOC "libfreenect requires the libusb-win library for windows"
 NO_DEFAULT_PATH
)
ENDIF()

SET( KINECT_FOUND FALSE )

# todo: make windows compatible
IF( LIBFREENECT_INCLUDE_DIR AND LIBFREENECT_LIBRARY)# AND LIBFREENECT_FAKE_LIBRARY )
  SET( KINECT_FOUND TRUE )
  SET( KINECT_INCLUDE_DIRS ${LIBFREENECT_INCLUDE_DIR} ${LIBFREENECT_INCLUDE_DIR}/libfreenect )
  SET( KINECT_LIBRARIES ${LIBFREENECT_LIBRARY} ${LIBUSB_WIN32_LIBRARY})
ELSE()
  MESSAGE("Could not find libfreenect libraries")  
ENDIF()


MARK_AS_ADVANCED(
  LIBFREENECT_INCLUDE_DIR
  LIBFREENECT_LIBRARY
  LIBFREENECT_FAKE_LIBRARY
  LIBUSB_WIN32_LIBRARY
  )


