# 
# This module finds if MIP is available and determines where the
# include files and libraries are. 
# This code sets the following variables:
#
#
#
# MIP_FOUND            = system has MIP lib
#
# MIP_LIBRARIES        = full path to the libraries
#                         on Unix/Linux with additional linker flags from "mip-config --libs"
# 
# CMAKE_MIP_CXX_FLAGS  = Unix compiler flags for MIP, essentially "`mip-config --cxxflags`"
#
# MIP_INCLUDE_DIR      = where to find headers "wx/wx.h" "wx/setup.h"
#
# MIP_LINK_DIRECTORIES = link directories, useful for rpath on Unix
# MIP_EXE_LINKER_FLAGS = rpath on Unix#
#
#
# typical USAGE in user projects CMakeLists.txt: 
#    FIND_PACKAGE(MIP REQUIRED)
#    IF (NOT MIP_FOUND)
#      MESSAGE(FATAL_ERROR "MIP was not found. Please set MIP_DIR pointing to MIPConfig.cmake build dir.")
#    ELSE(NOT MIP_FOUND)
#      INCLUDE(${MIP_USE_FILE})
#    ENDIF (NOT MIP_FOUND)
#
# Jan Woetzel
# www.mip.informatik.uni-kiel.de/~jw
# major rewrite Felix Woelk woelk@vision-n.de


FIND_FILE(MIP_CONFIG_CMAKE_SCRIPT MIPConfig.cmake 
  ${MIP_ROOT_DIR}
  $ENV{MIP_ROOT_DIR}
  ${MIP_DIR}
  $ENV{MIP_DIR}
  "${CMAKE_CURRENT_BINARY_DIR}/../MIP"
  )

IF(MIP_CONFIG_CMAKE_SCRIPT)
  #  MESSAGE("Found: ${MIP_CONFIG_CMAKE_SCRIPT}")
  INCLUDE(${MIP_CONFIG_CMAKE_SCRIPT})
  SET(MIP_FOUND 1)
  # update BIAS_DIR to match BIASConfig.cmake, convert to fwd. slashes
  GET_FILENAME_COMPONENT(MIP_DIR ${MIP_CONFIG_CMAKE_SCRIPT} PATH)
  MESSAGE(STATUS "FindMIP is Using ${MIP_CONFIG_CMAKE_SCRIPT}")

ELSE(MIP_CONFIG_CMAKE_SCRIPT)
  
 # make FIND_PACKAGE friendly
  IF(NOT MIP_FIND_QUIETLY)
    IF(MIP_FIND_REQUIRED)
      MESSAGE(FATAL_ERROR "MIP required, Please select MIP_CONFIG_CMAKE_SCRIPT by hand or guide cmake with MIP_DIR.")
    ELSE(MIP_FIND_REQUIRED)
      MESSAGE(STATUS       "ERROR: MIP was not found. Please select MIP_CONFIG_CMAKE_SCRIPT by hand or guide with MIP_DIR.")
    ENDIF(MIP_FIND_REQUIRED)
  ENDIF(NOT MIP_FIND_QUIETLY)
  
ENDIF(MIP_CONFIG_CMAKE_SCRIPT)

