# Try to find MathLink from Wolfram Mathematica package
#
# See http://www.wolfram.com/solutions/mathlink/
#
# Once run this will define: 
#
# MathLink_FOUND
# MathLink_INCLUDE_DIR
# MathLink_LIBRARIES
# MathLink_MPREP_BINARY
# 
# NOTES
# tested with Mathematica 5.1 (on Windows XP)
#
# AUTHOR
# Jan Woetzel <http://www.mip.informatik.uni-kiel.de/~jw> (03/2006)

# the precompiled MathLink library name depends on 
# - version (1 or 2) - we always use 2
# - compiler/linker  - MS Visual studio, mingw, borland, cygwin ...

#IF    (NOT MSVC)
IF (NOT WIN32)
  MESSAGE(STATUS "FindMathLink.cmake  is tested with WIN32 MSVC only. You may need to adapt the compiler dependant precompile libname to pick (which is ml32i2m for MSVC)")
ENDIF (NOT WIN32)
#ENDIF (NOT MSVC)

FIND_PATH(MathLink_INCLUDE_DIR mathlink.h
  $ENV{MathLink_DIR}/include
  $ENV{MathLink_DIR}
  $ENV{MathLink_HOME}/include
  $ENV{MathLink_HOME}
  $ENV{EXTRA}/include
  $ENV{EXTRA}
  "$ENV{ProgramFiles}/Wolfram Research/Mathematica/5.1/AddOns/MathLink/DeveloperKit/Windows/CompilerAdditions/mldev32/include"
)
#MESSAGE("DBG MathLink_INCLUDE_DIR=${MathLink_INCLUDE_DIR}")


  FIND_LIBRARY(MathLink_LIBRARY
    NAMES ml32i2m
    PATHS 
  $ENV{MathLink_DIR}/lib
  $ENV{MathLink_DIR}
  $ENV{MathLink_HOME}/lib
  $ENV{MathLink_HOME}
  $ENV{EXTRA}/include
  $ENV{EXTRA}
  "$ENV{ProgramFiles}/Wolfram Research/Mathematica/5.1/AddOns/MathLink/DeveloperKit/Windows/CompilerAdditions/mldev32/lib"
)
#MESSAGE("DBG MathLink_LIBRARY=${MathLink_LIBRARY}")


FIND_PROGRAM(MathLink_MPREP_BINARY 
 NAMES mprep 
 PATHS 
 "$ENV{ProgramFiles}/Wolfram Research/Mathematica/5.1/AddOns/MathLink/DeveloperKit/Windows/CompilerAdditions/mldev32/bin"
 DOC "Mathematica Mathlink mprep binary" )
  
# --------------------------------

IF(MathLink_LIBRARY)
  SET(MathLink_LIBRARIES ${MathLink_LIBRARY})
ELSE(MathLink_LIBRARY)
  MESSAGE(STATUS "MathLink_LIBRARY_2m library not found.")
ENDIF(MathLink_LIBRARY)

IF(NOT MathLink_INCLUDE_DIR)
  MESSAGE(STATUS "MathLink_INCLUDE_DIR include dir not found.")
ENDIF(NOT MathLink_INCLUDE_DIR)

IF   (NOT MathLink_MPREP_BINARY)
  MESSAGE(STATUS "MathLink_MPREP_BINARY not found.")
ENDIF(NOT MathLink_MPREP_BINARY)

IF   (MathLink_LIBRARIES AND MathLink_INCLUDE_DIR)
  SET(MathLink_FOUND TRUE)
ENDIF(MathLink_LIBRARIES AND MathLink_INCLUDE_DIR)
  

MARK_AS_ADVANCED(
  MathLink_LIBRARY
  MathLink_LIBRARIES
  MathLink_INCLUDE_DIR
  MathLink_MPREP_BINARY
  MathLink_FOUND
)


# ==========================================
IF(NOT MathLink_FOUND)
  # make FIND_PACKAGE friendly
  IF(NOT MathLink_FIND_QUIETLY)
    IF(MathLink_FIND_REQUIRED)
      MESSAGE(FATAL_ERROR "MathLink required, please specify it's location.")
    ELSE(MathLink_FIND_REQUIRED)
      MESSAGE(STATUS       "ERROR: MathLink was not found.")
    ENDIF(MathLink_FIND_REQUIRED)
  ENDIF(NOT MathLink_FIND_QUIETLY)
ENDIF(NOT MathLink_FOUND)
