# - Try to find OpenCL (nvidia)
# Once done this will define
#  
#  OPENCL_NV_FOUND        - system has OpenCL
#  OPENCL_NV_INCLUDE_DIRS - the include directory
#  OPENCL_NV_LIBRARIES    - Link these to use OpenCL
#   

SET(CL_POSSIBLE_INCLUDE_PATHS
 /usr/include
 /usr/local/cuda/include
 ${CMAKE_CURRENT_SOURCE_DIR}/OpenCLFramework/include
)

SET(CL_POSSIBLE_LIB_PATHS
 /usr/local/cuda/lib64
 /usr/local/cuda/lib
)

IF (WIN32)
SET(CUDA_INSTALL_PATH "C:/Program Files/NVIDIA GPU Computing Toolkit/CUDA/v6.5" CACHE PATH "Where CUDA Toolkit is installed")
SET(AMDAPP_INSTALL_PATH "C:/Program Files (x86)/AMD APP" CACHE PATH "Where AMD APP Sdk is installed")

LIST(APPEND CL_POSSIBLE_INCLUDE_PATHS
  "${CUDA_INSTALL_PATH}/include"
  "$ENV{EXTERN_LIBS_DIR}/OpenCL/include"
  "${AMDAPP_INSTALL_PATH}/include"
)

IF ("${CMAKE_SIZEOF_VOID_P}" EQUAL 8)
 LIST(APPEND CL_POSSIBLE_LIB_PATHS
  "${CUDA_INSTALL_PATH}/lib/x64"
  "${AMDAPP_INSTALL_PATH}/lib/x86_64"
  "$ENV{EXTERN_LIBS_DIR}/OpenCL/lib"
 )
ELSE ()
 LIST(APPEND CL_POSSIBLE_LIB_PATHS
  "${CUDA_INSTALL_PATH}/lib/Win32"
  "${AMDAPP_INSTALL_PATH}/lib/x86"
  "$ENV{EXTERN_LIBS_DIR}/OpenCL/lib"
 )
ENDIF()

ENDIF(WIN32)

FIND_PATH(OPENCL_NV_INCLUDE_DIRS
 CL/cl.hpp 
PATH 
 ${CL_POSSIBLE_INCLUDE_PATHS}
)

FIND_LIBRARY(OPENCL_NV_LIBRARIES
 OpenCL
PATH 
 ${CL_POSSIBLE_LIB_PATHS}
)

SET( OPENCL_NV_FOUND FALSE )

IF( OPENCL_NV_INCLUDE_DIRS AND OPENCL_NV_LIBRARIES )
  SET( OPENCL_NV_FOUND TRUE )
ELSE()
  MESSAGE("OpenCL: Please make sure you have installed the cuda toolkit.")
  MESSAGE("OpenCL: Then make sure cl.hpp is located in the include/CL folder of your toolkit installation")
  MESSAGE("OpenCL: If cl.hpp is missing, download it from http://www.khronos.org/registry/cl/")
ENDIF()

mark_as_advanced(OPENCL_NV_INCLUDE_DIRS OPENCL_NV_LIBRARIES)