/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003-2009    (see file CONTACT for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#ifndef __BlobDetectorBase_hh__
#define __BlobDetectorBase_hh__

#include <bias_config.h>
#include <string>
#include <Base/Image/Image.hh>
#include <Base/Math/Vector2.hh>


namespace BIAS {

  /**
     @struct BIASBlob
     Helper class to store blob corners
     @author fkellner/ischiller 07/2008 
     @ingroup g_feature
  */
  class BIASFeatureDetector_EXPORT BIASBlob {
  public:
  	BIASBlob(){weight=0; UL[0]=UL[1]=LR[0]=LR[1]=centerofmass[0]=centerofmass[1]=0; };
  	~BIASBlob(){};
  	
    BIAS::Vector2<double> UL;
    BIAS::Vector2<double > LR;
    unsigned int weight;
    BIAS::Vector2<double> centerofmass;
  } ;
    

  /** 
      @class BlobDetectorBase
      @ingroup g_feature
      @brief purly virtual interface defining class for blob detectors
      @author ischiller 08/2008 
  */
  template <class StorageType>
  class BIASFeatureDetector_EXPORT BlobDetectorBase
  {
  public:
    BlobDetectorBase();
    virtual ~BlobDetectorBase();

    /** purely virtual interface definition */
    virtual int Detect(Image<StorageType>& image,
                       std::vector<BIAS::BIASBlob>& blobs)=0;
    
    ///////////////////////////////////////////////////////////////////////

    inline void SetMaxNumBlobs(const int maxnum){
      dMaxNumBlobs_ = maxnum;
    }
    
    /** 
        @brief return maximum number of blobs
        @author ischiller 
    */
    inline int GetMaxNumBlobs(){
      return dMaxNumBlobs_;
    }

  protected:

    /// maximum number of corners to return
    int dMaxNumBlobs_;
  }; // class


} // namespace

#endif // __BlobDetectorBase_hh__
