/**
   @file TestCheckerDetectCF
   @brief Test the CrossFilter checkerboard detector on a synthetic checkerboard image
   @ingroup g_tests
   @author jordt
   @date 11/2010
*/

#include <FeatureDetector/CheckerboardDetectorCrossFilter.hh>
#include <Base/ImageUtils/ImageDraw.hh>

using  namespace BIAS;
using namespace std;


/// Generate a checkerboard and run the CheckercoardDetectorCrossFilter on it.
int main(int argc, char* argv[]){

  // Generate a checkerboard image
  Image<unsigned char> testImg;
  testImg.Init(1024,768,3);
  testImg.Clear();
  unsigned char white[] = {255,255,255};
  ImageDraw<unsigned char>::RectangleCornersFill(testImg,100,100,540,390,white);
  unsigned char black[] = {0,0,0};
  for (int row = 0; row < 3; row++)
    for (int col = 0; col < 4; col++){
      ImageDraw<unsigned char>::RectangleCornersFill(testImg,120+col*100,121+row*100,170+col*100,169+row*100,black);
      if (row != 2)
        ImageDraw<unsigned char>::RectangleCornersFill(testImg,170+col*100,171+row*100,220+col*100,219+row*100,black);
    }

  // Detect corners
  CheckerboardDetectorCrossFilter cbd;
  vector<HomgPoint2D> checkerCorners = cbd.GetCheckerboard(testImg,7,4,true);

  // Check if result is correct
  int index = 0;
  for (int y = 170; y <= 320; y+=50)
    for (int x = 170; x <= 470; x+=50){
      if ((int)checkerCorners[index][0] != x) return 1;
      if ((int)checkerCorners[index++][1] != y) return 1;
    }

  return 0;
}
