/*
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003-2009    (see file CONTACT for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

// std
#include <iostream>

// BIAS
#include <Filter/DensityFilter.hh>
#include <Base/Image/ImageIO.hh>

using namespace std;
using namespace BIAS;

#define HALF_WIN_SIZE    10
#define REQUIRED_DENSITY  0.5
#define EMPTY_PIXEL      -1   // pixel value representing empty

/**
 * @author rwulff
 * @date   10/2011
 */
int main(int argc, char* argv[])
{
  // ensure correct parameter count
  if (argc < 2) {
    cerr << "please specify a depth map (with storage type float)" << endl;
    return -1;
  }

  // load depth map
  Image<float> depthMap;

  if (ImageIO::Load(argv[1], depthMap) < 0) {
    cerr << "couldn't load depth map" << endl;
    return -1;
  }

  // filter it
  DensityFilter<float, float> filter;
  filter.SetHalfWinSize(HALF_WIN_SIZE);
  filter.SetRequiredDensity(REQUIRED_DENSITY);
  filter.SetUndefinedPixelValue(EMPTY_PIXEL);

  Image<float> filteredImage;

  filter.Filter(depthMap, filteredImage);

  // save filtered image
  if (ImageIO::Save("filteredImage.mip", filteredImage) < 0) {
    cerr << "couldn't save filtered image" << endl;
    return -1;
  }

  return 0;
}
