/*
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003-2009    (see file CONTACT for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/** 
	@example ExampleDilation.cpp
	@relates Dilation
	@brief Example for dilation of images
	@ingroup g_examples 
	@author MIP
*/

#include <Filter/Dilation.hh>
#include <Base/Image/Image.hh>
#include <Base/Image/ImageIO.hh>
#include <Base/ImageUtils/ImageDraw.hh>

using namespace BIAS;
using namespace std;

int main (int argc, char *argv[])
{
  if (argc!=2){
    cerr << argv[0] << "  <image>\n";
    return -1;
  }
  Image<unsigned char> im, res;

  if (ImageIO::Load(argv[1], im)!=0){
    BIASERR("error loading image "<<argv[1]);
    return -2;
  }

  Dilation<unsigned char, unsigned char> dilation;
  dilation.SetKernelSize(3); // kernelsize 3 wich is default is MUCH faster
  dilation.SetBorderHandling(FilterBase<unsigned char, unsigned char>::TBH_same);
  dilation.Filter(im,res);
  //ImageIO::Save("result.mip", res);
  ImageIO::Save("result.mip", res);
  cout<<"Written result.mip"<<endl;
  return 0;
}
