/* 
 This file is part of the BIAS library (Basic ImageAlgorithmS).

 Copyright (C) 2004
 Johan Skoglund
 skoglund@isy.liu.se
 CVL/isy
 university of Link"oping
 sweden

 BIAS is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 (at your option) any later version.

 BIAS is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with BIAS; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/** 
	@example ExampleMedian3x3x3.cpp
	@brief Example for median filtering images wiht 3x3
  @relates Median
	@ingroup g_examples 
	@author MIP
*/

#include "Filter/Median.hh"
#include "Base/Image/ImageIO.hh"
#include <Base/Image/ImageConvert.hh>
#include <iostream>

#include <Base/Debug/TimeMeasure.hh>

using namespace BIAS;
using namespace std;


int main(int argc, char *argv[]) {

  Median<unsigned char, unsigned char> median;

  ImageBase ucim;
  Image<unsigned char> im;
  Image<unsigned char> med;

  if (argc<2) {
    cerr << argv[0] << "source image \n";
    return -2;
  }
 
  vector<Image<unsigned char> > images;

  for (int i = 1; i < argc; i++) {
    if (ImageIO::Load(argv[i], im)!=0) {
      BIASERR("error loading image "<<argv[1]);
      return -1;
    } else {
      cerr << "read "<<argv[i]<<endl;
    }
//    if (im.GetChannelCount() != 1) {
//      if (ImageConvert::IP_ToGrey(im)!=0) {
//        BIASERR("error converting image "<<argv[1]);
//      }
//    }
    images.push_back(im);
  }

  
  //median.SetSize(2,0);
  median.Filter3x3x3Color(images, med);

  //ImageIO::Save("im1.mip", images[0]);
  //ImageIO::Save("im2.mip", images[1]);
  //ImageIO::Save("im3.mip", images[2]);

  //ImageIO::Save("imResult.mip", med);
  ImageIO::Save("imResult.mip", med);
  ImageIO::Save("imResult.png", med);
  
  return 0;

}
