/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2004
Johan Skoglund
skoglund@isy.liu.se
CVL/isy
university of Linkoeping
sweden

BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/** 
	@example ExampleRemoveSaltAndPepper.cpp
	@brief Example for median filtering images with 3x3
  @relates Median
	@ingroup g_examples 
	@author MIP
*/

#include "Filter/Median.hh"
#include "Base/Image/ImageIO.hh"
#include <iostream>

#include <Base/Debug/TimeMeasure.hh>

using namespace BIAS;
using namespace std;



int main(int argc, char *argv[])
{
    TimeMeasure timeri, timerf;

    Image<float> src; 
    if (!(argc>1)){
        BIASERR("no arg"); return -1;
    };
    BIASASSERT(argc>1);
    if (ImageIO::Load(argv[1], src)!=0){
        cout <<"Error loading "<<argv[1]<<endl;
        exit(-1);
    }

    Image<float> dest;

    Median<float, float> theMedian;
    timeri.Start();
    theMedian.FilterRemoveSaltAndPepper(src, dest);
    timeri.Stop();
    timeri.PrintRealTime();
    ImageIO::Save( "saltandpepper_filtered", dest);
    Image<float> orig,filtered,diff;
    orig.GetChannel(src,0);
    filtered.GetChannel(dest,0);
    diff = orig-filtered;
    ImageIO::Save( "saltandpepper_diff", diff);

    return 0;
}
