/* 
 This file is part of the BIAS library (Basic ImageAlgorithmS).

 Copyright (C) 2003, 2004    (see file CONTACTS for details)
 Multimediale Systeme der Informationsverarbeitung
 Institut fuer Informatik
 Christian-Albrechts-Universitaet Kiel


 BIAS is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 (at your option) any later version.

 BIAS is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with BIAS; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef TUKEYWINDOW_HH_
#define TUKEYWINDOW_HH_

#include "FilterNToN.hh"
#include <Base/Common/BIASpragmaStart.hh>

namespace BIAS {

  /**
   * Multiplies an image with a full image size tukey window. Smaller window sizes for convolution not implemented yet.
   * @author fkellner 05/09
   */
  template <class InputStorageType, class OutputStorageType>
  class BIASFilter_EXPORT TukeyWindow 
  : public FilterNToN<InputStorageType, OutputStorageType> 
  {
    public:
      /**
       * @brief Default Constructor, ratio 0.5, zeropad 0
       */
      TukeyWindow();
      /**
       * @brief Constructor with custom ratio and zeropad
       * @param ratio similar to matlab implementation: part of image multiplied with cos
       * @param zeropad pixels added to each border with 0 (black) value
       */
      TukeyWindow(float ratio, unsigned int zeropad = 0);
      TukeyWindow(const TukeyWindow<InputStorageType, OutputStorageType>& other);

      virtual ~TukeyWindow();
      
      virtual int Filter(const Image<InputStorageType>& src, 
                         Image<OutputStorageType>& dst);

      virtual int FilterInt(const Image<InputStorageType>& src, 
                            Image<OutputStorageType>& dst);

      virtual int FilterFloat(const Image<InputStorageType>& src, 
                              Image<OutputStorageType>& dst);

      virtual FilterNToN<InputStorageType, OutputStorageType>* Clone() const {
        return new TukeyWindow<InputStorageType, OutputStorageType>(ratio_);
      }
      
      inline void SetRatio(float ratio, unsigned int zeropad = 0) {
        ratio_ = ratio;
        zeropad_ = zeropad;
      }

    protected:

      virtual void GetBordersValid_(int& border_x, int& border_y) const {
        
      }

    private:
      float ratio_;
      unsigned int zeropad_;
  };

}

#include <Base/Common/BIASpragmaEnd.hh>

#endif /* TUKEYWINDOW_HH_ */
