#ifndef __CONTEXT_GLUT_HH__
#define __CONTEXT_GLUT_HH__

#include <bias_config.h>
#ifndef BIAS_HAVE_GLUT
#  error You need glut. Please recompile BIAS with USE_GLUT.
#endif //BIAS_HAVE_GLUT

#include "RenderContextBase.hh"
#include <vector>

//activate to have fps output on window title
//#define GLUT_TIME_MEASURE

namespace BIAS {

  // forward declaration of function (for gcc-4.2)
  void MouseMotionFunc_(int x, int y);
  void MousePassiveMotionFunc_(int x, int y);
  void MouseButtonFunc_(int button, int state, int x, int y);
  void SpecialKeyFunc_(int key, int x, int y);
  void SpecialKeyUpFunc_(int key, int x, int y);
  void StandardKeyFunc_(unsigned char key, int x, int y);
  void StandardKeyUpFunc_(unsigned char key, int x, int y);
  void Reshape_(int width, int height);
  void Render_();
  void WhenIdle_();

  /** \class ContextGLUT 
      \brief Wrapper to glut-library.
      Automatisation and unififcation of callback registration following 
      the model implemented in class hierarchy.
      Does not create windows nor start the main loop. The application still
      needs to do it herself.
      \author bartczak
      \ingroup g_glviewer
  */
  class BIASGLviewer_EXPORT ContextGLUT : public RenderContextBase
  {
  public:

    void SetDoubleClickInterval(unsigned int ms) {doubleClickInterval_ = ms;}

    virtual void Redisplay();

    ContextGLUT(int x0 = 0, int y0 = 0, 
                int width = 400, int height = 600, 
                const std::string& name = "ContextGLUT Window",
                const char* initDisplayString = NULL,
                int* commandLineArgc = NULL,
                char** commandLineArgv = NULL);

    ContextGLUT(std::string gamemodestring, 
                const std::string& name = "ContextGLUT Window",
                const char* initDisplayString = NULL,
                int* commandLineArgc = NULL,
                char** commandLineArgv = NULL);

    virtual ~ContextGLUT();

    virtual void MakeGLContextCurrent();

    virtual void SetTimer(unsigned int interval);
    virtual void DeactivateTimer();

    int GetGlutWindowID() {return windowID_;}
    
#ifdef GLUT_TIME_MEASURE
    static std::vector<double> fpsMeasures;
    static unsigned int measurementsCount_;
#endif


  protected:
    /** Initialization of glut library, initial display state and registration
     * of callbacks.
     * Same as calling glutInit(commandLineArgc, commandLineArgv) and
     * glutInitDisplayString(initDisplayString). Must be called before
     * CreateWindow() and glutMainLoop(). 
     * @{
     */
    virtual void InitContext_(const char* initDisplayString = NULL,
                      int* commandLineArgc = NULL, 
                      char** commandLineArgv = NULL);

    /**
     * @}
     */

    /** Creates a glut window and registers all known callbacks to it.
     */
    virtual void CreateWindow_(int x0, int y0, int width, int height, 
                       const std::string& name);

    virtual void DestroyWindow_();    

    friend void MouseMotionFunc_(int x, int y);
    friend void MousePassiveMotionFunc_(int x, int y);
    friend void MouseButtonFunc_(int button, int state, int x, int y);
    friend void SpecialKeyFunc_(int key, int x, int y);
    friend void StandardKeyFunc_(unsigned char key, int x, int y);
    friend void SpecialKeyUpFunc_(int key, int x, int y);
    friend void StandardKeyUpFunc_(unsigned char key, int x, int y);
    friend void Reshape_(int width, int height);
    friend void Render_();
    friend void WhenIdle_();

    void RegisterCallbacks_() const;  


    unsigned int doubleClickInterval_;
    
    int windowID_;
    std::vector<ContextGLUT*>::iterator registrationIterator_;
    
    void EnterGameMode_(const std::string gamemodestring);
    bool gameMode_;
  };
}

#endif
