#ifndef __CONTEXTWX_HH__
#define __CONTEXTWX_HH__

#include <bias_config.h>
#include <Gui/biasgl.h>

#ifndef BIAS_HAVE_WXWIDGETS
#  error Your BIAS needs WX. Please recompile BIAS with USE_WXWIDGETS.
#endif

#include <Base/Common/BIASpragmaStart.hh>
#include <wx/wx.h>
#include <wx/glcanvas.h>
#include <Base/Common/BIASpragmaEnd.hh>

#include "RenderContextBase.hh"

// check defines from wx/setup.h implicitly
#if !(defined(wxUSE_GLCANVAS) && wxUSE_GLCANVAS)
#  error WX has not been compiled with wxUSE_GLCANVAS. Please set it in wx/setup.h and recompile WX.
#endif

namespace BIAS {

  /** @class ContextWX
      @brief Context implementation for wxWidgets.
      @attention Call InitializationComplete() in order to enable rendering!
      @ingroup g_glviewer
      @author Jan Kollmann
   */
  class BIASGLviewer_EXPORT ContextWX : public wxGLCanvas, 
    public RenderContextBase
  {
  public:
    static int defaultAttribs[];

    ContextWX(wxWindow* parent, wxWindowID id = -1,
      const wxPoint& pos = wxDefaultPosition,
      const wxSize& size = wxDefaultSize,
      long style = 0, const wxString& name = wxT("GLCanvas"),
      int* attribList = defaultAttribs,
      const wxPalette& palette = wxNullPalette);

    // The shared context constructor is deprecated in wx >= 2.9
    // so this constructor will / should be removed
    /* ContextWX(wxWindow* parent, wxGLContext* shared, wxWindowID id = -1,
      const wxPoint& pos = wxDefaultPosition,
      const wxSize& size = wxDefaultSize,
      long style = 0, const wxString& name = wxT("GLCanvas"),
      int* attribList = defaultAttribs,
      const wxPalette& palette = wxNullPalette);*/

    int FindWorkingDisplayConfiguration();

    virtual void Redisplay() { Refresh(); }

    virtual void MakeGLContextCurrent();

    /** @brief Delay rendering until this method has been called. */
    virtual void InitializationComplete() { initialized_ = true; }

    virtual void SetTimer(unsigned int interval);

    virtual void DeactivateTimer();

  protected:

    bool initialized_;

    //wxGLContext *wxGLContext_;

    virtual void OnPaint(wxPaintEvent& event);

  private:

    virtual void OnLeftDown(wxMouseEvent& event);
    virtual void OnLeftUp(wxMouseEvent& event);
    virtual void OnMiddleDown(wxMouseEvent& event);
    virtual void OnMiddleUp(wxMouseEvent& event);
    virtual void OnRightDown(wxMouseEvent& event);
    virtual void OnRightUp(wxMouseEvent& event);
    virtual void OnLeftDClick(wxMouseEvent& event);
    virtual void OnRightDClick(wxMouseEvent& event);
    virtual void OnMotion(wxMouseEvent& event);
    virtual void OnEnterWindow(wxMouseEvent& event);
    virtual void OnLeaveWindow(wxMouseEvent& event);
    virtual void OnSize(wxSizeEvent& event);
    virtual void OnKey(wxKeyEvent& event);
    virtual void OnKeyDown(wxKeyEvent& event);
    virtual void OnKeyUp(wxKeyEvent& event);
    virtual void OnTimer(wxTimerEvent& event);
    virtual void OnEraseBG(wxEraseEvent& event);

    int _OldMouseX, _OldMouseY;
    bool _FirstPaint;
    wxTimer Timer_;

    // The implicit context of wxGLCanvas is deprecated in wx >= 2.9
    #if wxCHECK_VERSION(2, 9, 0)
      wxGLContext* context_;
    #endif

    DECLARE_EVENT_TABLE()
  };

} // namespace BIAS

#endif // __CONTEXTWX_HH__
