#include "DistanceMeasureControl.hh"

#include <Base/Math/Vector3.hh>
#include <Gui/biasgl.h>

using namespace BIAS;
using namespace BIAS;
using namespace std;


DistanceMeasureControl::
DistanceMeasureControl()
{  
  bDistMeasureActivated_ = false;
  dNumClick_ =0;
  bIsInitialized_ = false;
  firstHomgPoint_.Set(0.0,0.0,0.0,1.0);
  secondHomgPoint_.Set(0.0,0.0,0.0,1.0); 
}


void DistanceMeasureControl::
SetDistanceMeasure(bool on){ 
	bDistMeasureActivated_=on;
}

bool DistanceMeasureControl::
GetDistanceMeasure(){ 
	return bDistMeasureActivated_;
}

bool DistanceMeasureControl::
LeftMouseDoubleClicked(int x, int y, int m)
{
	if (! UseLeftDouble_) {
		return false;
	}
	if(bDistMeasureActivated_){
		// Get from 2D mouse position 3D point in scene.
		if(!controlledObject_->GetViewportCoordinates(x, y)) {
			BIASERR("outside used viewport!");
			return false;
		}

		BIAS::HomgPoint3D homgPoint=controlledObject_->UnProject(x, y);

		if (!(homgPoint[3] == 0.0)) {
			//cout << "DistanceMeasureControl::LeftMouseDoubleClicked("<<x<<","<<y<<") : "
			//<< "Setting point at("<< homgPoint[0] << ", "<< homgPoint[1] << ", "
			//<< homgPoint[2] << "). "<< endl;

			if(dNumClick_==0 || dNumClick_ ==2){
				dNumClick_ = 1;
				firstHomgPoint_ = homgPoint;
				secondHomgPoint_.Set(0.0,0.0,0.0,1.0);
			}
			else if(dNumClick_==1){
				dNumClick_ = 2;
				secondHomgPoint_ = homgPoint;
				double dist = secondHomgPoint_.Distance(firstHomgPoint_);
				cout<<"Final Distance: "<<dist<<endl;
			}
			return true;
		}
		else return false;
	}
	else{
		return TrackballControl::LeftMouseDoubleClicked(x,y,m);
  }
	return false;
}

bool DistanceMeasureControl::
MouseMoved(int x, int y){
	x_ = x;
	y_ = y;
	//cout<<"MOVE:"<<x<<","<<y<<endl;
	return true;
}

void DistanceMeasureControl::
Draw(){

	if (!bIsInitialized_) {
#ifdef BIAS_HAVE_FONTGL
		if (drawer_.InitFont(NULL) != 0){
			cerr<<"init of font failed ..."<<endl;
			return;
		}
#else
		BIASWARNONCE("FontGL is disabled in BIAS.");
		return;
#endif
		bIsInitialized_ = true;
	}
	
	if(bDistMeasureActivated_ && dNumClick_==1){
		BIAS::HomgPoint3D homgPoint = controlledObject_->UnProject(x_, y_);
		if (!(homgPoint[3] == 0.0)) {
			double dist = homgPoint.Distance(firstHomgPoint_);
			
			float red[4] = {1.0, 0.0, 0.0, 1.0};		
			float white[4] = {1.0, 1.0, 1.0, 1.0};
			float black[4] = {0.0, 0.0, 0.0, 1.0};
			glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT, black);
			glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, red);
			glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, white);

			glColor3f(1.0, 0.0, 0.0);
			glBegin(GL_LINES);	  
			glVertex3d(firstHomgPoint_[0],firstHomgPoint_[1], firstHomgPoint_[2]);
			glVertex3d(homgPoint[0],homgPoint[1], homgPoint[2]);
			glEnd();
			//cout<<"Distance: "<<dist<<endl;
			stringstream text; text<<dist<<endl;

#ifdef BIAS_HAVE_FONTGL		
			glDisable(GL_DEPTH_TEST);
			drawer_.Print3D(text.str(),homgPoint[0],homgPoint[1],homgPoint[2]);
			glEnable(GL_DEPTH_TEST);
#endif

			
		}
	}
}

