#ifndef __FlyControl_HH__
#define __FlyControl_HH__

#include <bias_config.h>
#include <GLviewer/Controller/ControlInterface.hh>
#include <Base/Math/Vector3.hh>

namespace BIAS {
/** 
  \class FlyControl
  \author cmenk
  \ingroup g_glviewer
  \brief class for camera movement in fly mode.
*/
  class BIASGLviewer_EXPORT FlyControl : public ControlInterface
  {   
  public:
    FlyControl();
    
    /** Stepsize_ defines how fast the camera moves, will be set to 5% of distance
     * to Point of Interest, when a new one is set. */
    void SetStepsize(double stepsize);
    /** Moves the camera in the named directions, calls MoveDirection */
    void MoveForward();
    void MoveBackward();
    void MoveLeft();
    void MoveRight();
    void MoveUp();
    void MoveDown();

    bool RightMouseMoved(int x1, int x2, int y1, int y2);
    bool LeftMouseMoved(int x1, int x2, int y1, int y2);
    bool MiddleMouseMoved(int x1, int x2, int y1, int y2);
    bool LeftMouseDoubleClicked(int x, int y);
    bool LeftAndRightMouseMoved(int x1, int x2, int y1, int y2);
    
    /** Computes from direction the new C vector of the camera.  */
    void MoveDirection(BIAS::Vector3<double> direction);
       
    void RotateAroundViewingDirection(double rotationAngleRAD);  
    void SetUseLeftDouble(bool ld) {UseLeftDouble_ = ld;}; 
    bool GetUseLeftDouble(){return UseLeftDouble_;};
  
  protected:
    double Stepsize_;
    bool UseLeftDouble_;
  };
}//end of namespace
#endif
