#include <GLviewer/GLProjectionParametersBase.hh>
#include <Base/Math/Vector3.hh>
#include <Base/Geometry/RMatrixBase.hh>
#include "OnlyRotationControl.hh"


#include <Gui/biasgl.h>

using namespace BIAS;
using namespace BIAS;
using namespace std;
double  rememroll;
bool upsidedown;

OnlyRotationControl::OnlyRotationControl()
{  
  rememroll=0;
}
 
bool OnlyRotationControl::
LeftMouseMoved(int x1, int x2, int y1, int y2)
{
  bool left=false;

  if (x1-x2 != 0){
    if (x1 > x2)   
      left=true;        
  }
  
  Vector3<double> C, up;
  controlledObject_->GetExtrinsics(C, up);
 
  RMatrix R = controlledObject_->GetMyselfAsProjectionParameterBase()->GetR();
  double w=1.0/180.0*M_PI;
  if (left){  w=-1.0/180.0*M_PI; }
   
  BIAS::Matrix3x3<double>n(cos(w),0,-sin(w), 0,1,0,sin(w),0,cos(w));  

  BIAS::RMatrix m(n); 
  BIAS::RMatrix ges;
  R.Mult(m, ges); 
 
  double phix,phiy,phiz;
  ges.GetRotationAnglesXYZ(phix,phiy,phiz);
  controlledObject_->SetExtrinsics(HomgPoint3D(C), ges);

   cout<<"actual yaw  (rad): "<<phiy<<" (deg): "<<phiy*180.0/M_PI <<" (!not unique)"<< endl; cout<<"and roll  (rad): "<<phiz<<" (deg): "<<phiz*180.0/M_PI 
     <<" (!not unique)"<< endl;   cout<<"actual pitch  (rad): "<<phix<<" (deg): "<<phix*180.0/M_PI 
      <<" (!not unique)"<< endl;
  return true;
}
 
bool OnlyRotationControl::
RightMouseMoved(int /*x1*/, int /*x2*/, int y1, int y2)
{
  bool upper=false;

  if (y1-y2 != 0){
    if (y1 > y2)      
      upper=true;    
  }
  
  Vector3<double> C, up;
  controlledObject_->GetExtrinsics(C, up);
  RMatrix R = controlledObject_->GetMyselfAsProjectionParameterBase()->GetR();
  double w=0.1/180.0*M_PI;
  if (upper){  w*=-1; }
   
  BIAS::Matrix3x3<double>n(1,0,0,0,cos(w),sin(w), 0,-sin(w),cos(w));  
  
  BIAS::RMatrix m(n);
  BIAS::RMatrix ges;
  m.Mult(R, ges);
  double phix,phiy,phiz;
  ges.GetRotationAnglesXYZ(phix,phiy,phiz);
  controlledObject_->SetExtrinsics(HomgPoint3D(C), ges);
  cout<<"actual pitch  (rad): "<<phix<<" (deg): "<<phix*180.0/M_PI 
      <<" (!not unique)"<< endl;
  cout<<"and roll  (rad): "<<phiz<<" (deg): "<<phiz*180.0/M_PI 
      <<" (!not unique)"<< endl; 
  cout<<"and yaw  (rad): "<<phiy<<" (deg): "<<phiy*180.0/M_PI 
      <<" (!not unique)"<< endl;
  return true;
}

bool OnlyRotationControl::
MiddleMouseMoved(int x1, int x2, int y1, int y2)
{
  bool left=false;

  if (x1-x2 != 0) {
    if (x1 > x2)
      left=true;
  }
  Vector3<double> C, up;
  controlledObject_->GetExtrinsics(C, up);
  // Vector3<double> C(0.0,0.0,0.0) ;
  RMatrix R = controlledObject_->GetMyselfAsProjectionParameterBase()->GetR();
  double w  =0.1/180.0*M_PI;
  

  if (left){  w*=-1; }
   
  BIAS::Matrix3x3<double>n(cos(w),sin(w), 0,-sin(w),cos(w),0,0,0,1);  
  
  BIAS::RMatrix m(n);
  BIAS::RMatrix ges;
  R.Mult(m, ges);
  double phix,phiy,phiz;
  ges.GetRotationAnglesXYZ(phix,phiy,phiz);
  rememroll=phiz;
  controlledObject_->SetExtrinsics(HomgPoint3D(C), ges);
  cout<<"actual roll  (rad): "<<phiz<<" (deg): "<<phiz*180.0/M_PI 
      <<" (!not unique)"<< endl;
  cout<<"actual pitch  (rad): "<<phix<<" (deg): "<<phix*180.0/M_PI 
      <<" (!not unique)"<< endl;
 
  cout<<"and yaw  (rad): "<<phiy<<" (deg): "<<phiy*180.0/M_PI 
      <<" (!not unique)"<< endl;

  //  RMatrix R = controlledObject_->GetR();
  //   Vector3<double> C, up, rotationAxis, A;
  
  //   rotationAxis = Vector3<double>(R[0][2], R[1][2], R[2][2]);
  //   controlledObject_->GetExtrinsics(C, up);

  //   Create quaternion and derive new up vector
  //   Quaternion<ROTATION_MATRIX_TYPE> delta;
  //   delta.SetValueAsAxisRad(rotationAxis, 1/180.0*M_PI);
  //   delta.MultVec(up, up);
  
  //   controlledObject_->SetExtrinsics(C, C + rotationAxis, up);

  return true;
}

bool OnlyRotationControl::
LeftMouseDoubleClicked(int x, int y)
{
  //Vector3<double> C(0.0,0.0,0.0) ;
  Vector3<double> C, up;
  controlledObject_->GetExtrinsics(C, up);
  //  if ( upsidedown) {rememroll=180+(rememroll*180/M_PI);rememroll=rememroll/180*M_PI;}
  double w=rememroll;

  RMatrix R = controlledObject_->GetMyselfAsProjectionParameterBase()->GetR();
   
  BIAS::Matrix3x3<double>n(cos(w),sin(w), 0,-sin(w),cos(w),0,0,0,1);  
  
  BIAS::RMatrix m(n);
  BIAS::RMatrix ges;
  m.Mult(R, ges);
  double phix,phiy,phiz;
  ges.GetRotationAnglesXYZ(phix,phiy,phiz);
  controlledObject_->SetExtrinsics(HomgPoint3D(C), ges);
  rememroll=0;
  //  if (upsidedown) rememroll=180;
  
  cout<<"actual roll  (rad): "<<phiz<<" (deg): "<<phiz*180.0/M_PI <<" (!not unique)"<< endl;
  return true;
 
}

bool OnlyRotationControl::
LeftAndRightMouseMoved(int x1, int x2, int y1, int y2)
{ 

  //bool rightMove=false;
  bool frontMove=false;

  /*if (x1-x2 != 0)
    {
      if (x1 > x2)
        {
          rightMove=true;
        }
    
    }*/
  if (y1-y2 != 0)                                    
    {
      if (y1 > y2)
        {
          frontMove=true;
        }
    }
  Vector3<double> C, up;
  controlledObject_->GetExtrinsics(C, up);
  //  if (rightMove)
  //     {
  //       C[0]+=0.5;
  //     }else
  //       { C[0]-=0.5;
  //       } 
  if (frontMove){ 
    C[2]+=0.5;
  }
  else
    C[2]-=0.5;
  
  cout<<"actual position: x= "<<C[0]<<" y: "<<C[1]<<"z: "<<C[2]<<endl;
  RMatrix R = controlledObject_->GetMyselfAsProjectionParameterBase()->GetR();
  controlledObject_->SetExtrinsics(HomgPoint3D(C), R);
  return true;
}
  
bool OnlyRotationControl::
RightMouseDoubleClicked(int /*x*/, int /*y*/) { 

  
  Vector3<double> C, up;  
  Vector3<double> rotationAxis;
  RMatrix R = controlledObject_->GetMyselfAsProjectionParameterBase()->GetR();
  rotationAxis = Vector3<double>(R[0][2], R[1][2], R[2][2]);
  controlledObject_->GetExtrinsics(C, up);
  up[1]*=-1;
  rememroll=180+(rememroll*180/M_PI);
  rememroll=rememroll*180/M_PI;
  controlledObject_->SetExtrinsics(C,  C+rotationAxis,up);
  upsidedown=!upsidedown;

  return true;
}
//TODO 
//keylistener???  -pitch reset, center(0,0,0), print/save actual yaw/pitch/roll
