#include "ScreenShotListener.hh"
#include <Base/Image/ImageIO.hh>
#include <Image/Camera.hh>

using namespace std;
using namespace BIAS;
using namespace BIAS;

BIAS::ScreenShotListener::
ScreenShotListener() :
  activeSlotParameter_(-1)
{}

BIAS::ScreenShotListener::
ScreenShotListener(RenderContextBase* context,
                   const std::string& baseName,
                   const std::string& extension,
                   const std::string& descriptor,
                   bool storeDepthMap,
                   bool storeZBuffer,
                   bool storeMetricZMap)
{
	storeProjectionParams_= false;
  Init(context, baseName, extension, descriptor,
       storeDepthMap, storeZBuffer, storeMetricZMap);
}

void BIAS::ScreenShotListener::
Init(RenderContextBase* context,
     const std::string& baseName,
     const std::string& extension,
     const std::string& descriptor,
     bool storeDepthMap,
     bool storeZBuffer,
     bool storeMetricZMap)
{
  context_ = context;
  screenShotBaseName_ = baseName;
  screenShotExtension_ = extension;
  descriptor_ = descriptor;
  screenshotCounter_ = 0;
  storeDepthMap_ = storeDepthMap;
  storeMetricZMap_ = storeMetricZMap;
  storeZBuffer_ = storeZBuffer;
  screenShotOnTimer_ = false;
  storeGrey_ = false;
}



bool BIAS::ScreenShotListener::
SpecialKeyPressed(int key)
{
  if(key == KEY_F5) {
    StoreNextScreenShot_();
    return true;
  }
  if(key == KEY_F6) {
    StoreNextScreenShotCameraParams_();
    return true;
  }

  return false;
}


bool BIAS::ScreenShotListener::
TimerExpired()
{

  if(!screenShotOnTimer_)
    return false;

  return StoreNextScreenShot_();
}


bool BIAS::ScreenShotListener::
StoreNextScreenShotCameraParams_()
{
  if(context_->GetCamera() == NULL)
    return false;

  stringstream sstr;
  sstr.clear();
  sstr.str("");
  sstr<<screenShotBaseName_;
  if(descriptor_.size()>0)
    sstr<<"_"<<descriptor_<<"_";
  sstr.fill('0');
  sstr.width(5);
  if(screenshotCounter_>0)
    sstr<<screenshotCounter_-1;
  else
    sstr<<screenshotCounter_;
  sstr<<".projection";
  const std::string tmp(sstr.str().c_str());
#ifdef BIAS_HAVE_XML2
  if((context_->GetCamera()->XMLWrite(tmp))!=0) {
    return false;
  }
  else {
    //cout<<sstr.str()<<" stored\n";
  }
  return true;
#else
  BIASWARN("XMLWrite() is not implemented, build BIAS with XML2!");
  return false;
#endif
}


bool BIAS::ScreenShotListener::
StoreNextScreenShot_()
{
  if(context_->Render()!=0) {
    return false;
  }
  glFinish();
  Camera<unsigned char> screenshot;
  if(context_->GetImage(screenshot)!=0)
    return false;
  Projection UsedProjection;
  //  if(activeSlotParameter_>=0 &&
  //      activeSlotParameter_ < (int)context_->GetCamera()->GetNumSlots()) {
  //     ProjectionParametersBase* clone =
  //       context_->GetCamera()->
  //       GetParameterCloneWithAbsolutePose(activeSlotParameter_);
  //     Projection P(*clone);
  //     UsedProjection = P;
  //     delete clone;
  //   }
  //   else

  UsedProjection = Projection(*(context_->GetCamera()->
                                GetMyselfAsProjectionParametersBase()));

  screenshot.SetProj(UsedProjection);

  screenshot.UpdateMetaData();
  stringstream sstr;
  sstr.clear();
  sstr.str("");
  sstr<<screenShotBaseName_;
  if(descriptor_.size()>0)
    sstr<<"_"<<descriptor_<<"_";
  sstr.fill('0');
  sstr.width(5);
  sstr<<screenshotCounter_<<"."<<screenShotExtension_;
  if(ImageIO::Save(sstr.str(), screenshot)!=0) {
    return false;
  }
  else {
    cout<<sstr.str()<<" stored\n";
  }

  // write projection for image
  if(storeProjectionParams_){
  	sstr.clear();
  	sstr.str("");
  	sstr<<screenShotBaseName_;
  	sstr<<"projection";
  	if(descriptor_.size()>0)
    	sstr<<"_"<<descriptor_<<"_";
  	sstr.fill('0');
  	sstr.width(5);
  	sstr<<screenshotCounter_<<".proj";
#ifdef BIAS_HAVE_XML2
  	UsedProjection.XMLWrite(sstr.str());
#else
    BIASWARNONCE("XMLWrite() is not implemented, build BIAS with XML2!");
#endif
  }

  if(storeGrey_) {
    Camera<unsigned char> greyImage;
    if(context_->GetGreyImage(greyImage)!=0)
      return false;
    greyImage.SetProj(UsedProjection);
    greyImage.UpdateMetaData();
    sstr.clear();
    sstr.str("");
    sstr<<screenShotBaseName_<<"-grey-";
    if(descriptor_.size()>0)
      sstr<<"_"<<descriptor_<<"_";
    sstr.fill('0');
    sstr.width(4);
    sstr<<screenshotCounter_<<".pgm";
    if(ImageIO::Save(sstr.str(), greyImage)!=0) {
      return false;
    }
    else {
      cout<<sstr.str()<<" stored\n";
    }
  }


  if(storeDepthMap_) {
    Camera<float> depthMap;
    if(context_->GetDepthMap(depthMap)!=0)
      return false;
    depthMap.SetProj(UsedProjection);
    depthMap.UpdateMetaData();
    sstr.clear();
    sstr.str("");
    sstr<<screenShotBaseName_<<"-depth-";
    if(descriptor_.size()>0)
      sstr<<"_"<<descriptor_<<"_";

    sstr.fill('0');
    sstr.width(4);
    sstr<<screenshotCounter_<<".mip";
    if(ImageIO::Save(sstr.str(), depthMap)!=0) {
      return false;
    }
    else {
      // cout<<sstr.str()<<" stored\n";
    }
  }

  if(storeMetricZMap_) {
    Camera<float> depthMap;
    if(context_->GetMetricZMap(depthMap)!=0)
      return false;
    depthMap.SetProj(UsedProjection);
    depthMap.UpdateMetaData();
    sstr.clear();
    sstr.str("");
    sstr<<screenShotBaseName_<<"-metricZ-";
    if(descriptor_.size()>0)
      sstr<<"_"<<descriptor_<<"_";

    sstr.fill('0');
    sstr.width(4);
    sstr<<screenshotCounter_<<".mip";
    if(ImageIO::Save(sstr.str(), depthMap)!=0) {
      return false;
    }
    else {
      // cout<<sstr.str()<<" stored\n";
    }
  }


  if(storeZBuffer_) {
    Camera<float> depthMap;
    if(context_->GetZBuffer(depthMap)!=0)
      return false;
    depthMap.SetProj(UsedProjection);
    depthMap.UpdateMetaData();
    sstr.clear();
    sstr.str("");
    sstr<<screenShotBaseName_<<"-zbuffer-";
    if(descriptor_.size()>0)
      sstr<<"_"<<descriptor_<<"_";

    sstr.fill('0');
    sstr.width(4);
    sstr<<screenshotCounter_<<".mip";
    if(ImageIO::Save(sstr.str(), depthMap)!=0) {
      return false;
    }
    else {
      //    cout<<sstr.str()<<" stored\n";
    }
  }

  if(storeParams_) {
    StoreNextScreenShotCameraParams_();
  }

  screenshotCounter_++;
  return true;



}
