#ifndef __TrackballBase_HH__
#define __TrackballBase_HH__

#include <bias_config.h>

#ifndef BIAS_HAVE_OPENGL
#  error You need BIAS with USE_OpenGL Please recompile BIAS.
#endif // BUILD_GLviewer

#include <GLviewer/Controller/ControlInterface.hh>
#include <GLviewer/GLProjectionParametersBase.hh>
#include <Base/Math/Vector3.hh>

namespace BIAS {
/** 
  \class TrackballBase
  \author grauel, cmenk
  \ingroup g_glviewer
  \brief functionality for editing cameras or transformations in trackball mode.
   parts of trackballcontroller code have been moved to this base class
   so they can be used in other trackball classes like trackballTransController. */
  class BIASGLviewer_EXPORT TrackballBase : public ControlInterface
  {   
  public:
    TrackballBase();
    ~TrackballBase() {};
   
    void SetStepsize(double stepsize);
    bool SpecialKeyPressed(int key);
   
    void SetUseLeftDouble(bool ld) {UseLeftDouble_ = ld;}; 
    bool GetUseLeftDouble(){return UseLeftDouble_;};
    	
  protected:
    
    void GetMoveinViewDirection(GLProjectionParametersInterface* cam,
                                const double stepsize,
                                const BIAS::Vector3<double>& viewdir,
                                BIAS::Vector3<double>& res_translation);

    bool GetRotationAxisAndAngle(GLProjectionParametersInterface* cam,
                                 const int x1, 
                                 const int x2, 
                                 const int y1, 
                                 const int y2,
                                 BIAS::Vector3<double>& axis,
                                 double& angleRAD);
    
    // the translation from mouse movement (middlemousemoved case)    
    void GetTranslation(GLProjectionParametersInterface* cam,
                        const int x1, 
                        const int x2, 
                        const int y1, 
                        const int y2,
                        const BIAS::Vector3<double>& viewdir,
                        BIAS::Vector3<double>& transl);

    //get rotation angle from mouse movement for rotation around view direction
    double GetRotationAngle(GLProjectionParametersInterface* cam,
                             const int x1, 
                             const int x2, 
                             const int y1, 
                             const int y2,
                             const BIAS::Vector3<double> viewdir);
    
    void GetViewDirection(GLProjectionParametersInterface* cam,
                          BIAS::Vector3<double>& direction);

    double Stepsize_;
    bool UseLeftDouble_;
  };
}//end of namespace
#endif
