#ifndef __DISTORTIONRENDERING_HH__
#define __DISTORTIONRENDERING_HH__

#include <bias_config.h>

#ifndef BIAS_HAVE_OPENGL
#  error You need BIAS with USE_OpenGL Please recompile BIAS.
#endif // BUILD_GLviewer

#include <Gui/biasgl.h>
#include <Base/Debug/Debug.hh>
#include <GLviewer/Scenes/SceneBase.hh>

#if defined(BIAS_HAVE_GLEW)

#define BIAS_HAVE_CLASS_DISTORTIONRENDERING

//#include "GPU/GPUCG/ShaderCgF3img.h"

#include <Geometry/ProjectionParametersPerspective.hh>

#include <OpenGLFramework/Base/glfTexture2D.hh>
#include <OpenGLFramework/Base/glfFramebufferObject.hh>
#include <OpenGLFramework/Base/glfShaderProgram.hh>
#include <OpenGLFramework/Base/glfBatch.hh>
#include <OpenGLFramework/Base/glfViewport.hh>
#include <OpenGLFramework/Base/glfDepthBufferMode.hh>
#include <OpenGLFramework/Base/glfRenderTarget.hh>

#include <OpenGLFramework/Passive/glfPassiveRenderTarget.hh>
#include <OpenGLFramework/Passive/glfPassiveDepthBufferMode.hh>

namespace BIAS {

  /** @brief Utility class for distortion rendering used by GLProjection
      @warning This class can only be used if
               BIAS_HAVE_CLASS_DISTORTIONRENDERING is defined.
      @ingroup g_glviewer
      @author jkollmann */
  class BIASGLviewer_EXPORT DistortionRendering : public BIAS::Debug
  {
  public:
    DistortionRendering();

    ~DistortionRendering();

    int Init(BIAS::ProjectionParametersPerspective* projParams);

    void Reset();


    inline void SetClearColor(double r, double g, double b) {
      ClearColor_.Set((float)r, (float)g, (float)b, 0.0f);
    }

    inline void SetClearColor(const BIAS::Vector4<float>& cc){ 
      ClearColor_ = cc;
    }

    // optional interfaces that works without SceneBase.
    int BeginRendering(BIAS::ProjectionParametersPerspective* projParams,
                       bool useStencil = false);

    int EndRendering(BIAS::glfRenderTarget* renderTarget);

    void Bind();

    int DrawScenes(std::vector<SceneBase*>& scenes,
                   bool& FirstRenderingPassFinished,
                   bool& informScenesOfChange,
                   BIAS::ProjectionParametersPerspective* projParams,
                   BIAS::glfRenderTarget* renderTarget,
                   bool useStencil = false);
    

  private:
    /* moved this here (former global function) because
	   of linker errors
	   -> multiple defined symbols in OpenGLFramework
    */
    //	void SetQuadDataTexture(float* data);

    BIAS::glfFramebufferObject DistortionRenderingFBO_;
 
    // Texture holding texturecoordinates into undistorted image
    // accessed while generating distorted image
    BIAS::glfTexture2D distortionMap_;
    
    BIAS::Vector4<float> ClearColor_;

    bool loadDistortionShaders_;
    bool DistortionShadersInitialized_;

    BIAS::glfShader PerspectiveDistortionFragmentShader_;
    BIAS::glfShaderProgram distortionProgram_;

    // color texture for undistorted image
    BIAS::glfTexture2D NotDistortedColorTexture_;

    // depth texture for undistorted image
    BIAS::glfTexture2D NotDistortedDepthTexture_;

    bool      UseStencilBuffer_;
    
    //rendering the z-buffer here
    //DepthbufferAttachment NotDistortedDepthBuffer_;

    unsigned int NotDistortedImageWidth_, NotDistortedImageHeight_;

    // holding the viewport for the current distortion rendering
    // is set in InitDistortionShaders_().
    BIAS::Vector4<GLint> lastUsedGLViewport_;   
  
    BIAS::glfViewport undistortedViewport_;
    BIAS::glfViewport targetViewport_;
    BIAS::glfBatch distortionBatch_;
    BIAS::glfVertexBuffer vertices_;
    BIAS::glfElementBuffer elements_;
    BIAS::glfPassiveRenderTarget globalRenderTarget_;
    BIAS::glfPassiveDepthBufferMode globalDepthBufferMode_;
    BIAS::glfDepthBufferMode depthTest_;
    
    void SetupPrimitiveRendering_();

    void SetQuadDataTexture_(float* data);

    DistortionRendering(const DistortionRendering& d) {};
    DistortionRendering& operator=(const DistortionRendering& d) { return (*this); };

  };
  
  
}

#endif
#endif // __DISTORTIONRENDERING_HH__
