#include <GLviewer/OffscreenRendering.hh>
#include <OpenGLFramework/Base/glfScreen.hh>

#ifdef BIAS_HAVE_CLASS_OFFSCREENRENDERING

using namespace BIAS;
using namespace std;

OffscreenRendering::OffscreenRendering()
{
  initialized_ = false;
}

OffscreenRendering::~OffscreenRendering()
{
}

int OffscreenRendering::Init(int imgWidth, int imgHeight)
{    
    try {
    
    depthBuffer_.Create(GL_DEPTH_COMPONENT24, imgWidth, imgHeight);
    colorBuffer_.Create(GL_RGBA8, imgWidth, imgHeight);

    fbo_.Create();
    fbo_.AttachRenderbuffer(depthBuffer_, GL_DEPTH_ATTACHMENT_EXT);
    fbo_.AttachRenderbuffer(colorBuffer_, GL_COLOR_ATTACHMENT0_EXT);
    fbo_.CheckComplete();
    fbo_.ClearDepthBuffer();
    fbo_.ClearColorBuffer();
    
    // bind default render target
    glfScreen::GetInstance().Bind();
  
  } catch (glfException& e) {
    BIASERR("Failed to create offscreen fbo: " << e.GetMessageString());
    return -1;
  }
  
  initialized_ = true;
  return 0;
}

bool OffscreenRendering::IsInitialized() const
{
  return initialized_;
}

glfFramebufferObject* OffscreenRendering::GetFBO()
{
  if(!initialized_){
  	BIASWARN("OffscreenRendering has no valid framebuffer object, not initialised, returning NULL"); 
  	return NULL;
  }
  return &fbo_;
}

#endif
