#ifndef __OFFSCREENRENDERING_HH__
#define __OFFSCREENRENDERING_HH__

#include <bias_config.h>

#ifndef BIAS_HAVE_OPENGL
#  error You need BIAS with USE_OpenGL Please recompile BIAS.
#endif // BUILD_GLviewer

#include <Gui/biasgl.h>
#include <Base/Debug/Debug.hh>

#if defined(BIAS_HAVE_GLEW)

#define BIAS_HAVE_CLASS_OFFSCREENRENDERING

#include <OpenGLFramework/Base/glfFramebufferObject.hh>
#include <OpenGLFramework/Base/glfRenderbuffer.hh>
#include <OpenGLFramework/Base/glfTexture2D.hh>

namespace BIAS {

  /** @brief Utility class for offscreen rendering used by GLProjection
      @warning This class can only be used if
               BIAS_HAVE_CLASS_OFFSCREENRENDERING is defined.
      @ingroup g_glviewer
      @author jkollmann */
  class BIASGLviewer_EXPORT OffscreenRendering : public BIAS::Debug
  {
  public:
    OffscreenRendering();
    ~OffscreenRendering();

    int Init(int imgWidth, int imgHeight);
    bool IsInitialized() const;
    BIAS::glfFramebufferObject* GetFBO();

  private:
    BIAS::glfRenderbuffer depthBuffer_;
    BIAS::glfRenderbuffer colorBuffer_;
    BIAS::glfFramebufferObject fbo_;
    bool initialized_;

    OffscreenRendering(const OffscreenRendering&) {};
    OffscreenRendering& operator=(const OffscreenRendering&) {return (*this);};    
    
  };
  
}

#endif
#endif // __OFFSCREENRENDERING_HH__

