#include <GLviewer/OutputAttachments/OutputWowDisplay.hh>

using namespace std;
using namespace BIAS;

OutputWowDisplay::OutputWowDisplay(BIAS::glfRenderTarget *out, int factor, int offset) : GLProjectionOutputAttachment(out) {
  wowFactor_ = factor;
  wowOffset_ = offset;
  InitShader_();
}

OutputWowDisplay::~OutputWowDisplay() {

}

int OutputWowDisplay::
InitShader_() {

  string vertexCode =
      "void main()\r\n"
      "{\r\n"
      "  gl_TexCoord[0] = gl_MultiTexCoord0;\r\n"
      "  gl_Position = ftransform();\r\n"
      "}\r\n";
  string fragmentCode =
      "uniform sampler2D tex;\r\n"
      "uniform sampler2D dep;\r\n"
      "uniform float w;\r\n"
      "uniform float h;\r\n"
      "uniform int header[256];\r\n"
      "uniform float wow_disparity_M;\r\n"
      "uniform float wow_disparity_Zd;\r\n"
      "uniform float wow_disparity_vz;\r\n"
      "uniform float wow_disparity_C;\r\n"

      "float disparity(float Z)\r\n"
      "{\r\n"
      "  return (wow_disparity_M*(1.0-(wow_disparity_vz/(Z-wow_disparity_Zd+wow_disparity_vz)))\r\n"
      "          + wow_disparity_C) / 255.0;\r\n"
      "}\r\n"
      "\r\n"
      "void main()\r\n"
      "{\r\n"
      "  vec4 color;\r\n"
      "  vec2 pos = gl_TexCoord[0].st;\r\n"
      "  if (mod(pos[1]*h,2.0) < 0.5)\r\n"
      "  {\r\n"
      "    color = vec4(0.0,0.0,0.0,0.0);\r\n"
      "  }\r\n"
      "  else\r\n"
      "  {\r\n"
//      "    pos[1] = pos[1] * 2.0;\r\n"
      "    if (pos[0] < 0.5)\r\n"
      "    {\r\n"
      // write header
      "       \r\n"
//    "       if (pos[1]*h>=h-1.0 && (pos[0]*w<256) && (mod(pos[0]*w,2.0)<1.0))\r\n"
    "       if (pos[1]*h>=h-1.0 && (pos[0]*w<256))\r\n"
      "       {\r\n"
//    "         int xpos = int(pos[0]*w)/2;\r\n"
    "         int xpos = int(pos[0]*w);\r\n"
      "         color = vec4(0.0,0.0, header[ xpos ]/255.0,1.0);\r\n"
      "       }\r\n"
      "       else\r\n"
      // else take original color
      "       {\r\n"
      "         pos[0] = pos[0] * 2.0;\r\n"
      "         color = texture2D(tex,pos);\r\n"
      "       }\r\n"
      "       \r\n"
      "    }\r\n"
      "    else\r\n"
      "    {\r\n"
      "       pos[0] = (pos[0]-0.5) * 2.0;\r\n"
//    "       color = vec4(1.0,1.0,1.0,1.0) - texture2D(dep,pos);\r\n"
      "       float depth = texture2D(dep,pos).z;\r\n"
      "       if (depth == 1.0)\r\n"
      "         color = vec4(0.0);\r\n"
      "       else\r\n"
      "         color = vec4(disparity(depth)/2.0);\r\n"
      "    }\r\n"
      "  }\r\n"
      "  gl_FragColor = color;\r\n"
      "}\r\n";

  return GLProjectionOutputAttachment::InitShader(vertexCode, fragmentCode);
}

int OutputWowDisplay::
Draw_() {

  unsigned char wow_content=0x02;
  unsigned char wow_use_factor = 0x80; // 0x00 for default
  unsigned char wow_use_offset = 0x40; // 0x00 for default
  unsigned char wow_factor_UChar = wowFactor_;
  unsigned char wow_Offset_UChar = wowOffset_;
  unsigned char wow_use_factor_plut_offset = wow_use_factor+wow_use_offset;

  unsigned char header[]= {0xF1,wow_content,wow_factor_UChar,wow_Offset_UChar,wow_use_factor_plut_offset,0x00,0x00,0x00,0x00,0x00};
  // Calc the CRC32
  {
      unsigned long _register = 0;
      for(int i = 0; i < 10; ++i) {
          unsigned char mask = 0x80;
          unsigned char byte = header[i];
          for (int j = 0; j < 8; ++j)
          {
              bool topBit = (_register & 0x80000000) != 0;
              _register <<= 1;
              _register ^= ((byte & mask) != 0? 0x1: 0x0);
              if (topBit)
              {
                  _register ^= 0x04c11db7;
              }
              mask >>= 1;
          }
      }
      unsigned char *p = (unsigned char*) &_register;
      for(size_t i = 0; i < 4; ++i)
      {
          header[i+6] = p[3-i];
      }
  }

  int fullheader[256];
  {
      for (int x=0; x<256; ++x){
        fullheader[x] = 0;
      }
      for (int x=0; x<=9; ++x){
          for (int y=7; y>=0; --y){
              int i = 2*(7-y)+16*x;
              fullheader[i] = (((1<<(y))&(header[x])) << (7-(y)));
          }
      }
  }
  shaderProg_.SetUniform("header", 256, fullheader);

  // Philips Wow 42`` Display constants (from whitepaper)
  shaderProg_.SetUniform("wow_disparity_Zd", 0.467481f);
  shaderProg_.SetUniform("wow_disparity_vz", 7.655192f);
  shaderProg_.SetUniform("wow_disparity_M", -1960.37f);
  shaderProg_.SetUniform("wow_disparity_C", 127.5f);

  return GLProjectionOutputAttachment::StandardDraw_();
}
