#ifndef __NodeInfoOpenSGVisitor_hh__
#define __NodeInfoOpenSGVisitor_hh__

#include <bias_config.h>
#include <Base/Debug/Error.hh>
#include <osg/Node>
#include <osg/NodeVisitor>
#include "NodeInfo.hh"

namespace BIAS
{
  struct NodeInfoOpBase
  {
    virtual void ApplyNodeOP(osg::Node* ActualNode,
                             NodeInfoMap* map)=0;
    virtual ~NodeInfoOpBase(){}
  };
  
  struct NodeInfoEraseOp: public NodeInfoOpBase
  {
    virtual void ApplyNodeOP(osg::Node* ActualNode,
                             NodeInfoMap* map)
    {
      if(!ActualNode) return;
      if(!map) return;
      map->erase(ActualNode);
    }
    virtual ~NodeInfoEraseOp() {}
  };
  
  struct NodeInfoRestoreNameOp: public NodeInfoOpBase
  {
    virtual void ApplyNodeOP(osg::Node* ActualNode,
                             NodeInfoMap* map)
    {
      if(!ActualNode) return;
      if(!map) return;
      if(!targetMap) return;
      //find nodeinfo with ActualNode.name==nodeinfo.id and restore node name
      //and delete nodeinfo and add new ndoe info to targetmap with correct ptr
      NodeInfoMap::iterator it;
      for(it=map->begin();it!=map->end();it++)
      {
        if((*it).second.Id==ActualNode->getName()) //nodeinf exists for node
        {
          //restore name
          ActualNode->setName((*it).second.Name);
          (*targetMap)[ActualNode]=(*it).second; //create corrected copy
          map->erase((*it).first);  //delete found and reduce search space
          break;
        }
      }
    }
    virtual ~NodeInfoRestoreNameOp() {}
    NodeInfoMap* targetMap;
  };
  
  /** @class NodeInfoOpenSGVisitor
  @brief visitor for traversing the scene graph and applying some operation
         to the related nodeinfomap
  @author grauel 6/2008
  
   */
  class NodeInfoOpenSGVisitor: public osg::NodeVisitor
  {
    public:
      NodeInfoOpenSGVisitor(NodeInfoMap* nodeinfomap,
                            NodeInfoOpBase* Op):
                      osg::NodeVisitor(osg::NodeVisitor::TRAVERSE_ALL_CHILDREN )
      {
        NodeInfoMap_ = nodeinfomap;
        NodeOp_      = Op;
      };

      virtual void apply( osg::Node& node ); //node visitor code
    virtual ~NodeInfoOpenSGVisitor() {}
    private:
    
    NodeInfoMap* NodeInfoMap_;
    NodeInfoOpBase* NodeOp_;
  };


}//end of namespace

#endif
