/*
 This file is part of the BIAS library (Basic ImageAlgorithmS).

 Copyright (C) 2003, 2004    (see file CONTACTS for details)
 Multimediale Systeme der Informationsverarbeitung
 Institut fuer Informatik
 Christian-Albrechts-Universitaet Kiel


 BIAS is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 (at your option) any later version.

 BIAS is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with BIAS; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __SCENEBGIMAGE_HH__
#define __SCENEBGIMAGE_HH__

#include <GLviewer/Scenes/SceneBase.hh>
#include <bias_config.h>
#include <Gui/biasgl.h>
#include <Base/Image/Image.hh>
#include <Base/Math/Vector2.hh>
#include <Base/Math/Vector3.hh>

namespace BIAS
{
/** \class SceneBGImage
    \brief Scene that renders a background image behind all other scenes
    \ingroup g_glviewer
    \author MIP
 */
  class BIASGLviewer_EXPORT SceneBGImage : public SceneBase
  {
  public:
    virtual ~SceneBGImage(){}
    SceneBGImage();
    int SetImage(const BIAS::Image<unsigned char> &img);

    /// Query the maximum texture size supported by OpenGL. Only valid after calling SetImage() at least once.
    int GetMaxTextureSize(){return MaxTextureSize_;};
    /// set filter for minification and magnification , use GL_NEAREST, GL_LINEAR and co.
    void SetMinMagFilter(int MinFilter,int MagFilter);
  protected:
    virtual void Draw();

    // take a _copy_ of the image to allow resizing
    int InitializeTexture_(BIAS::Image<unsigned char> img);
    int MaxTextureSize_;
    bool IsInitialized_;
    GLuint BackgroundImageTex_;
    GLint MinFilter_,MagFilter_;
    unsigned int ImgWidth_,ImgHeight_;
    BIAS::Vector2<float> TexCoord_[4];
  };
}//end of namespace

#endif
