/*
 This file is part of the BIAS library (Basic ImageAlgorithmS).

 Copyright (C) 2003, 2004    (see file CONTACTS for details)
 Multimediale Systeme der Informationsverarbeitung
 Institut fuer Informatik
 Christian-Albrechts-Universitaet Kiel


 BIAS is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 (at your option) any later version.

 BIAS is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with BIAS; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __SCENECOORDINATEFRAME_HH_
#define __SCENECOORDINATEFRAME_HH_

#include <bias_config.h>

#include <Geometry/RMatrix.hh>
#include <GLviewer/Scenes/SceneBase.hh>

namespace BIAS
{
  /**
     @class SceneCoordinateFrame 
     @brief Display axes of a local coordinate frame.
     @author esquivel
     @ingroup g_glviewer
  */
  class BIASGLviewer_EXPORT SceneCoordinateFrame : public BIAS::SceneBase
  {
  public:

    /** @brief Create a coordinate frame with the given origin, orientation,
               and scale. By default the canonical coordinate frame is used. */
    SceneCoordinateFrame(const BIAS::Vector3<double> &origin
                         = BIAS::Vector3<double>(0, 0, 0),
                         const BIAS::RMatrix &orientation
                         = BIAS::RMatrix(BIAS::MatrixIdentity),
                         const double scale = 1.0);

    ~SceneCoordinateFrame();

    /** @brief Set origin, orientation and scale of coordinate frame. */
    void SetCoordinateFrame(const BIAS::Vector3<double> &origin,
                            const BIAS::RMatrix &orientation,
                            const double scale = 1.0);

    /** @brief Implementation of the SceneBase Draw method. */
    virtual void Draw();

    /** @brief Implementation of the SceneBase GetBoundingBox method. */
    virtual void GetBoundingBox(BIAS::Vector3<double> &minVal,
                                BIAS::Vector3<double> &maxVal);

  protected:

    BIAS::Vector3<float> origin_;
    BIAS::Vector3<float> axisX_, axisY_, axisZ_;

    BIAS::Vector3<double> boundingBoxMin_;
    BIAS::Vector3<double> boundingBoxMax_;

  };

}

#endif // __SCENECOORDINATEFRAME_HH_
