#include <GLviewer/Scenes/SceneGridGround.hh>
#include <GLviewer/GLProjectionParametersInterface.hh>

using namespace BIAS;

SceneGridGround::
SceneGridGround(double from, double to, double gridSize, double z)
{
  Resize(from,to,gridSize,z);
  colorR_ = 0.4;
  colorG_ = 0.4;
  colorB_ = 0.4;
  xyPlane_ = true;
}

SceneGridGround::~SceneGridGround()
{
}

void SceneGridGround::
Resize(double from, double to, double gridSize, double z)
{
  from_ = from;
  to_ = to;
  gridSize_ = gridSize;
  z_ = z;

  // Make a bounding box to fit the grid for any rotation
  const double extent = fabs(z);
  boundingBoxMin_= Vector3<double>(from - extent, from - extent, from - extent);
  boundingBoxMax_= Vector3<double>(to + extent, to + extent, to + extent);
}

void SceneGridGround::
GetBoundingBox(BIAS::Vector3<double> &minVal, BIAS::Vector3<double> &maxVal)
{
  minVal = boundingBoxMin_;
  maxVal = boundingBoxMax_;
}

void SceneGridGround::SetColor(float r, float g, float b)
{
  colorR_ = r;
  colorG_ = g;
  colorB_ = b;
}

void SceneGridGround::DrawInXYPlane(bool xyplane)
{
  xyPlane_ = xyplane;
}

void SceneGridGround::Draw()
{
  glPushAttrib(GL_LIGHTING);
  glDisable(GL_LIGHTING);

  if (!xyPlane_) {
    glMatrixMode(GL_MODELVIEW);
    glPushMatrix();
    // multiply with 90 degree rotation around x axis
    double m[16];
    memset((void*)m, 0, 16*sizeof(double));
    m[0] = 1; m[6] = 1; m[9] = -1; m[15] = 1;
    glMultMatrixd(m);
  }
  GLProjectionParametersInterface* glcam = GetGLProjectionParametersInterface();
  Vector4<float> bgcolor;
  if (glcam != NULL)
    bgcolor = glcam->GetClearColor();
  else
    bgcolor = Vector4<float>(0.0,0.0,0.0,0.0);

  // Draw the funky ground grid
  double col;
  glBegin(GL_LINES);
  for (float x = from_; x < to_; x += gridSize_) {
    for (float y = from_; y < to_; y += gridSize_) {
      // Draw the x and y (or z) axes at the center
      if (x == 0 && y == 0) {
        glColor3f(1.0,0.0,0.0);  // draw x axis
        glVertex3f(x,y,z_);
        glVertex3f(x+gridSize_,y,z_);
        if (xyPlane_)
          glColor3f(0.0,1.0,0.0); // draw y axis
        else
          glColor3f(0.0,0.0,1.0); // draw z axis
        glVertex3f(x,y,z_);
        glVertex3f(x,y+gridSize_,z_);
      } else {
        col = 1.0 - sqrt((x+gridSize_)*(x+gridSize_)+y*y) / (to_-from_ )*2.0;
        if (col < 0.0) col = 0.0;
        glColor3f(col*colorR_+(1.0-col)*bgcolor[0],
                  col*colorG_+(1.0-col)*bgcolor[1],
                  col*colorB_+(1.0-col)*bgcolor[2]);
        glVertex3f(x+gridSize_,y,z_);
        col = 1.0 - sqrt(x*x+y*y) / (to_-from_ )*2.0; // / (to_-from_ )*0.2;
        if (col < 0.0) col = 0.0;
        glColor3f(col*colorR_+(1.0-col)*bgcolor[0],
                  col*colorG_+(1.0-col)*bgcolor[1],
                  col*colorB_+(1.0-col)*bgcolor[2]);
        glVertex3f(x,y,z_);
        glVertex3f(x,y,z_);
        col = 1.0 - sqrt(x*x+(y+gridSize_)*(y+gridSize_)) / (to_-from_ )*2.0;
        if (col < 0.0) col = 0.0;
        glColor3f(col*colorR_+(1.0-col)*bgcolor[0],
                  col*colorG_+(1.0-col)*bgcolor[1],
                  col*colorB_+(1.0-col)*bgcolor[2]);
        glVertex3f(x,y+gridSize_,z_);
      }
    }
  }
  glEnd();

  if (!xyPlane_) {
    glPopMatrix();
  }
  glPopAttrib(); // GL_LIGHTING
}
