/*
 This file is part of the BIAS library (Basic ImageAlgorithmS).

 Copyright (C) 2003, 2004    (see file CONTACTS for details)
 Multimediale Systeme der Informationsverarbeitung
 Institut fuer Informatik
 Christian-Albrechts-Universitaet Kiel


 BIAS is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 (at your option) any later version.

 BIAS is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with BIAS; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __SCENEGRIDGROUND_HH_
#define __SCENEGRIDGROUND_HH_

#include <bias_config.h>

#include <GLviewer/Scenes/SceneBase.hh>

namespace BIAS
{
  /**
     @class SceneGridGround 
     @brief Display a grid ground for orientation.
     @author jordt
     @ingroup g_glviewer
  */
  class BIASGLviewer_EXPORT SceneGridGround : public BIAS::SceneBase
  {
  public:

    /** @brief Create a square grid ground from position (from, from, z) to
               position (to, to, z) at constant z value with given grid size,
               i.e. grid ground is parallel to x/y plane by default. */
    SceneGridGround(double from = -1000.0, double to = 1000.0,
                    double gridSize = 50.0, double z = 0.0);

    ~SceneGridGround();

    /** @brief Initialize the square ground. Only needed if changes are to be 
               applied to the parameters. */
    void Resize(double from = -1000.0, double to = 1000.0, double gridSize = 50.0,
                double z = 0.0);

    /** @brief Set the color of the grid ground. */
    void SetColor(float r, float g, float b);

    /** @brief Specify if grid ground is parallel to x/y plane or x/z plane. */
    void DrawInXYPlane(bool xyplane);

    /** @brief Implementation of the SceneBase Draw method. */
    virtual void Draw();

    /** @brief Implementation of the SceneBase GetBoundingBox method. */
    virtual void GetBoundingBox(BIAS::Vector3<double> &minVal,
                                BIAS::Vector3<double> &maxVal);

  protected:

    double from_;
    double to_;
    double gridSize_;
    double z_;
    float colorR_, colorG_, colorB_;
    bool xyPlane_; 

    BIAS::Vector3<double> boundingBoxMin_;
    BIAS::Vector3<double> boundingBoxMax_;
  };

}

#endif // __SCENEGRIDGROUND_HH_
