#ifndef __SCENE_PLAIN_OPENSCENEGRAPH_HH_
#define __SCENE_PLAIN_OPENSCENEGRAPH_HH_


//#include <mip_config.h>
#include <Gui/biasgl.h>

#ifdef WIN32
#undef GL_VERSION_2_1
#endif

#include <osgViewer/Viewer>
#include <osgViewer/ViewerEventHandlers>
#include <osgDB/ReadFile>
#include <Base/Math/Vector3.hh>

#include <GLviewer/Scenes/SceneBase.hh>

namespace BIAS {
/** 
 * \class ScenePlainOpenSceneGraph
 * \brief Implements slim openscenegraph scene without extras
 * \attention You must call Init()!
 * \author Falko Kellner
 * \ingroup g_glviewer
 */
class BIASGLviewer_EXPORT ScenePlainOpenSceneGraph : public SceneBase {

    public:
      ScenePlainOpenSceneGraph();

      virtual ~ScenePlainOpenSceneGraph();

      int Init(unsigned int width = 1200, unsigned int height = 900);

      void SetLighting(bool on);

      osg::ref_ptr<osg::Group> GetRootNode() {
        return rootNode_;
      }

      bool Load(std::string filename);

      int AppendSubTreeFromFile(const std::string& fileName,
                                osg::ref_ptr<osg::Node>& childNode);

      int AppendSubTreeFromFile(const std::string& fileName);

      void AppendNode(osg::ref_ptr<osg::Node> node);

      void RemoveNode(osg::ref_ptr<osg::Node> node);

      void SetRootNode(osg::ref_ptr<osg::Group> newRoot);

      virtual void GetBoundingBox(BIAS::Vector3<double> &themin, BIAS::Vector3<double> &themax);
      virtual Vector3<double> GetBoundingBoxCenter();

      osgViewer::GraphicsWindow* window() {
        return embedWindow_.get();
      }
      
      void Clear();
      
    protected:

      virtual void Draw();

      osg::ref_ptr<osgViewer::Viewer> sceneView_;
      osg::observer_ptr<osgViewer::GraphicsWindow> embedWindow_;
      osg::ref_ptr<osg::Group> rootNode_;
      
      osg::Timer timer_;
      osg::Timer_t startTick_;
      osg::ref_ptr<osg::FrameStamp> frameStamp_;
      unsigned int w_,h_;

    private:
      bool initialized_;
  };

}

#endif // __SCENE_PLAIN_OPENSCENEGRAPH_HH_
