#include <GLviewer/Scenes/ScenePointLight.hh>
#include <GLviewer/GLProjectionParametersBase.hh>

#include <Gui/biasgl.h>
#include <Base/Math/Vector3.hh>


using namespace BIAS;
using namespace BIAS;
using namespace std;

ScenePointLight::
ScenePointLight()
{
  lightPos_.SetZero();
  drawLight_ = false;
  lightAsHeadALight_ = false;

  diffuse_[0] = diffuse_[1] = diffuse_[2] = diffuse_[3] = 1.0;
  specular_[0] = specular_[1] = specular_[2] = specular_[3] = 1.0;
  ambient_[0] = ambient_[1] = ambient_[2] = 0.4; ambient_[3] =  1.0;

  useAttenuation_ = false;
  attenuationFactors_[0] = 1.0f;
  attenuationFactors_[1] = 0.2f;
  attenuationFactors_[2] = 0.08f;
}


void ScenePointLight::
Draw() 
{
  glEnable(GL_LIGHTING);
  glEnable(GL_LIGHT0);

  if( lightAsHeadALight_ && camera_ != NULL) {
    lightPos_  = camera_->GetMyselfAsProjectionParameterBase()->GetC();
  }

  float c[4] = { (float)lightPos_[0], (float)lightPos_[1], (float)lightPos_[2], 1.0 };
  glLightfv(GL_LIGHT0, GL_POSITION, c);
  glLightfv(GL_LIGHT0, GL_AMBIENT, ambient_);
  glLightfv(GL_LIGHT0, GL_DIFFUSE, diffuse_);
  glLightfv(GL_LIGHT0, GL_SPECULAR, specular_); 
  
  if (useAttenuation_) {
    glLightf(GL_LIGHT0, GL_CONSTANT_ATTENUATION, attenuationFactors_[0]);
    glLightf(GL_LIGHT0, GL_LINEAR_ATTENUATION, attenuationFactors_[1]);
    glLightf(GL_LIGHT0, GL_QUADRATIC_ATTENUATION, attenuationFactors_[2]);
  }

  if (drawLight_) DrawLightPosition_();
}

void ScenePointLight::
DrawLightPosition_()
{
  glDisable(GL_LIGHTING);

  GLint ps[1];
  glGetIntegerv(GL_POINT_SIZE,ps);

  glPointSize(20);
  glColor3f(1.0, 1.0, 0.0);
  float c[4] = {(float)lightPos_[0], (float)lightPos_[1], (float)lightPos_[2], 1.0};    
  glBegin(GL_POINTS);
  glVertex4fv(c);
  glEnd();
  
  glPointSize(ps[0]);

  glEnable(GL_LIGHTING);
}
