/*
 This file is part of the BIAS library (Basic ImageAlgorithmS).

 Copyright (C) 2003, 2004    (see file CONTACTS for details)
 Multimediale Systeme der Informationsverarbeitung
 Institut fuer Informatik
 Christian-Albrechts-Universitaet Kiel


 BIAS is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 (at your option) any later version.

 BIAS is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with BIAS; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __ScenePointLight_HH__
#define __ScenePointLight_HH__

#include <bias_config.h>

#ifndef BIAS_HAVE_OPENGL
#  error You need BIAS with USE_OpenGL Please recompile BIAS.
#endif // BUILD_GLviewer


#include <GLviewer/Scenes/SceneBase.hh>
#include <GLviewer/Controller/ControlInterface.hh>
#include <Base/Math/Vector3.hh>

namespace BIAS {

  /** @class SceneBase 
      @brief Simple point light in GL.
      @ingroup g_glviewer
      @author bartczak
  */
  class BIASGLviewer_EXPORT ScenePointLight : public SceneBase
  {
  public:

    ScenePointLight();
    
    void SetLightPosition(BIAS::Vector3<double> pos) {
      lightPos_ = pos;
    }

    void SetAmbient(BIAS::Vector4<double> am) {
      ambient_[0] = am[0]; ambient_[1] = am[1];
      ambient_[2] = am[2]; ambient_[3] = am[3];
    }

    void SetDiffuse(BIAS::Vector4<double> di) {
      diffuse_[0] = di[0]; diffuse_[1] = di[1];
      diffuse_[2] = di[2]; diffuse_[3] = di[3];
    }

    void SetSpecular(BIAS::Vector4<double> sp) {
      specular_[0] = sp[0]; specular_[1] = sp[1];
      specular_[2] = sp[2]; specular_[3] = sp[3];
    }

    void SetDrawLightPosition(bool drawLight) {
      drawLight_ = drawLight;
    }

    void SetLightAsHeadLight(bool asHeadLight) {
      lightAsHeadALight_ = asHeadLight;
    }

    /** @brief Enable/disable attenuation for point light. */
    void UseAttenuation(bool useAttenuation) {
      useAttenuation_ = useAttenuation;
    }

    /** @brief Set polynomial factors for light attenuation function,
               i.e. att(d) = 1 / (a0 + a1*d + a2*d*d) with distance d. */
    void SetAttenuationFactors(double a0, double a1, double a2) {
      attenuationFactors_[0] = a0; attenuationFactors_[1] = a1;
      attenuationFactors_[2] = a2;
    }

  protected:

    virtual void Draw();

    void DrawLightPosition_();
 
    BIAS::Vector3<double> lightPos_;
    bool drawLight_;
    bool lightAsHeadALight_;
    float diffuse_[4];
    float ambient_[4];
    float specular_[4];

    bool useAttenuation_;
    float attenuationFactors_[3];

  };

}

#endif
