#include "SceneTextScreen.hh"

#ifdef BIAS_HAVE_OPENGL
#  include <Gui/biasgl.h>
#  ifdef __APPLE__
#    include <OpenGL/glu.h>
#  else // __APPLE__
#    include <GL/glu.h>
#  endif // __APPLE__
#  include <Utils/StatekeeperGL.hh>
#endif


using namespace std;
using namespace BIAS;
using namespace BIAS;

SceneTextScreen::SceneTextScreen(bool MakeDemoText): isInitialized_(false) {
  if (MakeDemoText) {
    textPositions_.push_back(HomgPoint3D(0.0,0.0,1.0));
    textContents_.push_back("Northern Star");
    textColors_.push_back(RGBf(1.0f, 0.0f, 0.0f));
  }

  fontName_ = "-*-helvetica-bold-r-normal--12-*-*-*-p-*-iso8859-1";//DrawTextGL_DEFAULT_FONTNAME;
  fontHeight_  = 12;
}
void SceneTextScreen::Free(){
  #ifdef BIAS_HAVE_FONTGL
drawer_.DeleteFont();
#endif
}
void SceneTextScreen::Draw()
{
#ifdef WIN32
  BIASWARNONCE("windows font selection mechanism not yet implmented here.");
  return;
#endif
#ifdef BIAS_HAVE_OPENGL

  if (!isInitialized_) {
#ifdef BIAS_HAVE_FONTGL
    if (drawer_.InitFont(NULL, fontName_, fontHeight_)!=0) {
      cerr<<"init of font failed ..."<<endl;
      return;
    }
#else
    BIASWARNONCE("FontGL is disabled in BIAS.");
    return;
#endif
    isInitialized_ = true;
  }
  for (unsigned int i=0; i<textContents_.size(); i++) {
    glColor3f(textColors_[i][0], textColors_[i][1], textColors_[i][2]);
#ifdef BIAS_HAVE_FONTGL
    
   drawer_.PrintWinCoords(textContents_[i],(int) textPositions_[i][0], 
                    (int)textPositions_[i][1], 0,0);
    

    glColor3f(0.0, 0.0, 1.0);
    //drawer_.Print3D(".", textPositions_[i][0], 
    //                textPositions_[i][1], textPositions_[i][2]);
    glBegin(GL_POINTS);
    glVertex3f(textPositions_[i][0], textPositions_[i][1], textPositions_[i][2]);
    glEnd();
#endif
  }
#endif
 
}
 

#ifdef BIAS_HAVE_XML2

int SceneTextScreen::XMLGetClassName(std::string& TopLevelTag,
                               double& Version) const {
  TopLevelTag = "SceneTextScreen";
  Version = 0.1;
  return 0;
}

int SceneTextScreen::XMLOut(const xmlNodePtr mainNode, XMLIO& XMLObject) const {
  for (unsigned int i=0; i<textContents_.size(); i++) {
    xmlNodePtr childNode = XMLObject.addChildNode(mainNode, "Text");
     
    XMLObject.addAttribute(childNode, "r", textColors_[i][0]);
    XMLObject.addAttribute(childNode, "g", textColors_[i][1]);
    XMLObject.addAttribute(childNode, "b", textColors_[i][2]);

    XMLObject.addAttribute(childNode, "x", textPositions_[i][0]);
    XMLObject.addAttribute(childNode, "y", textPositions_[i][1]);
    XMLObject.addAttribute(childNode, "z", textPositions_[i][2]);
    XMLObject.addAttribute(childNode, "w", textPositions_[i][3]);
    
    XMLObject.addContent(childNode, textContents_[i]);
  }
  return 0;
}

int SceneTextScreen::XMLIn(const xmlNodePtr Node, XMLIO& XMLObject) {
  textColors_.clear();
  textPositions_.clear();
  textContents_.clear();
  xmlNodePtr childNode = XMLObject.getFirstChild(Node);
  while (!(childNode==NULL)) {
    if (XMLObject.getNodeName(childNode)=="Text") {
      RGBf thecolor;
      HomgPoint3D theposition;
     
      thecolor[0] = XMLObject.getAttributeValueDouble(childNode, "r");
      thecolor[1] = XMLObject.getAttributeValueDouble(childNode, "g");
      thecolor[2] = XMLObject.getAttributeValueDouble(childNode, "b");

      theposition[0] = XMLObject.getAttributeValueDouble(childNode, "x");
      theposition[1] = XMLObject.getAttributeValueDouble(childNode, "y");
      theposition[2] = XMLObject.getAttributeValueDouble(childNode, "z");
      theposition[3] = XMLObject.getAttributeValueDouble(childNode, "w");

      textColors_.push_back(thecolor);
      textPositions_.push_back(theposition);
      string theContent = "___"+XMLObject.getNodeContentString(childNode);
      textContents_.push_back(theContent);
      cout<<"Found "<<theContent<<" at "<<theposition<<" with color "
          <<thecolor<<endl;      
    }
    childNode = XMLObject.getNextChild();
  }
  cout<<"Read "<<textColors_.size()<<" texts"<<endl;  
 
  return 0;
}

#endif // BIAS_HAVE_XML2




