#include "SceneThreeDOutWrapper.hh"
#include <Base/Debug/TimeMeasure.hh>

using namespace std;
using namespace BIAS;
using namespace BIAS;


void SceneThreeDOutWrapper::Draw()
{  
  if(threeDOutObject_ != NULL){
  	//first render point uncertainties
  	if (GetFlag(DRAW_ELLIPSOIDS)){
  		if(covarianceColor_.NormL2() != 0)
  			threeDOutObject_->OpenGLOutEllipsoids(covarianceColor_);
  		else
  			threeDOutObject_->OpenGLOutEllipsoids();
  	}
  	//render images
    threeDOutObject_->OpenGLOutImages();

    //eventually render triangles with splatting
    switch(GetTriangleMeshDrawStyle()){
    case TRIANGLE_SIMPLE :
      threeDOutObject_->OpenGLOutIndexedFaceSets();
      break;
    case TRIANGLE_SPLATTING:
      triangleSplatting_.SetCamera(GetGLProjectionParametersInterface());
      triangleSplatting_.ComputeDepthTolerance(0.02);
      triangleSplatting_.Splat();
      break;
    }   
    //set cameras covariance flag
    if(GetFlag(DRAW_CAM_ELLIPSOIDS))
    	threeDOutObject_->SetParamsDrawCameraEllipse(true);
    else
    	threeDOutObject_->SetParamsDrawCameraEllipse(false);
    //render cameras
    if (GetFlag(RENDER_CAMERA)) {
    	if(cameraColor_.NormL2() != 0)
    		threeDOutObject_->OpenGLOutPMatrices(cameraColor_);
    	else
    		threeDOutObject_->OpenGLOutPMatrices();
      threeDOutObject_->OpenGLOutWCS();
    }
    //points and lines
    threeDOutObject_->OpenGLOutPoints();
    threeDOutObject_->OpenGLOutLines();
  }
}

void SceneThreeDOutWrapper::SetFlag(unsigned long int Flag)
{
  if ( !(RenderFlags_ & Flag) )
    RenderFlags_ += Flag;
}


void SceneThreeDOutWrapper::UnsetFlag(unsigned long int Flag)
{
  if ( RenderFlags_ & Flag )
    RenderFlags_ -= Flag;
}

bool SceneThreeDOutWrapper::GetFlag(unsigned long int Flag)
{
  bool tmp=false;
  if ( RenderFlags_ & Flag ) tmp=true;
  return tmp;
}

void SceneThreeDOutWrapper::
GetBoundingBox(Vector3<double> & min, Vector3<double> & max)
{
  BIASASSERT(threeDOutObject_!=NULL);
  threeDOutObject_->GetBoundingBox(min, max);
  if (min.NormL2()!=0 || max.NormL2()!=0) {
    for (unsigned int i=0; i<3; i++) {
      min[i] -= 1e-10;
      max[i] += 1e-10;
      double dist = 0.01*(max[i]-min[i]);
      min[i] -= dist;
      max[i] += dist;
    }
  }
}

BIAS::Vector3<double>  SceneThreeDOutWrapper::
GetBoundingBoxCenter()
{
  BIASASSERT(threeDOutObject_!=NULL);
  return threeDOutObject_->GetCenterOfPoints();
}

void SceneThreeDOutWrapper::MoveScene(const double& delta) {
  //cout<<"performing translation "<<delta<<"  *  "<<velocity_<<endl;
  threeDOutObject_->Translation(delta * velocity_);
}
