# -This file generates the bias library config header 
# bias_config.h (in BIAS_BINARY_DIR) automatically 
# from bias_config.h.in (in BIAS_SOURCE_DIR).
#
# from the appropriate ..._FOUND flags
#
# -------------------------------------------------------------------------
# @author Jan Woetzel <http://www.mip.informatik.uni-kiel.de/~jw> (11/2003)
# -------------------------------------------------------------------------

# set the in and out cmake configure files: 
SET(FILE_BIAS_CONFIG_H_IN "${BIAS_SOURCE_DIR}/bias_config.h.in")
SET(FILE_BIAS_CONFIG_H "${BIAS_BINARY_DIR}/bias_config.h")

# save build seting for bias_config.h to switch to correct exansion of dllim/export on WIN32 builds. 
# was BIAS built shared or static?
IF(BUILD_SHARED_LIBS)
  SET(BIAS_BUILD_SHARED_LIBS ON)
ENDIF(BUILD_SHARED_LIBS)


#
# translate cmake ..._FOUND and USE_..  to  BIAS_HAVE_...
#

# generate BIAS_HAVE_* 
FOREACH( ExtLib
    "BLAS"
    "LAPACK"
    "1394CameraCMU"
    "DC1394"
    "DevIL"
    "DSHOW"
    "DV"
    "FFTW3"
    "GLEW"
    "GLUT"
    "GSL"
    "ImageMagickLib"
    "LIBJPEG"
    "OpenCV"
    "OpenGL"
    "FONTGL"
    "OpenEXR"
    "OPENMP"
    "PMD"
    "PMDUSB"
    "pthreads"
    "StereoI"
    "TIFF"
    "UEYE"
    "V4L"
    "VLD"
    "wxWidgets"
    "XML2"
    "ffmpeg"
    )
  STRING(TOUPPER ${ExtLib} EXTLIB )
  IF    (${ExtLib}_FOUND OR ${EXTLIB}_FOUND )
    IF   (USE_${ExtLib} OR USE_${EXTLIB} )
      SET(BIAS_HAVE_${EXTLIB} ON)
    ENDIF (USE_${ExtLib} OR USE_${EXTLIB} )
  ENDIF (${ExtLib}_FOUND OR ${EXTLIB}_FOUND )
ENDFOREACH(ExtLib)

#
# switches that do not map a lib 1:1 : 
#

# compiled with class Videsource_DCAM ?
IF   (UNIX AND BIAS_HAVE_DC1394) 
  SET(BIAS_HAVE_DCAM TRUE)
  IF(DC1394_V2)
    #MESSAGE("Found libdc1394 version 2: DC1394_V2=${DC1394_V2}")
    SET(BIAS_DC1394_V2 TRUE)
    #MESSAGE("Found libdc1394 version 2: BIAS_DC1394_V2=${BIAS_DC1394_V2}")
  ELSE(DC1394_V2)
    #MESSAGE("Using old dc1394")
  ENDIF(DC1394_V2)
ENDIF(UNIX AND BIAS_HAVE_DC1394) 
IF (WIN32 AND BIAS_HAVE_1394CAMERACMU)
  SET(BIAS_HAVE_DCAM TRUE)
ENDIF (WIN32 AND BIAS_HAVE_1394CAMERACMU)

# C++11
IF(USE_CPP11)
  SET(BIAS_HAVE_CPP11 ON)
ENDIF(USE_CPP11)

# C++14
IF(USE_CPP14)
  SET(BIAS_HAVE_CPP14 ON)
ENDIF(USE_CPP14)

# always built, now.
SET(BIAS_HAVE_GUI TRUE)

# generate .h from .h.in file
#MESSAGE("DBG generating ${FILE_BIAS_CONFIG_H}  (from ${FILE_BIAS_CONFIG_H_IN})")
CONFIGURE_FILE(${FILE_BIAS_CONFIG_H_IN} ${FILE_BIAS_CONFIG_H})
