/*
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003-2009    (see file CONTACT for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifndef ELLIPSE_HH
#define ELLIPSE_HH

#include "bias_config.h"
#include <Base/Math/Matrix2x2.hh>


namespace BIAS {

  /** Ellipse in 2D with dfifferent representations.
    * \author bartczak 06/2011
   **/
class BIASGeometry_EXPORT Ellipse
{
public:
    Ellipse();
    /** Initialisation via affine transformation.  **/
    void SetParametric(const double scaleX,
                       const double scaleY,
                       const double rotAngleDeg,
                       const double centerX,
                       const double centerY);

    /** Uses quadric represenation. **/
    void SetImplicit(const BIAS::Matrix2x2<double>& implicitForm,
                     const double centerX, const double centerY);

    /** Uses quadric represenation and adapts size of ellipse. **/
    void SetImplicit(const BIAS::Matrix2x2<double>& implicitForm,
                     const double F,
                     const double centerX, const double centerY);

    /** Assumes ellipse to be a certain iso value line on a
      * 2D gaussian without cross correlation (no rotation).
      * Gaussian is g(x) = 1/N exp(-0.5 x^t Q x).
      * This ellipse will be Q = I*(1/sigmaX^2 , 1/sigmaY^2)
      * Ellipse calculated has the implicit form:
      * x^t Q x - F = 0.
    **/
    void SetGaussianFilterRange(const double& sigmaX,
                                const double& sigmaY,
                                const double F = 9.0);

    void CalculateAxisAlignedBoundingBox(double& minX, double& minY,
                                         double& maxX, double& maxY) const;


    BIAS::Matrix2x2<double> ReturnImplicitRepresentation() const;
    BIAS::Matrix2x2<double> ReturnParametricTransformation() const;

private:
    //different representations

    double centerX_;
    double centerY_;
    BIAS::Matrix2x2<double> parametricTransform_;

    void CalcImplicitFromParameteric_();
    void CalcParametericFromImplicit_();


    //implicit representation
    BIAS::Matrix2x2<double> implicitForm_;


};

}
#endif // ELLIPSE_HH
