/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003-2009    (see file CONTACT for details)
Multimediale Systeme der Informationsverarbeitung
Institut fuer Informatik
Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


/**
   @example ExampleProjectionParametersFactory.cpp
   @relates ProjectionParametersFactory
   @brief small example demonstrating the ProjectionParametersFactory
   @ingroup g_examples
   @author woelk 12/2006
*/

#include <Geometry/ProjectionParametersFactory.hh>

using namespace BIAS;
using namespace std;


int main(int argc, char *argv[])
{
  ProjectionParametersFactory ppf;
  if (argc!=2){
    cerr << "usage: \n"<<argv[0]<<" <projection parameters type>\n"
         << "\nvalid types are: ... "<<endl;
    vector<string> names;
    ppf.GetValidClassNames(names);
    for (unsigned i=0; i<names.size(); i++){
      cout << names[i] << endl;
    }
    return -1;
  }

  ProjectionParametersBase *ppb = ppf.Create(argv[1]);
  if (!ppb){
    cerr << "error creating \""<<argv[1]<<"\"\n";
    return -2;
  }

  Vector3<double> C(0,0,0);
  ppb->SetC(C);
  Quaternion<double> Q(0,0,0,1);
  ppb->SetQ(Q);

  cout << "created projection parameters: "<<*ppb<<endl;
  ostringstream oss;
  oss << argv[1]<<".xml";

#ifdef BIAS_HAVE_XML2
  int res = ppb->XMLWrite(oss.str());

  if (res!=0){
    cerr << "error writing \""<<oss.str()<<"\"\n";
    return -3;
  }
  cout << "XMLWrite wrote \""<<oss.str()<<"\"\n";
  
  ProjectionParametersBase *read = ppf.Load(oss.str());
  cout << "read projection parameters: "<<*read<<endl;

#else
  cout<<"did not XMLWrite because BIAS_HAVE_XML2 not available."<<endl;
#endif
  return 0;
}
