#ifndef __CameraViewExternals_h__
#define __CameraViewExternals_h__

// ----------------------------------------------------------
#include <bias_config.h>
#include <iostream>
// BIAS
#include <Base/Math/Vector3.hh>


namespace BIAS {

  /** @class CameraViewExternals
      \brief describes external params of a camera view
      Used e.g. OpenGL and manipularted by controller classes, e.g. trackball.
      \author Jan Woetzel */
  class BIASGui_EXPORT CameraViewExternals {
  public:
    virtual ~CameraViewExternals(){}; ///< destructor

    // ctors
    CameraViewExternals();

    CameraViewExternals(const BIAS::Vector3<double> & C,
      const BIAS::Vector3<double> & up,
      const BIAS::Vector3<double> & viewAt);


    std::ostream & Print(std::ostream & os=std::cout );

#ifdef BIAS_HAVE_OPENGL
    void DisplayGL() const;
    void DisplayViewAt(const double & size=1.0) const;
#endif // BIAS_HAVE_OPENGL

    void InitMembers()
    {
      C.Set(0,0,1);
      up.Set(0,1,0);
      viewAt.SetZero();
    }

  public:
    // external params : pose
    BIAS::Vector3<double> C;
    BIAS::Vector3<double> up;
    BIAS::Vector3<double> viewAt;

  }; // CameraViewExternals

} // namespace BIAS
#endif
