/* This file is part of the BIAS library (Basic ImageAlgorithmS).

   Copyright (C) 2003-2009    (see file CONTACT for details)
   Vision N GmbH
   Schauenburgerstr. 116
   24118 Kiel
   
   BIAS is free software; you can redistribute it and/or modify
   it under the terms of the GNU Lesser General Public License as published by
   the Free Software Foundation; either version 2.1 of the License, or
   (at your option) any later version.
   
   BIAS is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU Lesser General Public License for more details.
   
   You should have received a copy of the GNU Lesser General Public License
   along with BIAS; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA*/


/**	Simple example for BIAS::ImageCanvasGLBase(Base) usage.
@author: Jan Woetzel */
#include <bias_config.h>

//#include <Base/Common/LeakChecking.h>

#include "ExampleImageCanvasGLBase.hh"

// BIAS
#include <Gui/ConsoleRedirectIO.hh>
#include <Base/Image/Image.hh>
#include <Base/Image/ImageIO.hh>

// WX
#include <wx/sizer.h>
#include <wx/log.h>

//-----------------

// global var. for mode
int g_mode=0;
const int MODE_MAX=1; // 0,1,..

//-----------------

using namespace std;

// required for wx RTTI dynamic construction: 
IMPLEMENT_APP(ExampleImageCanvasGLBaseApp)

// --------------------------------------------------------

bool ExampleImageCanvasGLBaseApp::OnInit()
{
	// utility: create a Redirection console for cout, cerr etc.
	ConsoleRedirectIO();
	// testing output: 
	cout<<"Hello cout"<<endl;
	cout<<"Hello cerr"<<endl;

	MyFrame *frame = new MyFrame( wxT("ExampleImageCanvasGLBase"), wxPoint(0,0), wxSize(640,480) );
	frame->Show(TRUE);
	SetTopWindow(frame);

	return TRUE;
}


// --------------------------------------------------------
BEGIN_EVENT_TABLE(MyFrame, wxFrame)
EVT_MENU(ID_QUIT, MyFrame::OnQuit)
EVT_MENU(ID_ABOUT, MyFrame::OnAbout)
EVT_MENU(ID_MODECHANGE, MyFrame::OnModeChange)
EVT_MENU(ID_LOADIMAGE, MyFrame::OnLoadImage)
END_EVENT_TABLE()
// --------------------------------------------------------


MyFrame::MyFrame(const wxString& title, const wxPoint& pos, const wxSize& size)
: wxFrame((wxFrame *)NULL, -1, title, pos, size)
,p_canvas1(NULL)
,p_canvas2(NULL)
{
	// Create a simple top Menu
	wxMenu *menuFile = new wxMenu;  
	menuFile->Append( ID_ABOUT, wxT("&About...") );
	menuFile->Append( ID_LOADIMAGE, wxT("&Load Image...\tCtrl-L") );
  // TODO menuFile->Append( ID_MODECHANGE, wxT("Change &Mode...\tCtrl-M") );
	menuFile->AppendSeparator();
	menuFile->Append( ID_QUIT, wxT("E&xit") );

	wxMenuBar *menuBar = new wxMenuBar;
	menuBar->Append(menuFile, wxT("&File"));
	SetMenuBar(menuBar);

	// Create a simple bottom status bar 
	CreateStatusBar(1);

	// Canvas to test: 
	p_canvas1=new BIAS::ImageCanvasGLBase(this, -1);
	// create aseond canvas that shares the GL context: 
	p_canvas2=new BIAS::ImageCanvasGLBase(this, p_canvas1->GetContext(),-1 );

	// put them into sizers: 
	wxBoxSizer * sizer = new wxBoxSizer(wxHORIZONTAL);
	sizer->Add( p_canvas1 ,1,wxEXPAND|wxALL);
#if wxCHECK_VERSION(2, 6, 2)
	sizer->AddSpacer(4); // available in 2.6.2 but not 2.4.2 (JW)
#endif
	sizer->Add(p_canvas2 ,1,wxEXPAND|wxALL);

	// limit min. size:
	sizer->SetItemMinSize(p_canvas1, 200,100);
	sizer->SetItemMinSize(p_canvas2, 200,100);
	sizer->SetSizeHints(this);

	// set sizer and triggers Layout()
	SetSizerAndFit(sizer);

	// now work with the canvases: 
	// enable Mipmapping just for one of them to see the effect: 
	p_canvas1->EnableMipMapping(true);

	// HACK> context is created too late
#ifdef WIN32
  this->LoadImages();
#endif // WIN32
	
	// set the image to display:
	//p_canvas1->Set(img);
	//p_canvas2->Set(img); // allocates GPU texture meory twice
	// TODO: share texobjs "id" instead: See wglShareLists (JW)
	//p_canvas2->Set(p_canvas1->GetTexobj() );
}


void MyFrame::OnQuit(wxCommandEvent& ){ 
	Close(TRUE); 
}


void MyFrame::OnAbout(wxCommandEvent& )
{
	wxString title(wxT("About"));
	wxString msg; // message
	msg<<wxT("About - ")<<AsciiToWx(FUNCNAME)<<wxT("\n");
	msg<<wxT("author: Jan Woetzel (c) 2003-2005\n");
	msg<<wxT("build: ")<<AsciiToWx(__DATE__)<<wxT(" ")<<AsciiToWx(__TIME__)<<wxT("\n");
	msg<<wxT("from ")<<AsciiToWx(__FILE__)<<wxT("\n");
	msg<<wxT("GUI using: ")<<wxVERSION_STRING<<wxT("\n");
	msg<<wxT("\n");
	COUT(msg);
	wxMessageBox(msg, title, wxICON_INFORMATION | wxOK );
}


void MyFrame::OnModeChange(wxCommandEvent& )
{
  CALLINFO;
  g_mode++;
  if (g_mode>MODE_MAX)
    g_mode=0;
}


void MyFrame::LoadImages()
{
  CALLINFO;
  // set the image to display:
  // an image we want to display using OpenGL:
  BIAS::Image<unsigned char> img;
  string filename(BIAS_TESTS_DATA "r4.jpg");
  if (0!=BIAS::ImageIO::Load( filename,img)){
    BIASERR("could not load image"<<filename);
    return;
  };
  cout<<"loaded "<<filename<<" for display "<<endl;

  // now work with the canvases: 
  if (p_canvas1!=NULL) {
    // set the image to display:
    p_canvas1->Set(img);       
  } else {
    BIASERR("p_canvas1 is NULL");
  }
  
  if (p_canvas2!=NULL) {
    // set the image to display:
    p_canvas2->Set(img);       
  } else { 
    BIASERR("p_canvas2 is NULL");
  }
  
  COUT(FUNCNAME<<" done."<<endl);
}



void MyFrame::OnLoadImage(wxCommandEvent& )
{
  //CALLINFO;
  LoadImages();
}

