/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004    (see file CONTACTS for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


/**
   @example ParamGUISample1.cpp
   @relates ParamGUI
   @brief Example for using the parameter gui classes
   @ingroup g_examples
   @author MIP
*/

#include <wx/wx.h>

#include <Base/Common/BIASpragma.hh>

#include <Gui/ParamGUI.hh>
#include <Gui/StringConv.hh>
#include <Utils/Param.hh>
#include <iostream>
#include <vector>
#include <string>

using namespace BIAS;
using namespace std;

/** \cond HIDDEN_SYMBOLS
 *  \class MyAlgorithm */
class MyAlgorithm : public ParamCallbackInterface
{
public:
  MyAlgorithm(){coollevel = NULL;};
  ~MyAlgorithm(){};

  void  ParamAdd(Param &p) {
    coollevel = p.AddParamInt("coollevel","Really cool threshold",0,0,100,0,1);
    cooldbl   = p.AddParamDouble("cooldbl","Really cool double ",0,0,100,0,1);
    // set callback class instance to run on parameter change
    p.RegisterCallbackObject("coollevel",this); 
    p.RegisterCallbackObject("cooldbl",  this); 
  };


  void ParameterChanged(const std::string &name, const void *data){
    cout <<"Here is MyAlgorithm! My parameter \""<<name<<"\" changed"<<endl;
    if (data == coollevel) 
      cout <<"Hey, it was \"coollevel\", new cool level is: "<<*coollevel
      <<endl;
  };

protected:
  int *coollevel;
  double *cooldbl;

};


class MyApp : public wxApp 
{
public:
  virtual bool OnInit();
};

DECLARE_APP(MyApp)


class MyFrame : public wxFrame
{
public:
  MyFrame();
  ~MyFrame();
  ParamGUI param_;
};

// uncool global, for testing
MyAlgorithm *algo;

// implementation

IMPLEMENT_APP(MyApp) 

bool MyApp::OnInit() 
{ 
  MyFrame *frame = new MyFrame();
  frame->Show(TRUE); 
  SetTopWindow(frame);
  return true;
}

MyFrame::MyFrame() 
: wxFrame(NULL, -1, _T("ParamGUISample1"), wxDefaultPosition,
          wxSize(400,600) )
{
  param_.AddParamInt("SampleInt1","Hilfetext zu SampleInt1",0,-100,100,0,1);
  param_.AddParamBool("SampleBool1","Hilfetext zu SampleBool1",false,0,1);
  param_.AddParamDouble("SampleDouble1","Hilfetext zu SampleDouble1");
  param_.AddParamString("SampleString1","Hilfetext zu SampleString1");

  vector<string> enums;
  enums.push_back("Enum 1");
  enums.push_back("Enum 2");
  enums.push_back("Enum 3");
  enums.push_back("Enum 4");

  param_.AddParamEnum("SampleEnum","Hilfetext zu SampleEnum",enums);

  algo = new MyAlgorithm();
  algo->ParamAdd(param_);

  param_.ParseCommandLine(wxTheApp->argc,WxToAsciiArray(wxTheApp->argv,wxTheApp->argc));
  param_.ShowData();
  /* You can get the panel via: (commented out to avoid set-but-not-used warning)
  wxPanel *MyPanel;
  MyPanel = param_.GetPanel(this,1);
  */
}

MyFrame::~MyFrame()
{
  param_.ShowData();  
}

/** \endcond */
