/*
 This file is part of the BIAS library (Basic ImageAlgorithmS).

 Copyright (C) 2003-2009    (see file CONTACT for details)
 Multimediale Systeme der Informationsverarbeitung
 Institut fuer Informatik
 Christian-Albrechts-Universitaet Kiel


 BIAS is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 (at your option) any later version.

 BIAS is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with BIAS; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <Gui/FilterDialogCannyEdge.hh>
#include <Base/Image/ImageConvert.hh>

using namespace BIAS;

template <class InputST, class OutputST>
FilterDialogCannyEdge<InputST, OutputST>
::FilterDialogCannyEdge(const Image<InputST>& origImage,
                        wxWindow* parent,
                        wxWindowID id,
                        const wxString& title,
												const wxString& name) 
: FilterDialogBase<InputST, OutputST>(origImage, parent, id, title, name),
  CEFilter(CannyEdge<InputST, OutputST>()),
  Sigma(1.4f),
  Low(0),
  High(20),
  Edge(1),
	CEParamSizer(new wxBoxSizer(wxVERTICAL)),
  TRLowSizer(new wxBoxSizer(wxHORIZONTAL)),
  TRHighSizer(new wxBoxSizer(wxHORIZONTAL)),
  EdgeWidthSizer(new wxBoxSizer(wxHORIZONTAL))
{
  this->SigmaPanel= new PanelForDoubleValueWx(this,
																							BIAS::ID_PANEL_DOUBLE_VAL,
																							wxT("Sigma:"),
																							0.0,
																							3.0,
																							this->Sigma);

  this->TRLowSpinCtrl = new wxSpinCtrl(this,
																			 this->ID_SPINCTRL_CE_TR_LOW,
																			 wxEmptyString,
																			 wxDefaultPosition,
																			 wxDefaultSize,
																			 wxSP_ARROW_KEYS,
																			 0,
																			 20,
																			 this->Low);
  this->TRHighSpinCtrl = new wxSpinCtrl(this,
																				this->ID_SPINCTRL_CE_TR_HIGH,
																				wxEmptyString,
																				wxDefaultPosition,
																				wxDefaultSize,
																				wxSP_ARROW_KEYS,
																				20,
																				40,
																				this->High);
  this->EdgeWidthSpinCtrl = new wxSpinCtrl(this,
																					 this->ID_SPINCTRL_CE_EDGE_WIDTH,
																					 wxEmptyString,
																					 wxDefaultPosition,
																					 wxDefaultSize,
																					 wxSP_ARROW_KEYS,
																					 1,
																					 10,
																					 this->Edge);
  this->TRLowSlider = new wxSlider(this,
																	 this->ID_SLIDER_CE_TR_LOW,
																	 Low,
																	 0,
																	 20,
																	 wxDefaultPosition,
																	 wxSize(204, -1),
																	 wxSL_HORIZONTAL | wxSL_AUTOTICKS);
  this->TRHighSlider = new wxSlider(this,
																		this->ID_SLIDER_CE_TR_HIGH,
																		this->High,
																		20,
																		40,
																		wxDefaultPosition,
																		wxSize(204, -1),
																		wxSL_HORIZONTAL | wxSL_AUTOTICKS);
  this->EdgeWidthSlider = new wxSlider(this,
																			 this->ID_SLIDER_CE_EDGE_WIDTH,
																			 this->Edge,
																			 1,
																			 10,
																			 wxDefaultPosition,
																			 wxSize(204, -1),
																			 wxSL_HORIZONTAL | wxSL_AUTOTICKS);
  this->TRLowLabel = new wxStaticText(this,
																			wxID_STATIC,
																			wxT("Low:"),
																			wxDefaultPosition,
																			wxDefaultSize,
																			0);
  this->TRHighLabel = new wxStaticText(this,
																			 wxID_STATIC,
																			 wxT("High:"),
																			 wxDefaultPosition,
																			 wxDefaultSize,
																			 0);
  this->EdgeWidthLabel = new wxStaticText(this,
																					wxID_STATIC,
																					wxT("Edge Width:"),
																					wxDefaultPosition,
																					wxDefaultSize,
																					0);

  // add elements for low threshold into sizer
  this->TRLowSizer->Add(this->TRLowLabel,
                        0,
                        wxALIGN_CENTER_VERTICAL | wxALIGN_LEFT | wxALL,
                        5);
  this->TRLowSizer->AddSpacer(45);
  this->TRLowSizer->Add(this->TRLowSpinCtrl,
                        0,
                        wxALIGN_CENTER_VERTICAL
                          | wxALIGN_CENTER_HORIZONTAL
                          | wxALL,
                        5);
  this->TRLowSizer->Add(this->TRLowSlider,
                        0,
                        wxALIGN_CENTER_VERTICAL | wxALL,
                        5);

  // add elements for high threshold into sizer
  this->TRHighSizer->Add(this->TRHighLabel,
                         0,
                         wxALIGN_CENTER_VERTICAL | wxALIGN_LEFT | wxALL,
                         5);
  this->TRHighSizer->AddSpacer(42);
  this->TRHighSizer->Add(this->TRHighSpinCtrl,
                         0,
                         wxALIGN_CENTER_VERTICAL
                           | wxALIGN_CENTER_HORIZONTAL
                           | wxALL,
                         5);
  this->TRHighSizer->Add(this->TRHighSlider,
                         0,
                         wxALIGN_CENTER_VERTICAL | wxALL,
                         5);

  // add elements for edge width into sizer
  this->EdgeWidthSizer->Add(this->EdgeWidthLabel,
                            0,
                            wxALIGN_CENTER_VERTICAL | wxALIGN_LEFT | wxALL,
                            5);
  this->EdgeWidthSizer->Add(this->EdgeWidthSpinCtrl,
                            0,
                            wxALIGN_CENTER_VERTICAL
                              | wxALIGN_CENTER_HORIZONTAL
                              | wxALL,
                            5);
  this->EdgeWidthSizer->Add(this->EdgeWidthSlider,
                            0,
                            wxALIGN_CENTER_VERTICAL | wxALL,
                            5);

  // put sizers for param values into this sizer and also the sigma panel
  this->CEParamSizer->Add(this->SigmaPanel, 0, wxALIGN_LEFT | wxGROW | wxALL, 0);
  this->CEParamSizer->Add(this->TRLowSizer, 0, wxALIGN_LEFT | wxGROW | wxALL, 0);
  this->CEParamSizer->Add(this->TRHighSizer, 0, wxALIGN_LEFT | wxGROW | wxALL, 0);
  this->CEParamSizer->Add(this->EdgeWidthSizer, 0, wxALIGN_LEFT | wxGROW | wxALL, 0);

  // now add ceparamsizer to the main param sizer from the base class
  wxBoxSizer* paramsizer = this->GetParamSizer();
  paramsizer->Add(this->CEParamSizer,
                  0,
                  wxALIGN_CENTER_VERTICAL | wxGROW | wxALL,
                  0);

  // connect events to handler

  this->Connect(BIAS::ID_BTN_FILTER_FDB,
                wxEVT_COMMAND_BUTTON_CLICKED,
                wxCommandEventHandler(FilterDialogCannyEdge::OnFilterButton));

  this->Connect(this->ID_SPINCTRL_CE_TR_LOW,
                wxEVT_COMMAND_SPINCTRL_UPDATED,
                wxSpinEventHandler(FilterDialogCannyEdge::OnSpinCtrlTRLow));

  this->Connect(this->ID_SPINCTRL_CE_TR_HIGH,
                  wxEVT_COMMAND_SPINCTRL_UPDATED,
                  wxSpinEventHandler(FilterDialogCannyEdge::OnSpinCtrlTRHigh));

  this->Connect(this->ID_SPINCTRL_CE_EDGE_WIDTH,
                  wxEVT_COMMAND_SPINCTRL_UPDATED,
                  wxSpinEventHandler(FilterDialogCannyEdge::OnSpinCtrlEdgeWidth));

  this->Connect(this->ID_SLIDER_CE_TR_LOW,
                wxEVT_SCROLL_CHANGED,
                wxScrollEventHandler(FilterDialogCannyEdge::OnSliderTRLow));

  this->Connect(this->ID_SLIDER_CE_TR_HIGH,
                  wxEVT_SCROLL_CHANGED,
                  wxScrollEventHandler(FilterDialogCannyEdge::OnSliderTRHigh));

  this->Connect(this->ID_SLIDER_CE_EDGE_WIDTH,
                  wxEVT_SCROLL_CHANGED,
                  wxScrollEventHandler(FilterDialogCannyEdge::OnSliderEdgeWidth));

  this->GetParent()->Disable();
  this->CreateStatusBar(1);
  this->Fit();
  this->Centre();

}


template <class InputST, class OutputST>
FilterDialogCannyEdge<InputST, OutputST>::~FilterDialogCannyEdge()
{
  if (!this->GetParent()->IsEnabled()) {
    this->GetParent()->Enable();
  }
}


/******************************************************************************/
/* ============================== protected ================================= */
/******************************************************************************/

template <class InputST, class OutputST>
void FilterDialogCannyEdge<InputST, OutputST>
::OnFilterButton(wxCommandEvent& WXUNUSED(event))
{
  // disable some buttons to avoid interference, e.g. pressing preview during
  // filtering...
  this->EnableSaveButton(false);
  this->EnableFilterButton(false);
  this->EnablePreviewButton(false);
  this->EnableResetButton(false);
  this->EnableCancelButton(false);

  float sigma = (float)(this->SigmaPanel->GetParamValue());
  unsigned int low    = this->TRLowSpinCtrl->GetValue();
  unsigned int high   = this->TRHighSpinCtrl->GetValue();
  unsigned int edge   = this->EdgeWidthSpinCtrl->GetValue();

  bool dofilter = !((this->Sigma == sigma)
                    && (this->Low == low)
                    && (this->High == high)
                    && (this->Edge == edge)
                    && this->GetImageIsFiltered());

  if (dofilter) {

    Image<InputST> src = this->GetOriginalImage();
    Image<InputST> srcImg;
    bool planar = src.IsPlanar();

    if (0 == ImageConvert::Convert(src, srcImg, ImageBase::CM_Grey, planar)) {

      this->CEFilter.SetGaussSigma(sigma);
      this->CEFilter.SetThresholdsRelative(low, high);
      this->CEFilter.SetEdgeWidth(edge);
      Image<OutputST> dstImg;

      if (0 == this->CEFilter.Filter(srcImg, dstImg)) {

        this->SetFilteredImage(dstImg);
        this->SetImageIsFiltered(true);
        this->Sigma = sigma;
        this->Low   = low;
        this->High  = high;
        this->Edge  = edge;
        this->SetStatusText(wxT("Image SuccessFully Filtered"));

      } else {

        BIASERR("FilterDialogCannyEdge::OnFilterButton: Filter Error");
        this->SetImageIsFiltered(false);
        this->SetStatusText(wxT("ERROR during FILTER"));

      } // end if else filter

    } else {

      BIASERR("FilterDialogCannyEdge::OnFilterButton: Error Converting To Grey");
      this->SetStatusText(wxT("ERROR during CONVERT"));

    } // end if else convert

  } else {

    wxString msg(wxT("Nothing Done To The Filtered Image"));
    msg += wxT(" (Already Filtered With The Same Values?)");
    this->SetStatusText(msg);

  } // end if else dofilter


  // enable the buttons again
  this->EnableSaveButton();
  this->EnableFilterButton();
  this->EnablePreviewButton();
  this->EnableResetButton();
  this->EnableCancelButton();
}


template <class InputST, class OutputST>
void FilterDialogCannyEdge<InputST, OutputST>
::OnResetButton(wxCommandEvent& event)
{
  // call the Base's class OnResetButton() method and pass the event
  this->FilterDialogBase<InputST, OutputST>::OnResetButton(event);
  this->SetImageIsFiltered(false);
  this->SetStatusText(wxT("Filtered Image Reseted"));
}


/******************************************************************************/
/* =============================== private ================================== */
/******************************************************************************/

template <class InputST, class OutputST>
void FilterDialogCannyEdge<InputST, OutputST>
::OnSpinCtrlTRLow(wxSpinEvent& WXUNUSED(event))
{
  this->TRLowSlider->SetValue(this->TRLowSpinCtrl->GetValue());
}


template <class InputST, class OutputST>
void FilterDialogCannyEdge<InputST, OutputST>
::OnSpinCtrlTRHigh(wxSpinEvent& WXUNUSED(event))
{
  this->TRHighSlider->SetValue(this->TRHighSpinCtrl->GetValue());
}


template <class InputST, class OutputST>
void FilterDialogCannyEdge<InputST, OutputST>
::OnSpinCtrlEdgeWidth(wxSpinEvent& WXUNUSED(event))
{
  this->EdgeWidthSlider->SetValue(this->EdgeWidthSpinCtrl->GetValue());
}


template <class InputST, class OutputST>
void FilterDialogCannyEdge<InputST, OutputST>
::OnSliderTRLow(wxScrollEvent& WXUNUSED(event))
{
  this->TRLowSpinCtrl->SetValue(this->TRLowSlider->GetValue());
}


template <class InputST, class OutputST>
void FilterDialogCannyEdge<InputST, OutputST>
::OnSliderTRHigh(wxScrollEvent& WXUNUSED(event))
{
  this->TRHighSpinCtrl->SetValue(this->TRHighSlider->GetValue());
}


template <class InputST, class OutputST>
void FilterDialogCannyEdge<InputST, OutputST>
::OnSliderEdgeWidth(wxScrollEvent& WXUNUSED(event))
{
  this->EdgeWidthSpinCtrl->SetValue(this->EdgeWidthSlider->GetValue());
}


/* =================== supported storagetype combinations =================== */
namespace BIAS{
//template class FilterDialogCannyEdge<unsigned char, unsigned char>;
template class FilterDialogCannyEdge<unsigned char, float>;
//template class FilterDialogCannyEdge<float, unsigned char>;
template class FilterDialogCannyEdge<float, float>;
}
