/*
 This file is part of the BIAS library (Basic ImageAlgorithmS).

 Copyright (C) 2003-2009    (see file CONTACT for details)
 Multimediale Systeme der Informationsverarbeitung
 Institut fuer Informatik
 Christian-Albrechts-Universitaet Kiel


 BIAS is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 (at your option) any later version.

 BIAS is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with BIAS; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __FILTER_DIALOG_CANNY_EDGE_HH_
#define __FILTER_DIALOG_CANNY_EDGE_HH_

#include <Gui/FilterDialogBase.hh>
#include <Gui/PanelForDoubleValueWx.hh>
#include <Filter/CannyEdge.hh>
#include <wx/spinctrl.h>
#include <wx/slider.h>
#include <wx/stattext.h>

namespace BIAS {

/**
 * @class FilterDialogCannyEdge
 * @ingroup g_gui
 * @brief Implementation of a CannyEdge filter dialog
 * @author jfernand 08/09
 */
template <class InputST, class OutputST>
class BIASGui_EXPORT FilterDialogCannyEdge : public FilterDialogBase<InputST, OutputST>
{
public:

  FilterDialogCannyEdge(const Image<InputST>& origImage,
                        wxWindow* parent,
                        wxWindowID id = BIAS::ID_WIN_CANNY_EDGE,
                        const wxString& title = wxT("Canny Edge Filter Dialog"),
                        const wxString& name = wxT("FilterDialogCannyEdge"));
  virtual ~FilterDialogCannyEdge();


protected:
  /**
   * concrete Implementation of the pure OnFilterButton() from the base class.
   */
  void OnFilterButton(wxCommandEvent& WXUNUSED(event));

  void OnResetButton(wxCommandEvent& event);


private:

  /**
   * The Core: our CannyEdge filter
   */
  CannyEdge<InputST, OutputST> CEFilter;

  //Params: Needed for some checks to avoid necessary filtering
  float Sigma;
  unsigned int Low;
  unsigned int High;
  unsigned int Edge;

  PanelForDoubleValueWx* SigmaPanel;

  /**
   * This sizer holds TRLowSizer, TRHighSizer, EdgeWidthSizer and SigmaPanel.
   * This will be added to the base class' ParamSizer.
   * Orientation: wxVERTICAL
   */
  wxBoxSizer* CEParamSizer;

  /**
   * This sizer hold the elements for low threshold param input.
   * Orientation: wxHORIZONTAL
   */
  wxBoxSizer* TRLowSizer;

  /**
   * This sizer holds the elements for high threshold param input.
   * Orientation: wxHORIZONTAL
   */
  wxBoxSizer* TRHighSizer;

  /**
   * This sizer holds
   */
  wxBoxSizer* EdgeWidthSizer; // horizontal

  wxSpinCtrl* TRLowSpinCtrl;
  wxSpinCtrl* TRHighSpinCtrl;
  wxSpinCtrl* EdgeWidthSpinCtrl;

  wxSlider* TRLowSlider;
  wxSlider* TRHighSlider;
  wxSlider* EdgeWidthSlider;

  wxStaticText* TRLowLabel;
  wxStaticText* TRHighLabel;
  wxStaticText* EdgeWidthLabel;

  // handlers for some events
  void OnSpinCtrlTRLow(wxSpinEvent& WXUNUSED(event));
  void OnSpinCtrlTRHigh(wxSpinEvent& WXUNUSED(event));
  void OnSpinCtrlEdgeWidth(wxSpinEvent& WXUNUSED(event));

  void OnSliderTRLow(wxScrollEvent& WXUNUSED(event));
  void OnSliderTRHigh(wxScrollEvent& WXUNUSED(event));
  void OnSliderEdgeWidth(wxScrollEvent& WXUNUSED(event));

  enum FilterDialogCannyEdge_IDs {
    ID_SPINCTRL_CE_TR_LOW = 68923,
    ID_SPINCTRL_CE_TR_HIGH,
    ID_SPINCTRL_CE_EDGE_WIDTH,

    ID_SLIDER_CE_TR_LOW,
    ID_SLIDER_CE_TR_HIGH,
    ID_SLIDER_CE_EDGE_WIDTH
  };

};

} // end namespace BIAS


#endif // __FILTER_DIALOG_CANNY_EDGE_HH_
