/*
 This file is part of the BIAS library (Basic ImageAlgorithmS).

 Copyright (C) 2003-2009    (see file CONTACT for details)
 Multimediale Systeme der Informationsverarbeitung
 Institut fuer Informatik
 Christian-Albrechts-Universitaet Kiel


 BIAS is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 (at your option) any later version.

 BIAS is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with BIAS; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <Gui/FilterDialogGauss.hh>

using namespace BIAS;

template <class InputST, class OutputST>
FilterDialogGauss<InputST, OutputST>
::FilterDialogGauss(const Image<InputST>& origImage,
                    wxWindow* parent,
                    wxWindowID id,
                    const wxString& title,
                    const wxString& name)
: FilterDialogBase<InputST, OutputST>(origImage, parent, id, title, name),
  GaussFilter(Gauss<InputST, OutputST>()),
  SigmaAndRatioSizer(new wxBoxSizer(wxVERTICAL)),
  Sigma(0.0),
  Ratio(0.0)
{
	this->SigmaPanel = new PanelForDoubleValueWx(this,
																							 BIAS::ID_PANEL_DOUBLE_VAL,
																							 wxT("Sigma:"),
																							 0.0,
																							 1.0,
																							 0.0);
  this->RatioPanel = new PanelForDoubleValueWx(this,
																							 BIAS::ID_PANEL_DOUBLE_VAL,
																							 wxT("Ratio: "),
																							 -1.0,
																							 1.0,
																							 0.0);
  this->SigmaAndRatioSizer->Add(this->SigmaPanel,
                                0,
                                wxALIGN_CENTER_HORIZONTAL | wxGROW | wxALL,
                                0);
  this->SigmaAndRatioSizer->Add(this->RatioPanel,
                                0,
                                wxALIGN_CENTER_HORIZONTAL | wxGROW | wxALL,
                                0);
  wxBoxSizer* paramSizer = this->GetParamSizer();
  paramSizer->Add(this->SigmaAndRatioSizer, 1, wxALL, 0);

  this->Connect(BIAS::ID_BTN_FILTER_FDB,
                wxEVT_COMMAND_BUTTON_CLICKED,
                wxCommandEventHandler(FilterDialogGauss::OnFilterButton));

  this->GetParent()->Disable();
  this->CreateStatusBar(1);
  this->Fit();
  this->Centre();

}


template <class InputST, class OutputST>
FilterDialogGauss<InputST, OutputST>::~FilterDialogGauss()
{
  if (!this->GetParent()->IsEnabled()) {
    this->GetParent()->Enable();
  }
}


/******************************************************************************/
/* ============================== protected ================================= */
/******************************************************************************/


template <class InputST, class OutputST>
void FilterDialogGauss<InputST, OutputST>
::OnFilterButton(wxCommandEvent& WXUNUSED(event))
{
  // disable some buttons to avoid interference, e.g. pressing preview during
  // filtering...
  this->EnableSaveButton(false);
  this->EnableFilterButton(false);
  this->EnablePreviewButton(false);
  this->EnableResetButton(false);
  this->EnableCancelButton(false);

  double sigma_tmp = this->SigmaPanel->GetParamValue();
  double ratio_tmp = this->RatioPanel->GetParamValue();

  if ((sigma_tmp > 0.0) && (ratio_tmp <= 0.0)) {

    // this combination will crash the programm...so we set a message
    this->SetStatusText(wxT("Invalid Combination: Sigma > 0 and Ratio <= 0"));

  } else if ((this->Sigma == sigma_tmp)
             && (this->Ratio == ratio_tmp)
             && this->GetImageIsFiltered()) {

    this->SetStatusText(wxT("Nothing Done To The Filtered Image"));

  } else {

    this->GaussFilter.SetSigma(sigma_tmp);
    this->GaussFilter.SetRatio(ratio_tmp);

    Image<OutputST> dstImg;

    if (0 != this->GaussFilter.Filter(this->GetOriginalImage(), dstImg)) {

      BIASERR("FilterDialogGauss::OnFilterButton: Filter Error");
      this->SetImageIsFiltered(false);
      this->SetStatusText(wxT("ERROR during FILTER"));

    } else {

      this->SetFilteredImage(dstImg);
      this->SetImageIsFiltered(true);
      this->Sigma = sigma_tmp;
      this->Ratio = ratio_tmp;
      wxString msg;
      msg.Printf(wxT("Image Successfully Filtered With Sigma = %f and Ratio = %f"),
                 sigma_tmp,
                 ratio_tmp);
      this->SetStatusText(msg);

    }

  }

  // enable the buttons again
  this->EnableSaveButton();
  this->EnableFilterButton();
  this->EnablePreviewButton();
  this->EnableResetButton();
  this->EnableCancelButton();
}


template <class InputST, class OutputST>
void FilterDialogGauss<InputST, OutputST>
::OnResetButton(wxCommandEvent& event)
{
  // call the Base's class OnResetButton() method and pass the event
  this->FilterDialogBase<InputST, OutputST>::OnResetButton(event);
  this->SetImageIsFiltered(false);
  this->SetStatusText(wxT("Filtered Image Reseted"));
}


/* =================== supported storagetype combinations =================== */
namespace BIAS{
template class FilterDialogGauss<unsigned char, unsigned char>;
template class FilterDialogGauss<unsigned char, float>;
//template class FilterDialogGauss<float, unsigned char>;
template class FilterDialogGauss<float, float>;
}
