/*
 This file is part of the BIAS library (Basic ImageAlgorithmS).

 Copyright (C) 2003-2009    (see file CONTACT for details)
 Multimediale Systeme der Informationsverarbeitung
 Institut fuer Informatik
 Christian-Albrechts-Universitaet Kiel


 BIAS is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 (at your option) any later version.

 BIAS is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with BIAS; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __FILTER_DIALOG_GAUSS_HH_
#define __FILTER_DIALOG_GAUSS_HH_

#include <Gui/FilterDialogBase.hh>
#include <Gui/PanelForDoubleValueWx.hh>
#include <Filter/Gauss.hh>

namespace BIAS {

/**
 * @class FilterDialogGauss
 * @ingroup g_gui
 * @brief Implementation of a Gauss filter dialog
 * @author jfernand 08/09
 */
template <class InputST, class OutputST>
class BIASGui_EXPORT FilterDialogGauss : public FilterDialogBase<InputST, OutputST>
{
public:

  FilterDialogGauss(const Image<InputST>& origImg,
                    wxWindow* parent,
                    wxWindowID id = BIAS::ID_WIN_GAUSS,
                    const wxString& title = wxT("Gauss Filter Dialog"),
                    const wxString& name = wxT("FilterDialogGauss"));
  virtual ~FilterDialogGauss();


protected:

  /**
   * concrete Implementation of the pure OnFilterButton() from the base class.
   */
  void OnFilterButton(wxCommandEvent& WXUNUSED(event));

  void OnResetButton(wxCommandEvent& event);


private:

  /**
   * The Core: our Gauss filter
   */
  Gauss<InputST, OutputST> GaussFilter;

  /**
   * Panel for Sigma parameter input.
   */
  PanelForDoubleValueWx* SigmaPanel;

  /**
   * Panel for Ratio parameter input.
   */
  PanelForDoubleValueWx* RatioPanel;

  /**
   * This sizer holds SigmaPanel and RatioPanel and is added to the base class'
   * ParamSizer.
   * Orientation: wxVERTICAL
   */
  wxBoxSizer* SigmaAndRatioSizer;

  // paramvalues needed to avoid unnecessary filtering
  double Sigma;
  double Ratio;

};

} // end namespace BIAS


#endif // __FILTER_DIALOG_GAUSS_HH_
