/*
 This file is part of the BIAS library (Basic ImageAlgorithmS).

 Copyright (C) 2003-2009    (see file CONTACT for details)
 Multimediale Systeme der Informationsverarbeitung
 Institut fuer Informatik
 Christian-Albrechts-Universitaet Kiel


 BIAS is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 (at your option) any later version.

 BIAS is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with BIAS; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <Gui/FilterDialogGradientSobel3x3.hh>
#include <Base/Image/ImageConvert.hh>

using namespace BIAS;

template <class InputST, class OutputST>
FilterDialogGradientSobel3x3<InputST, OutputST>
::FilterDialogGradientSobel3x3(const Image<InputST>& origImage,
                               wxWindow* parent,
                               wxWindowID id,
                               const wxString& title,
                               const wxString& name)
: FilterDialogBase<InputST, OutputST>(origImage, parent, id, title, name),
  SobelFilter(GradientSobel3x3<InputST, OutputST>())
{
  this->Connect(BIAS::ID_BTN_FILTER_FDB,
                wxEVT_COMMAND_BUTTON_CLICKED,
                wxCommandEventHandler(FilterDialogGradientSobel3x3::OnFilterButton));

  this->GetParent()->Disable();

  // we don't need that line and the paramsizer
  this->RemoveLine();
  this->GetMainSizer()->Remove(this->GetParamSizer());
  this->CreateStatusBar(1);
  this->GetSizer()->Fit(this);
  this->Centre();
}


template <class InputST, class OutputST>
FilterDialogGradientSobel3x3<InputST, OutputST>
::~FilterDialogGradientSobel3x3()
{
  if (!this->GetParent()->IsEnabled()) {
    this->GetParent()->Enable();
  }
}


/******************************************************************************/
/* ============================== protected ================================= */
/******************************************************************************/


template <class InputST, class OutputST>
void FilterDialogGradientSobel3x3<InputST, OutputST>
::OnFilterButton(wxCommandEvent& WXUNUSED(event))
{
  // disable some buttons to avoid interference, e.g. pressing preview during
  // filtering...
  this->EnableSaveButton(false);
  this->EnableFilterButton(false);
  this->EnablePreviewButton(false);
  this->EnableResetButton(false);
  this->EnableCancelButton(false);

  if (!this->GetImageIsFiltered()) {

    Image<InputST> src = this->GetOriginalImage();
    Image<InputST> srcImg;
    bool planar = src.IsPlanar();

    if (0 != ImageConvert::Convert(src, srcImg, ImageBase::CM_Grey, planar)) {
      BIASERR("FilterDialogGradientSobel3x3::OnFilterButton: Error Converting To Grey");
      this->SetStatusText(wxT("ERROR during CONVERT"));
    } else {
      Image<OutputST> dstImgX;
      Image<OutputST> dstImgY;
      Image<OutputST> dstImg;
      if (0 != this->SobelFilter.Filter(srcImg, dstImgX, dstImgY, dstImg)) {
        BIASERR("FilterDialogGradientSobel3x3::OnFilterButton: Filter Error");
        this->SetImageIsFiltered(false);
        this->SetStatusText(wxT("ERROR during FILTER"));
      } else {
        // we don't care about dstImgX and dstImgY, we just need dstImg
        this->SetFilteredImage(dstImg);
        this->SetImageIsFiltered(true);
        this->SetStatusText(wxT("Image SuccessFully Filtered"));
      }
    }

  } else {
    this->SetStatusText(wxT("Filtering Twice Has No Effects"));
  }

  // enable the buttons again
  this->EnableSaveButton();
  this->EnableFilterButton();
  this->EnablePreviewButton();
  this->EnableResetButton();
  this->EnableCancelButton();
}


template <class InputST, class OutputST>
void FilterDialogGradientSobel3x3<InputST, OutputST>
::OnResetButton(wxCommandEvent& event)
{
  this->FilterDialogBase<InputST, OutputST>::OnResetButton(event);
  this->SetImageIsFiltered(false);
  this->SetStatusText(wxT("Filtered Image Reseted"));
}


/* =================== supported storagetype combinations =================== */
namespace BIAS{
//template class FilterDialogGradientSobel3x3<unsigned char, unsigned char>;
template class FilterDialogGradientSobel3x3<unsigned char, float>;
//template class FilterDialogGradientSobel3x3<float, unsigned char>;
template class FilterDialogGradientSobel3x3<float, float>;
}


