/*
 This file is part of the BIAS library (Basic ImageAlgorithmS).

 Copyright (C) 2003-2009    (see file CONTACT for details)
 Multimediale Systeme der Informationsverarbeitung
 Institut fuer Informatik
 Christian-Albrechts-Universitaet Kiel


 BIAS is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 (at your option) any later version.

 BIAS is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with BIAS; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __FILTER_DIALOG_GRADIENT_SOBEL_3X3_HH_
#define __FILTER_DIALOG_GRADIENT_SOBEL_3X3_HH_

#include <Gui/FilterDialogBase.hh>
#include <Filter/GradientSobel3x3.hh>

namespace BIAS {

/**
 * @class FilterDialogGradientSobel3x3
 * @ingroup g_gui
 * @brief Implementation of a GradientSobel3x3 filter dialog
 * @author jfernand 08/09
 */
template <class InputST, class OutputST>
class BIASGui_EXPORT FilterDialogGradientSobel3x3
: public FilterDialogBase<InputST, OutputST>
{
public:

  FilterDialogGradientSobel3x3(const Image<InputST>& origImage,
                               wxWindow* parent,
                               wxWindowID id = BIAS::ID_WIN_GRADIENT_SOBEL_3X3,
                               const wxString& title = wxT("Gradient Sobel 3x3 Filter Dialog"),
                               const wxString& name = wxT("FilterDialogGradientSobel3x3"));
  virtual ~FilterDialogGradientSobel3x3();


protected:

  /**
   * concrete Implementation of the pure OnFilterButton() from the base class.
   */
  void OnFilterButton(wxCommandEvent& WXUNUSED(event));

  void OnResetButton(wxCommandEvent& event);


private:

  /**
   * The Core: our Sobel3x3 filter
   */
  GradientSobel3x3<InputST, OutputST> SobelFilter;

};

} // end namespace BIAS

#endif //  __FILTER_DIALOG_GRADIENT_SOBEL_3X3_HH_
