/*
 This file is part of the BIAS library (Basic ImageAlgorithmS).

 Copyright (C) 2003-2009    (see file CONTACT for details)
 Multimediale Systeme der Informationsverarbeitung
 Institut fuer Informatik
 Christian-Albrechts-Universitaet Kiel


 BIAS is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 (at your option) any later version.

 BIAS is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with BIAS; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __FILTER_DIALOG_MEDIAN_HH_
#define __FILTER_DIALOG_MEDIAN_HH_

#include <Gui/FilterDialogBase.hh>
#include <Filter/Median.hh>
#include <wx/spinctrl.h>
#include <wx/slider.h>
#include <wx/stattext.h>
#include <wx/choice.h>

namespace BIAS {

/**
 * @class FilterDialogMedian
 * @ingroup g_gui
 * @brief Implementation of a Median filter dialog
 * @author jfernand 08/09
 */
template <class InputST, class OutputST>
class BIASGui_EXPORT FilterDialogMedian : public FilterDialogBase<InputST, OutputST>
{
public:

  FilterDialogMedian(const Image<InputST>& origImage,
                     wxWindow* parent,
                     wxWindowID id = BIAS::ID_WIN_MEDIAN,
                     const wxString& title = wxT("Median Filter Dialog"),
                     const wxString& name = wxT("FilterDialogMedian"));
  virtual ~FilterDialogMedian();

protected:

  /**
   * concrete Implementation of the pure OnFilterButton() from the base class.
   */
  void OnFilterButton(wxCommandEvent& WXUNUSED(event));

  void OnResetButton(wxCommandEvent& event);


private:

  /**
   * The Core: our Median filter
   */
  Median<InputST, OutputST> MedianFilter;

  // paramvalues needed to avoid some unnecessary filtering
  int NewSize;
  int SecondSize;
  bool Color;
  bool OldColor;

  /**
   * This sizer holds NewSizeSizer, SecondSizeSizer and ColorChoice.
   * This will be added to the base class' ParamSizer
   * Orientation: wxVERTICAL
   */
  wxBoxSizer* MedParamSizer;

  /**
   * This sizer holds the elements for NewSize param input
   */
  wxBoxSizer* NewSizeSizer;

  /**
   * This sizer holds the elements for SecondSize param input
   */
  wxBoxSizer* SecondSizeSizer;

  wxSpinCtrl* NewSizeSpinCtrl;
  wxSpinCtrl* SecondSizeSpinCtrl;

  wxSlider* NewSizeSlider;
  wxSlider* SecondSizeSlider;

  wxStaticText* NewSizeLabel;
  wxStaticText* SecondSizeLabel;

  // user can choose whether she/he wants colored or grey image output
  wxChoice* ColorChoice;

  void OnSpinCtrlNewSize(wxSpinEvent& WXUNUSED(event));
  void OnSpinCtrlSecondSize(wxSpinEvent& WXUNUSED(event));

  void OnSliderNewSize(wxScrollEvent& WXUNUSED(event));
  void OnSliderSecondSize(wxScrollEvent& WXUNUSED(event));

  void OnChoice(wxCommandEvent& WXUNUSED(event));

  enum FilterDialogMedian_IDs {
    ID_SPINCTRL_MEDIAN_NS = 248921,
    ID_SPINCTRL_MEDIAN_SS,
    ID_SLIDER_MEDIAN_NS,
    ID_SLIDER_MEDIAN_SS,
    ID_CHOICE_COLOR
  };

};

} // end namespace BIAS

#endif // __FILTER_DIALOG_MEDIAN_HH_
