/*
 This file is part of the BIAS library (Basic ImageAlgorithmS).

 Copyright (C) 2003-2009    (see file CONTACT for details)
 Multimediale Systeme der Informationsverarbeitung
 Institut fuer Informatik
 Christian-Albrechts-Universitaet Kiel


 BIAS is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 (at your option) any later version.

 BIAS is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with BIAS; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __FILTER_DIALOG_RESCALE_HH_
#define __FILTER_DIALOG_RESCALE_HH_

#include <wx/wx.h>
#include <wx/radiobox.h>
#include <Filter/Rescale.hh>
#include <Gui/FilterDialogBase.hh>
#include <Gui/PanelForDoubleValueWx.hh>



namespace BIAS {

/**
 * @class FilterDialogRescale
 * @ingroup g_gui
 * @brief Implementation of a Rescale filter dialog
 *
 * Note: GaussThreshold is currently disabled because it is not implemented yet.
 *
 * @author jfernand 08/09
 */
template <class InputST, class OutputST>
class BIASGui_EXPORT FilterDialogRescale : public FilterDialogBase<InputST, OutputST>
{
public:

  FilterDialogRescale(const Image<InputST>& origImg,
                      wxWindow* parent,
                      wxWindowID id = BIAS::ID_WIN_RESCALE,
                      const wxString& title = wxT("Rescale Dialog"),
                      const wxString& name = wxT("FilterDialogRescale"));
  virtual ~FilterDialogRescale();


protected:

  /**
   * concrete Implementation of the pure OnFilterButton() from the base class.
   */
  void OnFilterButton(wxCommandEvent& WXUNUSED(event));

  void OnResetButton(wxCommandEvent& event);
  void OnRadioBox(wxCommandEvent& WXUNUSED(event));


private:

  /**
   * The Core: our Rescale filter
   */
  Rescale<InputST, OutputST> RescaleFilter;

  /**
   * Panel for Factor param input
   */
  PanelForDoubleValueWx* ParamPanel;

  /**
   * This sizer hold ParamPanel ParamPanel and LowPassFilter and is added
   * to the base class' ParamSizer
   */
  wxBoxSizer* RescaleParamSizer;

  /**
   * An option box for filter selection
   */
  wxRadioBox* LowPassFilter;

  const wxString MeanString;
  const wxString GaussString;
  const wxString BinomString;
  const wxString GaussTDString;

  unsigned int Selection;

  enum FilterDialogRescale_IDs {
    ID_RB_RADIOBOX = 6842565
  };

};

} // end namespace BIAS

#endif //  __FILTER_DIALOG_RESCALE_HH_
