//include first, avoid macro collision
#include <Gui/StringConv.hh>

#include "GuiWX.hh"

using namespace BIAS;
using namespace std;

//////////////////////////////////////////////////////////////////////////////
GuiWX::GuiWX()
{
	pthread_mutex_init(&guiMutex_,NULL);
	if(pthread_create(&guiThread_, NULL,
		 (void* (*)(void*))(GuiWX::guiThreadFunc_),this)) {
		BIASERR("Could not create gui thread");
	}
#ifndef WIN32
	sleep(1);
#else
  Sleep(1000);
#endif
}

GuiWX::~GuiWX(){
	pthread_cancel(guiThread_);
	wxTheApp->OnExit();
	wxEntryCleanup();
}

void* GuiWX::
guiThreadFunc_(GuiWX *caller){
	int oldstate=0;
	pthread_setcancelstate(PTHREAD_CANCEL_ENABLE,&oldstate);

	caller->app_ = new GuiWXApp();
	wxApp::SetInstance(caller->app_);
	int i=0; wxChar **c=NULL;
	wxEntryStart(i,c);
	caller->app_->Init();
	wxTheApp->OnInit();
	wxTheApp->OnRun();
	return NULL;
}

void GuiWX::
SetTitle(std::string const& Title){
	pthread_mutex_lock(&guiMutex_);
	app_->SetTitle(Title);
	pthread_mutex_unlock(&guiMutex_);
}

int GuiWX::
ShowConvertedImage_(BIAS::Image<unsigned char> &image){
	//cout<<"GuiWX::Show Converted image:"<<endl;
	pthread_mutex_lock(&guiMutex_);
	app_->ShowConvertedImage(image);
	pthread_mutex_unlock(&guiMutex_);
	return 0;
}

//////////////////////////////////////////////////////////////////////////////
GuiWXApp::GuiWXApp(){
	frame_ = NULL;
}

void GuiWXApp::Init(){
	//cout<<"OnInit:"<<endl;
	frame_ = new GuiWXFrame(wxT("Image"));
	SetTopWindow(frame_);
	frame_->Show();
}

void GuiWXApp::
SetTitle(std::string const& Title){
	if(frame_!=NULL) frame_->SetTitle(Title);
}

int GuiWXApp::
ShowConvertedImage(Image<unsigned char> &image)
{
	//cout<<"GuiWXApp::Show Converted image:"<<endl;
	if(frame_!=NULL)
		return frame_->ShowConvertedImage(image);
	else
		return -1;
}

/////////////////////////////////////////////////////////////////////////////////
GuiWXFrame::GuiWXFrame(const wxString title,
						 const wxPoint pos, const wxSize size)
: wxFrame((wxFrame *)NULL, -1, title, pos, size)
{

	panel_ = new wxPanel(this);

	panel_->Connect(wxEVT_KEY_DOWN,wxKeyEventHandler(GuiWXFrame::OnKeyDown),NULL,this);
	panel_->Connect(wxEVT_CHAR,wxKeyEventHandler(GuiWXFrame::OnKeyDown),NULL,this);

	wxSizer *pSizer = new wxBoxSizer( wxHORIZONTAL );

	ic_ = new PlainImageCanvas(panel_,NULL,0);
	ic_->SetSize(1024,768);
	pSizer->Add(panel_,1, wxEXPAND |wxALL , 5);
	SetSizer( pSizer );

	Timer_.SetOwner((wxFrame*)this, ID_TIMER);
	Timer_.Start(20,true);
  SetAutoLayout(true);
  Layout();
  //cout<<"Created new GuiWXFrame"<<endl;;
}

void GuiWXFrame::
SetTitle(std::string const& Title){
	wxString t = AsciiToWx(Title);
	wxTopLevelWindow::SetTitle(t);
}

int GuiWXFrame::
ShowConvertedImage(Image<unsigned char> &image)
{
	//cout<<"GuiWXFrame::Show Converted image:"<<endl;
	image_ = image;
  ic_->Show(image_);
  //WindowCreated_ = true;
  Refresh();
  return 0;
}

void GuiWXFrame::
OnKeyDown(wxKeyEvent &event){
	int KeyBuffer_ = event.GetKeyCode();
	cout<<"Key:"<<KeyBuffer_<<endl;
	event.Skip();
}

void GuiWXFrame::
OnQuit(wxCommandEvent& WXUNUSED(event))
{
  Timer_.Stop();
#ifndef WIN32
	sleep(1);
#else
  Sleep(1000);
#endif
  Destroy();
}

void GuiWXFrame::
OnTimer(wxTimerEvent& event)
{
	Refresh();
	Timer_.Start(20, true);
}


/////////////////////////////////////////////////////////////////////////////////
BEGIN_EVENT_TABLE(GuiWXFrame, wxFrame)
  EVT_TIMER(ID_TIMER, GuiWXFrame::OnTimer)
END_EVENT_TABLE()
