/*
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003-2009    (see file CONTACT for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include <Base/Common/W32Compat.hh>
#include <Base/Image/ImageIO.hh>
#include <Base/Image/ImageConvert.hh>
#include <Base/Image/Image.hh>
#include <Gui/GuiBase.hh>
#include <Gui/PlainImageCanvas.hh>
#include <pthread.h>

namespace BIAS{

	//forward declaration
	/**
	 * \class GuiWX
	 * \ingroup g_gui
	 * \brief simple wxWindow to display an image. WxWidgets is not optimal
	 * to do this, so a lot of workaround is needed. Still does not react to
	 * key presses, better use GuiCV
	 * \author ischiller
	 * \date 06/2010
	 */
	class BIASGui_EXPORT GuiWXFrame;
	class BIASGui_EXPORT GuiWXApp;
	////////////////////////////////////////////////////////////////////////////
	class BIASGui_EXPORT GuiWX : public BIAS::GuiBase
	{
	public:
		GuiWX();
		~GuiWX();
		virtual void SetTitle(std::string const& Title);

	protected:
		virtual int ShowConvertedImage_(BIAS::Image<unsigned char> &image);
		GuiWXApp *app_;

		#ifdef BIAS_HAVE_PTHREADS
		static void* guiThreadFunc_(GuiWX *caller);
		pthread_t       guiThread_;
		pthread_mutex_t guiMutex_;
		#endif
	};
	//////////////////////////////////////////////////////////////////////////////
	class BIASGui_EXPORT GuiWXApp: public wxApp
	{
	public:
		GuiWXApp();
		void Init();
		void SetTitle(std::string const& Title);
		int ShowConvertedImage(BIAS::Image<unsigned char> &image);

	protected:
		BIAS::GuiWXFrame *frame_;
	};
	//////////////////////////////////////////////////////////////////////////////
	enum
	{
		ID_Quit = 1,
		ID_BUTTON_QUIT,
		ID_TIMER = 1001
	};
	//////////////////////////////////////////////////////////////////////////////
	class BIASGui_EXPORT GuiWXFrame: public wxFrame
	{

	public:
		GuiWXFrame(const wxString title=wxT(""),
				const wxPoint pos=wxDefaultPosition,
				const wxSize size=wxDefaultSize);

		virtual void SetTitle(std::string const& Title);
		virtual	void OnQuit(wxCommandEvent& event);
		virtual void OnTimer(wxTimerEvent& event);
		virtual void OnKeyDown(wxKeyEvent &event);
		BIAS::PlainImageCanvas * GetCanvas(){return ic_;};


		virtual int ShowConvertedImage(BIAS::Image<unsigned char> &image);

		DECLARE_EVENT_TABLE()

	protected:
		wxPanel                    *panel_;
		BIAS::PlainImageCanvas     *ic_;

		wxTimer                    Timer_;
		BIAS::Image<unsigned char> image_;
	};


}
