/*
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003-2009    (see file CONTACT for details)
Multimediale Systeme der Informationsverarbeitung
Institut fuer Informatik
Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <Base/Common/W32Compat.hh>
#include "HistoFrame.hh"
#include <Gui/HistoImageCanvas.hh>

using namespace BIAS;
using namespace std;

// icon
#if !defined(__WXPM__)
#  include <img/biasviewwx.xpm>
#endif



//Constructor
//additional WINDOWS-ONLY-styles were required specially for WINDOWS,
//because of several repaint-resize bugs
HistoFrame::
HistoFrame(wxWindow* parent, const wxString& title,
           wxPoint pos, wxSize size)
  : wxFrame(parent, -1, title, pos, size,
            wxDEFAULT_FRAME_STYLE | wxCLIP_CHILDREN | wxFULL_REPAINT_ON_RESIZE)
{
  SetIcon(wxIcon(biasviewwx_xpm));
  Setup(size);
  Show();

#ifdef WIN32
  Window_->Refresh();
#endif
}


void HistoFrame::
Setup(wxSize size)
{
  int widths[] = {100, -1};

  CreateStatusBar(2);

  GetStatusBar()->SetStatusWidths(2, widths);
  GetStatusBar()->SetStatusText(wxT("Histogramm"), 0);
  GetStatusBar()->Show();

  SetMinSize(wxSize(300, 250));

  //GetStatusBar()->SetStatusStyles(2, wxSB_NORMAL);
  //GetStatusBar()->SetMinHeight(30);

  //QuitButton_ = new wxButton(this, MIA_HistoQuitButton, wxT("Close"));
  //Correct the size of the canvas (it's 6 pxl thinner and 31 pxl smaller)
  wxSize size_canvas = wxSize(size.GetWidth()-6,
                              size.GetHeight()-31);
  Window_ = new HistoImageCanvas(this, GetStatusBar(), 1, -1,
                                 wxDefaultPosition, size_canvas);

  wxBoxSizer *box = new wxBoxSizer(wxVERTICAL);

  //box->Add(QuitButton_, 0, wxALL | wxGROW, 3);
  box->Add(Window_, 1, wxALL | wxGROW, 3);
  //box->SetSizeHints(this);

  SetSizer(box);
}


//Destructor
HistoFrame::
~HistoFrame()
{}


BEGIN_EVENT_TABLE (HistoFrame, wxWindow)
  EVT_CLOSE       (HistoFrame::OnClose)
  EVT_SIZE        (HistoFrame::OnSize)
  //EVT_BUTTON      (MIA_HistoQuitButton, HistoFrame::OnQuit)
  //EVT_MOTION      (HistoImageCanvas::OnMouseMove)
  //EVT_CHAR        (HistoFrame::OnKey)
  //EVT_SIZE        (HistoImageCanvas::OnResize)
  //EVT_LEAVE_WINDOW(HistoImageCanvas::OnMouseLeave)
END_EVENT_TABLE()


void HistoFrame::
NewImage(BIAS::ImageBase& im, const wxString& name)
{
  if(!(Window_->SameName((wxString)name, true))){
    Window_->NewImage(im);
  }
}


void HistoFrame::
OnQuit(wxCommandEvent & event)
{
  Close(true);
}


void HistoFrame::
OnClose(wxCloseEvent & event)
{
  Hide();
}


void HistoFrame::
OnSize(wxSizeEvent& event)
{
  //-28 in Height is for the StatusBar to be visible
  Window_->SetSize(this->GetSize().GetWidth(), this->GetSize().GetHeight()-28);
}


