#ifndef __HistoImageCanvas_hh__
#define __HistoImageCanvas_hh__

#include <bias_config.h>

#ifndef BIAS_HAVE_WXWIDGETS
#  error BIAS_HAVE_WXWIDGETS not defined but required by HistoImageCanvas. Please enable BIAS_HAVE_WXWIDGETS in CMake configure step.
#endif

#include <vector>
#include <wx/wx.h>
#include <Base/Image/Image.hh>
#include <Base/Common/BIASpragmaStart.hh>
#include <Base/Common/BIASpragmaEnd.hh>


namespace BIAS{

  /** @class Histogramm_float_Entry
      @brief base class for storing computed histogramm information
      in HistoCanvas 
      @author schiller */
  class Histo_Entry_Float {
  public:
    //Interval ends
    float value_begin;
    float value_end;
    //Number of values in this interval
    int   value_count;
  };


  /** @class HistoImageCanvas
      @brief It's a Canvas used to compute Histogram for current image
      @status working
      @author amattal
      @date 03/2007 */
  class BIASGui_EXPORT HistoImageCanvas : public wxPanel
  {
  public:
    HistoImageCanvas(wxWindow* parent, wxStatusBar *sbar=NULL, int sbarid=0,
                     wxWindowID id = -1,
                     const wxPoint& pos = wxDefaultPosition,
                     const wxSize& size = wxSize(272, 216),
                     long style = wxTAB_TRAVERSAL,
                     const wxString& name = wxT("HistoImageCanvas"));
    ~HistoImageCanvas();

    void UpdateHistogramm(ImageBase image);
    void NewImage(BIAS::ImageBase& im);
    bool SameName(wxString name, bool settothenewone);
    void RefitHistogramm();

  protected:

    void OnMouseLeave(wxMouseEvent&);
    void OnMouseMove(wxMouseEvent &event);

    void UpdateStatusBar(unsigned mouseX, unsigned mouseY);

    void OnResize(wxSizeEvent & event);
    void OnPaint(wxPaintEvent& event);

    wxBitmap                   *Bitmap_;
    wxImage                    *wximage_;
    BIAS::Image<unsigned char>  ucim_, zim_;
    BIAS::ImageBase             origim_;
    wxStatusBar                *SBar_;

    //indicates how wide should be 1 step of histogramm, when painted
    //(in pixels), is set automatically when the histoframe is resized
    wxCoord                     histoZoom_;

    int SBarID_;

    unsigned int mousex_, mousey_;

    int wwidth_, wheight_; // width & height of Window_

    wxMutex            wxbitmap_mutex_, origim_mutex_;

    //----HISTOGRAMMS----
    //Histogramm type:
    // -1 - no histogramm computed
    //
    //  0 - unsigned char
    //  1 - floating point (float)
    int                         histo_type;

    //Histogramm for float-images for each channel
    //std::vector<std::vector<Histo_Entry_Float> > histo_float;
    std::vector<std::vector<Histo_Entry_Float> > histo_float;
    int                              histo_float_zero;
    int                              histo_float_steps;

    float                            histo_float_min;
    float                            histo_float_step;

    //Histogramm for unsigned char-images for each channel
    std::vector<std::vector<int> >   histo_UC;

    int                              histo_channels;

    std::vector<std::vector<float> > histo_percentage;
    std::vector<float>               histo_percentage_biggest;

    //count of pixels in an image with histogramm
    int                         histo_size;
    //-------------------
    wxString                    histo_name;

    DECLARE_EVENT_TABLE()
  }; // class HistoImageCanvas

} // namespace

#endif // __HistoImageCanvas_hh__
