#ifndef __ImageCanvas_hh__
#define __ImageCanvas_hh__



// this should not be necessary because it is already checked in the CMakeLists.txt
//#ifndef BIAS_HAVE_WXWIDGETS
//#  error BIAS_HAVE_WXWIDGETS not defined but required by ImageCanvas. Please enable BIAS_HAVE_WXWIDGETS in CMake configure step.
//#endif
#include <wx/wx.h>
#include <wx/dcbuffer.h>  
#include <wx/thread.h> 

#include <bias_config.h>
#include <Base/Common/BIASpragmaStart.hh>
#include <Base/Common/BIASpragmaEnd.hh>
#include <Base/Debug/Debug.hh>
#include <Base/Image/Image.hh>



#define D_IC_IM (1<<0)


namespace BIAS{

  class ZoomFrame;
  class HistoFrame;
  class ImageLegendFrame;
  template <class T> class PyramidImage;


  /** @class ImageCanvas
      @ingroup g_gui
      @brief display image in wx application, provides zoom and investigation
      functionality
      @author woelk */
  class BIASGui_EXPORT ImageCanvas 
    : public wxScrolledWindow, public BIAS::Debug
  {
  public:
    ImageCanvas(wxWindow* parent, wxStatusBar* sbar=NULL,
                int sbarid=0, wxWindowID id = -1,
                const wxPoint& pos = wxDefaultPosition,
                const wxSize& size = wxSize(320, 240),
                long style = ( (wxHSCROLL | wxVSCROLL) & wxTRANSPARENT),
                const wxString& name = wxT("ScrolledWindow"));
    ~ImageCanvas();


    virtual void Show(BIAS::Image<unsigned char>& im, std::string name="");
    virtual void Show(BIAS::ImageBase& origim, BIAS::Image<unsigned char>& scaledrgbim,
                      std::string name="");
    virtual void Show(BIAS::PyramidImage<unsigned char>& pim, std::string name="");

    //returns the position of the histogramm window
    wxPoint GetHistoPos();
    //returns the size of the histogramm window
    wxSize GetHistoSize();
    //returns the position of the histogramm window
    wxPoint GetValueBarPos();
    //returns the size of the histogramm window
    wxSize GetValueBarSize();

    //The Method that shows Histogramm-Window if it's not visible yet
    void ShowHistogramm(wxPoint pos  = wxPoint(10,10),
                        wxSize  size = wxSize(100,100));
    void ShowHistogramm(ImageBase& image,
                        wxPoint pos  = wxPoint(10,10),
                        wxSize  size = wxSize(100,100));

    void ShowImageValueBar(wxPoint pos = wxPoint(10,10),
    		                   wxSize size = wxSize(100,100));
    void ShowImageValueBar(ImageBase& image,
    		                   wxPoint pos = wxPoint(10,10),
                           wxSize size = wxSize(100,100));
    void HideImageValueBar();

    // to let user access classic hide/show function
    virtual bool Show(bool show=true) {return wxScrolledWindow::Show(show);}

    inline BIAS::ImageBase *GetOrigImage() { return &origim_; }
    //return the 3 channel image which is displays on screen
    inline BIAS::Image<unsigned char>* GetImageShown(){ return &rgbim_;}

    bool GetMousePos(int &x, int &y);

    inline void SetDefaultZoom(unsigned z)
    { DefaultZoom_=z; }

    /**
     * Canvas now uses double buffering to avoid flickering, so it should be ok
     * to always clear the canvas.
     */
    inline void SetClearDrawingContext(bool clear)
    { ClearDrawingContext_=clear; }

    inline void SetShowScaled(bool on){ShowScaled_ = on;}
    inline bool GetShowScaled(){return ShowScaled_;}

    /** Sets the size of the scaled image. Only used if show scaled is on. */
    inline void SetScaledImageSize(int width, int height)
    {
      ScaledImageSizeX_ = width;
      ScaledImageSizeY_ = height;
    }

    /**
      Set the quality used for image size rescaling
      false: Uses the normal default scaling method of pixel replication
      true: Uses bicubic and box averaging resampling methods for upsampling 
      and downsampling respectively
    */
    inline void SetHighScaleQuality(bool on)
    {
      UseHighScaleQuality_ = on;
    }

    bool HistogrammIsShown();
    bool ImageValueBarIsShown();
    void KeepScrollPosition(bool flag);

    /** \brief Set the color which is used to draw the roi.  */
    void SetRoiColor(unsigned char red,unsigned char green,
                     unsigned char blue, unsigned alpha = 128);    
    inline void DrawRoi(bool enable){ DrawRoi_ = enable; Refresh();}
    inline bool IsRoiDrawn(){return DrawRoi_; }
    // raises or lowers the zoom window if it is shown
    void RaiseZoomWindow(const bool raise_or_lower); 

  protected:
    void InitHisto(wxPoint pos  = wxPoint(10,10),
                   wxSize  size = wxSize(100,100));
    void InitImageValueBar(wxPoint pos  = wxPoint(10,10),
									  wxSize  size = wxSize(100,100));

    void UpdateStatusBar(unsigned x, unsigned y);
    void Zoom(unsigned x, unsigned y);

    virtual void OnPaint(wxPaintEvent& event);
    virtual void OnErase(wxEraseEvent& event);
    virtual void OnSize(wxSizeEvent& event);
    virtual void OnMouseMove(wxMouseEvent &event);
    virtual void OnMouseLeave(wxMouseEvent &event);
    virtual void OnMouseEnter(wxMouseEvent &event);
    virtual void OnMouseWheel(wxMouseEvent &event);
    virtual void OnRightMouseButton(wxMouseEvent &event);
    virtual void OnLeftMouseButton(wxMouseEvent &event);
    virtual void OnKeyPress(wxKeyEvent &event);
    virtual void PaintRoiOnDc_(wxBufferedPaintDC& dc);

    ZoomFrame *MyZoomWin_;
    HistoFrame *MyHistoWin_;
    ImageLegendFrame *MyValueBarFrame_;

    BIAS::ImageBase origim_;
    BIAS::Image<unsigned char> rgbim_;
    BIAS::Image<unsigned char> roimask_;

    wxBitmap *RoiBitmap_;
    wxBitmap *Bitmap_;
    wxImage  *wxRoiImage_;
    wxImage *wximage_;
    wxMutex wxbitmap_mutex_, origim_mutex_;
    wxStatusBar *SBar_;

    // for scaled-down display
    bool ShowScaled_;
    bool DrawRoi_;
    bool KeepScrollPosition_;
    // Canvas now uses double buffering to avoid flickering,
    // so it should be ok to always clear the canvas.
    bool ClearDrawingContext_;

    int ScaledImageSizeX_,ScaledImageSizeY_;
    bool UseHighScaleQuality_;
    int SBarID_;
    int SBarWidths_[2];
    int mousex_, mousey_;

    unsigned DefaultZoom_;
    unsigned width_, height_;
    unsigned lastwidth_, lastheight_;

    std::string ImageName_;
    wxString Title_;

    DECLARE_EVENT_TABLE()
  }; // ImageCanvas
} // namespace

#endif // __ImageCanvas_hh__
