#include "ImageCanvasRectangularSelection.hh"

#include <Base/Image/ImageConvert.hh>
#include <Base/ImageUtils/ImageDraw.hh>

using namespace BIAS;
using namespace std;

ICECallbackInterface::ICECallbackInterface()
{
}

ICECallbackInterface::~ICECallbackInterface()
{
}

//void ICECallbackInterface::RectangleSelected(int tl[2], int br[2])
void ICECallbackInterface::RectangleSelected(int[2], int[2])
{
  BIASERR("error: function of derived class should be called");
}

/////////////////////////////////////////////////////////////////

ImageCanvasRectangularSelection::
ImageCanvasRectangularSelection(wxWindow* parent,
                                wxStatusBar* sbar,
                                int sbarid, wxWindowID id,
                                const wxPoint& pos,
                                const wxSize& size,
                                long style,
                                const wxString& name)
  : ImageCanvas(parent, sbar, sbarid, id, pos, size, style, name)
{
  NumLeftClicks_=0;
  SelectRectangle_=false;
  CallbackObj_=NULL;
  BottomRight_[0]=0;
  BottomRight_[1]=0;
  TopLeft_[0]=0;
  TopLeft_[1]=0;
}

ImageCanvasRectangularSelection::
~ImageCanvasRectangularSelection()
{
}

void ImageCanvasRectangularSelection::
SelectRectangle(ICECallbackInterface& obj)
{
  origim_mutex_.Lock();
  BIASDOUT(D_IC_IM,"origim_ in SelectRectangle adr: "<<&origim_);
  //  BIASDOUT(D_IC_IM,"rgbim_  in SelectRectangle adr: "<<&rgbim_);
  origim_mutex_.Unlock();
  NumLeftClicks_=0;
  SelectRectangle_=true;
  CallbackObj_=&obj;

}

BEGIN_EVENT_TABLE (ImageCanvasRectangularSelection, ImageCanvas)
    EVT_LEFT_DOWN  (ImageCanvasRectangularSelection::OnLeftMouseButton)
    EVT_RIGHT_DOWN  (ImageCanvasRectangularSelection::OnRightMouseButton)
    EVT_MOTION      (ImageCanvasRectangularSelection::OnMouseMove)
    EVT_PAINT       (ImageCanvasRectangularSelection::OnPaint)
END_EVENT_TABLE()

void ImageCanvasRectangularSelection::
OnLeftMouseButton(wxMouseEvent &event)
{
  if (SelectRectangle_){
    wxClientDC dc(this);
    PrepareDC(dc);
    wxPoint pos = event.GetPosition();
    unsigned x = (unsigned) dc.DeviceToLogicalX( pos.x );
    unsigned y = (unsigned) dc.DeviceToLogicalY( pos.y );
    switch (NumLeftClicks_){
    case 0:
      TopLeft_[0]=x; TopLeft_[1]=y;
      NumLeftClicks_++;
      break;
    case 1:
      {
        BottomRight_[0]=x; BottomRight_[1]=y;
        CallbackObj_->RectangleSelected(TopLeft_, BottomRight_);
        SelectRectangle_=false;
        CallbackObj_=NULL;
        NumLeftClicks_=0;
        RefreshDrawImage_();
        wxPaintEvent pe;
        wxPostEvent(this->GetEventHandler(), pe);
      }
      break;
    default:
      BIASERR("something strange happend");
      break;
    }
  } else {
    event.Skip();
  }
}

void ImageCanvasRectangularSelection::
OnRightMouseButton(wxMouseEvent &event)
{
  if (SelectRectangle_){
    NumLeftClicks_ = 0;
    SelectRectangle_=false;
    RefreshDrawImage_();
    wxPaintEvent pe;
    wxPostEvent(this->GetEventHandler(), pe);
  }
  else
	 event.Skip();
}

void ImageCanvasRectangularSelection::
OnPaint(wxPaintEvent &event) //wxCommandEvent &event)
{
    if(NumLeftClicks_==1 && SelectRectangle_)
    {
	   event.Skip();
    }
    else
	   event.Skip();
}

void ImageCanvasRectangularSelection::OnMouseMove(wxMouseEvent &event)
{
  if (NumLeftClicks_==1 && SelectRectangle_){
    wxClientDC dc(this);
    PrepareDC(dc);
    wxPoint pos = event.GetPosition();
    int x = dc.DeviceToLogicalX( pos.x );
    int y = dc.DeviceToLogicalY( pos.y );
    BottomRight_[0]=x;
    BottomRight_[1]=y;
    DrawRectangle_(TopLeft_, BottomRight_);
  }

  event.Skip();
}

void ImageCanvasRectangularSelection::
DrawRectangle_(int tl[2], int br[2])
{
  unsigned char color[]={255, 0, 0};
  RefreshDrawImage_();
  int tmp;
  int mtl[]={tl[0], tl[1]}, mbr[]={br[0], br[1]};

  if (mtl[0]>mbr[0]) { tmp=mtl[0]; mtl[0]=mbr[0]; mbr[0]=tmp; }
  if (mtl[1]>mbr[1]) { tmp=mtl[1]; mtl[1]=mbr[1]; mbr[1]=tmp; }
  ImageDraw<unsigned char>::RectangleCorners(rgbim_, mtl[0], mtl[1], mbr[0],
                                             mbr[1], color);
  BIASDOUT(D_IC_IM,"Rectangle drawn in Image adr: "<<&rgbim_);
  wxPaintEvent pe;
  wxPostEvent(this->GetEventHandler(), pe);
}

void ImageCanvasRectangularSelection::
RefreshDrawImage_()
{
  Image<unsigned char> tmprgb;
  if (origim_.GetColorModel()!=ImageBase::CM_RGB){
    if (ImageConvert::Convert(origim_, tmprgb, ImageBase::CM_RGB)!=0){
      BIASERR("error converting");
    }
  } else {
    tmprgb=origim_;
  }
  Show(origim_, tmprgb, ImageName_);
}
