#ifndef __ImageCanvasRectangularSelection_hh__
#define __ImageCanvasRectangularSelection_hh__

#include "ImageCanvas.hh"
#include <Gui/StringConv.hh>

namespace BIAS {
  
  /** @class ICECallbackInterface
      @ingroup g_gui
      @brief callback interface for ImageCanvasExtended

      instructions:
      derive your class e.g. MyFrame from ICECallBackInterface
      and override the method RectangleSelected(...)to react to a finished 
      selection of a rectangle
      
      also declare a pointer to ImageCanvas Enhanced, e.g.
      ImageCanvasRectangularSelection *_ic;
      
      
      to select a rectangle then call
      _ic->SelectRectangle(*this);
      where *this is a reference to a ICECallBackInterface
      (if your class is derived from ICECallBackInterface this is *this)
      
      @author woelk 08/2004*/
  class BIASGui_EXPORT ICECallbackInterface{
  public:
    ICECallbackInterface();
    virtual ~ICECallbackInterface();
    /// this function should be overloaded
    virtual void RectangleSelected(int tl[2], int br[2]);
  };

  /** @class ImageCanvasRectangularSelection
      @ingroup g_gui
      @brief provides functionality for selection of rectangle, point, ...
      
      

      @author woelk 08/2004 */
  class BIASGui_EXPORT ImageCanvasRectangularSelection : public ImageCanvas
  {
  public:
    ImageCanvasRectangularSelection(wxWindow* parent, wxStatusBar* sbar=NULL,
                        int sbarid=0, wxWindowID id = -1, 
                        const wxPoint& pos = wxDefaultPosition, 
                        const wxSize& size = wxSize(320, 240), 
                        long style = wxHSCROLL | wxVSCROLL, 
                        const wxString& name = AsciiToWx("ScrolledWindow"));
    ~ImageCanvasRectangularSelection();
    
    /** Call SelectRectangle with an object derived from ICECallbackInterface.
        The function RectangleSelected() should be overloaded in this object.
        After rectangle selection is finished (2 clicks with left mouse,
        right button cancels) the function obj.RectangleSelected() is called
        with top left (tl) and bottom right (br) coordinates.
        @author woelk 08/2004 */
    void SelectRectangle(ICECallbackInterface& obj);

    virtual void OnLeftMouseButton(wxMouseEvent &event);
    virtual void OnMouseMove(wxMouseEvent &event);
    virtual void OnRightMouseButton(wxMouseEvent &event);
    virtual void OnPaint(wxPaintEvent& event);

  protected:
    int NumLeftClicks_;
    int TopLeft_[2], BottomRight_[2];
    bool SelectRectangle_;

    ICECallbackInterface *CallbackObj_;
    
    void DrawRectangle_(int tl[2], int br[2]);
    
    void RefreshDrawImage_();

    DECLARE_EVENT_TABLE()
  }; // class 

} // namespace

#endif // __ImageCanvasRectangularSelection_hh__
