/*
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003-2009    (see file CONTACT for details)
Multimediale Systeme der Informationsverarbeitung
Institut fuer Informatik
Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <Base/Common/W32Compat.hh>
#include "ImageLegendFrame.hh"
#include <Gui/ImageCanvas.hh>
#include <Base/Image/ImageConvert.hh>
#include <Base/Image/ImageIO.hh>
#include <Gui/StringConv.hh>
#include <Base/Common/FileHandling.hh>



using namespace BIAS;
using namespace std;

// icon
#if !defined(__WXPM__)
#  include <img/biasviewwx.xpm>
#endif


ImageLegendFrame::
ImageLegendFrame(wxWindow* parent, const wxString& title,
           wxPoint pos, wxSize size)
  : wxFrame(parent, -1, title, pos, size,
            wxDEFAULT_FRAME_STYLE | wxCLIP_CHILDREN | wxFULL_REPAINT_ON_RESIZE)
{
  SetIcon(wxIcon(biasviewwx_xpm));
  defaultDir_="";
  defaultFile_="";
  vbWidth_ = 120;
  vbHeight_ = 400;

  Setup(size);
  Show();

#ifdef WIN32
  window_->Refresh();
#endif

}


void ImageLegendFrame::
Setup(wxSize size)
{
	// create tool bar
	wxToolBar* toolBar = new wxToolBar(this, -1);

  wxSize ticksSize(40,25);
  ticksSpin_= new wxSpinCtrl(toolBar,
  		ID_Ticks,
  		wxT("10"),
  		wxDefaultPosition, ticksSize,
  		wxSP_ARROW_KEYS, 1, 2000);
  wxStaticText *tx= new wxStaticText(toolBar, ID_Static, AsciiToWx("# Ticks: "),wxDefaultPosition);
  toolBar->AddControl(tx);
  toolBar->AddControl(ticksSpin_);

  stringstream wi; wi<<vbWidth_;
  stringstream he; he<<vbHeight_;
  wxSize sizeSize(60,25);
  toolBar->AddSeparator();
  xSpin_= new wxSpinCtrl(toolBar,ID_X,AsciiToWx(wi.str()),wxDefaultPosition, sizeSize,
												wxSP_ARROW_KEYS, 1, 2000);
  ySpin_= new wxSpinCtrl(toolBar,ID_Y,AsciiToWx(he.str()),wxDefaultPosition, sizeSize,
  												wxSP_ARROW_KEYS, 1, 2000);

  toolBar->AddControl(new wxStaticText(toolBar, ID_Static, AsciiToWx(" Size:" ),wxDefaultPosition));
  toolBar->AddControl(xSpin_);
  toolBar->AddControl(new wxStaticText(toolBar, ID_Static, AsciiToWx(" X "),wxDefaultPosition));
  toolBar->AddControl(ySpin_);
  toolBar->AddControl(new wxStaticText(toolBar, ID_Static, AsciiToWx(" Y" ),wxDefaultPosition));
  toolBar->AddSeparator();
  wxButton *saveButton = new wxButton(toolBar, ID_Save, wxT("Save"),wxDefaultPosition,sizeSize);
  toolBar->AddControl(saveButton);

  toolBar->Realize();
  SetToolBar(toolBar);


	int widths[] = {100, -1};
  CreateStatusBar(2);
  GetStatusBar()->SetStatusWidths(2, widths);
  GetStatusBar()->SetStatusText(wxT("Legend"), 0);
  GetStatusBar()->Show();
  SetMinSize(wxSize(400, 470));
  GetStatusBar()->SetStatusStyles(2, wxSB_NORMAL);
  GetStatusBar()->SetMinHeight(30);

  wxBoxSizer *box = new wxBoxSizer(wxVERTICAL);
  //Correct the size of the canvas (it's 6 pxl thinner and 31 pxl smaller)
  wxSize size_canvas = wxSize(size.GetWidth()-6,
                              size.GetHeight()-61);
  window_ = new ImageCanvas(this, GetStatusBar(), 1, -1,
                             wxDefaultPosition, size_canvas);
  box->Add(window_, 1, wxALL | wxGROW, 3);

  SetSizer(box);
}


//Destructor
ImageLegendFrame::
~ImageLegendFrame()
{}


BEGIN_EVENT_TABLE (ImageLegendFrame, wxWindow)
  EVT_CLOSE       (ImageLegendFrame::OnClose)
  EVT_SIZE        (ImageLegendFrame::OnSize)
  EVT_BUTTON      (ImageLegendFrame::ID_Save, ImageLegendFrame::OnSave)
  EVT_SPINCTRL    (ImageLegendFrame::ID_Ticks,ImageLegendFrame::OnTicks)
  EVT_SPINCTRL    (ImageLegendFrame::ID_X,ImageLegendFrame::OnLegendSize)
  EVT_SPINCTRL    (ImageLegendFrame::ID_Y,ImageLegendFrame::OnLegendSize)
END_EVENT_TABLE()


void ImageLegendFrame::
SetLegendSize(unsigned width, unsigned height){
	vbWidth_ = width; vbHeight_=height;
	xSpin_->SetValue(vbWidth_);
	ySpin_->SetValue(vbHeight_);
}

void ImageLegendFrame::
SetImage(BIAS::ImageBase& im)
{
	if(im.GetStorageType()==ImageBase::ST_unsignedchar){
		Image<unsigned char> imUC= (Image<unsigned char>)im;
		SetImage(imUC);
	}
	else if(im.GetStorageType()==ImageBase::ST_float){
		Image<float> imFL= (Image<float>)im;
		SetImage(imFL);
	}

}
void ImageLegendFrame::
SetImage(BIAS::Image<unsigned char>& im)
{
	if(vbUC_.IsEmpty() ||im.GetChannelCount()!= vbUC_.GetChannelCount() ){
		vbUC_.Clear();
		vbUC_.Init(vbWidth_,vbHeight_, im.GetChannelCount());
	}

	unsigned char min,max;
	im.GetMinMaxPixelValue(min,max);
	vbUC_.SetValueRange(min,max);
	vbUC_.SetDisplayRange(min,max);
	vbUC_.SetUnit("");
	vbUC_.SetColorModel(ImageBase::CM_RGB);
	vbUC_.Update();
	window_->Show(vbUC_, "Legend");
}

void ImageLegendFrame::
SetImage(BIAS::Image<float>& im)
{
	float min,max;
  im.GetMinMaxPixelValue(min,max);
	Image<float> tmp(im); tmp.ScaleShiftBetween(0,255);
	Image<unsigned char> scaled;
	ImageConvert::ConvertST(tmp, scaled,ImageBase::ST_unsignedchar);

	if(vbUC_.IsEmpty() ||scaled.GetChannelCount()!= vbUC_.GetChannelCount() ){
		vbUC_.Clear();
		vbUC_.Init(vbWidth_,vbHeight_, scaled.GetChannelCount());
	}
	vbUC_.SetDisplayRange(min,max);
	unsigned char minUC,maxUC;
	scaled.GetMinMaxPixelValue(minUC,maxUC);
	vbUC_.SetValueRange(minUC,maxUC);
	vbUC_.SetUnit("mm");
	vbUC_.SetColorModel(ImageBase::CM_Grey);
	vbUC_.Update();
	window_->Show(vbUC_, "Legend");
}


void ImageLegendFrame::
OnSave(wxCommandEvent& event){

	wxString message   			 = _T("Save legend to");
	wxString wildcard        = _T("*.*");

	wxFileDialog wd(this,
			message,
			AsciiToWx(defaultDir_),
			AsciiToWx(defaultFile_),
			wildcard,
			wxFD_SAVE|wxFD_OVERWRITE_PROMPT);

	if (wxID_OK == wd.ShowModal()) {
		defaultFile_ = WxToAscii(wd.GetPath());
		defaultDir_ = WxToAscii(wd.GetDirectory());
		ImageIO::Save(defaultFile_,*window_->GetImageShown());
	}
}

void ImageLegendFrame::
OnQuit(wxCommandEvent & event)
{
  Close(true);
}


void ImageLegendFrame::
OnClose(wxCloseEvent & event)
{
  Hide();
}


void ImageLegendFrame::
OnSize(wxSizeEvent& event)
{
  //-61 in Height is for the StatusBar to be visible
  window_->SetSize(this->GetSize().GetWidth(), this->GetSize().GetHeight()-61);
}


void ImageLegendFrame::
OnTicks(wxSpinEvent& WXUNUSED(event))
{
	if(!vbUC_.IsEmpty()){
		int num = ticksSpin_->GetValue();
		vbUC_.SetTicks(num);
		vbUC_.Update();
		window_->Show(vbUC_, "Legend");
	}
}

void ImageLegendFrame::
OnLegendSize(wxSpinEvent& WXUNUSED(event))
{
	if(!vbUC_.IsEmpty()){
		int x = xSpin_->GetValue();
		int y = ySpin_->GetValue();
		vbUC_.ReInit(x,y,vbUC_.GetChannelCount());
		vbUC_.Update();
		window_->Show(vbUC_, "Legend");
	}
}
