/*
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003-2009    (see file CONTACT for details)
Multimediale Systeme der Informationsverarbeitung
Institut fuer Informatik
Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#ifndef __ImageLegendFrame_hh__
#define __ImageLegendFrame_hh__

#include <bias_config.h>

#ifndef BIAS_HAVE_WXWIDGETS
#  error BIAS_HAVE_WXWIDGETS not defined but required by HistoFrame. Please enable BIAS_HAVE_WXWIDGETS in CMake configure step.
#endif
#include <Base/Common/BIASpragmaStart.hh>
#include <wx/wx.h>
#include <Base/Common/BIASpragmaEnd.hh>
#include <Utils/ImageValueBar.hh>
#include <wx/spinctrl.h>

namespace BIAS{

	class ImageCanvas;

  /** \class ImageLegendFrame
      \brief  Frame to display a Image legend, for example in biasviewwx
      \author ischiller 11/2010 */
  class BIASGui_EXPORT ImageLegendFrame : public wxFrame
  {

  	enum{
  		ID_Save = wxID_HIGHEST+764,
  		ID_Ticks,
  		ID_X,
  		ID_Y,
  		ID_Static
  	};

  public:
    ImageLegendFrame(wxWindow* parent, const wxString& title,
               wxPoint pos = wxPoint(10,10),
               wxSize size = wxSize(300, 250));
    ~ImageLegendFrame();

    void SetImage(BIAS::Image<unsigned char>& im);
    void SetImage(BIAS::Image<float>& im);
    void SetImage(BIAS::ImageBase& im);
    void SetLegendSize(unsigned width, unsigned height);

  protected:
    std::string  defaultDir_,defaultFile_;
    wxSpinCtrl *ticksSpin_,*xSpin_,*ySpin_;
    void Setup(wxSize size = wxSize(300, 250));
    void OnSize(wxSizeEvent& event);
    void OnTicks(wxSpinEvent& event);
    void OnLegendSize(wxSpinEvent& event);

    void OnSave(wxCommandEvent& event);
    void OnQuit(wxCommandEvent & event);
    void OnClose(wxCloseEvent & event);
    BIAS::ImageCanvas      *window_;
    unsigned vbWidth_,vbHeight_;

    ImageValueBar<unsigned char> vbUC_;
    ImageValueBar<float> vbFL_;

    DECLARE_EVENT_TABLE()
  }; // class HistoFrame

} // namespace

#endif // __HistoFrame_hh__
