/*
 This file is part of the BIAS library (Basic ImageAlgorithmS).

 Copyright (C) 2003-2009    (see file CONTACT for details)
 Multimediale Systeme der Informationsverarbeitung
 Institut fuer Informatik
 Christian-Albrechts-Universitaet Kiel


 BIAS is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 (at your option) any later version.

 BIAS is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with BIAS; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __PANEL_FOR_DOUBLE_VALUE_WX_HH_
#define __PANEL_FOR_DOUBLE_VALUE_WX_HH_

#include <bias_config.h>
#include <wx/panel.h>
#include <wx/stattext.h>
#include <wx/textctrl.h>
#include <wx/spinbutt.h>
#include <wx/spinctrl.h>
#include <wx/slider.h>
#include <wx/sizer.h>

namespace BIAS {

/**
 * @class PanelForDoubleValueWx
 * @brief Helper Class For Parameter Input To Take Floating Point Numbers
 * @author jfernand 08/09
 */
class BIASGui_EXPORT PanelForDoubleValueWx : public wxPanel
{
public:

  PanelForDoubleValueWx(wxWindow* parent,
                        wxWindowID id = wxID_ANY,
                        const wxString& label = wxT("LABEL:"),
                        const double minlimit = 0.0,
                        const double maxlimit = 1.0,
                        const double initialval = 0.5,
                        const int stepsize = 10000,
                        const wxPoint& pos = wxDefaultPosition,
                        const wxSize& size = wxDefaultSize,
                        long style = wxTAB_TRAVERSAL);
  ~PanelForDoubleValueWx();

  /**
   * @return the assigned value for a parameter
   */
  double GetParamValue() const;

  /**
   * @return the maximum limit for a parameter
   */
  double GetParamMaxLimit() const;

  /**
   * @return the minimum limit for a parameter
   */
  double GetParamMinLimit() const;

  /**
   * @return the RANGE for the slider beginning at ZERO
   * (unfortunately the name is a little bit misleading)
   */
  int GetStepSize() const;

  /**
   * Sets the maximum limit for a parameter
   * @return TRUE if setting was successful otherwise FALSE
   */
  bool SetParamMaxLimit(const double maxlimit);

  /**
   * Sets the minimum limit for a parameter
   * @return TRUE if setting was successful otherwise FALSE
   */
  bool SetParamMinLimit(const double minlimit);

  /**
   * Gives the parameter a name
   */
  void SetParamLabel(const wxString& label);

  /**
   * Sets the RANGE for the slider for precision
   * (and also here: name is misleading)
   */
  void SetStepSize(const int stepsize);


private:

  void OnTextCtrl(wxCommandEvent& event);
  void OnSlider(wxCommandEvent& event);
  void OnSpinCtrl(wxSpinEvent& event);
  void UpdateSlider();

  wxBoxSizer* MainSizer;

  wxStaticText* Label;
  wxStaticText* StepSizeLabel;
  wxTextCtrl* TextCtrl;
  wxSlider* Slider;
  wxSpinCtrl* SpinCtrl;

  int StepSize;
  double TextCtrlValue;
  double ParamMaxLimit;
  double ParamMinLimit;

  enum BSCD_IDs {
      ID_TEXT_CTRL = 358426,
      ID_SLIDER,
      ID_SPIN_CTRL
  };

};

} // end namespace BIAS

#endif //  __PANEL_FOR_DOUBLE_VALUE_WX_HH_
