#ifndef __StringConv_hh__
#define __StringConv_hh__

#ifdef WIN32
#  pragma warning(push)
#  pragma warning(disable: 4005 4996)
#endif // WIN32

#include <wx/string.h>
#include <string>
#include <iostream>
#include <wx/regex.h>

namespace BIAS {

/*
inline const char* WxToAscii(const wxString& thestring)
{
  return (const char*)thestring.mb_str(wxConvUTF8);
}
*/

/// Converts a wxString to a std::string.
///
/// Please note the existing wx macros _  and _T/wxT, too
#define WxToAscii(s) (const char*)(wxString(s).mb_str(wxConvUTF8))
#define WxToFilenames(s) (const char*)(wxString(s).mb_str(wxConvFile))
#define WxToFunctionString(s) (const char*)(wxString(s).fn_str())

/// Converts a C string to a wxString.
/// Please note the existing wx macros _  and _T/wxT, too
inline wxString AsciiToWx(const char* thestring)
{
  return wxString(thestring, wxConvUTF8);
}

inline wxString FilenamesToWx(const char* thestring)
{
  return wxString(thestring, wxConvFile);
}

/// Converts a std::string to a wxString.
/// Please note the existing wx macros _  and _T/wxT, too
inline wxString AsciiToWx(const std::string& thestring)
{
  return wxString(thestring.c_str(), wxConvUTF8);
}

inline wxString FilenamesToWx(const std::string& thestring)
{
  return wxString(thestring.c_str(), wxConvFile);
}


/**
  Class for converting an array of wxStrings to an array
  of non-const ASCII strings. Required for using getopt
  in UNICODE wx tools.
*/
class WxToAsciiArray
{
public:
  WxToAsciiArray(wxChar** strings, int size)
  {
    _AsciiStrings = new char*[size];
    for (int i = 0; i < size; i++)
    {
      _AsciiStrings[i] = strdup(WxToAscii(strings[i]));
    }
    _Size = size;
  }

  ~WxToAsciiArray()
  {
    for (int i = 0; i < _Size; i++)
    {
      free(_AsciiStrings[i]);
    }
    delete[] _AsciiStrings;
  }

  operator char**()
  {
    return _AsciiStrings;
  }

private:
  char** _AsciiStrings;
  int    _Size;
};


inline std::string IsoLatin1ToUtf8(const std::string &s)
{
  return std::string(wxString(s.c_str(),wxCSConv(_T("ISO8859-1"))).mb_str(wxConvUTF8));
}
      

  inline void CleanStringIP(wxString &input, const wxString replaceWith=wxEmptyString)
{ 
  wxRegEx rex( wxT("[^[:alnum:]_()-,]") );
  if ( rex.IsValid() ) {
    rex.ReplaceAll(&input, replaceWith);
  }
}

inline wxString CleanString(const wxString &input, const wxString replaceWith=wxEmptyString)
{ 
  wxString res = input;
  CleanStringIP(res, replaceWith);
  return res;
}


} // namespace BIAS

#ifdef WIN32
#  pragma warning(pop)
#endif // WIN32

#endif // __StringConv_hh__
