#ifdef WIN32
// workaround for wx 2.6.x redefiniton of _CRT_SECURE_NO_DEPRECATE 
// fixed in wx 2.7.2 
#  pragma warning(disable: 4005)
#endif

#include "VideoSource_Controller_PMD.hh"

#ifdef WIN32
#  include <Base/Common/getopt_W32.h>  
#endif

#include <iostream>
#include <Base/Image/ImageIO.hh>
#include <Image/Camera.hh>
#include <Gui/StringConv.hh>

using namespace std;
using namespace BIAS;

BEGIN_EVENT_TABLE(VideoSource_Controller_PMD, wxFrame)
  EVT_BUTTON(ID_HIDE,       VideoSource_Controller_PMD::OnHide)
  EVT_CHECKBOX( ID_CHECK_SHOWDEPTH, VideoSource_Controller_PMD::OnCheckShowDepth )
  EVT_COMBOBOX(ID_MOD,            VideoSource_Controller_PMD::OnModFreq)
  EVT_COMMAND_SCROLL(ID_DIST,     VideoSource_Controller_PMD::OnDistance)
  EVT_SCROLL_THUMBRELEASE( VideoSource_Controller_PMD::OnIntegration)
  EVT_BUTTON(ID_BUTTON_UPDATE, VideoSource_Controller_PMD::OnUpdate)
END_EVENT_TABLE()




VideoSource_Controller_PMD::
VideoSource_Controller_PMD(VideoSource *cam, wxWindow *parent, const std::string &title)
  : VideoSource_Controller_Base(parent, -1, AsciiToWx(title)
  ,wxDefaultPosition,wxSize(500,500),wxSTAY_ON_TOP |wxCAPTION)
{

  Cam_ = dynamic_cast<VideoSource_PMD*>(cam);
  //now it is in milliseconds for one image
  min_integrationTime_=0; max_integrationTime_=120;
  min_distanceOffset_=0; max_distanceOffset_=1500;

  Initialize();

  SetDistanceOffsetRange(min_distanceOffset_, max_distanceOffset_);
  SetIntegrationTimeRange(min_integrationTime_, max_integrationTime_);

  Update();
}



VideoSource_Controller_PMD::
~VideoSource_Controller_PMD()
{

}

void VideoSource_Controller_PMD::
Initialize(){
  
  sliderBox_ShGa_Visible = true;    
  vBoxVisible = true;

  wxSize SlSize(100,200);
  wxSize BtSize(55,30);
  
  wxBoxSizer *slInt = new wxBoxSizer(wxVERTICAL);
  wxStaticText *tInt = new wxStaticText(this,ID_TEXT, wxT("Integration Time \n[ms]"));
  SliderIntegration = new wxSlider(this, ID_INT, 0, min_integrationTime_, 
                                   max_integrationTime_,
                                   wxDefaultPosition,SlSize,
                                   wxSL_VERTICAL | wxSL_LABELS | wxSL_RIGHT,
                                   wxDefaultValidator, wxT("Int"));

  SliderIntegration->Enable(true);
  slInt->Add(tInt,0,wxALL,2);  
  slInt->Add(SliderIntegration,0, wxALL,2);
  
  checkBoxShowDepth = new wxCheckBox( this, ID_CHECK_SHOWDEPTH, wxT("Show Depth"), wxDefaultPosition, wxDefaultSize, 0 );
 
  wxBoxSizer *slMod = new wxBoxSizer(wxVERTICAL);
  wxStaticText *tMod = new wxStaticText(this,ID_TEXT, wxT("Modulation Freq"));
  wxSize comboSize(150,25);
  wxString choices[10] = {wxT("MF_60MHz"),
                         wxT("MF_40MHz"), 
                         wxT("MF_31MHz"),
                         wxT("MF_30MHz"),
                         wxT("MF_29MHz"), 
                         wxT("MF_21MHz"),
                         wxT("MF_20MHz"),
                         wxT("MF_19MHz"), 
                         wxT("MF_15MHz"),
                         wxT("MF_10MHz")};
  comboBoxModFreq = new wxComboBox(this,ID_MOD,wxT("ModFreq"),wxDefaultPosition,
                                   comboSize,10,choices,wxCB_READONLY);
  comboBoxModFreq->SetSelection(6,6);
  comboBoxModFreq->SetValue(wxT("MF_20MHz"));

  comboBoxModFreq->Enable(true);
  slMod->Add(tMod,0,wxALL,2);  
  slMod->Add(comboBoxModFreq,0, wxALL,2);

  slMod->Add( checkBoxShowDepth, 0, wxALL, 5 );


  wxBoxSizer *slDist = new wxBoxSizer(wxVERTICAL);
  wxStaticText *tDist = new wxStaticText(this,ID_TEXT, wxT("Distance Offset"));
  SliderDistanceOffset = new wxSlider(this, ID_DIST, 0, 0, 30000,
                            wxDefaultPosition,SlSize,
                            wxSL_VERTICAL | wxSL_LABELS | wxSL_RIGHT,
                            wxDefaultValidator, wxT("Dist"));

  SliderDistanceOffset->Enable(true);
  slDist->Add(tDist,0,wxALL,2);  
  slDist->Add(SliderDistanceOffset,0, wxALL,2);
  

  showButton = new wxButton(this, ID_HIDE, wxT("Hide"),
			    wxDefaultPosition,BtSize);
  update_Button = new wxButton(this, ID_BUTTON_UPDATE, 
			       wxT("UPDATE"),wxDefaultPosition,BtSize);

  vBox = new wxBoxSizer(wxHORIZONTAL);
  vBox->Add(showButton,0,wxALIGN_LEFT   | wxBOTTOM);
  vBox->Add(update_Button, 0, wxALIGN_RIGHT   | wxBOTTOM); 

  wxBoxSizer *sl = new wxBoxSizer(wxHORIZONTAL);

  sl->Add(slMod,0,wxALL,2);
  sl->Add(slInt,0,wxALL,2); 
  sl->Add(slDist,0,wxALL,2); 
  
  sliderBox_ShGa = new wxBoxSizer(wxVERTICAL);
  sliderBox_ShGa->Add(sl,0,wxALL, 2);
  sliderBox_ShGa->Add(vBox,0,wxALL,2);

  vBox->SetSizeHints(this);
  this->SetSizer(sliderBox_ShGa);
  SetAutoLayout(true);
  Layout();
  Fit();
  Update();
  Show(TRUE);
}

void VideoSource_Controller_PMD::
SetVideoSource(BIAS::VideoSource *cam){
  Cam_ = dynamic_cast<BIAS::VideoSource_PMD*>(cam);
}


void VideoSource_Controller_PMD::
OnHide(wxCommandEvent& event){
  Show(false);
}

void VideoSource_Controller_PMD::
OnClose(wxCloseEvent& event) {

  Show(false);
}

void VideoSource_Controller_PMD::
OnIntegration(wxScrollEvent& event) {
  double value = SliderIntegration->GetValue();
  //cout<<"Value from Integrationtime Slider:"<<value<<endl;
  if (Cam_!= NULL) {
  	//SetIntegrationTime in ms
    Cam_->SetIntegrationTimeMS(unsigned(rint(value)));
  }
}


void VideoSource_Controller_PMD::
OnDistance(wxScrollEvent& event) {
  double value = SliderDistanceOffset->GetValue();
  if (Cam_!= NULL) {
    Cam_->SetDistanceOffset(unsigned(rint(value)));		
  }
}



void VideoSource_Controller_PMD::
Update() {
  if (Cam_!= NULL) {
    unsigned int currentInt = 0;
    Cam_->GetIntegrationTimeMS(currentInt);
    //cout<<"Cam says integration:"<<currentInt<<endl;
    SliderIntegration->SetValue(currentInt);
 		Cam_->GetDistanceOffset(currentInt);
    SliderDistanceOffset->SetValue(currentInt);
  }
}

void VideoSource_Controller_PMD::
OnUpdate(wxCommandEvent& event) {
  Update();
}

void  VideoSource_Controller_PMD::
SetDistanceOffsetRange(unsigned int min,unsigned int max)
{
 SliderDistanceOffset->SetRange(min,max);
}

void  VideoSource_Controller_PMD::
SetIntegrationTimeRange(unsigned int min,unsigned int max)
{
 SliderIntegration->SetRange(min,max);
}


void VideoSource_Controller_PMD::
OnCheckShowDepth(wxCommandEvent& event) {
  Cam_->GrabImageReturnsDepth(event.IsChecked());
}


void VideoSource_Controller_PMD::
OnModFreq(wxCommandEvent& event) {
 std::string text = WxToAscii(comboBoxModFreq->GetValue());
 unsigned int freq = 20000000; //20Mhz

 if(text.compare("MF_60MHz") == 0)
   freq = 60000000;
 else if(text.compare("MF_40MHz") == 0)
   freq = 40000000;
 else if(text.compare("MF_31MHz") == 0)
   freq = 31000000;
 else if(text.compare("MF_30MHz") == 0)
   freq = 30000000;
 else if(text.compare("MF_29MHz") == 0)
   freq = 29000000;
 else if(text.compare("MF_21MHz") == 0)
   freq = 21000000;
 else if(text.compare("MF_20MHz") == 0)
   freq = 20000000;
 else if(text.compare("MF_19MHz") == 0)
   freq = 19000000;
 else if(text.compare("MF_15MHz") == 0)
   freq = 15000000;
 else if(text.compare("MF_10MHz") == 0)
   freq = 10000000;

  if (Cam_!= NULL) {
    Cam_->SetModulationFrequency(freq);		
  }
}
