#ifndef __VideoSource_Controller_PMD_HH__
#define __VideoSource_Controller_PMD_HH__

#include <bias_config.h>

#include <VideoSource/VideoSource_Base.hh>

#ifdef BIAS_HAVE_PMD_PMDTec
#include <VideoSource/VideoSource_PMD.hh>
#endif

#include <Base/Image/ImageIO.hh>
#include <Base/Image/ImageConvert.hh>

#include <Image/Camera.hh>

#include <Gui/VideoSource_Controller_Base.hh>

namespace BIAS {

class BIASGui_EXPORT VideoSource_Controller_PMD : public VideoSource_Controller_Base {
public:

  VideoSource_Controller_PMD(VideoSource *cam, wxWindow *parent, const std::string &title);

  ~VideoSource_Controller_PMD();

  void Initialize();

  void SetVideoSource(BIAS::VideoSource *cam);
  void SetDistanceOffsetRange(unsigned int min,unsigned int max);
  void SetIntegrationTimeRange(unsigned int min,unsigned int max);
  void Update(); // trigger an update, useful for auto-mode
  
  wxBoxSizer        *vBox;
    
  enum {
    ID_HIDE,
    ID_MOD,
    ID_DIST,
    ID_INT,
    ID_BUTTON_UPDATE,
    ID_CHECK_SHOWDEPTH,
    ID_TEXT
  };

private:

  void OnEscape(wxCommandEvent& event);
  void OnHide(wxCommandEvent& event);
  void OnClose(wxCloseEvent& event);
  void OnDistance(wxScrollEvent& event) ;
  void OnIntegration(wxScrollEvent& event) ;
  void OnUpdate(wxCommandEvent& event) ;
  void OnCheckShowDepth(wxCommandEvent& event);
  void OnModFreq(wxCommandEvent& event);
protected:

  BIAS::VideoSource_PMD *Cam_;

  wxSlider          *SliderAmplitude;
  wxSlider          *SliderSaturation;
  wxSlider          *SliderIntegration;
  wxSlider          *SliderDistanceOffset;
  wxSlider          *SliderModCoeff;
  wxBoxSizer        *sliderBox_ShGa;
  bool               sliderBox_ShGa_Visible;
  wxComboBox        *comboBoxModFreq;
  bool               vBoxVisible;
  wxCheckBox        *checkBoxShowDepth;
  wxButton          *showButton;
  wxButton          *update_Button;

  unsigned int min_integrationTime_, max_integrationTime_;
  unsigned int min_distanceOffset_, max_distanceOffset_;

  DECLARE_EVENT_TABLE()

private:
    };

}
#endif
