/*
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003-2009    (see file CONTACT for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <Base/Common/BIASpragmaStart.hh>
#ifdef WIN32
// workaround for wx 2.6.x redefiniton of _CRT_SECURE_NO_DEPRECATE 
// fixed in wx 2.7.2 
#  pragma warning(disable: 4005)
#endif
//include first, avoid macro collision
#include <Gui/StringConv.hh>
#include <iostream>
#include "VideoSource_Controller_SwissRanger.hh"
#include <Base/Common/getopt_W32.h>  
#include <Base/Image/ImageIO.hh>
#include <Image/Camera.hh>
#include <Gui/StringConv.hh>

using namespace std;
using namespace BIAS;

BEGIN_EVENT_TABLE(VideoSource_Controller_SwissRanger, wxFrame)
  EVT_BUTTON(ID_HIDE,       VideoSource_Controller_SwissRanger::OnHide)
  EVT_COMMAND_SCROLL(ID_AMP,      VideoSource_Controller_SwissRanger::OnAmplitude)
  EVT_COMMAND_SCROLL(ID_SAT,      VideoSource_Controller_SwissRanger::OnSaturation)
  EVT_COMBOBOX(ID_MOD,            VideoSource_Controller_SwissRanger::OnModFreq)
  EVT_COMMAND_SCROLL(ID_DIST,     VideoSource_Controller_SwissRanger::OnDistance)
  EVT_COMMAND_SCROLL(ID_INT,      VideoSource_Controller_SwissRanger::OnIntegration)
  EVT_BUTTON(ID_BUTTON_UPDATE, VideoSource_Controller_SwissRanger::OnUpdate)
  EVT_CHECKBOX( ID_CHECK_ENSURELATEST, VideoSource_Controller_SwissRanger::OnCheckEnsureLatest )
  EVT_CHECKBOX( ID_CHECK_SHOWDEPTH, VideoSource_Controller_SwissRanger::OnCheckShowDepth )
END_EVENT_TABLE()


VideoSource_Controller_SwissRanger::
VideoSource_Controller_SwissRanger(VideoSource *cam, wxWindow *parent, const std::string& title)
  : VideoSource_Controller_Base(parent, -1, AsciiToWx(title)
  ,wxDefaultPosition,wxSize(500,500),wxSTAY_ON_TOP |wxCAPTION)
{
  Cam_ = dynamic_cast<VideoSource_SwissRanger*>(cam);

  SliderAmplitude = NULL;
  SliderSaturation = NULL;
  SliderIntegration = NULL;
  mIntTimeMs = NULL,mSaturation = NULL,mAmplitude = NULL,mDistance = NULL;
  SliderDistanceOffset = NULL;
  comboBoxModFreq = NULL;
  checkBoxEnsureLatest = NULL;
  checkBoxShowDepth = NULL;
  sliderBox_ShGa = NULL;
  showButton = NULL;
  update_Button = NULL;

  min_amplitudeThreshold_=0; max_amplitudeThreshold_=2000;
  min_saturationThreshold_=0; max_saturationThreshold_=2000;
  min_integrationTime_=0; max_integrationTime_=0xFF;
  min_distanceOffset_=0; max_distanceOffset_=10000;

  //create controls
  Initialize();
  //set slider values
  SetAmplitudeThresholdRange(min_amplitudeThreshold_, max_amplitudeThreshold_);
  SetSaturationThresholdRange(min_saturationThreshold_, max_saturationThreshold_);
  SetDistanceOffsetRange(min_distanceOffset_, max_distanceOffset_);
  SetIntegrationTimeRange(min_integrationTime_, max_integrationTime_);
}



VideoSource_Controller_SwissRanger::
~VideoSource_Controller_SwissRanger()
{}

void VideoSource_Controller_SwissRanger::
Initialize(){
  
  sliderBox_ShGa_Visible = true;    
  vBoxVisible = true;

  wxSize SlSize(200,35);
  wxSize BtSize(80,30);
  wxSize reduzedDefSize(60,wxDefaultSize.y);
  // the sizer
  wxFlexGridSizer* itemFlexGridSizer = new wxFlexGridSizer(11, 3, 0, 0);
  this->SetSizer(itemFlexGridSizer);
  wxStaticText *tInt = new wxStaticText(this,ID_TEXT, wxT("Integration Time"));
   SliderIntegration = new wxSlider(this, ID_INT, 0, 0, 30000,
                             wxDefaultPosition,SlSize,
                             wxSL_HORIZONTAL  |wxSL_LABELS,                
                             wxDefaultValidator, wxT("Int"));

   SliderIntegration->Enable(true);

   mIntTimeMs = new wxTextCtrl( this, wxID_ANY, wxEmptyString, wxDefaultPosition, reduzedDefSize, 0 );
   mIntTimeMs->SetMaxLength( 4 );
   mIntTimeMs->Enable( false );

   wxStaticText *tms = new wxStaticText(this,ID_TEXT, wxT("ms"));
   wxBoxSizer *vbms = new wxBoxSizer(wxHORIZONTAL);
   vbms->Add(mIntTimeMs,0,wxALIGN_LEFT   | wxBOTTOM);
   vbms->Add(tms, 0, wxALIGN_RIGHT   | wxBOTTOM);

   itemFlexGridSizer->Add(tInt, 0,
                             wxALIGN_CENTER_HORIZONTAL|wxALL, 5);
   itemFlexGridSizer->Add(SliderIntegration, 0,
                             wxALIGN_CENTER_HORIZONTAL|wxALL, 5);
   itemFlexGridSizer->Add(vbms, 0,
                              wxALIGN_CENTER_HORIZONTAL|wxALL, 5);

  wxStaticText *tAmp = new wxStaticText(this,ID_TEXT, wxT("Amplitude Threshold"));

  SliderAmplitude = new wxSlider(this, ID_AMP, 0, 0, 30000,
                            wxDefaultPosition,SlSize,
                            wxSL_HORIZONTAL | wxSL_LABELS ,
                            wxDefaultValidator, wxT("Amp"));
  mAmplitude = new wxTextCtrl( this, wxID_ANY, wxEmptyString, wxDefaultPosition, wxDefaultSize, 0 );
  mAmplitude->SetMaxLength( 4 );
  mAmplitude->Enable( false );

  SliderAmplitude->Enable(true);
  itemFlexGridSizer->Add(tAmp, 0,
                          wxALIGN_CENTER_HORIZONTAL|wxALL, 5);
  itemFlexGridSizer->Add(SliderAmplitude, 0,
                          wxALIGN_CENTER_HORIZONTAL|wxALL, 5);
  itemFlexGridSizer->Add(mAmplitude, 0,
                          wxALIGN_CENTER_HORIZONTAL|wxALL, 5);

  wxStaticText *tSat = new wxStaticText(this,ID_TEXT, wxT("Saturation Threshold"));
  SliderSaturation = new wxSlider(this, ID_SAT, 0, 0, 30000,
                            wxDefaultPosition,SlSize,
                            wxSL_HORIZONTAL | wxSL_LABELS,
                            wxDefaultValidator, wxT("Sat"));
  mSaturation = new wxTextCtrl( this, wxID_ANY, wxEmptyString, wxDefaultPosition, wxDefaultSize, 0 );
  mSaturation->SetMaxLength( 4 );
  mSaturation->Enable( false );

  SliderSaturation->Enable(true);
  itemFlexGridSizer->Add(tSat, 0,
                            wxALIGN_CENTER_HORIZONTAL|wxALL, 5);
  itemFlexGridSizer->Add(SliderSaturation, 0,
                            wxALIGN_CENTER_HORIZONTAL|wxALL, 5);
  itemFlexGridSizer->Add(mSaturation, 0,
                               wxALIGN_CENTER_HORIZONTAL|wxALL, 5);
  
  
  wxStaticText *tDist = new wxStaticText(this,ID_TEXT, wxT("Distance Offset"));
  SliderDistanceOffset = new wxSlider(this, ID_DIST, 0, 0, 30000,
                            wxDefaultPosition,SlSize,
                            wxSL_HORIZONTAL | wxSL_LABELS,
                            wxDefaultValidator, wxT("Dist"));
  SliderDistanceOffset->Enable(true);

  mDistance = new wxTextCtrl( this, wxID_ANY, wxEmptyString, wxDefaultPosition, reduzedDefSize, 0 );
  mDistance->SetMaxLength( 4 );
  mDistance->Enable( false );

  wxStaticText *tms2 = new wxStaticText(this,ID_TEXT, wxT("mm"));
  wxBoxSizer *vbmd = new wxBoxSizer(wxHORIZONTAL);
  vbmd->Add(mDistance,0,wxALIGN_LEFT   | wxBOTTOM);
  vbmd->Add(tms2, 0, wxALIGN_RIGHT   | wxBOTTOM);

  itemFlexGridSizer->Add(tDist, 0,
                            wxALIGN_CENTER_HORIZONTAL|wxALL, 5);
  itemFlexGridSizer->Add(SliderDistanceOffset, 0,
                            wxALIGN_CENTER_HORIZONTAL|wxALL, 5);
  itemFlexGridSizer->Add(vbmd, 0,
                             wxALIGN_CENTER_HORIZONTAL|wxALL, 5);


  wxBoxSizer *slMod = new wxBoxSizer(wxVERTICAL);
  wxStaticText *tMod = new wxStaticText(this,ID_TEXT, wxT("Modulation Freq"));
  wxSize comboSize(150,25);
  wxString choices[10] = {wxT("MF_60MHz"),
                         wxT("MF_40MHz"), 
                         wxT("MF_31MHz"),
                         wxT("MF_30MHz"),
                         wxT("MF_29MHz"), 
                         wxT("MF_21MHz"),
                         wxT("MF_20MHz"),
                         wxT("MF_19MHz"), 
                         wxT("MF_15MHz"),
                         wxT("MF_10MHz")};
  comboBoxModFreq = new wxComboBox(this,ID_MOD,wxT("ModFreq"),wxDefaultPosition,
                                   comboSize,10,choices,wxCB_READONLY);
  comboBoxModFreq->SetSelection(6,6);
  comboBoxModFreq->SetValue(wxT("MF_20MHz"));

  comboBoxModFreq->Enable(true);
  slMod->Add(tMod,0,wxALL,2);  
  slMod->Add(comboBoxModFreq,0, wxALL,2);
  
  checkBoxEnsureLatest = new wxCheckBox( this, ID_CHECK_ENSURELATEST, wxT("Ensure Latest"), wxDefaultPosition, wxDefaultSize, 0 );
  checkBoxShowDepth = new wxCheckBox( this, ID_CHECK_SHOWDEPTH, wxT("Show Depth"), wxDefaultPosition, wxDefaultSize, 0 );
  wxBoxSizer *slCheckBox = new wxBoxSizer(wxVERTICAL);
  slCheckBox->Add( checkBoxEnsureLatest, 0, wxALL, 5 );
  slCheckBox->Add( checkBoxShowDepth, 0, wxALL, 5 );
  
  wxBoxSizer *slButton = new wxBoxSizer(wxVERTICAL);
  showButton = new wxButton(this, ID_HIDE, wxT("Hide"),
			    wxDefaultPosition,BtSize);
  update_Button = new wxButton(this, ID_BUTTON_UPDATE, 
			       wxT("Update"),wxDefaultPosition,BtSize);

  slButton->Add(showButton,0,wxALIGN_LEFT   | wxBOTTOM);
  slButton->Add(update_Button, 0, wxALIGN_RIGHT   | wxBOTTOM);

  itemFlexGridSizer->Add(slMod,0,wxALIGN_CENTER_HORIZONTAL|wxALL, 5);
  itemFlexGridSizer->Add(slCheckBox, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 5);
  itemFlexGridSizer->Add(slButton, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 5);
  
  SetAutoLayout(true);
  Layout();
  Fit();
  Update();
  Show(TRUE);
}

void VideoSource_Controller_SwissRanger::
SetVideoSource(BIAS::VideoSource *cam){
  Cam_ = dynamic_cast<BIAS::VideoSource_SwissRanger*>(cam);
  Initialize();
}


void VideoSource_Controller_SwissRanger::
OnHide(wxCommandEvent& event){
  Show(false);
}

void VideoSource_Controller_SwissRanger::
OnClose(wxCloseEvent& event) {

  Show(false);
}

void VideoSource_Controller_SwissRanger::
OnAmplitude(wxScrollEvent& event) {
	double value = SliderAmplitude->GetValue();
	if (Cam_!= NULL) {
		Cam_->SetAmplitudeThreshold((unsigned short)value);
		wxString tval;
		unsigned short val;
		Cam_->GetAmplitudeThreshold(val);
		tval << val;
		mAmplitude->SetValue(tval);
	}
}

void VideoSource_Controller_SwissRanger::
OnSaturation(wxScrollEvent& event) {
	double value = SliderSaturation->GetValue();
	if (Cam_!= NULL) {
		Cam_->SetSaturationThreshold((unsigned short)value);
		unsigned short val;
		wxString tval;
		Cam_->GetSaturationThreshold(val);
		tval << val;
		mSaturation->SetValue(tval);
	}
}

void VideoSource_Controller_SwissRanger::
OnIntegration(wxScrollEvent& event) {
  double value = SliderIntegration->GetValue();
  if (Cam_!= NULL) {
    Cam_->SetIntegrationTime((unsigned char)value);		
  }
  wxString tval;
  float fval;
  Cam_->GetIntegrationTimeMS(fval);
  tval << fval;
  mIntTimeMs->SetValue(tval);
}

void VideoSource_Controller_SwissRanger::
OnModFreq(wxCommandEvent& event) {
 std::string text = WxToAscii(comboBoxModFreq->GetValue());
 ModulationFrq freq = MF_20MHz;

 if(text.compare("MF_60MHz") == 0)
   freq = MF_60MHz;
 else if(text.compare("MF_40MHz") == 0)
   freq = MF_40MHz;
 else if(text.compare("MF_31MHz") == 0)
   freq = MF_31MHz;
 else if(text.compare("MF_30MHz") == 0)
   freq = MF_30MHz;
 else if(text.compare("MF_29MHz") == 0)
   freq = MF_29MHz;
 else if(text.compare("MF_21MHz") == 0)
   freq = MF_21MHz;
 else if(text.compare("MF_20MHz") == 0)
   freq = MF_20MHz;
 else if(text.compare("MF_19MHz") == 0)
   freq = MF_19MHz;
 else if(text.compare("MF_15MHz") == 0)
   freq = MF_15MHz;
 else if(text.compare("MF_10MHz") == 0)
   freq = MF_10MHz;

  if (Cam_!= NULL) {
    Cam_->SetModulationFrequency(freq);		
  }
}

void VideoSource_Controller_SwissRanger::
OnDistance(wxScrollEvent& event) {
  double value = SliderDistanceOffset->GetValue();
  if (Cam_!= NULL) {
  	Cam_->SetDistanceOffset((unsigned short)value);
  	unsigned short val;
  	wxString tval;
  	Cam_->GetDistanceOffset(val);
  	tval << val;
  	mDistance->SetValue(tval);
  }
}



void VideoSource_Controller_SwissRanger::
Update() {
  if (Cam_!= NULL) {
    unsigned short currentShort =0;
    unsigned char currentChar =0;
    unsigned int currentInt = 0;

		Cam_->GetAmplitudeThreshold(currentShort);
    SliderAmplitude->SetValue(int(currentShort));
		Cam_->GetSaturationThreshold(currentShort);
    SliderSaturation->SetValue(int(currentShort));
    Cam_->GetIntegrationTime(currentChar);
    SliderIntegration->SetValue(int(currentChar));
 		Cam_->GetDistanceOffset(currentShort);
    SliderDistanceOffset->SetValue(int(currentShort));
    unsigned int freq = MF_20MHz;
 		Cam_->GetModulationFrequency(freq);
    
    if(freq == MF_60MHz)
      currentInt = 0;
    else if(freq == MF_40MHz)
      currentInt = 1;
    else if(freq == MF_31MHz)
      currentInt = 2;
    else if(freq == MF_30MHz)
      currentInt = 3;
    else if(freq == MF_29MHz)
      currentInt = 4;
    else if(freq == MF_21MHz)
      currentInt = 5;
    else if(freq == MF_20MHz)
      currentInt = 6;
    else if(freq == MF_19MHz)
      currentInt = 7;
    else if(freq == MF_15MHz)
      currentInt = 8;
    else if(freq == MF_10MHz)
      currentInt = 9;

    comboBoxModFreq->SetSelection(currentInt,currentInt);
  }
   
}

void VideoSource_Controller_SwissRanger::
OnUpdate(wxCommandEvent& event) {
  Update();
}

void  VideoSource_Controller_SwissRanger::
SetAmplitudeThresholdRange(unsigned int min,unsigned int max)
{
  SliderAmplitude->SetRange(min,max);

}

void  VideoSource_Controller_SwissRanger::
SetSaturationThresholdRange(unsigned int min,unsigned int max)
{
  SliderSaturation->SetRange(min,max);

}


void  VideoSource_Controller_SwissRanger::
SetDistanceOffsetRange(unsigned int min,unsigned int max)
{
 SliderDistanceOffset->SetRange(min,max);
}

void  VideoSource_Controller_SwissRanger::
SetIntegrationTimeRange(unsigned int min,unsigned int max)
{
 SliderIntegration->SetRange(min,max);
}

void VideoSource_Controller_SwissRanger::
OnCheckEnsureLatest(wxCommandEvent& event) {
  Cam_->EnsureLatestFrame(event.IsChecked());
}

void VideoSource_Controller_SwissRanger::
OnCheckShowDepth(wxCommandEvent& event) {
  Cam_->GrabImageReturnsDepth(event.IsChecked());
}


#include <Base/Common/BIASpragmaEnd.hh>

