/*
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003-2009    (see file CONTACT for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __VideoSource_Controller_SWISSRANGER_HH__
#define __VideoSource_Controller_SWISSRANGER_HH__

#include <Gui/VideoSource_Controller_Base.hh>
#include <bias_config.h>

#ifdef WIN32
#include <windows.h>
#endif

#include <VideoSource/VideoSource_Base.hh>

#ifdef BIAS_HAVE_SWISSRANGER
#include <VideoSource/VideoSource_SwissRanger.hh>
#endif


#include <Base/Image/ImageIO.hh>
#include <Base/Image/ImageConvert.hh>
#include <Image/Camera.hh>
#include <Image/StereoRedGreen.hh>


namespace BIAS {

class BIASGui_EXPORT VideoSource_Controller_SwissRanger : public VideoSource_Controller_Base {
public:

  VideoSource_Controller_SwissRanger(VideoSource *cam, wxWindow *parent, const std::string& title="VideoSourceController SwissRanger");
  ~VideoSource_Controller_SwissRanger();

  void Initialize();
  void SetVideoSource(BIAS::VideoSource *cam);
  void SetAmplitudeThresholdRange(unsigned int min,unsigned int max);
  void SetSaturationThresholdRange(unsigned int min,unsigned int max);
  void SetDistanceOffsetRange(unsigned int min,unsigned int max);
  void SetIntegrationTimeRange(unsigned int min,unsigned int max);
  void Update(); // trigger an update, useful for auto-mode
  
  wxBoxSizer        *vBox;
    
  enum {
    ID_HIDE,
    ID_AMP,
    ID_SAT,
    ID_MOD,
    ID_DIST,
    ID_INT,
    ID_BUTTON_UPDATE,
    ID_TEXT,
    ID_CHECK_ENSURELATEST,
    ID_CHECK_SHOWDEPTH
  };

private:

  void OnEscape(wxCommandEvent& event);
  void OnHide(wxCommandEvent& event);
  void OnClose(wxCloseEvent& event);
  void OnAmplitude(wxScrollEvent& event);
  void OnSaturation(wxScrollEvent& event);
  void OnModulation(wxScrollEvent& event);
  void OnDistance(wxScrollEvent& event) ;
  void OnIntegration(wxScrollEvent& event) ;
  void OnModFreq(wxCommandEvent& event) ;
  void OnUpdate(wxCommandEvent& event) ;
  void OnCheckEnsureLatest(wxCommandEvent& event);
  void OnCheckShowDepth(wxCommandEvent& event);
  
protected:
  
  BIAS::VideoSource_SwissRanger *Cam_;
  
  wxSlider          *SliderAmplitude;
  wxSlider          *SliderSaturation;
  wxSlider          *SliderIntegration;
  wxTextCtrl        *mIntTimeMs,*mSaturation,*mAmplitude,*mDistance;
  wxSlider          *SliderDistanceOffset;
  wxComboBox        *comboBoxModFreq;
  wxCheckBox        *checkBoxEnsureLatest;
  wxCheckBox        *checkBoxShowDepth;
  wxBoxSizer        *sliderBox_ShGa;
  wxButton          *showButton;
  wxButton          *update_Button;

  bool               sliderBox_ShGa_Visible; 
  bool               vBoxVisible;

  unsigned short min_amplitudeThreshold_, max_amplitudeThreshold_;
  unsigned short min_saturationThreshold_, max_saturationThreshold_;
  unsigned char min_integrationTime_, max_integrationTime_;
  unsigned short min_distanceOffset_, max_distanceOffset_;

  DECLARE_EVENT_TABLE()

private:
  };

}
#endif
