#ifndef __VideoSource_Controller_XB3_HH__
#define __VideoSource_Controller_XB3_HH__

#include <bias_config.h>

#include <Base/Common/W32Compat.hh>
#include <VideoSource/VideoSource_Base.hh>

#include <VideoSource/VideoSource_DCAM_BumbleBee.hh>

#include <Base/Image/ImageIO.hh>
#include <Base/Image/ImageConvert.hh>

#include <Image/Camera.hh>
#include <Image/StereoRedGreen.hh>

#include <Gui/VideoSource_Controller_Base.hh>

namespace BIAS {

class BIASGui_EXPORT VideoSource_Controller_XB3 : public VideoSource_Controller_Base {
public:

  VideoSource_Controller_XB3(VideoSource *cam, wxWindow *parent, const std::string &title="VideoSourceConstroller XB3",
    const unsigned int imageToShow=(unsigned int)0,
		double minShutter=1.0, double maxShutter=80.0,
		double minGain = 0.0,double maxGain=3000.0);
  ~VideoSource_Controller_XB3();

  void Initialize();

  void SetVideoSource(BIAS::VideoSource *cam);
  void SetShutterRange(unsigned int min,unsigned int max);
  void SetGainRange(unsigned int min,unsigned int max);
  void Update(); // trigger an update, useful for auto-mode
  int GetSelectedImage();

  wxBoxSizer        *vBox;
    
  enum ids{
    ID_HIDE,
    //All 16 FEATURES of DCAM
    //GAIN, SHUTTER PANEL
    ID_GAIN,
    ID_SHUTTER,
    ID_BUTTON_AUTO_GAIN,
    ID_BUTTON_AUTO_SHUTTER,
    ID_BUTTON_UPDATE,
    ID_BUTTON_WHITEBALANCE,
    //HUE, SATURATION
    ID_HUE,
    ID_SATURATION,

    //GAMMA, BRIGHTNESS
    ID_GAMMA,
    ID_BRIGHTNESS,

    ID_EXPOSURE,
    ID_SHARPNESS,
    ID_WHITE_BALANCE,
    ID_IRIS,
    ID_FOCUS,
    ID_TEMPERATURE,
    ID_TRIGGER,
    ID_TRIGGER_DELAY,
    ID_WHITE_SHADING,
    ID_FRAME_RATE,

    ID_TEXT,
    ID_CHECKBOX_LEFT,
    ID_CHECKBOX_CENTER,
    ID_CHECKBOX_RIGHT,
    ID_CHECKBOX_ALL,
    ID_CHECKBOX_RAW
  };

private:

  void OnEscape(wxCommandEvent& event);
  
  void OnHide(wxCommandEvent& event);
  void OnClose(wxCloseEvent& event);
  void OnWhiteBalance(wxCommandEvent& event);
  void OnShutter(wxScrollEvent& event);
  void OnGain(wxScrollEvent& event);
  void OnAuto_Shutter(wxCommandEvent& event);
  void OnAuto_Gain(wxCommandEvent& event);
  void OnUpdate(wxCommandEvent& event) ;
  void OnRadioButton(wxCommandEvent& event) ;

  void auto_on(int param);
  void auto_off(int param);

protected:
  
  BIAS::VideoSource_DCAM_BumbleBee *Cam_;
  
  wxSlider          *SliderGain;
  wxSlider          *SliderShutter;
  wxBoxSizer        *sliderBox_ShGa;
  wxBoxSizer        *layoutBox;
  bool               sliderBox_ShGa_Visible;
  bool               auto_shutter;
  bool               auto_gain;
  
  bool               vBoxVisible;

  wxButton          *showButton;
  wxButton          *whiteBalanceButton; 
  wxButton          *auto_sh_Button;
  wxButton          *auto_ga_Button;
  wxButton          *update_Button;

  wxRadioButton        *left_CheckBox;
  wxRadioButton        *center_CheckBox;
  wxRadioButton        *right_CheckBox;
  wxRadioButton        *all_CheckBox;
  wxRadioButton        *raw_CheckBox;
  double minShutter_, maxShutter_, minGain_,maxGain_;

  DECLARE_EVENT_TABLE()

private:

 unsigned int ImageToShow_;
    };

}
#endif
