#include <Base/Common/W32Compat.hh>
#include "ZoomFrame.hh"
#include <Base/Image/Image.hh>
#include <Gui/ZoomImageCanvas.hh>
#include <Gui/StringConv.hh>


using namespace BIAS;
using namespace std;

ZoomFrame::
~ZoomFrame()
{}


ZoomFrame::
ZoomFrame(wxWindow* parent, const wxString& title)
  : wxFrame(parent, -1, title)
{
  Setup();
  Show();
}


void ZoomFrame::
Setup()
{
  int widths[] = { -1, 280 };
  CreateStatusBar(2);
  SetStatusWidths(2, widths);

  QuitButton_ = new wxButton(this, MIA_ZoomQuitButton, wxT("Close"));

  Window_ = new ZoomImageCanvas(this, GetStatusBar(), 1);

  SLScale_ = new wxSlider(this, -1, 10, 1, 25, wxDefaultPosition,
    wxSize(-1, -1), wxSL_HORIZONTAL);

  wxBoxSizer *box = new wxBoxSizer(wxVERTICAL);
  wxBoxSizer *hbox = new wxBoxSizer(wxHORIZONTAL);

  box->Add(QuitButton_, 0, wxALL | wxGROW, 3);
  box->Add(Window_, 1, wxALL | wxGROW, 3);
  hbox->Add(new wxStaticText( this, -1, _T("Zoom") ), 0, wxALL, 5);
  hbox->Add(SLScale_, 1, wxALL | wxGROW, 5);
  box->Add(hbox, 0, wxGROW, 0);

  box->SetSizeHints(this);
  SetSizer(box);
}

void ZoomFrame::
NewImage(BIAS::ImageBase& im,
         BIAS::Image<unsigned char>& ucim,
         const wxString& name)
{
  Window_->NewImage(im, ucim);
  if(GetStatusBar()!=NULL)
  	SetStatusText(name, 0);
}


void ZoomFrame::
NewCoo(unsigned x, unsigned y)
{ Window_->NewCoo(x, y); }



BEGIN_EVENT_TABLE (ZoomFrame, wxFrame)
EVT_BUTTON      (MIA_ZoomQuitButton, ZoomFrame::OnQuit)
EVT_SCROLL      (ZoomFrame::OnZoomFactor)
EVT_CLOSE       (ZoomFrame::OnClose)
//EVT_KEY_DOWN    (ZoomFrame::OnKeyPress)
END_EVENT_TABLE()

//void ZoomFrame::OnQuit(wxCommandEvent & event)
void ZoomFrame::OnQuit(wxCommandEvent &)
{
  Close(true);
}

//void ZoomFrame::OnClose(wxCloseEvent & event)
void ZoomFrame::OnClose(wxCloseEvent &)
{
  /*
  ((ImageCanvas*)GetParent())->SetZoomed(false);
  Destroy();
  */
  Hide();
}

void ZoomFrame::SetZoom(unsigned z)
{
  BIASASSERT(z>=1);
  BIASASSERT(z<=25);
  SLScale_->SetValue(z);
  Window_->NewZoom(SLScale_->GetValue());
}

//void ZoomFrame::OnZoomFactor(wxScrollEvent & event)
void ZoomFrame::OnZoomFactor(wxScrollEvent&)
{ Window_->NewZoom(SLScale_->GetValue()); }

// void ZoomFrame::OnKeyPress(wxKeyEvent &event)
// {
//   cerr <<"ZoomFrame::OnKeyPress\n";
//   event.Skip();
// }

