#include "ZoomImageCanvas.hh"

#include <Gui/StringConv.hh>

using namespace BIAS;
using namespace std;

ZoomImageCanvas::
ZoomImageCanvas(wxWindow* parent, wxStatusBar *sbar,
                 int sbarid, wxWindowID id, const wxPoint& pos,
                 const wxSize& size, long style,
                 const wxString& name)
                 : wxPanel(parent, id, pos, size, style, name)
{
  Bitmap_=NULL;
  wximage_ = NULL;

  ZoomFactor_=10;
  ZoomX_=0;
  ZoomY_=0;
  SBar_=sbar;
  SBarID_=sbarid;
  mousex_=mousey_=-1;
  ClearDrawingContext_ = true;
  ZoomXPos_ = ZoomYPos_ = 0;
  wwidth_ = wheight_ = 0;
}


ZoomImageCanvas::
~ZoomImageCanvas()
{
  if (wximage_) { delete wximage_; wximage_=NULL; }
  if (Bitmap_) { delete Bitmap_; Bitmap_=NULL; }
}


void ZoomImageCanvas::
GetZoomImage()
{
#ifndef UNIX
#ifdef __APPLE__
  if (wwidth_==0 || wheight_==0){
#endif
  GetSize(&wwidth_, &wheight_);
  ZoomXPos_=wwidth_/2-ZoomFactor_/2;
  ZoomYPos_=wheight_/2-ZoomFactor_/2;
#ifdef __APPLE__
  }
#endif 
#endif //WIN32



  if (zim_.GetWidth()!=(unsigned)wwidth_ ||
    zim_.GetHeight()!=(unsigned)wheight_)
    zim_.Release();
  if (zim_.IsEmpty())
    zim_.Init(wwidth_, wheight_, 3);

  zim_.FillImageWithConstValue((unsigned char)0);
  unsigned char **ida = zim_.GetImageDataArray();
  unsigned char **sida= ucim_.GetImageDataArray();
  int xsize=wwidth_/ZoomFactor_/2+1;
  int ysize=wheight_/ZoomFactor_/2+1;
  register unsigned int xstart, ystart;
  register unsigned char res[3];
  unsigned width=ucim_.GetWidth(), height=ucim_.GetHeight();
  int mx, my;

  //cout << "wwidth/wheight: "<<wwidth_<<"x"<<wheight_<<endl;
  //cout << "width/height: "<<width<<"x"<<height<<endl;
  //cout << "x "<<(int)ZoomX_-(int)xsize<<" - "<<(int)ZoomX_+(int)xsize<<endl;
  //cout << "y "<<(int)ZoomY_-(int)ysize<<" - "<<(int)ZoomY_+(int)ysize<<endl;
  for (int x=(int)ZoomX_-(int)xsize; x<=(int)ZoomX_+(int)xsize; x++){
    for (int y=(int)ZoomY_-(int)ysize; y<=(int)ZoomY_+(int)ysize; y++){
      if (x<0 || y<0 || x>=(int)width || y>=(int)height){
        //cout <<"out of src range: "<< x << "x" <<y<<endl;
        res[0]=res[1]=res[2]=0;
      } else {
        mx=3*x;
        res[0]=sida[y][mx]; res[1]=sida[y][mx+1]; res[2]=sida[y][mx+2];
      }
      xstart = (x-(int)ZoomX_)*(int)ZoomFactor_+(int)ZoomXPos_;
      ystart = (y-(int)ZoomY_)*(int)ZoomFactor_+(int)ZoomYPos_;
      for (register int i=0; i<(int)ZoomFactor_; i++){
        for (register int j=0; j<(int)ZoomFactor_; j++){
          my=ystart+i; mx=(xstart+j);
          if (mx>=0 && my>=0 && mx<wwidth_ && my<wheight_){
            mx*=3;
            ida[my][mx] = res[0];
            ida[my][mx+1] = res[1];
            ida[my][mx+2] = res[2];
          }
        }
      }
    }
  }
  wxbitmap_mutex_.Lock();
  if (wximage_) { delete wximage_; wximage_=NULL; }
  wximage_ = new wxImage(zim_.GetWidth(), zim_.GetHeight(),
                         zim_.GetImageData(), true);
  wxbitmap_mutex_.Unlock();
  if (mousex_>=0 && mousey_>=0){
    UpdateStatusBar((unsigned) mousex_, (unsigned) mousey_);
  }
}

void ZoomImageCanvas::
UpdateStatusBar(unsigned mouseX, unsigned mouseY)
{
  if(SBar_ == NULL)
    return;
  if(Bitmap_==NULL)
    return;

  // compute original coordinate for zoomed canvas
  int ix=(int)mouseX-(int)ZoomXPos_;
  int iy=(int)mouseY-(int)ZoomYPos_;
  if (ix<0)
    ix=ix/(int)ZoomFactor_+(int)ZoomX_-1;
  else
    ix=ix/(int)ZoomFactor_+(int)ZoomX_;

  if (iy<0)
    iy=iy/(int)ZoomFactor_+(int)ZoomY_-1;
  else
    iy=iy/(int)ZoomFactor_+(int)ZoomY_;

  //BIAS_MUTEX_LOCK(wxbitmap_mutex_);
  origim_mutex_.Lock();

  {
    if(ix>=0 && ix<(int)origim_.GetWidth()
       && iy>=0 && iy<(int)origim_.GetHeight())
      {
        //wxbitmap_mutex_.Unlock();

        ostringstream os;
        //display coordinate:
        os << "("<<setw(3)<<ix<<", "<<setw(3)<<iy<<") : (";

        // determine pixel type relying on storage type
        // dynamic_check to BIAS::Image<T> does not work because base image is
        // loaded
        {
          for (unsigned int channel=0;
               channel<origim_.GetChannelCount();
               channel++){
            if (channel!=0)
              os <<","; // no comma before first value
            BIAS::ImageBase::PrintPixelValue(origim_, ix,iy, (unsigned short)channel, os );
          }
        }
        os <<")";
        SBar_->SetStatusText(AsciiToWx(os.str()), SBarID_);
      };
  }
  origim_mutex_.Unlock();
  //wxbitmap_mutex_.Unlock();
}


void ZoomImageCanvas::
NewImage(ImageBase& im, Image<unsigned char>& ucim)
{
  origim_mutex_.Lock();
  origim_ = im;
  origim_mutex_.Unlock();
  ucim_ = ucim;
  GetZoomImage();
#ifdef WIN32
  this->Refresh();
#else
  wxPaintEvent pe;
  wxPostEvent(this->GetEventHandler(),pe);
#endif
}


void ZoomImageCanvas::
NewCoo(unsigned x, unsigned y)
{
  if (ZoomX_!=x || ZoomY_!=y){
    ZoomX_=x;
    ZoomY_=y;
    ZoomXPos_=wwidth_/2-ZoomFactor_/2;
    ZoomYPos_=wheight_/2-ZoomFactor_/2;
    GetZoomImage();
#ifdef WIN32
    this->Refresh();
#else
    wxPaintEvent pe;
    wxPostEvent(this->GetEventHandler(),pe);
#endif
  }
}


void ZoomImageCanvas::
NewZoom(int zoom)
{
  if ((unsigned)zoom!=ZoomFactor_){
    ZoomFactor_=zoom;
    GetZoomImage();
#ifdef WIN32
    this->Refresh();
#else
    wxPaintEvent pe;
    wxPostEvent(this->GetEventHandler(),pe);
#endif
  }
}


BEGIN_EVENT_TABLE (ZoomImageCanvas, wxWindow)
EVT_PAINT       (ZoomImageCanvas::OnPaint)
EVT_MOTION      (ZoomImageCanvas::OnMouseMove)
EVT_RIGHT_DOWN  (ZoomImageCanvas::OnRightMouse)
EVT_LEFT_DOWN   (ZoomImageCanvas::OnLeftMouse)
EVT_LEFT_DCLICK (ZoomImageCanvas::OnLeftMouseDoubleClicked)
EVT_CHAR        (ZoomImageCanvas::OnKey)
EVT_SIZE        (ZoomImageCanvas::OnResize)
EVT_LEAVE_WINDOW(ZoomImageCanvas::OnMouseLeave)
END_EVENT_TABLE()


void ZoomImageCanvas::
OnRightMouse(wxMouseEvent &event)
{
  wxClientDC dc(this);
  PrepareDC(dc);
  wxPoint pos = event.GetPosition();
  int x = dc.DeviceToLogicalX( pos.x );
  int y = dc.DeviceToLogicalY( pos.y );
  int ix=(x-(int)(wwidth_>>1))/(int)ZoomFactor_+(int)ZoomX_;
  int iy=(y-(int)(wheight_>>1))/(int)ZoomFactor_+(int)ZoomY_;
  //cerr << "new coo: "<<ix<<", "<<iy; cerr.flush();
  NewCoo(ix, iy);
  //cerr << " .... finished\n";
}

//void ZoomImageCanvas::OnMouseLeave(wxMouseEvent &event)
void ZoomImageCanvas::
OnMouseLeave(wxMouseEvent&)
{
  if (SBar_){ SBar_->SetStatusText(wxT(""), SBarID_); }
  mousex_=mousey_=-1;
}


void ZoomImageCanvas::
OnResize(wxSizeEvent &event)
{
  GetSize(&wwidth_, &wheight_);
  ZoomXPos_=wwidth_/2-ZoomFactor_/2;
  ZoomYPos_=wheight_/2-ZoomFactor_/2;
  GetZoomImage();
  event.Skip();
#ifdef WIN32
  this->Refresh();
#else
  wxPaintEvent pe;
  wxPostEvent(this->GetEventHandler(),pe);
#endif
}

//void ZoomImageCanvas::OnPaint(wxPaintEvent & event)
void ZoomImageCanvas::
OnPaint(wxPaintEvent & )
{
  //  wxPanel::OnPaint(event);

  wxBufferedPaintDC dc(this);
  PrepareDC(dc);
  wxMutexLocker locker(wxbitmap_mutex_);
  if (wximage_ && wximage_->IsOk()){
    if (Bitmap_ != NULL) {
      delete Bitmap_; Bitmap_ = NULL;
    }
    Bitmap_ = new wxBitmap(*wximage_);
  }
  if (Bitmap_ && Bitmap_->IsOk() && dc.IsOk()) {
    // since now using double buffering, turning of clearing is actually
    // unnecessary
    if (ClearDrawingContext_)  {
      dc.Clear();
    }
    dc.DrawBitmap(*Bitmap_, 0, 0);
  }
}

void ZoomImageCanvas::OnMouseMove(wxMouseEvent &event)
{
  wxClientDC dc(this);
  PrepareDC(dc);
  wxPoint pos = event.GetPosition();
  mousex_ = dc.DeviceToLogicalX( pos.x );
  mousey_ = dc.DeviceToLogicalY( pos.y );
  UpdateStatusBar((unsigned) mousex_, (unsigned) mousey_);
}


void ZoomImageCanvas::
OnLeftMouse(wxMouseEvent &event)
{
  // pass event through to ImageCanvas which is parent of parent
  if (GetParent() && GetParent()->GetParent()){
    wxClientDC dc(this);
    PrepareDC(dc);
    wxPoint pos = event.GetPosition();
    unsigned int x = dc.DeviceToLogicalX( pos.x );
    unsigned int y = dc.DeviceToLogicalY( pos.y );
    int ix=(int)x-(int)ZoomXPos_, iy=(int)y-(int)ZoomYPos_;
    if (ix<0) ix=ix/(int)ZoomFactor_+(int)ZoomX_-1;
    else ix=ix/(int)ZoomFactor_+(int)ZoomX_;
    if (iy<0) iy=iy/(int)ZoomFactor_+(int)ZoomY_-1;
    else iy=iy/(int)ZoomFactor_+(int)ZoomY_;

    wxMouseEvent ke=event;
    ke.m_x=ix;
    ke.m_y=iy;
    ke.SetId(GetParent()->GetParent()->GetId());
    wxPostEvent(GetParent()->GetParent()->GetEventHandler(), ke);
    //cerr << "passing mouse event at ("<<ix<<", "<<iy
    //     <<") from ZoomImageCanvas to ImageCanvas\n";
  } else {
    BIASERR("no ImageCanvas belonging to ZoomImageCanvas\n");
  }
  event.Skip();
}


void ZoomImageCanvas::
OnLeftMouseDoubleClicked(wxMouseEvent &event)
{
  // pass event through to ImageCanvas which is parent of parent
  if (GetParent()){
    //  if (GetParent() && GetParent()->GetParent()){
    wxClientDC dc(this);
    PrepareDC(dc);
    wxPoint pos = event.GetPosition();
    unsigned int x = dc.DeviceToLogicalX( pos.x );
    unsigned int y = dc.DeviceToLogicalY( pos.y );
    int ix=(int)x-(int)ZoomXPos_, iy=(int)y-(int)ZoomYPos_;
    if (ix<0) ix=ix/(int)ZoomFactor_+(int)ZoomX_-1;
    else ix=ix/(int)ZoomFactor_+(int)ZoomX_;
    if (iy<0) iy=iy/(int)ZoomFactor_+(int)ZoomY_-1;
    else iy=iy/(int)ZoomFactor_+(int)ZoomY_;

    wxMouseEvent ke=event;
    ke.m_x=ix;
    ke.m_y=iy;
    ke.SetId(GetParent()->GetParent()->GetId());
    wxPostEvent(GetParent()->GetParent()->GetEventHandler(), ke);
    //cerr << "passing mouse event at ("<<ix<<", "<<iy
    //     <<") from ZoomImageCanvas to ImageCanvas\n";
  } else {
    BIASERR("no ImageCanvas belonging to ZoomImageCanvas\n");
  }
  event.Skip();
}


void ZoomImageCanvas::
OnKey(wxKeyEvent &event)
{
  // pass event through to ImageCanvas which is parent of parent
  if (GetParent()){
    //  if (GetParent() && GetParent()->GetParent()){
    wxClientDC dc(this);
    PrepareDC(dc);
    wxPoint pos = event.GetPosition();
    unsigned int x = dc.DeviceToLogicalX( pos.x );
    unsigned int y = dc.DeviceToLogicalY( pos.y );
    int ix=(int)x-(int)ZoomXPos_, iy=(int)y-(int)ZoomYPos_;
    if (ix<0) ix=ix/(int)ZoomFactor_+(int)ZoomX_-1;
    else ix=ix/(int)ZoomFactor_+(int)ZoomX_;
    if (iy<0) iy=iy/(int)ZoomFactor_+(int)ZoomY_-1;
    else iy=iy/(int)ZoomFactor_+(int)ZoomY_;

    wxKeyEvent ke=event;
    ke.m_x=ix;
    ke.m_y=iy;
    ke.SetId(GetParent()->GetParent()->GetId());
    ke.m_keyCode=event.GetKeyCode();
    wxPostEvent(GetParent()->GetParent()->GetEventHandler(), ke);
    // cerr << "passing KEY event at ("<<ix<<", "<<iy
    //     <<") from ZoomImageCanvas to ImageCanvas\n";
  } else {
    BIASERR("no ImageCanvas belonging to ZoomImageCanvas\n");
  }
  event.Skip();
}
