#ifndef _biasgl_h_
#define _biasgl_h_

/** 
\brief wrapper to include all gl, glut, glew headers that corresponds to current HAVE_.. configurations settings.

Requires OpenGL (BIAS_HAVE_OPENGL).
Switches glew usage optionally based on B
\attention Does NOT include glu.h.

- keep choice of using glew/gl/glut/...  flexible in client apps
  while asserting glew is included first!
- disable GL warnings in a central place.
- allows precompiled headers
- Please note that the app. has to call glewinit like this 
iff glew is used: 
\verbatim
  if (!(GLEW_VERSION_1_1)) {
    // init and try again:
    GLenum result = glewInit();
    if (result != GLEW_OK){
      BIASERR("glewInit failed.");
    };
  }
\endverbatim
\author: Jan Woetzel, Bogumil Bartczak 
*/

#include <bias_config.h>
#include <Base/Common/W32Compat.hh>
#ifndef BIAS_HAVE_OPENGL
#  error Please recompile BIAS with USE_OPENGL to use OpenGL.
#endif // BIAS_HAVE_OPENGL 


#ifdef WIN32
// NO lean and mean here because too many BIAS files 
// rely on global windows.h instead of direct includes 
// for the specialized headers like Winsock2.h JW
//#  define WIN32_LEAN_AND_MEAN
#  include <windows.h>
#  ifdef BIAS_HAVE_WXWIDGETS
#    include <wx/msw/winundef.h>
#  endif
#endif


#if defined(BIAS_HAVE_GLEW) || defined(MIP_HAVE_GLEW)

// USE glew:
// must define kind of lib/src for glew extension handler lib
// (linker errors otherwise) !
// decide weterh we want to link against static or dll glew

// Win32 static build or Linux

#ifdef BIAS_BUILD_SHARED_LIBS
/*
 GLEW_STATIC needs to be set when using the static version.
 GLEW_BUILD is set when building the DLL version.

 this is what happens in glew.h :
// #ifdef GLEW_STATIC
// #  define GLEWAPI extern
// #else
// #  ifdef GLEW_BUILD
// #    define GLEWAPI extern __declspec(dllexport)
// #  else
// #    define GLEWAPI extern __declspec(dllimport)
// #  endif
// #endif 

*/ 
  #ifdef WIN32
  //adding the declspecs described above by defining glew_build
  //creates linker errors when linking against the glew32.dll.
  //if we link against the glew.lib statically while building shared bias.libs
  //we create multiple instances of glew which is quite undesirable
  //not defining glew_build seems to work.
  // #define GLEW_BUILD   
 /*   #  ifdef GLEW_STATIC
    #    error GLEW_STATIC is defined but shouldnt be in DLL BUILD.
    #  endif*/
  #else //win32
    #  ifndef GLEW_STATIC
    #    define GLEW_STATIC
    #  endif // GLEW_STATIC
  #endif //win32
#else  //MIP_BUILD_SHARED_LIBS
  #  ifndef GLEW_STATIC
  #    define GLEW_STATIC
  #  endif // GLEW_STATIC
#endif //MIP_BUILD_SHARED_LIBS

//// checks, only one of both!
//#  ifdef GLEW_BUILD
//#    error GLEW_BUILD is defined but shouldnt be, GLEW_STATIC is too.
//#  endif


//#endif // WIN32 _DLL



#  include <GL/glew.h> // includes gl.h and glu.h
#  ifdef WIN32
#    include <GL/wglew.h>
#  else // WIN32
// workaround for glx
// do NOT use glxew for now because glxChooseFBConfig crashes, then.
//#    include <GL/glxew.h>

#  endif //WIN32

#else //BIAS_HAVE_GLEW or MIP_HAVE_GLEW

# ifdef __APPLE__
#  include <OpenGL/gl.h>
# else // __APPLE__
#  include <GL/gl.h>
# endif // __APPLE__

#endif // BIAS_HAVE_GLEW or MIP_HAVE_GLEW



//
// Do not include wxglcanvas here 
// because then we are bound to glew if compiling with wx
// because wx/glcanvas.h includes glx.h (JW)
// Thus, include glxew.h *before* the few
//
//#if defined(BIAS_HAVE_WXWIDGETS) || defined(MIP_HAVE_WXWIDGETS)
// order is mandatory: include glew first to avoid redefinitions !!! (JW)
// there *must not* be a gl.h inclusion before glew !
// wxglcanvas includes glx.h, 
// thus we must include glxew first to avoid conflict in glew case
//#  if defined(BIAS_HAVE_GLEW) || defined(MIP_HAVE_GLEW) 
//#    if !defined(WIN32)
//#      include <GL/glxew.h>
//#    endif // GLEW 
//#  endif // Linux
//#  include <wx/glcanvas.h> 
//#endif 


#if defined (BIAS_HAVE_GLUT) || defined(MIP_HAVE_GLUT)
# ifdef WIN32
// avoid some entry calls /4505 warnings with this define on WIN32: 
#  define GLUT_BUILDING_LIB
#  include <GL/glut.h> 
# else // WIN32
#  ifdef _APPLE__
#    include <GLUT/glut.h>
#  else // __APPLE__
#    include <GL/glut.h>
#  endif // __APPLE__
# endif // WIN32
#endif // BIAS_HAVE_GLUT


// macros, e.g. CHECK_GL_ERROR
#include <Gui/glcheck.h>


// --- define special token extension values for "hidden" features (jw) ---
//#ifndef GL_CRYCBY_422_NVX
//#  define GL_CRYCBY_422_NVX   0x600A
//#endif 
//#ifndef GL_YCRYCB_422_NVX   
//#  define GL_YCRYCB_422_NVX   0x600B 
//#endif

//#if defined WIN32 && !defined(EXT_timer_query)
// -- Nvidia OpenGL rendering timing 
// -- without stalling the render pipeline (in contrast to glFinish)
//#  ifndef GLint64EXT
//    typedef          __int64 GLint64EXT; 
//#  endif // GLint64EXT
//#  ifndef GLuint64EXT
//    typedef unsigned __int64 GLuint64EXT; 
//#  endif // GLuint64EXT
//#  ifndef GL_TIME_ELAPSED_EXT
//#    define GL_TIME_ELAPSED_EXT 0x88BF
//#  endif // GL_TIME_ELAPSED_EXT
//  void glGetQueryObjecti64vEXT (GLuint id, GLenum pname, GLint64EXT  *params);
//  void glGetQueryObjectui64vEXT(GLuint id, GLenum pname, GLuint64EXT *params);
//#endif // WIN32 & !EXT_timer_query


#endif // _biasgl_h_ 
// DO NOT ADD CODE AFTER THIS LINE

