/*
 * wxMatrixPanel.cpp
 *
 *  Created on: Jun 25, 2010
 *      Author: ischiller
 */

#include "wxMatrixPanel.hh"
#include <Gui/StringConv.hh>
#include <sstream>

using namespace std;

namespace BIAS {

/*
===============================================================================
wxMatrixPanel
===============================================================================
 */

wxMatrixPanel::wxMatrixPanel(wxWindow* parent, wxWindowID id, bool editable)
: wxPanel(parent, id)
{
	editable_ = editable;
  rows_ = 0;
  cols_ = 0;
}

void wxMatrixPanel::SetMatrix(const Matrix<double>& matrix)
{
  // need to re-create controls?
	if ((int)matrix.GetRows() != rows_ || (int)matrix.GetCols() != cols_)
	{
    controls_.clear();
		DestroyChildren();
    wxSizer* sizer = new wxFlexGridSizer(matrix.GetCols(), matrix.GetRows(), 0, 0);
		SetSizer(sizer);

		rows_ = matrix.GetRows();
		cols_ = matrix.GetCols();

		for (int i = 0; i < rows_; i++)
		{
			for (int j = 0; j < cols_; j++)
			{
				wxTextCtrl* control = new wxTextCtrl(this, GetId());
				control->SetEditable(editable_);
				sizer->Add(control);
				controls_.push_back(control);
			}
		}
	}

	// fill controls
  int index = 0;
	for (int i = 0; i < rows_; i++)
	{
    for (int j = 0; j < cols_; j++, index++)
		{
      //if (index < (int)controls_.size() && FindFocus() == controls_[index])
      //{
			stringstream str;
      str << matrix[i][j];
      controls_[index]->SetValue(AsciiToWx(str.str()));
      //}
		}
	}
}

Matrix<double> wxMatrixPanel::GetMatrix()
{
	Matrix<double> matrix(rows_, cols_);
  int index = 0;
	for (int i = 0; i < rows_; i++)
	{
    for (int j = 0; j < cols_; j++, index++)
		{
      stringstream str(WxToAscii(controls_[index]->GetValue()));
			str >> matrix[i][j];
		}
	}
	return matrix;
}

}
