/* This file is part of the BIAS library (Basic ImageAlgorithmS).
   
   Copyright (C) 2003-2009    (see file CONTACT for details)
   Multimediale Systeme der Informationsverarbeitung
   Institut fuer Informatik
   Christian-Albrechts-Universitaet Kiel
   
   BIAS is free software; you can redistribute it and/or modify
   it under the terms of the GNU Lesser General Public License as published by
   the Free Software Foundation; either version 2.1 of the License, or
   (at your option) any later version.
   
   BIAS is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU Lesser General Public License for more details.
   
   You should have received a copy of the GNU Lesser General Public License
   along with BIAS; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#ifndef __ProjectionGUI_hh__
#define __ProjectionGUI_hh__

#include <bias_config.h>

#ifndef BIAS_HAVE_WXWIDGETS
#  error BIAS_HAVE_WXWIDGETS not defined but required by ProjectionGUI. Please enable BIAS_HAVE_WXWIDGETS in CMake configure step.
#endif

#include <wx/wx.h>
#include <wx/notebook.h>
#include <Geometry/Projection.hh>
#include <vector>
#include <Gui/wxVectorPanel.hh>
#include <Gui/wxMatrixPanel.hh>

#include <Base/Math/Vector.hh>
#include <Base/Math/Vector2.hh>

namespace BIAS {


  /** @brief Panel for displaying and editing projections.
      @author jkollmann */
  class BIASGui_EXPORT wxProjectionPanel : public wxPanel
  {
  public:
  	wxProjectionPanel(wxWindow* parent, wxWindowID id = -1,
                  const wxPoint& pos = wxDefaultPosition,
                  const wxSize& size = wxDefaultSize,
                  long style = wxTAB_TRAVERSAL,
                  const wxString& name = wxT("panel"),
                  bool createButtons = true);
    void SetProjection(const Projection& projection);
    const Projection& GetProjection() const;

  private:
    wxVectorPanel* AddVectorPanel(wxWindow* parent,const wxString& name, wxWindowID id, bool editable);
    wxMatrixPanel* AddMatrixPanel(wxWindow* parent,const wxString& name, wxWindowID id, bool editable);

    void UpdateGUI(unsigned int cameraNr);

    void OnNormalizeQuaternion(wxCommandEvent& event);
    void OnModified(wxCommandEvent& event);
    void OnNew(wxCommandEvent& event);
    void OnLoad(wxCommandEvent& event);
    void OnSave(wxCommandEvent& event);

    bool selfModifying_;
    Projection projection_;
    wxScrolledWindow* projectionPanel_;
    wxNotebook*       noteBook_;

    std::vector<wxVectorPanel* > panelCenter_;
    std::vector<wxVectorPanel* > panelQuaternion_;
    std::vector<wxMatrixPanel* > panelRMatrix_;
    std::vector<wxVectorPanel* > panelImageSize_;
    std::vector<wxVectorPanel* > panelAspectRatio_;
    std::vector<wxVectorPanel* > panelPrincipal_;
    std::vector<wxMatrixPanel* > panelKMatrix_;
    std::vector<wxVectorPanel* > panelFocalLength_;
    std::vector<wxVectorPanel* > panelUndistortion_;
    std::vector<wxVectorPanel* > panelRadius_;
    std::vector<wxScrolledWindow* > notebookPages_;
    DECLARE_EVENT_TABLE()
  };
} // namespace BIAS

#endif // __ProjectionGUI_hh__
