#include "wxTauControlFrame.h"
#ifdef WIN32
#include <windows.h>
#endif
#include <Base/Common/W32Compat.hh>
#include <Gui/StringConv.hh>
#include <wx/msgdlg.h>
#include <iostream>
#include <sstream>

using namespace BIAS;
using namespace std;

wxTauControlFrame::wxTauControlFrame( wxWindow* parent )
:
wxTauControlFrameInterface( parent )
{
  m_notebook1->Enable(false);
  wxString text= wxT("Not Connected!");
  SetStatusText(text, 0);
#ifdef WIN32
  m_textCtrlDevice->SetValue(AsciiToWx("COM0"));
#endif
}

void wxTauControlFrame::OnSetFFC( wxCommandEvent& event )
{
	if(!serialFlirControl_.IsConnected()) return;
  //0 = manual, 1= auto, 2= external
	int sel = m_radioBoxFFC->GetSelection();
  int ffcInterval = atoi(m_textFFCInterval->GetValue().ToAscii());
  int ffcTmpChange = atoi(m_textCtrlTempChange->GetValue().ToAscii());
  int ffcLGInterval = atoi(m_textCtrlLGFFCInt->GetValue().ToAscii());
  int ffcLGTmpChange = atoi(m_textCtrlLGTC->GetValue().ToAscii());
  //mode
	FLIR_STATUS_MESSAGE ret = serialFlirControl_.SetFFCMode(sel);
	if(ret != CAM_OK){
		biasusleep(500);
		ret = serialFlirControl_.SetFFCMode(sel);
		if(ret != CAM_OK){
			wxString text= wxT("Failed, try again!");
			SetStatusText(text, 0);
		}
	}
  //interval
  ret = serialFlirControl_.SetFFCInterval(ffcInterval,ffcLGInterval);
	if(ret != CAM_OK){
		biasusleep(500);
		ret = serialFlirControl_.SetFFCInterval(ffcInterval,ffcLGInterval);
		if(ret != CAM_OK){
			wxString text= wxT("Failed, try again!");
			SetStatusText(text, 0);
		}
	}
  //temp change
  ret = serialFlirControl_.SetFFCTempChange(ffcTmpChange,ffcLGTmpChange);
	if(ret != CAM_OK){
		biasusleep(500);
		ret = serialFlirControl_.SetFFCTempChange(ffcTmpChange,ffcLGTmpChange);
		if(ret != CAM_OK){
			wxString text= wxT("Failed, try again!");
			SetStatusText(text, 0);
		}
	}
}

void wxTauControlFrame::OnOperationMode( wxCommandEvent& event )
{
	if(!serialFlirControl_.IsConnected()) return;
	int sel = event.GetInt();
	if(sel == 0) sel = 0; //real time
	if(sel == 1) sel = 1; //frozen
	else sel = 0; //real time
	FLIR_STATUS_MESSAGE ret = serialFlirControl_.SetVideoMode(sel);
	if(ret != CAM_OK){
		biasusleep(500);
		ret = serialFlirControl_.SetVideoMode(sel);
		if(ret != CAM_OK){
			wxString text= wxT("Failed, try again!");
			SetStatusText(text, 0);
		}
	}
}

void wxTauControlFrame::OnTestPattern( wxCommandEvent& event )
{
	// TODO: Implement OnTestPattern
}

void wxTauControlFrame::OnExternalSync( wxCommandEvent& event )
{
	if(!serialFlirControl_.IsConnected()) return;
	int sel = event.GetInt();
  //Disable=0, slave=1, master=2
	FLIR_STATUS_MESSAGE ret = serialFlirControl_.SetExternalSync(sel);
	if(ret != CAM_OK){
		biasusleep(500);
		ret = serialFlirControl_.SetExternalSync(sel);
	}
   if(ret == CAM_OK){
		wxString text= wxT("Set external sync mode.");
		SetStatusText(text, 0);
	}
	else{
		stringstream txt; txt<<"Could not set external sync:";
		txt<<serialFlirControl_.StatusCodeToString(ret);
		wxString text= AsciiToWx(txt.str());
		SetStatusText(text, 0);
	}
}

void wxTauControlFrame::OnGainMode( wxCommandEvent& event )
{
	if(!serialFlirControl_.IsConnected()) return;
	int sel = event.GetInt();
  //Auto=0, Low=1, High=2, Manual=3
	FLIR_STATUS_MESSAGE ret = serialFlirControl_.SetGainMode(sel);
	if(ret != CAM_OK){
		biasusleep(500);
		ret = serialFlirControl_.SetGainMode(sel);
	}
  if(ret == CAM_OK){
		wxString text= wxT("Set gain mode.");
		SetStatusText(text, 0);
	}
	else{
		stringstream txt; txt<<"Could not set gain mode:";
		txt<<serialFlirControl_.StatusCodeToString(ret);
		wxString text= AsciiToWx(txt.str());
		SetStatusText(text, 0);
	}
}

void wxTauControlFrame::OnReset( wxCommandEvent& event )
{
	if(!serialFlirControl_.IsConnected()) return;
	FLIR_STATUS_MESSAGE ret = serialFlirControl_.SetDefaults();
	if(ret != CAM_OK){
		biasusleep(500);
		ret = serialFlirControl_.SetDefaults();
	}
	if(ret == CAM_OK){
		wxString text= wxT("Reset Camera.");
		SetStatusText(text, 0);
	}
	else{
		stringstream txt; txt<<"Could not reset camera:";
		txt<<serialFlirControl_.StatusCodeToString(ret);
		wxString text= AsciiToWx(txt.str());
		SetStatusText(text, 0);
	}
}

void wxTauControlFrame::OnOrientation( wxCommandEvent& event )
{
	if(!serialFlirControl_.IsConnected()) return;
	int sel = event.GetInt();
	cout<<"Orientation selection:"<<sel<<endl;
	FLIR_STATUS_MESSAGE ret = serialFlirControl_.SetVideoOrientation(sel);
	if(ret != CAM_OK){
		biasusleep(500);
		ret = serialFlirControl_.SetVideoOrientation(sel);
	}
	if(ret == CAM_OK){
		wxString text= wxT("Set Orientation.");
		SetStatusText(text, 0);
	}
	else{
		stringstream txt; txt<<"Could not set orientation:";
		txt<<serialFlirControl_.StatusCodeToString(ret);
		wxString text= AsciiToWx(txt.str());
		SetStatusText(text, 0);
	}
}

void wxTauControlFrame::OnPolarity( wxCommandEvent& event )
{
	if(!serialFlirControl_.IsConnected()) return;
	FLIR_STATUS_MESSAGE ret = serialFlirControl_.SetColoring(m_choicePolarity->GetSelection());
	if(ret != CAM_OK){
		biasusleep(500);
		ret = serialFlirControl_.SetColoring(m_choicePolarity->GetSelection());
	}
	if(ret == CAM_OK){
		wxString text= wxT("Set Polarity.");
		SetStatusText(text, 0);
	}
	else{
		stringstream txt; txt<<"Could not set polarity:";
		txt<<serialFlirControl_.StatusCodeToString(ret);
		wxString text= AsciiToWx(txt.str());
		SetStatusText(text, 0);
	}
}

void wxTauControlFrame::OnZoom( wxCommandEvent& event )
{
	if(!serialFlirControl_.IsConnected()) return;
	int sel = event.GetInt();
	if(sel == 0) sel = 0; //no zoom
	if(sel == 1) sel = 4; //2x zoom
	if(sel == 2) sel = 8; //4x zoom
	FLIR_STATUS_MESSAGE ret = serialFlirControl_.SetVideoMode(sel);
	if(ret != CAM_OK){
		biasusleep(500);
		ret = serialFlirControl_.SetVideoMode(sel);
	}
	if(ret == CAM_OK){
		wxString text= wxT("Set Zoom.");
		SetStatusText(text, 0);
	}
	else{
		stringstream txt; txt<<"Could not set zoom:";
		txt<<serialFlirControl_.StatusCodeToString(ret);
		wxString text= AsciiToWx(txt.str());
		SetStatusText(text, 0);

	}
}

void wxTauControlFrame::OnFFCWarning( wxCommandEvent& event )
{
	// TODO: Implement OnFFCWarning
}

void wxTauControlFrame::OnScrollDDE( wxScrollEvent& event )
{
	// TODO: Implement OnScrollDDE
}

void wxTauControlFrame::OnVideoStandard( wxCommandEvent& event )
{
	if(!serialFlirControl_.IsConnected()) return;
	int sel = event.GetInt();

	FLIR_STATUS_MESSAGE ret = serialFlirControl_.SetVideoStandard(sel);
	if(ret != CAM_OK){
		biasusleep(500);
		ret = serialFlirControl_.SetVideoStandard(sel);
	}
	if(ret == CAM_OK){
		wxString text= wxT("Set Video Standard.");
		SetStatusText(text, 0);
	}
	else{
		stringstream txt; txt<<"Could not set video standard:";
		txt<<serialFlirControl_.StatusCodeToString(ret);
		wxString text= AsciiToWx(txt.str());
		SetStatusText(text, 0);
	}
}

void wxTauControlFrame::OnAlgorithm( wxCommandEvent& event )
{
if(!serialFlirControl_.IsConnected()) return;
	int sel = event.GetInt();

	FLIR_STATUS_MESSAGE ret = serialFlirControl_.SetAGCAlgorithm(sel);
	if(ret != CAM_OK){
		biasusleep(500);
		ret = serialFlirControl_.SetAGCAlgorithm(sel);
	}
	if(ret == CAM_OK){
		wxString text= wxT("Set AGC Algorithm.");
		SetStatusText(text, 0);
	}
	else{
    stringstream txt; txt<<"Could not set AGC Algorithm :";
		txt<<serialFlirControl_.StatusCodeToString(ret);
		wxString text= AsciiToWx(txt.str());
		SetStatusText(text, 0);
	}
}

void wxTauControlFrame::OnPlateauValue( wxCommandEvent& event )
{
	// TODO: Implement OnPlateauValue
}

void wxTauControlFrame::OnMaxGain( wxCommandEvent& event )
{
	// TODO: Implement OnMaxGain
}

void wxTauControlFrame::OnContrast( wxScrollEvent& event )
{
	// TODO: Implement OnContrast
}

void wxTauControlFrame::OnBrightness( wxScrollEvent& event )
{
	// TODO: Implement OnBrightness
}

void wxTauControlFrame::OnBrightnessBias( wxScrollEvent& event )
{
	// TODO: Implement OnBrightnessBias
}

void  wxTauControlFrame::
OnSetSpotMeterMode( wxCommandEvent& event ){
	if(!serialFlirControl_.IsConnected()) return;
	int sel = event.GetInt();
	FLIR_STATUS_MESSAGE ret = serialFlirControl_.SetSpotMeterMode(sel);
	if(ret != CAM_OK){
		biasusleep(500);
		ret = serialFlirControl_.SetSpotMeterMode(sel);
	}
	if(ret == CAM_OK){
		wxString text= wxT("Set Spot Meter Mode.");
		SetStatusText(text, 0);
	}
	else{
		stringstream txt; txt<<"Could not set Spot Meter Mode:";
		txt<<serialFlirControl_.StatusCodeToString(ret);
		wxString text= AsciiToWx(txt.str());
		SetStatusText(text, 0);
	}
}

void  wxTauControlFrame::
OnGetSpotMeterValue( wxCommandEvent& event ){
	if(!serialFlirControl_.IsConnected()) return;
	double deg=0;
	FLIR_STATUS_MESSAGE ret = serialFlirControl_.GetSpotMeterCelcius(deg);
	if(ret != CAM_OK){
		biasusleep(500);
		ret = serialFlirControl_.GetSpotMeterCelcius(deg);
	}
	if(ret == CAM_OK){
		stringstream d; d<< deg;
		m_textCtrSpotMeterValue->SetValue(AsciiToWx(d.str()));
		wxString text= wxT("Got Spot Meter Mode Value.");
		SetStatusText(text, 0);
	}
	else{
		stringstream txt; txt<<"Could not get Spot Meter Mode:";
		txt<<serialFlirControl_.StatusCodeToString(ret);
		wxString text= AsciiToWx(txt.str());
		SetStatusText(text, 0);
	}
}

void wxTauControlFrame::
OnSpotMeterDisplay( wxCommandEvent& event ){
	if(!serialFlirControl_.IsConnected()) return;
	int sel = event.GetInt();
	FLIR_STATUS_MESSAGE ret = serialFlirControl_.SetSpotMeterDisplay(sel);
	if(ret != CAM_OK){
		biasusleep(500);
		ret = serialFlirControl_.SetSpotMeterDisplay(sel);
	}
	if(ret == CAM_OK){
		wxString text= wxT("Set Spot Meter Mode.");
		SetStatusText(text, 0);
	}
	else{
		stringstream txt; txt<<"Could not set display:";
		txt<<serialFlirControl_.StatusCodeToString(ret);
		wxString text= AsciiToWx(txt.str());
		SetStatusText(text, 0);
	}
}

void wxTauControlFrame::OnConnect( wxCommandEvent& event )
{
	wxString dev = m_textCtrlDevice->GetValue();

	FLIR_STATUS_MESSAGE ret = CAM_OK;
        //bool tryToConnect=false;
	if(serialFlirControl_.IsConnected()){
                //tryToConnect = false;
		ret = serialFlirControl_.DisconnectCamera();
	}
	else{
                //tryToConnect = true;
		ret = serialFlirControl_.ConnectCamera(WxToAscii(dev));
	}

	if(ret!=CAM_OK){
		wxMessageDialog messageDialog(this,
				wxT("Sorry, could not communicate with Flir thermal camera!"),
				wxT("ShowCamWx - FLIR "), wxICON_ERROR | wxOK);
		messageDialog.ShowModal();
	}

	if(serialFlirControl_.IsConnected()){
		m_buttonConnect->SetLabel(AsciiToWx("Disconnect"));
    m_notebook1->Enable(true);
    string serial; serialFlirControl_.GetSerialNumber(serial);
    stringstream txt; txt<<"Connected! Camera Serial Number:Sensor Serial Number  ";
    txt<<serial;
    wxString text= AsciiToWx(txt.str());
    SetStatusText(text, 0);
	}
	else{
		m_buttonConnect->SetLabel(AsciiToWx("Connect"));
	  m_notebook1->Enable(false);
    wxString text= wxT("Disconnected!");
    SetStatusText(text, 0);
  }
	
}

void wxTauControlFrame::OnClose( wxCommandEvent& event )
{
	if(serialFlirControl_.IsConnected())
		serialFlirControl_.DisconnectCamera();
	Show(false);
}
