/* This file is part of the BIAS library (Basic ImageAlgorithmS).
   
   Copyright (C) 2003-2009    (see file CONTACT for details)
   Multimediale Systeme der Informationsverarbeitung
   Institut fuer Informatik
   Christian-Albrechts-Universitaet Kiel
   
   BIAS is free software; you can redistribute it and/or modify
   it under the terms of the GNU Lesser General Public License as published by
   the Free Software Foundation; either version 2.1 of the License, or
   (at your option) any later version.
   
   BIAS is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU Lesser General Public License for more details.
   
   You should have received a copy of the GNU Lesser General Public License
   along with BIAS; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

/*
 * wxVectorPanel.cpp
 *
 *  Created on: Jun 25, 2010
 *      Author: ischiller
 */

#include "wxVectorPanel.hh"
#include <Gui/StringConv.hh>
#include <sstream>

#ifdef WIN32
#  pragma warning (disable: 4005) //M_PI redefinition
#endif

using namespace std;

namespace BIAS {

/*
===============================================================================
wxVectorPanel
===============================================================================
*/

wxVectorPanel::wxVectorPanel(wxWindow* parent, wxWindowID id, bool editable)
  : wxPanel(parent, id)
{
  editable_ = editable;
}

void wxVectorPanel::SetVector(const Vector<double>& vector)
{
  // need to re-create controls?
  if (vector.Size() != controls_.size())
  {
    controls_.clear();
    DestroyChildren();
    wxSizer* sizer = new wxFlexGridSizer(vector.Size(), 1, 0, 0);
    SetSizer(sizer);

    for (int i = 0; i < (int)vector.Size(); i++)
    {
      wxTextCtrl* control = new wxTextCtrl(this, GetId(), _T(""),
                                           wxDefaultPosition, wxDefaultSize,
                                           wxTE_PROCESS_ENTER);
      control->SetEditable(editable_);
      sizer->Add(control);
      controls_.push_back(control);
    }
  }

  // fill controls
  for (int i = 0; i < (int)vector.Size(); i++)
  {
    //instead of using this blocking technique when focused
    //use TEXT_ENTER event which occures when enter is pressed
    //within the TextCtr.
    //if (FindFocus() == controls_[i]) continue;

    stringstream str;
    str << vector[i];    
    controls_[i]->SetValue(AsciiToWx(str.str()));
  }
}

Vector<double> wxVectorPanel::GetVector() const
{
  Vector<double> v(controls_.size());
  for (int i = 0; i < (int)controls_.size(); i++)
  {
    stringstream str(WxToAscii(controls_[i]->GetValue()));
    str >> v[i];
  }
  return v;
}


}
