#include "wxVideoSettingsDialog.h"
#include <wx/filedlg.h>
#include <Gui/StringConv.hh>

#ifdef BIAS_HAVE_FFMPEG
  #ifdef __cplusplus
  extern "C" {
  #endif
  #include <libavcodec/avcodec.h>
  #ifdef __cplusplus
  }
  #endif
#endif

wxVideoSettingsDialog::wxVideoSettingsDialog( wxWindow* parent )
:
wxVideoSettingsDialogBase( parent )
{
  //default settings as in VideoSink_FFmpeg
   mFilename_    = wxT("");
   mWidth_       = 352;
   mHeight_      = 288;
   mFps_         = 25;
   mBitrate_     = 1000000;
   mGoPSize_     = 12;
   mMaxBFrames_  = 0;

   #ifdef BIAS_HAVE_FFMPEG
   mCodecID_     = CODEC_ID_MPEG1VIDEO;
   #endif
}

void wxVideoSettingsDialog::OnSelectFile( wxCommandEvent& event )
{
  wxFileDialog dialog(this,wxT("Select File"),wxT(""),wxT(""),
      wxT("Movie files(*.avi;*.mpeg)|*.avi;*.mpeg|All files (*.*)|*.*"),
      wxFD_SAVE|wxFD_OVERWRITE_PROMPT);
  int ret = dialog.ShowModal();
  if(ret == wxID_OK){    
    SetOutputFilename(dialog.GetPath());
  }
}

void wxVideoSettingsDialog::OnOk( wxCommandEvent& event ){
  mFilename_ = m_textOutFilenames->GetValue();
  wxString value =m_textGoPSize->GetValue();
  mGoPSize_  = atoi(WxToAscii(value));

  value = m_textWidth->GetValue();
  mWidth_ = atoi(WxToAscii(value));
  value = m_textHeight->GetValue();
  mHeight_ = atoi(WxToAscii(value));
  value = m_textBitrate->GetValue();
  mBitrate_= atoi(WxToAscii(value));
  value = m_textFPS->GetValue();
  mFps_= atoi(WxToAscii(value));
  value = m_textMaxBFrames->GetValue();
  mMaxBFrames_= atoi(WxToAscii(value));

  int id  = m_choiceCodec->GetSelection();
  mCodecID_ = SelectedCodecToFFMPEGCodecID_(id);
  event.Skip();
}

int wxVideoSettingsDialog::GetGoPSize(){
  return mGoPSize_;
}

int wxVideoSettingsDialog::GetFPS(){
  return mFps_;
}

int wxVideoSettingsDialog::GetBitrate(){
  return mBitrate_;
}

int wxVideoSettingsDialog::GetWidth(){
  return mWidth_;
}

int wxVideoSettingsDialog::GetHeight(){
  return mHeight_;
}

int wxVideoSettingsDialog::GetMaxBFrames(){
  return mMaxBFrames_;
}

wxString wxVideoSettingsDialog::GetOutputFilename(){
  return mFilename_;
}

int wxVideoSettingsDialog::GetCodecID(){
  return mCodecID_;
}

/////////////////////////////////////////////////
void wxVideoSettingsDialog::SetGoPSize(int value){
  mGoPSize_ = value;
  wxString v; v.Printf(wxT("%d"),value);
  m_textGoPSize->SetValue(v);
}

void wxVideoSettingsDialog::SetFPS(int value){
  mFps_ = value;
  wxString v; v.Printf(wxT("%d"),value);
  m_textFPS->SetValue(v);
}

void wxVideoSettingsDialog::SetBitrate(int value){
  mBitrate_ = value;
  wxString v; v.Printf(wxT("%d"),value);
  m_textBitrate->SetValue(v);
}

void wxVideoSettingsDialog::SetWidth(int value){
  mWidth_ = value;
  wxString v; v.Printf(wxT("%d"),value);
  m_textWidth->SetValue(v);
}

void wxVideoSettingsDialog::SetHeight(int value){
  mHeight_ = value;
  wxString v; v.Printf(wxT("%d"),value);
  m_textHeight->SetValue(v);
}

void wxVideoSettingsDialog::SetMaxBFrames(int value){
  mMaxBFrames_ = value;
  wxString v; v.Printf(wxT("%d"),value);
  m_textMaxBFrames->SetValue(v);
}

void wxVideoSettingsDialog::SetOutputFilename(wxString value){
  mFilename_ = value;
  m_textOutFilenames->SetValue(value);
}
///////////////////////////////////////////////////////////
int wxVideoSettingsDialog::SelectedCodecToFFMPEGCodecID_(int id){
#ifdef BIAS_HAVE_FFMPEG
  switch(id)
  {
  case 0: return CODEC_ID_MPEG1VIDEO;
  case 1: return CODEC_ID_MPEG2VIDEO;
  case 2: return CODEC_ID_MPEG4;
  case 3: return CODEC_ID_FFV1;
  case 4: return CODEC_ID_H261;
  case 5: return CODEC_ID_H263;
  case 6: return CODEC_ID_H264;
  case 7: return CODEC_ID_MJPEG;
  case 8: return CODEC_ID_RAWVIDEO;
  case 9: return CODEC_ID_MSMPEG4V1;
  case 10: return CODEC_ID_MSMPEG4V2;
  case 11: return CODEC_ID_MSMPEG4V3;
  case 12: return CODEC_ID_WMV1;
  case 13: return CODEC_ID_WMV2;
  case 14: return CODEC_ID_THEORA;
  case 15: return CODEC_ID_TRUEMOTION1;
  }
#endif
  return -1;
}
