#include "wxViscaControlPanel.h"
#include <Gui/StringConv.hh>
#include <wx/msgdlg.h>

using namespace std;
using namespace BIAS;

wxViscaControlPanel::wxViscaControlPanel( wxWindow* parent )
:
wxViscaControlPanelInterface( parent )
{ 
  #ifdef WIN32
  m_textCtrlDevice->SetValue(AsciiToWx("COM0"));
  #endif
  m_scrolledWindow1->Enable(false);
}

void wxViscaControlPanel::OnScrollZoom( wxScrollEvent& event )
{
  int step = viscaControl_.GetMaxZoom()/m_sliderZoom->GetMax();
  cout<<"Setting zoom step:"<<step<<endl;
  viscaControl_.ZoomDirect(short(m_sliderZoom->GetValue()*step));
}

void wxViscaControlPanel::OnScrollPan( wxScrollEvent& event )
{
  if(viscaControl_.IsConnected())
    viscaControl_.MoveAbsolute((short)m_sliderPan->GetValue(),(short)m_sliderTilt->GetValue());
}

void wxViscaControlPanel::OnScrollTilt( wxScrollEvent& event )
{
  if(viscaControl_.IsConnected())
    viscaControl_.MoveAbsolute((short)m_sliderPan->GetValue(),(short)m_sliderTilt->GetValue());
}

void wxViscaControlPanel::OnButtonReset( wxCommandEvent& event )
{
  if(viscaControl_.IsConnected()) viscaControl_.ResetPanTilt();
}

void wxViscaControlPanel::OnButtonTele( wxCommandEvent& event )
{
  if(viscaControl_.IsConnected()) viscaControl_.ZoomTele();
}
void wxViscaControlPanel::OnButtonWide( wxCommandEvent& event )
{
  if(viscaControl_.IsConnected()) viscaControl_.ZoomWide();
}
void  wxViscaControlPanel::OnButtonStop( wxCommandEvent& event )
{
  if(viscaControl_.IsConnected()) viscaControl_.ZoomStop();
}


void wxViscaControlPanel::OnConnect( wxCommandEvent& event )
{
  bool ret = false;

  if(!viscaControl_.IsConnected()){
    wxString dev = m_textCtrlDevice->GetValue();
    ret = viscaControl_.InitPanTiltUnit(WxToAscii(dev));
    if (!ret) {
      wxMessageDialog messageDialog(this,
          wxT("Initialising Visca pan-tilt unit failed!"),
          wxT("Visca Controller"), wxICON_ERROR | wxOK);
      messageDialog.ShowModal();
      return;
    }
  }
  else{
    ret = viscaControl_.ClosePanTiltUnit();
    if (!ret) {
      wxMessageDialog messageDialog(this,
          wxT("Closing Visca pan-tilt unit failed!"),
          wxT("Visca Controller"), wxICON_ERROR | wxOK);
      messageDialog.ShowModal();
      return;
    }
  }

  if(viscaControl_.IsConnected()){
    m_buttonConnect->SetLabel(AsciiToWx("Disconnect"));
    m_scrolledWindow1->Enable(true);
    wxString text= wxT("Connected!");
  }
  else{
    m_buttonConnect->SetLabel(AsciiToWx("Connect"));
    m_scrolledWindow1->Enable(false);
    wxString text= wxT("Disonnected!");
  }
}

void wxViscaControlPanel::OnClose( wxCommandEvent& event )
{
  Show(false);
}

void wxViscaControlPanel::OnCloseCam( wxCommandEvent& event ){
  if(viscaControl_.IsConnected()){
    viscaControl_.CamPower(false);
  }
}

void wxViscaControlPanel::OnOpenCam( wxCommandEvent& event ){
  if(viscaControl_.IsConnected()){
    viscaControl_.CamPower(true);
  }
}

void wxViscaControlPanel::OnProgressive( wxCommandEvent& event ){
  if(viscaControl_.IsConnected()){
    int sel = event.GetInt();
    if(sel == 0){//interlaced
      viscaControl_.SetCCDScanningMode(ViscaControl::VISCA_INTERLACED);
    }
    else if(sel == 1){ //progressive
      viscaControl_.SetCCDScanningMode(ViscaControl::VISCA_PROGRESSIVE);
    }

    ViscaControl::VISCA_CONSTANT ret = viscaControl_.GetCCDScanningMode();
    cout<<"CCD Scanning:"<<ret<<endl;

  }
}
