#include "wxViscaControlPanelButtons.h"
#include <Gui/StringConv.hh>
#include <Base/Common/W32Compat.hh>
#include <wx/msgdlg.h>
#include <wx/artprov.h>
#include <img/Up.xpm>
#include <img/UpLeft.xpm>
#include <img/UpRight.xpm>
#include <img/Down.xpm>
#include <img/DownLeft.xpm>
#include <img/DownRight.xpm>
#include <img/Left.xpm>
#include <img/Right.xpm>


using namespace BIAS;
using namespace std;

wxViscaControlPanelButtons::wxViscaControlPanelButtons( wxWindow* parent )
:
wxViscaControlPanelButtonsInterface( parent )
{
  #ifdef WIN32
  m_textCtrlDevice->SetValue(AsciiToWx("COM0"));
  m_textCtrlDevice->SetEditable(true);        
  #endif
  m_scrolledWindow1->Enable(false);		

  m_bpButtonLeft->SetBitmapLabel(wxBitmap(Left_xpm));
  m_bpButtonRight->SetBitmapLabel(wxBitmap(Right_xpm));
  m_bpButtonUp->SetBitmapLabel(wxBitmap(Up_xpm));
  m_bpButtonUpLeft->SetBitmapLabel(wxBitmap(UpLeft_xpm));
  m_bpButtonUpRight->SetBitmapLabel(wxBitmap(UpRight_xpm));
  m_bpButtonDown->SetBitmapLabel(wxBitmap(Down_xpm));
  m_bpButtonDownLeft->SetBitmapLabel(wxBitmap(DownLeft_xpm));
  m_bpButtonDownRight->SetBitmapLabel(wxBitmap(DownRight_xpm));

  m_bpButtonStop->SetBitmapLabel(wxBitmap(wxArtProvider::GetIcon(wxART_ERROR,wxART_BUTTON)));
}

void wxViscaControlPanelButtons::OnScrollZoom( wxScrollEvent& event )
{
  int step = viscaControl_.GetMaxZoom()/m_sliderZoom->GetMax();
  cout<<"Setting zoom step:"<<step<<endl;
  viscaControl_.ZoomDirect(short(m_sliderZoom->GetValue()*step));
}

void wxViscaControlPanelButtons::OnButtonWide( wxCommandEvent& event )
{
  if(viscaControl_.IsConnected()) viscaControl_.ZoomWide();
}

void wxViscaControlPanelButtons::OnButtonTele( wxCommandEvent& event )
{
  if(viscaControl_.IsConnected()) viscaControl_.ZoomTele();
}

void wxViscaControlPanelButtons::OnButtonStop( wxCommandEvent& event )
{
  if(viscaControl_.IsConnected()) viscaControl_.ZoomStop();
}

void wxViscaControlPanelButtons::OnProgressive( wxCommandEvent& event )
{
  if(viscaControl_.IsConnected()){
    int sel = event.GetInt();
    if(sel == 0){//interlaced
      viscaControl_.SetCCDScanningMode(ViscaControl::VISCA_INTERLACED);
    }
    else if(sel == 1){ //progressive
      viscaControl_.SetCCDScanningMode(ViscaControl::VISCA_PROGRESSIVE);
    }

    ViscaControl::VISCA_CONSTANT ret = viscaControl_.GetCCDScanningMode();
    cout<<"CCD Scanning:"<<ret<<endl;

  }
}

void wxViscaControlPanelButtons::OnButtonReset( wxCommandEvent& event )
{
  if(viscaControl_.IsConnected())  viscaControl_.ResetPanTilt();
}

void wxViscaControlPanelButtons::OnOpenCam( wxCommandEvent& event )
{
  if(viscaControl_.IsConnected()){
    viscaControl_.CamPower(true);
  }
}

void wxViscaControlPanelButtons::OnCloseCam( wxCommandEvent& event )
{
  if(viscaControl_.IsConnected()){
    viscaControl_.CamPower(false);
  }
}

void wxViscaControlPanelButtons::OnUp( wxCommandEvent& event )
{
  if(viscaControl_.IsConnected()){
    viscaControl_.MoveUp();
    //viscaControl_.PanTiltPosInq();
    stringstream txt; txt<<viscaControl_.GetTilt()<<"";
    m_textTilt->SetLabel(AsciiToWx(txt.str()));
  }
}

void wxViscaControlPanelButtons::OnUpLeft( wxCommandEvent& event )
{
  if(viscaControl_.IsConnected()){
    viscaControl_.MoveUpLeft();
    //viscaControl_.PanTiltPosInq();
    stringstream txt; txt<<viscaControl_.GetTilt()<<"";
    m_textTilt->SetLabel(AsciiToWx(txt.str()));
  }
}

void wxViscaControlPanelButtons::OnUpRight( wxCommandEvent& event )
{
  if(viscaControl_.IsConnected()){
    viscaControl_.MoveUpRight();
    //viscaControl_.PanTiltPosInq();
    stringstream txt; txt<<viscaControl_.GetTilt()<<"";
    m_textTilt->SetLabel(AsciiToWx(txt.str()));
  }
}

void wxViscaControlPanelButtons::OnLeft( wxCommandEvent& event )
{
  if(viscaControl_.IsConnected()){
    viscaControl_.MoveLeft();
    //viscaControl_.PanTiltPosInq();
    stringstream txt; txt<<viscaControl_.GetPan()<<"";
    m_textPan->SetLabel(AsciiToWx(txt.str()));
  }
}

void wxViscaControlPanelButtons::OnRight( wxCommandEvent& event )
{
  if(viscaControl_.IsConnected()){
    viscaControl_.MoveRight();
    //viscaControl_.PanTiltPosInq();
    stringstream txt; txt<<viscaControl_.GetPan()<<"";
    m_textPan->SetLabel(AsciiToWx(txt.str()));
  }
}

void wxViscaControlPanelButtons::OnDown( wxCommandEvent& event )
{
  if(viscaControl_.IsConnected()){
    viscaControl_.MoveDown();
    //viscaControl_.PanTiltPosInq();
    stringstream txt; txt<<viscaControl_.GetTilt()<<"";
    m_textTilt->SetLabel(AsciiToWx(txt.str()));
  }
}
void wxViscaControlPanelButtons::OnDownLeft( wxCommandEvent& event )
{
  if(viscaControl_.IsConnected()){
    viscaControl_.MoveDownLeft();
    //viscaControl_.PanTiltPosInq();
    stringstream txt; txt<<viscaControl_.GetTilt()<<"";
    m_textTilt->SetLabel(AsciiToWx(txt.str()));
  }
}
void wxViscaControlPanelButtons::OnDownRight( wxCommandEvent& event )
{
  if(viscaControl_.IsConnected()){
    viscaControl_.MoveDownRight();
    //viscaControl_.PanTiltPosInq();
    stringstream txt; txt<<viscaControl_.GetTilt()<<"";
    m_textTilt->SetLabel(AsciiToWx(txt.str()));
  }
}

void wxViscaControlPanelButtons::OnStop( wxCommandEvent& event )
{
  if(viscaControl_.IsConnected()){
    viscaControl_.MoveStop();
  }
}

void wxViscaControlPanelButtons::OnButtonHome( wxCommandEvent& event ){
  if(viscaControl_.IsConnected()){
    viscaControl_.MoveStop();
    biassleep(1);
    viscaControl_.MoveHome();
  }
}

void wxViscaControlPanelButtons::OnConnect( wxCommandEvent& event )
{
  bool ret = false;

  if(!viscaControl_.IsConnected()){
    wxString dev = m_textCtrlDevice->GetValue();
    ret = viscaControl_.InitPanTiltUnit(WxToAscii(dev));
    if (!ret) {
      wxMessageDialog messageDialog(this,
          wxT("Initialising Visca pan-tilt unit failed!"),
          wxT("Visca Controller"), wxICON_ERROR | wxOK);
      messageDialog.ShowModal();
      return;
    }   
  }
  else{
    ret = viscaControl_.ClosePanTiltUnit();
    if (!ret) {
      wxMessageDialog messageDialog(this,
          wxT("Closing Visca pan-tilt unit failed!"),
          wxT("Visca Controller"), wxICON_ERROR | wxOK);
      messageDialog.ShowModal();
      return;
    }
  }

  if(viscaControl_.IsConnected()){
    m_buttonConnect->SetLabel(AsciiToWx("Disconnect"));
    m_scrolledWindow1->Enable(true);
    wxString text= wxT("Connected!");
  }
  else{
    m_buttonConnect->SetLabel(AsciiToWx("Connect"));
    m_scrolledWindow1->Enable(false);
    wxString text= wxT("Disonnected!");
  }
}

void wxViscaControlPanelButtons::OnClose( wxCommandEvent& event )
{
   Show(false);
}

bool wxViscaControlPanelButtons::SetColour(const wxColour &colour){

  m_panel2->SetBackgroundColour(colour);
  m_scrolledWindow1->SetBackgroundColour(colour);
  m_buttonReset->SetBackgroundColour(colour);
  m_buttonOpen->SetBackgroundColour(colour);
  m_buttonCloseCam->SetBackgroundColour(colour);
  m_sliderZoom->SetBackgroundColour(colour);
  m_buttonWide->SetBackgroundColour(colour);
  m_buttonTele->SetBackgroundColour(colour);
  m_buttonZoomStop->SetBackgroundColour(colour);
  m_radioBox1->SetBackgroundColour(colour);
  m_staticText03->SetBackgroundColour(colour);
  m_textPan->SetBackgroundColour(colour);
  m_staticText04->SetBackgroundColour(colour);
  m_textTilt->SetBackgroundColour(colour);
  m_bpButtonUp->SetBackgroundColour(colour);
  m_bpButtonUpLeft->SetBackgroundColour(colour);
  m_bpButtonUpRight->SetBackgroundColour(colour);
  m_bpButtonLeft->SetBackgroundColour(colour);
  m_bpButtonStop->SetBackgroundColour(colour);
  m_bpButtonRight->SetBackgroundColour(colour);
  m_bpButtonDown->SetBackgroundColour(colour);
  m_bpButtonDownLeft->SetBackgroundColour(colour);
  m_bpButtonDownRight->SetBackgroundColour(colour);
  m_buttonConnect->SetBackgroundColour(colour);
  m_textCtrlDevice->SetBackgroundColour(colour);
  m_buttonClose->SetBackgroundColour(colour);
  m_buttonHome->SetBackgroundColour(colour);
  m_panelPanTilt->SetBackgroundColour(colour);
  return this->SetBackgroundColour(colour);
}
