/*
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003-2009    (see file CONTACT for details)
Multimediale Systeme der Informationsverarbeitung
Institut fuer Informatik
Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "BackwardMapping.hh"
                         //#include <Base/Image/ImageIO.hh>
                         //#include <Base/Common/FileHandling.hh>



// backward compatibility to gcc < 4.1, all other must have omp.h !
 #ifdef BIAS_HAVE_OPENMP
 #  if  !defined(_OPENMP) && defined(WIN32)
 #    error Please check your OpenMP flags and defines - seem inconsistent.
 #  endif
 #  include <omp.h>
 #endif



using namespace std;

namespace BIAS {

  /** LUT Datastruct Interface **/
  float* BWM_LUT_Entry_Bilinear::
  SerializedOut(float* pointerToHeadOfOutputMem)
  {
    float out;
    if(mapped) out = 1;
    else out = 0;
    *pointerToHeadOfOutputMem = out;
    pointerToHeadOfOutputMem++;
    *pointerToHeadOfOutputMem = xy_weight;
    pointerToHeadOfOutputMem++;
    *pointerToHeadOfOutputMem = static_cast<float>(index_0_0);
    pointerToHeadOfOutputMem++;
    *pointerToHeadOfOutputMem = xY_weight;
    pointerToHeadOfOutputMem++;
    *pointerToHeadOfOutputMem = static_cast<float>(index_0_1);
    pointerToHeadOfOutputMem++;
    *pointerToHeadOfOutputMem = Xy_weight;
    pointerToHeadOfOutputMem++;
    *pointerToHeadOfOutputMem = static_cast<float>(index_1_0);
    pointerToHeadOfOutputMem++;
    *pointerToHeadOfOutputMem = XY_weight;
    pointerToHeadOfOutputMem++;
    *pointerToHeadOfOutputMem = static_cast<float>(index_1_1);
    pointerToHeadOfOutputMem++;
    return pointerToHeadOfOutputMem;
  }

  const float* BWM_LUT_Entry_Bilinear::
  SerializedIn(const float* pointerToHeadOfInputMem)
  {
    float in;
    in = *pointerToHeadOfInputMem;
    if(in==1) mapped = true;
    else mapped = false;
    pointerToHeadOfInputMem++;
    xy_weight = *pointerToHeadOfInputMem;
    pointerToHeadOfInputMem++;
    index_0_0 = static_cast<int>(*pointerToHeadOfInputMem);
    pointerToHeadOfInputMem++;
    xY_weight = *pointerToHeadOfInputMem;
    pointerToHeadOfInputMem++;
    index_0_1 = static_cast<int>(*pointerToHeadOfInputMem);
    pointerToHeadOfInputMem++;
    Xy_weight = *pointerToHeadOfInputMem;
    pointerToHeadOfInputMem++;
    index_1_0 = static_cast<int>(*pointerToHeadOfInputMem);
    pointerToHeadOfInputMem++;
    XY_weight = *pointerToHeadOfInputMem;
    pointerToHeadOfInputMem++;
    index_1_1 = static_cast<int>(*pointerToHeadOfInputMem);
    pointerToHeadOfInputMem++;
    return pointerToHeadOfInputMem;
  }

  int BWM_LUT_Entry_Bilinear::
  GetSerializedSize() {
    return 9;
  }


  float* BWM_LUT_Entry_Trilinear::
  SerializedOut(float* pointerToHeadOfOutputMem)
  {
    float out;
    if(mapped) out = 1;
    else out = 0;
    *pointerToHeadOfOutputMem = out;
    pointerToHeadOfOutputMem++;
    *pointerToHeadOfOutputMem = pyr_diff_inv;
    pointerToHeadOfOutputMem++;
    *pointerToHeadOfOutputMem = static_cast<float>(pyr_level_low);
    pointerToHeadOfOutputMem++;
    *pointerToHeadOfOutputMem = pyr_diff;
    pointerToHeadOfOutputMem++;
    *pointerToHeadOfOutputMem = static_cast<float>(pyr_level_high);
    pointerToHeadOfOutputMem++;
    pointerToHeadOfOutputMem = stage_0.SerializedOut(pointerToHeadOfOutputMem);
    pointerToHeadOfOutputMem = stage_1.SerializedOut(pointerToHeadOfOutputMem);
    return pointerToHeadOfOutputMem;
  }

  const float* BWM_LUT_Entry_Trilinear::
  SerializedIn(const float* pointerToHeadOfInputMem)
  {
    float in;
    in = *pointerToHeadOfInputMem;
    if(in==1) mapped = true;
    else mapped = false;
    pointerToHeadOfInputMem++;
    pyr_diff_inv = *pointerToHeadOfInputMem;
    pointerToHeadOfInputMem++;
    pyr_level_low = static_cast<int>(*pointerToHeadOfInputMem);
    pointerToHeadOfInputMem++;
    pyr_diff = *pointerToHeadOfInputMem;
    pointerToHeadOfInputMem++;
    pyr_level_high = static_cast<int>(*pointerToHeadOfInputMem);
    pointerToHeadOfInputMem++;
    pointerToHeadOfInputMem = stage_0.SerializedIn(pointerToHeadOfInputMem);
    pointerToHeadOfInputMem = stage_1.SerializedIn(pointerToHeadOfInputMem);
    return pointerToHeadOfInputMem;
  }

  int BWM_LUT_Entry_Trilinear::
  GetSerializedSize() {
    return 5+2*BWM_LUT_Entry_Bilinear::GetSerializedSize();
  }

} // namespace BIAS
