/*
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003-2009    (see file CONTACT for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef _ColorSegmentation_hh_
#define _ColorSegmentation_hh_

#include <bias_config.h>
#ifndef BIAS_HAVE_PTHREADS
#  error You need pthreads. Please recompile BIAS wih USE_PTHREADS
#endif // BIAS_HAVE_PTHREADS

#include <Base/Common/W32Compat.hh>
#include <cmath>
#include <Base/Image/Image.hh>

namespace BIAS {

  /** @class ColorSegmentation
      @brief
      @author africk */
  class BIASImage_EXPORT ColorSegmentation : public BIAS::Debug
  {
  public:
    ColorSegmentation();
    void SetColorSpace(BIAS::ImageBase::EColorModel farbraum);
    void Segment(BIAS::Image<unsigned char> &src,
                 BIAS::Image<unsigned char> &res,
                 int hue,int hueIntervall,int sat, int satIntervall,
                 int light=0, int lightIntervall=0);

    /**
       @brief This Method is used for segmenting the values of channel one within
       a given range. Additionally you can set a treshold for channel two. This
       method is usefull for working in HSV or HSL. Other ranges aren't tested.
       @author goerriss
    */
    void SegmentTreshold(BIAS::Image<unsigned char> &src,
                         BIAS::Image<unsigned char> &res, 
                         int min,int max, int satTreshhold);

    void GetHueImage(BIAS::Image<unsigned char> &imgHue);
    void GetSatImage(BIAS::Image<unsigned char> &imgSat);
    void GetGreyImage(BIAS::Image<unsigned char> &imgGrey);

    inline void Segment(const unsigned char **src, unsigned char **dst,
                        const int tl[2], const int br[2],
                        const int hue, const int hueIntervall,
                        const int sat, const int satIntervall,
                        const int light=0, const int lightIntervall=0)
    {
      register int x, y;
      int h, s,l, dh, ds, dl;
      for (y = tl[1]; y < br[1]; y++) {
        for (x = tl[0]; x < br[0]; x++) {
          h = (int)src[y][3*x]; // hue
          s = (int)src[y][3*x+1]; //saturation
          l = (int)src[y][3*x+2]; //light
          dh = std::min ( abs(hue-h), abs(hue-(255+h)));
          ds = abs(s - sat);
          dl = abs(l-light);
          
          //if (dh>=128) d=256-d;
          if (dh<=hueIntervall && ds<=satIntervall){
            if(lightIntervall > 0){
              if(dl <= lightIntervall) dst[y][x] = 255;
              else dst[y][x] = 0;
            }
            else
              dst[y][x] = 255;
          }
          else
            dst[y][x] = 0;
        }
      }
    };

  protected:
    BIAS::Image<unsigned char> image;
    BIAS::ImageBase::EColorModel stdFarbRaum;

  };

  /** @class ColorSegmentationThreaded
      @brief Threaded implementation of ColorSegmentation.
      @author africk */
  class BIASImage_EXPORT ColorSegmentationThreaded
  {
  public:
    void Segment(BIAS::Image<unsigned char> &src,
                 BIAS::Image<unsigned char> &res,
                 int hue,int hueIntervall,int sat, int satIntervall);
  protected:
    static void *Segment_(void *data);
  };

}

#endif
