#include "ConvertBayerPattern.hh"
#include <Filter/Convolution.hh>
#include <Base/Image/ImageConvert.hh>
#include <Base/Image/ImageBase.hh>
#include <Base/Image/ImageIO.hh>


using namespace BIAS;
using namespace std;


template <class StorageType>
ConvertBayerPattern<StorageType>::ConvertBayerPattern(){

}
template <class StorageType>
ConvertBayerPattern<StorageType>::~ConvertBayerPattern(){

}

template <class StorageType>
int ConvertBayerPattern<StorageType>::
Convert(Image<StorageType>& src, Image<StorageType>& dst,
				BayerDemosaicMethod method, ImageBase::EColorModel bayer_pattern)
{

  StorageType* pSrc = src.GetImageData();

  if(src.GetChannelCount() == 3){
    Image<StorageType> input(src.GetWidth(), src.GetHeight(),1);

      StorageType value;
      for(unsigned int y = 0; y < src.GetHeight(); y++){
        for(unsigned int x = 0; x < src.GetWidth() ;x++){
          value = 0;

          value = src.PixelValue (x,y,0) + src.PixelValue (x,y,1) + src.PixelValue (x,y,2);

          input.SetPixel (value,x,y);
        }
      }

      pSrc = input.GetImageData();
  }

  dc1394bayer_method_t dcMethod = SelectMethod_(method);

  if(bayer_pattern == ImageBase::CM_invalid){
    bayer_pattern = src.GetColorModel();
  }

  dc1394color_filter_t coding = SelectFilter_(bayer_pattern);

  dst.SetColorModel(ImageBase::CM_RGB);

  switch(src.GetStorageType()){
    case ImageBase::ST_unsignedchar:
        dc1394_bayer_decoding_8bit((unsigned char*)pSrc,(unsigned char*)dst.GetImageData(),src.GetWidth(),src.GetHeight(),coding,dcMethod);
      break;

    case ImageBase::ST_unsignedshortint:{
    //case ImageBase::ST_unsignedint:
      int bits = src.GetBitDepth();
      dst.SetBitDepth(bits);
      dc1394_bayer_decoding_16bit((unsigned short*)pSrc,(unsigned short*)dst.GetImageData(),src.GetWidth(),src.GetHeight(),coding,dcMethod,bits);
    }
      break;

    default:
      return -1;
      break;
  }

  return 0;
}

template <class StorageType>
dc1394bayer_method_t ConvertBayerPattern<StorageType>::SelectMethod_(BayerDemosaicMethod method){

    if(method == BAYER_DEMOSAIC_METHOD_AHD)
      return DC1394_BAYER_METHOD_AHD;
    else if(method == BAYER_DEMOSAIC_METHOD_BILINEAR)
      return DC1394_BAYER_METHOD_BILINEAR;
    else if(method == BAYER_DEMOSAIC_METHOD_HQLINEAR)
      return DC1394_BAYER_METHOD_HQLINEAR;
    else if(method == BAYER_DEMOSAIC_METHOD_NEAREST)
      return DC1394_BAYER_METHOD_NEAREST;
    else if(method == BAYER_DEMOSAIC_METHOD_SIMPLE)
      return DC1394_BAYER_METHOD_SIMPLE;
    else if(method == BAYER_DEMOSAIC_METHOD_VNG)
      return DC1394_BAYER_METHOD_VNG;

    return DC1394_BAYER_METHOD_AHD;
}

template <class StorageType>
dc1394color_filter_t ConvertBayerPattern<StorageType>::SelectFilter_(ImageBase::EColorModel bayer_pattern){
  if(bayer_pattern == ImageBase::CM_Bayer_BGGR)
      return DC1394_COLOR_FILTER_BGGR;
    else if(bayer_pattern == ImageBase::CM_Bayer_GBRG)
      return DC1394_COLOR_FILTER_GBRG;
    else if(bayer_pattern == ImageBase::CM_Bayer_GRBG)
      return DC1394_COLOR_FILTER_GRBG;
    else if(bayer_pattern == ImageBase::CM_Bayer_RGGB)
      return DC1394_COLOR_FILTER_RGGB;

  return DC1394_COLOR_FILTER_RGGB;
}

#define INST(type) template class ConvertBayerPattern<type>;
INST(unsigned char)

#ifdef BUILD_IMAGE_USHORT
INST(unsigned short)
#endif

