/*
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003-2009    (see file CONTACT for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __ConvertBayerPattern_hh__
#define __ConvertBayerPattern_hh__

#include <bias_config.h>

#include <iostream>

#include <Base/Debug/Debug.hh>
#include <Base/Debug/Error.hh>

#include <Base/Math/Matrix.hh>
#include <Base/Math/Vector.hh>
#include <Base/Image/Image.hh>
#include <Base/Image/ImageConvert.hh>

//#if !defined(WIN32) && !defined(BIAS_HAVE_DC1394)
//#  error You need DC1394 for BIAS ConvertBayerPattern. Please recompile BIAS with USE_DC1394.
//#endif

#include <dc1394/dc1394.h>
#include <dc1394/conversions.h>



namespace BIAS {

  /** @class ConvertBayerPattern
      @brief Wrapper class to dc1394 bayer conversion class

      @author bangerer 03/2009
  */
  template <class StorageType>
class BIASImage_EXPORT ConvertBayerPattern
    : public Debug  {

  public:
    ConvertBayerPattern();
    ~ConvertBayerPattern();

    /**
     * @brief converts image with bayerpattern to rgb. there are several
     * methods for conversion.
     * @param src source image with bayer pattern
     * @param dst destination image in rgb
     * @param method the demosaic method that should be used
     * @param bayer_pattern this parameter is optional. bayer pattern of the source image
     */
    int Convert(Image<StorageType>& src, Image<StorageType>& dst,
							  BayerDemosaicMethod method = BAYER_DEMOSAIC_METHOD_AHD,
							  ImageBase::EColorModel bayer_pattern = ImageBase::CM_invalid);

  private:
    dc1394bayer_method_t SelectMethod_(BayerDemosaicMethod method);

    dc1394color_filter_t SelectFilter_(ImageBase::EColorModel bayer_pattern);
  }; // class

} // namespace

#endif

//#include <Base/Common/BIASpragmaEnd.hh>
