/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2007   (see file CONTACT for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <Base/Common/BIASpragma.hh>

#define _CRT_SECURE_NO_DEPRECATE
#include <stdio.h>

#include <Base/Image/Image.hh>
#include "CorrespondenceMap.hh"


using namespace BIAS;
using namespace std;


template <class StorageType> 
CorrespondenceMap<StorageType>::CorrespondenceMap() : Image<StorageType>()
{
   
}

template <class StorageType>
int CorrespondenceMap<StorageType>::Read(const std::string filename, 
                                         unsigned int width,
                                         unsigned int height) {
  if (!this->IsEmpty()) this->Release();
  this->Init(width, height, 2);
  //this->SetColorModel(ImageBase::CM_Grey);
  ifstream thefile(filename.c_str());
  StorageType** IDA=this->GetImageDataArray();
  cout<<"Loading correspondences from file "<<filename<<" ";
  for (unsigned int y=0; y<height; y++) {
    if (y%50==0) cout<<y*100/width<<"% "<<flush;
    for (unsigned int x=0; x<width; x++) {
      thefile >> (IDA[y][2*x]);
      thefile >> (IDA[y][2*x+1]);
      if (!thefile.good()) return -1;
    }
  }
  cout<<endl;
  return 0;
}


// template instantiation

#define INST(type)\
template class BIASImage_EXPORT CorrespondenceMap<type>
//template ostream& operator<<<type>(ostream & , Image<type> const & );
//template istream& operator>><type>(istream& , Image<type>& );
namespace BIAS{
INST(unsigned char);
INST(float);

#ifdef BUILD_IMAGE_INT
INST(int);
#endif
#ifdef BUILD_IMAGE_CHAR
INST(char);
#endif
#ifdef BUILD_IMAGE_SHORT
INST(short);
#endif
#ifdef BUILD_IMAGE_USHORT
INST(unsigned short);
#endif
#ifdef BUILD_IMAGE_UINT
INST(unsigned int);
#endif
#ifdef BUILD_IMAGE_DOUBLE
INST(double);
#endif
}
