/*
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003-2009    (see file CONTACT for details)
Multimediale Systeme der Informationsverarbeitung
Institut fuer Informatik
Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#ifndef _CYLINDRICDEPTHPANORAMA_HH_
#define _CYLINDRICDEPTHPANORAMA_HH_

#include <Base/Common/W32Compat.hh>
#include <math.h>
#include <vector>
#include <Base/Image/Image.hh>
#include <Geometry/PMatrix.hh>
#include <Geometry/Projection.hh>
#include <Geometry/ProjectionParametersCylindric.hh>
#include <Utils/TriangleMesh.hh>
#include <Image/ProjectionMapping.hh>

namespace BIAS {

  /** \class CylindricDepthPanorama
   * \author ischiller
   * \brief create and represent cylindric depth panorama
   */
  class BIASImage_EXPORT CylindricDepthPanorama {
  public:

    CylindricDepthPanorama();
    /**
        @brief Constructor with default params
        @param range: the range of degrees in which a panorama is to be build
               360 = full circle
        @author ischiller
        @date 05/07
    */
    CylindricDepthPanorama(const Projection &iP,
                           unsigned int channels);
    CylindricDepthPanorama(const ProjectionParametersCylindric& iP,
                           unsigned int channels);
    ~CylindricDepthPanorama();

    int Init(const Projection& iP, 
             unsigned int channels);
    int Init(const ProjectionParametersCylindric& iP,
             unsigned int channels);

    /** Call Process to update the measurement */
    int ProcessCylindricProjection(BIAS::Image<float> &depthMap,
                                   BIAS::Projection & p,
                                   BIAS::Image<unsigned char> *texture=NULL,
                                   bool useProjection = true);

    int GetPanorama(BIAS::Image<float> &image,
                    BIAS::Image<unsigned char> &texture);

    int GetMappedImages(BIAS::Image<float> &image,
                        BIAS::Image<unsigned char> &texture);

    int GetTriangleMesh(BIAS::TriangleMesh &mesh);

    void SetFillArea(int area);
  protected:
    unsigned fillArea_;
    bool bCostImageCreated_;
    BIAS::Image<float> panorama_,panoramaHitCount_,panoramaVarianz_;
    BIAS::Image<unsigned char> panoramaTexture_;

    BIAS::Image<float> destDepth_;
    BIAS::Image<unsigned char> destTex_;

    BIAS::Image<float> costIm_;

    BIAS::ProjectionParametersCylindric*  pCylinderProj_;
    BIAS::Projection initialProjection_;

    unsigned int panoramaWidth_,panoramaHeight_,dTexChannels_;    
    int dOffsetX_,dOffsetY_;

    bool bTextureFilled_,bFirstPoseSet_,inited_;    

    BIAS::ProjectionMapping<unsigned char, unsigned char> projectionMappingUC_;
    BIAS::ProjectionMapping<float, float> projectionMappingFL_;

  };  //end class

}//end namespace


#endif
