/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004    (see file CONTACTS for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __DCDATA_HH__
#define __DCDATA_HH__

#include <bias_config.h>

#include <iostream>

#include <Base/Debug/Debug.hh>
#include <Base/Debug/Error.hh>

#define D_DC_DATA_CALC  0x0001 // 1

namespace BIAS
{
  class BIASImage_EXPORT DC_ptu : public Debug
  {
  public:
    DC_ptu();
    DC_ptu(const DC_ptu& ptu);
    ~DC_ptu();
    
    void Set(double PanPos, double TiltPos, unsigned long int TimeStamp,
             unsigned long StartReadTimeStamp,unsigned long StopReadTimeStamp);

    inline int GetBinaryLength()
    { return 2*sizeof(double)+3*sizeof(unsigned long); };
    void ReadBinary(char *data);
    void WriteBinary(char *data);

    friend std::istream& operator>>(std::istream& is, DC_ptu& ptu);
    friend std::ostream& operator<<(std::ostream& os, const DC_ptu& ptu);

    double PanPos, TiltPos;
    unsigned int TimeStamp, StartReadTimeStamp, StopReadTimeStamp;
  };

  class BIASImage_EXPORT DC_inertial : public Debug
  {
  public:
    DC_inertial();
    DC_inertial(const DC_inertial& in);
    ~DC_inertial();

    void Set(double yaw, double yawrateraw, double yawrateoffset, 
             double speed, double steeringangle); 

    inline int GetBinaryLength()
    { return 13*sizeof(double)+6*sizeof(int); };
    void ReadBinary(char *data);
    void WriteBinary(char *data);

    friend std::istream& operator>>(std::istream& is, DC_inertial& inert);
    friend std::ostream& 
    operator<<(std::ostream& os, const DC_inertial& inert);
    
    double Yaw, YawRateRaw, YawRateOffset;
    double SteeringAngle;
    double Pitch;
    double Speed;
    double LongAcceleration, LatAcceleration;
    int TempomatLever, BlinkLever, SteeringWheelLever;
    double ArtLeadVehicleDistance, ArtLeadVehicleVelocity, ArtSollVelocity;
    int ArtFlags;
    double AcceleratorPedal, EngineSpeed;
    int Gear, VehicleFlags;
  };

  class BIASImage_EXPORT DC_GPS : public Debug
  {
  public:
    DC_GPS();
    DC_GPS(const DC_GPS& gps);
    ~DC_GPS();

    void Set(int framenum, double latPos, double longPos, double azimuth, 
             double speed, double distanceToMap); 

    inline int GetBinaryLength()
    { return 12*sizeof(double)+sizeof(int); };

    void ReadBinary(char *data);
    void WriteBinary(char *data);

    friend std::istream& operator>>(std::istream& is, DC_GPS& gsp);
    friend std::ostream& operator<<(std::ostream& os, const DC_GPS& gps);

    int framenum;
    double latPos; // latitude
    double longPos; // longitude
    double azimuth; // angle to geographic north in rad
    double offset; 
    double speed; 
    double gpsTime; 
    double curvature; 
    double clothoidParameter; 
    double mapIndex; 
    double scaleFactorSpeed; 
    double yawRateOffset; 
    double distanceToMap;
  };

} // namespace BIAS

#endif // __DCDATA_HH__
