/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003-2009    (see file CONTACT for details)
Multimediale Systeme der Informationsverarbeitung
Institut fuer Informatik
Christian-Albrechts-Universitaet Kiel

BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <Base/Common/BIASpragma.hh>
#if defined(WIN32) && defined(_MSC_VER)
#  pragma warning (disable: 4661) /* no suitable def. for explicit instantiation */
#endif

#include "DisplacementMapping.hh"
#include <Base/Geometry/HomgPoint2D.hh>
#include <Base/Image/ImageIO.hh>
using namespace std;
using namespace BIAS;

template <class InputStorageType, class OutputStorageType>
DisplacementMapping<InputStorageType, OutputStorageType>::
~DisplacementMapping() 
{}

//Example of how to use DisplacementMapping is in ExampleHomography
//state: 18-05-2007

template <class InputStorageType, class OutputStorageType>
DisplacementMapping<InputStorageType, OutputStorageType>::
DisplacementMapping(){
  mapped_ = false;
}

template <class InputStorageType, class OutputStorageType>
DisplacementMapping<InputStorageType, OutputStorageType>::
DisplacementMapping(Image<float>& dismap){
  mapped_ = false;
  SetDisplacementMap(dismap);
}

template <class InputStorageType, class OutputStorageType>
int DisplacementMapping<InputStorageType, OutputStorageType>::
SetDisplacementMap(Image<float>& dismap) {
  if(dismap.GetChannelCount()!=3){
    cout << "Displacement Map must have 3 Channels but has only "
         << dismap.GetChannelCount() << endl;
    return -1;
  }
  if(!dismap.IsInterleaved()){
    cout << "Displacement map must be INTERLEAVED but is not" << endl;
    return -1;
  }
  dismap_ = dismap;
  mapped_ = true;
  return 0;
}

template <class InputStorageType, class OutputStorageType>
int DisplacementMapping<InputStorageType, OutputStorageType>::
GetSourceCoordinates_(const HomgPoint2D& sink,
                      HomgPoint2D& source) const {
  BIASASSERT(sink[2]==1.0);

  //check if the displacement map is computed
  if(!mapped_){
    //cout << "No Displacement Map in DisplacementMapper activated" << endl;
    return -1;
  }

  //check if sink-point is within the displacement map
  if(sink[0] >= dismap_.GetWidth() || sink[1] >= dismap_.GetHeight())
    return -1;
  if(sink[0] < 0 || sink[1] < 0)
    return -1;
  
  //get the index
  const float* Data = dismap_.GetImageData();
  
  //get index of the displacement-map image (unflipped index)
  int index = (int)(3.0*(((double)dismap_.GetHeight()-1.0-sink[1])*
                          (double)dismap_.GetWidth()+sink[0]));

  //result
  source[0] = Data[index];
  source[1] = Data[index+1];
  source[2] = Data[index+2];

  return 0;
}

namespace BIAS{
template class DisplacementMapping<unsigned char, unsigned char>;
template class DisplacementMapping<float, float>;
template class DisplacementMapping<unsigned char, float>;
template class DisplacementMapping<float, unsigned char>;

#if defined(BUILD_IMAGE_CHAR)
template class DisplacementMapping<unsigned char, char>;
template class DisplacementMapping<char, char>;
#endif

#if defined(BUILD_IMAGE_USHORT)
template class DisplacementMapping<unsigned short,unsigned short>;
#endif

#if defined(BUILD_IMAGE_SHORT)
template class DisplacementMapping<short, short>;
#endif

#if defined(BUILD_IMAGE_SHORT)&&defined(BUILD_IMAGE_USHORT)
template class DisplacementMapping<unsigned short, short>;
#endif

#if defined(BUILD_IMAGE_INT)
template class DisplacementMapping<int,int>;
#endif

#if defined(BUILD_IMAGE_USHORT)
template class DisplacementMapping<unsigned short,float>;
#endif

#if defined(BUILD_IMAGE_USHORT) && defined(BUILD_IMAGE_INT)
template class DisplacementMapping<unsigned short,int>;
#endif

#if defined(BUILD_IMAGE_DOUBLE)
template class DisplacementMapping<double,double>;
#endif
}
