/*
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003-2009    (see file CONTACT for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/**
	@example ExampleConvertBayerPattern.cpp
	@relates ConvertBayerPattern
	@brief Example for converting Bayer Pattern to RGB images using BIAS::ConvertBayerPattern
	@ingroup g_examples
	@author bangerer
*/


#ifndef WIN32
#include <Image/ConvertBayerPattern.hh>
#else
#error ConvertBayerPattern requires LINUX
#endif
#include <Base/Image/ImageIO.hh>
#include <Base/Image/ImageBase.hh>
#include <Base/Image/ImageConvert.hh>
#include <dc1394/dc1394.h>
#include <dc1394/conversions.h>

using namespace BIAS;
using namespace std;

int main(int argc, char *argv[])
{
	Image <unsigned char> source;

	// read the image
	if (ImageIO::Load(argv[1], source)!=0){
		BIASERR("error loading image "<<argv[1]);
		//return -2;
		return 0; // for Dart testing
	}

	// define an image with 3 channels
	Image <unsigned char> out(source.GetWidth(), source.GetHeight(),3);

	// instantiate ConvertBayerPattern class with correct storage type
	ConvertBayerPattern<unsigned char> converter;

	// demosaic image. you can choose the method for ConvertBayerPattern with the third parameter
	converter.Convert(source,out,BAYER_DEMOSAIC_METHOD_AHD,
			//                    ImageBase::CM_Bayer_BGGR);
			ImageBase::CM_Bayer_RGGB);

	// write the image
	if (ImageIO::Save(argv[2],out)!=0){
		BIASERR("error writing image ");
		//return -4;
		return 0; // for Dart testing
	}

	return 0;
}
