/*
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003-2009    (see file CONTACT for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/**
   @example ExampleConvertHDR.cpp
   @relates ConvertHDR 
   @brief Example for usage of ConvertHDR class   
   @ingroup g_examples
   @author MIP
*/

#include <Base/Image/Image.hh>
#include <Base/Image/ImageIO.hh>
#include <Image/ConvertHDR.hh>
#include <Base/Common/FileHandling.hh>
#include <Base/Image/ImageConvert.hh>

#include <iostream>

using namespace BIAS;
using namespace std;

int main(int argc, char *argv[])
{

  if (argc <2) {
    cerr<< argv[0] <<" <ppmfile1> <ppmfile2> ... "<<endl;
    exit(1);
  }


  ConvertHDR<float> convertF;
  Image<unsigned char> dstF;
  Image<float> inputImgF;
  Image<float> imageF;

  Image<float> greyF;
  Image<unsigned char> greyImg;
  for(int i = 1; i < argc; i++){
    imageF.SetZero();
    cout << "argv[i] " << argv[i] << endl;
    if ((ImageIO::ImportImage(argv[i], inputImgF)) != 0){
      cerr << "error loading file " << argv[i] << endl;
      exit (-1);
    }

    ImageConvert::ToGrey(inputImgF, greyF);

    if(convertF.ToUnsignedCharGamma(inputImgF, dstF, 0.5, 65535)!=0){
      BIASERR("could not convert ushort image to uchar");
      return -1;
    }

    if(convertF.ToUnsignedCharGamma(greyF, greyImg, 0.5, 65535)!=0){
      BIASERR("could not convert ushort image to uchar");
      return -1;
    }

    string fileName = "gammaF" + FileHandling::LeadingZeroString(i, 4) + ".mip";
    //ImageIO::Save(fileName, dstF);
    ImageIO::Save(fileName, dstF);

    string fileNameGrey = "gammaGrey" + FileHandling::LeadingZeroString(i, 4) + ".mip";
    //ImageIO::Save(fileNameGrey, greyImg);
    ImageIO::Save(fileNameGrey, greyImg);

  }



#ifdef BUILD_IMAGE_USHORT
  Image<unsigned short int> inputImg;
  ConvertHDR<unsigned short> converter;
  Image<unsigned char> dst;
  Image<unsigned short> image;

  for(int i = 1; i < argc; i++){
    image.SetZero();
    cout << "argv[i] " << argv[i] << endl;
    if ((ImageIO::ImportImage(argv[i], image)) != 0){
      cerr << "error loading file " << argv[i] << endl;
      exit (-1);
    }
//    image.ScaleShiftBetween(0, 65535);
    if(converter.ToUnsignedCharGamma(image, dst, 0.5, 65535)!=0){
      BIASERR("could not convert ushort image to uchar");
      return -1;
    }

    string fileName = "gammaUC" + FileHandling::LeadingZeroString(i, 4) + ".mip";
    //ImageIO::Save(fileName, dst);
    ImageIO::Save(fileName, dst);
  }

#endif
  return 0;
}

