/*
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003-2009    (see file CONTACT for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/**
 * @example ExampleHistogramEqualization.cpp
   @relates HistogramEqualization
   @brief Example for HistogramEqualization usage
   @ingroup g_examples
   @author  MIP
*/

#include <Base/Image/ImageIO.hh>
#include <Base/Image/ImageConvert.hh>
#include <Base/Common/FileHandling.hh>
#include <Image/HistogramEqualization.hh>
#include <Image/HistogramImage.hh>
#include <iostream>
#include <fstream>

using namespace BIAS;
using namespace std;

//writes a data textfile for plotting with e.g. gnuplot
void WriteCumulativeHistogram(Image<unsigned char>& im)
{
   HistogramImage hsrc;
   hsrc.Init(256,256,1);
   hsrc.InitHist(256);
   hsrc.AddHist<unsigned char>(im);
  
   ofstream outfile("cumulathist.dat");
   double sum = 0.0;
   for(unsigned b=0;b<256; b++)
   {
     sum += hsrc.GetBin(b);
     outfile<<sum<<endl;
   }
   outfile.close();
}


int main(int argc, char *argv[]) 
{
  HistogramEqualization<unsigned char, unsigned char> HistEq;  
  if(argc<2)
  {
    cout<<"usage: "<<argv[0]<<" <image file>"<<endl;
    return -1;
  }
  Image<unsigned char> InputImg,GreyImg,OutPutImg;
  if(ImageIO::Load(argv[1],InputImg)!=0)
  {
    cout<<"can't load "<<argv[1]<<endl;
    return -2;
  }
  string full(argv[1]);
  string dir,base,suffix;
  FileHandling::SplitName(full,dir,base,suffix);
  ImageConvert::ToGrey(InputImg,GreyImg);
  ImageIO::Save(dir+base+"grey.mip",GreyImg);
  HistEq.Filter(GreyImg,OutPutImg); 
  ImageIO::Save(dir+base+"HstEq.mip",OutPutImg);
  WriteCumulativeHistogram(OutPutImg);
  return 0;
}
