/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003-2009    (see file CONTACT for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

  
/**
 * @example ExampleLogPolarMapping.cpp
   @relates LogPolarMapping
   @brief transforms an image to the logarithmic polar mapping
   @ingroup g_examples
   @author  oniemann
*/

#include <Image/LogPolarMapping.hh>
#include <Base/Image/ImageIO.hh>
#include <Base/Debug/TimeMeasure.hh>
#include <Utils/Param.hh>

using namespace BIAS;
using namespace std;


int main(int argc, char *argv[])
{

  Param *param = new Param();
  string sourceImage;

  param->AddParamString("SourceImage",
			"Absolute path to the source Image",
                        "",
                        'i');

  param->ParseCommandLine(argc,argv);

  //Read and load source image from parameters
  sourceImage = *param->GetParamString("SourceImage");
  if(sourceImage == "")
    {
      BIASERR(argv[0]<<" -i<image-file>");
    return -1;
    }

 
  Image<unsigned char> im;
  if (ImageIO::Load(sourceImage, im)!=0){
    BIASERR("error loading "<<sourceImage);
    return -2;
  }

  LogPolarMapping<unsigned char, unsigned char> mapper;
  int width  = im.GetWidth();
  int height = im.GetHeight();
  //We want the center of rotation to be in center of the image

  mapper.SetImageCenter(width,height/2);

  cout<<"ROI of source image is "<<*im.GetROI()
      <<" while image size is "<<width<<" x "<<height<<endl;
  Image<unsigned char> 
      imrestrilinear(2*width,height, im.GetChannelCount());
  imrestrilinear.SetColorModel(im.GetColorModel());

  cout<<"timing ..."<<endl<<flush;
  TimeMeasure t1;

  t1.Start();
  // trilinear mapping with automatic scale and pyramid size selection
  mapper.Map(im, imrestrilinear, MapBilinear);
  t1.Stop();
  //if (ImageIO::Save("ImgLogPolar.mip", imrestrilinear)!=0){
  if (ImageIO::Save("ImgLogPolar.mip", imrestrilinear)!=0){
    BIASERR("error image");
  }

  cout <<"Trilinear mapping took "
       <<t1.GetUserTime()
       <<" ms,"<<endl;

  return 0;
}
