#ifndef _FORWARDMAPPINGNEARESTNEIGHBOUR_HH_
#define _FORWARDMAPPINGNEARESTNEIGHBOUR_HH_

#include <Base/Common/BIASpragmaStart.hh>
#include <Base/Image/Image.hh>
#include <Geometry/Projection.hh>

namespace BIAS {

  template <class InputStorageType, class OutputStorageType> 
  class BIASImage_EXPORT ForwardMappingNearestNeighbour {
    
  public:
    ForwardMappingNearestNeighbour();
    ForwardMappingNearestNeighbour(const ForwardMappingNearestNeighbour& src);

    ~ForwardMappingNearestNeighbour();

    ForwardMappingNearestNeighbour<InputStorageType, OutputStorageType>&  
      operator=(const ForwardMappingNearestNeighbour<InputStorageType, OutputStorageType>& src);

    void SetSourceCam(const Projection& P) { 
      SourceP_ = P;
      SourceP_.GetParameters()->GetImageSize(dSourceWidth_,
                                             dSourceHeight_);
    }

    void SetSinkCam(const Projection& P, 
                    const unsigned int dTexChannels = 1) {
      dTexChannels_ = dTexChannels;
      SinkP_ = P;
      SinkP_.GetParameters()->GetImageSize(dTargetWidth_, 
                                           dTargetHeight_);
      if (intermediate_.IsEmpty())
        intermediate_.Init(dTargetWidth_, 
                           dTargetHeight_);
      if (intermediateTex_.IsEmpty())
        intermediateTex_.Init(dTargetWidth_, dTargetHeight_, 
                              dTexChannels_);
    }

    int Map(const BIAS::Image<InputStorageType> *in, 
            BIAS::Image<OutputStorageType>& out, 
            const BIAS::Image<InputStorageType> *texin,
            BIAS::Image<OutputStorageType>& texout);
    
    void GetROI(BIAS::HomgPoint2D &UL,  BIAS::HomgPoint2D &LR){
      UL = boundMin_;
      LR = boundMax_;      
    };

    void SetFillAreaWindowSize(unsigned int size){
      fillAreaWindowSize_ = size;
    }
    
  protected:
    BIAS::HomgPoint2D boundMin_, boundMax_,boundMinWrap_, boundMaxWrap_;
    unsigned int fillAreaWindowSize_;
    BIAS::Projection SourceP_, SinkP_;
    BIAS::Image<OutputStorageType> intermediate_;
    BIAS::Image<OutputStorageType> intermediateTex_;
    int ulx_, uly_, lrx_, lry_;
    unsigned int dTexChannels_;
    unsigned int dSourceWidth_, dSourceHeight_;
    unsigned int dTargetWidth_, dTargetHeight_;
    bool fillblack_;
    
  private:
    unsigned int GetSummedValue_(const OutputStorageType* pixel);
    void SetValue_(const OutputStorageType* in, OutputStorageType* out);
  };


} // namespace

#include <Base/Common/BIASpragmaEnd.hh>


#endif /*_FORWARDMAPPINGNEARESTNEIGHBOUR_HH_*/
