/* 
   This file is part of the BIAS library (Basic ImageAlgorithmS).

   Copyright (C) 2003-2009    (see file CONTACT for details)
   Multimediale Systeme der Informationsverarbeitung
   Institut fuer Informatik
   Christian-Albrechts-Universitaet Kiel
   
   
   BIAS is free software; you can redistribute it and/or modify
   it under the terms of the GNU Lesser General Public License as published by
   the Free Software Foundation; either version 2.1 of the License, or
   (at your option) any later version.
   
   BIAS is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public License
   along with BIAS; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <Base/Common/BIASpragmaStart.hh>

#ifndef __SphericalRectification_hh__
#define __SphericalRectification_hh__

#include <Image/RectificationViaProjectionMappingBase.hh>

namespace BIAS {

  /** Class implements rectification by image \b projection onto a sphere.
   *  The sphere is resampled using the great circles camera implementation
   *  from ProjectionParametersGreatCircles. The input projections are for now 
   *  restricted to *Spherical cameras.
   *
   * \ingroup g_filter
   *  \author bartczak 12/2006
   */
  template <class InputStorageType, class OutputStorageType> 
  class BIASImage_EXPORT SphericalRectification :
    public RectificationViaProjectionMappingBase<InputStorageType, 
                                                 OutputStorageType>
  {
  public:
    virtual ~SphericalRectification();
    SphericalRectification();

    /** only spherical cameras are allowed for now */
    virtual bool 
    IsInputCameraValid(const BIAS::Image<InputStorageType>& img,
                       const BIAS::ProjectionParametersBase* proj);

  protected:
    virtual int DetermineRectificationParameters_();
  };

}//namespace
#endif
#include <Base/Common/BIASpragmaEnd.hh>

