/*
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003-2009    (see file CONTACT for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include "CornerMatcher.hh"

using namespace BIAS;
using namespace std;


#define INST(uctype)\
template BIASMatcher2D_EXPORT int CornerMatcher::SSD<uctype>(const HomgPoint2D& p1, const HomgPoint2D& p2,\
  const Image<uctype>& im1,\
  const Image<uctype>& im2, const unsigned int halfwinsize, double& result)const ;\
template BIASMatcher2D_EXPORT int CornerMatcher::NCC<uctype>(const HomgPoint2D& p1, const HomgPoint2D& p2,\
  const Image<uctype>& im1,\
  const Image<uctype>& im2, const unsigned int halfwinsize, double& result)const ;\
template BIASMatcher2D_EXPORT int CornerMatcher::NCCSearch<uctype>(const HomgPoint2D& p1, const HomgPoint2D& p2,\
  const Image<uctype>& im1, const Image<uctype>& im2, \
  const unsigned int halfnccwinsize, const unsigned int halfsearchwinsize, \
  HomgPoint2D& resultpoint,\
  const double mincorrelation, \
  double& result);\
template BIASMatcher2D_EXPORT int CornerMatcher::ParabolaNCC(const HomgPoint2D& p1, const HomgPoint2D& p2,\
  const Image<uctype>& im1, const Image<uctype>& im2, const unsigned int halfwinsize,\
  HomgPoint2D& result)const ;\
template BIASMatcher2D_EXPORT int CornerMatcher::ParabolaNCC5(const HomgPoint2D& p1, const HomgPoint2D& p2,\
  const Image<uctype>& im1, const Image<uctype>& im2, const unsigned int halfwinsize,\
  HomgPoint2D& result)const ;\
template BIASMatcher2D_EXPORT int CornerMatcher::RefinePyramideNCC<uctype>(\
  std::vector<std::vector<HomgPoint2D> >& ps1, PyramidImage<uctype>& im1, \
  PyramidImage<uctype>& im2, unsigned int halfnccwinsize, \
  std::vector<std::vector<HomgPoint2D> >& matches,  \
  std::vector<std::vector<double> >& matchquality)



INST(unsigned char);
INST(float);

// fill in instances as required
#ifdef BUILD_IMAGE_INT
INST(int);
#endif
#ifdef BUILD_IMAGE_CHAR
INST(char);
#endif
#ifdef BUILD_IMAGE_SHORT
INST(short);
#endif
#ifdef BUILD_IMAGE_USHORT
INST(unsigned short);
#endif
#ifdef BUILD_IMAGE_UINT
INST(unsigned int);
#endif
#ifdef BUILD_IMAGE_DOUBLE
INST(double);
#endif


CornerMatcher::CornerMatcher()
  : RegionMatcher()
{}

CornerMatcher::~CornerMatcher()
{}

template <class StorageType>
int CornerMatcher::
SSD(const HomgPoint2D& p1, const HomgPoint2D& p2,
    const Image<StorageType>& im1, const Image<StorageType>& im2,
    const unsigned int halfwinsize, double& result) const
{
  int res=0;
  int ip1[2], ip2[2];
  unsigned int uip1[2], uip2[2];
  if (p1.IsAtInfinity() || p2.IsAtInfinity())
    res=-2;
  else {
    ip1[0]=(int)rint(p1[0]/p1[2]);
    ip1[1]=(int)rint(p1[1]/p1[2]);
    ip2[0]=(int)rint(p2[0]/p2[2]);
    ip2[1]=(int)rint(p2[1]/p2[2]);
    if (ip1[0]<(int)halfwinsize ||
        ip1[0]+(int)halfwinsize>=(int)im1.GetWidth() ||
        ip1[1]<(int)halfwinsize ||
        ip1[1]+(int)halfwinsize>=(int)im1.GetHeight() ||
        ip2[0]<(int)halfwinsize ||
        ip2[0]+(int)halfwinsize>=(int)im2.GetWidth() ||
        ip2[1]<(int)halfwinsize ||
        ip2[1]+(int)halfwinsize>=(int)im2.GetHeight())
      res=-1;
    else{
      uip1[0]=(unsigned int)ip1[0];
      uip1[1]=(unsigned int)ip1[1];
      uip2[0]=(unsigned int)ip2[0];
      uip2[1]=(unsigned int)ip2[1];
      RegionMatcher::SSD(uip1, uip2, im1.GetImageDataArray(),
                         im2.GetImageDataArray(), halfwinsize, result);
    }
  }
  return res;
}

template <class StorageType>
int CornerMatcher::
NCC(const HomgPoint2D& p1, const HomgPoint2D& p2,
    const Image<StorageType>& im1, const Image<StorageType>& im2,
    const unsigned int halfwinsize, double& result) const
{
  int res=0;
  int ip1[2], ip2[2];
  unsigned int uip1[2], uip2[2];
  if (p1.IsAtInfinity() || p2.IsAtInfinity())
    res=-2;
  else {
    ip1[0]=(int)rint(p1[0]/p1[2]);
    ip1[1]=(int)rint(p1[1]/p1[2]);
    ip2[0]=(int)rint(p2[0]/p2[2]);
    ip2[1]=(int)rint(p2[1]/p2[2]);
    if (ip1[0]<(int)halfwinsize ||
        ip1[0]+(int)halfwinsize>=(int)im1.GetWidth() ||
        ip1[1]<(int)halfwinsize ||
        ip1[1]+(int)halfwinsize>=(int)im1.GetHeight() ||
        ip2[0]<(int)halfwinsize ||
        ip2[0]+(int)halfwinsize>=(int)im2.GetWidth() ||
        ip2[1]<(int)halfwinsize ||
        ip2[1]+(int)halfwinsize>=(int)im2.GetHeight())
      res=-1;
    else{
      uip1[0]=(unsigned int)ip1[0];
      uip1[1]=(unsigned int)ip1[1];
      uip2[0]=(unsigned int)ip2[0];
      uip2[1]=(unsigned int)ip2[1];
      RegionMatcher::NCC(uip1, uip2, im1.GetImageDataArray(),
                         im2.GetImageDataArray(), halfwinsize, result);
    }
  }
  return res;
}


template <class StorageType>
int CornerMatcher::
NCCSearch(const HomgPoint2D& p1, const HomgPoint2D& p2,
          const Image<StorageType>& im1, const Image<StorageType>& im2,
          const unsigned int halfnccwinsize,
          const unsigned int halfsearchwinsize,
          HomgPoint2D& resultpoint,
          const double mincorrelation,
          double& result )
{
  BIASASSERT(im1.GetColorModel()==BIAS::ImageBase::CM_Grey);
  BIASASSERT(im2.GetColorModel()==BIAS::ImageBase::CM_Grey);

  const double THRESHOLD=1e-8;
  int res=0;
  int ip1[2], ip2[2];
  unsigned int uip1[2], uip2[2], resp[2];
  int minedgedist = halfnccwinsize + halfsearchwinsize;
  if (p1.IsAtInfinity() || p2.IsAtInfinity()) {
    res=-2;
  } else {
    ip1[0]=(int)rint(p1[0]/p1[2]);
    ip1[1]=(int)rint(p1[1]/p1[2]);
    ip2[0]=(int)rint(p2[0]/p2[2]);
    ip2[1]=(int)rint(p2[1]/p2[2]);
    if (fabs(p2[0]/p2[2]-ip2[0])<THRESHOLD &&
        fabs(p2[1]/p2[2]-ip2[1])<THRESHOLD )
    {
      if (ip1[0] <= minedgedist || ip1[0] + minedgedist>=(int)im1.GetWidth() ||
          ip1[1] <= minedgedist || ip1[1] + minedgedist>=(int)im1.GetHeight() ||
          ip2[0] <= minedgedist || ip2[0] + minedgedist>=(int)im2.GetWidth() ||
          ip2[1] <= minedgedist || ip2[1] + minedgedist>=(int)im2.GetHeight() ){
        res=-1;
      } else {
        uip1[0]=(unsigned int)ip1[0];
        uip1[1]=(unsigned int)ip1[1];
        uip2[0]=(unsigned int)ip2[0];
        uip2[1]=(unsigned int)ip2[1];
        BIASDOUT(D_CORNER_MATCHER_SEARCH,"p1 " << p1 << " " << uip1[0] << " "
                 << uip1[1] << "\tp2 " << p2 << " " << uip2[0]<<" "<< uip2[1]);
        BIASDOUT(D_CORNER_MATCHER_SEARCH,"using NCCSearchOdd for " << p2
                 << " " << uip2[0] << " "<< uip2[1]);
        res = NCCSearchOdd(uip1, uip2, im1.GetImageDataArray(),
                           im2.GetImageDataArray(), halfnccwinsize,
                           halfsearchwinsize, resp, mincorrelation, result);
        resultpoint.Set((double)resp[0], (double)resp[1]);
        BIASDOUT(D_CORNER_MATCHER_SEARCH,"resultpoint " << resultpoint);
      } // within image
    } else { // searcheven/searchodd
      ip1[0]=(int)rint(p1[0]/p1[2]);
      ip1[1]=(int)rint(p1[1]/p1[2]);
      ip2[0]=(int)floor(p2[0]/p2[2]);
      ip2[1]=(int)floor(p2[1]/p2[2]);
      if (ip1[0] < minedgedist || ip1[0] + minedgedist>=(int)im1.GetWidth() ||
          ip1[1] < minedgedist || ip1[1] + minedgedist>=(int)im1.GetHeight() ||
          ip2[0] < minedgedist || ip2[0] + minedgedist>=(int)im2.GetWidth() ||
          ip2[1] < minedgedist || ip2[1] + minedgedist>=(int)im2.GetHeight() ){
        res=-1;
      } else {
        uip1[0]=(unsigned int)ip1[0];
        uip1[1]=(unsigned int)ip1[1];
        uip2[0]=(unsigned int)ip2[0];
        uip2[1]=(unsigned int)ip2[1];
        BIASDOUT(D_CORNER_MATCHER_SEARCH,"using NCCSearchEven for " << p2
                 << " " << uip2[0] << " "<< uip2[1]);
        res = NCCSearchEven(uip1, uip2, im1.GetImageDataArray(),
                            im2.GetImageDataArray(), halfnccwinsize,
                            halfsearchwinsize, resp, mincorrelation, result);
        resultpoint.Set((double)resp[0], (double)resp[1]);
        BIASDOUT(D_CORNER_MATCHER_SEARCH,"resultpoint " << resultpoint);
      } // within image
    } // if searcheven | serachodd
  } // if (p1.IsAtInfinity() || p2.IsAtInfinity()) {
  return res;
}

/*
template <class StorageType>
int CornerMatcher::KLT(HomgPoint2D& p1, HomgPoint2D& p2,
                       Image<StorageType>& im1, Image<StorageType>& im2,
                       Image<StorageType>& gradx, Image<StorageType>& grady,
                       unsigned int halfwinsize, unsigned int gradientborder,
                       unsigned int numiterations, HomgPoint2D& result,
                       KLT_TYPE& error)
{
  int res=0;
  KLT_TYPE dp1[2], dp2[2], resp[2];
  double minedgedist = (double)(halfwinsize + gradientborder);

  if (p1.IsAtInfinity() || p2.IsAtInfinity()) {
    res=-2;
  } else {
    dp1[0]=p1[0]/p1[2];
    dp1[1]=p1[1]/p1[2];
    dp2[0]=p2[0]/p2[2];
    dp2[1]=p2[1]/p2[2];
    if (dp1[0] < minedgedist || dp1[0] + minedgedist >= im1.GetWidth() ||
        dp1[1] < minedgedist || dp1[1] + minedgedist >= im1.GetHeight() ||
        dp2[0] < minedgedist || dp2[0] + minedgedist >= im2.GetWidth() ||
        dp2[1] < minedgedist || dp2[1] + minedgedist >= im2.GetHeight() ) {
      res=-1;
    } else {
      BIASDOUT(D_CORNER_MATCHER_KLT,"p1: " << p1 << "\tp2: "<< p2);
      res = RegionMatcher::
        KLT(dp1, dp2, im1.GetImageDataArray(), im2.GetImageDataArray(),
            gradx.GetImageDataArray(), grady.GetImageDataArray(),
            im1.GetWidth(), im1.GetHeight(), halfwinsize,
            numiterations, resp, error);
      result.Set(resp[0], resp[1]);
    } // p1 / p1 within image
  } // p1 || p2 at infinity

  return res;
}

template <class StorageType>
int CornerMatcher::KLT(HomgPoint2D& p1, HomgPoint2D& p2,
                       Image<StorageType>& im1, Image<StorageType>& im2,
                       Image<StorageType>& gradx, Image<StorageType>& grady,
                       unsigned int halfwinsize, unsigned int gradientborder,
                       unsigned int maxnumiterations, KLT_TYPE maxerror,
                       HomgPoint2D& result, KLT_TYPE& error)
{
  int res=0;
  KLT_TYPE dp1[2], dp2[2], resp[2];
  double minedgedist = (double)(halfwinsize + gradientborder);

  if (p1.IsAtInfinity() || p2.IsAtInfinity()) {
    BIASDOUT(D_CORNER_MATCHER_KLT," a point is at infinty p1: "
             << p1 << "\tp2: "<< p2);
    res=-2;
  } else {
    dp1[0]=p1[0]/p1[2];
    dp1[1]=p1[1]/p1[2];
    dp2[0]=p2[0]/p2[2];
    dp2[1]=p2[1]/p2[2];
    if (dp1[0] < minedgedist || dp1[0] + minedgedist >= im1.GetWidth() ||
        dp1[1] < minedgedist || dp1[1] + minedgedist >= im1.GetHeight() ||
        dp2[0] < minedgedist || dp2[0] + minedgedist >= im2.GetWidth() ||
        dp2[1] < minedgedist || dp2[1] + minedgedist >= im2.GetHeight() ) {
      BIASDOUT(D_CORNER_MATCHER_KLT,
               " a point is to cloose to image border p1: "
               << p1 << "\tp2: "<< p2);
      res=-1;
    } else {
      BIASDOUT(D_CORNER_MATCHER_KLT,"p1: " << p1 << "\tp2: "<< p2);
      res = RegionMatcher::
        KLT(dp1, dp2, im1.GetImageDataArray(), im2.GetImageDataArray(),
            gradx.GetImageDataArray(), grady.GetImageDataArray(),
            im1.GetWidth(), im1.GetHeight(), halfwinsize,
            maxnumiterations, maxerror, resp, error);
      result.Set(resp[0], resp[1]);
    } // p1 / p1 within image
  } // p1 || p2 at infinity

  return res;
}

template <class StorageType>
int CornerMatcher::KLT2(KLT_TYPE point1[2], KLT_TYPE point2[2],
                        PyramidImage2<StorageType>& pyim1,
                        PyramidImage2<StorageType>& pyim2,
                        PyramidImage2<StorageType>& pygradx1,
                        PyramidImage2<StorageType>& pygrady1,
                        PyramidImage2<StorageType>& pygradx2,
                        PyramidImage2<StorageType>& pygrady2, unsigned pyindex,
                        unsigned hws, unsigned maxiterations, double SADThresh,
                        KLT_TYPE maxerror, KLT_TYPE res[2], KLT_TYPE& error)
{
#ifdef TIMING
  TimeMeasure timer;
  timer.Start();
#endif
  int pysize, width, height, j;
  StorageType **ida1, **ida2, **idagx1, **idagy1, **idagx2, **idagy2;
  unsigned x1, x2, y1, y2, corr, mh, mw, sadthresh3x3, sadthresh5x5;
  KLT_TYPE p1[2], p2[2], fx, fy, div;

  sadthresh3x3=(unsigned)(SADThresh*9.0);
  sadthresh5x5=(unsigned)(SADThresh*25.0);
  pysize=pyim1.size();
  // first track on smallest level
  j=pysize-1;
  ida1=pyim1[j]->GetImageDataArray();
  ida2=pyim2[j]->GetImageDataArray();
  idagx1=pygradx1[j]->GetImageDataArray();
  idagy1=pygrady1[j]->GetImageDataArray();
  idagx2=pygradx2[j]->GetImageDataArray();
  idagy2=pygrady2[j]->GetImageDataArray();
  width=pyim1[j]->GetWidth();
  height=pyim1[j]->GetHeight();
  mh=height-hws-1; mw=width-hws-1;

  // now track smalles level
#ifdef WIN32
  // jw, sunmin: (09/2004)  do NOT use std:: because of compex instance
  div=1.0/pow(2.0, double(pysize-pyindex-1));
#else
  // jw, woelk: (09/2004): Felix is *sure* that he needs std:: ;-)
  div=1.0/std::pow(2.0, double(pysize-pyindex-1));
#endif

  p1[0]=point1[0]*div;
  p1[1]=point1[1]*div;
  p2[0]=p1[0]; p2[1]=p1[1];
  fx=fy=0.0;
  BIASCDOUT(D_CORNER_MATCHER_PYKLT, j<<" maybe tracking between ("<<p1[0]<<", "
            <<p1[1]<<") and ("<<p2[0]<<", "<<p2[1]<<")\n");
  if (p2[0]>hws && p2[1]>hws && p2[0]<mw && p2[1]<mh){
    BIASCDOUT(D_CORNER_MATCHER_PYKLT, j<<" tracking between ("<<p1[0]<<", "
              <<p1[1]<<") and ("<<p2[0]<<", "<<p2[1]<<")\n");
    if (RegionMatcher::KLT2(p1, p2,ida1,ida2,idagx1,idagy1,idagx2,idagy2,width,
                            height,hws,maxiterations,maxerror, res, error)==0){
      BIASCDOUT(D_CORNER_MATCHER_PYKLT,j<<" tracking succeeded, maybe sad "
                  <<"between ("<<p1[0]<<", "<<p1[1]<<") and ("<<res[0]<<", "
                  <<res[1]<<")\n");
      x1=(unsigned)rint(p1[0]);
      y1=(unsigned)rint(p1[1]);
      x2=(unsigned)rint(res[0]);
      y2=(unsigned)rint(res[1]);
      if (x2>=2 && y2>=2 && x2<mw && y2<mh){
        BIASCDOUT(D_CORNER_MATCHER_PYKLT, j<<" sad between ("
                  <<x1<<", "<<y1<<") and ("<<x2<<", "<<y2<<")\n");
        SAD3x3(x1, y1, x2, y2, ida1, ida2, corr);
      } else {
        BIASCDOUT(D_CORNER_MATCHER_PYKLT, j<<" no sad, out of bounds\n");
        if (j==(int)pyindex) {
          return -1;
        } else {
          corr=sadthresh3x3; // cannot use sad so fake failure
        }
      }
      if (corr<sadthresh3x3){
        BIASCDOUT(D_CORNER_MATCHER_PYKLT, j<<" sad succeeded ("<<p1[0]<<", "<<
                  p1[1]<<") and ("<<res[0]<<", "<<res[1]<<") = "<<corr<<"\n");
        fx=res[0]-p1[0];
        fy=res[1]-p1[1];
      } else {
        BIASCDOUT(D_CORNER_MATCHER_PYKLT, j<<" sad failed ("<<p1[0]<<", "<<
                  p1[1]<<") and ("<<res[0]<<", "<<res[1]<<") = "<<corr<<"\n");
        fx=fy=0.0;
        if (j==(int)pyindex) { return -2; }
      }
    } else {
      BIASCDOUT(D_CORNER_MATCHER_PYKLT, j<<" KLT failed\n");
      if (j==(int)pyindex) { return -3; }
    }
  } else {
    BIASCDOUT(D_CORNER_MATCHER_PYKLT, j<<" not tracking, out of bounds\n");
    if (j==(int)pyindex) { return -4; }
  }

#ifdef TIMING
  timer.Stop();
  GetDebugStream() << "smallest pyramide leveltook "<<timer.GetRealTime()<<" us"<<endl;
  timer.Reset();
  timer.Start();
#endif
  // hws=2;
  for (j=pysize-2; j>=(int)pyindex; j--){
    ida1=pyim1[j]->GetImageDataArray();
    ida2=pyim2[j]->GetImageDataArray();
    idagx1=pygradx1[j]->GetImageDataArray();
    idagy1=pygrady1[j]->GetImageDataArray();
    idagx2=pygradx2[j]->GetImageDataArray();
    idagy2=pygrady2[j]->GetImageDataArray();
    width=pyim1[j]->GetWidth();
    height=pyim1[j]->GetHeight();
    mh=height-hws-1; mw=width-hws-1;

    if (j==(int)pyindex){
      p1[0]=point1[0];
      p1[1]=point1[1];
    } else {
      p1[0]*=2.0;
      p1[1]*=2.0;
    }
    p2[0]=p1[0]+fx*2;
    p2[1]=p1[1]+fy*2;

    BIASCDOUT(D_CORNER_MATCHER_PYKLT, j<<" maybe tracking between ("<<p1[0]
              <<", "<<p1[1]<<") and ("<<p2[0]<<", "<<p2[1]<<")\n");
    if (p2[0]>hws && p2[1]>hws && p2[0]<mw && p2[1]<mh){
      BIASCDOUT(D_CORNER_MATCHER_PYKLT, j<<" tracking between ("<<p1[0]<<", "
                <<p1[1]<<") and ("<<p2[0]<<", "<<p2[1]<<")\n");
      if (RegionMatcher::KLT2(p1,p2,ida1,ida2,idagx1,idagy1,idagx2,idagy2,
                              width,height,hws,maxiterations,maxerror,
                              res,error)==0){
        BIASCDOUT(D_CORNER_MATCHER_PYKLT,j<<" tracking succeeded, maybe sad "
                  <<"between ("<<p1[0]<<", "<<p1[1]<<") and ("<<res[0]<<", "
                  <<res[1]<<")\n");
        x1=(unsigned)rint(p1[0]);
        y1=(unsigned)rint(p1[1]);
        x2=(unsigned)rint(res[0]);
        y2=(unsigned)rint(res[1]);

        if (x2>=3 && y2>=3 && x2<mw && y2<mh){
          BIASCDOUT(D_CORNER_MATCHER_PYKLT, j<<" sad between ("
                    <<x1<<", "<<y1<<") and ("<<x2<<", "<<y2<<")\n");
          SAD5x5(x1, y1, x2, y2, ida1, ida2, corr);
        } else {
          BIASCDOUT(D_CORNER_MATCHER_PYKLT, j<<" no sad, out of bounds\n");
          if (j==(int)pyindex) {
            return -1;
          } else {
            corr=sadthresh5x5;
          } // cannot use sad so fake failure
        }
        if (corr<sadthresh5x5){
          fx=res[0]-p1[0];
          fy=res[1]-p1[1];
          BIASCDOUT(D_CORNER_MATCHER_PYKLT, j<<" sad succeeded, adding ("<<
                    p1[0]<<", "<<p1[1]<<") -> ("<<res[0]<<", "<<res[1]<<"\n");
        } else {
          BIASCDOUT(D_CORNER_MATCHER_PYKLT, j<<" sad failed: "<<corr<<"\n");
          if (j==(int)pyindex) { return -2; }
          else { fx*=2.0; fy*=2.0; }
        }
      }  else {
        BIASCDOUT(D_CORNER_MATCHER_PYKLT, j<<" KLT failed\n");
        if (j==(int)pyindex) { return -3; }
      }
    } else {
      BIASCDOUT(D_CORNER_MATCHER_PYKLT, j<<" not tracking, out of bounds\n");
      if (j==(int)pyindex) { return -4; }
    }
  }
#ifdef TIMING
  timer.Stop();
  GetDebugStream() << "Pyramid KLT2 took "<<timer.GetRealTime()<<" us"<<endl;
#endif
  return 0;

}
*/
template <class StorageType>
int CornerMatcher::
ParabolaNCC(const HomgPoint2D& p1, const HomgPoint2D& p2,
            const Image<StorageType>& im1, const Image<StorageType>& im2,
            const unsigned int halfwinsize,HomgPoint2D& result) const
{
  int res=0;
  KLT_TYPE resu[2];
  unsigned int uip1[2], uip2[2];
  double dp1x, dp1y;
  int ip1[2], ip2[2];
  int minedgedist = halfwinsize + 1;
  if (p1.IsAtInfinity() || p2.IsAtInfinity()) {
    res=-2;
  } else {
    ip1[0]=(int)rint(p1[0]/p1[2]);
    ip1[1]=(int)rint(p1[1]/p1[2]);
    ip2[0]=(int)rint(p2[0]/p2[2]);
    ip2[1]=(int)rint(p2[1]/p2[2]);
    if (ip1[0] < minedgedist || ip1[0] + minedgedist>=(int)im1.GetWidth() ||
        ip1[1] < minedgedist || ip1[1] + minedgedist>=(int)im1.GetHeight() ||
        ip2[0] < minedgedist || ip2[0] + minedgedist>=(int)im2.GetWidth() ||
        ip2[1] < minedgedist || ip2[1] + minedgedist>=(int)im2.GetHeight() ){
        res=-1;
      } else {
        uip1[0]=(unsigned int)ip1[0];
        dp1x=p1[0]-(double)uip1[0];
        uip1[1]=(unsigned int)ip1[1];
        dp1y=p1[1]-(double)uip1[1];
        uip2[0]=(unsigned int)ip2[0];
        uip2[1]=(unsigned int)ip2[1];
        res=RegionMatcher::ParabolaNCC(uip1, uip2, im1.GetImageDataArray(),
                                       im2.GetImageDataArray(), halfwinsize,
                                       resu);
        result.Set(resu[0]+dp1x, resu[1]+dp1y);
        BIASDOUT(D_CORNER_MATCHER_PARABOLA, "\np1 "<<p1<<"\tp2 "<<p2
                 <<"\tuip1 "<<uip1[0]<<" "<<uip1[1]<<"\tuip2 "<<uip2[0]
                 <<" "<<uip2[1]<<"\tresu: "<<resu[0]<<" "<<resu[1]
                 <<"\tdp1x/y "<<dp1x<<" "<<dp1y);
      }
  }
  return res;
}


template <class StorageType>
int CornerMatcher::
ParabolaNCC5(const HomgPoint2D& p1, const HomgPoint2D& p2,
             const Image<StorageType>& im1, const Image<StorageType>& im2,
             const unsigned int halfwinsize, HomgPoint2D& result)const
{
  int res=0;
  KLT_TYPE resu[2];
  unsigned int uip1[2], uip2[2];
  double dp1x, dp1y;
  int ip1[2], ip2[2];
  int minedgedist = halfwinsize + 2;
  if (p1.IsAtInfinity() || p2.IsAtInfinity()) {
    res=-2;
  } else {
    ip1[0]=(int)rint(p1[0]/p1[2]);
    ip1[1]=(int)rint(p1[1]/p1[2]);
    ip2[0]=(int)rint(p2[0]/p2[2]);
    ip2[1]=(int)rint(p2[1]/p2[2]);
    if (ip1[0] < minedgedist || ip1[0] + minedgedist>=(int)im1.GetWidth() ||
        ip1[1] < minedgedist || ip1[1] + minedgedist>=(int)im1.GetHeight() ||
        ip2[0] < minedgedist || ip2[0] + minedgedist>=(int)im2.GetWidth() ||
        ip2[1] < minedgedist || ip2[1] + minedgedist>=(int)im2.GetHeight() ){
        res=-1;
      } else {
        uip1[0]=(unsigned int)ip1[0];
        dp1x=p1[0]-(double)uip1[0];
        uip1[1]=(unsigned int)ip1[1];
        dp1y=p1[1]-(double)uip1[1];
        uip2[0]=(unsigned int)ip2[0];
        uip2[1]=(unsigned int)ip2[1];
        res=RegionMatcher::ParabolaNCC5(uip1, uip2, im1.GetImageDataArray(),
                                        im2.GetImageDataArray(), halfwinsize,
                                        resu);
        result.Set(resu[0]+dp1x, resu[1]+dp1y);
        BIASDOUT(D_CORNER_MATCHER_PARABOLA, "\np1 "<<p1<<"\tp2 "<<p2
                 <<"\tuip1 "<<uip1[0]<<" "<<uip1[1]<<"\tuip2 "<<uip2[0]
                 <<" "<<uip2[1]<<"\tresu: "<<resu[0]<<" "<<resu[1]
                 <<"\tdp1x/y "<<dp1x<<" "<<dp1y);
      }
  }
  return res;
}


template <class StorageType>
int CornerMatcher::
RefinePyramideNCC(vector<vector<HomgPoint2D> >& ps1,
                  PyramidImage<StorageType>& im1,
                  PyramidImage<StorageType>& im2,
                  unsigned int halfnccwinsize,
                  vector<vector<HomgPoint2D> >& matches,
                  vector<vector<double> >& matchquality)
{
  int res=0;
  unsigned p1[2], p2[2];
  const StorageType **ida1, **ida2;

  for (int lev=im1.size()-2; lev>=0; lev--){
    matches[lev].resize(ps1[lev].size());
    matchquality[lev].resize(ps1[lev].size());
    ida1=(const StorageType **)im1[lev]->GetImageDataArray();
    ida2=(const StorageType **)im2[lev]->GetImageDataArray();
    unsigned maxx=im1[lev]->GetWidth()-halfnccwinsize-1;
    unsigned maxy=im1[lev]->GetHeight()-halfnccwinsize-1;
    for (unsigned i=0; i<ps1[lev].size(); i++){
      p1[0]=(unsigned int)rint(ps1[lev][i][0]);
      p1[1]=(unsigned int)rint(ps1[lev][i][1]);
      p2[0]=((unsigned int)rint(matches[lev+1][i][0]))<<1;
      p2[1]=((unsigned int)rint(matches[lev+1][i][1]))<<1;
      if (p1[0]>halfnccwinsize && p1[0]<maxx &&
          p1[1]>halfnccwinsize && p1[1]<maxy &&
          p2[0]>halfnccwinsize && p2[0]<maxx &&
          p2[1]>halfnccwinsize && p2[1]<maxy &&
          matchquality[lev+1][i] > -1.0){
        NCCSearchEven(p1, p2, ida1, ida2, halfnccwinsize, 3, p2,
                      1.0, matchquality[lev][i]);
      } else {
        matchquality[lev][i]=-5.0;
      }
      matches[lev][i].Set((double)p2[0], (double)p2[1]);
      /*      cerr << lev << " : "<<setw(4)<<i<<" : "<<ps1[lev][i]
           <<"  <-->  "<<matches[lev][i]
           << "\t@ "<<matchquality[lev][i]<<"\t"<<p2[0]<<", "<<p2[1]<<endl;*/
    } // for (ps1it=ps1[lev].begin(); ps1it!=ps1[lev].end(); ps1it++){
  } // for (int lev=small-1; lev>=0; lev--)
  return res;
}

