/* This file is part of the BIAS library (Basic ImageAlgorithmS).

 Copyright (C) 2003-2009    (see file CONTACT for details)
 Multimediale Systeme der Informationsverarbeitung
 Institut fuer Informatik
 Christian-Albrechts-Universitaet Kiel

 BIAS is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 (at your option) any later version.

 BIAS is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with BIAS; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA*/
#include <Base/Common/W32Compat.hh>
#include <math.h>

#include "RegionMatcher.hh"

using namespace BIAS;
using namespace std;

#include "LineMatcher.cpp"
#include "SimilarityMeasure.cpp"
#include <Base/Image/Image.hh>

RegionMatcher::RegionMatcher() :
	Debug() {
	_klthws = _ncchws = _winsize = _nccwinsize = 0;
	_bl1_5.newsize(5, 5);
	_bl2_5.newsize(5, 5);
	_gx1_5.newsize(5, 5);
	_gy1_5.newsize(5, 5);
	_gx2_5.newsize(5, 5);
	_gy2_5.newsize(5, 5);
	_gsx_5.newsize(5, 5);
	_gsy_5.newsize(5, 5);
}

RegionMatcher::~RegionMatcher() {
}

template<class StorageType>
bool RegionMatcher::BilinearRegionColor(const Image<StorageType>& im,
		const double &x, const double &y, const unsigned hws,
		Matrix<double>& res) {
	if (im.GetChannelCount() == 3u)
		return BilinearRegionColor3(im, x, y, hws, res);
	else if (im.GetChannelCount() == 1u)
		return BilinearRegion(im, x, y, hws, res);
	else
	BIASERR("unsupported number of channels: "<<im.GetChannelCount());
	BIASABORT;
	return false;
	// const unsigned w=im.GetWidth(), h=im.GetHeight();
	//   const unsigned channel_count=im.GetChannelCount();
	//   if (channel_count!=1u){
	//     BIASASSERT(im.IsInterleaved());
	//   }
	//   const StorageType **ida = im.GetImageDataArray();
	//   const unsigned size = 2*hws+1;
	//   res.newsize(size, size*channel_count);
	//   unsigned x_floor = (unsigned)floor(x);
	//   unsigned y_floor = (unsigned)floor(y);
	//   unsigned x_floor_end = x_floor+hws+1;
	//   unsigned y_floor_end = y_floor+hws+1;
	//   if (x_floor<hws || y_floor<hws || x_floor_end>=w || y_floor_end>=h)
	//       return false;
	//   unsigned rf, rc, cf, cc, r, c, channel;
	//   double ul, ur, ll, lr;
	//   register const double dy = y - (double)y_floor;
	//   register const double dx = x - (double)x_floor;
	//   register const double dxy = dx*dy;
	//   for (rf=y_floor-hws, rc=rf+1, r=0; rf<y_floor_end; rf++, rc++, r++){
	//     for (cf=x_floor-hws, cc=cf+1, c=0; cf<x_floor_end; cf++, cc++, c++){
	//       for (channel=0; channel<channel_count; channel++){
	//         ul = (double)(ida[rf][cf*channel_count+channel]);
	//         ur = (double)(ida[rf][cc*channel_count+channel]);
	//         ll = (double)(ida[rc][cf*channel_count+channel]);
	//         lr = (double)(ida[rc][cc*channel_count+channel]);
	//         res[r][c*channel_count+channel] =
	//           ul + dy*(ll - ul) + dx*(ur - ul) + dxy*(ul + lr - ll - ur);
	//       }
	//     }
	//   }

	// // check color 3
	//   Matrix<double> res2;
	//   BIASASSERT(BilinearRegionColor3(im, x, y, hws, res2));
	//   for (int my=0; my<size; my++){
	//     for (int mx=0; mx<size*3; mx++){
	//       cout << "["<<my<<"]["<<mx<<"] = "<<res2[my][mx]<<", "<<res[my][mx]<<endl;
	//       BIASASSERT(Equal(res2[my][mx], res[my][mx]));
	//     }
	//   }

	return true;
}

template<class StorageType>
bool RegionMatcher::BilinearRegionColor3(const Image<StorageType>& im,
		const double &x, const double &y, const unsigned hws,
		Matrix<double>& res) {
	BIASASSERT(im.GetChannelCount()==3u);
	BIASASSERT(im.IsInterleaved());
	const int w = im.GetWidth(), h = im.GetHeight();
	const StorageType **ida = im.GetImageDataArray();
	const int size = 2 * hws + 1;
	res.newsize(size, size * 3);
	const int y_floor_start = (int) floor(y) - hws;
	const int y_floor_end = y_floor_start + size;
	const int x_floor_start = ((int) floor(x) - hws) * 3;
	const int x_floor_end = x_floor_start + 3 * size;
	if (x_floor_start < 0 || y_floor_start < 0 || x_floor_end + 3 > 3 * w
			|| y_floor_end >= h)
		return false;
	register const double dy = y - (double) floor(y);
	register const double dx = x - (double) floor(x);
	register const double dxy = dx * dy;
	register int col_floor, col_ceil, row_floor, row_ceil, dst_row, dst_col;
	register double ul, ur, ll, lr;
	for (row_floor = y_floor_start; row_floor < y_floor_end; row_floor++) {
		row_ceil = row_floor + 1;
		dst_row = row_floor - y_floor_start;
		for (col_floor = x_floor_start; col_floor < x_floor_end; col_floor++) {
			dst_col = col_floor - x_floor_start;
			col_ceil = col_floor + 3;
			ul = (double) (ida[row_floor][col_floor]);
			ur = (double) (ida[row_floor][col_ceil]);
			ll = (double) (ida[row_ceil][col_floor]);
			lr = (double) (ida[row_ceil][col_ceil]);
			res[dst_row][dst_col] = ul + dy * (ll - ul) + dx * (ur - ul) + dxy
					* (ul + lr - ll - ur);
		}
	}
	return true;
}

template<class StorageType>
bool RegionMatcher::BilinearRegion(const Image<StorageType>& im,
		const double &x, const double &y, const unsigned hws,
		Matrix<double>& res) {
	BIASASSERT(im.GetChannelCount()==1u);
	const unsigned w = im.GetWidth(), h = im.GetHeight();
	const StorageType **ida = im.GetImageDataArray();
	const unsigned size = 2 * hws + 1;
	res.newsize(size, size);
	unsigned x_floor = (unsigned) floor(x);
	unsigned y_floor = (unsigned) floor(y);
	unsigned x_floor_end = x_floor + hws + 1;
	unsigned y_floor_end = y_floor + hws + 1;
	if (x_floor < hws || y_floor < hws || x_floor_end >= w || y_floor_end >= h)
		return false;
	unsigned rf, rc, cf, cc, r, c;
	double ul, ur, ll, lr;
	register double dy = y - (double) y_floor;
	register double dx = x - (double) x_floor;
	register double dxy = dx * dy;
	for (rf = y_floor - hws, rc = rf + 1, r = 0; rf < y_floor_end; rf++, rc++, r++) {
		for (cf = x_floor - hws, cc = cf + 1, c = 0; cf < x_floor_end; cf++, cc++, c++) {
			ul = (double) (ida[rf][cf]);
			ur = (double) (ida[rf][cc]);
			ll = (double) (ida[rc][cf]);
			lr = (double) (ida[rc][cc]);
			res[r][c] = ul + dy * (ll - ul) + dx * (ur - ul) + dxy * (ul + lr
					- ll - ur);
		}
	}
	return true;
}

template<class StorageType>
bool RegionMatcher::LinearRegionX(const Image<StorageType>& im,
		const double &x, const unsigned &y, const unsigned hws,
		Matrix<double>& res) {
	const unsigned w = im.GetWidth(), h = im.GetHeight();
	//const StorageType **ida = im.GetImageDataArray();
	const unsigned size = 2 * hws + 1;
	res.newsize(size, size);
	unsigned x_floor = (unsigned) floor(x);
	unsigned x_floor_end = x_floor + hws + 1;
	const unsigned maxy = y + hws;
	if (x_floor < hws || y < hws || x_floor_end >= w || maxy >= h)
		return false;
	// const unsigned w=im.GetWidth(), h=im.GetHeight();
	//   const unsigned channel_count=im.GetChannelCount();
	//   if (channel_count!=1u){
	//     BIASASSERT(im.IsInterleaved());
	//   }
	//   const StorageType **ida = im.GetImageDataArray();
	//   const unsigned size = 2*hws+1;
	//   res.newsize(size, size*channel_count);
	//   unsigned x_floor = (unsigned)floor(x);
	//   unsigned y_floor = (unsigned)floor(y);
	//   unsigned x_floor_end = x_floor+hws+1;
	//   unsigned y_floor_end = y_floor+hws+1;
	//   if (x_floor<hws || y_floor<hws || x_floor_end>=w || y_floor_end>=h)
	//       return false;
	//   unsigned rf, rc, cf, cc, r, c, channel;
	//   double ul, ur, ll, lr;
	//   register const double dy = y - (double)y_floor;
	//   register const double dx = x - (double)x_floor;
	//   register const double dxy = dx*dy;
	//   for (rf=y_floor-hws, rc=rf+1, r=0; rf<y_floor_end; rf++, rc++, r++){
	//     for (cf=x_floor-hws, cc=cf+1, c=0; cf<x_floor_end; cf++, cc++, c++){
	//       for (channel=0; channel<channel_count; channel++){
	//         ul = (double)(ida[rf][cf*channel_count+channel]);
	//         ur = (double)(ida[rf][cc*channel_count+channel]);
	//         ll = (double)(ida[rc][cf*channel_count+channel]);
	//         lr = (double)(ida[rc][cc*channel_count+channel]);
	//         res[r][c*channel_count+channel] =
	//           ul + dy*(ll - ul) + dx*(ur - ul) + dxy*(ul + lr - ll - ur);
	//       }
	//     }
	//   }

	// // check color 3
	//   Matrix<double> res2;
	//   BIASASSERT(BilinearRegionColor3(im, x, y, hws, res2));
	//   for (int my=0; my<size; my++){
	//     for (int mx=0; mx<size*3; mx++){
	//       cout << "["<<my<<"]["<<mx<<"] = "<<res2[my][mx]<<", "<<res[my][mx]<<endl;
	//       BIASASSERT(Equal(res2[my][mx], res[my][mx]));
	//     }
	//   }

	return true;
}

template<class StorageType>
int RegionMatcher::NCCSearchOdd(const unsigned int p1[2],
		const unsigned int p2[2], const StorageType **ida1,
		const StorageType **ida2, const unsigned int halfnccwinsize,
		const unsigned int halfsearchwinsize, unsigned int resultpoint[2],
		const double mincorrelation, double& result) const {
	int res = 0; // return value
	unsigned int p3[2], p4[2];
	double tmpresult, maxresult;

	BIASDOUT(D_REGION_MATCHER_SEARCH_ODD, "\np1: "<<p1[0]<<" "<<p1[1]
			<<"\tp2: "<<p2[0]<<" "<<p2[1]<<"\tida1: "<<&ida1[0]
			<<" ida2: "<<&ida2[0]<<" halfnccwinsize: "<<halfnccwinsize);
	// initialize maxresult and point
	resultpoint[0] = p2[0];
	resultpoint[1] = p2[1];
	NCC(p1, p2, ida1, ida2, halfnccwinsize, maxresult);
	BIASDOUT(D_REGION_MATCHER_SEARCH_ODD,"("<<p2[0]<<", "<<p2[1]<<") : "
			<<maxresult);
	if (maxresult > mincorrelation) {
		result = maxresult;
		BIASDOUT(D_REGION_MATCHER_SEARCHNCC, "first correlation succeeded "
				<<result);
	} else {
		BIASDOUT(D_REGION_MATCHER_SEARCHNCC, "first correlation "<<maxresult);
		for (register unsigned int sr = 1; sr <= halfsearchwinsize; sr++) {
			BIASDOUT(D_REGION_MATCHER_SEARCHNCC, "search radius " << sr);
			p3[1] = p2[1] - sr;
			p4[1] = p2[1] + sr;
			for (p3[0] = p2[0] - sr, p4[0] = p2[0] - sr; p3[0] <= p2[0] + sr; p3[0]++, p4[0]++) {
				NCC(p1, p3, ida1, ida2, halfnccwinsize, tmpresult);
				BIASDOUT(D_REGION_MATCHER_SEARCH_ODD,"("<<p3[0]<<", "<<p3[1]<<") : "
						<<tmpresult);
				if (tmpresult > maxresult) {
					maxresult = tmpresult;
					resultpoint[0] = p3[0];
					resultpoint[1] = p3[1];
				}
				if (maxresult > mincorrelation)
					break;

				NCC(p1, p4, ida1, ida2, halfnccwinsize, tmpresult);
				BIASDOUT(D_REGION_MATCHER_SEARCH_ODD,"("<<p4[0]<<", "<<p4[1]<<") : "
						<<tmpresult);
				if (tmpresult > maxresult) {
					maxresult = tmpresult;
					resultpoint[0] = p4[0];
					resultpoint[1] = p4[1];
				}
				if (maxresult > mincorrelation)
					break;
			}
			if (maxresult < result) {
				p3[0] = p2[0] - sr;
				p4[0] = p2[0] + sr;
				for (p3[1] = p2[1] - sr - 1, p4[1] = p2[1] - sr - 1; p3[1]
						<= p2[1] + sr - 1; p3[1]++, p4[1]++) {
					NCC(p1, p3, ida1, ida2, halfnccwinsize, tmpresult);
					BIASDOUT(D_REGION_MATCHER_SEARCH_ODD,"("<<p3[0]<<", "<<p3[1]<<") : "
							<<tmpresult);
					if (tmpresult > maxresult) {
						maxresult = tmpresult;
						resultpoint[0] = p3[0];
						resultpoint[1] = p3[1];
					}
					if (maxresult > mincorrelation)
						break;

					NCC(p1, p4, ida1, ida2, halfnccwinsize, tmpresult);
					BIASDOUT(D_REGION_MATCHER_SEARCH_ODD,"("<<p4[0]<<", "<<p4[1]<<") : "
							<<tmpresult);
					if (tmpresult > maxresult) {
						maxresult = tmpresult;
						resultpoint[0] = p4[0];
						resultpoint[1] = p4[1];
					}
					if (maxresult > mincorrelation)
						break;
				}
			}
			if (maxresult > mincorrelation) {
				result = maxresult;
				BIASDOUT(D_REGION_MATCHER_SEARCHNCC, "succeeded at searchradius "<<sr
						<<" with "<<result<<" correlation point: "<<resultpoint);
				break;
			}
		} // for (sr=1; sr<=halfsearchwinsize; sr++)
		result = maxresult;
		if (mincorrelation < 1.0 && result < mincorrelation) {
			BIASDOUT(D_REGION_MATCHER_SEARCHNCC, "no success within "<<p2[0] <<" "
					<< p2[1]
					<<" +/- "<<halfsearchwinsize<<"  max correlation "<<result
					<<" at "<<resultpoint<< " still lower than "<<mincorrelation);
			res = -3;
		} else if (mincorrelation >= 1.0) {
			BIASDOUT(D_REGION_MATCHER_SEARCHNCC, "searched within "<<p2[0]<<" "
					<<p2[1]
					<<" +/- "<<halfsearchwinsize<<" found max correlation "<<result
					<<" at "<<resultpoint[0]<<", "<<resultpoint[1]);
		}
	} // if (maxresult > result)

	return res;
}

template<class StorageType>
int RegionMatcher::NCCSearchEven(const unsigned int p1[2],
		const unsigned int p2[2], const StorageType **ida1,
		const StorageType **ida2, const unsigned int halfnccwinsize,
		const unsigned int halfsearchwinsize, unsigned int resultpoint[2],
		const double mincorrelation, double& result) const {
	int res = 0; // return value
	unsigned int p3[2], p4[2];
	double tmpresult, maxresult = -1.0;

	/*
	 // initialize maxresult and point
	 resultpoint[0]=p2[0];
	 resultpoint[1]=p2[1];
	 NCC(p1, p2, ida1, ida2, halfnccwinsize, maxresult);
	 BIASDOUT(D_REGION_MATCHER_SEARCH_EVEN,"("<<p2[0]<<", "<<p2[1]<<") : "
	 <<maxresult);
	 if (maxresult > mincorrelation){
	 result = maxresult;
	 BIASDOUT(D_REGION_MATCHER_SEARCHNCC,"first correlation succeeded "
	 <<result);
	 } else {
	 BIASDOUT(D_REGION_MATCHER_SEARCHNCC, "first correlation = "<<maxresult
	 <<"  at "<<p2[0]<<" "<<p2[1]);
	 */
	BIASDOUT(D_REGION_MATCHER_SEARCHNCC, "\np1: "<<p1[0]<<" "<<p1[1]
			<<"\tp2: "<<p2[0]<<" "<<p2[1]<<"\tida1: "<<&ida1[0]
			<<" ida2: "<<&ida2[0]<<" halfnccwinsize: "<<halfnccwinsize);
	for (register unsigned int sr = 1; sr <= halfsearchwinsize; sr++) {
		BIASDOUT(D_REGION_MATCHER_SEARCHNCC, "search radius " << sr);
		// top and bottom side of quadrate
		p3[1] = p2[1] - sr + 1;
		p4[1] = p2[1] + sr;
		for (p3[0] = p2[0] - sr + 1, p4[0] = p2[0] - sr + 1; p3[0] <= p2[0]
				+ sr; p3[0]++, p4[0]++) {
			NCC(p1, p3, ida1, ida2, halfnccwinsize, tmpresult);
			BIASDOUT(D_REGION_MATCHER_SEARCH_EVEN,"("<<p3[0]<<", "<<p3[1]<<") : "
					<<tmpresult);
			if (tmpresult > maxresult) {
				maxresult = tmpresult;
				resultpoint[0] = p3[0];
				resultpoint[1] = p3[1];
			}
			if (maxresult > mincorrelation)
				break;

			NCC(p1, p4, ida1, ida2, halfnccwinsize, tmpresult);
			BIASDOUT(D_REGION_MATCHER_SEARCH_EVEN,"("<<p4[0]<<", "<<p4[1]<<") : "
					<<tmpresult);
			if (tmpresult > maxresult) {
				maxresult = tmpresult;
				resultpoint[0] = p4[0];
				resultpoint[1] = p4[1];
			}
			if (maxresult > mincorrelation)
				break;
		}
		if (maxresult < mincorrelation) {
			p3[0] = p2[0] - sr + 1;
			p4[0] = p2[0] + sr;
			for (p3[1] = p2[1] - sr + 2, p4[1] = p2[1] - sr + 2; p3[1] <= p2[1]
					+ sr - 1; p3[1]++, p4[1]++) {
				NCC(p1, p3, ida1, ida2, halfnccwinsize, tmpresult);
				BIASDOUT(D_REGION_MATCHER_SEARCH_EVEN,"("<<p3[0]<<", "<<p3[1]<<") : "
						<<tmpresult);
				if (tmpresult > maxresult) {
					maxresult = tmpresult;
					resultpoint[0] = p3[0];
					resultpoint[1] = p3[1];
				}
				if (maxresult > mincorrelation)
					break;

				NCC(p1, p4, ida1, ida2, halfnccwinsize, tmpresult);
				BIASDOUT(D_REGION_MATCHER_SEARCH_EVEN,"("<<p4[0]<<", "<<p4[1]<<") : "
						<<tmpresult);
				if (tmpresult > maxresult) {
					maxresult = tmpresult;
					resultpoint[0] = p4[0];
					resultpoint[1] = p4[1];
				}
				if (maxresult > mincorrelation)
					break;
			}
		}
		if (maxresult > mincorrelation) {
			result = maxresult;
			BIASDOUT(D_REGION_MATCHER_SEARCHNCC, "succeeded at searchradius "<<sr
					<<" with "<<result<<" correlation point: "<<resultpoint);
			break;
		}
	} // for (sr=1; sr<=halfsearchwinsize; sr++)
	result = maxresult;
	if (mincorrelation < 1.0 && result < mincorrelation) {
		BIASDOUT(D_REGION_MATCHER_SEARCHNCC,"no success within "<<p2[0]<<" "
				<<p2[1]
				<<" +/- "<<halfsearchwinsize<<"  max correlation "<<result
				<<" at "<<resultpoint<< " still lower than "<<mincorrelation);
		res = -3;
	} else if (mincorrelation >= 1.0) {
		BIASDOUT(D_REGION_MATCHER_SEARCHNCC,"searched within "<<p2[0]<<" "<<p2[1]
				<<" +/- "<<halfsearchwinsize<<" found max correlation "<<result
				<<" at "<<resultpoint[0]<<", "<<resultpoint[1]);
	}
	//  } // if (maxresult > result)

	return res;
}

template<class StorageType>
KLT_TYPE RegionMatcher::_Bilinear(const StorageType **ida, const KLT_TYPE x,
		const KLT_TYPE y) const {
	unsigned int x_floor = (unsigned int) floor(x);
	unsigned int y_floor = (unsigned int) floor(y);
	unsigned int x_ceil = (unsigned int) ceil(x);
	unsigned int y_ceil = (unsigned int) ceil(y);
	StorageType ul = ida[y_floor][x_floor];
	StorageType ur = ida[y_floor][x_ceil];
	StorageType ll = ida[y_ceil][x_floor];
	StorageType lr = ida[y_ceil][x_ceil];
	KLT_TYPE dy = y - y_floor;
	KLT_TYPE dx = x - x_floor;

	return ul + dy * (ll - ul) + dx * (ur - ul) + dy * dx * (ul + lr - ll - ur);
}

KLT_TYPE RegionMatcher::_Bilinear(const unsigned char**ida, const KLT_TYPE x,
		const KLT_TYPE y) const {
	//   unsigned int x_floor=(unsigned int)floor(x);
	//   unsigned int y_floor=(unsigned int)floor(y);
	//   unsigned int x_ceil=(unsigned int)ceil(x);
	//   unsigned int y_ceil=(unsigned int)ceil(y);
	//   int ul = (int)(ida[y_floor][x_floor]);
	//   int ur = (int)(ida[y_floor][x_ceil]);
	//   int ll = (int)(ida[y_ceil][x_floor]);
	//   int lr = (int)(ida[y_ceil][x_ceil]);
	//   KLT_TYPE dy = y - y_floor;
	//   KLT_TYPE dx = x - x_floor;

	//   return ul + dy*(ll - ul) + dx*(ur - ul) + dy*dx*(ul + lr - ll - ur );
	register unsigned int x_floor = (unsigned int) floor(x);
	register unsigned int y_floor = (unsigned int) floor(y);
	register unsigned int x_ceil = x_floor + 1;
	register unsigned int y_ceil = y_floor + 1;
	register unsigned int ul = (unsigned int) ida[y_floor][x_floor];
	register unsigned int ur = (unsigned int) ida[y_floor][x_ceil];
	register unsigned int ll = (unsigned int) ida[y_ceil][x_floor];
	register unsigned int lr = (unsigned int) ida[y_ceil][x_ceil];
	register unsigned int dy = (unsigned int) rint((y - y_floor) * 256);
	register unsigned int dx = (unsigned int) rint((x - x_floor) * 256);

	return (unsigned char) (((ul << 16) + (dy << 8) * (ll - ul) + (dx << 8)
			* (ur - ul) + dy * dx * (ul - ll - ur + lr)) >> 16);
}

template<class StorageType>
void RegionMatcher::_BilinearRegion(const StorageType **ida, const KLT_TYPE x,
		const KLT_TYPE y, const unsigned hws, Matrix<KLT_TYPE>& res) const {
	unsigned x_floor = (unsigned int) floor(x);
	unsigned y_floor = (unsigned int) floor(y);
	unsigned x_floor_end = x_floor + hws + 1;
	unsigned y_floor_end = y_floor + hws + 1;
	unsigned rf, rc, cf, cc, r, c;
	StorageType ul, ur, ll, lr;
	register KLT_TYPE dy = (KLT_TYPE) (y - y_floor);
	register KLT_TYPE dx = (KLT_TYPE) (x - x_floor);
	register KLT_TYPE dxy = dx * dy;
	for (rf = y_floor - hws, rc = rf + 1, r = 0; rf < y_floor_end; rf++, rc++, r++) {
		for (cf = x_floor - hws, cc = cf + 1, c = 0; cf < x_floor_end; cf++, cc++, c++) {
			ul = ida[rf][cf];
			ur = ida[rf][cc];
			ll = ida[rc][cf];
			lr = ida[rc][cc];
			res[r][c] = (KLT_TYPE) (ul + dy * (ll - ul) + dx * (ur - ul) + dxy
					* (ul + lr - ll - ur));
		}
	}
}

void RegionMatcher::_BilinearRegion(const unsigned char **ida,
		const KLT_TYPE x, const KLT_TYPE y, const unsigned hws,
		Matrix<KLT_TYPE>& res) const {
	unsigned int x_floor = (unsigned int) floor(x);
	unsigned int y_floor = (unsigned int) floor(y);
	unsigned int x_floor_end = x_floor + hws + 1;
	unsigned int y_floor_end = y_floor + hws + 1;
	unsigned rf, rc, cf, cc, r, c;
	unsigned ul, ur, ll, lr;
	register unsigned int dy = (unsigned int) rint((y - y_floor) * 256);
	register unsigned int dx = (unsigned int) rint((x - x_floor) * 256);
	register unsigned int dxy = dx * dy;
	for (rf = y_floor - hws, rc = rf + 1, r = 0; rf < y_floor_end; rf++, rc++, r++) {
		for (cf = x_floor - hws, cc = cf + 1, c = 0; cf < x_floor_end; cf++, cc++, c++) {
			ul = (unsigned) ida[rf][cf];
			ur = (unsigned) ida[rf][cc];
			ll = (unsigned) ida[rc][cf];
			lr = (unsigned) ida[rc][cc];
			res[r][c] = (unsigned char) (((ul << 16) + (dy << 8) * (ll - ul)
					+ (dx << 8) * (ur - ul) + dxy * (ul - ll - ur + lr)) >> 16);
		}
	}
	//BIASERR("unfinished and untested");
}

/*
 template <class StorageType>
 KLT_TYPE RegionMatcher::_BilinearGrad(StorageType **ida, KLT_TYPE x, KLT_TYPE y)
 {
 unsigned int x_floor=(unsigned int)floor(x);
 unsigned int y_floor=(unsigned int)floor(y);
 unsigned int x_ceil=(unsigned int)ceil(x);
 unsigned int y_ceil=(unsigned int)ceil(y);
 KLT_TYPE ul = (KLT_TYPE)(ida[y_floor][x_floor])-SOBEL_GRAD_UC_ZERO;
 KLT_TYPE ur = (KLT_TYPE)(ida[y_floor][x_ceil])-SOBEL_GRAD_UC_ZERO;
 KLT_TYPE ll = (KLT_TYPE)(ida[y_ceil][x_floor])-SOBEL_GRAD_UC_ZERO;
 KLT_TYPE lr = (KLT_TYPE)(ida[y_ceil][x_ceil])-SOBEL_GRAD_UC_ZERO;
 KLT_TYPE dy = y - y_floor;
 KLT_TYPE dx = x - x_floor;

 return ul + dy*(ll - ul) + dx*(ur - ul) + dy*dx*(ul + lr - ll - ur );
 }

 // also moves gradient in [-127, 127] befor interpolating

 template <>
 KLT_TYPE RegionMatcher::_BilinearGrad<unsigned char>(unsigned char **ida, KLT_TYPE x, KLT_TYPE y)
 {
 unsigned int x_floor=(unsigned int)floor(x);
 unsigned int y_floor=(unsigned int)floor(y);
 unsigned int x_ceil=(unsigned int)ceil(x);
 unsigned int y_ceil=(unsigned int)ceil(y);
 int ul = (int)(ida[y_floor][x_floor])-SOBEL_GRAD_UC_ZERO;
 int ur = (int)(ida[y_floor][x_ceil])-SOBEL_GRAD_UC_ZERO;
 int ll = (int)(ida[y_ceil][x_floor])-SOBEL_GRAD_UC_ZERO;
 int lr = (int)(ida[y_ceil][x_ceil])-SOBEL_GRAD_UC_ZERO;
 KLT_TYPE dy = y - y_floor;
 KLT_TYPE dx = x - x_floor;

 return ul + dy*(ll - ul) + dx*(ur - ul) + dy*dx*(ul + lr - ll - ur );
 }

 template <class StorageType>
 void RegionMatcher::_BilinearRegionGrad(StorageType **ida, KLT_TYPE x,
 KLT_TYPE y, unsigned hws,
 Matrix<KLT_TYPE>& res)
 {
 unsigned x_floor = (unsigned int)floor(x);
 unsigned y_floor = (unsigned int)floor(y);
 unsigned x_floor_end = x_floor+hws+1;
 unsigned y_floor_end = y_floor+hws+1;
 unsigned rf, rc, cf, cc, r, c;
 KLT_TYPE ul, ur, ll, lr;
 register KLT_TYPE dy = (KLT_TYPE)(y - y_floor);
 register KLT_TYPE dx = (KLT_TYPE)(x - x_floor);
 register KLT_TYPE dxy = dx*dy;
 for (rf=y_floor-hws, rc=rf+1, r=0; rf<y_floor_end; rf++, rc++, r++){
 for (cf=x_floor-hws, cc=cf+1, c=0; cf<x_floor_end; cf++, cc++, c++){
 ul = (KLT_TYPE)ida[rf][cf]-(KLT_TYPE)SOBEL_GRAD_UC_ZERO;
 ur = (KLT_TYPE)ida[rf][cc]-(KLT_TYPE)SOBEL_GRAD_UC_ZERO;
 ll = (KLT_TYPE)ida[rc][cf]-(KLT_TYPE)SOBEL_GRAD_UC_ZERO;
 lr = (KLT_TYPE)ida[rc][cc]-(KLT_TYPE)SOBEL_GRAD_UC_ZERO;
 res[r][c]=(ul + dy*(ll - ul) + dx*(ur - ul) + dxy*(ul + lr - ll - ur));
 }
 }

 }
 */

template<class StorageType>
int RegionMatcher::ParabolaNCC(const unsigned int p1[2],
		const unsigned int p2[2], const StorageType **ida1,
		const StorageType **ida2, const unsigned int halfwinsize,
		KLT_TYPE result[2]) const {
	int res = 0;

	double x1, x2, x3, y1, y2, y3;
	unsigned int p2tmp[2];
	BIASDOUT(D_REGION_MATCHER_PARABOLA, "p1 ("<<p1[0]<<", "<<p1[1]<<")\tp2 ("
			<<p2[0]<<", "<<p2[1]<<")\tida1: "<<&ida1[0]<<" ida2: "<<&ida2[0]
			<<"  halfwinsize: "<<halfwinsize);
	NCC(p1, p2, ida1, ida2, halfwinsize, x2);
	//cerr <<"x2: "<<x2<<endl;
	y2 = x2;
	p2tmp[0] = p2[0];
	p2tmp[1] = p2[1] - 1;
	NCC(p1, p2tmp, ida1, ida2, halfwinsize, y1);
	p2tmp[1] = p2[1] + 1;
	NCC(p1, p2tmp, ida1, ida2, halfwinsize, y3);
	p2tmp[0] = p2[0] - 1;
	p2tmp[1] = p2[1];
	NCC(p1, p2tmp, ida1, ida2, halfwinsize, x1);
	p2tmp[0] = p2[0] + 1;
	NCC(p1, p2tmp, ida1, ida2, halfwinsize, x3);

	if (x3 >= x2 || x1 >= x2) {
		BIASERR("invalid correlation data "<<x1<<" "<<x2
				<<" "<<x3<<"\n\tcorrelation between ("
				<<p1[0]<<", "<<p1[1]<<") and ("<<p2[0]
				<<", "<<p2[1]<<") is not a local maximum");
		return -1;
	}
	if (y3 >= y2 || y1 >= y2) {
		BIASERR("invalid correlation data "<<y1<<" "<<y2
				<<" "<<y3<<"\n\tcorrelation between ("
				<<p1[0]<<", "<<p1[1]<<") and ("<<p2[0]
				<<", "<<p2[1]<<") is not a local maximum");
		return -1;
	}

	result[0] = p2[0] + _ParabolaApprox(x1, x2, x3);
	result[1] = p2[1] + _ParabolaApprox(y1, y2, y3);
	BIASDOUT(D_REGION_MATCHER_PARABOLA, "\n(x1 x2 x3) : res\t("<<x1<<" "<<x2<<" "
			<<x3<<") : "<<result[0]-p2[0]<<"\n(y1 y2 y3) : res\t("
			<<y1<<" "<<y2<<" "<<y3<<") : "<<result[1]-p2[1]);
	return res;
}

template<class StorageType>
int RegionMatcher::ParabolaNCC5(const unsigned int p1[2],
		const unsigned int p2[2], const StorageType **ida1,
		const StorageType **ida2, const unsigned int halfwinsize,
		KLT_TYPE result[2]) const {
	int res = 0;

	double x1, x2, x3, x4, x5, y1, y2, y3, y4, y5;
	unsigned int p2tmp[2];
	BIASDOUT(D_REGION_MATCHER_PARABOLA, "p1 ("<<p1[0]<<", "<<p1[1]<<")\tp2 ("
			<<p2[0]<<", "<<p2[1]<<")\tida1: "<<&ida1[0]<<" ida2: "<<&ida2[0]
			<<"  halfwinsize: "<<halfwinsize);
	NCC(p1, p2, ida1, ida2, halfwinsize, x3);
	//cerr <<"x2: "<<x2<<endl;
	y3 = x3;
	p2tmp[0] = p2[0];
	p2tmp[1] = p2[1] - 1;
	NCC(p1, p2tmp, ida1, ida2, halfwinsize, y2);
	p2tmp[1] = p2[1] - 2;
	NCC(p1, p2tmp, ida1, ida2, halfwinsize, y1);
	p2tmp[1] = p2[1] + 1;
	NCC(p1, p2tmp, ida1, ida2, halfwinsize, y4);
	p2tmp[1] = p2[1] + 2;
	NCC(p1, p2tmp, ida1, ida2, halfwinsize, y5);
	p2tmp[0] = p2[0] - 1;
	p2tmp[1] = p2[1];
	NCC(p1, p2tmp, ida1, ida2, halfwinsize, x2);
	p2tmp[0] = p2[0] - 2;
	NCC(p1, p2tmp, ida1, ida2, halfwinsize, x1);
	p2tmp[0] = p2[0] + 1;
	NCC(p1, p2tmp, ida1, ida2, halfwinsize, x4);
	p2tmp[0] = p2[0] + 2;
	NCC(p1, p2tmp, ida1, ida2, halfwinsize, x5);

	if (x1 >= x3 || x2 >= x3 || x4 >= x3 || x5 >= x3) {
		BIASERR("invalid correlation data "<<x1<<" "<<x2
				<<" "<<x3<<" "<<x4<<" "<<x5<<"\n\tcorrelation between ("
				<<p1[0]<<", "<<p1[1]<<") and ("<<p2[0]
				<<", "<<p2[1]<<") is not a local maximum");
		return -1;
	}
	if (y1 >= y3 || y2 >= y3 || y4 >= y3 || y5 >= y3) {
		BIASERR("invalid correlation data "<<y1<<" "<<y2
				<<" "<<y3<<" "<<y4<<" "<<y5<<"\n\tcorrelation between ("
				<<p1[0]<<", "<<p1[1]<<") and ("<<p2[0]
				<<", "<<p2[1]<<") is not a local maximum");
		return -1;
	}

	result[0] = p2[0] + _ParabolaApprox(x1, x2, x3, x4, x5);
	result[1] = p2[1] + _ParabolaApprox(y1, y2, y3, y4, y5);
	BIASDOUT(D_REGION_MATCHER_PARABOLA,
			"\n(x1 x2 x3 x4 x5) : res\t("<<x1<<" "<<x2<<" "<<x3<<" "<<x4
			<<" "<<x5<<") : "<<result[0]-p2[0]<<"\n(y1 y2 y3 y4 y5) : res\t("
			<<y1<<" "<<y2<<" "<<y3<<" "<<y4<<" "<<y5<<") : "<<result[1]-p2[1]);
	return res;
}

template<class StorageType>
float RegionMatcher::AD(const unsigned int x1, const unsigned int y1, Image<
		StorageType>& img1, const unsigned int x2, const unsigned int y2,
		Image<StorageType>& img2) {
	float res = 0;

	for (unsigned int ch = 0; ch < img1.GetChannelCount(); ch++) {
		res += fabs((float) img1.PixelValue(x1, y1,
				static_cast<unsigned short int> (ch))
				- (float) img2.PixelValue(x2, y2,
						static_cast<unsigned short int> (ch)));
	}

	return res;
}

///////////////////////////////////////////////////////////////////////////
// instantiation
///////////////////////////////////////////////////////////////////////////


#define INSTUC(uctype)\
template BIASMatcher2D_EXPORT void RegionMatcher::NCC<uctype>(const double p1[2],\
  const double p2[2], const uctype **ida1, const uctype **ida2, const unsigned halfwinsize,\
  double& result);\
template BIASMatcher2D_EXPORT void RegionMatcher::NCC<uctype>(\
  const unsigned int p1[2], const unsigned int p2[2], const uctype **ida1, const uctype **roi1,\
  const uctype **ida2, const uctype **roi2, const unsigned hww, const unsigned hwh, double& result)const ;\
template BIASMatcher2D_EXPORT int RegionMatcher::NCCSearchOdd<uctype>(\
  const unsigned int p1[2], const unsigned int p2[2], const uctype **ida1, const uctype **ida2,\
  const unsigned int halfnccwinsize, const unsigned int halfsearchwinsize,\
  unsigned int resultpoint[2], const double mincorrelation, double& result)const ;\
template BIASMatcher2D_EXPORT int RegionMatcher::NCCSearchEven<uctype>(\
  const unsigned int p1[2], const unsigned int p2[2],const uctype **ida1, const uctype **ida2,\
  const unsigned int halfnccwinsize, const unsigned int halfsearchwinsize,\
  unsigned int resultpoint[2], const double mincorrelation, double& result)const ;\
template BIASMatcher2D_EXPORT int RegionMatcher::LineMatcherNCC<uctype>(\
  const unsigned int p1[2], const unsigned int start2[2], const unsigned int end2[2],\
  const uctype **ida1, const uctype **ida2, const uctype **grad1, const uctype **grad2,\
  const unsigned int halfwinsize, const unsigned int epsilon, const double gradientscale,\
  unsigned int result[2], double& correlation)const ;\
template BIASMatcher2D_EXPORT int RegionMatcher::LineMatcherSAD<uctype>(\
  const unsigned int p1[2], const unsigned int start2[2], const unsigned int end2[2],\
  const uctype **ida1, const uctype **ida2, const uctype **grad1, const uctype **grad2,\
  const unsigned int halfwinsize, const unsigned int epsilon, const double gradientscale,\
  unsigned int result[2], double& sad)const ;\
template BIASMatcher2D_EXPORT int RegionMatcher::LineMatcherSSD<uctype>(\
  const unsigned int p1[2], const unsigned int start2[2], const unsigned int end2[2],\
  const uctype **ida1, const uctype **ida2, const uctype **grad1, const uctype **grad2,\
  const unsigned int halfwinsize, const unsigned int epsilon, const double gradientscale,\
  unsigned int result[2], double& ssd)const ;\
template BIASMatcher2D_EXPORT int RegionMatcher::ParabolaNCC(\
  const unsigned int p1[2], const unsigned int p2[2], const uctype **ida1, const uctype **ida2,\
  const unsigned int halfwinsize, KLT_TYPE result[2])const ;\
template BIASMatcher2D_EXPORT int RegionMatcher::ParabolaNCC5(\
  const unsigned int p1[2], const unsigned int p2[2], const uctype **ida1, const uctype **ida2,\
  const unsigned int halfwinsize, KLT_TYPE result[2])const ;\
template BIASMatcher2D_EXPORT void RegionMatcher::ColorNCC<uctype>(\
  const unsigned int p1[2], const unsigned int p2[2], const uctype **ida1, const uctype **ida2,\
  const unsigned int halfwinsize, double& result)const ;\
template BIASMatcher2D_EXPORT void RegionMatcher::CNCC<uctype>(\
  const unsigned int p1[2], const unsigned int p2[2], const uctype **ida1, const uctype **ida2,\
  const unsigned int halfwinsize, double& result) const;
/*
 template BIASMatcher2D_EXPORT void RegionMatcher::HammingDistance<uctype>(\
  const unsigned int p1[2], const unsigned int p2[2], const uctype **ida1, const uctype **ida2,\
  const unsigned int halfwinsize, const unsigned int censussize, double& result) const;*/

#define INST(itype)\
template BIASMatcher2D_EXPORT void RegionMatcher::SSD<itype>(const unsigned int p1[2]\
  , const unsigned int p2[2], const itype **ida1, const itype **ida2, const unsigned int halfwinsize,\
  double& result) const;\
template BIASMatcher2D_EXPORT void RegionMatcher::NCC<itype>(const unsigned int p1[2]\
  , const unsigned int p2[2], const itype **ida1, const itype **ida2, const unsigned int halfwinsize,\
  double& result) const;\
  template BIASMatcher2D_EXPORT void RegionMatcher::SAD<itype>(const unsigned int p1[2]\
  , const unsigned int p2[2], const itype **ida1, const itype **ida2, const unsigned int halfwinsize,\
  double& result) const;\
template BIASMatcher2D_EXPORT void RegionMatcher::SAD<itype>(const unsigned int p1[2]\
 , const unsigned int p2[2], const itype **ida1, const itype **ida2, const unsigned int halfwinsize, const unsigned channel_count, double& result) const; \
template BIASMatcher2D_EXPORT void RegionMatcher::SADN<itype>(\
  const unsigned int p1[2], const unsigned int p2[2], const itype **ida1, const itype **ida2,\
  const unsigned int halfwinsize, double& result) const;

#define INST_IM(itype)\
template BIASMatcher2D_EXPORT bool RegionMatcher::BilinearRegion<itype>(\
  const Image<itype>& im, const double &x, const double &y, \
  const unsigned hws, Matrix<double>& res); \
template BIASMatcher2D_EXPORT bool RegionMatcher::BilinearRegionColor<itype>(\
  const Image<itype>& im, const double &x, const double &y, \
  const unsigned hws, Matrix<double>& res); \
template BIASMatcher2D_EXPORT bool RegionMatcher::BilinearRegionColor3<itype>(\
  const Image<itype>& im, const double &x, const double &y, \
  const unsigned hws, Matrix<double>& res); \
template BIASMatcher2D_EXPORT bool RegionMatcher::LinearRegionX<itype>(\
  const Image<itype>& im, const double &x, const unsigned &y, \
  const unsigned hws, Matrix<double>& res);\
template BIASMatcher2D_EXPORT float RegionMatcher::AD<itype>(const unsigned int x1, const unsigned int y1,\
  Image<itype>& img1, const unsigned int x2, const unsigned int y2, Image<itype>& img2);

// You can't instantiate unsigned char if the
// instantiantion is already given in a specialised implemented function
//INST(unsigned char)
INSTUC(unsigned char)
INSTUC(float)
INSTUC(int)
INSTUC(char)
INSTUC(short)
INSTUC(unsigned short)
INSTUC(unsigned int)
INSTUC(double)
INST(float)
INST(int)
INST(char)
INST(short)
INST(unsigned short)
INST(unsigned int)
INST(double)

INST_IM(unsigned char)
INST_IM(float)
// fill in instances as required
#ifdef BUILD_IMAGE_INT
INST_IM(int)
#endif
#ifdef BUILD_IMAGE_CHAR
INST_IM(char)
#endif
#ifdef BUILD_IMAGE_SHORT
INST_IM(short)
#endif
#ifdef BUILD_IMAGE_USHORT
INST_IM(unsigned short)
#endif
#ifdef BUILD_IMAGE_UINT
INST_IM(unsigned int)
#endif
#ifdef BUILD_IMAGE_DOUBLE
INST_IM(double)
#endif
