/*
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004    (see file CONTACTS for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/



/** @example ExampleInterpolation.cpp
    @relates Interpolator
    @ingroup g_examples    
    @brief   Example for Interpolator
    @author MIP
*/

#include <Base/Common/BIASpragma.hh>

#include <stdlib.h>
#include <vector>

// use MIPMath library
#include "MathAlgo/Interpolator.hh"
#include "Base/Image/Image.hh"
#include "Base/ImageUtils/ImageDraw.hh"
#include "Gui/GuiWX.hh"

using namespace BIAS;
using namespace std;


Interpolator interpolator;
std::vector<double> listT;
std::vector<double> listX;
std::vector<double> listY;
std::vector<Vector2<double> > listControlPoints;
void initDim2()
{
  double t;
  Vector2<double> vec;
  t=0; vec[0]=1; vec[1]=599;
  listT.push_back(t); listControlPoints.push_back(vec);
  t=50; vec[0]=100; vec[1]=450;
  listT.push_back(t);  listControlPoints.push_back(vec);
  t=100;vec[0]=200; vec[1]=400;
  listT.push_back(t);  listControlPoints.push_back(vec);
  t=200; vec[0]=300;vec[1]=400;
  listT.push_back(t);  listControlPoints.push_back(vec);
  t=300;vec[0]=400;vec[1]=400;
  listT.push_back(t);  listControlPoints.push_back(vec);
  t=400;vec[0]=500;vec[1]=200;
  listT.push_back(t);  listControlPoints.push_back(vec);
  t=500;vec[0]=600;vec[1]=100;
  listT.push_back(t);  listControlPoints.push_back(vec);
  t=600;vec[0]=700; vec[1]=100;
  listT.push_back(t);  listControlPoints.push_back(vec);
  t=700;vec[0]=600;vec[1]=200;
  listT.push_back(t);  listControlPoints.push_back(vec);
  t=800;vec[0]=500;vec[1]=400;
  listT.push_back(t);  listControlPoints.push_back(vec);
  t=900;vec[0]=200;vec[1]=200;
  listT.push_back(t);  listControlPoints.push_back(vec);

  interpolator.SetKnotPoints(listT);
  interpolator.SetControlPoints(listControlPoints);
  interpolator.InitSpline();
  interpolator.InitBezier(2);
}

void initDim1()
{
  double t,y;
  t=0; y=100;
  listT.push_back(t); listY.push_back(y);
  t=50; y=100;
  listT.push_back(t); listY.push_back(y);
  t=100; y=100;
  listT.push_back(t); listY.push_back(y);
  t=150; y=500;
  listT.push_back(t); listY.push_back(y);
  t=200;y=500;
  listT.push_back(t); listY.push_back(y);
  t=300;y=450;
  listT.push_back(t); listY.push_back(y);
  t=400;y=410;
  listT.push_back(t); listY.push_back(y);
  t=500;y=100;
  listT.push_back(t); listY.push_back(y);
  t=600;y=200;
  listT.push_back(t); listY.push_back(y);
  t=650;y=100;
  listT.push_back(t); listY.push_back(y);
  t=700;y=350;
  listT.push_back(t); listY.push_back(y);

  /*double x,y;
  x=300.0*0.000; y=1.000*x;
  listT.push_back(x); listY.push_back(y);
  x=300.0*0.138; y=1.000*x;
  listT.push_back(x); listY.push_back(y);
  x=300.0*0.424; y=0.993*x;
  listT.push_back(x); listY.push_back(y);
  x=300.0*0.757; y=0.974*x;
  listT.push_back(x); listY.push_back(y);
  x=300.0*1.045; y=0.941*x;
  listT.push_back(x); listY.push_back(y);
  x=300.0*1.342; y=0.899*x;
  listT.push_back(x); listY.push_back(y);
  x=300.0*1.436; y=0.881*x;
  listT.push_back(x); listY.push_back(y);
  x=300.0*1.652; y=0.828*x;
  listT.push_back(x); listY.push_back(y);
  */
  /*
  double x,y;
  x=350*0.005; y=350*1.000;
  listT.push_back(x); listY.push_back(y);
  x=350*0.092; y=350*1.000;
  listT.push_back(x); listY.push_back(y);
  x=350*0.179; y=350*1.000;
  listT.push_back(x); listY.push_back(y);
  x=350*0.266; y=350*1.003;
  listT.push_back(x); listY.push_back(y);
  x=350*0.354; y=350*1.003;
  listT.push_back(x); listY.push_back(y);
  x=350*0.439; y=350*1.003;
  listT.push_back(x); listY.push_back(y);
  x=350*0.527; y=350*1.003;
  listT.push_back(x); listY.push_back(y);
  x=350*0.615; y=350*1.005;
  listT.push_back(x); listY.push_back(y);
  x=350*0.701; y=350*1.005;
  listT.push_back(x); listY.push_back(y);
  x=350*0.788; y=350*1.005;
  listT.push_back(x); listY.push_back(y);
  x=350*0.875; y=350*1.005;
  listT.push_back(x); listY.push_back(y);
  x=350*0.962; y=350*1.003;
  listT.push_back(x); listY.push_back(y);
  x=350*1.049; y=350*0.998;
  listT.push_back(x); listY.push_back(y);
  x=350*1.135; y=350*0.993;
  listT.push_back(x); listY.push_back(y);
  x=350*1.223; y=350*0.983;
  listT.push_back(x); listY.push_back(y);
  x=350*1.310; y=350*0.966;
  listT.push_back(x); listY.push_back(y);
  x=350*1.398; y=350*0.941;
  listT.push_back(x); listY.push_back(y);
  x=350*1.485; y=350*0.897;
  listT.push_back(x); listY.push_back(y);
  x=350*1.572; y=350*0.828;
  listT.push_back(x); listY.push_back(y);
  x=350*1.660; y=350*0.716;
  listT.push_back(x); listY.push_back(y);
  */
  interpolator.SetKnotPoints(listT);
  interpolator.SetControlPoints(listY);
  interpolator.InitSpline();
  interpolator.InitBezier(1);
}


int main (int argc, char **argv)
{
  double dt;

  GuiWX window;
  char key = '+';
  initDim1();
  Image<unsigned char> img;
  img.Init(800,600);
  img.FillImageWithConstValue((unsigned char)0);
  for (unsigned int i=0;i<listT.size()-1;i++){
    //img.DrawLine(listT[i],600-listY[i],listT[i],600-listY[i],255);
    if(listT[i]>=0 && 600-listY[i]>=0 && listT[i+1]<img.GetWidth() && 600-listY[i+1]<img.GetHeight())
     ImageDraw<unsigned char>::Line(img,
                                    (unsigned int)listT[i],
                                    600-(unsigned int)listY[i],
                                    (unsigned int)listT[i+1],
                                    600-(unsigned int)listY[i+1],
                                    255);
  }
  window.SetTitle(string("Interpolation Test"));
  window.ShowImage(img);
  int x0=0, y0=0, x1=0, y1=0, k=3;
  double pos;
  bool useSpline=true;
  //  Vector2<double> startTang(5,-5),endTang(0,0);
  double startTang=-10000,endTang=-10000;
  cerr<<"Press h for help."<<endl;

  while (key!='q') {
    window.ShowImage(img);
    key=window.WaitForKeyEvent();
    if (key=='s') {
      if (startTang==-10000) {
        startTang=0;
      }
      startTang-=0.1;
       cout<<"startTang:"<<startTang<<endl;
      cout<<"endTang:"<<endTang<<endl;
    }
    if (key=='w') {
      if (startTang==-10000) {
        startTang=0;
      }
      startTang+=0.1;
      cout<<"startTang:"<<startTang<<endl;
      cout<<"endTang:"<<endTang<<endl;
    }
    if (key=='a') {
      startTang=-10000;
      cout<<"startTang:"<<startTang<<endl;
      cout<<"endTang:"<<endTang<<endl;
   }
    if (key=='i') {
      if (endTang==-10000) {
        endTang=0;
      }
      endTang+=0.1;
      cout<<"startTang:"<<startTang<<endl;
      cout<<"endTang:"<<endTang<<endl;
    }
    if (key=='k') {
      if (endTang==-10000) {
        endTang=0;
      }
      endTang-=0.1;
      cout<<"startTang:"<<startTang<<endl;
      cout<<"endTang:"<<endTang<<endl;
    }
    if (key=='j') {
      endTang=-10000;
      cout<<"startTang:"<<startTang<<endl;
      cout<<"endTang:"<<endTang<<endl;
    }
    if (key=='1') k=1;
    if (key=='2') k=2;
    if (key=='3') k=3;
    if (key=='4') k=4;
    if (key=='b') {
      if (useSpline) {
        cout << "showing Bezier" << endl;
        useSpline=false;
      } else {
        cout << "showing Spline" << endl;
        useSpline=true;
      }
    }
    if (key=='h') {
      cerr<<"keys: "<<endl;
      cerr<<"w,s:   change startTangent"<<endl;
      cerr<<"a:     relaxed start condition"<<endl;
      cerr<<"i,k:   change endTangent"<<endl;
      cerr<<"j:     relaxed end condition"<<endl;
      cerr<<"1,2,3: change degree of interpolation"<<endl;
      cerr<<"4:     use akima-spline"<<endl;
      cerr<<"b :    switch Spline and Bezier"<<endl;
      cerr<<"q :    quits"<<endl;
    }

    img.FillImageWithConstValue((unsigned char)0);
    int deg=3;
    interpolator.SetStartTangent(startTang);
    interpolator.SetEndTangent(endTang);
    for (int j=1;j<=2;j++) {
      if (j==1) deg=1;
      else deg=k;
      dt=0;
      for (int i=0;i<700;i++) {
        if (useSpline || j==1) {
          interpolator.Spline(pos,dt,deg);
        } else {
          interpolator.Bezier3(pos,dt);
        }
        x1=(unsigned int)dt;y1=600-(unsigned int)pos;
        //      interpolator.Spline(y,dt,k);
        //cerr<<"dt:"<<dt<<" pos:"<<pos<<endl;
        //y1=600-(int)y;
        //      window.ShowImage(img);
        //key=window.WaitForKeyEvent();
        if (i>0){

          if(x0>=0 && y0>=0 && (unsigned int)x1<img.GetWidth() && (unsigned int)y1<img.GetHeight()&&
              (unsigned int)x0<img.GetWidth() && (unsigned int)y0<img.GetHeight() && x1>0 && y1 >0)
            ImageDraw<unsigned char>::Line(img, x0,y0,x1,y1,j*126);
        }
        x0=x1;
        y0=y1;
        dt+=1;
      }
    }

  }
}





