/*
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003-2009    (see file CONTACT for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __Laguerre_hh__
#define __Laguerre_hh__

#include "bias_config.h" 
#include <complex>
#include <vector>

namespace BIAS{
/**
  @class LaguerreSolver
  @test tested with TestLaguerre.cpp
  @ingroup g_mathalgo
  @brief class encapsulating a laguerre solver for polynomials
*/
class BIASMathAlgo_EXPORT LaguerreSolver
{
  public:
    LaguerreSolver(){_doPolish = true;}
    /**
     @brief compute roots of a polynomial using the laguerre method
     @param[in] coeffs the coefficients in in ascending order
     @param[out] roots the roots of the polynomial
    
     p(x) = coeffs[0] + coeffs[1] * x + ... + coeffs[n]*(x^n)
    
    */
    int Solve(const std::vector< std::complex<double> >& coeffs,
                    std::vector< std::complex<double> >& roots,
                    const double eps = 1.0e-14);
    
    int Solve(const std::vector< double >& coeffs,
              std::vector< std::complex<double> >& roots,
              const double eps = 1.0e-14);
    
    void SetPolish(bool b){_doPolish=b;};
  private:
    //returns number of iterations
    int _Laguerre(const std::vector< std::complex<double> >& coeffs,
                  std::complex<double>& x);
    
    bool _doPolish;
};

} //namespace

#endif
