#include "LevenbergMarquardtHelper.hh"
#include <cstring>

using namespace BIAS;

LevenbergMarquardtHelper::LevenbergMarquardtHelper()
{
  vars_ = NULL;
  numVars_ = numResiduals_ = 0;
  sumOfSquares_ = -1.0;
}

LevenbergMarquardtHelper::
LevenbergMarquardtHelper(int numVars, int numResiduals)
{
  vars_ = NULL;
  Init(numVars, numResiduals);
}

LevenbergMarquardtHelper::~LevenbergMarquardtHelper()
{
  if (vars_ != NULL)
    delete[] vars_;
}

void LevenbergMarquardtHelper::Init(int numVars, int numResiduals)
{
  if (vars_ != NULL)
    delete[] vars_;
  numVars_ = numVars;
  numResiduals_ = numResiduals;
  sumOfSquares_ = -1.0;
  vars_ = new double[numVars];
  memset(vars_, 0, numVars_*sizeof(double));
}

bool LevenbergMarquardtHelper::Update(const double *x, const double *fvec)
{
  double sumsq = 0.0;
  for (register int i = 0; i < numResiduals_; i++)
    sumsq += fvec[i] * fvec[i];
  if (sumOfSquares_ < 0 || sumsq < sumOfSquares_) {
    sumOfSquares_ = sumsq;
    memcpy(vars_, x, numVars_*sizeof(double));
    return true;
  }
  return false;
}
