/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003-2009    (see file CONTACT for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "Median1D.hh"
#include <Base/Debug/LogFacility.hh>
#include <algorithm>

using namespace BIAS;

template<class DataType>
void Median1D<DataType>::Compute(const std::vector<DataType> &vec)
{
  Values_ = vec;
  if (Values_.empty()) {
    BIASERR("Failed to compute median for empty vector!");
    Median_ = DataType(); // undefined
  } else {
    std::sort(Values_.begin(), Values_.end());
    const unsigned int i = (Values_.size() - 1) / 2;
    Median_ = Values_[i];
  }
}

template<class DataType>
DataType Median1D<DataType>::GetQuantile(double p) const
{
  if (Values_.empty()) {
    BIASERR("Failed to compute quantile for empty vector!");
    return DataType(); // undefined
  }
  const double d = std::min(1.0, std::max(0.0, p)) * (Values_.size() - 1);
  const unsigned int i = (unsigned int)d;
  return Values_[i];
}

template<class DataType>
DataType Median1D<DataType>::GetX84(const std::vector<DataType> &vec,
                                    DataType median) const
{
  if (vec.empty()) {
    BIASERR("Failed to compute X84 for empty vector!");
    return DataType(); // undefined
  }
  const unsigned int n = vec.size();
  std::vector<DataType> diff(n);
  for (unsigned int i = 0; i < n; i++)
    diff[i] = (vec[i] > median) ? (vec[i] - median) : (median - vec[i]);
  std::sort(diff.begin(), diff.end());
  return diff[(n - 1) / 2];
}

// instance definitions
template class BIASMathAlgo_EXPORT BIAS::Median1D<float>;
template class BIASMathAlgo_EXPORT BIAS::Median1D<double>;
template class BIASMathAlgo_EXPORT BIAS::Median1D<int>;
template class BIASMathAlgo_EXPORT BIAS::Median1D<long>;
template class BIASMathAlgo_EXPORT BIAS::Median1D<char>;
