/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003-2009    (see file CONTACT for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __Median1D_hh__
#define __Median1D_hh__

#include <bias_config.h>
#include <vector>

namespace BIAS
{
  /** @class Median1D
      @brief Computes the median and p-quantile of a vector.
             The vectors is stored internally and sorted using std::sort.
             Hence this class should be used only with basic data types.
      @note For even vector sizes, the lower median is computed.
      @ingroup g_mathalgo
      @author esquivel 11/2013
  */
  template <class DataType> class BIASMathAlgo_EXPORT Median1D
  {
  public:

    Median1D() {}

    ~Median1D() {}

    /** @brief Compute median and store sorted vector internally. */
    Median1D(const std::vector<DataType> &vec) { Compute(vec); }

    /** @brief Compute median and store sorted vector internally. */
    void Compute(const std::vector<DataType> &vec);

    /** @brief Return computed median. */
    inline DataType GetMedian() const { return Median_; }

    /** @brief Compute and return p-quantile of vector. */
    DataType GetQuantile(double p) const;

    /** @brief Return median of absolute differences to median (X84 rule). */
    inline DataType GetX84() const { return GetX84(Values_, Median_); }

    /** @brief Return median of absolute differences to median (X84 rule)
               for given vector without storing it internally. */
    DataType GetX84(const std::vector<DataType> &vec, DataType median) const;

  protected:

    DataType Median_;
    std::vector<DataType> Values_;

  };

}

#endif // __Median1D_hh__
