/*
 * Nurbs2D.cpp
 *
 *  Created on: Jul 13, 2010
 *      Author: jordt
 */

#include <Base/Debug/Error.hh>
#include "Nurbs1D.hh"
#include <Base/Math/Vector2.hh>
#include <Base/Math/Vector3.hh>
#include <Base/Math/Vector4.hh>
#include <Base/Geometry/HomgPoint2D.hh>
#include <Base/Geometry/HomgPoint3D.hh>
#include <Base/Math/Operators.hh>

#include "NurbsTemplateInit.hh"

using namespace std;
using namespace BIAS;


// This function allocates and initializes the control point- and weight-arrays
template <class ControlPointsT>
Nurbs1D<ControlPointsT>::Nurbs1D(int knotsPerDim, int degree ) : NurbsBase(knotsPerDim,degree,1){

  // Allocate and initialize
  controlPoints_ = new ControlPointsT[numCtrlPoints_];
  weights_ = new double[numCtrlPoints_];

  for (int x = 0; x < numCtrlPoints_; x++){
      TemplateInit::Init(controlPoints_[x]);
      weights_[x] = 1.0;
  }


}


template <class ControlPointsT>
void Nurbs1D<ControlPointsT>::Eval(const double &nurbsParam, ControlPointsT &result) const {

  BIASASSERT(nurbsParam >= 0.0 && nurbsParam <= 1.0)

  // Initialize the return value
  TemplateInit::Init(result);

  // Get the relevant control points
  double weightSum = 0.0;
  int relevantBegin;
  int relevantEnd;
  GetRelevantBaseCtrlPoints(nurbsParam, relevantBegin, relevantEnd);
  double* baseFuncCache = new double[degree_+1];

  // Evaluate the relevant base functions and use them to wheight the
  // control points
  for (int x = relevantBegin; x <= relevantEnd; x++){
      double weight = BaseFunc(0,x,degree_,nurbsParam,baseFuncCache)* weights_[x];
        weightSum += weight;

        result += (weight * controlPoints_[x]);
      }
  if (weightSum != 0.0)
    result = result * (1.0 / weightSum);

  delete[] baseFuncCache;
}


template <class ControlPointsT>
double Nurbs1D<ControlPointsT>::GetKnot(int index) const {
  BIASASSERT(index >= 0 && index <= knotsPerDim_+1)
  return  knotVector_[0][index];
}

template <class ControlPointsT>
void Nurbs1D<ControlPointsT>::SetKnot(int index, double value){
  BIASASSERT(index >= 0 && index <= knotsPerDim_+1)
  knotVector_[0][index] = value;
}

namespace BIAS {

  // Implement NURBS1D for all common types of vectors:
  template class Nurbs1D<double >;
  template class Nurbs1D<Vector<double> >;
  template class Nurbs1D<Vector2<double> >;
  template class Nurbs1D<Vector3<double> >;
  template class Nurbs1D<Vector4<double> >;
  template class Nurbs1D<HomgPoint2D>;
  template class Nurbs1D<HomgPoint3D>;

}

