/*
 * NurbsBase.cpp
 *
 *  Created on: Jul 1, 2010
 *      Author: jordt
 */
#include <Base/Debug/Error.hh>
#include "NurbsBase.hh"
#include "iostream"
#include <cmath>
#include <cstdlib>

using namespace std;
using namespace BIAS;

NurbsBase::NurbsBase(int knotsPerDim, int degree, int dimension){

  knotsPerDim_ = knotsPerDim;
  degree_ = degree;
  vector<double> kVec;
  for(int i = 0; i < degree; i++) kVec.push_back(0.0);
  for(int i = 0; i < knotsPerDim ; i++) kVec.push_back((double)i / (double)(knotsPerDim-1));
  for(int i = 0; i < degree; i++) kVec.push_back(1.0);
  for(int d = 0; d < dimension; d++) knotVector_.push_back(kVec);
  #ifndef NURBS_BASE_THREAD_SAFE
  evalCache_.resize(degree+1);
  #endif //NURBS_BASE_THREAD_SAFE

  numCtrlPoints_ = knotsPerDim+degree_-1;

}

#ifdef NURBS_BASE_RECURSIVE_EVALUATION
// The classic evaluation function
/*
double NurbsBase::BaseFunc(const int &dim, const int &knot, const int &degree, const double &u) const{
  
  if (degree == 0){
    if (u < knotVector_[dim][knot]) return 0.0;
    if (u > knotVector_[dim][knot+1]) return 0.0;
    if (u == knotVector_[dim][knot+1] && u != 1.0) return 0.0;
    return 1.0;
  }
  
  double recursiveA = 0.0;
  if (knotVector_[dim][knot+degree] != knotVector_[dim][knot])
   recursiveA = (u - knotVector_[dim][knot]) / (knotVector_[dim][knot+degree]-knotVector_[dim][knot]) * BaseFunc(dim,knot,degree - 1,u);
  
  double recursiveB  = 0.0;
  if (knotVector_[dim][knot+degree+1] != knotVector_[dim][knot+1])
     recursiveB = (knotVector_[dim][knot+degree+1]-u) / (knotVector_[dim][knot+degree+1]-knotVector_[dim][knot+1]) * BaseFunc(dim,knot+1,degree - 1,u);

  return recursiveA + recursiveB;
}*/
#endif

int NurbsBase::GetNumKnots() const {
  return knotsPerDim_;
}

int NurbsBase::GetDegree() const {
  return degree_;
}

int NurbsBase::GetNumCtrlPts() const {
  return numCtrlPoints_;
}

double NurbsBase::GetKnot(int dim, int index) const {
  BIASASSERT(dim >= 0 && dim < (int)knotVector_.size())
  BIASASSERT(index >= 0 && index <= knotsPerDim_+1)
  return  knotVector_[dim][index];
}

void NurbsBase::SetKnot(int dim, int index, double value){
  BIASASSERT(dim >= 0 && dim < (int)knotVector_.size())
  BIASASSERT(index >= 0 && index <= knotsPerDim_+1)
  knotVector_[dim][index] = value;
}


