#ifndef NURBSTEMPLATEINIT_HH
#define NURBSTEMPLATEINIT_HH

#include <Base/Math/Vector2.hh>
#include <Base/Math/Vector3.hh>
#include <Base/Math/Vector4.hh>
#include <Base/Geometry/HomgPoint2D.hh>
#include <Base/Geometry/HomgPoint3D.hh>

/** Helper class for the Nurbs class set. It Provides init function
  * for possible template types while keeping the code efficient.
  * Add your init function here if you want to use Nurbs with a
  * ControlPoint type not listed here.
  * Please use the most efficient way to init the template type.
  * @author jordt
  */
class TemplateInit{
  public:
  static inline void Init(double &value){
    value = 0.0;
  }
  static inline void Init(BIAS::Vector<double> &value){
    value.Fill(0.0);
  }
  static inline void Init(BIAS::Vector2<double> &value){
    value[0] = 0.0;
    value[1] = 0.0;
  }
  static inline void Init(BIAS::Vector3<double> &value){
    value[0] = 0.0;
    value[1] = 0.0;
    value[2] = 0.0;
  }
  static inline void Init(BIAS::Vector4<double> &value){
    value[0] = 0.0;
    value[1] = 0.0;
    value[2] = 0.0;
    value[3] = 0.0;
  }
  static inline void Init(BIAS::HomgPoint2D &value){
    value[0] = 0.0;
    value[1] = 0.0;
    value[2] = 1.0;
  }
  static inline void Init(BIAS::HomgPoint3D &value){
    value[0] = 0.0;
    value[1] = 0.0;
    value[2] = 0.0;
    value[3] = 1.0;
  }
};


#endif // NURBSTEMPLATEINIT_HH
