/*
 * NurbsSurface.hh
 *
 *  Created on: Mar 8, 2010
 *      Author: bruenger
 */

#ifndef NURBSSURFACE_HH_
#define NURBSSURFACE_HH_

#include <vector>
#include <Base/Math/Vector2.hh>
#include <Base/Geometry/HomgPoint3D.hh>

namespace BIAS {

class NurbsSurface {
public:
	NurbsSurface();
	virtual ~NurbsSurface();
	void Init(int dimu, int dimv, int degree);
	void SetKnotsToDefault();
	void SetCTRLPointsToRandom(Vector2<double> start, Vector2<double> ende);
	Vector3<double> GetSurfacePoint(double u, double v);
#ifdef BIAS_HAVE_XML2
	int LoadFromXML(std::string xmlfile);
	void LoadFromXMLWeighted(std::string xmlfile,float weight);
	void SaveToXML(std::string xmlfile);
#endif

	void SetCtrlPoint(int i, int j, HomgPoint3D val)
	{
		ctrlpoints_[i][j] = val;
	}

	HomgPoint3D GetCtrlPoint(int i, int j)
	{
		return ctrlpoints_[i][j];
	}

	void SetWeight(int i, int j, double val)
	{
		HomgPoint3D tmp;
		tmp.Set(ctrlpoints_[i][j].GetX(),ctrlpoints_[i][j].GetY(),ctrlpoints_[i][j].GetZ(),val);
		SetCtrlPoint(i,j,tmp);
	}

	double GetWeight(int i, int j)
	{
		return ctrlpoints_[i][j].GetW();
	}

	void SetKnotU(int i, double val)
	{
		knotsu_[i] = val;
	}

	double GetKnotU(int i)
	{
		return knotsu_[i];
	}

	void SetKnotV(int i, double val)
	{
		knotsv_[i] = val;
	}

	double GetKnotV(int i)
	{
		return knotsv_[i];
	}

	std::vector<double> GetKnotsU() const
	{
		return knotsu_;
	}

	std::vector<double> GetKnotsV() const
	{
		return knotsv_;
	}

	int GetDimU() const
	{
		return dimu_;
	}

	void SetDimU(int dimu)
	{
		dimu_ = dimu;
	}

	int GetDimV() const
	{
		return dimv_;
	}

	void SetDimV(int dimv)
	{
		dimv_ = dimv;
	}

	int GetDegree() const
	{
		return degree_;
	}

	void SetDegree(int degree)
	{
		degree_ = degree;
	}

	bool GetInitialized() const
	{
		return initialized_;
	}

	void SetInitialized(bool initialized)
	{
		initialized_ = initialized;
	}

protected:
	int FindSpan(int n, int p, double u, std::vector<double> U);
	std::vector<double> BasisFuns(int i, double u, int p, std::vector<double> U);
	std::vector<std::vector<HomgPoint3D> > ctrlpoints_;
	std::vector<double> knotsu_;
	std::vector<double> knotsv_;
	int dimu_;
	int dimv_;
	int degree_;
	bool initialized_;
};

}
#endif /* NURBSSURFACE_HH_ */
