/*
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004    (see file CONTACTS for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include <bias_config.h>

#include <vector>
#include <iostream>
#include <math.h>

using namespace std;

namespace BIAS
{
  /** @class StandardDeviation
      @ingroup g_mathalgo
      @brief Computes the standard deviation and mean of given values.
      @author MIP
  */
  //template <class T>
  class BIASMathAlgo_EXPORT StandardDeviation
  {
    public:

      StandardDeviation();

      /** @brief Create statistics from given values. */
      StandardDeviation(vector<double> &values);

      /** @brief Compute median of internally stored values. */
      double CalcAverage();

      /** @brief Compute standard deviation of internally sorted values. */
      double CalcStandardDeviation();

      /** @brief Add a value to the vector of internally stored values. */
      void AddValue(double v);

      /** @brief Clear the internally stored values. */
      void Clear();

    private:

      /** @brief Stores values to compute statistics for internally. */
      vector<double> val;

  };

}
