/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003-2009    (see file CONTACT for details)
Multimediale Systeme der Informationsverarbeitung
Institut fuer Informatik
Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

//#include <Base/Common/W32Compat.hh>

#ifdef WIN32
#  define _CRT_SECURE_NO_DEPRECATE // vs8
#  pragma warning (disable: 4251)  // STL MAP DLL warning noise (JW)
#endif

#include "CScommClient.hh"

#ifdef WIN32
#  include <winsock.h>
//#  include <winsock2.h>
#else //WIN32
#  include <sys/poll.h>
#  include <sys/socket.h>
#  include <netinet/in.h>
#  include <netdb.h>
#  include <unistd.h>
#  include <netinet/tcp.h>
#endif //WIN32

#include <errno.h>
#include <cstdio>
#include <cstring>

#include <Base/Debug/Error.hh>

using namespace std;
using namespace BIAS;

CScommClient::CScommClient(bool threaded) : CScommBase(threaded)
{
  isServer_=false;
  char hostName[255];
  if (gethostname(hostName,255)==0)
    name_=hostName;
}

int CScommClient::ConnectServer(string serverName, unsigned int port,
                                unsigned int timeOut)
{  
  struct hostent *roothostent;
  struct sockaddr_in control_addr;
  unsigned int counter = 0;
  int socketFd;

  roothostent = gethostbyname(serverName.c_str());
  if (roothostent == NULL) {
    BIASERR("Host not found: "<<serverName);
    return -2;
  }
  connectPort_=port;
  memset(&control_addr,0 , sizeof(struct sockaddr_in));
  control_addr.sin_family = AF_INET;
  memcpy((char *) &control_addr.sin_addr.s_addr,
    (char *) roothostent->h_addr,
    roothostent->h_length);
  control_addr.sin_port = htons((u_short) connectPort_);
  socketFd = (int)socket(AF_INET, SOCK_STREAM, 0);

  bool * flag = new bool(true);
  int length = sizeof(bool);
  setsockopt(socketFd, IPPROTO_TCP, TCP_NODELAY, (char *) flag, length); 

  if (verbose_)    
    logOut_<<"Trying to connect with server "<<serverName<<endl;

  int res=-1;
  while ( (res<0)  &&  (counter<(timeOut/100)) ) {
    control_addr.sin_port = htons((u_short) connectPort_);
    if (log_)    
      logOut_<<"Trying to connect with server "<<serverName<<" on port "<<connectPort_<<endl;
    res=connect(socketFd,(struct sockaddr *)&control_addr,
      sizeof(control_addr));
    counter++;
    if ( (res!=0) &&  portChange_ ) {
      connectPort_++;
      if ((counter % 10) ==0) connectPort_=D_CS_DEFAULT_PORT;
    }
#ifdef WIN32
    Sleep(1);
#else
    usleep (100);
#endif
  }
  if (counter>=(timeOut/100)) {
    BIASERR("Time out connecting to server "<<serverName);
    return -1;
  }

  res = AcceptConn_(socketFd);
  // all channels are busy
  if (res <0) {
    BIASERR("Error creating receive thread for client "<<name_);
    return -4;
  }
  commPartners_[res].name = serverName;
  if (verbose_)
    verboseOut_<<"Successfully connected to server "<<serverName<<" on port "<<connectPort_<<endl;

  //// send client name
  char firstMsg[255];
  sprintf(firstMsg, "[CLIENTNAME] %s",name_.c_str());
  BIASDOUT(D_CS_CONNECT, "sending \""<<firstMsg<<"\"");
#ifdef WIN32
  res = send(commPartners_[res].fd, firstMsg, (int)strlen(firstMsg)+1, 0);
#else
# ifdef __APPLE__
  res = send(commPartners_[res].fd, &firstMsg, strlen(firstMsg)+1, 0);
# else 
  res = send(commPartners_[res].fd, &firstMsg, strlen(firstMsg)+1, MSG_NOSIGNAL);
# endif
#endif
  if (res <= 0) {
    BIASERR("Failed to send client name to server ('"<<strerror(errno)<<"')!");
    return -3;
  } else if (verbose_) {
    verboseOut_<<"Send client name to server"<<endl;
  }
  return 0;
}
