/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003-2009    (see file CONTACT for details)
Multimediale Systeme der Informationsverarbeitung
Institut fuer Informatik
Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __BIASCScommClient__
#define __BIASCScommClient__

#include "CScommBase.hh"


namespace BIAS {
  /**
    @class CScommClient
    @ingroup network
		@brief class for sending/receiving data between clients and servers
	  
		Only registered msgs will be accepted at the server side,
		similar the client will accept only registered msgs from the server too.
		The client needs to know the hostname of the server. 
		Usage: <br>
	
		-# Konstruktor: CScommClient client;
	
		-# Register msgs:  client.RegisterMsg(msgName, BIAS::CS_FLOAT, 3);
	
		-# connect server:  client.ConnectServer("visu1");
	
		-# send and/or get Data:  client.SendMsg(msgName,vecOfFloats);<br>
		client.GetData(msgName, newVecOffloats);
	
		See the examples also.
		@author Daniel Grest, Jan. 2004
  */
  class BIASNetworkComm_EXPORT CScommClient : public CScommBase {
  public:

    /** Set threaded to false for deactivating the receive-thread. Data will be
    read from TCP-buffer only when GetData methods are called. Threaded
    receive is activated by default!
    Beware: if there's no msg in tcp-buffer, GetData will BLOCK until there
    is any new msg (this need not be the one GetData was called for,
    for example binary instead of int)
    if receive-buffer is completly filled, send blocks until msg
    can be delivered!
    */
    CScommClient(bool threaded = true);
    ~CScommClient() {}

    /** give the client a name, default ist $HOSTNAME
    */
    void SetName(std::string clientName) { name_=clientName;}

    /** for a client: tries to connect to server with hostname serverName.    
    @param timeOut in ms
    @param serverName hostName of the server as ip or name
    @return 0 on success, -1 on timeOut, < -1 on other errors
    */
    int ConnectServer(std::string serverName,unsigned int port=D_CS_DEFAULT_PORT,
      unsigned int timeOut=20000);

  };
}

#endif
