#ifndef __UDPSERVER__
#define __UDPSERVER__

#include <bias_config.h>

#include "UDPServerClientDefinitions.hh"
#include <Base/Debug/Debug.hh>


#ifdef WIN32
#include <winsock.h>
#else
#include <netinet/in.h>
#endif


#define D_UDP_SEND 0x00000001 
namespace BIAS {


/** 
  @class UDPServer
  @brief UDP server class
	@ingroup network
  @author Ingo Thomsen
  */
class BIASNetworkComm_EXPORT UDPServer : public BIAS::Debug {

public:

  /**
     Constructor, 
     @author Ingo Thomsen
  */
  UDPServer();
  /** open a UDP socket to the given host:port
      @author evers */
  int Connect(const char* ipaddress_or_name, int port, bool UseTCP = false);

  /** open a UDP socket to the given port
      @author Ingo Schiller */
  int Connect(int port, bool UseTCP = false);

  int Disconnect();

  /** 
      Send a message
      @author Ingo Thomsen
      @param message A char pointer to the beginning of the message
      @param message_size The Size of the message
      @return -1 in any case of failure, otherwise the number of 
      UDP packages successfully sent
  */
  int Send (const char *message, const size_t message_size);

	  /** 
      Send a message to a given host:port (UDP only)
      @author mfranke
      @param message A char pointer to the beginning of the message
      @param message_size The size of the message
			@param ipAddress The host ip address
			@param port The host port
      @return -1 in any case of failure, otherwise the number of 
      UDP packages successfully sent
  */
  int SendToUDP (const char* message, const size_t message_size,
		             const char* ipAddress, int port);
 
  /** 
      Destructor
      @author Ingo Thomsen
  */      
  ~UDPServer();


  void SetThrottle(const unsigned int t) {Throttle_ = t;}
  unsigned int GetMessageSize(){return MessageSize_;}

protected:

  // socket address
  struct sockaddr_in UDPSockAddr_;
  struct sockaddr TCPSockAddr_;
  // descriptor of the UDP socket
  int socket_descriptor_;

  // send buffer
  char* send_buffer_;

  // ring counter for messages sent, maximimum defined in 
  // UDPServerClientDefinitions.hh, included by UPDServer.cc
  int message_counter_;

  // to prevent UDP-packet collision on the receiver side, use Throttle_
  // (in ms) to sleep between packets
  unsigned int Throttle_; 
  // total size of messagte including all header
  unsigned int MessageSize_;

  bool TCPinsteadOfUDP_;

  // this buffer contains struct ImageHeader + data
  char *ImgBuffer_;
  // and has this size
  unsigned int ImgBufferSize_;


}; // class UDPServer



} // namespace BIAS
#endif
