/////////////////////////////////////////////////////////////////////////////////////////
//
// This file contains utility functions, that are needed by several opencl algorithms.
//
// author: smahmens
// date: 03/2013
//
/////////////////////////////////////////////////////////////////////////////////////////



//***************************************************************************************
//
// This functions packs 16 one channel unsigned char images
// into one four channel unsigned int image
//
//***************************************************************************************

inline uint4 PackUchar16ToUint4(uchar16 unpackedVal)
{
  uint4 packedVal;

  uint val;
  uchar* valp = (uchar*)&val;
  valp[0] = (uchar) unpackedVal.s0;
  valp[1] = (uchar) unpackedVal.s1;
  valp[2] = (uchar) unpackedVal.s2;
  valp[3] = (uchar) unpackedVal.s3;
  packedVal.x = val;

  valp[0] = (uchar) unpackedVal.s4;
  valp[1] = (uchar) unpackedVal.s5;
  valp[2] = (uchar) unpackedVal.s6;
  valp[3] = (uchar) unpackedVal.s7;
  packedVal.y = val;

  valp[0] = (uchar) unpackedVal.s8;
  valp[1] = (uchar) unpackedVal.s9;
  valp[2] = (uchar) unpackedVal.sa;
  valp[3] = (uchar) unpackedVal.sb;
  packedVal.z = val;

  valp[0] = (uchar) unpackedVal.sc;
  valp[1] = (uchar) unpackedVal.sd;
  valp[2] = (uchar) unpackedVal.se;
  valp[3] = (uchar) unpackedVal.sf;
  packedVal.w = val;

  return packedVal;
}


//***************************************************************************************
//
// This functions unpacks 1 four channel unsigned int image
// into 16 one channel unsigned char images
//
//***************************************************************************************

inline uchar16 UnPackUint4ToUchar16(uint4 packedVal)
{
  uchar16 imgs;

  uint v = packedVal.x;
  uchar* ptr = (uchar*) &v;
  imgs.s0 = (uchar) ptr[0];
  imgs.s1 = (uchar) ptr[1];
  imgs.s2 = (uchar) ptr[2];
  imgs.s3 = (uchar) ptr[3];

  v = packedVal.y;
  imgs.s4 = (uchar) ptr[0];
  imgs.s5 = (uchar) ptr[1];
  imgs.s6 = (uchar) ptr[2];
  imgs.s7 = (uchar) ptr[3];

  v = packedVal.z;
  imgs.s8 = (uchar) ptr[0];
  imgs.s9 = (uchar) ptr[1];
  imgs.sa = (uchar) ptr[2];
  imgs.sb = (uchar) ptr[3];

  v = packedVal.w;
  imgs.sc = (uchar) ptr[0];
  imgs.sd = (uchar) ptr[1];
  imgs.se = (uchar) ptr[2];
  imgs.sf = (uchar) ptr[3];

  return imgs;
}

//*****************************************************************************
//
// Initialize Buffer with value v
//
//*****************************************************************************

__kernel void InitializeBuffer(__global float* buffer, float value) {
  int pos = get_global_id(0);

  buffer[pos] = value;
}

__kernel void ConvertFloatImageToUcharImage(__read_only image2d_t imageFloat, __write_only image2d_t imageUchar, float scale) {

const sampler_t smp = CLK_NORMALIZED_COORDS_FALSE | CLK_ADDRESS_CLAMP_TO_EDGE | CLK_FILTER_NEAREST;
  int2 xy;
  xy.x = get_global_id(0);
  xy.y = get_global_id(1);

  uint4 ival = convert_uint4(max(0.0f,min(255.0f,read_imagef(imageFloat,smp,xy) * scale)));

  write_imageui(imageUchar, xy, ival);
}

//*****************************************************************************
//
// copies data from buffer to an image
//
//*****************************************************************************
__kernel void BufferToImage(__global float* buffer, __write_only image2d_t image, int bufferWidth) {

  int x = get_global_id(0);
  int y = get_global_id(1);

  int pos = 3 * (y * bufferWidth + x);

  write_imagef(image, (int2) (x, y), (float4) (buffer[pos],  buffer[pos + 1],  buffer[pos + 2], 0));
}

__kernel void OneChannelBufferToImage(__global float* buffer, __write_only image2d_t image, float scale) {

  int x = get_global_id(0);
  int y = get_global_id(1);

int width = get_image_width(image);

  int pos = (y * width + x);

  write_imagef(image, (int2) (x, y), buffer[pos]*scale);
}

__kernel void ShiftVecToImage(__global float* buffer, __write_only image2d_t image, int bufferWidth) {

  int x = get_global_id(0);
  int y = get_global_id(1);

  int pos = 3 * (y * bufferWidth + x);
  float shiftX = buffer[pos + 1];
 // float shiftY = buffer[pos + 2];

  write_imagef(image, (int2) (x, y), (float4) (shiftX, shiftX, shiftX, shiftX));
}

__kernel void ShiftVecToImage2D(__global float* buffer, __write_only image2d_t image) {

  int x = get_global_id(0);
  int y = get_global_id(1);

  int pos = 3 * (y * get_image_width(image) + x);
  float shiftX = buffer[pos + 1];
  float shiftY = buffer[pos + 2];

  write_imagef(image, (int2) (x, y), (float4) (shiftX, shiftY, shiftX, shiftY));
}
