

////////////////////////////
__kernel void Primal
(
__read_only image2d_t Y, // dual variable (4 values)
__read_only image2d_t X0, // original flow
__read_only image2d_t X, // optical flow (4 values) (X1,X2,C,0)
__read_only image2d_t G, // G is evaluated for X0, 4 channels float image with values: gradRo0X, gradRo0Y, gradLength2, Ro0
__read_only image2d_t Q, // dual brightness compensation
__write_only image2d_t newX,
float tau,
float lambda, // smoothness
float gamma // brightness compensation
)
{

const sampler_t smp = CLK_NORMALIZED_COORDS_FALSE | CLK_ADDRESS_CLAMP_TO_EDGE | CLK_FILTER_NEAREST;

int2 xy;
xy.x = get_global_id( 0 );
xy.y = get_global_id( 1 );

int2 lxy;
lxy.x = get_local_id( 0 );
lxy.y = get_local_id( 1 );

__local float8 values[17][17];

float8 yval;
yval.s0123 = read_imagef(Y,smp,xy);
yval.s45 = read_imagef(Q,smp,xy).xy;

values[lxy.x+1][lxy.y+1] = yval;
if (lxy.x == 0) {

float8 tmpVal;
tmpVal.s0123 = read_imagef(Y,smp,(int2) (xy.x-1,xy.y));
tmpVal.s45 = read_imagef(Q,smp,(int2) (xy.x-1,xy.y)).xy;

values[lxy.x][lxy.y+1] = tmpVal;
}

if (lxy.y == 0) {
float8 tmpVal;
tmpVal.s0123 = read_imagef(Y,smp,(int2) (xy.x,xy.y-1));
tmpVal.s45 = read_imagef(Q,smp,(int2) (xy.x,xy.y-1)).xy;

values[lxy.x+1][lxy.y] = tmpVal;
}

barrier(CLK_LOCAL_MEM_FENCE);

int width = get_image_width(newX);
int height = get_image_height(newX);

 if (xy.x >=  width || xy.y >= height)
 {
 return;
 }

 //--------------------------------------

float3 xval = read_imagef(X,smp, xy).xyz;
float3 x0val = read_imagef(X0,smp, xy).xyz;
float8 yvalnx = values[lxy.x][lxy.y+1];
float8 yvalny = values[lxy.x+1][lxy.y];

float4 gradVals = read_imagef(G,smp, xy);

float arg1 = (yvalnx.s0-yval.s0);
float arg2 = (yvalny.s1-yval.s1);
float arg3 = (yvalnx.s2-yval.s2);
float arg4 = (yvalny.s3-yval.s3);
float arg5 = (yvalnx.s4-yval.s4);
float arg6 = (yvalny.s5-yval.s5);

if (xy.x == 0) {
arg1 = -yval.s0;
arg3 = -yval.s2;
arg5 = -yval.s4;
}

if (xy.y == 0) {
arg2 = -yval.s1;
arg4 = -yval.s3;
arg6 = -yval.s5;
}

if (xy.x == width) {
arg1 = yvalnx.s0;
arg3 = yvalnx.s2;
arg5 = yvalnx.s4;
}

if (xy.y == height) {
arg2 = yvalny.s1;
arg4 = yvalny.s3;
arg6 = yvalny.s5;
}

float3 oldXval = xval;

xval.x =  xval.x - tau * (arg1 + arg2);
xval.y =  xval.y - tau * (arg3 + arg4);
xval.z =  xval.z - tau * (arg5 + arg6);

//-------------


float3 delta = xval - x0val;

float tau_lambda = tau*lambda;
float tau_lambda_gamma = tau_lambda*gamma;
float thr = tau_lambda*gradVals.z;
float roX0 = gradVals.w; // diff warped I1 - I0
float roX1 = roX0 + dot(delta.xy,gradVals.xy)+gamma*xval.z;

float2 deltaGrad = tau_lambda*gradVals.xy;
if (roX1< (-thr))
{
xval.xy += deltaGrad;
xval.z += tau_lambda_gamma;
} else if (roX1> thr)
{
xval.xy -= deltaGrad;
xval.z -= tau_lambda_gamma;
} else if (fabs(roX1) <= thr)
{
float rij = roX1/gradVals.z;
xval.xy -= (gradVals.xy*rij);
xval.z -= rij*gamma;

}

/////////////

xval = 2*xval - oldXval;

write_imagef(newX, xy, (float4) (xval.x,xval.y,xval.z,0));


}

///////////////////////////////
__kernel void Dual
(
__read_only image2d_t X, // X is opticalflow (2 values)
__read_only image2d_t Y,
__read_only image2d_t Q,
__write_only image2d_t newY,// (4 values)
__write_only image2d_t newQ, // (2 values)
float sigma
)
{

const sampler_t smp = CLK_NORMALIZED_COORDS_FALSE | CLK_ADDRESS_CLAMP_TO_EDGE | CLK_FILTER_NEAREST;

int2 xy;
xy.x = get_global_id( 0 );
xy.y = get_global_id( 1 );

int2 lxy;
lxy.x = get_local_id( 0 );
lxy.y = get_local_id( 1 );

__local float3 values[17][17];

float4 xval = read_imagef(X,smp,xy);

values[lxy.x][lxy.y] = xval.xyz;
if (lxy.x == 0) {
values[lxy.x+16][lxy.y] = read_imagef(X,smp,(int2) (xy.x+16,xy.y)).xyz;
}

if (lxy.y == 0) {
values[lxy.x][lxy.y+16] = read_imagef(X,smp,(int2) (xy.x,xy.y+16)).xyz;
}

barrier(CLK_LOCAL_MEM_FENCE);

int width = get_image_width(newY);
int height = get_image_height(newY);

 if (xy.x >= width || xy.y >= height)
 {
 return;
 }

//--------------------------------------
float4 yval = read_imagef(Y,smp, xy);
float2 qval = read_imagef(Q,smp, xy).xy;
float3 xvalnx = values[lxy.x+1][lxy.y];
float3 xvalny = values[lxy.x][lxy.y+1];

float4 args1;
float2 args2;

args1.s0 = xvalnx.x - xval.x;
args1.s1 = xvalny.x - xval.x;
args1.s2 = xvalnx.y - xval.y;
args1.s3 = xvalny.y - xval.y;
args2.s0 = xvalnx.z - xval.z;
args2.s1 = xvalny.z - xval.z;


if (xy.x == width) {
args1.s0 = 0;
args1.s2 = 0;
args2.s0 = 0;
}

if (xy.y == height) {
args1.s1 = 0;
args1.s3 = 0;
args2.s1 = 0;
}

yval += sigma*args1;
qval += sigma*args2;

/////////////

// project

float lengthy1 = fast_length(yval);
float lengthy2 = fast_length(qval.xy);
float denom1 = max(1.0f,lengthy1);
float denom2 = max(1.0f,lengthy2);
yval /= denom1;
qval.xy /= denom2;

write_imagef(newY, xy, yval);
write_imagef(newQ, xy, (float4) (qval.x, qval.y, 0,0));

}

__kernel void Warp
(
__read_only image2d_t X, // X is opticalflow
__read_only image2d_t I, // I0, I1, 0, 0
__write_only image2d_t WarpedI
)
{
const sampler_t smp = CLK_NORMALIZED_COORDS_FALSE | CLK_ADDRESS_CLAMP_TO_EDGE | CLK_FILTER_NEAREST;
const sampler_t fsmp = CLK_NORMALIZED_COORDS_FALSE | CLK_ADDRESS_CLAMP_TO_EDGE | CLK_FILTER_LINEAR;

int2 xy;
xy.x = get_global_id( 0 );
xy.y = get_global_id( 1 );

float2 xval = read_imagef(X,smp,xy).xy;

float width = (float) get_image_width(X);
float height = (float) get_image_height(X);

float2 coords = (float2)((xy.x+0.5f+xval.x),(xy.y+0.5f+xval.y));

if (get_image_channel_data_type(I) == CLK_FLOAT) {
  float wival = read_imagef(I,fsmp,coords).y; // warped I1
  float ival = read_imagef(I,smp,xy).x; // I0


  // we have (warpedI1,warpedI1 - I0,0,0)
  write_imagef(WarpedI, xy, (float4) (wival,wival-ival,0,0));
  } else {

  // TO DO calc diff

  uint4 ival = read_imageui(I,fsmp,coords);

  write_imageui(WarpedI, xy, ival);
  }

}

///////////////////////////////
// gradient is calculated for channel 0
__kernel void Gradient
(
__read_only image2d_t Input, // (warpedI1,warpedI1-I0,0,0)
__write_only image2d_t Gradient, //4 channels float image with values: gradX, gradY, gradLength2, warpedI1-I0
float gamma // brightness compensation
)
{

const sampler_t smp = CLK_NORMALIZED_COORDS_FALSE | CLK_ADDRESS_CLAMP_TO_EDGE | CLK_FILTER_NEAREST;

int2 xy;
xy.x = get_global_id( 0 );
xy.y = get_global_id( 1 );

int2 lxy;
lxy.x = get_local_id( 0 );
lxy.y = get_local_id( 1 );

__local float values[18][18];

int channelType = get_image_channel_data_type(Input);

float2 ival;
if (channelType ==CLK_FLOAT) {
ival = read_imagef(Input,smp,xy).xy;
} else {
uint2 tmpVal =read_imageui(Input,smp,xy).xy;
ival = (float2) (tmpVal.x,tmpVal.y);
}

values[lxy.x+1][lxy.y+1] = ival.x;
if (lxy.x == 0) {
if (channelType ==CLK_FLOAT) {
values[lxy.x][lxy.y+1] = read_imagef(Input,smp,(int2) (xy.x-1,xy.y)).x;
values[lxy.x+17][lxy.y+1] = read_imagef(Input,smp,(int2) (xy.x+16,xy.y)).x;
} else {
values[lxy.x][lxy.y+1] = read_imageui(Input,smp,(int2) (xy.x-1,xy.y)).x;
values[lxy.x+17][lxy.y+1] = read_imageui(Input,smp,(int2) (xy.x+16,xy.y)).x;
}
}

if (lxy.y == 0) {
if (channelType ==CLK_FLOAT) {
values[lxy.x+1][lxy.y] = read_imagef(Input,smp,(int2) (xy.x,xy.y-1)).x;
values[lxy.x+1][lxy.y+17] = read_imagef(Input,smp,(int2) (xy.x,xy.y+16)).x;
} else {
values[lxy.x+1][lxy.y] = read_imageui(Input,smp,(int2) (xy.x,xy.y-1)).x;
values[lxy.x+1][lxy.y+17] = read_imageui(Input,smp,(int2) (xy.x,xy.y+16)).x;
}
}

barrier(CLK_LOCAL_MEM_FENCE);

int width = get_image_width(Gradient);
int height = get_image_height(Gradient);

 if (xy.x >= width || xy.y >= height)
 {
 return;
 }

//--------------------------------------
float valpx = values[lxy.x][lxy.y+1];
float valnx = values[lxy.x+2][lxy.y+1];
float valny = values[lxy.x+1][lxy.y+2];
float valpy = values[lxy.x+1][lxy.y];

float4 result;
result.x =  (valnx - valpx)/2.0f;
result.y =  (valny - valpy)/2.0f;



if (xy.x == 0 || xy.x == width - 1) {
result.x = 0;
}

if (xy.y == 0 || xy.y == height - 1) {
result.y = 0;
}


result.z = dot(result.xy,result.xy) + gamma*gamma;
result.w = ival.y; // save difference (warpedI1 - I0)


write_imagef(Gradient, xy, result);


}

__kernel void DifferenceAbsImage(__read_only image2d_t imA, __read_only image2d_t imB,__write_only image2d_t dst)
{
const sampler_t smp = CLK_NORMALIZED_COORDS_FALSE | CLK_ADDRESS_CLAMP_TO_EDGE | CLK_FILTER_NEAREST;

  int2 coo = (int2)(get_global_id(0),get_global_id(1));
  float4 valA;
  float4 valB;

   if (get_image_channel_data_type(imA) == CLK_FLOAT) {
   valA = read_imagef(imA, smp, coo);
   } else {
   uint4 tmpVal = read_imageui(imA, smp, coo);
   valA.x = (float)tmpVal.x;
   valA.y = (float)tmpVal.y;
   valA.z = (float)tmpVal.z;
   valA.w = (float)tmpVal.w;
   }

   if (get_image_channel_data_type(imB) == CLK_FLOAT) {
   valB = read_imagef(imB, smp, coo);
   } else {
   uint4 tmpVal = read_imageui(imB, smp, coo);
   valB.x = (float)tmpVal.x;
   valB.y = (float)tmpVal.y;
   valB.z = (float)tmpVal.z;
   valB.w = (float)tmpVal.w;
   }

float4 result = fabs(valA  - valB);
write_imagef(dst, coo, result);
}

// sum 2 channels and replace the third one
__kernel void Sum(__read_only image2d_t imA, __read_only image2d_t imB,__write_only image2d_t dst)
{
const sampler_t smp = CLK_NORMALIZED_COORDS_FALSE | CLK_ADDRESS_CLAMP_TO_EDGE | CLK_FILTER_NEAREST;

  int2 coo = (int2)(get_global_id(0),get_global_id(1));
  float4 valA;
  float4 valB;

   if (get_image_channel_data_type(imA) == CLK_FLOAT) {
   valA = read_imagef(imA, smp, coo);
   } else {
   uint4 tmpVal = read_imageui(imA, smp, coo);
   valA.x = (float)tmpVal.x;
   valA.y = (float)tmpVal.y;
   valA.z = (float)tmpVal.z;
   valA.w = (float)tmpVal.w;
   }

   if (get_image_channel_data_type(imB) == CLK_FLOAT) {
   valB = read_imagef(imB, smp, coo);
   } else {
   uint4 tmpVal = read_imageui(imB, smp, coo);
   valB.x = (float)tmpVal.x;
   valB.y = (float)tmpVal.y;
   valB.z = (float)tmpVal.z;
   valB.w = (float)tmpVal.w;
   }

float4 result = valA  + valB;
result.z = valA.z;
result.w = 0;
write_imagef(dst, coo, result);
}


__constant float weightedmask[5] =
    { 1.0f / 12.0f, -8.0f / 12.0f, 0.0, 8.0f / 12.0f, -1.0f / 12.0f };


__kernel void WeightedGradX(__read_only image2d_t src,
                       __write_only image2d_t dst)
{
const sampler_t smp = CLK_NORMALIZED_COORDS_FALSE | CLK_ADDRESS_CLAMP_TO_EDGE | CLK_FILTER_NEAREST;
   //int w = 5;
   int wBy2 = 2;

    __local float4 P[16+4][16];

    //Identification of this workgroup
   int i = get_group_id(0);
   int j = get_group_id(1);

   //Identification of work-item
   int idX = get_local_id(0);
   int idY = get_local_id(1);

   int ii = i*16 + idX; // == get_global_id(0);
   int jj = j*16 + idY; // == get_global_id(1);

   int2 coords = (int2)(ii, jj);

   //Reads pixels
   P[idX+wBy2][idY] = read_imagef(src, smp, coords);

   //Needs to read extra elements for the filter in the borders
   if (idX < wBy2)
   {
      coords.x = ii - wBy2;
      P[idX][idY] = read_imagef(src, smp, coords);
      coords.x = ii + 16;
      P[idX+16+wBy2][idY] = read_imagef(src, smp, coords);
   }

   barrier(CLK_LOCAL_MEM_FENCE);

if (get_global_id(0) >= get_image_width(src) || get_global_id(1) >= get_image_height(src))
{
return;
}

   //Computes convolution
   float4 convPix =
      P[idX+0][idY] * weightedmask[0]
      + P[idX+1][idY] * weightedmask[1]
      + P[idX+2][idY] * weightedmask[2]
      + P[idX+3][idY] * weightedmask[3]
      + P[idX+4][idY] * weightedmask[4];

   coords = (int2)(XM*(ii), jj);
   write_imagef(dst, coords, convPix);
}

__kernel void WeightedGradY(__read_only image2d_t src,
                       __read_only image2d_t GradX,
                       __write_only image2d_t dst,
                       float gamma)
{
const sampler_t smp = CLK_NORMALIZED_COORDS_FALSE | CLK_ADDRESS_CLAMP_TO_EDGE | CLK_FILTER_NEAREST;
   //int w = 5;
   int wBy2 = 2; //w divided by 2

    __local float4 P[16+4][16];

    //Identification of this workgroup
   int i = get_group_id(0);
   int j = get_group_id(1);

   //Identification of work-item
   int idX = get_local_id(0);
   int idY = get_local_id(1);

   int ii = i*16 + idX; // == get_global_id(0);
   int jj = j*16 + idY; // == get_global_id(1);

   int2 coords = (int2)(ii, jj);

   //Reads pixels
   P[idY+wBy2][idX] = read_imagef(src, smp, coords);

   //Needs to read extra elements for the filter in the borders
   if (idY < wBy2)
   {
      coords.y = jj - wBy2;
      P[idY][idX] = read_imagef(src, smp, coords);
      coords.y = jj + 16;
      P[idY+16+wBy2][idX] = read_imagef(src, smp, coords);
   }
   barrier(CLK_LOCAL_MEM_FENCE);
   if (get_global_id(0) >= get_image_width(src) || get_global_id(1) >= get_image_height(src))
   {
   return;
   }

   //Computes convolution
   float4 convPix =
        P[idY+0][idX] * weightedmask[0]
      + P[idY+1][idX] * weightedmask[1]
      + P[idY+2][idX] * weightedmask[2]
      + P[idY+3][idX] * weightedmask[3]
      + P[idY+4][idX] * weightedmask[4];

   convPix.y = convPix.x;
   convPix.x = read_imagef(GradX, smp, (int2)(ii, jj)).x;

   convPix.z = convPix.x*convPix.x+convPix.y*convPix.y+gamma*gamma;
   convPix.w = P[idY+wBy2][idX].y;

   coords = (int2)(XM*(ii), jj);
   write_imagef(dst, coords, convPix);
}

// combine 2 images to one channel image in which first channel is
// image 1 in grey and second image is second image in grey
// dst image has to be float!!
__kernel void Combine(__read_only image2d_t imA, __read_only image2d_t imB,__write_only image2d_t dst)
{
  const sampler_t smp = CLK_NORMALIZED_COORDS_FALSE | CLK_ADDRESS_CLAMP_TO_EDGE | CLK_FILTER_NEAREST;

  int2 coo = (int2)(get_global_id(0),get_global_id(1));  
  float valA,valB;

  if (get_image_channel_data_type(imA) == CLK_FLOAT) {
     float4 tmpVal = read_imagef(imA, smp, coo);
     valA = 0.299*tmpVal.x + 0.587*tmpVal.y + 0.114*tmpVal.z;
  } else {
     uint4 tmpVal = read_imageui(imA, smp, coo);
     valA = 0.299*(float)tmpVal.x + 0.587*(float)tmpVal.y + 0.114*(float)tmpVal.z;
  }

  if (get_image_channel_data_type(imB) == CLK_FLOAT) {
    float4 tmpVal = read_imagef(imB, smp, coo);
    valB = 0.299*tmpVal.x + 0.587*tmpVal.y + 0.114*tmpVal.z;
  } else {
    uint4 tmpVal = read_imageui(imB, smp, coo);
    valB = 0.299*(float)tmpVal.x + 0.587*(float)tmpVal.y + 0.114*(float)tmpVal.z;
  }

  float4 result;
  result.x = valA/255.0;
  result.y = valB/255.0;
  result.z = 0;
  result.w = 0;
  write_imagef(dst, coo, result);
}
