__constant sampler_t smp = CLK_NORMALIZED_COORDS_FALSE | CLK_ADDRESS_CLAMP_TO_EDGE | CLK_FILTER_NEAREST;

#define BLOCK_SIZE 16

__kernel void DifferenceImage(__read_only image2d_t imA, __read_only image2d_t imB, __write_only image2d_t dst)
{
  int2 coo = (int2)(get_global_id(0),get_global_id(1));

float4 valA;
float4 valB;

 if (get_image_channel_data_type(imA) == CLK_FLOAT) {
valA = read_imagef(imA, smp, coo);
} else {
uint4 tmpVal = read_imageui(imA, smp, coo);
valA.x = (float)tmpVal.x;
valA.y = (float)tmpVal.y;
valA.z = (float)tmpVal.z;
valA.w = (float)tmpVal.w;
}

 if (get_image_channel_data_type(imB) == CLK_FLOAT) {
valB = read_imagef(imB, smp, coo);
} else {
uint4 tmpVal = read_imageui(imB, smp, coo);
valB.x = (float)tmpVal.x;
valB.y = (float)tmpVal.y;
valB.z = (float)tmpVal.z;
valB.w = (float)tmpVal.w;
}
  float4 result = valA  - valB;
  coo.x *= XM;
  write_imagef(dst, coo, result);
}

__kernel void DifferenceAbsImage(__read_only image2d_t imA, __read_only image2d_t imB,__write_only image2d_t dst)
{
  int2 coo = (int2)(get_global_id(0),get_global_id(1));
  float4 valA;
  float4 valB;

   if (get_image_channel_data_type(imA) == CLK_FLOAT) {
  valA = read_imagef(imA, smp, coo);
  } else {
  uint4 tmpVal = read_imageui(imA, smp, coo);
  valA.x = (float)tmpVal.x;
  valA.y = (float)tmpVal.y;
  valA.z = (float)tmpVal.z;
  valA.w = (float)tmpVal.w;
  }

   if (get_image_channel_data_type(imB) == CLK_FLOAT) {
  valB = read_imagef(imB, smp, coo);
  } else {
  uint4 tmpVal = read_imageui(imB, smp, coo);
  valB.x = (float)tmpVal.x;
  valB.y = (float)tmpVal.y;
  valB.z = (float)tmpVal.z;
  valB.w = (float)tmpVal.w;
  }

float4 result = fabs(valA  - valB);
  coo.x *= XM;
  write_imagef(dst, coo, result);
}

__kernel void DifferenceZeroLimitImage(__read_only image2d_t imA, __read_only image2d_t imB,  __write_only image2d_t dst)
{
  int2 coo = (int2)(get_global_id(0),get_global_id(1));
  float4 result = read_imagef(imA, smp, coo) - read_imagef(imB, smp, coo);
  result = max((float4)(0.0,0.0,0.0,0.0), result);
  coo.x *= XM;
  write_imagef(dst, coo, result);
}

__kernel void CutOffMinThreshold(__read_only image2d_t imA, __write_only image2d_t dst, float minCutoff)
{
  int2 coo = (int2)(get_global_id(0),get_global_id(1));
  float4 result = read_imagef(imA, smp, coo);
  result = max((float4)(minCutoff), result);
  coo.x *= XM;
  write_imagef(dst, coo, result);
}


