__constant sampler_t smpf = CLK_NORMALIZED_COORDS_TRUE |
CLK_ADDRESS_CLAMP_TO_EDGE |
CLK_FILTER_LINEAR;
__constant sampler_t smp = CLK_NORMALIZED_COORDS_TRUE |
CLK_ADDRESS_CLAMP_TO_EDGE |
CLK_FILTER_NEAREST;


__kernel void rescaleImage(__read_only image2d_t input,
                             __write_only image2d_t output,
                             float colorScale,
                             int flip, int flop)
{
int2 xy;
xy.x = get_global_id( 0 ); // x index
xy.y = get_global_id( 1 ); // y index

int new_width = get_image_width(output);
int new_height = get_image_height(output);

int2 tmpXY;

tmpXY.x = abs((new_width-1)*flop - xy.x);
tmpXY.y = abs((new_height-1)*flip - xy.y);

int old_width = get_image_width(input);
int old_height = get_image_height(input);

float2 new_xy;
new_xy.x = ((float) tmpXY.x +0.5f) / (float) new_width;
new_xy.y = ((float) tmpXY.y + 0.5f)/ (float) new_height;

float4 color = read_imagef(input, smpf, new_xy)*colorScale;

xy.x *= XM;
write_imagef(output, xy, color);

}

__kernel void rescaleAndMergeImages(__read_only image2d_t input1,
                                    __read_only image2d_t input2,
                                    __write_only image2d_t output,
                                    float alpha,
                                    float colorScale1,
                                    float colorScale2,
                                    int flip1, int flop1,
                                    int flip2, int flop2)
{
int2 xy;
xy.x = get_global_id( 0 ); // x index
xy.y = get_global_id( 1 ); // y index

int new_width = get_image_width(output);
int new_height = get_image_height(output);

int2 tmpXY1;

tmpXY1.x = abs((new_width-1)*flop1 - xy.x);
tmpXY1.y = abs((new_height-1)*flip1 - xy.y);

int2 tmpXY2;

tmpXY2.x = abs((new_width-1)*flop2 - xy.x);
tmpXY2.y = abs((new_height-1)*flip2 - xy.y);

float2 new_xy1;
new_xy1.x = ((float) tmpXY1.x +0.5f) / (float) new_width;
new_xy1.y = ((float) tmpXY1.y + 0.5f)/ (float) new_height;

float2 new_xy2;
new_xy2.x = ((float) tmpXY2.x +0.5f) / (float) new_width;
new_xy2.y = ((float) tmpXY2.y + 0.5f)/ (float) new_height;

float4 color1 = read_imagef(input1, smpf, new_xy1)*colorScale1;
float4 color2 = read_imagef(input2, smpf, new_xy2)*colorScale2;

xy.x *= XM;
write_imagef(output, xy, (1-alpha)*color1+alpha*color2);

}


__kernel void MergeImages(__read_only image2d_t input1,
                          __read_only image2d_t input2,
                          __write_only image2d_t output,
                          float alpha,
                          float colorScale1,
                          float colorScale2,
                          int flip1, int flop1,
                          int flip2, int flop2)
{
int2 xy;
xy.x = get_global_id( 0 ); // x index
xy.y = get_global_id( 1 ); // y index

int width = get_image_width(output);
int height = get_image_height(output);

int2 tmpXY1;

tmpXY1.x = abs((width-1)*flop1 - xy.x);
tmpXY1.y = abs((height-1)*flip1 - xy.y);

int2 tmpXY2;

tmpXY2.x = abs((width-1)*flop2 - xy.x);
tmpXY2.y = abs((height-1)*flip2 - xy.y);

float2 new_xy1;
new_xy1.x = ((float) tmpXY1.x +0.5f) ;
new_xy1.y = ((float) tmpXY1.y + 0.5f);

float2 new_xy2;
new_xy2.x = ((float) tmpXY2.x +0.5f);
new_xy2.y = ((float) tmpXY2.y + 0.5f);

float4 color1 = read_imagef(input1, smpf, new_xy1)*colorScale1;
float4 color2 = read_imagef(input2, smpf, new_xy2)*colorScale2;

xy.x *= XM;
write_imagef(output, xy, (1-alpha)*color1+alpha*color2);

}
