__kernel void findNearestPairs(__global const float *modelpoints, __global const float *datapoints, __global float *output, int len)
{
  int workid = get_global_id( 0 );

  int vtx = workid * 7; // 3 pos, 4 color
  int otx = workid * 7; // 3 pos, 3 pos, 1 dist

  output[otx+0] = datapoints[vtx+0];
  output[otx+1] = datapoints[vtx+1];
  output[otx+2] = datapoints[vtx+2];

  int minidx = 0;
  float mindist = 100000000.0;
  float curdist = 0;
  float comp[3];
  for (int i=0;i<len*7;i+=7) 
  {
    comp[0] = datapoints[vtx+0] - modelpoints[i+0];
    comp[1] = datapoints[vtx+1] - modelpoints[i+1];
    comp[2] = datapoints[vtx+2] - modelpoints[i+2];
    curdist = comp[0] * comp[0] + comp[1] * comp[1] + comp[2] * comp[2];
    if (curdist < mindist) {
      mindist = curdist;
      minidx = i;
    }
  }
  output[otx+3] = modelpoints[minidx+0];
  output[otx+4] = modelpoints[minidx+1];
  output[otx+5] = modelpoints[minidx+2];
	
  output[otx+6] = mindist;
}

__kernel void movePointCloud(__global float *datapoints, __global const float *tmat)
{
  int workid = get_global_id( 0 );
  int vtx = workid * 7; // 3 pos, 4 color

  float result[3] = { 0,0,0 };
  float vertex[3];
  vertex[0] = datapoints[vtx+0] - tmat[12];
  vertex[1] = datapoints[vtx+1] - tmat[13];
  vertex[2] = datapoints[vtx+2] - tmat[14];
  for (int row = 0; row < 3;row++) {
      for (int col = 0;col < 3;col++) {
	result[row] += tmat[row*3+col] * vertex[col];
      }
  }
//  result[0] = vertex[0];
//  result[1] = vertex[1];
//  result[2] = vertex[2];
  datapoints[vtx+0] = result[0]+tmat[9]  +tmat[12];
  datapoints[vtx+1] = result[1]+tmat[10] +tmat[13];
  datapoints[vtx+2] = result[2]+tmat[11] +tmat[14];
}







