__constant sampler_t smp = CLK_NORMALIZED_COORDS_FALSE | CLK_ADDRESS_CLAMP_TO_EDGE | CLK_FILTER_NEAREST;
__constant sampler_t smpf = CLK_NORMALIZED_COORDS_TRUE | CLK_ADDRESS_CLAMP_TO_EDGE | CLK_FILTER_LINEAR;

#define INVALIDVALUE 256.0
#define BLOCK_SIZE 16

inline void readToLocal(int idX, int idY, int ii, int jj, int wBy2, const image2d_t src, const sampler_t smp, __local float4 P[BLOCK_SIZE+14][BLOCK_SIZE+14]) {
   //Reads pixels
   int2 coords = (int2)(ii, jj);
   P[idX+wBy2][idY+wBy2] = read_imagef(src, smp, coords);

   //Needs to read extra elements for the filter in the borders
   if (idX < wBy2)
   {
      coords.x = ii - wBy2; coords.y = jj;
      P[idX][idY+wBy2] = read_imagef(src, smp, coords);
      coords.x = ii + BLOCK_SIZE; coords.y = jj;
      P[idX+BLOCK_SIZE+wBy2][idY+wBy2] = read_imagef(src, smp, coords);
   }

   if (idY < wBy2)
   {
      coords.x = ii; coords.y = jj - wBy2;
      P[idX+wBy2][idY] = read_imagef(src, smp, coords);
      coords.x = ii; coords.y = jj + BLOCK_SIZE;
      P[idX+wBy2][idY+BLOCK_SIZE+wBy2] = read_imagef(src, smp, coords);
   }
   if (idX < wBy2 && idY < wBy2)
   {
      // upper left corner
   	  coords.x = ii - wBy2; coords.y = jj - wBy2;
      P[idX][idY] = read_imagef(src, smp, coords);
      // upper right corner
   	  coords.x = ii + BLOCK_SIZE; coords.y = jj - wBy2;
      P[idX+BLOCK_SIZE+wBy2][idY] = read_imagef(src, smp, coords);
      // lower left corner
   	  coords.x = ii - wBy2; coords.y = jj + BLOCK_SIZE;
      P[idX][idY+BLOCK_SIZE+wBy2] = read_imagef(src, smp, coords);
      // lower right corner
   	  coords.x = ii + BLOCK_SIZE; coords.y = jj + BLOCK_SIZE;
      P[idX+BLOCK_SIZE+wBy2][idY+BLOCK_SIZE+wBy2] = read_imagef(src, smp, coords);
   }
}

__kernel void ColPackGreyToRGBA(__read_only image2d_t src, __write_only image2d_t dst)
{
  int cooX = get_global_id(0)*4;
  int cooY = get_global_id(1);
  
  float4 color;
  color.x = read_imagef(src, smp, (int2)(cooX+0, cooY)).x; 
  color.y = read_imagef(src, smp, (int2)(cooX+1, cooY)).x; 
  color.z = read_imagef(src, smp, (int2)(cooX+2, cooY)).x; 
  color.w = read_imagef(src, smp, (int2)(cooX+3, cooY)).x;
  
  cooX /= 4;
  cooX *= XM;
  write_imagef(dst, (int2)(cooX, cooY), color);
}
__kernel void ColUnPackRGBAToGrey(__read_only image2d_t src, __write_only image2d_t dst)
{
  int cooX = get_global_id(0);
  int cooY = get_global_id(1);
  
  float4 color = read_imagef(src, smp, (int2)(cooX, cooY));
  
  cooX *= 4;
  cooX *= XM;
  write_imagef(dst, (int2)(cooX+0*XM, cooY), (float4)(color.x));
  write_imagef(dst, (int2)(cooX+1*XM, cooY), (float4)(color.y));
  write_imagef(dst, (int2)(cooX+2*XM, cooY), (float4)(color.z));
  write_imagef(dst, (int2)(cooX+3*XM, cooY), (float4)(color.w));
}

__kernel void downsample2x(__read_only image2d_t src, __write_only image2d_t dst)
{
  int w = get_image_width(src);
  int h = get_image_height(src);
  
  int2 coords;
  coords.x = get_global_id(0);
  coords.y = get_global_id(1);
  float2 srcCoo = (float2)( (2.0f*(float)coords.x+1.0f) / (float)w, (2.0f*coords.y+1.0f) / (float)h);
  float4 color = read_imagef(src, smpf, srcCoo);
  coords.x *= XM;
  write_imagef(dst, coords, color);
}

__kernel void upsample2x(__read_only image2d_t src, __write_only image2d_t dst)
{
  int w = get_image_width(src);
  int h = get_image_height(src);
  
  int2 coords;
  coords.x = get_global_id(0);
  coords.y = get_global_id(1);
  float2 srcCoo = (float2)( (0.5f*(float)coords.x) / (float)w, (0.5f*coords.y) / (float)h);
  float4 color = read_imagef(src, smpf, srcCoo);
  int2 d=get_image_dim(dst);
  if (coords.x<d.x && coords.y<d.y) {
    coords.x *= XM;
    write_imagef(dst, coords, color);
  }
}

__kernel void downsample4x(__read_only image2d_t src, __write_only image2d_t dst)
{
  int w = get_image_width(src);
  int h = get_image_height(src);
  
  int2 coords;
  coords.x = get_global_id(0);
  coords.y = get_global_id(1);
  float4 color = read_imagef(src, smpf, (float2)( (4.0f*(float)coords.x+1.0f) / (float)w, (4.0f*coords.y+1.0f) / (float)h));
  color += read_imagef(src, smpf, (float2)( (4.0f*(float)coords.x+3.0f) / (float)w, (4.0f*coords.y+1.0f) / (float)h));
  color += read_imagef(src, smpf, (float2)( (4.0f*(float)coords.x+1.0f) / (float)w, (4.0f*coords.y+3.0f) / (float)h));
  color += read_imagef(src, smpf, (float2)( (4.0f*(float)coords.x+3.0f) / (float)w, (4.0f*coords.y+3.0f) / (float)h));
  coords.x *= XM;
  write_imagef(dst, coords, 0.25*color);
}

__kernel void upsample4x(__read_only image2d_t src, __write_only image2d_t dst)
{
  int w = get_image_width(src);
  int h = get_image_height(src);
  
  int2 coords;
  coords.x = get_global_id(0);
  coords.y = get_global_id(1);
  float2 srcCoo = (float2)( (0.25f*(float)coords.x) / (float)w, (0.25f*coords.y) / (float)h);
  float4 color = read_imagef(src, smpf, srcCoo);
  int2 d=get_image_dim(dst);
  if (coords.x<d.x && coords.y<d.y) {
    coords.x *= XM;
    write_imagef(dst, coords, color);
  }
}

// col-packs grey image
__kernel void scaledExpInv(__read_only image2d_t src, 
					       __write_only image2d_t dst,
					       float gradscale, float scale)
{
  int xcoo = get_global_id(0)*4;
  int ycoo = get_global_id(1);
  float4 pix;
  pix.x = scale * exp(-gradscale * read_imagef(src, smp, (int2)(xcoo+0, ycoo)).z);
  pix.y = scale * exp(-gradscale * read_imagef(src, smp, (int2)(xcoo+1, ycoo)).z);
  pix.z = scale * exp(-gradscale * read_imagef(src, smp, (int2)(xcoo+2, ycoo)).z);
  pix.w = scale * exp(-gradscale * read_imagef(src, smp, (int2)(xcoo+3, ycoo)).z);
  xcoo/=4;
  xcoo*=XM;
  write_imagef(dst, (int2)(xcoo, ycoo), pix);
}

// col-packs grey image
__kernel void scaleShift(__read_only image2d_t srcColor,
  __write_only image2d_t colorScaled,
  float colorScale,
  float colorMin)
{
  int xcoo = get_global_id(0)*4;
  int ycoo = get_global_id(1);
  float4 col;
  col.x = colorScale * (read_imagef(srcColor, smp, (int2)(xcoo+0, ycoo)).x - colorMin);
  col.y = colorScale * (read_imagef(srcColor, smp, (int2)(xcoo+1, ycoo)).x - colorMin);
  col.z = colorScale * (read_imagef(srcColor, smp, (int2)(xcoo+2, ycoo)).x - colorMin);
  col.w = colorScale * (read_imagef(srcColor, smp, (int2)(xcoo+3, ycoo)).x - colorMin);
  xcoo /= 4;
  xcoo *= XM;
  write_imagef(colorScaled, (int2)(xcoo, ycoo), col);
}

__kernel void levelDistance(int level,
  __read_only image2d_t srcColor,
  __write_only image2d_t dists,
  float sigma)
{
  int2 coords;
  coords.x = get_global_id(0);
  coords.y = get_global_id(1);

  float4 col = read_imagef(srcColor, smp, coords);
  float4 dist = (float4)level - col;
  dist = dist*dist;

  float fn = (1.0f/sigma)* 0.5f * M_SQRT1_2_F * M_2_SQRTPI_F;
  float fe = -0.5f * ( 1.0f/sigma ) * ( 1.0f/sigma );
  float4 weight = fn * exp( (float4)fe * dist );
  coords.x = XM*coords.x;
  write_imagef(dists, coords, weight);
}

__kernel void createWeightFiltered(
    __read_only image2d_t levelDistance,
	__read_only image2d_t credMap,
	__read_only image2d_t srcDepth,
    __write_only image2d_t weights,
    __write_only image2d_t values)
{
  int2 coords;
  coords.x = get_global_id(0);
  coords.y = get_global_id(1);

  float4 col = read_imagef(levelDistance, smp, coords);
  float4 cred = read_imagef(credMap, smp, coords);
  float4 dep = read_imagef(srcDepth, smp, coords);
  
  float4 weight = col * cred;
  
  coords.x = XM*coords.x;
  write_imagef(values, coords, weight*dep);
  write_imagef(weights, coords, weight);
}

__kernel void divideImage(
  __read_only image2d_t srcValues,
  __read_only image2d_t srcWeights,
  __write_only image2d_t result)
{
  int2 coords;
  coords.x = get_global_id(0);
  coords.y = get_global_id(1);

  float4 col = read_imagef(srcValues, smp, coords);
  float4 wei = read_imagef(srcWeights, smp, coords);
  wei = max((float4)(.0000000000001f), wei);
  col = col / wei;
  col = min((float4)10,col);
  col = max((float4)0.00000000001f,col);
  coords.x = XM*coords.x;
  write_imagef(result, coords, col);
}

__kernel void combineImage (
    __read_only image2d_t colorScaled_,
	__read_only image2d_t im0,
	__read_only image2d_t im1,
    __write_only image2d_t combined,
    float scale,
    float cmin, int level)
{
  int2 coords;
  coords.x = get_global_id(0);
  coords.y = get_global_id(1);

  float4 col = read_imagef(colorScaled_, smp, coords);
  float4 col0 = read_imagef(im0, smp, coords);
  float4 col1 = read_imagef(im1, smp, coords);
  
  float flevel = (float)level;
  
  coords.x = XM*coords.x*4;
  if (col.x>flevel && col.x <= flevel+1.0) { 
    float result = ( (col.x-flevel)*col0.x + (flevel+1.0-col.x)*col1.x )/scale  + cmin;
    write_imagef(combined, coords, (float4)result);
  }
  coords.x += 1*XM;
  if (col.y>flevel && col.y <= flevel+1.0) { 
    float result = ( (col.y-flevel)*col0.y + (flevel+1.0-col.y)*col1.y )/scale  + cmin;
    write_imagef(combined, coords, (float4)result);
  }
  coords.x += 1*XM;
  if (col.z>flevel && col.z <= flevel+1.0) { 
    float result = ( (col.z-flevel)*col0.z + (flevel+1.0-col.z)*col1.z )/scale  + cmin;
    write_imagef(combined, coords, (float4)result);
  }
  coords.x += 1*XM;
  if (col.w>flevel && col.w <= flevel+1.0) { 
    float result = ( (col.w-flevel)*col0.w + (flevel+1.0-col.w)*col1.w )/scale  + cmin;
    write_imagef(combined, coords, (float4)result);
  }
}

__kernel void betaCombine(
    __read_only image2d_t colorResult,
    __read_only image2d_t depthResult,
	__read_only image2d_t confidence,
	__write_only image2d_t combined)
{
  int xcoo = get_global_id(0);
  int ycoo = get_global_id(1);

  float4 col;
  xcoo *= 4;
  col.x = read_imagef(colorResult, smp, (int2)(xcoo,ycoo)).x;
  col.y = read_imagef(colorResult, smp, (int2)(xcoo+1,ycoo)).x;
  col.z = read_imagef(colorResult, smp, (int2)(xcoo+2,ycoo)).x;
  col.w = read_imagef(colorResult, smp, (int2)(xcoo+3,ycoo)).x;
  float4 dep;
  dep.x = read_imagef(depthResult, smp, (int2)(xcoo,ycoo)).x;
  dep.y = read_imagef(depthResult, smp, (int2)(xcoo+1,ycoo)).x;
  dep.z = read_imagef(depthResult, smp, (int2)(xcoo+2,ycoo)).x;
  dep.w = read_imagef(depthResult, smp, (int2)(xcoo+3,ycoo)).x;
  xcoo /= 4;
  float4 con = read_imagef(confidence, smp, (int2)(xcoo,ycoo));

  float4 result = con * dep + (1.0-con) * col;
  xcoo *= 4;
  xcoo *= XM;
  write_imagef(combined, (int2)(xcoo+0*XM,ycoo), (float4)(result.x));
  write_imagef(combined, (int2)(xcoo+1*XM,ycoo), (float4)(result.y));
  write_imagef(combined, (int2)(xcoo+2*XM,ycoo), (float4)(result.z));
  write_imagef(combined, (int2)(xcoo+3*XM,ycoo), (float4)(result.w));

}
