/*
 * clfAlgorithm.cpp
 *
 *  Created on: Nov 1, 2011
 *      Author: fkellner
 */

#include <OpenCLFramework/Algorithm/clfAlgorithm.hh>
#include <Base/Common/BIASpragma.hh>

namespace BIAS {

clfAlgorithm::clfAlgorithm() {
  context_ = NULL;
  programCL_ = NULL;
  isSharedGL_ = false;
  ownContext_ = false;
  device_ = 0;
  maxComputeUnits_ = 0;
}

clfAlgorithm::clfAlgorithm(clfContext *ctx, bool sharedGL, unsigned int device) {
  InitInternal_(ctx, sharedGL, device);
}

clfAlgorithm::~clfAlgorithm() {
  delete programCL_;
  if (ownContext_) {
    delete context_;
  }
}

void clfAlgorithm::InitInternal_(clfContext *ctx, bool sharedGL, unsigned int device) {
  if (ctx == NULL) {
    ownContext_ = true;
    context_ = new clfContext(sharedGL);
    if (device < context_->GetNumDevices()) {
      context_->SetActiveDevice(device);
    } else {
      device = 0;
    }
  } else {
    context_ = ctx;
    sharedGL = context_->IsSharedGLContext();
    ownContext_ = false;
  }
  isSharedGL_ = sharedGL;
  maxComputeUnits_ = context_->GetDeviceInfo().maxComputeUnits;
  programCL_ = context_->CreateProgram();
  device_ = device;
}

void clfAlgorithm::getlocsize(const unsigned int val, unsigned int &result) {
  result=clfAlgorithm::getlocsize(val);
}

unsigned int clfAlgorithm::getlocsize(const unsigned int val) {
  //if (val%32==0) return 32;
  if (val%16==0) return 16;
  if (val%8==0) return 8;
  return 0;
}


} /* namespace BIAS */
