/*
 * clfCopy.cpp
 *
 *  Created on: Nov 7, 2012
 *      Author: fkellner
 */

#include <OpenCLFramework/Algorithm/clfCopy.hh>
#include <Base/Common/BIASpragma.hh>
#include <Base/Image/Image.hh>
#include <Base/Image/ImageIO.hh>
#include <Base/Debug/TimeMeasure.hh>

using namespace BIAS;
using namespace BIAS;

clfCopy::clfCopy(clfContext *ctx, bool sharedGL, unsigned int device) : clfAlgorithm(ctx,sharedGL,device) {

  programCL_->AddSourceFromString(
    "__constant sampler_t smp_cpy = CLK_NORMALIZED_COORDS_FALSE | CLK_ADDRESS_CLAMP_TO_EDGE | CLK_FILTER_NEAREST;\n"
    "\n"
    "__kernel void SimpleCopy(__read_only image2d_t input, __write_only image2d_t output) {\n"
    "  int x = get_global_id(0);\n"
    "  int y = get_global_id(1);\n"
    "  if (x>=get_image_width(output) || y>=get_image_height(output)) return;\n"
    "  float4 res = read_imagef(input, smp_cpy, (int2)(x,y));"
    "  write_imagef(output, (int2)(x,y), res);\n"
    "}\n"
    );
  //programCL_->AddSource("/OpenCLFramework/Algorithm/cl/copy.cl");

  programCL_->Build();
  programCL_->AddKernel("SimpleCopy");
}

clfCopy::~clfCopy() {
}

void clfCopy::Copy(clfImage2D *src, clfImage2D *dest) {
  programCL_->KernelSetArgument("SimpleCopy", 0, *src);
  programCL_->KernelSetArgument("SimpleCopy", 1, *dest);
  context_->RunOn2DRange(*programCL_, "SimpleCopy", src->Width(), src->Height(), 16,16);
}
