/*
 * clfImageOperator.cpp
 *
 *  Created on: Dec 16, 2011
 *      Author: fkellner
 */

#include <OpenCLFramework/Algorithm/clfImageOperator.hh>
#include <Base/Common/BIASpragma.hh>

using namespace BIAS;

clfImageOperator::clfImageOperator(clfContext *ctx, bool sharedGL, unsigned int device)
    : clfAlgorithm(ctx,sharedGL,device)
{
  // add code to our program
  programCL_->AddSource("/OpenCLFramework/Algorithm/cl/imageOperator.cl");
  programCL_->Build();
  programCL_->AddKernel("DifferenceImage");
  programCL_->AddKernel("DifferenceAbsImage");
  programCL_->AddKernel("DifferenceZeroLimitImage");
  programCL_->AddKernel("CutOffMinThreshold");
}

clfImageOperator::~clfImageOperator()
{
}

void clfImageOperator::Difference(clfImage2D *im0, clfImage2D *im1, clfImage2D *result, bool asAbs, bool zeroCut)
{
  std::string kernelname = "DifferenceImage";
  if (asAbs) kernelname = "DifferenceAbsImage";
  if (zeroCut) kernelname = "DifferenceZeroLimitImage";
  programCL_->KernelSetArgument(kernelname, 0, *im0);
  programCL_->KernelSetArgument(kernelname, 1, *im1);
  programCL_->KernelSetArgument(kernelname, 2, *result);
  unsigned int w,h;
  result->GetImageDim(w,h);
  unsigned int lx=0, ly=0;
  if (w%16==0) lx=16;
  if (h%16==0) ly=16;
  context_->RunOn2DRange(*programCL_, kernelname, w,h, lx,ly);
}

void clfImageOperator::CutOffMinThreshold(clfImage2D *im0, clfImage2D *result, float minVal) {
  programCL_->KernelSetArgument("CutOffMinThreshold", 0, *im0);
  programCL_->KernelSetArgument("CutOffMinThreshold", 1, *result);
  unsigned int w,h;
  result->GetImageDim(w,h);
  unsigned int lx=0, ly=0;
  if (w%16==0) lx=16;
  if (h%16==0) ly=16;
  context_->RunOn2DRange(*programCL_, "CutOffMinThreshold", w,h, lx,ly);
}

