/*
 * clfRecursiveGauss.hh
 *
 *  Created on: Oct 28, 2011
 *      Author: fkellner
 */

#ifndef CLFRECURSIVEGAUSS_HH_
#define CLFRECURSIVEGAUSS_HH_

#include <Base/Common/BIASpragmaStart.hh>
#include <bias_config.h>
#include <OpenCLFramework/Algorithm/clfAlgorithm.hh>

namespace BIAS {
/**
 * @brief recursive approximation of gaussian blur (IIR)
 * opencl implementation of deriche, recursively implementing the gaussian and its derivatives (icip, 1992)
 */
class BIASOpenCLFramework_EXPORT clfRecursiveGauss : public clfAlgorithm {
public:
  /**
   * @brief constructor, give context or let
   */
  clfRecursiveGauss(float sigma = 2.0f, clfContext *ctx = NULL, bool sharedGL = false, unsigned int device = 0);
  virtual ~clfRecursiveGauss();

  /**
   * @brief gauss filter rgba image
   */
  int Filter(clfImage2D *src, clfImage2D *&dst);

  /**
   * @brief optimized gauss filter for grey images
   */
  int FilterGrey(clfImage2D *src, clfImage2D *&dst);

  /**
   * @brief optimized gauss filter for already packed grey images
   */
  int FilterGreyColPacked(clfImage2D *src, clfImage2D *&dst);

  /**
   * @brief Set Sigma and calculate coefficients for iir filter
   * @param sigma gaussian sigma
   * @param order derivative
   */
  void SetSigma(float sigma, int order = 0);

  /**
   * @brief clear intermediate images
   * resets storage of internal images, needs to be called if image sizes differ from frame to frame
   */
  void Reset();

  /**
   * @brief filter in X-direction
   */
  int FilterX(clfImage2D *src, clfImage2D *&dst);
  /**
   * @brief filter in Y-direction
   */
  int FilterY(clfImage2D *src, clfImage2D *&dst);

protected:

  clfImage2D *intermediateImage_;
  clfImage2D *oneDirFilteredImage_;

  clfImage2D *packedGreyImage_;

  std::vector<float> params_;

  std::vector<std::string> kernelNames_;

};

} /* namespace BIAS */
#include <Base/Common/BIASpragmaEnd.hh>
#endif /* CLFRECURSIVEGAUSS_HH_ */
