/*
 * clfUnifiedMultilateralFilter.cpp
 *
 *  Created on: Oct 21, 2011
 *      Author: fkellner
 */
#include <Base/Common/BIASpragma.hh>
#include <OpenCLFramework/Algorithm/clfUnifiedMultilateralFilter.hh>
#include <Base/Image/ImageIO.hh>
#include <Base/Math/Vector3.hh>
#include <Base/Common/FileHandling.hh>

using namespace std;
using namespace BIAS;
using namespace BIAS;
using namespace BIAS;

#define WRITE_DBG_IMG

clfUnifiedMultilateralFilter::clfUnifiedMultilateralFilter(clfContext *ctx, bool sharedGL, unsigned int device)
: clfAlgorithm(ctx, sharedGL, device)
{
  colorConvCL_ = new clfColorConversion(context_);

  recursiveGaussFilter_ = NULL;

  // add code to our program
  programCL_->AddSource("/OpenCLFramework/Algorithm/cl/unifiedmultilateralfilter.cl");
  programCL_->Build();
  programCL_->AddKernel("downsample2x");
  programCL_->AddKernel("upsample2x");
  programCL_->AddKernel("downsample4x");
  programCL_->AddKernel("upsample4x");
  programCL_->AddKernel("scaledExpInv");
  programCL_->AddKernel("createWeightFiltered");
  programCL_->AddKernel("levelDistance");
  programCL_->AddKernel("divideImage");
  programCL_->AddKernel("scaleShift");
  programCL_->AddKernel("combineImage");
  programCL_->AddKernel("betaCombine");

  programCL_->AddKernel("ColPackGreyToRGBA");
  programCL_->AddKernel("ColUnPackRGBAToGrey");

  downSampleFactor_ = 2;
}

clfUnifiedMultilateralFilter::~clfUnifiedMultilateralFilter() {
// delete images
}

void clfUnifiedMultilateralFilter::Init(BIAS::ImageBase &color, BIAS::ImageBase &depth) {
  sourceColor_ = context_->CreateImage2D();
  sourceDepth_ = context_->CreateImage2D();
  sourceLuv_ = context_->CreateImage2D();
  credibilityMap_ = context_->CreateImage2D();
  sourceDepthGradient_ = context_->CreateImage2D();
  colorValuesCombined_ = context_->CreateImage2D();
  depthValuesCombined_ = context_->CreateImage2D();
  if (downSampleFactor_>1) {
    sourceColorLarge_ = context_->CreateImage2D();
    sourceDepthLarge_ = context_->CreateImage2D();
  }

  if (downSampleFactor_>1) {
    sourceColorLarge_->AllocateFromBiasTemplate( color, false, false );
    sourceDepthLarge_->AllocateFromBiasTemplate( depth, false, false );
    unsigned int w,h;
    sourceColorLarge_->GetImageDim(w,h);
    w/=downSampleFactor_;
    h/=downSampleFactor_;
    if (w%64 != 0) {
      w += 64-(w%64);
    }
    if (h%16 != 0) {
      h += 64-(h%64);
    }
    sourceColor_->Allocate(sourceColorLarge_->StorageType(), sourceColorLarge_->ColorModel(), w,h,0, false, false );
    sourceDepth_->Allocate(sourceDepthLarge_->StorageType(), sourceDepthLarge_->ColorModel(), w,h,0, false, false );
  } else {
    sourceColor_->AllocateFromBiasTemplate( color, false, false );
    sourceDepth_->AllocateFromBiasTemplate( depth, false, false );
  }
  sourceLuv_->AllocateFromTemplate(*sourceColor_, false, false );
  sourceDepthGradient_->AllocateFromTemplate(*sourceDepth_, false, false );
  colorValuesCombined_->AllocateFromTemplate(*sourceColor_, false, false );
  depthValuesCombined_->AllocateFromTemplate(*sourceDepth_, false, false );
  sourceColor_->GetImageDim(colorw_, colorh_);
  sourceDepth_->GetImageDim(depthw_, depthh_);
  colorw_/=4;
  depthw_/=4;
}

void clfUnifiedMultilateralFilter::InitColorFilter( float sigma, float colorscale, float colorMin, int levels) {
  colorLevels_ = levels;
  colorMin_ = colorMin;
  if (colorLevels_>1000) {
    BIASERR("too many levels");
  }
  colorScale_ = colorscale;
  colorSigma_ = sigma;

  colorValues_ = context_->CreateImage2D();
  colorWeights_ = context_->CreateImage2D();
  colorValuesGauss_ = context_->CreateImage2D();
  colorWeightsGauss_ = context_->CreateImage2D();
  colorIntermediate_ = context_->CreateImage2D();
  colorScaled_ = context_->CreateImage2D();
  colorValues_->Allocate(ImageBase::ST_float, ImageBase::CM_RGBA, colorw_, colorh_, 0, false, false);
  colorWeights_->Allocate(ImageBase::ST_float, ImageBase::CM_RGBA, colorw_, colorh_, 0, false, false);
  colorValuesGauss_->Allocate(ImageBase::ST_float, ImageBase::CM_RGBA, colorw_, colorh_, 0, false, false);
  colorWeightsGauss_->Allocate(ImageBase::ST_float, ImageBase::CM_RGBA, colorw_, colorh_, 0, false, false);
  colorIntermediate_->Allocate(ImageBase::ST_float, ImageBase::CM_RGBA, colorw_, colorh_, 0, false, false);
  colorScaled_->Allocate(ImageBase::ST_float, ImageBase::CM_RGBA, colorw_, colorh_, 0, false, false);
  colorFiltered_.resize(levels);
  for (int i=0;i<levels;i++) {
    colorFiltered_[i] = context_->CreateImage2D();
    colorFiltered_[i]->Allocate(ImageBase::ST_float, ImageBase::CM_RGBA, colorw_, colorh_, 0, false, false);
  }

}

void clfUnifiedMultilateralFilter::InitDepthFilter( float sigma, float depthscale, float depthMin, int levels) {
  depthScale_ = depthscale;
  depthMin_ = depthMin;
  depthLevels_ = levels;
  if (depthLevels_>1000) {
    BIASERR("too many levels");
  }
  depthSigma_ = sigma;
  depthValues_ = context_->CreateImage2D();
  depthWeights_ = context_->CreateImage2D();
  depthValuesGauss_ = context_->CreateImage2D();
  depthWeightsGauss_ = context_->CreateImage2D();
  depthIntermediate_ = context_->CreateImage2D();
  depthScaled_ = context_->CreateImage2D();
  depthValues_->Allocate(ImageBase::ST_float, ImageBase::CM_RGBA, depthw_, depthh_, 0, false, false);
  depthWeights_->Allocate(ImageBase::ST_float, ImageBase::CM_RGBA, depthw_, depthh_, 0, false, false);
  depthValuesGauss_->Allocate(ImageBase::ST_float, ImageBase::CM_RGBA, depthw_, depthh_, 0, false, false);
  depthWeightsGauss_->Allocate(ImageBase::ST_float, ImageBase::CM_RGBA, depthw_, depthh_, 0, false, false);
  depthIntermediate_->Allocate(ImageBase::ST_float, ImageBase::CM_RGBA, depthw_, depthh_, 0, false, false);
  depthScaled_->Allocate(ImageBase::ST_float, ImageBase::CM_RGBA, depthw_, depthh_, 0, false, false);
  depthFiltered_.resize(levels);
  for (int i=0;i<levels;i++) {
    depthFiltered_[i] = context_->CreateImage2D();
    depthFiltered_[i]->Allocate(ImageBase::ST_float, ImageBase::CM_RGBA, depthw_, depthh_, 0, false, false);
  }
}

void clfUnifiedMultilateralFilter::InitSpatialFilter ( float sigma ) {
  recursiveGaussFilter_ = new clfRecursiveGauss(sigma, context_);
//  recursiveGaussFilter_ = new clfGauss<float, float>(context_);
//  recursiveGaussFilter_->Build(15);
}

void clfUnifiedMultilateralFilter::InitCredibilityFilter ( int size, float scaleGradient, float scaleResult ) {
  credScale_ = scaleResult;
  gradGaussFilter_ = new clfGradientGauss<float, float>(context_);
  gradGaussFilter_->Build(size);
  credibilityMap_->Allocate(ImageBase::ST_float, ImageBase::CM_RGBA, depthw_, depthh_, 0, false, false);

  programCL_->KernelSetArgument("scaledExpInv", 0, *sourceDepthGradient_);
  programCL_->KernelSetArgument("scaledExpInv", 1, *credibilityMap_);
  programCL_->KernelSetArgument("scaledExpInv", 2, scaleGradient);
  programCL_->KernelSetArgument("scaledExpInv", 3, scaleResult);
}


void clfUnifiedMultilateralFilter::SetImage(BIAS::ImageBase &color, BIAS::ImageBase &depth) {
  if (downSampleFactor_>1) {
    sourceColorLarge_->WriteToImage( color.GetImageData() );
    sourceDepthLarge_->WriteToImage( depth.GetImageData() );
  } else {
    sourceColor_->WriteToImage( color.GetImageData() );
    sourceDepth_->WriteToImage( depth.GetImageData() );
  }
}

void clfUnifiedMultilateralFilter::SetColorImage(clfImage2D *image) {
  if (downSampleFactor_>1) {
    image->CopyToImage(*sourceColorLarge_);
  } else {
    image->CopyToImage(*sourceColor_);
  }
}
clfImage2D *clfUnifiedMultilateralFilter::GetColorImage() {
  if (downSampleFactor_>1) {
    return sourceColorLarge_;
  } else {
    return sourceColor_;
  }
}
void clfUnifiedMultilateralFilter::SetDepthImage(clfImage2D *image) {
  if (downSampleFactor_>1) {
    image->CopyToImage(*sourceDepthLarge_);
  } else {
    image->CopyToImage(*sourceDepth_);
  }
}

clfImage2D *clfUnifiedMultilateralFilter::GetDepthImage() {
  if (downSampleFactor_>1) {
    return sourceDepthLarge_;
  } else {
    return sourceDepth_;
  }
}


void clfUnifiedMultilateralFilter::Compute() {
#ifdef WRITE_DBG_IMG
  ImageBase test;
#endif

  int localX = 16;
  int localY = 16;

  string factorString = "2x";
  if (downSampleFactor_ == 4)
    factorString = "4x";

  if (downSampleFactor_>1) {
    programCL_->KernelSetArgument("downsample"+factorString, 0, *sourceColorLarge_);
    programCL_->KernelSetArgument("downsample"+factorString, 1, *sourceColor_);
    context_->RunOn2DRange(*programCL_, "downsample"+factorString, colorw_*4, colorh_, localX, localY);
#ifdef WRITE_DBG_IMG
    sourceColor_->CopyToBiasImage( test );
    ImageIO::Save("colordownsampled.mip", test);
#endif
    programCL_->KernelSetArgument("downsample"+factorString, 0, *sourceDepthLarge_);
    programCL_->KernelSetArgument("downsample"+factorString, 1, *sourceDepth_);
    context_->RunOn2DRange(*programCL_, "downsample"+factorString, depthw_*4, depthh_, localX, localY);
#ifdef WRITE_DBG_IMG
    sourceDepthLarge_->CopyToBiasImage( test );
    ImageIO::Save("depthlarge.mip", test);
    sourceDepth_->CopyToBiasImage( test );
    ImageIO::Save("depthdownsampled.mip", test);
#endif
  }

  colorConvCL_->RGBToGrey( sourceColor_, sourceLuv_ );
  gradGaussFilter_->Filter(sourceDepth_, sourceDepthGradient_ );

  context_->RunOn2DRange(*programCL_, "scaledExpInv", depthw_, depthh_, localX, localY);
#ifdef WRITE_DBG_IMG
  sourceLuv_->CopyToBiasImage( test );
  ImageIO::Save("luv.mip", test);
  credibilityMap_->CopyToBiasImage( test );
  ImageIO::Save("credibilityMap.mip", test);
#endif

  programCL_->KernelSetArgument("scaleShift", 0, *sourceLuv_);
  programCL_->KernelSetArgument("scaleShift", 1, *colorScaled_);
  programCL_->KernelSetArgument("scaleShift", 2, colorScale_);
  programCL_->KernelSetArgument("scaleShift", 3, colorMin_);

  context_->RunOn2DRange(*programCL_, "scaleShift", colorw_, colorh_, localX, localY);

#ifdef WRITE_DBG_IMG
  colorScaled_->CopyToBiasImage(test);
  ImageIO::Save("colorscaled.mip", test);
#endif
  programCL_->KernelSetArgument("scaleShift", 0, *sourceDepth_);
  programCL_->KernelSetArgument("scaleShift", 1, *depthScaled_);
  programCL_->KernelSetArgument("scaleShift", 2, depthScale_);
  programCL_->KernelSetArgument("scaleShift", 3, depthMin_);

  context_->RunOn2DRange(*programCL_, "scaleShift", depthw_, depthh_, localX, localY);
#ifdef WRITE_DBG_IMG
  depthScaled_->CopyToBiasImage(test);
  ImageIO::Save("depthscaled.mip", test);
#endif
  programCL_->KernelSetArgument("levelDistance", 1, *colorScaled_);
  programCL_->KernelSetArgument("levelDistance", 2, *colorIntermediate_);
  programCL_->KernelSetArgument("levelDistance", 3, colorSigma_);

  programCL_->KernelSetArgument("createWeightFiltered", 0, *colorIntermediate_);
  programCL_->KernelSetArgument("createWeightFiltered", 1, *credibilityMap_);
  programCL_->KernelSetArgument("createWeightFiltered", 2, *depthScaled_);
  programCL_->KernelSetArgument("createWeightFiltered", 3, *colorWeights_);
  programCL_->KernelSetArgument("createWeightFiltered", 4, *colorValues_);

  for (unsigned int i=0;i<colorFiltered_.size();i++) {
    try {
      programCL_->KernelSetArgument("levelDistance", 0, (int)i);
      context_->RunOn2DRange(*programCL_, "levelDistance", colorw_, colorh_, localX, localY);
#ifdef WRITE_DBG_IMG
      colorIntermediate_->CopyToBiasImage(test);
      ImageIO::Save("leveldistanceColor-"+FileHandling::LeadingZeroString(i,3)+".mip", test);
#endif
      context_->RunOn2DRange(*programCL_, "createWeightFiltered", colorw_, colorh_, localX, localY);
    } catch (clfException &e) {
      cout << "filter exception\n" << e.GetDetailedString() << endl;
      BIASABORT;
    }
    context_->Finish();

    try {
      recursiveGaussFilter_->FilterGreyColPacked(colorWeights_, colorWeightsGauss_ );
#ifdef WRITE_DBG_IMG
      colorWeights_->CopyToBiasImage(test);
      ImageIO::Save("colorWeights-"+FileHandling::LeadingZeroString(i,3)+".mip", test);
      colorWeightsGauss_->CopyToBiasImage(test);
      ImageIO::Save("colorWeightsGauss-"+FileHandling::LeadingZeroString(i,3)+".mip", test);
#endif
      recursiveGaussFilter_->FilterGreyColPacked(colorValues_, colorValuesGauss_ );
      context_->Finish();
#ifdef WRITE_DBG_IMG
      colorWeights_->CopyToBiasImage(test);
      ImageIO::Save("colorWeights-"+FileHandling::LeadingZeroString(i,3)+".mip", test);
      colorWeightsGauss_->CopyToBiasImage(test);
      ImageIO::Save("colorWeightsGauss-"+FileHandling::LeadingZeroString(i,3)+".mip", test);
      colorValues_->CopyToBiasImage(test);
      ImageIO::Save("colorValues-"+FileHandling::LeadingZeroString(i,3)+".mip", test);
      colorValuesGauss_->CopyToBiasImage(test);
      ImageIO::Save("colorValuesGauss-"+FileHandling::LeadingZeroString(i,3)+".mip", test);
#endif
    } catch (clfException &e) {
      cout << "gaussian exception\n" << e.GetDetailedString() << endl;
      BIASABORT;
    }
    try {
      programCL_->KernelSetArgument("divideImage", 0, *colorValuesGauss_);
      programCL_->KernelSetArgument("divideImage", 1, *colorWeightsGauss_);
      programCL_->KernelSetArgument("divideImage", 2, *(colorFiltered_[i]));

      context_->RunOn2DRange(*programCL_, "divideImage", colorw_, colorh_, localX, localY);
      context_->Finish();
    } catch (clfException &e) {
      cout << "divide exception\n" << e.GetDetailedString() << endl;
      BIASABORT;
    }
#ifdef WRITE_DBG_IMG
    colorFiltered_[i]->CopyToBiasImage(test);
    ImageIO::Save("colorFiltered-"+FileHandling::LeadingZeroString(i,3)+".mip", test);
#endif
  }
  programCL_->KernelSetArgument("levelDistance", 1, *depthScaled_);
  programCL_->KernelSetArgument("levelDistance", 2, *depthIntermediate_);
  programCL_->KernelSetArgument("levelDistance", 3, depthSigma_);

  programCL_->KernelSetArgument("createWeightFiltered", 0, *depthIntermediate_);
  programCL_->KernelSetArgument("createWeightFiltered", 1, *credibilityMap_);
  programCL_->KernelSetArgument("createWeightFiltered", 2, *depthScaled_);
  programCL_->KernelSetArgument("createWeightFiltered", 3, *depthWeights_);
  programCL_->KernelSetArgument("createWeightFiltered", 4, *depthValues_);

  for (unsigned int i=0;i<depthFiltered_.size();i++) {

    try {
      programCL_->KernelSetArgument("levelDistance", 0, (int)i);
      context_->RunOn2DRange(*programCL_, "levelDistance", depthw_, depthh_, localX, localY);
#ifdef WRITE_DBG_IMG
      depthIntermediate_->CopyToBiasImage(test);
      ImageIO::Save("leveldistancedepth-"+FileHandling::LeadingZeroString(i,3)+".mip", test);
#endif
      context_->RunOn2DRange(*programCL_, "createWeightFiltered", depthw_, depthh_, localX, localY);
    } catch (clfException &e) {
      cout << "filter exception\n" << e.GetDetailedString() << endl;
      BIASABORT;
    }
    context_->Finish();

    try {
      recursiveGaussFilter_->FilterGreyColPacked(depthWeights_, depthWeightsGauss_ );
      recursiveGaussFilter_->FilterGreyColPacked(depthValues_, depthValuesGauss_ );
#ifdef WRITE_DBG_IMG
      depthWeightsGauss_->CopyToBiasImage(test);
      ImageIO::Save("depthWeightsGauss-"+FileHandling::LeadingZeroString(i,3)+".mip", test);
      depthValues_->CopyToBiasImage(test);
      ImageIO::Save("depthValues-"+FileHandling::LeadingZeroString(i,3)+".mip", test);
#endif
    } catch (clfException &e) {
      cout << "gaussian exception\n" << e.GetDetailedString() << endl;
      BIASABORT;
    }

    try {
      programCL_->KernelSetArgument("divideImage", 0, *depthValuesGauss_);
      programCL_->KernelSetArgument("divideImage", 1, *depthWeightsGauss_);
      programCL_->KernelSetArgument("divideImage", 2, *(depthFiltered_[i]));

      context_->RunOn2DRange(*programCL_, "divideImage", depthw_, depthh_, localX, localY);
    } catch (clfException &e) {
      cout << "divide exception\n" << e.GetDetailedString() << endl;
      BIASABORT;
    }
#ifdef WRITE_DBG_IMG
    depthFiltered_[i]->CopyToBiasImage(test);
    ImageIO::Save("depthFiltered-"+FileHandling::LeadingZeroString(i,3)+".mip", test);
#endif
  }

  for (unsigned int i=0;i<colorFiltered_.size()-1;i++) {

    programCL_->KernelSetArgument("combineImage", 0, *colorScaled_); // here was depth scaled
    programCL_->KernelSetArgument("combineImage", 1, *colorFiltered_[i]);
    programCL_->KernelSetArgument("combineImage", 2, *colorFiltered_[i+1]);
    programCL_->KernelSetArgument("combineImage", 3, *colorValuesCombined_);
    programCL_->KernelSetArgument("combineImage", 4, depthScale_);
    programCL_->KernelSetArgument("combineImage", 5, depthMin_);
    programCL_->KernelSetArgument("combineImage", 6, (int)i);
    context_->RunOn2DRange(*programCL_, "combineImage", colorw_, colorh_, localX, localY);
  }
#ifdef WRITE_DBG_IMG
  colorValuesCombined_->CopyToBiasImage(test);
  ImageIO::Save("colorFilteredFinal.mip", test);
#endif
  for (unsigned int i=0;i<depthFiltered_.size()-1;i++) {

    programCL_->KernelSetArgument("combineImage", 0, *depthScaled_);
    programCL_->KernelSetArgument("combineImage", 1, *depthFiltered_[i]);
    programCL_->KernelSetArgument("combineImage", 2, *depthFiltered_[i+1]);
    programCL_->KernelSetArgument("combineImage", 3, *depthValuesCombined_);
    programCL_->KernelSetArgument("combineImage", 4, depthScale_);
    programCL_->KernelSetArgument("combineImage", 5, depthMin_);
    programCL_->KernelSetArgument("combineImage", 6, (int)i);
    context_->RunOn2DRange(*programCL_, "combineImage", depthw_, depthh_, localX, localY);
  }
#ifdef WRITE_DBG_IMG
  depthValuesCombined_->CopyToBiasImage(test);
  ImageIO::Save("depthFilteredFinal.mip", test);
#endif
  programCL_->KernelSetArgument("betaCombine", 0, *colorValuesCombined_);
  programCL_->KernelSetArgument("betaCombine", 1, *depthValuesCombined_);
  programCL_->KernelSetArgument("betaCombine", 2, *credibilityMap_);
  programCL_->KernelSetArgument("betaCombine", 3, *sourceDepth_);
  context_->RunOn2DRange(*programCL_, "betaCombine", depthw_, depthh_, localX, localY);

#ifdef WRITE_DBG_IMG
  sourceDepth_->CopyToBiasImage(test);
  ImageIO::Save("filteredFinal.mip", test);
#endif

  if (downSampleFactor_>1) {
    programCL_->KernelSetArgument("upsample"+factorString, 0, *sourceDepth_);
    programCL_->KernelSetArgument("upsample"+factorString, 1, *sourceDepthLarge_);
    context_->RunOn2DRange(*programCL_, "upsample"+factorString, 4*depthw_*downSampleFactor_, depthh_*downSampleFactor_, localX, localY);
#ifdef WRITE_DBG_IMG
    cout << sourceDepthLarge_->Width() << " " << sourceDepthLarge_->Height() << " this " << endl;
    sourceDepthLarge_->CopyToBiasImage(test);
    ImageIO::Save("filteredFinalUps.mip", test);
#endif
  }

}

void clfUnifiedMultilateralFilter::GetResult(BIAS::ImageBase &image) {
  if (downSampleFactor_>1) {
    sourceDepthLarge_->CopyToBiasImage(image);
  } else {
    sourceDepth_->CopyToBiasImage(image);
  }

}

