/*
 * clfUnifiedMultilateralFilter.hh
 *
 *  Created on: Oct 21, 2011
 *      Author: fkellner
 */

#ifndef CLFUNIFIEDMULTILATERALFILTER_HH_
#define CLFUNIFIEDMULTILATERALFILTER_HH_

#include <bias_config.h>
#include <Base/Common/BIASpragmaStart.hh>
#include <OpenGLFramework/Base/glfTexture2D.hh>
#include <OpenCLFramework/Algorithm/clfAlgorithm.hh>
#include <OpenCLFramework/Filter/clfColorConversion.hh>
#include <OpenCLFramework/Filter/clfFilter.hh>
#include <OpenCLFramework/Filter/clfGauss.hh>
#include <OpenCLFramework/Filter/clfGradientGauss.hh>
#include <OpenCLFramework/Algorithm/clfRecursiveGauss.hh>

namespace BIAS {

class BIASOpenCLFramework_EXPORT clfUnifiedMultilateralFilter : public clfAlgorithm {
public:
  clfUnifiedMultilateralFilter(clfContext *ctx = NULL,
                               bool sharedGL = false, unsigned int device = 0);
  virtual ~clfUnifiedMultilateralFilter();

  void SetImage(BIAS::ImageBase &color, BIAS::ImageBase &depth);

  void SetColorImage(clfImage2D *image);
  clfImage2D *GetColorImage();

  void SetDepthImage(clfImage2D *image);
  clfImage2D *GetDepthImage();

  void Compute();
  void GetResult(BIAS::ImageBase &image);

  void Init(BIAS::ImageBase &color, BIAS::ImageBase &depth);

  void InitColorFilter( float sigma, float colorscale, float colorMin, int levels);

  void InitDepthFilter( float sigma, float depthscale, float depthMin, int levels);

  void InitSpatialFilter ( float sigma );

  void InitCredibilityFilter ( int fsize, float scaleGradient, float scaleResult );

  void SetDownsample(int factor) {
    if (factor == 1 || factor == 2 || factor == 4)
      downSampleFactor_ = factor;
    else
      BIASABORT;
  }
private:

  BIAS::glfTexture2D *sourceColorTex_;
  BIAS::glfTexture2D *sourceDepthTex_;

  clfImage2D *sourceColor_;
  clfImage2D *sourceDepth_;

  clfImage2D *sourceColorLarge_;
  clfImage2D *sourceDepthLarge_;

  // convert to grey, think about using luv!
  clfImage2D *sourceLuv_;
  clfImage2D *sourceDepthGradient_;
  clfImage2D *credibilityMap_;

  // EIk
  clfImage2D* colorValues_;
  clfImage2D* colorValuesCombined_;
  // FIk
  clfImage2D* colorWeights_;
  // EIk
  clfImage2D* colorValuesGauss_;
  // FIk
  clfImage2D* colorWeightsGauss_;
  clfImage2D* colorIntermediate_;
  clfImage2D* colorScaled_;
  // J5k
  std::vector<clfImage2D*> colorFiltered_;

  // GIk
  clfImage2D* depthValues_;
  clfImage2D* depthValuesCombined_;
  // HIk
  clfImage2D* depthWeights_;
  // GIk
  clfImage2D* depthValuesGauss_;
  // HIk
  clfImage2D* depthWeightsGauss_;
  clfImage2D* depthIntermediate_;
  clfImage2D* depthScaled_;
  // J5k
  std::vector<clfImage2D*> depthFiltered_;

  float colorScale_;
  float colorMin_;
  float depthScale_;
  float depthMin_;
  float credScale_;

  clfColorConversion *colorConvCL_;
  clfGradientGauss<float, float> *gradGaussFilter_;

  clfRecursiveGauss *recursiveGaussFilter_;

  int colorLevels_;
  int depthLevels_;

  unsigned int depthw_, depthh_, colorw_, colorh_;

  int downSampleFactor_;

  float colorSigma_;
  float depthSigma_;

};

} /* namespace BIAS */
#include <Base/Common/BIASpragmaEnd.hh>
#endif /* CLFUNIFIEDMULTILATERALFILTER_HH_ */
