/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004    (see file CONTACTS for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <Utils/Param.hh>
#include <Utils/IOUtils.hh>
#include <Base/Image/Image.hh>
#include <Base/Image/ImageIO.hh>
#include <Base/Image/ImageConvert.hh>
#include <OpenCLFramework/clfContext.hh>
#include <iostream>
#include <OpenCLFramework/Filter/clfGauss.hh>
#include <Filter/Mean.hh>
#include <Filter/Gauss.hh>
#include <OpenCLFramework/Filter/clfColorConversion.hh>
#include <OpenCLFramework/Filter/clfSimpleFilter.hh>
#include <Filter/StructureTensor.hh>
#include <FeatureDetector/CornerDetectorKLT.hh>
#include <Filter/GradientSimple.hh>


using namespace BIAS;
using namespace std;

/**
 * @brief mean filtering an image using opencl
 * 
 * test program for clfImage2D objects. The kernel mean3x3 filters an image
 * 
 */

#define CLWS 11

int main(int argc, char* argv[]) {
  
  Param par;
  string *inname = par.AddParamString("image", "image", "", 'i');
  string *outname = par.AddParamString("out", "result", "climage.mip", 'o');
  
  if (IOUtils::ParseCommandLineEvalHelp(par, argc, argv) == 0) {
    par.Usage();
  }
  
  Image<unsigned char> in;
  Image<float> grey,out;
  ImageIO::Load(*inname, in);
//  in.Init(1024,1024,3);
  unsigned int w = in.GetWidth();
  unsigned int h = in.GetHeight();
  if (h % 16 != 0) {
    unsigned int hnew = (h / 16) * 16 + 16;
    in.Pad(w, hnew);
    h = hnew;
  }
  grey.Init(w,h,4);
  out.Init(w,h,4);

  try {

    // create context on first GPU found
    clfContext context(false);
    
    clfColorConversion convert(&context);
    
    clfImage2D *inImage = context.CreateImage2D();
    clfImage2D *greyImage = context.CreateImage2D();
    clfImage2D *gaussImage = context.CreateImage2D();
    clfImage2D *gradImage = context.CreateImage2D();
    clfImage2D *structureImage = context.CreateImage2D();
    clfImage2D *cornerness = context.CreateImage2D();
    clfImage2D *dstImage = context.CreateImage2D();
    inImage->AllocateFromBiasImage(in, true, false, true);
    greyImage->AllocateFromBiasImage(grey, false, false);
    gaussImage->AllocateFromBiasImage(grey, false, false);
    gradImage->AllocateFromBiasImage(grey, false, false);
    structureImage->AllocateFromBiasImage(grey, false, false);
    cornerness->AllocateFromBiasImage(grey, false, false);
    dstImage->AllocateFromBiasImage(grey, false, true);
    
    convert.RGBToGrey(inImage, greyImage);
    
    clfGauss<float, float> filter(&context);
//    clfFilter<float, float> filter(&context);
    
    //filter.Build(3);
    
    filter.Filter(greyImage, gaussImage);
    
    gaussImage->CopyToBiasImage(grey);
    
    clfSimpleFilter sfilter(&context, "Gradient3x3");
    
    sfilter.Filter(greyImage/*gaussImage*/, gradImage);
    
    clfStructureTensor struc(&context, 7);
    struc.Filter(gradImage, structureImage);
    
    sfilter.SetProgram("CornernessKLT");
    
    sfilter.Filter(structureImage, cornerness);
    
    clfExtractLocalMaxima maxima(&context, 3, 1.0f);
    
    maxima.Filter(cornerness, dstImage);
    
    
//    structureImage->CopyToImage(*dstImage);
    
    dstImage->CopyToBiasImage(out);
  } catch (clfException &err) {
    // the detailed error string
    cout << err.GetDetailedString() << endl;
  }
  // save result
  ImageIO::Save(*outname, out);
  
  Image<unsigned char> biasgrey;
  CornerDetectorKLT<float, float> klt;
  GradientSimple<unsigned char, float> grad;
  ImageConvert::ToGrey(in, biasgrey);
  Image<float> gx,gy,cornerness;
  grad.Filter(biasgrey, gx, gy);
  klt.SetStructureTensorHalfWinSize(3);
  klt.Cornerness(gx,gy, cornerness);
  ImageIO::Save("biasresult.mip", cornerness);
  ImageIO::Save("gx.mip", gx);
  ImageIO::Save("gy.mip", gy);
  
//  BIAS::StructureTensor<float,float> st;
//  st.SetHalfWinSize(3);
//  st.Filter(grey, out);
//  
////  BIAS::Gauss<float, float> mean;
////  mean.SetHalfWinSize(1,false);
////  mean.Filter(inf,out);
//  ImageIO::Save("biasresult.mip", out);
  return 0;
}
