inline int2 getrgbcoordR(int width, int x, int y)
{
   int srcX = x*3;
   int srcY = y*3;
   int overlap = srcX / width;
   srcX = srcX - overlap*width;
   srcY = srcY+overlap;
   
   return (int2)(srcX, srcY);
}

inline int2 getrgbcoordinc(int width, int2 coord)
{
  int2 ret = coord;
  ret.x ++;
  if (ret.x >= width) {
    ret.x = 0;
    ret.y ++;
  }
  return ret;
}

uchar readColorAsGrey(__read_only image2d_t input, int2 xy, int intype, const sampler_t smp) {
uint4 color;
if (intype == CLK_FLOAT) {
float4 tmpColor = read_imagef(input, smp, xy);

color.x = min((uint)255,(uint)rint((0.299*tmpColor.x + 0.587*tmpColor.y + 0.114*tmpColor.z)*255.0f));
} else {
color = read_imageui(input, smp, xy);

color.x = min((uint)255,(uint)rint(0.299*color.x + 0.587*color.y + 0.114*color.z));
}

return color.x;
}

__kernel void clear(__write_only image2d_t output, float r,float g,float b, float a) {
   int x = get_global_id(0);
   int y = get_global_id(1);
   int2 outdim = get_image_dim(output);
   int outtype  = get_image_channel_data_type(output);
   if (x < outdim.x && y < outdim.y) {
     if (outtype == CLK_FLOAT) {
       float4 rgba = (float4)(r,g,b,a);
       write_imagef(output, (int2)(x,y), rgba);
     } else {
       uint4 rgbaui;
       rgbaui = (uint4)( (uint)r, (uint)g, (uint)b, (uint)a);
       write_imageui(output, (int2)(x,y), rgbaui);
     }
   }
}


__kernel void RGBToRGBA(__read_only image2d_t src,
                    __write_only image2d_t dst, float scale)
{
   const sampler_t smp = CLK_NORMALIZED_COORDS_FALSE | CLK_ADDRESS_CLAMP_TO_EDGE | CLK_FILTER_NEAREST;
         
   int x = get_global_id(0);
   int y = get_global_id(1);

   int2 wh = get_image_dim(src);
   int intype  = get_image_channel_data_type(src);
   int outtype  = get_image_channel_data_type(dst);
   
   int2 coordsR, coordsG, coordsB;
   coordsR = getrgbcoordR(wh.x, x, y);
   coordsG = getrgbcoordinc(wh.x, coordsR);
   coordsB = getrgbcoordinc(wh.x, coordsG);
   
   float4 rgba;
   if (intype == CLK_FLOAT) {
     rgba.x = read_imagef(src, smp, coordsR).x;
     rgba.y = read_imagef(src, smp, coordsG).x;
     rgba.z = read_imagef(src, smp, coordsB).x;
     rgba.w = 255.0f;
   } else {
     rgba.x = (float)(read_imageui(src, smp, coordsR).x);
     rgba.y = (float)(read_imageui(src, smp, coordsG).x);
     rgba.z = (float)(read_imageui(src, smp, coordsB).x);
     rgba.w = 255.0f;
   }
   
   rgba = scale * rgba;
   
   coordsR = (int2)(XM*x, y);
   
   if (outtype == CLK_FLOAT) {
     write_imagef(dst, coordsR, rgba);
   } else {
   	 uint4 rgbaui;
   	 rgbaui = (uint4)( (uint)rgba.x, (uint)rgba.y, (uint)rgba.z, (uint)rgba.w);
     write_imageui(dst, coordsR, rgbaui);
   }
}

__kernel void UCharRGBToFloatGreyNormalized(__read_only image2d_t src,
                    __write_only image2d_t dst)
{
   const sampler_t smp = CLK_NORMALIZED_COORDS_FALSE | CLK_ADDRESS_CLAMP_TO_EDGE | CLK_FILTER_NEAREST;
         
   int x = get_global_id(0);
   int y = get_global_id(1);
   
   int2 wh = get_image_dim(src);
   
   int srcX = x*3;
   int srcY = y*3;
   int overlap = srcX / wh.x;
   srcX = srcX - overlap*wh.x;
   srcY = srcY+overlap;
   
   int2 coords = (int2)(srcX, srcY);
   float4 rgba;
   rgba.x = (float)(read_imageui(src, smp, coords).x);
   coords.x += 1;
   if (coords.x >= wh.x) {
     coords.x = 0;
     coords.y = coords.y +1;
   }
   rgba.y = (float)(read_imageui(src, smp, coords).x);
   coords.x += 1;
   if (coords.x >= wh.x) {
     coords.x = 0;
     coords.y = coords.y +1;
   }
   rgba.z = (float)(read_imageui(src, smp, coords).x);
   rgba.w = 1.0f;
   
   float grey = (1.0/255.0) * (0.299*rgba.x + 0.587*rgba.y + 0.114*rgba.z);

   rgba = (float4)(grey,grey,grey,grey);
   
   coords = (int2)(XM*x, y);
   
   write_imagef(dst, coords, rgba);
}

__kernel void RGBToGrey(__read_only image2d_t src,
                    __write_only image2d_t dst, float scale)
{
   const sampler_t smp = CLK_NORMALIZED_COORDS_FALSE | CLK_ADDRESS_CLAMP_TO_EDGE | CLK_FILTER_NEAREST;
         
   int x = get_global_id(0);
   int y = get_global_id(1);
   
   int2 wh = get_image_dim(src);
   int intype  = get_image_channel_data_type(src);
   int outtype  = get_image_channel_data_type(dst);
   
   int2 coordsR, coordsG, coordsB;
   coordsR = getrgbcoordR(wh.x, x, y);
   coordsG = getrgbcoordinc(wh.x, coordsR);
   coordsB = getrgbcoordinc(wh.x, coordsG);
   
   float4 rgba;
   if (intype == CLK_FLOAT) {
     rgba.x = (float)(read_imagef(src, smp, coordsR).x);
     rgba.y = (float)(read_imagef(src, smp, coordsG).x);
     rgba.z = (float)(read_imagef(src, smp, coordsB).x);
     rgba.w = 255.0f;
   } else {
     rgba.x = (float)(read_imageui(src, smp, coordsR).x);
     rgba.y = (float)(read_imageui(src, smp, coordsG).x);
     rgba.z = (float)(read_imageui(src, smp, coordsB).x);
     rgba.w = 255.0f;
   }
   
   coordsR = (int2)(XM*x, y);
   
   float grey = 0.299*rgba.x + 0.587*rgba.y + 0.114*rgba.z;
   grey *= scale;

   rgba = (float4)(grey,grey,grey,grey);
   
   coordsR = (int2)(XM*x, y);
   
   if (outtype == CLK_FLOAT) {
     write_imagef(dst, coordsR, (float4)(grey));
   } else {
     write_imageui(dst, coordsR, (uint4)(grey));
   }
}

__kernel void RGBAToGrey(__read_only image2d_t src,
                    __write_only image2d_t dst, float scale)
{
   const sampler_t smp = CLK_NORMALIZED_COORDS_FALSE | CLK_ADDRESS_CLAMP_TO_EDGE | CLK_FILTER_NEAREST;
         
   int x = get_global_id(0);
   int y = get_global_id(1);
   
   int2 wh = get_image_dim(src);
   
   int2 coords = (int2)(x,y);
   
   int intype  = get_image_channel_data_type(src);
   int outtype  = get_image_channel_data_type(dst);
   
   float4 rgba;
   if (intype == CLK_FLOAT) {
     rgba = read_imagef(src, smp, coords);
   } else {
     uint4 urgba = read_imageui(src, smp, coords);
     rgba.x = (float)urgba.x;
     rgba.y = (float)urgba.y;
     rgba.z = (float)urgba.z;
     rgba.w = (float)urgba.w;
   }
   
   float grey = 0.299*rgba.x + 0.587*rgba.y + 0.114*rgba.z;
   grey *= scale;

   coords = (int2)(XM*x, y);
   if (outtype == CLK_FLOAT) {
     write_imagef(dst, coords, (float4)(grey));
   } else {
     write_imageui(dst, coords, (uint4)(min(255u,(uint)rint(grey))));
   }
}

__kernel void FloatGreyToFloatGrey(__read_only image2d_t src,
                    __write_only image2d_t dst)
{
   const sampler_t smp = CLK_NORMALIZED_COORDS_FALSE | CLK_ADDRESS_CLAMP_TO_EDGE | CLK_FILTER_NEAREST;

   int2 coords;         
   coords.x = get_global_id(0);
   coords.y = get_global_id(1);
   
   float4 rgba;   
   float val = (float)(read_imagef(src, smp, coords).x);

   rgba = (float4)(val,val,val,val);
   
   coords.x *= XM;
   
   write_imagef(dst, coords, rgba);
}

__kernel void FloatGreyToRGBA(__read_only image2d_t src,
                              __write_only image2d_t dst,
                              float scale)
{
   const sampler_t smp = CLK_NORMALIZED_COORDS_FALSE | CLK_ADDRESS_CLAMP_TO_EDGE | CLK_FILTER_NEAREST;

   int2 coords;
   coords.x = get_global_id(0);
   coords.y = get_global_id(1);

   int intype  = get_image_channel_data_type(src);
   int outtype  = get_image_channel_data_type(dst);

   float val;
   if (intype == CLK_FLOAT) {
     val = (float)(read_imagef(src, smp, coords).x);
   } else {
     uint uval = (uint)(read_imageui(src, smp, coords).x);
     val = (float)uval;
   }

   val *= scale;

   float4 rgba;
   rgba = (float4)(val,val,val,val);
   coords.x *= XM;

   if (outtype == CLK_FLOAT) {
     write_imagef(dst, coords, rgba);
   } else {
     uint4 rgbaui;
     rgbaui = (uint4)( (uint)rgba.x, (uint)rgba.y, (uint)rgba.z, (uint)rgba.w);
     write_imageui(dst, coords, rgbaui);
   }
}

__kernel void UCharGreyToFloatGrey(__read_only image2d_t src,
                    __write_only image2d_t dst, float scale)
{
   const sampler_t smp = CLK_NORMALIZED_COORDS_FALSE | CLK_ADDRESS_CLAMP_TO_EDGE | CLK_FILTER_NEAREST;

   int2 coords;         
   coords.x = get_global_id(0);
   coords.y = get_global_id(1);
   
   float4 rgba;   
   float val = (float)(read_imageui(src, smp, coords).x);

   rgba = scale * (float4)(val,val,val,val);
   
   coords.x *= XM;
   
   write_imagef(dst, coords, rgba);
}

__kernel void UCharYUYV422ToRGBA(__read_only image2d_t src,
                    				 __write_only image2d_t dst)
{
   const sampler_t smp = CLK_NORMALIZED_COORDS_FALSE | CLK_ADDRESS_CLAMP_TO_EDGE | CLK_FILTER_NEAREST;

   int x = get_global_id(0)*2;
   int y = get_global_id(1);
   
   int type = get_image_channel_data_type(dst);
   
   float yc1 = 0.0f;
   float yc2 = 0.0f;
   float uc = 0.0f;
   float vc = 0.0f;
   float4 rgba1;
   float4 rgba2;
   uint4 result;
   
   uint4 val1 = read_imageui(src, smp, (int2)(x,y));
   uint4 val2 = read_imageui(src, smp, (int2)(x+1,y));
   
   yc1 = (float)val1.x - 16.0f;
   yc2 = (float)val2.x - 16.0f;
   uc = (float)val1.y - 128.0f;
   vc = (float)val2.y - 128.0f;
   
   // 1.164 0.000 1.793 * y-16
   // 1.164 -.213 -.533 * u-128
   // 1.164 2.112 0.000 * v-128
   
   rgba1.w = 255;
   rgba2.w = 255;

   rgba1.x = 1.164 * yc1 + 1.793 * vc;
   rgba1.y = 1.164 * yc1 - 0.213 * uc - 0.533 * vc;
   rgba1.z = 1.164 * yc1 + 2.112 * uc;
   rgba2.x = 1.164 * yc2 + 1.793 * vc;
   rgba2.y = 1.164 * yc2 - 0.213 * uc - 0.533 * vc;
   rgba2.z = 1.164 * yc2 + 2.112 * uc;

   if (type == CLK_FLOAT) {   
     write_imagef(dst, (int2)(x*XM,y), clamp(rgba1, 0.0f, 255.0f)/255.0f);
     write_imagef(dst, (int2)(x*XM+XM,y), clamp(rgba2, 0.0f, 255.0f)/255.0f);
   } else {
     result.x = clamp(rgba1.x, 0.0f, 255.0f);
     result.y = clamp(rgba1.y, 0.0f, 255.0f);
     result.z = clamp(rgba1.z, 0.0f, 255.0f);
     result.w = clamp(rgba1.w, 0.0f, 255.0f);
     write_imageui(dst, (int2)(x*XM,y), result);
     result.x = clamp(rgba2.x, 0.0f, 255.0f);
     result.y = clamp(rgba2.y, 0.0f, 255.0f);
     result.z = clamp(rgba2.z, 0.0f, 255.0f);
     result.w = clamp(rgba2.w, 0.0f, 255.0f);
     write_imageui(dst, (int2)(x*XM+XM,y), result);
   }
}

inline void yuyvtorgba10bit(float yc1, float yc2, float uc, float vc, float4 rgba[2]) {
   yc1 -= 16.0f * 4.0f;
   yc2 -= 16.0f * 4.0f;
   uc -= 128.0f * 4.0f;
   vc -= 128.0f * 4.0f;

   rgba[0] = (float4)(
     1.164 * yc1 + 1.793 * vc,
     1.164 * yc1 - 0.213 * uc - 0.533 * vc,
     1.164 * yc1 + 2.112 * uc,
     1023.0f // 10bit max
   ); 
   rgba[1] = (float4)(
     1.164 * yc2 + 1.793 * vc,
     1.164 * yc2 - 0.213 * uc - 0.533 * vc,
     1.164 * yc2 + 2.112 * uc,
     1023.0f // 10bit max
   ); 
}

__kernel void UCharYUYV422DPXToRGBA(__read_only image2d_t src,
                             __write_only image2d_t dst)
{
   const sampler_t smp = CLK_NORMALIZED_COORDS_FALSE | CLK_ADDRESS_CLAMP_TO_EDGE | CLK_FILTER_NEAREST;

   const int xraw = get_global_id(0)*8;
   const int yraw = get_global_id(1);
   
   const int2 srcdim = get_image_dim(src);
   const int2 dstdim = get_image_dim(dst);
   
   uint val[4];
   
   int x = xraw;
   
   uint4 pixel;
   
   pixel = read_imageui(src, smp, (int2)(xraw+0,yraw));
   val[0] = pixel.x * 0x1000000; // 2^24
   val[0] += pixel.y * 0x10000; // 2^16
   pixel = read_imageui(src, smp, (int2)(xraw+1,yraw));
   val[0] += pixel.x * 0x100; // 2^8
   val[0] += pixel.y;

   pixel = read_imageui(src, smp, (int2)(xraw+2,yraw));
   val[1] = pixel.x * 0x1000000; // 2^24
   val[1] += pixel.y * 0x10000; // 2^16
   pixel = read_imageui(src, smp, (int2)(xraw+3,yraw));
   val[1] += pixel.x * 0x100; // 2^8
   val[1] += pixel.y;

   pixel = read_imageui(src, smp, (int2)(xraw+4,yraw));
   val[2] = pixel.x * 0x1000000; // 2^24
   val[2] += pixel.y * 0x10000; // 2^16
   pixel = read_imageui(src, smp, (int2)(xraw+5,yraw));
   val[2] += pixel.x * 0x100; // 2^8
   val[2] += pixel.y;

   pixel = read_imageui(src, smp, (int2)(xraw+6,yraw));
   val[3] = pixel.x * 0x1000000; // 2^24
   val[3] += pixel.y * 0x10000; // 2^16
   pixel = read_imageui(src, smp, (int2)(xraw+7,yraw));
   val[3] += pixel.x * 0x100; // 2^8
   val[3] += pixel.y;
   
   float yc1[3];
   float yc2[3];
   float uc[3];
   float vc[3];
   
   yc1[0] = (float) ( (val[0] & 0xFFC00000) / 0x400000);
   uc[0]  = (float) ( (val[0] & 0x3FF000) / 0x1000);
   yc2[0] = (float) ( (val[0] & 0xFFC) / 0x4);

   vc[0]  = (float) ( (val[1] & 0xFFC00000) / 0x400000);
   yc1[1] = (float) ( (val[1] & 0x3FF000) / 0x1000);
   uc[1]  = (float) ( (val[1] & 0xFFC) / 0x4);

   yc2[1] = (float) ( (val[2] & 0xFFC00000) / 0x400000);
   vc[1]  = (float) ( (val[2] & 0x3FF000) / 0x1000);
   yc1[2] = (float) ( (val[2] & 0xFFC) / 0x4);

   uc[2]  = (float) ( (val[3] & 0xFFC00000) / 0x400000);
   yc2[2] = (float) ( (val[3] & 0x3FF000) / 0x1000);
   vc[2]  = (float) ( (val[3] & 0xFFC) / 0x4);
   
   const float max10bit = 1023.0f;

   float4 rgba[2];
   int xpos = (yraw * srcdim.x + xraw)*6/8;
   
   if ( (xpos+4) / dstdim.x < dstdim.y) {
   
     x = xpos % dstdim.x;
     int y = xpos / dstdim.x;
     yuyvtorgba10bit(yc1[0], yc2[0], uc[0], vc[0], rgba);
     write_imagef(dst, (int2)(x*XM,y), clamp(rgba[0], 0.0f, max10bit)/max10bit);
     write_imagef(dst, (int2)(x*XM+XM,y), clamp(rgba[1], 0.0f, max10bit)/max10bit);

     xpos = xpos +2;
     x = xpos % dstdim.x;
     y = xpos / dstdim.x;
     yuyvtorgba10bit(yc1[1], yc2[1], uc[1], vc[1], rgba);
     write_imagef(dst, (int2)(x*XM,y), clamp(rgba[0], 0.0f, max10bit)/max10bit);
     write_imagef(dst, (int2)(x*XM+XM,y), clamp(rgba[1], 0.0f, max10bit)/max10bit);

     xpos = xpos +2;
     x = xpos % dstdim.x;
     y = xpos / dstdim.x;
     yuyvtorgba10bit(yc1[2], yc2[2], uc[2], vc[2], rgba);
     write_imagef(dst, (int2)(x*XM,y), clamp(rgba[0], 0.0f, max10bit)/max10bit);
     write_imagef(dst, (int2)(x*XM+XM,y), clamp(rgba[1], 0.0f, max10bit)/max10bit);

   }
}

__kernel void UCharYUYV422DPXToRGBA1280x720(__read_only image2d_t src,
                             __write_only image2d_t dst)
{
   const sampler_t smp = CLK_NORMALIZED_COORDS_FALSE | CLK_ADDRESS_CLAMP_TO_EDGE | CLK_FILTER_NEAREST;

   const int xraw = (get_global_id(0)*8) % 1926;
   const int yraw = (get_global_id(0)*8) / 1926;
   
   const int2 srcdim = get_image_dim(src);
   const int2 dstdim = get_image_dim(dst);
   
   uint val[4];
   
   int x = xraw;
   int y = yraw;
   
   uint4 pixel;
   
   pixel = read_imageui(src, smp, (int2)(x,y));
   val[0] = pixel.x * 0x1000000; // 2^24
   val[0] += pixel.y * 0x10000; // 2^16
   x = (x+1)%1926;
   y = y+(x==0 && get_global_id(0)!=0);
   pixel = read_imageui(src, smp, (int2)(x,y));
   val[0] += pixel.x * 0x100; // 2^8
   val[0] += pixel.y;

   x = (x+1)%1926;
   y = y+(x==0);
   pixel = read_imageui(src, smp, (int2)(x,y));
   val[1] = pixel.x * 0x1000000; // 2^24
   val[1] += pixel.y * 0x10000; // 2^16
   x = (x+1)%1926;
   y = y+(x==0);
   pixel = read_imageui(src, smp, (int2)(x,y));
   val[1] += pixel.x * 0x100; // 2^8
   val[1] += pixel.y;

   x = (x+1)%1926;
   y = y+(x==0);
   pixel = read_imageui(src, smp, (int2)(x,y));
   val[2] = pixel.x * 0x1000000; // 2^24
   val[2] += pixel.y * 0x10000; // 2^16
   x = (x+1)%1926;
   y = y+(x==0);
   pixel = read_imageui(src, smp, (int2)(x,y));
   val[2] += pixel.x * 0x100; // 2^8
   val[2] += pixel.y;

   x = (x+1)%1926;
   y = y+(x==0);
   pixel = read_imageui(src, smp, (int2)(x,y));
   val[3] = pixel.x * 0x1000000; // 2^24
   val[3] += pixel.y * 0x10000; // 2^16
   x = (x+1)%1926;
   y = y+(x==0);
   pixel = read_imageui(src, smp, (int2)(x,y));
   val[3] += pixel.x * 0x100; // 2^8
   val[3] += pixel.y;
   
   float yc1[3];
   float yc2[3];
   float uc[3];
   float vc[3];
   
   yc1[0] = (float) ( (val[0] & 0xFFC00000) / 0x400000);
   uc[0]  = (float) ( (val[0] & 0x3FF000) / 0x1000);
   yc2[0] = (float) ( (val[0] & 0xFFC) / 0x4);

   vc[0]  = (float) ( (val[1] & 0xFFC00000) / 0x400000);
   yc1[1] = (float) ( (val[1] & 0x3FF000) / 0x1000);
   uc[1]  = (float) ( (val[1] & 0xFFC) / 0x4);

   yc2[1] = (float) ( (val[2] & 0xFFC00000) / 0x400000);
   vc[1]  = (float) ( (val[2] & 0x3FF000) / 0x1000);
   yc1[2] = (float) ( (val[2] & 0xFFC) / 0x4);

   uc[2]  = (float) ( (val[3] & 0xFFC00000) / 0x400000);
   yc2[2] = (float) ( (val[3] & 0x3FF000) / 0x1000);
   vc[2]  = (float) ( (val[3] & 0xFFC) / 0x4);
   
   const float max10bit = 1023.0f;

   float4 rgba[2];
   int xpos = (yraw * srcdim.x + xraw)*6/8;
   
   if ( (xpos+4+1) < 1280*720) {
     x = xpos % 1284;
     if (x%1284<1280) {
       y = xpos / 1284;
       yuyvtorgba10bit(yc1[0], yc2[0], uc[0], vc[0], rgba);
       write_imagef(dst, (int2)(x*XM,y), clamp(rgba[0], 0.0f, max10bit)/max10bit);
       write_imagef(dst, (int2)(x*XM+XM,y), clamp(rgba[1], 0.0f, max10bit)/max10bit);
     }
     x = (xpos+2) % 1284;
     if (x%1284<1280) {
       y = (xpos+2) / 1284;
       yuyvtorgba10bit(yc1[1], yc2[1], uc[1], vc[1], rgba);
       write_imagef(dst, (int2)(x*XM,y), clamp(rgba[0], 0.0f, max10bit)/max10bit);
       write_imagef(dst, (int2)(x*XM+XM,y), clamp(rgba[1], 0.0f, max10bit)/max10bit);
     }
     x = (xpos+4) % 1284;
     if (x%1284<1280) {
       y = (xpos+4) / 1284;
       yuyvtorgba10bit(yc1[2], yc2[2], uc[2], vc[2], rgba);
       write_imagef(dst, (int2)(x*XM,y), clamp(rgba[0], 0.0f, max10bit)/max10bit);
       write_imagef(dst, (int2)(x*XM+XM,y), clamp(rgba[1], 0.0f, max10bit)/max10bit);
     }
   }
}

__kernel void UCharRGBDVSToRGBA(__read_only image2d_t src,
                             __write_only image2d_t dst)
{
   const sampler_t smp = CLK_NORMALIZED_COORDS_FALSE | CLK_ADDRESS_CLAMP_TO_EDGE | CLK_FILTER_NEAREST;

   const int x = get_global_id(0);
   const int y = get_global_id(1);
   
   uint4 rgba = (read_imageui(src, smp, (int2)(x,y)));
   
   float4 color = (float4)((float)rgba.x, (float)rgba.y, (float)rgba.z, 0.0f) * 4.0f;
   uint a = rgba.w;
   color = color + (float4)((float)(a & 0x03), (float)((a & 0x0C) / 4), (float) ((a & 0x30) / 16), 0.0f);
   color.w = 1023.0f;
   write_imagef(dst, (int2)(x*XM,y), color);
}

__kernel void RGBToLuv(__read_only image2d_t src,
                       __write_only image2d_t dst)
{
   const sampler_t smp = CLK_NORMALIZED_COORDS_FALSE | CLK_ADDRESS_CLAMP_TO_EDGE | CLK_FILTER_NEAREST;

   int x = get_global_id(0);
   int y = get_global_id(1);
   
   int intype  = get_image_channel_data_type(src);
   
   float4 rgba;
   if (intype == CLK_FLOAT) {
     rgba = read_imagef(src, smp, (int2)(x,y));
   } else {
     uint4 urgba = read_imageui(src, smp, (int2)(x,y));
     // todo nicer way?
     rgba.x = (1.0f/255.0f) * (float)urgba.x;
     rgba.y = (1.0f/255.0f) * (float)urgba.y;
     rgba.z = (1.0f/255.0f) * (float)urgba.z;
     rgba.w = (1.0f/255.0f) * (float)urgba.w;
   }
    
   float X = 0.412453*rgba.x + 0.35758 *rgba.y + 0.180423*rgba.z;
   float Y = 0.212671*rgba.x + 0.71516 *rgba.y + 0.072169*rgba.z;
   float Z = 0.019334*rgba.x + 0.119193*rgba.y + 0.950227*rgba.z;
   
   float xn = 0.312713;
   float yn = 0.329016;
   float Yn = 1.0;
   
   float un = 4.0*xn / (-2.0*xn + 12.0*yn + 3.0);
   float vn = 9.0*yn / (-2.0*xn + 12.0*yn + 3.0);
   float u = 4.0*X / (X + 15.0*Y + 3.0*Z);
   float v = 9.0*Y / (X + 15.0*Y + 3.0*Z);
   float4 luv;
   luv.x = 116.0 * pow(Y/Yn,1.0f/3.0f) - 16.0f;
   luv.y = 13.0*luv.x*(u-un);
   luv.z = 13.0*luv.x*(v-vn);
   luv.w = 1.0f;
   
   write_imagef(dst, (int2)(x*XM,y), luv);
}

__kernel void ScaleColorOfRGBPlusDepth(__read_only image2d_t src,
                       __write_only image2d_t dst, float scale)
{
   const sampler_t smp = CLK_NORMALIZED_COORDS_FALSE | CLK_ADDRESS_CLAMP_TO_EDGE | CLK_FILTER_NEAREST;

   int x = get_global_id(0);
   int y = get_global_id(1);
   
   float4 rgba = read_imagef(src, smp, (int2)(x,y));
   float4 scaled = (float4)(scale*rgba.x,scale*rgba.y,scale*rgba.z,1.0f*rgba.w);
   write_imagef(dst, (int2)(x*XM,y), scaled);
}


__kernel void RGBAToPackedGrey
(
__read_only image2d_t img1,
__read_only image2d_t img2,
__write_only image2d_t converted,
int downsampleBy2 // 0 no , 1 both dirs, 2 horizontal only, 3 vertical only
)
{
const sampler_t smp = CLK_NORMALIZED_COORDS_FALSE | CLK_ADDRESS_CLAMP_TO_EDGE | CLK_FILTER_NEAREST;

int2 xy;
xy.x = get_global_id( 0 );
xy.y = get_global_id( 1 );

int2 down_xy = xy;
if (downsampleBy2==1) {
down_xy *= 2;
}

if (downsampleBy2==2) {
down_xy.y *= 2;
}

if (downsampleBy2==3) {
down_xy.x *= 2;
}

int intype  = get_image_channel_data_type(img1);

uchar color1 = readColorAsGrey(img1, down_xy, intype, smp);
uchar color2 = readColorAsGrey(img2, down_xy, intype, smp);

write_imageui(converted, xy, (uint4)(color1,color2,0,0));
}


__kernel void UCharRGBAToYUV420P(__read_only image2d_t src, __write_only image2d_t dst)
{
   const sampler_t smp = CLK_NORMALIZED_COORDS_FALSE | CLK_ADDRESS_CLAMP_TO_EDGE | CLK_FILTER_NEAREST;

   int x = get_global_id(0);
   int y = get_global_id(1);
   
   int2 dim = get_image_dim(src);
   
   uint4 rgba = read_imageui(src, smp, (int2)(x,y));
   
   uint Y = clamp ( (( 66 * (rgba.x) + 129 * (rgba.y) +  25 * (rgba.z) + 128) >> 8) +  16, (uint)0, (uint)255);
   uint U = clamp ( (( -38 * ((int)rgba.x) -  74 * ((int)rgba.y) + 112 * ((int)rgba.z) + 128) >> 8) + 128, 0, 255);
   uint V = clamp ( (( 112 * ((int)rgba.x) -  94 * ((int)rgba.y) -  18 * ((int)rgba.z) + 128) >> 8) + 128, 0, 255);

   write_imageui(dst, (int2)(x,y), Y);
   write_imageui(dst, (int2)(x/2+y%2*dim.x/2,dim.y+y/4), U);
   write_imageui(dst, (int2)(x/2+y%2*dim.x/2,dim.y+dim.y/4+y/4), V);
}

__kernel void FloatRGBAToYUV420P(__read_only image2d_t src, __write_only image2d_t dst, float scale)
{
   const sampler_t smp = CLK_NORMALIZED_COORDS_FALSE | CLK_ADDRESS_CLAMP_TO_EDGE | CLK_FILTER_NEAREST;

   int x = get_global_id(0);
   int y = get_global_id(1);
   
   int2 dim = get_image_dim(src);
   
   float4 rgbaf = read_imagef(src, smp, (int2)(x,y)) * scale;
   uint4 rgba = (uint4)(rgbaf.x,rgbaf.y,rgbaf.z,rgbaf.w);
   
   uint Y = clamp ( (( 66 * (rgba.x) + 129 * (rgba.y) +  25 * (rgba.z) + 128) >> 8) +  16, (uint)0, (uint)255);
   uint U = clamp ( (( -38 * ((int)rgba.x) -  74 * ((int)rgba.y) + 112 * ((int)rgba.z) + 128) >> 8) + 128, 0, 255);
   uint V = clamp ( (( 112 * ((int)rgba.x) -  94 * ((int)rgba.y) -  18 * ((int)rgba.z) + 128) >> 8) + 128, 0, 255);

   write_imageui(dst, (int2)(x,y), Y);
   write_imageui(dst, (int2)(x/2+y%2*dim.x/2,dim.y+y/4), U);
   write_imageui(dst, (int2)(x/2+y%2*dim.x/2,dim.y+dim.y/4+y/4), V);
}

