#define BLOCK_SIZE 16

inline void readToLocalUi(const int x, const int y, const int lx, const int ly, const int2 blocksize,
                          image2d_t src, __local uint P[BLOCK_SIZE+2][BLOCK_SIZE+2]) {

   sampler_t smp = CLK_NORMALIZED_COORDS_FALSE | CLK_ADDRESS_CLAMP_TO_EDGE | CLK_FILTER_NEAREST;
   
   // load pixel
   P[lx][ly] = (read_imageui(src, smp, (int2)(x,y)).x);
   
   if (lx == blocksize.x) {
     P[blocksize.x+1][ly] = (read_imageui(src, smp, (int2)(x+1,y)).x);
   }
   if (ly == blocksize.y) {
     P[lx][blocksize.y+1] = (read_imageui(src, smp, (int2)(x,y+1)).x);
   }
   if (lx == blocksize.x && ly == blocksize.y) {
     P[blocksize.x+1][blocksize.y+1] = (read_imageui(src, smp, (int2)(x+1,y+1)).x);
   }
   if (lx == 1) {
     P[0][ly] = (read_imageui(src, smp, (int2)(x-1,y)).x);
   }
   if (ly == 1) {
     P[lx][0] = (read_imageui(src, smp, (int2)(x,y-1)).x);
   }
   if (lx == 1 && ly == 1) {
     P[0][0] = (read_imageui(src, smp, (int2)(x-1,y-1)).x);
     P[blocksize.x+1][0] = (read_imageui(src, smp, (int2)(x+blocksize.x,y-1)).x);
     P[0][blocksize.y+1] = (read_imageui(src, smp, (int2)(x-1,y+blocksize.y)).x);
   }
}

__kernel void UCharBayerRGGBToRGBA(__read_only image2d_t src,
                    __write_only image2d_t dst, float scale)
{
         
   int x = get_global_id(0);
   int y = get_global_id(1);
   
   int lx = get_local_id(0) + 1;
   int ly = get_local_id(1) + 1;
   
   int type = get_image_channel_data_type(dst);
   
   __local uint P[BLOCK_SIZE+2][BLOCK_SIZE+2];
   
   const int2 blocksize = (int2)(get_local_size(0), get_local_size(1));
   readToLocalUi(x, y, lx, ly, blocksize, src, P);
   
   barrier(CLK_LOCAL_MEM_FENCE);
   
   float4 rgba;
   rgba.x = 1.0;
   rgba.y = 1.0;
   rgba.z = 1.0;
   rgba.w = 1.0;
   
   if (y % 2 == 0) { // even row
     if (x % 2 == 0) { // even col
       rgba.x = P[lx][ly];
       rgba.y = 0.25 * (P[lx+1][ly] + P[lx-1][ly] + P[lx][ly+1] + P[lx][ly-1]);
       rgba.z = 0.25 * (P[lx-1][ly-1] + P[lx+1][ly-1] + P[lx-1][ly+1] + P[lx+1][ly+1]);
     } 
     else { // odd col
       rgba.x = 0.5 * (P[lx-1][ly] + P[lx+1][ly]);
       rgba.y = P[lx][ly];
       rgba.z = 0.5 * (P[lx][ly-1] + P[lx][ly+1]);
     }
   } 
   else { // odd row
     if (x % 2 == 0) { // even col
       rgba.x = 0.5 * (P[lx][ly-1] + P[lx][ly+1]);
       rgba.y = P[lx][ly];
       rgba.z = 0.5 * (P[lx-1][ly] + P[lx+1][ly]);
     } 
     else { // odd col
       rgba.x = 0.25 * (P[lx-1][ly-1] + P[lx+1][ly-1] + P[lx-1][ly+1] + P[lx+1][ly+1]);
       rgba.y = 0.25 * (P[lx+1][ly] + P[lx-1][ly] + P[lx][ly+1] + P[lx][ly-1]);
       rgba.z = P[lx][ly];
     }
   }
   rgba.w = scale;
   if (type == CLK_FLOAT) {
   	 write_imagef(dst, (int2)(XM*x,y), rgba/scale);
   } else {
     uint4 result;
     result.x = clamp(rgba.x, 0.0f,scale);
     result.y = clamp(rgba.y, 0.0f,scale);
     result.z = clamp(rgba.z, 0.0f,scale);
     result.w = clamp(rgba.w, 0.0f,scale);
   	 write_imageui(dst, (int2)(XM*x,y), result);
   }
}


__kernel void UCharBayerGRBGToRGBA(__read_only image2d_t src,
                    __write_only image2d_t dst, float scale)
{
         
   int x = get_global_id(0);
   int y = get_global_id(1);
   
   int lx = get_local_id(0) + 1;
   int ly = get_local_id(1) + 1;
   
   int type = get_image_channel_data_type(dst);
   
   __local uint P[BLOCK_SIZE+2][BLOCK_SIZE+2];
   
   const int2 blocksize = (int2)(get_local_size(0), get_local_size(1));
   readToLocalUi(x, y, lx, ly, blocksize, src, P);
   
   barrier(CLK_LOCAL_MEM_FENCE);
   
   float4 rgba;
   rgba.x = 1.0;
   rgba.y = 1.0;
   rgba.z = 1.0;
   rgba.w = 1.0;
   
   if (y % 2 == 0) { // even row
     if (x % 2 == 0) { // even col
       rgba.x = 0.5 * (P[lx-1][ly] + P[lx+1][ly]);
       rgba.y = P[lx][ly];
       rgba.z = 0.5 * (P[lx][ly-1] + P[lx][ly+1]);
     } 
     else { // odd col
       rgba.x = P[lx][ly];
       rgba.y = 0.25 * (P[lx+1][ly] + P[lx-1][ly] + P[lx][ly+1] + P[lx][ly-1]);
       rgba.z = 0.25 * (P[lx-1][ly-1] + P[lx+1][ly-1] + P[lx-1][ly+1] + P[lx+1][ly+1]);
     }
   } 
   else { // odd row
     if (x % 2 == 0) { // even col
       rgba.x = 0.25 * (P[lx-1][ly-1] + P[lx+1][ly-1] + P[lx-1][ly+1] + P[lx+1][ly+1]);
       rgba.y = 0.25 * (P[lx+1][ly] + P[lx-1][ly] + P[lx][ly+1] + P[lx][ly-1]);
       rgba.z = P[lx][ly];
     } 
     else { // odd col
       rgba.x = 0.5 * (P[lx][ly-1] + P[lx][ly+1]);
       rgba.y = P[lx][ly];
       rgba.z = 0.5 * (P[lx-1][ly] + P[lx+1][ly]);
     }
   }
   rgba.w = scale;
   if (type == CLK_FLOAT) {
     write_imagef(dst, (int2)(XM*x,y), rgba/scale);
   } else {
     uint4 result;
     result.x = clamp(rgba.x, 0.0f,scale);
     result.y = clamp(rgba.y, 0.0f,scale);
     result.z = clamp(rgba.z, 0.0f,scale);
     result.w = clamp(rgba.w, 0.0f,scale);
     write_imageui(dst, (int2)(XM*x,y), result);
   }
}

__kernel void UCharBayerGBRGToRGBA(__read_only image2d_t src,
                    __write_only image2d_t dst, float scale)
{
         
   int x = get_global_id(0);
   int y = get_global_id(1);
   
   int lx = get_local_id(0) + 1;
   int ly = get_local_id(1) + 1;
   
   int type = get_image_channel_data_type(dst);
   
   __local uint P[BLOCK_SIZE+2][BLOCK_SIZE+2];
   
   const int2 blocksize = (int2)(get_local_size(0), get_local_size(1));
   readToLocalUi(x, y, lx, ly, blocksize, src, P);
   
   barrier(CLK_LOCAL_MEM_FENCE);
   
   float4 rgba;
   rgba.x = 1.0;
   rgba.y = 1.0;
   rgba.z = 1.0;
   rgba.w = 1.0;
   
   if (y % 2 == 0) { // even row
     if (x % 2 == 0) { // even col
       rgba.z = 0.5 * (P[lx-1][ly] + P[lx+1][ly]);
       rgba.y = P[lx][ly];
       rgba.x = 0.5 * (P[lx][ly-1] + P[lx][ly+1]);
     } 
     else { // odd col
       rgba.z = P[lx][ly];
       rgba.y = 0.25 * (P[lx+1][ly] + P[lx-1][ly] + P[lx][ly+1] + P[lx][ly-1]);
       rgba.x = 0.25 * (P[lx-1][ly-1] + P[lx+1][ly-1] + P[lx-1][ly+1] + P[lx+1][ly+1]);
     }
   } 
   else { // odd row
     if (x % 2 == 0) { // even col
       rgba.z = 0.25 * (P[lx-1][ly-1] + P[lx+1][ly-1] + P[lx-1][ly+1] + P[lx+1][ly+1]);
       rgba.y = 0.25 * (P[lx+1][ly] + P[lx-1][ly] + P[lx][ly+1] + P[lx][ly-1]);
       rgba.x = P[lx][ly];
     } 
     else { // odd col
       rgba.z = 0.5 * (P[lx][ly-1] + P[lx][ly+1]);
       rgba.y = P[lx][ly];
       rgba.x = 0.5 * (P[lx-1][ly] + P[lx+1][ly]);
     }
   }
   rgba.w = scale;
   if (type == CLK_FLOAT) {
     write_imagef(dst, (int2)(XM*x,y), rgba/scale);
   } else {
     uint4 result;
     result.x = clamp(rgba.x, 0.0f,scale);
     result.y = clamp(rgba.y, 0.0f,scale);
     result.z = clamp(rgba.z, 0.0f,scale);
     result.w = clamp(rgba.w, 0.0f,scale);
     write_imageui(dst, (int2)(XM*x,y), result);
   }
}

__kernel void UCharBayerBGGRToRGBA(__read_only image2d_t src,
                    __write_only image2d_t dst, float scale)
{

   int x = get_global_id(0);
   int y = get_global_id(1);

   int lx = get_local_id(0) + 1;
   int ly = get_local_id(1) + 1;

   int type = get_image_channel_data_type(dst);

   __local uint P[BLOCK_SIZE+2][BLOCK_SIZE+2];

   const int2 blocksize = (int2)(get_local_size(0), get_local_size(1));
   readToLocalUi(x, y, lx, ly, blocksize, src, P);
   barrier(CLK_LOCAL_MEM_FENCE);

   float4 rgba;
   rgba.x = 1.0;
   rgba.y = 1.0;
   rgba.z = 1.0;
   rgba.w = 1.0;

   if (y % 2 == 0) { // even row
     if (x % 2 == 0) { // even col
       rgba.x = 0.25 * (P[lx-1][ly-1] + P[lx+1][ly-1] + P[lx-1][ly+1] + P[lx+1][ly+1]);
       rgba.y = 0.25 * (P[lx+1][ly] + P[lx-1][ly] + P[lx][ly+1] + P[lx][ly-1]);
       rgba.z = P[lx][ly];
     }
     else { // odd col
       rgba.x = 0.5 * (P[lx][ly-1] + P[lx][ly+1]);
       rgba.y = P[lx][ly];
       rgba.z = 0.5 * (P[lx-1][ly] + P[lx+1][ly]);
     }
   }
   else { // odd row
     if (x % 2 == 0) { // even col
       rgba.x = 0.5 * (P[lx-1][ly] + P[lx+1][ly]);
       rgba.y = P[lx][ly];
       rgba.z = 0.5 * (P[lx][ly-1] + P[lx][ly+1]);
     }
     else { // odd col
       rgba.x = P[lx][ly];
       rgba.y = 0.25 * (P[lx+1][ly] + P[lx-1][ly] + P[lx][ly+1] + P[lx][ly-1]);
       rgba.z = 0.25 * (P[lx-1][ly-1] + P[lx+1][ly-1] + P[lx-1][ly+1] + P[lx+1][ly+1]);
     }
   }

   rgba.w = scale;
   if (type == CLK_FLOAT) {
     write_imagef(dst, (int2)(XM*x,y), rgba/scale);
   } else {
     uint4 result;
     result.x = clamp(rgba.x, 0.0f,scale);
     result.y = clamp(rgba.y, 0.0f,scale);
     result.z = clamp(rgba.z, 0.0f,scale);
     result.w = clamp(rgba.w, 0.0f,scale);
     write_imageui(dst, (int2)(XM*x,y), result);
   }
}
