/* 
 This file is part of the BIAS library (Basic ImageAlgorithmS).

 Copyright (C) 2003, 2004    (see file CONTACTS for details)
 Multimediale Systeme der Informationsverarbeitung
 Institut fuer Informatik
 Christian-Albrechts-Universitaet Kiel


 BIAS is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 (at your option) any later version.

 BIAS is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with BIAS; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <Base/Common/BIASpragma.hh>
#include <iostream>
#include "clfColorConversion.hh"
#include <Base/Image/ImageIO.hh>

using namespace std;
using namespace BIAS;

namespace BIAS {

clfColorConversion::
clfColorConversion(clfContext *ctx) {
  context_ = ctx;
  program_ = context_->CreateProgram();
  program_->AddSource("/OpenCLFramework/Filter/cl/colorconversion.cl");
  program_->AddSource("/OpenCLFramework/Filter/cl/colorconversionbayer.cl");
  program_->Build();
  program_->AddKernel("clear");
  program_->AddKernel("RGBToRGBA");
  program_->AddKernel("FloatGreyToRGBA");
  program_->AddKernel("RGBToGrey");
  program_->AddKernel("RGBAToGrey");
  program_->AddKernel("FloatGreyToFloatGrey");
  program_->AddKernel("UCharGreyToFloatGrey");
  program_->AddKernel("UCharRGBToFloatGreyNormalized");
  program_->AddKernel("UCharBayerRGGBToRGBA");
  program_->AddKernel("UCharBayerGRBGToRGBA");
  program_->AddKernel("UCharBayerGBRGToRGBA");
  program_->AddKernel("UCharBayerBGGRToRGBA");
  program_->AddKernel("UCharRGBAToYUV420P");
  program_->AddKernel("UCharYUYV422ToRGBA");
  program_->AddKernel("UCharYUYV422DPXToRGBA");
  program_->AddKernel("UCharYUYV422DPXToRGBA1280x720");
  program_->AddKernel("UCharRGBDVSToRGBA");
  program_->AddKernel("RGBToLuv");
  program_->AddKernel("ScaleColorOfRGBPlusDepth");
  program_->AddKernel("RGBAToPackedGrey");
}

clfColorConversion::
~clfColorConversion() {
  delete program_;
}

int clfColorConversion::Clear(clfImage2D *dst, float r,float g,float b,float a) {
  program_->KernelSetArgument( "clear", 0, *dst );
  program_->KernelSetArgument( "clear", 1, r );
  program_->KernelSetArgument( "clear", 2, g );
  program_->KernelSetArgument( "clear", 3, b );
  program_->KernelSetArgument( "clear", 4, a );
  unsigned int w,h;
  dst->GetImageDim(w,h);
  context_->RunOn2DRange( *program_, "clear", context_->DivUp(16,w),context_->DivUp(16,h),16,16);

  return 0;
}


//  int clfColorConversion::
//  RgbToRgba(clfImage2D *src, clfImage2D *dst) {
//    try {
//      program_->KernelSetArgument( "RgbToRgba", 0, *src );
//      program_->KernelSetArgument( "RgbToRgba", 0, *dst );
//      int w,h;
//      src->GetImageDim(w,h);
//      context_->RunOn2DRange( *program_, "RgbToRgba", w,h, 16,16);
//    } catch (clfException &e) {
//      BIASERR(e.GetDetailedString());
//      return 1;
//    }
//    return 0;
//  }

int clfColorConversion::
ToRGBA(clfImage2D *src, clfImage2D *dst, bool normalize) {
  if (src->StorageType() != ImageBase::ST_unsignedchar &&
      src->StorageType() != ImageBase::ST_float)
  {
    BIASERR("generic ToRGBA only available for unsigned char and float input");
    return -1;
  }
  switch (src->ColorModel()) {
  case ImageBase::CM_Bayer_BGGR:
  case ImageBase::CM_Bayer_GBRG:
  case ImageBase::CM_Bayer_GRBG:
  case ImageBase::CM_Bayer_RGGB:
    if (normalize){ BIASWARNONCE("normalize not available for bayer conversion");}
    return UCharBayerToRGBA(src, dst, src->ColorModel());
  case ImageBase::CM_YUYV422:
    if (normalize){ BIASWARNONCE("normalize not available for yuv conversion");}
    return UCharYUYV422ToRGBA(src, dst);
  case ImageBase::CM_RGB:
  case ImageBase::CM_Grey:
    return RGBToRGBA(src, dst, normalize);
  default:
    BIASERR("generic ToRGBA not available for this specific input color model");
    return -1;
  }
}

int clfColorConversion::
RGBToRGBA(clfImage2D *src, clfImage2D *dst, bool normalize) {
  try {
    float scale = 1.0f;
    if (normalize) scale = 1.0f / 255.0f;
    program_->KernelSetArgument( "RGBToRGBA", 0, *src );
    program_->KernelSetArgument( "RGBToRGBA", 1, *dst );
    program_->KernelSetArgument( "RGBToRGBA", 2, scale );
    unsigned int w,h;
    src->GetImageDim(w,h);
    h/=3;
    context_->RunOn2DRange( *program_, "RGBToRGBA", w,h, getlocsize_(w), getlocsize_(h));
  } catch (clfException &e) {
    BIASERR(e.GetDetailedString());
    return 1;
  }
  return 0;
}

int clfColorConversion::
UCharRGBToFloatGreyNormalized(clfImage2D *src, clfImage2D *dst) {
  try {
    program_->KernelSetArgument( "UCharRGBToFloatGreyNormalized", 0, *src );
    program_->KernelSetArgument( "UCharRGBToFloatGreyNormalized", 1, *dst );
    unsigned int w,h;
    src->GetImageDim(w,h);
    context_->RunOn2DRange( *program_, "UCharRGBToFloatGreyNormalized", w,h/3);
  } catch (clfException &e) {
    BIASERR(e.GetDetailedString());
    return 1;
  }
  return 0;
}

int clfColorConversion::
RGBToGrey(clfImage2D *src, clfImage2D *dst, bool normalize) {
  try {
    float scale = 1.0f;
    if (normalize) scale = 1.0f / 255.0f;
    unsigned int w,h;
    dst->GetImageDim(w,h);
    std::string kernelname = "RGBToGrey";
    if (src->ColorModel() == ImageBase::CM_RGBA || src->IsSharedGL()) {
      kernelname = "RGBAToGrey";
      //h*=3;
    } else {
      // nothing
    }
    program_->KernelSetArgument( kernelname, 0, *src );
    program_->KernelSetArgument( kernelname, 1, *dst );
    program_->KernelSetArgument( kernelname, 2, scale );
    context_->RunOn2DRange( *program_, kernelname, w,h);
  } catch (clfException &e) {
    BIASERR(e.GetDetailedString());
    return 1;
  }
  return 0;
}


int clfColorConversion::
FloatGreyToRGBA(clfImage2D *src, clfImage2D *dst, bool normalize){
  try {
    float scale = 1.0f;
    if (normalize) scale = 255.0f / 15.0f;
    program_->KernelSetArgument( "FloatGreyToRGBA", 0, *src );
    program_->KernelSetArgument( "FloatGreyToRGBA", 1, *dst );
    program_->KernelSetArgument( "FloatGreyToRGBA", 2, scale );
    unsigned int w,h;
    src->GetImageDim(w,h);
    context_->RunOn2DRange( *program_, "FloatGreyToRGBA", w,h);
  } catch (clfException &e) {
    BIASERR(e.GetDetailedString());
    return 1;
  }
  return 0;
}


int clfColorConversion::
FloatGreyToFloatGrey(clfImage2D *src, clfImage2D *dst) {
  try {
    program_->KernelSetArgument( "FloatGreyToFloatGrey", 0, *src );
    program_->KernelSetArgument( "FloatGreyToFloatGrey", 1, *dst );
    unsigned int w,h;
    src->GetImageDim(w,h);
    context_->RunOn2DRange( *program_, "FloatGreyToFloatGrey", w,h);
  } catch (clfException &e) {
    BIASERR(e.GetDetailedString());
    return 1;
  }
  return 0;
}

int clfColorConversion::
UCharGreyToFloatGrey(clfImage2D *src, clfImage2D *dst, bool normalize) {
  float scale = 1.0f;
  if (normalize) {
    scale = 1.0f / (float)((1<<src->GetBitDepth())-1);
  }
  try {
    program_->KernelSetArgument( "UCharGreyToFloatGrey", 0, *src );
    program_->KernelSetArgument( "UCharGreyToFloatGrey", 1, *dst );
    program_->KernelSetArgument( "UCharGreyToFloatGrey", 2, scale );
    unsigned int w,h;
    src->GetImageDim(w,h);
    context_->RunOn2DRange( *program_, "UCharGreyToFloatGrey", w,h);
  } catch (clfException &e) {
    BIASERR(e.GetDetailedString());
    return 1;
  }
  return 0;
}

int clfColorConversion::UCharBayerToRGBA(clfImage2D *src, clfImage2D *dst, BIAS::ImageBase::EColorModel bayermode, float scale) {
  string kernel = "";
  switch (bayermode) {
  case ImageBase::CM_Bayer_BGGR:
    kernel = "UCharBayerBGGRToRGBA";
    break;
  case ImageBase::CM_Bayer_GBRG:
    kernel = "UCharBayerGBRGToRGBA";
    break;
  case ImageBase::CM_Bayer_GRBG:
    kernel = "UCharBayerGRBGToRGBA";
    break;
  case ImageBase::CM_Bayer_RGGB:
    kernel = "UCharBayerRGGBToRGBA";
    break;
  default:
    kernel = "UCharBayerRGGBToRGBA";
    break;
  }
  try {
    //      if (src->GetBitDepth() > 16) {
    //        BIASERR("VERY BAD!")
    //      }
    unsigned long int maxval = (1 << src->GetBitDepth()) - 1;
    program_->KernelSetArgument( kernel, 0, *src );
    program_->KernelSetArgument( kernel, 1, *dst );
    if (scale > 0.0f) {
      program_->KernelSetArgument( kernel, 2, 1.0f/scale );
    } else {
      program_->KernelSetArgument( kernel, 2, (float)maxval );
    }
    unsigned int w,h;
    src->GetImageDim(w,h);
    context_->RunOn2DRange( *program_, kernel, w,h, getlocsize_(w), getlocsize_(h));
  } catch (clfException &e) {
    BIASERR(e.GetDetailedString());
    return 1;
  }
  return 0;
}

int clfColorConversion::UCharRGBAToYUV420P(clfImage2D *src, clfImage2D *dst) {
  try {
    program_->KernelSetArgument( "UCharRGBAToYUV420P", 0, *src );
    program_->KernelSetArgument( "UCharRGBAToYUV420P", 1, *dst );
    unsigned int w,h;
    src->GetImageDim(w,h);
//    w/=2;
//    h/=2;
    context_->RunOn2DRange( *program_, "UCharRGBAToYUV420P", w,h, getlocsize_(w), getlocsize_(h));
  } catch (clfException &e) {
    BIASERR(e.GetDetailedString());
    return 1;
  }
  return 0;
}

int clfColorConversion::UCharYUYV422ToRGBA(clfImage2D *src, clfImage2D *dst) {
  try {
    program_->KernelSetArgument( "UCharYUYV422ToRGBA", 0, *src );
    program_->KernelSetArgument( "UCharYUYV422ToRGBA", 1, *dst );
    unsigned int w,h;
    src->GetImageDim(w,h);
    w/=2;
    context_->RunOn2DRange( *program_, "UCharYUYV422ToRGBA", w,h, getlocsize_(w), getlocsize_(h));
  } catch (clfException &e) {
    BIASERR(e.GetDetailedString());
    return 1;
  }
  return 0;
}
int clfColorConversion::UCharYUYV422DPXToRGBA(clfImage2D *src, clfImage2D *dst) {
  try {
    unsigned int w,h;
    dst->GetImageDim(w,h);
    if (w!=1920) {
      program_->KernelSetArgument( "UCharYUYV422DPXToRGBA1280x720", 0, *src );
      program_->KernelSetArgument( "UCharYUYV422DPXToRGBA1280x720", 1, *dst );
      src->GetImageDim(w,h);
      unsigned int len = context_->DivUp(256,w*h / 8);
      context_->RunOn1DRange( *program_, "UCharYUYV422DPXToRGBA1280x720", len, 256);
    } else {
      program_->KernelSetArgument( "UCharYUYV422DPXToRGBA", 0, *src );
      program_->KernelSetArgument( "UCharYUYV422DPXToRGBA", 1, *dst );
      src->GetImageDim(w,h);
      w/=8;
      context_->RunOn2DRange( *program_, "UCharYUYV422DPXToRGBA", w,h, getlocsize_(w), getlocsize_(h));
    }
  } catch (clfException &e) {
    BIASERR(e.GetDetailedString());
    return 1;
  }
  return 0;
}
int clfColorConversion::UCharRGBDVSToRGBA(clfImage2D *src, clfImage2D *dst) {
  try {
    program_->KernelSetArgument( "UCharRGBDVSToRGBA", 0, *src );
    program_->KernelSetArgument( "UCharRGBDVSToRGBA", 1, *dst );
    unsigned int w,h;
    src->GetImageDim(w,h);
    context_->RunOn2DRange( *program_, "UCharRGBDVSToRGBA", w,h, getlocsize_(w), getlocsize_(h));
  } catch (clfException &e) {
    BIASERR(e.GetDetailedString());
    return 1;
  }
  return 0;
}

int clfColorConversion::RGBToFloatLuv(clfImage2D *src, clfImage2D *dst) {
  try {
    program_->KernelSetArgument( "RGBToLuv", 0, *src );
    program_->KernelSetArgument( "RGBToLuv", 1, *dst );
    unsigned int w,h;
    src->GetImageDim(w,h);
    context_->RunOn2DRange( *program_, "RGBToLuv", w,h, getlocsize_(w), getlocsize_(h));
  } catch (clfException &e) {
    BIASERR(e.GetDetailedString());
    return 1;
  }
  return 0;
}

int clfColorConversion::ScaleColorOfRGBPlusDepth(clfImage2D *src, clfImage2D *dst, float scale) {
  try {
    program_->KernelSetArgument( "ScaleColorOfRGBPlusDepth", 0, *src );
    program_->KernelSetArgument( "ScaleColorOfRGBPlusDepth", 1, *dst );
    program_->KernelSetArgument( "ScaleColorOfRGBPlusDepth", 2, scale );
    unsigned int w,h;
    src->GetImageDim(w,h);
    context_->RunOn2DRange( *program_, "ScaleColorOfRGBPlusDepth", w,h, getlocsize_(w), getlocsize_(h));
  } catch (clfException &e) {
    BIASERR(e.GetDetailedString());
    return 1;
  }
  return 0;
}


void clfColorConversion::getlocsize_(const unsigned int val, unsigned int &result) {
  result=getlocsize_(val);
}

unsigned int clfColorConversion::getlocsize_(const unsigned int val) {
  //if (val%32==0) return 32;
  if (val%16==0) return 16;
  if (val%8==0) return 8;
  if (val%4==0) return 4;
  if (val%2==0) return 2;
  return 0;
}

int clfColorConversion::RGBAToPackedGrey(clfImage2D& image1, clfImage2D& image2,
                                         clfImage2D& converted, int downsample)
{

  unsigned int width = image1.Width();
  unsigned int height = image1.Height();

  if (downsample ==1) {
    width /= 2;
    height /=2;
  }

  if (downsample == 2) {
    height /= 2;
  }

  if (downsample == 3) {
    width /= 2;
  }

  if (converted.Width() != width || converted.Height() != height ||
      converted.GetStorageType()!= BIAS::ImageBase::ST_unsignedchar ||
      converted.GetColorModel() != BIAS::ImageBase::CM_RGBA) {
    cout << "Allocating image clfColorConversion " << endl;

    converted.Allocate(ImageBase::ST_unsignedchar,ImageBase::CM_RGBA,width,height);
  }

  std::string kernelname = "RGBAToPackedGrey";

  if (image1.IsSharedGL()) {
    context_->AcquireGLObject(image1);
  }

  if (image2.IsSharedGL()) {
    context_->AcquireGLObject(image2);
  }

  if (converted.IsSharedGL()) {
    context_->AcquireGLObject(converted);
  }

  program_->KernelSetArgument(kernelname, 0, image1);
  program_->KernelSetArgument(kernelname, 1, image2);
  program_->KernelSetArgument(kernelname, 2, converted);


  program_->KernelSetArgument(kernelname, 3, downsample);


  context_->RunOn2DRange(*program_, kernelname, width,height, 0,0);

  if (image1.IsSharedGL()) {
    context_->ReleaseGLObject(image1);
  }

  if (image2.IsSharedGL()) {
    context_->ReleaseGLObject(image2);
  }

  if (converted.IsSharedGL()) {
    context_->ReleaseGLObject(converted);
  }


  return 0;
}

} /* namespace DECS */


