/* 
 This file is part of the BIAS library (Basic ImageAlgorithmS).

 Copyright (C) 2003, 2004    (see file CONTACTS for details)
 Multimediale Systeme der Informationsverarbeitung
 Institut fuer Informatik
 Christian-Albrechts-Universitaet Kiel


 BIAS is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 (at your option) any later version.

 BIAS is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with BIAS; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef CLFCOLORCONVERSION_HH_
#define CLFCOLORCONVERSION_HH_
#include <Base/Image/Image.hh>
#include <OpenCLFramework/clfOpenCL.hh>
#include <OpenCLFramework/clfContext.hh>

namespace BIAS {

  class BIASOpenCLFramework_EXPORT clfColorConversion {
    public:
      clfColorConversion(clfContext *ctx);
      virtual ~clfColorConversion();
      
      int Clear(clfImage2D *dst, float r,float g,float b,float a);

      //int RgbToRgba(clfImage2D *src, clfImage2D *dst);
      int ToRGBA(clfImage2D *src, clfImage2D *dst, bool normalize = false);
      
      int RGBToRGBA(clfImage2D *src, clfImage2D *dst, bool normalize = false);
      int UCharRGBToFloatGreyNormalized(clfImage2D *src, clfImage2D *dst);
      int RGBToGrey(clfImage2D *src, clfImage2D *dst, bool normalize = false);
      int FloatGreyToFloatGrey(clfImage2D *src, clfImage2D *dst);
      int FloatGreyToRGBA(clfImage2D *src, clfImage2D *dst, bool normalize=true);
      int UCharGreyToFloatGrey(clfImage2D *src, clfImage2D *dst, bool normalize = true);
      // works for unsigned short too, rename?, scale = 0.0f => automatic normalize
      int UCharBayerToRGBA(clfImage2D *src, clfImage2D *dst, BIAS::ImageBase::EColorModel bayermode, float scale = 0.0f);
      int UCharRGBAToYUV420P(clfImage2D *src, clfImage2D *dst);
      int UCharYUYV422ToRGBA(clfImage2D *src, clfImage2D *dst);
      int UCharYUYV422DPXToRGBA(clfImage2D *src, clfImage2D *dst);
      int UCharRGBDVSToRGBA(clfImage2D *src, clfImage2D *dst);
      int RGBToFloatLuv(clfImage2D *src, clfImage2D *dst);

      /**
       * @brief RGBAToPackedGrey
       * Converts two color image (RGBA) to one combined grey image (4 channels).
       * The grey version of image1 is saved in channel 0 and the grey version of
       * image2 is saved in channel 1
       * @param image1 color image RGBA
       * @param image2 color image RGBA
       * @param converted holds two converted images in channel 0 and 1
       */
      int RGBAToPackedGrey(clfImage2D& image1, clfImage2D& image2, clfImage2D& converted, int downsample = 0);

      int ScaleColorOfRGBPlusDepth(clfImage2D *src, clfImage2D *dst, float scale = 1.0f);

    protected:

      void getlocsize_(const unsigned int val, unsigned int &result);
      unsigned int getlocsize_(const unsigned int val);

      clfContext *context_;
      clfProgram *program_;
    private:

  };

} /* namespace BIAS */
#endif /* CLFCOLORCONVERSION_HH_ */
