/* 
 This file is part of the BIAS library (Basic ImageAlgorithmS).

 Copyright (C) 2003, 2004    (see file CONTACTS for details)
 Multimediale Systeme der Informationsverarbeitung
 Institut fuer Informatik
 Christian-Albrechts-Universitaet Kiel


 BIAS is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 (at your option) any later version.

 BIAS is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with BIAS; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef CLFGAUSS_HH_
#define CLFGAUSS_HH_

#include <Base/Common/BIASpragmaStart.hh>
#include <Base/Image/Image.hh>
#include <OpenCLFramework/clfOpenCL.hh>
#include <OpenCLFramework/clfContext.hh>
#include <OpenCLFramework/Filter/clfFilter.hh>

namespace BIAS {

  template <class InputStorageType, class OutputStorageType> 
  class BIASOpenCLFramework_EXPORT clfGauss : public clfFilter<InputStorageType, OutputStorageType> {
    public:
      clfGauss(clfContext *ctx);
      virtual ~clfGauss();
      
      virtual int Filter(clfImage2D *src, clfImage2D *dst);
      
      virtual int Build(unsigned int size);

      
      inline void SetSigma(const double si) { 
        GaussSigma_=si;
        CalculateKernels_(GaussSigma_, GaussRatio_);
      }

      inline double GetSigma() const  { return GaussSigma_; }

      inline void SetRatio(const double ratio) { 
        GaussRatio_=ratio; 
        CalculateKernels_(GaussSigma_, GaussRatio_);
      }

      inline double GetRatio() const { return GaussRatio_; } 

      /** define the half win size of the kernel, if AdjustSigma is true
          sigma is computed according to the cut-off ratio, otherwise the ratio
          is adapted to the sigma */
      inline void SetWinSize(const int ws, bool AdjustSigma=true)
      {
        int hws = ws >> 1;
        if (AdjustSigma) {
          GaussSigma_ = sqrt((double)hws*(double)hws/(-2.0*log(GaussRatio_))); 
        } else {
          GaussRatio_ = 0.5*(exp(((double)hws*(double)hws)
                                 /(-2.0*(GaussSigma_)*(GaussSigma_)))+
                              exp(((double)(hws+1.0)*(double)(hws+1.0))
                                 /(-2.0*(GaussSigma_)*(GaussSigma_)))) ;
        }
      }


      // these are interfaces for class Tracker is Track is instantiated
      // with out param object
      double GetSigma(){ return GaussSigma_;}
      double GetRatio(){ return GaussRatio_;}
    
    protected:

      /// calculates the kernel
      virtual void CalculateKernels_(double Sigma, double Ratio);

      double GaussSigma_;
      double GaussRatio_;

      double LastSigma_;
      double LastRatio_;

      using clfFilter<InputStorageType, OutputStorageType>::winSize_;
      using clfFilter<InputStorageType, OutputStorageType>::program_;

    private:

  };

} /* namespace BIAS */
#include <Base/Common/BIASpragmaEnd.hh>
#endif /* CLFGAUSS_HH_ */
