/* 
 This file is part of the BIAS library (Basic ImageAlgorithmS).

 Copyright (C) 2003, 2004    (see file CONTACTS for details)
 Multimediale Systeme der Informationsverarbeitung
 Institut fuer Informatik
 Christian-Albrechts-Universitaet Kiel


 BIAS is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 (at your option) any later version.

 BIAS is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with BIAS; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef CLFGRADIENTGAUSS_HH_
#define CLFGRADIENTGAUSS_HH_
#include <Base/Common/BIASpragmaStart.hh>
#include <Base/Image/Image.hh>
#include <OpenCLFramework/clfOpenCL.hh>
#include <OpenCLFramework/clfContext.hh>
#include <OpenCLFramework/Filter/clfGauss.hh>

namespace BIAS {

  template <class InputStorageType, class OutputStorageType> 
  class BIASOpenCLFramework_EXPORT clfGradientGauss : public clfGauss<InputStorageType, OutputStorageType> {
    public:
    clfGradientGauss(clfContext *ctx);
    virtual ~clfGradientGauss();
    
    int Filter(clfImage2D *src, clfImage2D *dst);
    
    protected:

    int SetFilterVert_(unsigned int size, const float *mask); 
    int SetFilterHoriz_(unsigned int size, const float *mask); 
      /// calculates the kernel
      virtual void CalculateKernels_(double Sigma, double Ratio);

    private:

      clfBuffer *filterMaskVert_;
      using clfGauss<InputStorageType, OutputStorageType>::GaussSigma_;
      using clfGauss<InputStorageType, OutputStorageType>::GaussRatio_;

      using clfGauss<InputStorageType, OutputStorageType>::LastSigma_;
      using clfGauss<InputStorageType, OutputStorageType>::LastRatio_;

      using clfFilter<InputStorageType, OutputStorageType>::winSize_;
      int vertSize_;
      using clfFilter<InputStorageType, OutputStorageType>::program_;

      using clfFilter<InputStorageType, OutputStorageType>::filterMask_;
      using clfFilter<InputStorageType, OutputStorageType>::context_;
      using clfFilter<InputStorageType, OutputStorageType>::tmpImage_;
      
  };

} /* namespace BIAS */
#include <Base/Common/BIASpragmaEnd.hh>
#endif /* CLFGRADIENTGAUSS_HH_ */
