/* 
 This file is part of the BIAS library (Basic ImageAlgorithmS).

 Copyright (C) 2003, 2004    (see file CONTACTS for details)
 Multimediale Systeme der Informationsverarbeitung
 Institut fuer Informatik
 Christian-Albrechts-Universitaet Kiel


 BIAS is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 (at your option) any later version.

 BIAS is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with BIAS; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <Base/Common/BIASpragma.hh>
#include "clfSimpleFilter.hh"

namespace BIAS {

  clfSimpleFilter::clfSimpleFilter(clfContext *ctx, std::string programName) {
    context_ = ctx;
    program_ = context_->CreateProgram();
    program_->AddSource("/OpenCLFramework/Filter/cl/filteroperation.cl");
    program_->Build();
    programName_ = programName;
    program_->AddKernel(programName_);
  }
  
  void clfSimpleFilter::SetProgram(std::string programName) {
    programName_ = programName;
    program_->AddKernel(programName_);
  }


  clfSimpleFilter::~clfSimpleFilter() {
    delete program_;
  }
  
  int clfSimpleFilter::Filter(clfImage2D *src, clfImage2D *dst) {
    program_->KernelSetArgument( programName_, 0, *src );
    program_->KernelSetArgument( programName_, 1, *dst );
    unsigned int w,h;
    src->GetImageDim(w,h);
    context_->RunOn2DRange( *program_, programName_, w, h, 16, 16);
    return 0;
  }

  int clfSimpleFilter::FilterScale(clfImage2D *src, clfImage2D *dst, float scale) {
    program_->KernelSetArgument( programName_, 0, *src );
    program_->KernelSetArgument( programName_, 1, *dst );
    unsigned int w,h;
    src->GetImageDim(w,h);
    w*=scale;
    h*=scale;
    context_->RunOn2DRange( *program_, programName_, w, h, 16, 16 );
    return 0;
  }

  //////////////////////////////////////////////////////////////////
  int clfStructureTensor::Filter(clfImage2D *src, clfImage2D *dst) {
    program_->KernelSetArgument( programName_, 0, *src );
    program_->KernelSetArgument( programName_, 1, winSize_ );
    program_->KernelSetArgument( programName_, 2, *dst );
    unsigned int w,h;
    src->GetImageDim(w,h);
    context_->RunOn2DRange( *program_, programName_, w, h, 16, 16 );
    return 0;
    
  }
  //////////////////////////////////////////////////////////////////
  int clfExtractLocalMaxima::Filter(clfImage2D *src, clfImage2D *dst) {
    program_->KernelSetArgument( programName_, 0, *src );
    program_->KernelSetArgument( programName_, 1, winSize_ );
    program_->KernelSetArgument( programName_, 2, minCornerness_ );
    program_->KernelSetArgument( programName_, 3, *dst );
    unsigned int w,h;
    src->GetImageDim(w,h);
    context_->RunOn2DRange( *program_, programName_, w, h, 16, 16 );
    return 0;
    
  }

  int clfScaleImage::Filter(clfImage2D *src, clfImage2D *dst) {
    program_->KernelSetArgument( programName_, 0, *src );
    program_->KernelSetArgument( programName_, 1, *dst );
    program_->KernelSetArgument( programName_, 2, scale_ );
    unsigned int w,h;
    src->GetImageDim(w,h);
    context_->RunOn2DRange( *program_, programName_, w,h,16,16 );
    return 0;
  }

  int clfResampleImage::Filter(clfImage2D *src, clfImage2D *dst) {
    program_->KernelSetArgument( programName_, 0, *src );
    program_->KernelSetArgument( programName_, 1, *dst );
    unsigned int w,h;
    dst->GetImageDim(w,h);
    context_->RunOn2DRange( *program_, programName_, context_->DivUp(16,w),context_->DivUp(16,h),16,16 );
    return 0;
  }

} /* namespace DECS */
