/* 
 This file is part of the BIAS library (Basic ImageAlgorithmS).

 Copyright (C) 2003, 2004    (see file CONTACTS for details)
 Multimediale Systeme der Informationsverarbeitung
 Institut fuer Informatik
 Christian-Albrechts-Universitaet Kiel


 BIAS is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 (at your option) any later version.

 BIAS is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with BIAS; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef CLFSIMPLEFILTER_HH_
#define CLFSIMPLEFILTER_HH_

#include <Base/Common/BIASpragmaStart.hh>
#include <OpenCLFramework/clfOpenCL.hh>
#include <OpenCLFramework/clfContext.hh>

namespace BIAS {

  class BIASOpenCLFramework_EXPORT clfSimpleFilter {
    public:
      clfSimpleFilter(clfContext *ctx, std::string programName);
      virtual ~clfSimpleFilter();
      
      virtual int Filter(clfImage2D *src, clfImage2D *dst);
      virtual int FilterScale(clfImage2D *src, clfImage2D *dst, float scale);
      void SetProgram(std::string programName);
      
      inline clfProgram* GetProgram() { return program_; }
    protected:
      clfContext *context_;
      clfProgram *program_;
      std::string programName_;
      
    private:
  };

  class BIASOpenCLFramework_EXPORT clfGradient3x3 : public clfSimpleFilter {
    public:
      clfGradient3x3(clfContext *ctx) 
      : clfSimpleFilter(ctx, "Gradient3x3") {}
  };

  class BIASOpenCLFramework_EXPORT clfStructureTensor : public clfSimpleFilter {
    public:
      clfStructureTensor(clfContext *ctx, int winSize) 
      : clfSimpleFilter(ctx, "StructureTensor") {
        winSize_ = winSize;
      }
      inline void SetWinSize(int winsize) { winSize_ = winsize; }
      int Filter(clfImage2D *src, clfImage2D *dst);
    private:
      int winSize_;
  };

  class BIASOpenCLFramework_EXPORT clfExtractLocalMaxima : public clfSimpleFilter {
    public:
      clfExtractLocalMaxima(clfContext *ctx, int winSize, float minCornerness) 
      : clfSimpleFilter(ctx, "ExtractLocalMaxima") {
        winSize_ = winSize;
        minCornerness_ = minCornerness;
      }
      inline void SetWinSize(int winsize) { winSize_ = winsize; }
      inline void SetMinCornerness(float c) { minCornerness_ = c; }
      int Filter(clfImage2D *src, clfImage2D *dst);
    private:
      float minCornerness_;
      int winSize_;
  };

  class BIASOpenCLFramework_EXPORT clfScaleImage : public clfSimpleFilter {
    public:
    clfScaleImage(clfContext *ctx)
      : clfSimpleFilter(ctx, "Scale") {}
      inline void SetScale(float scale) { scale_ = scale; }
      int Filter(clfImage2D *src, clfImage2D *dst);
    private:
      float scale_;
  };

  class BIASOpenCLFramework_EXPORT clfResampleImage : public clfSimpleFilter {
    public:
    clfResampleImage(clfContext *ctx)
      : clfSimpleFilter(ctx, "Resample") {}
      int Filter(clfImage2D *src, clfImage2D *dst);
  };

} /* namespace BIAS */

#include <Base/Common/BIASpragmaEnd.hh>
#endif /* CLFGRADIENT3X3_HH_ */
