/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004    (see file CONTACTS for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <iostream>

#include <OpenCLFramework/clfContext.hh>
#include <Utils/Param.hh>
#include <Utils/IOUtils.hh>

using namespace std;
using namespace BIAS;
using namespace BIAS;

int main(int argc, char* argv[]) {
  
  Param par;
  bool *printimgformats = par.AddParamBool("imageformats", "print available image formats", false, 'i');
  
  if (IOUtils::ParseCommandLineEvalHelp(par, argc, argv) == 0) {
    par.Usage();
  }
  
  try {
    
    clfContext context;
    
    context.PrintPlatformsInfo();
    
    for (unsigned int d=0;d<context.GetNumDevices();d++) {
      context.PrintDeviceInfo(d, true);
    }
      
    if (*printimgformats) {
      clfImage2D *img = context.CreateImage2D();
      vector<string> formats = img->GetSupportedImageFormats(false, false);
      cout << "Available Image Formats for Device 0:" << endl;
      for (unsigned int i=0;i<formats.size();i++) {
        cout << formats[i] << endl;
      }
      delete img;
    }
    
  } catch (clfException &err) {
    cout << err.GetDetailedString() << endl;
  }
  try {

    clfContext context(false, clfContext::clfContextTypeCPU);

    context.PrintPlatformsInfo();

    for (unsigned int d=0;d<context.GetNumDevices();d++) {
      context.PrintDeviceInfo(d, true);
    }

    if (*printimgformats) {
      clfImage2D *img = context.CreateImage2D();
      vector<string> formats = img->GetSupportedImageFormats(false, false);
      cout << "Available Image Formats for Device 0:" << endl;
      for (unsigned int i=0;i<formats.size();i++) {
        cout << formats[i] << endl;
      }
      delete img;
    }

  } catch (clfException &err) {
    cout << err.GetDetailedString() << endl;
  }

  return 0;
}
