
#include <bias_config.h>

#include <Base/Common/BIASpragmaStart.hh>
#include <Base/Common/BIASpragmaEnd.hh>
#include <Base/Image/ImageConvert.hh>
#include <Base/Common/FileHandling.hh>
#include <Utils/IOUtils.hh>
#include <Gui/biasgl.h> 

#include <Utils/Param.hh>
#include <Utils/IOUtils.hh>

#include <iostream>
#include <OpenCLFramework/Algorithm/clfTVL1Flow.hh>
#include <Base/Debug/TimeMeasure.hh>

using namespace std;
using namespace BIAS;
using namespace BIAS;

/**
 * "A duality based approach for realtime TV-L1 optical flow" by Zach et.al.
 * see OpenCLFramework/Algorithm/clfTVL1Flow.hh for details
 *
 * usage: mipTVL1FlowCL -i imagelist -o outname
 *
 * generates color coded flow images with name outname-imgNr.mip
 *
 * @author fkellner 08/11
 */
int main(int argc, char* argv[]) {
  Param p;

  string *infile = p.AddParamString("imglist", "image list (uchar grey or rgb images, all same size)", "img.lst" ,'i');
  string *outname = p.AddParamString("out", "out image base name (number and .mip are added)", "out", 'o');
  int *dispModus = p.AddParamInt("dispModus", "1 yes, 0 no", 0);
  int *iter = p.AddParamInt("iter", "number of iterations", 20);
  double *smoothness = p.AddParamDouble("smoothness", "values for smoothness, greater value => smaller smoothnes", 20);
  p.ParseCommandLine(argc,argv);

  p.ShowData();
  
  IOUtils::ParseCommandLineEvalHelp(p, argc, argv);
  
  vector<string> names;
  Param::ParseListFile(*infile, names);
  
  if (names.size()<2) {
    exit(0);
  }
  Image<unsigned char> image;
  Image<float> result;
  Image<float> cutResult;
  BIAS::TimeMeasure tm;
  
  try {
    clfContext context;
    clfTVL1Flow flowcl(&context);
    flowcl.SetSmoothness( (float)(*smoothness));
    flowcl.SetNumberOfIterations(*iter);
    ImageIO::Load(names[0], image);
    cutResult.Init(image.GetWidth(),image.GetHeight(),3);
    image.PadToPowerOfTwo(0);
    flowcl.AddImage(image);

    for (unsigned int i=1;i<names.size();i++) {
      ImageIO::Load(names[i], image);
      image.PadToPowerOfTwo(0);
      flowcl.AddImage(image);
      context.Finish();
      tm.Reset();
      tm.Start();
      flowcl.Compute();
      context.Finish();
      tm.Stop();
      tm.Print(cout);
      //      flowcl.GetColorCoded( result);
      flowcl.GetUVW(result);


      // test
      for (unsigned int x = 0; x < cutResult.GetWidth(); x++) {
        for (unsigned int y = 0; y < cutResult.GetHeight(); y++) {
          for (unsigned int ch = 0; ch < cutResult.GetChannelCount(); ch++) {
            float val;
            if (*dispModus) {
              val = result.PixelValue(x,y,(unsigned short)0);
            } else {
              val = result.PixelValue(x,y,(unsigned short)ch);
            }
            cutResult.SetPixel(val,x,y,(unsigned short)ch);
          }
        }

      }

      ImageIO::Save(*outname+"-"+FileHandling::LeadingZeroString(i,2)+".mip", cutResult);
    }
  } catch (clfException &e) {
    cout << e.GetDetailedString() << endl;
    return 1;
  }

  return 0;
}

