/* 
 This file is part of the BIAS library (Basic ImageAlgorithmS).

 Copyright (C) 2003, 2004    (see file CONTACTS for details)
 Multimediale Systeme der Informationsverarbeitung
 Institut fuer Informatik
 Christian-Albrechts-Universitaet Kiel


 BIAS is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 (at your option) any later version.

 BIAS is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with BIAS; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef OPENCLEXCEPTION_HH_
#define OPENCLEXCEPTION_HH_

#include <bias_config.h>
#include <Base/Common/BIASpragmaStart.hh>
#include <OpenCLFramework/clfOpenCL.hh>
#include <exception>

namespace BIAS {

#define THROW_CL_EXCEPTION(err) \
    throw clfException(err, __FILE__, __LINE__)

  /**
   * 
   * @brief clf Exception wrapper, is thrown in case of most clf errors
   * 
   * clf Exceptions are enabled in this library. This class wraps these 
   * exceptions and add more information. Almost all functions in the library 
   * throw an exception instead of returning error codes, so catch them! 
   * 
   * @author fkellner 06/11
   */
  class BIASOpenCLFramework_EXPORT clfException : public std::exception {
    public:
      /// @brief is called by THROW_CL_EXCEPTION macro
      clfException(cl::Error clerror, std::string file, int line) throw();
      
      virtual ~clfException() throw() {}
      
      /// @brief file in which error occured
      const std::string& GetFileName() const throw();
      /// @brief line in which error occured
      int GetLineNumber() const throw();
      /// @brief clf error code
      int GetErrorCode() const throw();
      /// @brief clf error message (usually name of failing c function)
      const std::string& GetMessageString() const throw();
      /// @brief clf translated error code (todo clean up text)
      const std::string& GetReasonString() const throw();
      /// @brief detailed combination of all info available
      const std::string& GetDetailedString() const throw();

      virtual const char* what() const throw();

    private:
      std::string fileName_;
      int lineNumber_;
      std::string message_;
      int errorCode_;
      std::string reason_;
      std::string detailed_;

  };

}
#include <Base/Common/BIASpragmaEnd.hh>
#endif /* clfEXCEPTION_HH_ */
