/* 
 This file is part of the BIAS library (Basic ImageAlgorithmS).

 Copyright (C) 2003, 2004    (see file CONTACTS for details)
 Multimediale Systeme der Informationsverarbeitung
 Institut fuer Informatik
 Christian-Albrechts-Universitaet Kiel


 BIAS is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 (at your option) any later version.

 BIAS is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with BIAS; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <Base/Common/BIASpragma.hh>
#include <OpenCLFramework/clfMemory.hh>
#include <OpenCLFramework/clfException.hh>

namespace BIAS {

  clfMemory::clfMemory(cl::Context *context, cl::CommandQueue *queue) {
    context_ = context;
    queue_ = queue;
    size_ = 0;
    initialized_ = false;
    sharedGL_ = false;
    glid_ = 0;
  }

  clfMemory::~clfMemory() {
  }
  
  cl::Memory& clfMemory::memory() {
    return (cl::Memory&)buffer_;
  }
  
  unsigned int clfMemory::GLId() {
    if (!sharedGL_) {
      THROW_CL_EXCEPTION(cl::Error(-100, "This is not a gl shared buffer"));
    }
    return glid_;
  }
  
  void clfMemory::UnMap(void *data) {
    try {
      queue_->enqueueUnmapMemObject(memory(), data);
    } catch (cl::Error &error) {
      THROW_CL_EXCEPTION(error);
    }
  }

  bool clfMemory::IsReadOnly() {
    cl_mem_flags data;
    buffer_.getInfo(CL_MEM_FLAGS, &data);
    return data & CL_MEM_READ_ONLY;
  }
  bool clfMemory::IsWriteOnly() {
    cl_mem_flags data;
    buffer_.getInfo(CL_MEM_FLAGS, &data);
    return data & CL_MEM_WRITE_ONLY;
  }

  int clfMemory::DetermineMemFlags_(bool readonly, bool writeonly, const void *hostptr, bool copy) {
    int cl_mem_flags = 0;
    if (readonly && writeonly) {
      BIASWARN("trying to create buffer read-only and write-only. fail");
    } else if (readonly) {
      cl_mem_flags |= CL_MEM_READ_ONLY; 
    } else if (writeonly) {
      cl_mem_flags |= CL_MEM_WRITE_ONLY; 
    } else {
      cl_mem_flags |= CL_MEM_READ_WRITE; 
    }
    if (hostptr != NULL) {
      if (copy) {
        cl_mem_flags |= CL_MEM_COPY_HOST_PTR;
      } else {
        cl_mem_flags |= CL_MEM_USE_HOST_PTR;
      }
    }
    return cl_mem_flags;
  }


}
