/*
 This file is part of the BIAS library (Basic ImageAlgorithmS).

 Copyright (C) 2003-2009    (see file CONTACT for details)
 Multimediale Systeme der Informationsverarbeitung
 Institut fuer Informatik
 Christian-Albrechts-Universitaet Kiel


 BIAS is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 (at your option) any later version.

 BIAS is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with BIAS; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "glfBlendMode.hh"
#include "glfException.hh"

using namespace BIAS;

const glfBlendMode glfBlendMode::DEFAULT;

glfBlendMode::glfBlendMode()
{
  enable_ = GL_FALSE;
  equation_ = GL_FUNC_ADD;
  sfactor_ = GL_ONE;
  dfactor_ = GL_ZERO;
}

void glfBlendMode::EnableBlending(bool enable)
{
  enable_ = enable ? GL_TRUE : GL_FALSE;
}

void glfBlendMode::SetBlendEquation(GLenum equation)
{
  equation_ = equation;
}

void glfBlendMode::SetBlendFunc(GLenum sfactor, GLenum dfactor)
{
  sfactor_ = sfactor;
  dfactor_ = dfactor;
}

void glfBlendMode::Bind() const
{
  if (enable_ == GL_TRUE) {
    glEnable(GL_BLEND);
    glBlendEquation(equation_);
    glBlendFunc(sfactor_, dfactor_);
  } else {
    glDisable(GL_BLEND);
  }

  GLF_THROW_ON_OPENGL_ERROR;
}
