#include "glfBufferObject.hh"
#include "glfException.hh"

using namespace BIAS;

glfBufferObject::glfBufferObject(GLenum target, GLenum queryName) :
  target_(target), queryName_(queryName), bufferObjID_(0), boundRevertably_(false), previousBinding_(0)
{}

glfBufferObject::~glfBufferObject()
{
  glDeleteBuffers(1, &bufferObjID_);
  bufferObjID_ = 0;
  GLF_THROW_ON_OPENGL_ERROR_DEBUG;
}


void glfBufferObject::Create()
{
  if(bufferObjID_==0)
    glGenBuffers(1, &bufferObjID_);
  GLF_THROW_ON_OPENGL_ERROR_DEBUG;
}


void glfBufferObject::Bind()
{
  BIASASSERT(  bufferObjID_ != 0);
  glBindBuffer(target_, bufferObjID_);
  boundRevertably_ = false;
  GLF_THROW_ON_OPENGL_ERROR_DEBUG;
}

void glfBufferObject::BindRevertable()
{
  BIASASSERT(  bufferObjID_ != 0);
  glGetIntegerv(queryName_, &previousBinding_);
  boundRevertably_ = true;
  glBindBuffer(target_, bufferObjID_);
  GLF_THROW_ON_OPENGL_ERROR_DEBUG;
}

void glfBufferObject::Revert()
{
  if(boundRevertably_) {
    glBindBuffer(target_, previousBinding_);
    boundRevertably_ = false;
    GLF_THROW_ON_OPENGL_ERROR_DEBUG;
  } else {
    BIASWARN("Unable to revert binding! Use BindRevertable() without subsequent call to Bind()!");
  }
}


void glfBufferObject::
UploadData(GLenum usageHint, GLsizeiptr size, GLvoid* data)
{
  BindRevertable();
  glBufferData(target_, size, data, usageHint);
  GLF_THROW_ON_OPENGL_ERROR;
  Revert();
}

void glfBufferObject::
UploadData(float* data, unsigned int numElements, GLenum usageHint)
{
  UploadData(usageHint, sizeof(float)*numElements, data);
}


// GLuint glfBufferObject::GetID()
// {
//   return bufferObjID_;
// }
