/*
 This file is part of the BIAS library (Basic ImageAlgorithmS).

 Copyright (C)  2008, 2009    (see file CONTACT for details)
 Multimediale Systeme der Informationsverarbeitung
 Institut fuer Informatik
 Christian-Albrechts-Universitaet Kiel


 BIAS is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 (at your option) any later version.

 BIAS is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with BIAS; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __glfBufferObject_hh__
#define __glfBufferObject_hh__

#include "glfCommon.hh"

namespace BIAS {

  /** \brief Base class of wrapper for OpenGL 3.x buffer objects.
   * \ingroup g_openglframework	
   * \author bartczak 08/2009
   **/
  class BIASOpenGLFramework_EXPORT glfBufferObject {
  public:
    glfBufferObject(GLenum target, GLenum queryName);
    virtual ~glfBufferObject();
    
    /* 
       \attention Call this after context is available and before any other member function ! 
    */
    void Create();
    
    
    void Bind();
    void BindRevertable();
    void Revert();

    /** 
        \param UsageHint is one of the usage params found in section 2.9.2 of the OpenGL 3.2 specs.
    **/
    void UploadData(float* data, unsigned int numElements, GLenum usageHint=GL_STATIC_DRAW);
    
    //    GLuint GetID();

  protected:
    GLenum target_;
    /* object will not be bound afterwards! Previous binding is restored!! */
    void UploadData(GLenum usageHint, GLsizeiptr size, GLvoid* data);

  private:

    GLenum queryName_;//used to find the currently bound object with same target
    GLuint bufferObjID_;
    

    bool boundRevertably_;
    GLint previousBinding_;
    
    //    glfBufferObject& operator=(const glfBufferObject& b);
  };
}

#endif
