/*
 This file is part of the BIAS library (Basic ImageAlgorithmS).

 Copyright (C) 2003-2009    (see file CONTACT for details)
 Multimediale Systeme der Informationsverarbeitung
 Institut fuer Informatik
 Christian-Albrechts-Universitaet Kiel


 BIAS is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 (at your option) any later version.

 BIAS is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with BIAS; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "glfDepthBufferMode.hh"
#include "glfException.hh"

using namespace BIAS;

const glfDepthBufferMode glfDepthBufferMode::DEFAULT;

glfDepthBufferMode::glfDepthBufferMode()
{
  enableTest_ = GL_FALSE;
  depthMask_ = GL_TRUE;
  depthFunc_ = GL_LESS;
  nearVal_ = 0.0;
  farVal_ = 1.0;
}

void glfDepthBufferMode::SetDepthTest(bool enable)
{
  enableTest_ = enable;
}

void glfDepthBufferMode::SetDepthMask(bool flag)
{
  depthMask_ = flag ? GL_TRUE : GL_FALSE;
}

void glfDepthBufferMode::SetDepthFunc(GLenum depthFunc)
{
  depthFunc_ = depthFunc;
}

void glfDepthBufferMode::SetDepthRange(double nearVal, double farVal)
{
  nearVal_ = nearVal;
  farVal_ = farVal;
}


void glfDepthBufferMode::Bind() const
{
  if (enableTest_) {
    glEnable(GL_DEPTH_TEST);
    glDepthMask(depthMask_);
    glDepthFunc(depthFunc_);
    glDepthRange(nearVal_, farVal_);
  } else {
     glDisable(GL_DEPTH_TEST);
  }
 

  GLF_THROW_ON_OPENGL_ERROR;
}
