/*
 This file is part of the BIAS library (Basic ImageAlgorithmS).

 Copyright (C) 2003-2009    (see file CONTACT for details)
 Multimediale Systeme der Informationsverarbeitung
 Institut fuer Informatik
 Christian-Albrechts-Universitaet Kiel


 BIAS is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 (at your option) any later version.

 BIAS is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with BIAS; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __glfDepthBufferMode_hh__
#define __glfDepthBufferMode_hh__

#include "glfCommon.hh"

namespace BIAS {

  /**
   * \brief Defines the usage of the depth buffer.
   * \ingroup g_openglframework	
   * \author jkollmann
   */
  class BIASOpenGLFramework_EXPORT glfDepthBufferMode {
  public:
    glfDepthBufferMode();
    
    virtual ~glfDepthBufferMode() {}

    /**
     * Sets whether to use depth buffer tests.
     * \note: This must be enabled, even you only want
     * to write to the depth buffer. Use the depth func
     * GL_ALWAYS to use the depth buffer write-only.
     */
    void SetDepthTest(bool enable);

    /**
     * Sets whether to write to the depth buffer.
     */
    void SetDepthMask(bool flag);

    /**
     * Sets the depth buffer comparison function.
     * Valid values are: GL_NEVER, GL_LESS, GL_EQUAL, GL_LEQUAL, GL_GREATER,
     * GL_NOTEQUAL, GL_GEQUAL, GL_ALWAYS.
     */
    void SetDepthFunc(GLenum depthFunc);

    /**
     * Sets the mapping the used range of the depth buffer.
     * The parameters nearVal and farVal must be in [0,1].
     */
    void SetDepthRange(double nearVal, double farVal);

    /**
     * Binds the depth buffer mode.
     */
    void Bind() const;

    /**
     * The default depth buffer mode.
     */
    static const glfDepthBufferMode DEFAULT;

  private:
    bool enableTest_;
    GLboolean depthMask_;
    GLenum depthFunc_;
    GLclampd nearVal_, farVal_;
  };

} // namespace BIAS

#endif // __glfDepthBufferMode_hh__
