#include "glfTexture3D.hh"
#include "glfException.hh"
#include <Base/Image/ImageIO.hh>
#include "glfFormatDB.hh"

using namespace std;
using namespace BIAS;

glfTexture3D::glfTexture3D() :
        glfTexture(GL_TEXTURE_3D)
{
}

glfTexture3D&
        glfTexture3D::operator=(const glfTexture3D& b)
                               {
    return (*this);
}

glfTexture3D::glfTexture3D(const glfTexture3D& t) :
        glfTexture(GL_TEXTURE_3D)
{
}

void
        glfTexture3D::Set(GLenum minFilter, GLenum magFilter, GLenum wrapS,
                          GLenum wrapT, GLint textureNr)
{
    SetMinFilter(minFilter);
    SetMagFilter(magFilter);
    SetWrapS(wrapS);
    SetWrapT(wrapT);
    SetTextureNr(textureNr);
}

void glfTexture3D::Allocate3DImage(int width, int height, int depth,
                                   GLenum internalFormat, int mipmap)
{  
    GLenum format;
    int numChannels;
    glfFormatDB::GetFormatForInternalFormat(internalFormat, format, numChannels);

    Bind();

    GLenum type = GL_UNSIGNED_BYTE;
    if (internalFormat == GL_DEPTH24_STENCIL8_EXT)
      type = GL_UNSIGNED_INT_24_8_EXT;

    glTexImage3D(target_, mipmap, internalFormat, width, height, depth, 0,
                 format, type, NULL);

    GLF_THROW_ON_OPENGL_ERROR;
}

void glfTexture3D::Allocate3DImage(int width, int height, int depth,
                                   ImageBase::EStorageType storageType,
                                   ImageBase::EColorModel colorModel,
                                   int mipmap) {
    GLenum internalFormat = glfFormatDB::ProposeInternalFormat(storageType, colorModel);
    Allocate3DImage(width, height, depth, internalFormat, mipmap);
}

void
        glfTexture3D::Upload2DImage(const BIAS::ImageBase& image,  unsigned int level)
{

    GLenum format = glfFormatDB::GetFormatForColorModel(image.GetColorModel());
    GLenum pixelType = glfFormatDB::GetPixelTypeForStorageType(image.GetStorageType());

//    cout << "pixel type = " << ( pixelType == GL_FLOAT) << endl;

    // upload data.
    Bind();
  //  glPixelStorei(GL_UNPACK_ALIGNMENT, 1);


    glTexSubImage3D(GL_TEXTURE_3D, 0,0,0,level, image.GetWidth(), image.GetHeight(), 1,
                    format, pixelType, image.GetImageData());

    GLF_THROW_ON_OPENGL_ERROR;
}
