
#include "glfVertexArrayObject.hh"
#include "glfException.hh"

#ifdef GL_VERSION_3_0

using namespace BIAS;
using namespace std;

glfVertexArrayObject::
glfVertexArrayObject() : 
vertexArrayObjID_(0), boundRevertably_(false), previousBinding_(0)
{}

glfVertexArrayObject::~glfVertexArrayObject() 
{
  glDeleteVertexArrays(1, &vertexArrayObjID_);
  GLF_THROW_ON_OPENGL_ERROR_DEBUG;
}

    /* 
       \attention Call this after context is available and before any other member function ! 
    */
void glfVertexArrayObject::
Create() 
{
#ifdef BIAS_DEBUG
  if(GLEW_VERSION_3_0 != GLEW_OK) GLF_THROW_EXCEPTION("glfVertexArrayObject is only functional with OpenGL 3.0 up!");
#endif
  if(vertexArrayObjID_==0)
    glGenVertexArrays(1, &vertexArrayObjID_);
  GLF_THROW_ON_OPENGL_ERROR_DEBUG;
}

void glfVertexArrayObject::
Bind()
{
  BIASASSERT(vertexArrayObjID_!=0);
  glBindVertexArray(vertexArrayObjID_);
  boundRevertably_ = false;
  GLF_THROW_ON_OPENGL_ERROR_DEBUG;
}


void glfVertexArrayObject::
BindRevertable()
{
  BIASASSERT(vertexArrayObjID_!=0);
  glGetIntegerv(GL_VERTEX_ARRAY_BINDING, &previousBinding_);
  boundRevertably_ = true;
  glBindVertexArray(vertexArrayObjID_);
  GLF_THROW_ON_OPENGL_ERROR_DEBUG;

}

void glfVertexArrayObject::
Revert()
{
  if(boundRevertably_) {
    glBindVertexArray(previousBinding_);
    boundRevertably_ = false;
    GLF_THROW_ON_OPENGL_ERROR_DEBUG;
  } else {
    BIASWARN("Unable to revert binding! Use BindRevertable() without subsequent call to Bind()!");
  }

}

void glfVertexArrayObject::
UploadData(GLuint slotIndex,                     
           GLint numValuesPerVertex,
           float* data, unsigned int numValues,                     
           GLenum usageHint)
{
  BIASASSERT(vertexArrayObjID_!=0);
  BIASASSERT(numValuesPerVertex>=1 && numValuesPerVertex<=4);
  map<GLuint, glfArrayBufferObject>::iterator it = internalArrayBufferObjects_.find(slotIndex);
  if(it != internalArrayBufferObjects_.end()) {
    it->second.UploadData(data, numValues, usageHint);

  } else {
    internalArrayBufferObjects_[slotIndex].Create();    
    internalArrayBufferObjects_[slotIndex].UploadData(data, numValues*numValuesPerVertex, usageHint);
      
  }

  BindRevertable();
  internalArrayBufferObjects_[slotIndex].Bind(); //BindRevertable() ?
  glVertexAttribPointer(slotIndex, numValuesPerVertex, GL_FLOAT, GL_FALSE, 0, 0);

  glEnableVertexAttribArray(slotIndex);
  Revert();
  //    internalArrayBufferObjects_[slotIndex].Revert(); // ??
  
}
   
    
glfVertexArrayObject& glfVertexArrayObject::
operator=(const glfVertexArrayObject& b) 
{ return *this; } 

#endif
