/*
 This file is part of the BIAS library (Basic ImageAlgorithmS).

 Copyright (C)  2008, 2009    (see file CONTACT for details)
 Multimediale Systeme der Informationsverarbeitung
 Institut fuer Informatik
 Christian-Albrechts-Universitaet Kiel


 BIAS is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 (at your option) any later version.

 BIAS is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with BIAS; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __glfViewport_hh__
#define __glfViewport_hh__

#include "glfCommon.hh"

namespace BIAS {

  /**
   * \brief class for setting viewports
   * \author jkollmann
   * \ingroup g_openglframework   
   */
  class BIASOpenGLFramework_EXPORT glfViewport {
  public:
    glfViewport();
    glfViewport(int x, int y, int width, int height);
    
    virtual ~glfViewport() {}

    /**
     * Sets the position of the upper-left corner
     * of the viewport in pixel coordinates.
     */
    void SetOrigin(int x, int y);

    /**
     * Sets the size of the viewport in pixel coordinates.
     */
    void SetSize(int width, int height);

    int GetWidth() const { return width_; }
    int GetHeight() const { return height_; }

    /**
     * Binds the viewport.
     */
    virtual void Bind() const;

  private:
    GLint x_, y_;
    GLsizei width_, height_;
  };

} // namespace BIAS

#endif // __glfViewport_hh__
