/*
 This file is part of the BIAS library (Basic ImageAlgorithmS).

 Copyright (C)  2008, 2009    (see file CONTACTS for details)
 Multimediale Systeme der Informationsverarbeitung
 Institut fuer Informatik
 Christian-Albrechts-Universitaet Kiel


 BIAS is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 (at your option) any later version.

 BIAS is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with BIAS; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "glfPBuffer_GLX.hh"
#include <OpenGLFramework/Base/glfException.hh>

using namespace BIAS;

glfPBuffer_GLX::glfPBuffer_GLX()
{
  pbuffer_ = None;
}

glfPBuffer_GLX::~glfPBuffer_GLX()
{
  DestroyDrawable();
  BIASASSERT(pbuffer_ == None);
}

GLXDrawable glfPBuffer_GLX::CreateDrawable(int width, int height, GLXFBConfig fbConfig)
{
  // create pbuffer attrib list
  int attribs[] = {
    GLX_PBUFFER_WIDTH,      width,
    GLX_PBUFFER_HEIGHT,     height,
    GLX_PRESERVED_CONTENTS, True,
    GLX_LARGEST_PBUFFER,    False,
    None
  };

  // create pbuffer
  pbuffer_ = glXCreatePbuffer(display_, fbConfig, attribs);

  return pbuffer_;
}

void glfPBuffer_GLX::DestroyDrawable()
{
  if (pbuffer_ != None) {
    glXDestroyPbuffer(display_, pbuffer_);
    pbuffer_ = None;
  }
}

int glfPBuffer_GLX::GetDrawableType() const
{
  return GLX_PBUFFER_BIT;
}

