/*
 This file is part of the BIAS library (Basic ImageAlgorithmS).

 Copyright (C)  2008, 2009    (see file CONTACT for details)
 Multimediale Systeme der Informationsverarbeitung
 Institut fuer Informatik
 Christian-Albrechts-Universitaet Kiel


 BIAS is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 (at your option) any later version.

 BIAS is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with BIAS; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "glfPBuffer_WGL.hh"
#include <OpenGLFramework/Base/glfException.hh>
#include <Gui/biasgl.h>

using namespace BIAS;
using namespace std;

glfPBuffer_WGL::glfPBuffer_WGL()
{
  hglfPBuffer_ = NULL;
}

glfPBuffer_WGL::~glfPBuffer_WGL()
{
  BIASASSERT(hglfPBuffer_ == NULL);
}

HDC glfPBuffer_WGL::CreateDeviceContext(int width, int height, int pixelFormat)
{
  // check required extensions
  if (!WGLEW_ARB_pbuffer) {
    GLF_THROW_EXCEPTION("WGL_ARB_pbuffer extension is unsupported");
  }

  // create pbuffer
  int attribs[] = {
    WGL_PBUFFER_LARGEST_ARB, 0,
    0
  };
  hglfPBuffer_ = wglCreatePbufferARB(wglGetCurrentDC(), pixelFormat,
                                  width, height, attribs);
  if (hglfPBuffer_ == NULL) {
    GLF_THROW_EXCEPTION("Failed to create glfPBuffer");
  }

  // create device context for pbuffer
  HDC hDC = wglGetPbufferDCARB(hglfPBuffer_);
  if (hDC == NULL) {
    GLF_THROW_EXCEPTION("Failed to get device context for pbuffer");
  }

  return hDC;
}

void glfPBuffer_WGL::DestroyDeviceContext()
{
  if (hDC_ != NULL) {
    wglReleasePbufferDCARB(hglfPBuffer_, hDC_);
    hDC_ = NULL;
  }

  if (hglfPBuffer_ != NULL) {
    wglDestroyPbufferARB(hglfPBuffer_);
    hglfPBuffer_ = NULL;
  }
}

void glfPBuffer_WGL::GetSize(int& width, int& height)
{
  BIASASSERT(hglfPBuffer_ != NULL);

  wglQueryPbufferARB(hglfPBuffer_, WGL_PBUFFER_WIDTH_ARB, &width);
  wglQueryPbufferARB(hglfPBuffer_, WGL_PBUFFER_HEIGHT_ARB, &height);
}

int glfPBuffer_WGL::GetWglRenderTargetAttribute() const
{
  return WGL_DRAW_TO_PBUFFER_ARB;
}
