/*
 This file is part of the BIAS library (Basic ImageAlgorithmS).

 Copyright (C)  2008, 2009    (see file CONTACT for details)
 Multimediale Systeme der Informationsverarbeitung
 Institut fuer Informatik
 Christian-Albrechts-Universitaet Kiel


 BIAS is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 (at your option) any later version.

 BIAS is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with BIAS; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "glfRenderingContext.hh"
#include <OpenGLFramework/Base/glfException.hh>
#include <Utils/GlewInitWrapper.hh>

using namespace BIAS;
using namespace std;

glfRenderingContextConfig::glfRenderingContextConfig()
{
  width         = 800;
  height        = 600;
  doubleBuffer  = false;
  redSize       = 8;
  greenSize     = 8;
  blueSize      = 8;
  alphaSize     = 0;
  depthSize     = 24;
  stencilSize   = 0;
}

void glfRenderingContextConfig::Dump(ostream& out)
{
  out << "Width: " << width << endl;
  out << "Height: " << height << endl;
  out << "DoubleBuffer: " << doubleBuffer << endl;
  out << "RedSize: " << redSize << endl;
  out << "GreenSize: " << greenSize << endl;
  out << "BlueSize: " << blueSize << endl;
  out << "AlphaSize: " << alphaSize << endl;
  out << "DepthSize: " << depthSize << endl;
  out << "StencilSize: " << stencilSize << endl;
}

void glfRenderingContext::Grab(Image<unsigned char>& image, GrabSource source,
                               int x, int y, int width, int height)
{
  // activate this rendering context
  MakeCurrent();

  // choose format for glReadPixels and numChannels for image.
  GLenum format;
  int numChannels;
  switch (source) {
  case GRAB_COLOR:
    format = GL_RGB;
    numChannels = 3;
    break;
  case GRAB_DEPTH:
    format = GL_DEPTH_COMPONENT;
    numChannels = 1;
    break;
  case GRAB_STENCIL:
    format = GL_STENCIL_INDEX;
    numChannels = 1;
    break;
  default:
    GLF_THROW_EXCEPTION("Invalid grab source");
    break;
  };

  // get configuration of context
  glfRenderingContextConfig config;
  GetConfig(config);

  // fix offset and size
  if (width == 0) {
    x = 0;
    width = config.width;
  }
  if (height == 0) {
    y = 0;
    height = config.height;
  }

  // verify that offset and size are sane
  if (x < 0 || y < 0 ||
      x + width > config.width ||
      y + height > config.height) {
    GLF_THROW_EXCEPTION("Grab rectangle is outside valid range");
  }

  // create image
  if (!image.IsEmpty()) {
    image.Release();
  }
  image.Init(width, height, numChannels);

  // copy data to image
  glReadPixels(x, y, width, height, format,
               GL_UNSIGNED_BYTE, image.GetImageData());
}

void glfRenderingContext::InitGlew()
{
#if defined (BIAS_HAVE_GLEW) 
  GlewInitWrapper::GetInstance()->Init();
#endif
}
