/*
 * GpuImageFilter.cpp
 *
 *  Created on: Mar 10, 2010
 *      Author: djung
 */

#include <OpenGLFramework/SpecializedBatches/GpuImageFilter.hh>
#include <OpenGLFramework/Base/glfFormatDB.hh>
#include <Base/Image/ImageIO.hh>

using namespace std;
using namespace BIAS;

GpuImageFilter::GpuImageFilter() {
  width_ = 0;
  height_ = 0;
  gaussArray_ = NULL;
  pTextureA_ = new glfTexture2D();
  pTextureB_ = new glfTexture2D();
  factor_ = 1. / sqrt(2 * M_PI);
  exp_ = 0;

  internalFormat_ = GL_LUMINANCE32F_ARB;
  ft_ = none;

  fragmentShaderCodeMean1_ = ""
    " uniform sampler2D inImg;"
    " uniform float stepW;"
    " uniform int rad;"
    ""
    " void main()"
    " {"
    "   vec4 result;"
    "   vec2 pos = gl_TexCoord[0].st;"
    "   pos[0] -= (float)rad * stepW;"
    "   for(int i = 0; i <= rad * 2; i++) {"
    "     result += texture2D(inImg, pos);"
    "     pos[0] += stepW;"
    "   }"
    "   gl_FragColor = result / (float) (rad * 2. + 1.);"
    " }";

  fragmentShaderCodeMean2_ = ""
    " uniform sampler2D inImg;"
    " uniform float stepH;"
    " uniform int rad;"
    ""
    " void main()"
    " {"
    "   vec4 result;"
    "   vec2 pos = gl_TexCoord[0].st;"
    "   pos[1] -= (float)rad * stepH;"
    "   for(int i = 0; i <= rad * 2; i++) {"
    "     result += texture2D(inImg, pos);"
    "     pos[1] += stepH;"
    "   }"
    "   gl_FragColor = result / (float) (rad * 2. + 1.);"
    " }";

  fragmentShaderCodeMedian3x3planar_ = ""
    "uniform sampler2D inImg;"
    "uniform float stepW;"
    "uniform float stepH;"
    ""
    "void main() {"
    "  vec2 pos = gl_TexCoord[0].st;"
    "  vec2 curPos = pos;"
    "  vec4 sort[9];"
    "  float tmp;"
    "  sort[0] = texture2D(inImg, curPos);"
    "  curPos[0] -= stepW;"
    "  sort[1] = texture2D(inImg, curPos);"
    "  curPos[0] += stepW * 2.0;"
    "  sort[2] = texture2D(inImg, curPos);"
    "  curPos[1] -= stepH;"
    "  sort[3] = texture2D(inImg, curPos);"
    "  curPos[0] -= stepW;"
    "  sort[4] = texture2D(inImg, curPos);"
    "  curPos[0] -= stepW;"
    "  sort[5] = texture2D(inImg, curPos);"
    "  curPos[1] += stepH * 2.0;"
    "  sort[6] = texture2D(inImg, curPos);"
    "  curPos[0] += stepW;"
    "  sort[7] = texture2D(inImg, curPos);"
    "  curPos[0] += stepW;"
    "  sort[8] = texture2D(inImg, curPos);"
    "  for (int i = 0; i < 8; i++) {"
    "    for (int j = i + 1; j < 9; j++) {"
    "      if (sort[j].x < sort[i].x) {"
    "        tmp = sort[i].x;"
    "        sort[i].x = sort[j].x;"
    "        sort[j].x = tmp;"
    "      }"
    "      if (sort[j].y < sort[i].y) {"
    "        tmp = sort[i].y;"
    "        sort[i].y = sort[j].y;"
    "        sort[j].y = tmp;"
    "      }"
    "      if (sort[j].z < sort[i].z) {"
    "        tmp = sort[i].z;"
    "        sort[i].z = sort[j].z;"
    "        sort[j].z = tmp;"
    "      }"
    "      if (sort[j].w < sort[i].w) {"
    "        tmp = sort[i].w;"
    "        sort[i].w = sort[j].w;"
    "        sort[j].w = tmp;"
    "      }"
    "    }"
    "  }"
    "  gl_FragColor = sort[4];"
    "}";

  // 31x31 filter max
  fragmentShaderCodeMedianPlanar_ = ""
    "uniform sampler2D inImg;"
    "uniform float stepW;"
    "uniform float stepH;"
    "uniform int rad;"
    ""
    "void main() {"
    "  vec2 pos = gl_TexCoord[0].st;"
    "  pos[0] -= (float)rad * stepW;"
    "  pos[1] -= (float)rad * stepH;"
    "  float resPos = pos[0];"
    "  int rad2 = rad*2+1;"
    "  int rad3 = rad2*rad2;"
    "  vec4 sort[961];"
    "  float tmp;"
    "  for (int i = 0; i < rad2; i++) {"
    "    for (int j = 0; j < rad2; j++) {"
    "      sort[j + i*rad2] = texture2D(inImg, pos);"
    "      pos[0] += stepW;"
    "    }"
    "    pos[0] = resPos;"
    "    pos[1] += stepH;"
    "  }"
    "  for (int i = 0; i < rad3-1; i++) {"
    "    for (int j = i + 1; j < rad3; j++) {"
    "      if (sort[j].x < sort[i].x) {"
    "        tmp = sort[i].x;"
    "        sort[i].x = sort[j].x;"
    "        sort[j].x = tmp;"
    "      }"
    "      if (sort[j].y < sort[i].y) {"
    "        tmp = sort[i].y;"
    "        sort[i].y = sort[j].y;"
    "        sort[j].y = tmp;"
    "      }"
    "      if (sort[j].z < sort[i].z) {"
    "        tmp = sort[i].z;"
    "        sort[i].z = sort[j].z;"
    "        sort[j].z = tmp;"
    "      }"
    "      if (sort[j].w < sort[i].w) {"
    "        tmp = sort[i].w;"
    "        sort[i].w = sort[j].w;"
    "        sort[j].w = tmp;"
    "      }"
    "    }"
    "  }"
    "  gl_FragColor = sort[(int)((rad3-1)/2)];"
    "}";

  // 31x31 filter max
  fragmentShaderCodeMedianGrey_ = ""
    "uniform sampler2D inImg;"
    "uniform float stepW;"
    "uniform float stepH;"
    "uniform int rad;"
    ""
    "void main() {"
    "  vec2 pos = gl_TexCoord[0].st;"
    "  pos[0] -= (float)rad * stepW;"
    "  pos[1] -= (float)rad * stepH;"
    "  float resPos = pos[0];"
    "  vec4 color;"
    "  int rad2 = rad*2+1;"
    "  int rad3 = rad2*rad2;"
    "  vec3 sortMe[961];"
    "  vec3 tmp;"
    "  for (int i = 0; i < rad2; i++) {"
    "    for (int j = 0; j < rad2; j++) {"
    "      color = texture2D(inImg, pos);"
    "      sortMe[j + i*rad2].x = (color.x+color.y+color.z+color.w)/4.;"
    "      sortMe[j + i*rad2].yz = pos;"
    "      pos[0] += stepW;"
    "    }"
    "    pos[0] = resPos;"
    "    pos[1] += stepH;"
    "  }"
    "  for (int i = 0; i < rad3-1; i++) {"
    "    for (int j = i + 1; j < rad3; j++) {"
    "      if (sortMe[j].x < sortMe[i].x) {"
    "        tmp = sortMe[i];"
    "        sortMe[i] = sortMe[j];"
    "        sortMe[j] = tmp;"
    "      }"
    "    }"
    "  }"
    "  gl_FragColor = texture2D(inImg, sortMe[(int)((rad3-1)/2)].yz);"
    "}";

  fragmentShaderCodeMedian3x3grey_ = ""
    "uniform sampler2D inImg;"
    "uniform float stepW;"
    "uniform float stepH;"
    ""
    "void main() {"
    "  vec2 pos = gl_TexCoord[0].st;"
    "  vec2 curPos = pos;"
    "  vec4 sort[9];"
    "  float sortMe[9];"
    "  int sortMe2[9];"
    "  float tmp;"
    "  int tmp2;"
    "  sort[0] = texture2D(inImg, curPos);"
    "  sortMe[0] = (sort[0].x + sort[0].y + sort[0].z + sort[0].w)/4.;"
    "  sortMe2[0] = 0;"
    "  curPos[0] -= stepW;"
    "  sort[1] = texture2D(inImg, curPos);"
    "  sortMe[1] = (sort[1].x + sort[1].y + sort[1].z + sort[1].w)/4.;"
    "  sortMe2[1] = 1;"
    "  curPos[0] += stepW * 2.0;"
    "  sort[2] = texture2D(inImg, curPos);"
    "  sortMe[2] = (sort[2].x + sort[2].y + sort[2].z + sort[2].w)/4.;"
    "  sortMe2[2] = 2;"
    "  curPos[1] -= stepH;"
    "  sort[3] = texture2D(inImg, curPos);"
    "  sortMe[3] = (sort[3].x + sort[3].y + sort[3].z + sort[3].w)/4.;"
    "  sortMe2[3] = 3;"
    "  curPos[0] -= stepW;"
    "  sort[4] = texture2D(inImg, curPos);"
    "  sortMe[4] = (sort[4].x + sort[4].y + sort[4].z + sort[4].w)/4.;"
    "  sortMe2[4] = 4;"
    "  curPos[0] -= stepW;"
    "  sort[5] = texture2D(inImg, curPos);"
    "  sortMe[5] = (sort[5].x + sort[5].y + sort[5].z + sort[5].w)/4.;"
    "  sortMe2[5] = 5;"
    "  curPos[1] += stepH * 2.0;"
    "  sort[6] = texture2D(inImg, curPos);"
    "  sortMe[6] = (sort[6].x + sort[6].y + sort[6].z + sort[6].w)/4.;"
    "  sortMe2[6] = 6;"
    "  curPos[0] += stepW;"
    "  sort[7] = texture2D(inImg, curPos);"
    "  sortMe[7] = (sort[7].x + sort[7].y + sort[7].z + sort[7].w)/4.;"
    "  sortMe2[7] = 7;"
    "  curPos[0] += stepW;"
    "  sort[8] = texture2D(inImg, curPos);"
    "  sortMe[8] = (sort[8].x + sort[8].y + sort[8].z + sort[8].w)/4.;"
    "  sortMe2[8] = 8;"
    "  for (int i = 0; i < 8; i++) {"
    "    for (int j = i + 1; j < 9; j++) {"
    "      if (sortMe[j] < sortMe[i]) {"
    "        tmp = sortMe[i];"
    "        tmp2 = sortMe2[i];"
    "        sortMe[i] = sortMe[j];"
    "        sortMe2[i] = sortMe2[j];"
    "        sortMe[j] = tmp;"
    "        sortMe2[j] = tmp2;"
    "      }"
    "    }"
    "  }"
    "  gl_FragColor = sort[sortMe2[4]];"
    "}";

  fragmentShaderCodeGauss1_ = ""
    "uniform sampler2D inImg;"
    "uniform float stepW;"
    "uniform int rad;"
    "uniform float gaussArray[100];"
    ""
    "void main()"
    "{"
    "  vec4 result;"
    "  vec2 pos = gl_TexCoord[0].st;"
    "  float weightSum = 0.;"
    "  int i = 0;"
    "  pos[0] -= (float)rad * stepW;"
    "  for(i; i <= rad; i++) {"
    "    result += texture2D(inImg, pos) * gaussArray[i];"
    "    weightSum += gaussArray[i];"
    "    pos[0] += stepW;"
    "  }"
    "  i = rad - 1;"
    "  for(i; i >= 0; i--) {"
    "    result += texture2D(inImg, pos) * gaussArray[i];"
    "    weightSum += gaussArray[i];"
    "    pos[0] += stepW;"
    "  }"
    "  gl_FragColor = result / weightSum;"
    "}";

  fragmentShaderCodeGauss2_ = ""
    "uniform sampler2D inImg;"
    "uniform float stepH;"
    "uniform int rad;"
    "uniform float gaussArray[100];"
    ""
    "void main()"
    "{"
    "  vec4 result;"
    "  vec2 pos = gl_TexCoord[0].st;"
    "  float weightSum = 0.;"
    "  int i = 0;"
    "  pos[1] -= (float)rad * stepH;"
    "  for(i; i <= rad; i++) {"
    "    result += texture2D(inImg, pos) * gaussArray[i];"
    "    weightSum += gaussArray[i];"
    "    pos[1] += stepH;"
    "  }"
    "  i = rad - 1;"
    "  for(i; i >= 0; i--) {"
    "    result += texture2D(inImg, pos) * gaussArray[i];"
    "    weightSum += gaussArray[i];"
    "    pos[1] += stepH;"
    "  }"
    "  gl_FragColor = result / weightSum;"
    "}";

  fragmentShaderCodeCp_ = ""
    " uniform sampler2D inImg;"
    " uniform float stepW;"
    " uniform float stepH;"
    ""
    " void main()"
    " {"
    "   vec2 pos = gl_TexCoord[0].st;"
    "   vec4 result = texture2D(inImg, pos);"
    "   gl_FragColor = result;"
    " }";
}

GpuImageFilter::~GpuImageFilter() {
  delete pTextureA_;
  delete pTextureB_;
  delete gaussArray_;
}

void GpuImageFilter::FilterMedian(ImageBase &in, ImageBase &out, enum ImageBase::EStorageType type, int rad,
    float threshold, bool planar) {
  ft_ = median;
  planar_ = planar;
  Filter_(in, out, type, rad, threshold);
}

void GpuImageFilter::FilterMean(ImageBase &in, ImageBase &out, enum ImageBase::EStorageType type, int rad,
    float threshold) {
  ft_ = mean;
  Filter_(in, out, type, rad, threshold);
}

void GpuImageFilter::FilterGauss(ImageBase &in, ImageBase &out, enum ImageBase::EStorageType type, int rad,
    float threshold) {
  cout << " factor_: " << factor_ << endl;
  if (fabs(threshold - 1.) < 1e-5) {
    exp_ = 0;
    cout << "S1 ";
  }
  else {
    exp_ = 1;
    cout << "S_threshold ";
  }
  gaussArray_ = new float[100]; // [rad]
  cout << "Gauss filter mask: ";
  for (int i = 0; i <= rad; i++) {
    switch (exp_) {
      case 0:
        gaussArray_[i] = (float) ExpS1_(rad - i);
      break;
      default:
        gaussArray_[i] = (float) Exp_(rad - i, threshold);
      break;
    }
    cout << gaussArray_[i] << " " << flush;
  }
  cout << endl;
  ft_ = gauss;
  Filter_(in, out, type, rad, threshold);
}

void GpuImageFilter::FilterCp(ImageBase &in, ImageBase &out, enum ImageBase::EStorageType type, int rad,
    float threshold) {
  ft_ = none;
  Filter_(in, out, type, rad, threshold);
}

void GpuImageFilter::Filter_(ImageBase &in, ImageBase &out, enum ImageBase::EStorageType type, int rad, float threshold) {
  width_ = in.GetWidth();
  height_ = in.GetHeight();
  internalFormat_ = glfFormatDB::ProposeInternalFormat(in.GetStorageType(), in.GetColorModel());
  Init_(rad);
  pTextureA_->UploadImage(in, internalFormat_);
  GetBatch_()->SetTexture(pTextureA_, 0);
  GetShaderProgram_()->SetUniform("inImg", (int) 0);
  GetShaderProgram_()->SetUniform("stepW", 1.0f / (float) width_);
  switch (ft_) {
    case mean:
      DrawMean_(rad);
    break;
    case median:
      DrawMedian_(rad, threshold);
    break;
    case gauss:
      DrawGauss_(rad);
    break;
    default:
      DrawCp_();
    break;
  }
  out.Init(width_, height_, in.GetChannelCount(), in.GetStorageType());
  pTextureB_->CopyToImage(out);
}

void GpuImageFilter::Init_(int rad) {
  SetViewPort(width_, height_);
  SetDefaultRenderTarget(width_, height_, internalFormat_);
  InitBatch_();
  switch (ft_) {
    case mean:
      CreateFragmentShaderProgram(fragmentShaderCodeMean1_);
    break;
    case median:
      switch (rad) {
        case 1:
          if (planar_) {
            CreateFragmentShaderProgram(fragmentShaderCodeMedian3x3planar_);
          }
          else {
            CreateFragmentShaderProgram(fragmentShaderCodeMedian3x3grey_);
          }
        break;
        default:
          if (planar_) {
            CreateFragmentShaderProgram(fragmentShaderCodeMedianPlanar_);
          }
          else {
            CreateFragmentShaderProgram(fragmentShaderCodeMedianGrey_);
          }
        break;
      }
    break;
    case gauss:
      CreateFragmentShaderProgram(fragmentShaderCodeGauss1_);
    break;
    default:
      CreateFragmentShaderProgram(fragmentShaderCodeCp_);
    break;
  }
  pTextureA_->Create();
  pTextureA_->SetMagFilter(GL_NEAREST);
  pTextureA_->SetMinFilter(GL_NEAREST);
  pTextureA_->SetWrapS(GL_CLAMP);
  pTextureA_->SetWrapT(GL_CLAMP);
  pTextureA_->Allocate(width_, height_, internalFormat_);
  pTextureB_->Create();
  pTextureB_->SetMagFilter(GL_NEAREST);
  pTextureB_->SetMinFilter(GL_NEAREST);
  pTextureB_->SetWrapS(GL_CLAMP);
  pTextureB_->SetWrapT(GL_CLAMP);
  pTextureB_->Allocate(width_, height_, internalFormat_);
}

void GpuImageFilter::DrawCp_() {
  if (pTextureA_ == NULL) {
    BIASERR("no glf textures available");
    BIASABORT;
  }
  GetRenderTarget()->AttachTexture(*pTextureB_, GL_COLOR_ATTACHMENT0_EXT);
  GetRenderTarget()->ClearDepthBuffer(1.0);
  GetRenderTarget()->ClearColorBuffer(0, 0, 0, 0);
  GetShaderProgram_()->SetUniform("stepH", 1.0f / (float) height_);
  GetBatch_()->Draw();
}

void GpuImageFilter::DrawMean_(int rad) {
  if (pTextureA_ == NULL || pTextureB_ == NULL) {
    BIASERR("no glf textures available");
    BIASABORT;
  }
  GetRenderTarget()->ClearDepthBuffer(1.0);
  GetRenderTarget()->ClearColorBuffer(0, 0, 0, 0);
  GetShaderProgram_()->SetUniform("rad", rad);
  GetBatch_()->Draw();

  // 2nd pass
  CreateFragmentShaderProgram(fragmentShaderCodeMean2_);
  GetBatch_()->SetTexture(GetTextureFromDefaultRenderTarget(), 0);
  GetRenderTarget()->AttachTexture(*pTextureB_, GL_COLOR_ATTACHMENT0_EXT);
  GetRenderTarget()->ClearDepthBuffer(1.0);
  GetRenderTarget()->ClearColorBuffer(0, 0, 0, 0);

  GetShaderProgram_()->SetUniform("inImg", (int) 0);
  GetShaderProgram_()->SetUniform("stepH", 1.0f / (float) height_);
  GetShaderProgram_()->SetUniform("rad", rad);
  GetBatch_()->Draw();
}

void GpuImageFilter::DrawGauss_(int rad) {
  if (pTextureA_ == NULL || pTextureB_ == NULL) {
    BIASERR("no glf textures available");
    BIASABORT;
  }
  GetRenderTarget()->ClearDepthBuffer(1.0);
  GetRenderTarget()->ClearColorBuffer(0, 0, 0, 0);
  GetShaderProgram_()->SetUniform("rad", rad);
  GetShaderProgram_()->SetUniform("gaussArray", 100, gaussArray_);
  GetBatch_()->Draw();

  // 2nd pass
  CreateFragmentShaderProgram(fragmentShaderCodeGauss2_);
  GetBatch_()->SetTexture(GetTextureFromDefaultRenderTarget(), 0);
  GetRenderTarget()->AttachTexture(*pTextureB_, GL_COLOR_ATTACHMENT0_EXT);
  GetRenderTarget()->ClearDepthBuffer(1.0);
  GetRenderTarget()->ClearColorBuffer(0, 0, 0, 0);

  GetShaderProgram_()->SetUniform("inImg", (int) 0);
  GetShaderProgram_()->SetUniform("gaussArray", 100, gaussArray_);
  GetShaderProgram_()->SetUniform("stepH", 1.0f / (float) height_);
  GetShaderProgram_()->SetUniform("rad", rad);
  GetBatch_()->Draw();
}

void GpuImageFilter::DrawMedian_(int rad, float threshold) {
  if (pTextureA_ == NULL) {
    BIASERR("no glf textures available")
    BIASABORT
  }
  GetRenderTarget()->AttachTexture(*pTextureB_, GL_COLOR_ATTACHMENT0_EXT);
  GetRenderTarget()->ClearDepthBuffer(1.0);
  GetRenderTarget()->ClearColorBuffer(0, 0, 0, 0);
  GetShaderProgram_()->SetUniform("stepH", 1.0f / (float) height_);
  switch (rad) {
    case 1:
    break;
    default:
      GetShaderProgram_()->SetUniform("rad", rad);
    break;
  }
  GetBatch_()->Draw();
}

int GpuImageFilter::Binomial_(int n, int k) { // n = halfwinsize * 2; k = position in window
  if (k <= 0 || k >= n) {
    return 1;
  }
  else {
    return Binomial_(n - 1, k - 1) + Binomial_(n - 1, k);
  }
}

