/*
 * PerPixelProcessingBase.cpp
 *
 *  Created on: Feb 4, 2010
 *      Author: africk
 */

#include <OpenGLFramework/SpecializedBatches/PerPixelProcessingBase.hh>

using namespace std;
using namespace BIAS;

PerPixelProcessingBase::PerPixelProcessingBase() {
	renderTargetP_ = NULL;
	zBufferP_ = NULL;
	textureP_ = NULL;
	shaderProgramP_ = NULL;
	fragmentShaderP_ = NULL;
	renderTargetCreated_ = false;
}

PerPixelProcessingBase::~PerPixelProcessingBase() {
	if (renderTargetP_ != NULL && renderTargetCreated_) {
		delete renderTargetP_;
	}

	if (textureP_ != NULL) {
		delete textureP_;
	}

	if (zBufferP_ != NULL) {
		delete zBufferP_;
	}

	if (shaderProgramP_ != NULL) {
		delete shaderProgramP_;
	}

	if (fragmentShaderP_ != NULL) {
		delete fragmentShaderP_;
	}
}

void PerPixelProcessingBase::SetDefaultRenderTarget(unsigned int width,
		unsigned height, GLenum internalFormat) {

	BIASASSERT(width > 0 && height > 0);
	try {
		if (zBufferP_ != NULL) {
			delete zBufferP_;
		}

		zBufferP_ = new glfTexture2D();

		if (textureP_ != NULL) {
			delete textureP_;
		}

		textureP_ = new glfTexture2D();

		if (renderTargetP_ != NULL && renderTargetCreated_) {
			delete renderTargetP_;
		}

		renderTargetP_ = new glfFramebufferObject();

		zBufferP_->Create();
		zBufferP_->SetMagFilter(GL_NEAREST);
		zBufferP_->SetMinFilter(GL_NEAREST);
		zBufferP_->SetWrapS(GL_CLAMP);
		zBufferP_->SetWrapT(GL_CLAMP);
		zBufferP_->Allocate(width, height, GL_DEPTH_COMPONENT);

		textureP_->Create();
		textureP_->SetMagFilter(GL_NEAREST);
		textureP_->SetMinFilter(GL_NEAREST);
		textureP_->SetWrapS(GL_CLAMP);
		textureP_->SetWrapT(GL_CLAMP);
		textureP_->Allocate(width, height, internalFormat);

		renderTargetP_->Create();
		renderTargetP_->AttachTexture(*textureP_, GL_COLOR_ATTACHMENT0_EXT);
		renderTargetP_->AttachTexture(*zBufferP_, GL_DEPTH_ATTACHMENT_EXT);
		renderTargetP_->CheckComplete();

		renderTargetCreated_ = true;

		batch_.SetRenderTarget(renderTargetP_);
	} catch (glfException& e) {
		std::cout << "Error: " << e.GetMessageString() << std::endl;
		BIASABORT;
	}

}

void PerPixelProcessingBase::SetDefaultFloatRenderTarget(unsigned int width,
     unsigned height) {
  SetDefaultRenderTarget(width, height, GL_LUMINANCE32F_ARB);
}

void PerPixelProcessingBase::SetDefaultRGBRenderTarget(unsigned int width,
		unsigned height) {
  SetDefaultRenderTarget(width, height, GL_RGB);
}

glfShaderProgram* PerPixelProcessingBase::CreateFragmentShaderProgram(
		const std::string& fragmentShaderCode) {
	try {
		if (fragmentShaderP_ != NULL) {
			delete fragmentShaderP_;
		}

		if (shaderProgramP_ != NULL) {
			delete shaderProgramP_;
		}

		shaderProgramP_ = new glfShaderProgram();
		fragmentShaderP_ = new glfShader();

		shaderProgramP_->Create();
		fragmentShaderP_->Create(GL_FRAGMENT_SHADER, fragmentShaderCode);
		shaderProgramP_->AttachShader(*fragmentShaderP_);
		shaderProgramP_->Link();
		batch_.SetShaderProgram(shaderProgramP_);
	} catch (glfException& e) {
		std::cout << "Error: " << e.GetMessageString() << std::endl;
		BIASABORT;
	}

	return shaderProgramP_;
}

glfShaderProgram* PerPixelProcessingBase::CreateFragmentShaderProgramFromFile(
		const std::string& fragmentShaderPath) {
	try {
		if (fragmentShaderP_ != NULL) {
			delete fragmentShaderP_;
		}

		if (shaderProgramP_ != NULL) {
			delete shaderProgramP_;
		}

		shaderProgramP_ = new glfShaderProgram();
		fragmentShaderP_ = new glfShader();

		shaderProgramP_->Create();
		fragmentShaderP_->CreateFromFile(GL_FRAGMENT_SHADER, fragmentShaderPath);
		shaderProgramP_->AttachShader(*fragmentShaderP_);
		shaderProgramP_->Link();
		batch_.SetShaderProgram(shaderProgramP_);
	} catch (glfException& e) {
		std::cout << "Error: " << e.GetMessageString() << std::endl;
		BIASABORT;
	}

	return shaderProgramP_;
}

//###################################
