/*
 * SeparabelBoxFilter.cpp
 *
 *  Created on: Feb 5, 2010
 *      Author: africk
 */

#include <OpenGLFramework/SpecializedBatches/SeparabelBoxFilter.hh>

using namespace std;
using namespace BIAS;

SeparabelBoxFilter::SeparabelBoxFilter() {
	width_ = 0;
	height_ = 0;
	filterWidth_ = 1;
	filterHeight_ = 1;
	sourceP_ = NULL;
	destP_ = NULL;
	filteredXP_ = NULL;
	renderTargetP_ = NULL;
	inited_ = false;

	fragmentShaderCode_ = ""
		"uniform sampler2D dispMap;"
		"uniform float width;"
		"uniform float height;"
		"uniform float filterWidth;"
		"uniform float filterHeight;"
		"uniform int hor;"
		""
		"void main () {"
		""
		"vec2 coord = gl_TexCoord[0];"
		"float disp = 0;"
		""
		"vec2 tmp_coord = coord;"
		""
		"float size = (filterWidth*2+1)*(filterHeight*2+1);"
		""
		"if (hor == 0) {"
		""
		"int x = coord[0]*width;"
		"int startX = max(0,x-filterWidth);"
		"int endX = min(width-1,x+filterWidth);"
		""
		"int counter = 0;"
		"for (;startX <= endX; startX++) {"
		""
		"tmp_coord[0]=((float) startX)/width;"
		"disp += texture2D(dispMap, tmp_coord).r;"
		"counter++;"
		"}"
		""
		"gl_FragColor.rg = vec2(disp,counter);"
		""
		"} else {"
		""
		"int y = coord[1]*height;"
		"int startY = max(0,y-filterHeight);"
		"int endY = min(height-1,y+filterHeight);"
		""
		"int counter = 0;"
		""
		"for (;startY <= endY; startY++) {"
		""
		"tmp_coord[1]=startY/height;"
		"disp += texture2D(dispMap, tmp_coord).r;"
		"counter++;"
		""
		"}"
		""
		"gl_FragColor.r = disp / (texture2D(dispMap, coord).g * counter);"
		""
		"}"
		""
		"}"
		""
		"";
}

SeparabelBoxFilter::~SeparabelBoxFilter() {
	if (filteredXP_ != NULL) {
		delete filteredXP_;
	}

	if (renderTargetP_ != NULL) {
		delete renderTargetP_;
	}
}

void SeparabelBoxFilter::SetSourceAndDest(glfTexture2D* source,
		glfTexture2D* dest) {
	BIASASSERT(destP_ != NULL && destP_ != NULL);
	sourceP_ = source;
	destP_ = dest;

	unsigned int oldWidth = width_;
	unsigned int oldHeight = height_;

	width_ = sourceP_->GetWidth();
	height_ = sourceP_->GetHeight();

	BIASASSERT(width_ > 0 && height_ > 0);BIASASSERT((int)width_ == destP_->GetWidth() && (int)height_ == destP_->GetHeight());
	if (width_ != oldWidth || height_ != oldHeight) {
		SetViewPort(width_, height_);
		if (renderTargetP_ != NULL) {
			delete renderTargetP_;
		}

		if (filteredXP_ != NULL) {
			delete filteredXP_;
		}

		CreateRenderTarget_();
		CreateFilteredXTexture_(width_, height_);
		GetBatch_()->SetRenderTarget(renderTargetP_);
	}
}

void SeparabelBoxFilter::Init() {
	InitBatch_();
	CreateFragmentShaderProgram(fragmentShaderCode_);

	inited_ = true;
}

void SeparabelBoxFilter::Draw() {
	if (inited_) {
		if (sourceP_ != NULL && destP_ != NULL) {
			renderTargetP_->AttachTexture(*filteredXP_,GL_COLOR_ATTACHMENT0_EXT);
			//renderTargetP_->ClearDepthBuffer(1.0);
			renderTargetP_->ClearColorBuffer(0, 0, 0, 0);
			GetBatch_()->SetTexture(sourceP_, 0);
			GetShaderProgram_()->SetUniform("dispMap", (int) 0);
			GetShaderProgram_()->SetUniform("width", (float) width_);
			GetShaderProgram_()->SetUniform("height", (float) height_);
			GetShaderProgram_()->SetUniform("filterWidth", (float) filterWidth_);
			GetShaderProgram_()->SetUniform("filterHeight",
					(float) filterHeight_);
			GetShaderProgram_()->SetUniform("hor", (int) 0);
			GetBatch_()->Draw();
			//-------------------

			GetBatch_()->SetTexture(filteredXP_, 0);
			renderTargetP_->AttachTexture(*destP_, GL_COLOR_ATTACHMENT0_EXT);
			//renderTargetP_->ClearDepthBuffer(1.0);
			renderTargetP_->ClearColorBuffer(0, 0, 0, 0);
			GetShaderProgram_()->SetUniform("hor", (int) 1);
			GetBatch_()->Draw();

		} else {
			BIASERR("set disparity map first !");
			BIASABORT;
		}
	} else {
		BIASERR("call init first !");
		BIASABORT;
	}
}
