/*
 * SeparabelBoxFilter.hh
 *
 *  Created on: Feb 5, 2010
 *      Author: africk
 */

#ifndef __SEPARABEL_BOXFILTER_HH__
#define __SEPARABEL_BOXFILTER_HH__

#include <OpenGLFramework/Base/glfTexture2D.hh>
#include <OpenGLFramework/Base/glfBatch.hh>
#include <OpenGLFramework/Base/glfFramebufferObject.hh>
#include <OpenGLFramework/SpecializedBatches/PerPixelProcessingBase.hh>

#include <bias_config.h>

namespace BIAS {

class BIASOpenGLFramework_EXPORT SeparabelBoxFilter: public PerPixelProcessingBase {
public:
	SeparabelBoxFilter();
	virtual ~SeparabelBoxFilter();

	void SetSourceAndDest(glfTexture2D* source, glfTexture2D* dest);
	void SetFilterSize(unsigned int width, unsigned int height) {
		filterWidth_ = width;
		filterHeight_ = height;
	}

	void Init();

	void Draw();

	glfTexture2D* GetFilteredTexture() {
		return destP_;
	}

	int GetFilteredTexture(ImageBase& texture) {
		if (destP_ != NULL) {
			destP_->CopyToImage(texture);
		} else {
			return -1;
		}

		return 0;
	}

private:

	bool inited_;
	unsigned int width_, height_;
	unsigned int filterWidth_, filterHeight_;

	glfTexture2D* destP_;
	glfTexture2D* sourceP_;

	glfTexture2D* filteredXP_;
	glfFramebufferObject* renderTargetP_;

	std::string fragmentShaderCode_;

	void CreateRenderTarget_() {
		renderTargetP_ = new glfFramebufferObject();
		renderTargetP_->Create();
	}

	void CreateFilteredXTexture_(unsigned int width, unsigned int height) {
		filteredXP_ = new glfTexture2D();
		filteredXP_->Create();
		filteredXP_->SetMagFilter(GL_NEAREST);
		filteredXP_->SetMinFilter(GL_NEAREST);
		filteredXP_->SetWrapS(GL_CLAMP);
		filteredXP_->SetWrapT(GL_CLAMP);
		filteredXP_->Allocate(width, height, GL_RGB32F_ARB);
	}
};

}

#endif /* __SEPARABEL_BOXFILTER_HH__ */
