/*
 * SeparableBilateralFilter.hh
 *
 *  Created on: Feb 15, 2010
 *      Author: africk
 */

#ifndef __SEPARABLE_BILATERAL_FILTER_HH__
#define __SEPARABLE_BILATERAL_FILTER_HH__

#include <OpenGLFramework/SpecializedBatches/PerPixelProcessingBase.hh>

#include <bias_config.h>

namespace BIAS {

class BIASOpenGLFramework_EXPORT SeparableBilateralFilter: public PerPixelProcessingBase {
public:
	SeparableBilateralFilter();
	virtual ~SeparableBilateralFilter();

	void SetColorAndSourceAndDest(glfTexture2D* color, glfTexture2D* source, glfTexture2D* dest);
	void SetFilterHalfSize(unsigned int halfSize) {
		filterHalfSize_ = halfSize;
	}

	void SetColorScale(float colorScale) {
		colorScale_ = colorScale;
	}

	void SetSpatialScale(float spatialScale) {
		spatialScale_ = spatialScale;
	}

	void Init();

	void Draw();

	glfTexture2D* GetFilteredTexture() {
		return destP_;
	}

	int GetFilteredTexture(ImageBase& texture) {
		if (destP_ != NULL) {
			destP_->CopyToImage(texture);
		} else {
			return -1;
		}

		return 0;
	}

private:
	bool inited_;
	unsigned int width_, height_;
	unsigned int filterHalfSize_;
	float colorScale_, spatialScale_;

	BIAS::Vector<float> pixelOffset_;

	glfTexture2D* destP_;
	glfTexture2D* sourceP_;
	glfTexture2D* colorP_;
	glfTexture2D* filteredXP_;

	glfFramebufferObject* renderTargetP_;

	void CreateRenderTarget_(unsigned int width, unsigned int height) {
		renderTargetP_ = new glfFramebufferObject();
		renderTargetP_->Create();
	}

	void CreateFilteredXTexture_(unsigned int width, unsigned int height) {
		filteredXP_ = new glfTexture2D();
		filteredXP_->Create();
		filteredXP_->SetMagFilter(GL_NEAREST);
		filteredXP_->SetMinFilter(GL_NEAREST);
		filteredXP_->SetWrapS(GL_CLAMP);
		filteredXP_->SetWrapT(GL_CLAMP);
		filteredXP_->Allocate(width, height, GL_RGB32F_ARB);
	}

};
}

#endif /* __SEPARABLE_BILATERAL_FILTER_HH__ */
