#include "SamplerBinding.hh"
#include <OpenGLFramework/Base/glfException.hh>

using namespace std;
using namespace BIAS;

SamplerBinding::SamplerBinding() : 
  cached_(false), program_(NULL)
{
  mappedTextures_.clear();
}

void 
SamplerBinding::Clear() 
{
  cached_ = false;
  program_=NULL;
  mappedTextures_.clear();
}

void 
SamplerBinding::Cache() 
{
  BIASASSERT(program_!=NULL);
  map<std::string, textureCache>::iterator it = mappedTextures_.begin();
  for(;it!=mappedTextures_.end(); it++) {
    GLint loc = program_->GetUniformLocation(it->first);
    if( loc == -1) {
      GLF_THROW_EXCEPTION("no active uniform named "<<it->first
                          <<"! Check: 1. Is Program Linked?, 2. Typo?, 3. Correct shader?");
    }
    glGetUniformiv(program_->GetProgramId(), loc, &it->second.tu);
  }
  cached_ = true;
}
    
/** Sets all mapped samplers to appropriate texture units**/
void 
SamplerBinding::Bind()
{
  // BIASASSERT(program_!=NULL);

  if(cached_) {
    map<std::string, textureCache>::iterator it = mappedTextures_.begin();
    for(;it!=mappedTextures_.end(); it++) {
      //      cout<<"binding "<<it->first<<" to TU "<<it->second.tu<<endl;
      glActiveTexture(GL_TEXTURE0 + it->second.tu);
      it->second.tex->Bind();
    }    
  } else {

    map<std::string, textureCache>::iterator it = mappedTextures_.begin();
    for(;it!=mappedTextures_.end(); it++) {
      GLint loc = program_->GetUniformLocation(it->first);
      if( loc == -1) {
        GLF_THROW_EXCEPTION("no active uniform named "<<it->first
                            <<"! Check: 1. Is Program Linked?, 2. Typo?, 3. Correct shader?");
      }
      GLint tu;
      glGetUniformiv(program_->GetProgramId(), loc, &tu);
      glActiveTexture(GL_TEXTURE0 + tu);
      it->second.tex->Bind();
    }
  }

}

void 
SamplerBinding::SetProgram(glfShaderProgram* program) 
{
  program_ = program;
  cached_ = false;
}

void 
SamplerBinding::MapSampler(glfTexture* texture, const std::string& UniformName)
{
  mappedTextures_[UniformName].tex = texture;
  cached_ = false;
}

