/*
 This file is part of the BIAS library (Basic ImageAlgorithmS).

 Copyright (C)  2008, 2009    (see file CONTACT for details)
 Multimediale Systeme der Informationsverarbeitung
 Institut fuer Informatik
 Christian-Albrechts-Universitaet Kiel


 BIAS is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 (at your option) any later version.

 BIAS is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with BIAS; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __SamplerBinding_hh__
#define __SamplerBinding_hh__

#include <OpenGLFramework/Base/glfTexture.hh>
#include <OpenGLFramework/Base/glfShaderProgram.hh>
#include <map>

namespace BIAS {

  /** Helper to manage the binding of textures to texture units via uniform names.
      Class does not change the set progra, it only uses it to query the required slot number.

     \author bartczak 08/2009
  */
  class BIASOpenGLFramework_EXPORT SamplerBinding {
  public:

    SamplerBinding();    

    /** Call this to build a cache from the current object states.
      \attention The caching is only tracked for the methods known of this class, all things changed
      directly on the objects will not update the instance.
     **/
    void Cache();
    
    /** Sets all mapped samplers to appropriate texture units**/
    void Bind();
    void SetProgram(glfShaderProgram* program);
    void MapSampler(glfTexture* texture, const std::string& UniformName);
    void Clear();

  private:
    bool cached_;
    glfShaderProgram* program_;
    struct textureCache {
      textureCache() : tex(NULL), tu(0)  {}
      glfTexture* tex;
      GLint tu;
    };
    std::map<std::string, textureCache> mappedTextures_;

  };
}
#endif
