#ifndef __ISEL_LINEAR_CONTROL_hh__
#define __ISEL_LINEAR_CONTROL_hh__

#ifndef WIN32
#include <unistd.h>
#endif
#include <stdio.h>
#include <iostream>
#include <vector>
#include <Base/Debug/Debug.hh>

extern "C"
{
#include "include/ptu.h"
#ifndef WIN32
#include "include/linuxser.h"
#else
#include "include/W32SERIA.H"
#endif
}

namespace BIAS{

  /**
     @class IselLinearControl
     This class controls the Isel Linear Unit 
     @author fkellner, 09/09
  */ 
  class BIASPanTilt_EXPORT IselLinearControl : public BIAS::Debug {
  public:

    /**
       standard constructor / destructor 
    */
    IselLinearControl();  
    ~IselLinearControl();
    
    /**
       Start Reference Tour
       @return bool indicating success of reset
    */
    int Reset();
    
    /**
       opens the serial port and initialises unit 
       @param int the number of the serial port
    */
    int Init(int nCOMportNumber);

    /** 
       opens the given port and initialises pan tilt unit 
       @param char[] the name of the port
    */
    int Init(const char *COMportName);
    
    /**
       closes the comport stream connected to ptu
    */
    void Close();
    
    //do exactly what you think, just move millimeters in direction
    int MoveLeft(float millimeters);
    int MoveRight(float millimeters);

    //set basic stuff for movement
    int SetSpeed(int speed);
    int SetAcceleration(int accel);

    /**
       @returns the current speed
    */
    void GetSpeed(int &speed);

    /**
       @returns the current Acceleration
    */
    void GetAcceleration(int &accel);

    /**
       @return the current position of the Unit
    */
    void GetCurrentPosition(int &pos); 
    
    int SetPosition(int pos, bool bWaitComplete=true);

    /**
       Send a command string, terminated by <cr> character
    */
    void SendRawCommand(std::string cmd);

    /**
       	Read a line from the serial port, returns length of read string,
        zero on nothing to read
    */
    int ReadRawLine(std::string &cmd);
	
  protected:

    int CheckStatus();
    int speed_;
    portstream_fd COMstream_;
    unsigned int steps_per_mm_;
  };//end class

}// end namespace BIAS

#endif // __ISEL_LINEAR_CONTROL_hh__
