#ifndef __ISEL_LINEAR_CONTROL_TWO_AXIS_hh__
#define __ISEL_LINEAR_CONTROL_TWO_AXIS_hh__

#ifndef WIN32
#include <unistd.h>
#endif
#include <stdio.h>
#include <iostream>
#include <vector>
#include <Base/Debug/Debug.hh>

extern "C"
{
#include "../../BIAS/PanTilt/include/ptu.h"
#ifndef WIN32
#include "../../BIAS/PanTilt/include/linuxser.h"
#else
#include "../../BIAS/PanTilt/include/W32SERIA.H"
#endif
}

namespace BIAS{

  /**
     @class IselLinearControlTwoAxis
     This class controls the Isel iMC-S8 Linear Unit (two axis controll)
     @author tboer, 02/12
  */
  class BIASPanTilt_EXPORT IselLinearControlTwoAxis : public BIAS::Debug {
  public:

    /**
       standard constructor, steps_per_mm set to default values
    */
    IselLinearControlTwoAxis();

    /**
       constructor
       @param int the steps per mm of the first axis
       @param int the steps per mm of the second axis
    */
    IselLinearControlTwoAxis(const int steps_per_mm_x, const int steps_per_mm_y);

    /**
       standard destructor
    */
    ~IselLinearControlTwoAxis();

    /**
       Start Reference Tour on given axis (1-x, 2-y, or 3 x+y)
       @return bool indicating success of reset
    */
    int Reset(const int axis = 3);

    /**
       opens the serial port and initialises unit
       @param int the number of the serial port
    */
    int Init(const int nCOMportNumber);

    /**
       opens the given port and initialises unit
       @param char[] the name of the port
    */
    int Init(const char *COMportName);

    /**
       closes the comport stream
    */
    void Close();

    /**
       move millimeters in directions
       @param float the millimeters to move on the first axis
       @param float the millimeters to move on the second axis
    */
    int Move(const float millimetersX, const float millimetersY);

    /**
       sets the maximum/desired speed for all movements in millimeter per second
       @param float the speed for the first axis
       @param float the speed for the second axis
    */
    int SetSpeed(const float speedX, const float speedY);
    int SetAcceleration(const int accel);

    /**
       @returns the current speed
    */
    void GetSpeed(float &speedX, float &speedY);

    /**
       @returns the current Acceleration
    */
    void GetAcceleration(int &accel);

    /**
       @return the current position of the Unit
    */
    void GetCurrentPosition(float &millimetersX, float &millimetersY);

    int SetPosition(const float millimetersX, const float millimetersY, const bool bWaitComplete=true);

    /**
       Send a command string, terminated by <cr> character
    */
    void SendRawCommand(const std::string cmd);

    /**
        Read a line from the serial port, returns length of read string,
        zero on nothing to read
    */
    int ReadRawLine(std::string &cmd);

  protected:

    int CheckStatus();
    int steps_per_second_x_, steps_per_second_y_;
    portstream_fd COMstream_;
    int steps_per_mm_x_, steps_per_mm_y_;
  };//end class

}// end namespace BIAS

#endif // __ISEL_LINEAR_CONTROL_TWO_AXIS_hh__
