/*
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003-2009    (see file CONTACT for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef PANTILTAUTOCONTROL_HH_
#define PANTILTAUTOCONTROL_HH_

#include <pthread.h>
#include "PanTiltControlInterface.hh"

namespace BIAS{

/**
 * \class PanTiltAutoControl, derived from PanTiltControlInterface
 * \brief Automatic control of the PanTilt Unit from Directed Perception
 * \author fkellner
 */
class BIASPanTilt_EXPORT PanTiltAutoControl : public PanTiltControlInterface {
public:
	/** \brief constructor has nothing to do,
	 * super-constructor will startup communication */
	PanTiltAutoControl(std::string device, bool heavyDutyMode);
	/** \brief destructor moves to (0,0), then
	 * 	calls super-destructor to release ttyS0 */
	~PanTiltAutoControl();
	/** \brief start moving around */
	void StartControl();
	/** \brief stop/pause moving around*/
	void StopControl();
	/** \brief resume moving around*/
	void ResumeControl();
	
	/** \brief move to desired position */
	void StartMoveTo(const long int pan, const long int tilt);
	/** \brief stop move to desired position*/
	void StopMoveTo();
	/** \brief check if position is reached */
	inline bool Stopped() {
	  return !started_;
	}
	
private:
	// thread to control camera movement
	pthread_t control_thread;
	// actual movement function, ptr should hold a pointer to this classes ptu-object
	static void* threadFunc_pan_only(void *ptr);
	// actual movement function, ptr should hold a pointer to this classes ptu-object
	static void* threadFunc_pan_tilt(void *ptr);
	
	// actual movement to a desired position, ptr should hold a pointer to this classes ptu-object
	static void* threadFunc_move_to(void *ptr);
	
	// conditions for the thread
	static pthread_mutex_t condition_mutex;
	static pthread_cond_t  condition_cond;
	
	// thread control variables
	static bool exiting_;
	static bool waiting_;
	static bool started_;
  static bool bSpeedMode_;

	// some useful helper functions, should be in some more general class
	static int abs(int val);
	static int min(int val1, int val2);
	static int max(int val1, int val2);
	
	static long int des_pan;
	static long int des_tilt;

};

/**
 * \class PanTiltAutoControlFactory
 * \brief Factory to create a PanTiltAutoControl
 */
class BIASPanTilt_EXPORT PanTiltAutoControlFactory : public PanTiltControlFactory {
public:
	PanTiltAutoControl* Create(std::string device, bool heavyDutyMode) {
		return new PanTiltAutoControl(device, heavyDutyMode);
	}
};
}
#endif /*PANTILTAUTOCONTROL_HH_*/
