/*
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003-2009    (see file CONTACT for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <Base/Common/BIASpragma.hh>
#include <PanTilt/DPPanTiltControl.hh>
#ifdef WIN32
  #include <conio.h>
#else // WIN32
  #include <ncurses.h>
#endif // WIN32
#include <fstream>

#include "PanTiltManualControl.hh"

using namespace std;
using namespace BIAS;

/* contructor, just calls base constructor */
PanTiltManualControl::PanTiltManualControl(std::string device, bool heavyDutyMode):
		PanTiltControlInterface(device, heavyDutyMode)
{

}
/* starts the control */
void PanTiltManualControl::StartControl() {
	int min_pan = -2000;
	int max_pan = 2000;
	int min_tilt = -900;
	int max_tilt = 700;
	int pan_mod = 100;
	int tilt_mod = 100;

	PanTiltControlInterface::ptControl.SetMode(ABSOLUTEMODE);
	await_completion();
	PanTiltControlInterface::ptControl.SetSpeed(MAXSPEED/4,MAXSPEED/4);

	ofstream myfile;
	myfile.open("ptu_data.txt", ios::out);

	int key=-1;
	bool looping = true;
	bool move_pt;
	while(looping) {
		key = getch();
		move_pt = false;
		switch (key) {
		case 'a':
			cur_pan+=pan_mod;
			if (cur_pan > max_pan)
				cur_pan = max_pan;
			move_pt = true;
			break;
		case 'd':
			cur_pan-=pan_mod;
			if (cur_pan < min_pan)
				cur_pan = min_pan;
			move_pt = true;
			break;
		case 'w':
			cur_tilt+=tilt_mod;
			if (cur_tilt > max_tilt)
				cur_tilt = max_tilt;
			move_pt = true;
			break;
		case 's':
			cur_tilt-=tilt_mod;
			if (cur_tilt < min_tilt)
				cur_tilt = min_tilt;
			move_pt = true;
			break;
		case 'o':
			PanTiltControlInterface::ptControl.SetSpeed(MAXSPEED/4, MAXSPEED/4);
			for (int i=0;i<=10;i+=2) {
				int tilt = -500+(tilt_mod*i);
				key = getch();
				if (key == 'q')
					break;
				PanTiltControlInterface::ptControl.SetPosition(1800, tilt);
				key = getch();
				if (key == 'q')
					break;
				PanTiltControlInterface::ptControl.SetPosition(-1800, tilt);
			}
			PanTiltControlInterface::ptControl.SetSpeed(MAXSPEED, MAXSPEED);
			move_pt=true;
			break;
		case ' ':
			looping = false;
			myfile.close();
			break;
		}
		if (move_pt) {
			PanTiltControlInterface::ptControl.SetPosition(cur_pan,cur_tilt);
			await_completion();
			double p,t;
			PanTiltControlInterface::GetPositionDegrees(p,t);
			myfile << p << " ";
			myfile << t << endl;
#ifdef WIN32
			cout << "PAN: " << cur_pan << " TILT: " << cur_tilt << endl;
			cout << "PAN: " << p << " TILT: " << t << endl;
#else
			mvprintw(4,0, "PAN: %d   Tilt: %d           \n", cur_pan, cur_tilt);
			mvprintw(5,0, "PAN: %f   Tilt: %f           \n", p, t);
#endif 			
		}
	}
	PanTiltControlInterface::ptControl.SetPosition(0,0);
	cout << "exited from ncurses loop" << endl;
}
