/*
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003-2009    (see file CONTACT for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#ifndef PANTILT_SOFT_MOTION_HH
#define PANTILT_SOFT_MOTION_HH

namespace BIAS{

	/** \class PanTiltSoftMotion
	 *  \brief
	 *  \author fkellner
	 */
  class BIASPanTilt_EXPORT PanTiltSoftMotion {
public:
	PanTiltSoftMotion();
	~PanTiltSoftMotion();

	void setLimits(int _min_pan, int _max_pan, int _min_pan_speed, int _max_pan_speed, int _min_tilt_speed, int _max_tilt_speed);

	void assert_pan(int &pan);
	void assert_tilt(int &tilt);

	int getPanSpeed(int start_pan, int end_pan, int cur_pan, int damp);
	int getTiltSpeed(int start_pan, int end_pan, int start_tilt, int end_tilt, int cur_pan, int cur_tilt, int damp);

private:
	int calcSpeed(int start_val, int end_val, int cur_val, int damp);
	bool tilt_active;
	int min_pan, max_pan, min_tilt, max_tilt, min_pan_speed, min_tilt_speed, max_pan_speed, max_tilt_speed;
	int min(int val1, int val2) {
		return (val1 < val2) ? val1 : val2;
	}
	int max(int val1, int val2) {
		return (val1 < val2) ? val2 : val1;
	}
};

typedef PanTiltSoftMotion t_SoftMotion;
}
#endif //PANTILT_SOFT_MOTION_HH
