# 
# common settings for all BIAS ctest builds
# 
# ctest -S script
# used by Dart/CMake for nightly build setup.
# 
# For additional information see the howto on
# http://public.kitware.com/Wiki/CMake_Scripting_Of_CTest
# 
# or contact the author: 
#
# TODO:
# - determine compiler name and version automatically 
#
# The source tree has to exist!
#
# Jan Woetzel 2005/02 - 2006/01
#

# the module name that should be checked out 
SET (CTEST_MODULE_NAME     "BIAS")

#
# macro to extract commadn line args separated by ";" or ","
# e.g. for "blah.cmake,MODEL=Experimental;" call.
# 
# The variable has to exists before because call by reference is used.
# Example usage to extract from arg ,blah;FOO=bar;blubb;
#
#   IF (NOT FOO)
#     SET(FOO " ")
#   ENDIF (NOT FOO)
#   GET_ARG(CTEST_SCRIPT_ARG FOO)
#
#
# Jan Woetzel 11/2005
#
MACRO(GET_ARG SRC VAR)
  # MESSAGE("SRC=${SRC}")
  # MESSAGE("VAR=${VAR}")
  IF ( ${SRC} )
    IF ( ${VAR} )
      IF ( ${SRC} MATCHES ".*(${VAR}).*")
        # extract value after VAR= until next separator: 
        # blahVAR=foo;BAR=blubb; --> VAR=foo
        STRING(REGEX MATCH "${VAR}=[^;,]*" arg${VAR}  "${${SRC}}" )
        STRING(REGEX REPLACE  "(${VAR}=)"  ""  ${VAR} "${arg${VAR}}")  
        STRING(REGEX REPLACE  "([,;])"  ""  ${VAR} "${${VAR}}")
        # MESSAGE(" xxxxx DBG52  arg${VAR}=${arg${VAR}}  ${VAR}=${${VAR}} xxxxx")
	# ELSE ( ${SRC} MATCHES ".*(${VAR}).*")
	#  MESSAGE("NOT contained.")
      ENDIF ( ${SRC} MATCHES ".*(${VAR}).*")
    ENDIF ( ${VAR} )
  ENDIF ( ${SRC} ) 
ENDMACRO(GET_ARG SRC VAR)



# we need a default generator if none is given
IF    (NOT GENERATOR) 
  IF    (WIN32)
    SET(GENERATOR "NMake Makefiles")
  ENDIF (WIN32)
  IF    (UNIX)
    SET(GENERATOR "Unix Makefiles")
  ENDIF (UNIX)  
ENDIF (NOT GENERATOR) 

# we need a default MODEL if none is given
IF    (NOT MODEL) 
  SET(MODEL "Experimental")  
  #SET(MODEL "Experimental Start Configure Build Test Submit")
  #SET(MODEL "Experimental Start Update Configure Build Test Coverage Purify Submit")
ENDIF (NOT MODEL) 

# we need a build type if none is given
IF    (NOT CMAKE_BUILD_TYPE)
  SET(CMAKE_BUILD_TYPE "Debug")
ENDIF (NOT CMAKE_BUILD_TYPE)


# parse the command line args given to ctest, e.g.
# ctest -V _S blah.cmake,MODEL=Experimental;CMAKE_BUILD_TYPE=Debug;GENERATOR=VS2003
IF (CTEST_SCRIPT_ARG)
  # MESSAGE("DBG CTEST_SCRIPT_ARG=${CTEST_SCRIPT_ARG}") 
  GET_ARG(CTEST_SCRIPT_ARG MODEL)  
  GET_ARG(CTEST_SCRIPT_ARG CMAKE_BUILD_TYPE)
  GET_ARG(CTEST_SCRIPT_ARG GENERATOR)
ENDIF (CTEST_SCRIPT_ARG)

#post debug messages
MESSAGE("  DBG MODEL=${MODEL}")
MESSAGE("  DBG CMAKE_BUILD_TYPE=${CMAKE_BUILD_TYPE}")


# BIAS_DEBUG in bias_config defined in all except Release builds?
IF    (CMAKE_BUILD_TYPE MATCHES ".*[rR][eE][lL].*")
  # Release build 
  SET(CTEST_BIAS_DEBUG OFF)
ELSE  (CMAKE_BUILD_TYPE MATCHES ".*[rR][eE][lL].*")
  # default in define it 
  SET(CTEST_BIAS_DEBUG ON)
ENDIF (CMAKE_BUILD_TYPE MATCHES ".*[rR][eE][lL].*")


# the root dir for source, build and our additional CMakeModules 
SET (DASHBOARD_DIR "$ENV{DASHBOARD_DIR}" )
IF    (NOT DASHBOARD_DIR)
  ## default: use HOME env. variable if available  
  SET (HOME_TMP "$ENV{HOME}")
  IF   (HOME_TMP)
    SET(DASHBOARD_DIR "$ENV{HOME}/DartDashboards")
  ELSE (HOME_TMP)
    # no HOME var available: 
    IF   (WIN32)
      SET(DASHBOARD_DIR "$ENV{USERPROFILE}")
    ELSE (WIN32)
      MESSAGE(FATAL_ERROR "could not determine DASHBOARD_DIR. Please set environment variable.")
    ENDIF(WIN32)
  ENDIF(HOME_TMP)
ENDIF (NOT DASHBOARD_DIR)
#MESSAGE("  DBG DASHBOARD_DIR=${DASHBOARD_DIR}")


# user name
IF (WIN32)
  SET(USER "$ENV{USERNAME}")
ELSE(WIN32)
  SET(USER "$ENV{USER}")
ENDIF(WIN32)
# username OK? 
IF(NOT USER)
  # SET(USER "nightlybuild")
  SET(USER "unknownUser")  
  MESSAGE("XXXXXXXXXX Error: USER could not be determined. defaulting to USER=${USER} XXXXXXXXXX")
ENDIF(NOT USER)


# code src directory
SET(CTEST_SOURCE_DIRECTORY 
  "${DASHBOARD_DIR}/source/${CTEST_MODULE_ID}/${CTEST_MODULE_NAME}" )

# compilation directory 
SET(CTEST_BINARY_DIRECTORY 
  "${DASHBOARD_DIR}/build/${CTEST_MODULE_ID}/${CTEST_MODULE_NAME}" )


MESSAGE("  DBG CTEST_SOURCE_DIRECTORY =${CTEST_SOURCE_DIRECTORY}")
MESSAGE("  DBG CTEST_BINARY_DIRECTORY =${CTEST_BINARY_DIRECTORY}")


# our additional cmake modules and find scripts
IF    (NOT CMAKE_MODULE_PATH)
  SET (CMAKE_MODULE_PATH
    "${DASHBOARD_DIR}/source/CMakeModules")
ENDIF (NOT CMAKE_MODULE_PATH)
MESSAGE("  DBG CMAKE_MODULE_PATH=${CMAKE_MODULE_PATH}")



# which binaries to use for running the dashboard 
IF(WIN32)
  SET (CTEST_COMMAND       "$ENV{ProgramFiles(x86)}/CMake 2.8/bin/ctest.exe")
  SET (CTEST_CMAKE_COMMAND "$ENV{ProgramFiles(x86)}/CMake 2.8/bin/cmake.exe")
ELSE(WIN32)
  SET (CTEST_COMMAND       "/usr/bin/ctest")
  SET (CTEST_CMAKE_COMMAND "/usr/bin/cmake")
ENDIF(WIN32)


#
# ctest command to run
# Please mind escaping all backslash and quotes
SET(CTEST_COMMAND "${CTEST_COMMAND} -D ${MODEL}Start -D ${MODEL}Configure -D ${MODEL}Build -D ${MODEL}Test -D ${MODEL}Submit -V -A \"${CTEST_BINARY_DIRECTORY}/bias_config.h\\;${CTEST_SCRIPT_DIRECTORY}/${CTEST_SCRIPT_NAME}\"  ")



# ###################################################################
# Specify the build options, here
# You could possibly use cmake -D 
# with an initial cache file instead, too.
# ###################################################################

# ctests should wipe the binary tree before running
SET (CTEST_START_WITH_EMPTY_BINARY_DIRECTORY TRUE)

# we want more than 1024 chars output
#SET(CTEST_CUSTOM_MAXIMUM_FAILED_TEST_OUTPUT_SIZE "20000")
#SET(CTEST_CUSTOM_MAXIMUM_PASSED_TEST_OUTPUT_SIZE "20000")


# check 
IF (NOT GENERATOR)
  MESSAGE(FATAL_ERROR "  xxxxx ERROR: You must set GENERATOR before including biasCommon.cmake. xxxxx")  
ENDIF (NOT GENERATOR)

#
# select appropriate generator choosen by calling "specific" cmake ctest script
#
# catch short names:

# Linux/Unix (makefiles)
IF   (GENERATOR MATCHES ".*[gG][mM][aA][kK][eE].*")
  SET(GENERATOR "Unix Makefiles")
ENDIF(GENERATOR MATCHES ".*[gG][mM][aA][kK][eE].*")
IF   (GENERATOR MATCHES ".*[uU][nN][iI][xX].*")
  SET(GENERATOR "Unix Makefiles")
ENDIF(GENERATOR MATCHES ".*[uU][nN][iI][xX].*")
IF   (GENERATOR MATCHES ".*[lL][iI][nN][uU][xX].*")
  SET(GENERATOR "Unix Makefiles")
ENDIF(GENERATOR MATCHES ".*[lL][iI][nN][uU][xX].*")

# nmake
IF   (GENERATOR MATCHES ".*[nN][mM][aA][kK][eE].*")
  SET(GENERATOR "NMake Makefiles")
ENDIF(GENERATOR MATCHES ".*[nN][mM][aA][kK][eE].*")

# VS 7.1 .NET 2003 (vs71)
IF   (GENERATOR MATCHES ".*(2003).*")
  SET(GENERATOR "Visual Studio 7 .NET 2003")
ENDIF(GENERATOR MATCHES ".*(2003).*")
IF   (GENERATOR MATCHES ".*[vV][sS][7][1].*")
  SET(GENERATOR "Visual Studio 7 .NET 2003")
ENDIF(GENERATOR MATCHES ".*[vV][sS][7][1].*")

# VS 8 2005 (vs8)
IF   (GENERATOR MATCHES ".*(2005).*")
  SET(GENERATOR "Visual Studio 8 2005")
ENDIF(GENERATOR MATCHES ".*(2005).*")
IF   (GENERATOR MATCHES ".*[vV][sS][8].*")
  SET(GENERATOR "Visual Studio 8 2005")  
ENDIF(GENERATOR MATCHES ".*[vV][sS][8].*")

# VS 9 2008 Win64 (vs9) -- force 64 bit
IF   (GENERATOR MATCHES ".*(2008).*")
  SET(GENERATOR "Visual Studio 9 2008 Win64")
ENDIF(GENERATOR MATCHES ".*(2008).*")
IF   (GENERATOR MATCHES ".*[vV][sS][9].*")
  SET(GENERATOR "Visual Studio 9 2008 Win64")  
ENDIF(GENERATOR MATCHES ".*[vV][sS][9].*")



#
# Set the compilations args for the selected generator
#
MESSAGE("  DBG GENERATOR=${GENERATOR}")

IF   (GENERATOR MATCHES "(NMake Makefiles)")
  MESSAGE("biasCommon.cmake detected/uses  NMake Makefiles  GENERATOR")
  SET (BIAS_COMMON_CMAKE_MAKE_PROGRAM "nmake")
  SET (BIAS_COMMON_MAKECOMMAND        "nmake -i")
ENDIF(GENERATOR MATCHES "(NMake Makefiles)")

IF   (GENERATOR MATCHES "(Unix Makefiles)")
  MESSAGE("biasCommon.cmake detected/uses  Unix Makefiles  GENERATOR")  
  # -k to continue compilation on error 
  # do not use -j4 (or higher) because 
  # there seem to be spurious problems with dependency checks
  # However, we need it for icecc.
  # icecream does not report compile command and output 
  # directly after eachother
  SET (BIAS_COMMON_CMAKE_MAKE_PROGRAM "make")
  IF   ("$ENV{CXX}" MATCHES ice)
    SET (BIAS_COMMON_MAKECOMMAND  "make -k -j4")
    MESSAGE("  DBG icecc detected.Using parallel   BIAS_COMMON_MAKECOMMAND=${BIAS_COMMON_MAKECOMMAND}")
  ELSE ("$ENV{CXX}" MATCHES ice)
    SET (BIAS_COMMON_MAKECOMMAND  "make -k -j1")
    MESSAGE("  DBG no icecc. Using sequential BIAS_COMMON_MAKECOMMAND=${BIAS_COMMON_MAKECOMMAND}")
  ENDIF("$ENV{CXX}" MATCHES ice)
ENDIF(GENERATOR MATCHES "(Unix Makefiles)")

IF   (GENERATOR MATCHES "(Visual Studio 7 .NET 2003)")
  MESSAGE("biasCommon.cmake detected/uses  Visual Studio 7 .NET 2003  GENERATOR")
  FIND_PROGRAM(BIAS_COMMON_CMAKE_MAKE_PROGRAM 
    NAMES devenv.com devenv VCExpress 
    PATHS
    "[HKEY_LOCAL_MACHINE\\SOFTWARE\\Microsoft\\VisualStudio\\7.1;InstallDir]"
    "$ENV{ProgramFiles}/Microsoft Visual Studio .NET 2003/Common7/IDE"    
    DOC "VS71 devenv" 
    NO_DEFAULT_PATH NO_SYSTEM_ENVIRONMENT_PATH )
  SET(BIAS_COMMON_MAKECOMMAND 
    "${BIAS_COMMON_CMAKE_MAKE_PROGRAM} ${CTEST_MODULE_NAME}.sln /build ${CMAKE_BUILD_TYPE} /project ALL_BUILD" )
ENDIF(GENERATOR MATCHES "(Visual Studio 7 .NET 2003)")

IF   (GENERATOR MATCHES "(Visual Studio 8 2005)")
  MESSAGE("biasCommon.cmake detected/uses  Visual Studio 8 2005 GENERATOR")
  FIND_PROGRAM(BIAS_COMMON_CMAKE_MAKE_PROGRAM 
    NAMES devenv.com devenv VCExpress
    PATHS
    "[HKEY_LOCAL_MACHINE\\SOFTWARE\\Microsoft\\VisualStudio\\8.0;InstallDir]"
    "[HKEY_LOCAL_MACHINE\\SOFTWARE\\Microsoft\\VisualStudio\\8.0\\Setup\\VS;EnvironmentDirectory]"
    "[HKEY_LOCAL_MACHINE\\SOFTWARE\\Microsoft\\VisualStudio\\8.0\\Setup;Dbghelp_path]"
    "$ENV{ProgramFiles}/Microsoft Visual Studio 8/Common7/IDE"
    "$ENV{ProgramFiles}/Microsoft Visual Studio 8/Common7/IDE"
    "$ENV{ProgramFiles}/Microsoft Visual Studio8/Common7/IDE"
    "$ENV{ProgramFiles} (x86)/Microsoft Visual Studio 8/Common7/IDE"
    "$ENV{ProgramFiles} (x86)/Microsoft Visual Studio8/Common7/IDE"
    #"D:/Programme/Microsoft Visual Studio 8/Common7/IDE"
    DOC "VS8 devenv" 
    NO_DEFAULT_PATH NO_SYSTEM_ENVIRONMENT_PATH )  
  SET(BIAS_COMMON_MAKECOMMAND 
    "${BIAS_COMMON_CMAKE_MAKE_PROGRAM} ${CTEST_MODULE_NAME}.sln /build ${CMAKE_BUILD_TYPE} /project ALL_BUILD" )
ENDIF(GENERATOR MATCHES "(Visual Studio 8 2005)")

IF   (GENERATOR MATCHES "(Visual Studio 9 2008)")
  MESSAGE("biasCommon.cmake detected/uses  Visual Studio 9 2008 GENERATOR")
  FIND_PROGRAM(BIAS_COMMON_CMAKE_MAKE_PROGRAM 
    NAMES devenv.com devenv VCExpress
    PATHS
    "[HKEY_LOCAL_MACHINE\\SOFTWARE\\Microsoft\\VisualStudio\\9.0;InstallDir]"
    "[HKEY_LOCAL_MACHINE\\SOFTWARE\\Microsoft\\VisualStudio\\9.0\\Setup\\VS;EnvironmentDirectory]"
    "[HKEY_LOCAL_MACHINE\\SOFTWARE\\Microsoft\\VisualStudio\\9.0\\Setup;Dbghelp_path]"
    "$ENV{ProgramFiles}/Microsoft Visual Studio 9/Common7/IDE"
    "$ENV{ProgramFiles}/Microsoft Visual Studio 9/Common7/IDE"
    "$ENV{ProgramFiles}/Microsoft Visual Studio9/Common7/IDE"
    "$ENV{ProgramFiles} (x86)/Microsoft Visual Studio 9/Common7/IDE"
    "$ENV{ProgramFiles} (x86)/Microsoft Visual Studio9/Common7/IDE"
    #"D:/Programme/Microsoft Visual Studio 8/Common7/IDE"
    DOC "VS9 devenv" 
    NO_DEFAULT_PATH NO_SYSTEM_ENVIRONMENT_PATH )  
  SET(BIAS_COMMON_MAKECOMMAND 
    "${BIAS_COMMON_CMAKE_MAKE_PROGRAM} ${CTEST_MODULE_NAME}.sln /build ${CMAKE_BUILD_TYPE} /project ALL_BUILD" )
ENDIF(GENERATOR MATCHES "(Visual Studio 9 2008)")

MESSAGE("  DBG BIAS_COMMON_CMAKE_MAKE_PROGRAM ${BIAS_COMMON_CMAKE_MAKE_PROGRAM}")
MESSAGE("  DBG BIAS_COMMON_MAKECOMMAND        ${BIAS_COMMON_MAKECOMMAND}")



#
# check arg for empty
#
IF    (NOT IDENTIFIER)
  SET(IDENTIFIER "_")
ENDIF (NOT IDENTIFIER)


IF   (WIN32)
  IF (MINGW)
    SET(CTEST_MODULE_OS "mingw")
  ELSE(MINGW)
    IF(CMAKE_SIZEOF_VOID_P EQUAL 4)
      SET(CTEST_MODULE_OS "win32")
    ELSE(CMAKE_SIZEOF_VOID_P EQUAL 4)
      SET(CTEST_MODULE_OS "win64")
    ENDIF(CMAKE_SIZEOF_VOID_P EQUAL 4)
  ENDIF(MINGW)
ELSE(WIN32)
  IF (UNIX)
    SET(CTEST_MODULE_OS "unix")
    #SET(CTEST_MODULE_OS "${CMAKE_SYSTEM_NAME}")
  ENDIF (UNIX)
ENDIF(WIN32)


#
# is this a Continuous build?
#
IF (MODEL MATCHES "(Continuous)")
  # parameters for Continuous:
  # how long shall the lopp run (minutes, e.g. 1080 for 18 hours)
  SET (CTEST_CONTINUOUS_DURATION 1080)
  
  # min. pause between builds
  SET (CTEST_CONTINUOUS_MINIMUM_INTERVAL 5)
  
  
  # testing "MIP depends on BIAS" settings:
  IF   (NOT WIN32)
    SET (CTEST_CONTINUOUS_DURATION 0)
    SET (CTEST_CONTINUOUS_MINIMUM_INTERVAL 0)
  ENDIF(NOT WIN32)
  
  
  # clean just on first build (to speed up build)?
  #SET (CTEST_START_WITH_EMPTY_BINARY_DIRECTORY_ONCE 1)
  SET (CTEST_START_WITH_EMPTY_BINARY_DIRECTORY_ONCE 0)
  
ENDIF (MODEL MATCHES "(Continuous)")


# store shared or static in buildname: 
IF(BIAS_CTEST_COMMON_OPTIONSCACHE MATCHES ".*BUILD_SHARED_LIBS:BOOL=OFF.*")
  SET (LIBS_TYPE "static_libs")
ELSE(BIAS_CTEST_COMMON_OPTIONSCACHE MATCHES ".*BUILD_SHARED_LIBS:BOOL=OFF.*")
  #default for shared/static libs not uniquely defined!
  # ==> double check is needed when not explicitly turned off
  SET (LIBS_TYPE "shared_libs")
ENDIF(BIAS_CTEST_COMMON_OPTIONSCACHE MATCHES ".*BUILD_SHARED_LIBS:BOOL=OFF.*")

#  identifier for the build, a short name. You can append additional short info here
#  e.g.:           unix/win322        BIAS                1001             Debug/Release     gcc334/nmake71   static_libs  <specific>  user
SET(BUILDNAME "${CTEST_MODULE_ID}_${CTEST_MODULE_NAME}_${CTEST_MODULE_OS}_${CMAKE_BUILD_TYPE}_${GENERATOR}_${LIBS_TYPE}_${USER}_${IDENTIFIER}")
# replace spaces: XSL does not like spaces within paths/filenames: 
STRING(REGEX REPLACE "[ ]"  "_"  BUILDNAME "${BUILDNAME}" )
MESSAGE("  DBG BUILDNAME=${BUILDNAME}")


#
# this is the initial cache to use for the binary tree.
# Be careful to escape any quotes inside of this string
#
SET (CTEST_INITIAL_CACHE "
CMAKE_MAKE_PROGRAM:FILEPATH=${BIAS_COMMON_CMAKE_MAKE_PROGRAM}
MAKECOMMAND:STRING=${BIAS_COMMON_MAKECOMMAND}
CMAKE_GENERATOR:INTERNAL=${GENERATOR}
CVSCOMMAND:FILEPATH=${CTEST_CVS_COMMAND}
// identifier for the build, a short name with additional info
BUILDNAME:STRING=${BUILDNAME}
// build options handed down to cmake
CMAKE_BUILD_TYPE:STRING=${CMAKE_BUILD_TYPE}
CMAKE_VERBOSE_MAKEFILE:BOOL=ON
BUILD_TESTS:BOOL=ON
BUILD_TESTING:BOOL=ON
BUILD_DOXYGEN:BOOL=OFF
//
// our own project specific build options
BIAS_DEBUG:BOOL=${CTEST_BIAS_DEBUG}
// user specified additionals: 
${BIAS_CTEST_COMMON_OPTIONSCACHE}
")


# set any OS environment variables here 
# we can avoid inheritance from user settings with this approach
# multiple settings are allowed, eg. PATH, CC, CXX etc.
IF    (NOT CTEST_ENVIRONMENT)
  SET (CTEST_ENVIRONMENT 
    "CMAKE_MODULE_PATH=${CMAKE_MODULE_PATH}"
    )
    #"PATH=$ENV{PATH};${LIBRARY_OUTPUT_PATH}"
ENDIF (NOT CTEST_ENVIRONMENT)


