/// Running this test is used for debugging 
/// Dart/ctest runtime paths, environment, ... 
/// Displays the true build configuration during compilation.
/// @author Jan Woetzel

#include <bias_config.h>
#include <iostream>

using namespace std;


/**
   @file testDefines.cpp
   @brief Test the defines used in BIAS
   @ingroup g_tests
   @author jwoetzel
*/

/// prints the compile-time definition to check build configurations
void print_defines(int argc, char ** argv){

#ifdef __DATE__
  cout<<"+ __DATE__ \tdefined as "<<__DATE__<<endl;
#else
  cout<<"- __DATE__ \tis NOT defined"<<endl;
#endif

#ifdef __TIME__
  cout<<"+ __TIME__ \tdefined as "<<__TIME__<<endl;
#else
  cout<<"- __TIME__ \tis NOT defined"<<endl;
#endif

#ifdef __TIMESTAMP__
  cout<<"+ __TIMESTAMP__ \tdefined as "<<__TIMESTAMP__<<endl;
#else
  cout<<"- __TIMESTAMP__ \tis NOT defined"<<endl;
#endif

#ifdef COMPILE_DEBUG
  cout<<"+ COMPILE_DEBUG \tdefined (compile with debug info)"<<endl;
#else 
  cout<<"- COMPILE_DEBUG \tis NOT defined (Release build?)"<<endl;
#endif

#ifdef COMPILE_NDEBUG
  cout<<"+ COMPILE_NDEBUG \tdefined (ignore assert)"<<endl;
#else
  cout<<"- COMPILE_NDEBUG \tis NOT defined (handle assertions)"<<endl;
#endif

#ifdef BIAS_DEBUG_ONLY_IN_DEBUG
  cout<<"+ BIAS_DEBUG_ONLY_IN_DEBUG \tdefined"<<endl;
#else
  cout<<"- BIAS_DEBUG_ONLY_IN_DEBUG \tis NOT defined"<<endl;
#endif

#ifdef BIAS_DEBUG
  cout<<"+ BIAS_DEBUG \tdefined"<<endl;
#else
  cout<<"- BIAS_DEBUG \tis NOT defined"<<endl;
#endif

#ifdef CMAKE_INTDIR
  cout<<"+ CMAKE_INTDIR \tdefined as "<<CMAKE_INTDIR<<endl;
#else
  cout<<"- CMAKE_INTDIR \tis NOT defined"<<endl;
#endif

#ifdef WIN32
  cout<<"+ WIN32 \tdefined"<<endl;
#else
  cout<<"- WIN32 \tis NOT defined"<<endl;
#endif

#ifdef _WIN32
  cout<<"+ _WIN32 \tdefined"<<endl;
#else
  cout<<"- _WIN32 \tis NOT defined"<<endl;
#endif

#ifdef _WIN64
  cout<<"+ _WIN64 \tdefined"<<endl;
#else
  cout<<"- _WIN64 \tis NOT defined"<<endl;
#endif

#ifdef WINDOWS
  cout<<"+ WINDOWS \tdefined"<<endl;
#else
  cout<<"- WINDOWS \tis NOT defined"<<endl;
#endif

#ifdef _WINDOWS
  cout<<"+ _WINDOWS \tdefined"<<endl;
#else
  cout<<"- _WINDOWS \tis NOT defined"<<endl;
#endif

  //#ifdef LINUX
  //  cout<<"+ LINUX \tdefined"<<endl;
  //#else
  //  cout<<"- LINUX \tis NOT defined"<<endl;
  //#endif
  //
  //#ifdef UNIX
  //  cout<<"+ UNIX \tdefined"<<endl;
  //#else
  //  cout<<"- UNIX \tis NOT defined"<<endl;
  //#endif

#ifdef unix
  cout<<"+ unix \tdefined"<<endl;
#else
  cout<<"- unix \tis NOT defined"<<endl;
#endif

#ifdef __unix__
  cout<<"+ __unix__ \tdefined"<<endl;
#else
  cout<<"- __unix__ \tis NOT defined"<<endl;
#endif

#ifdef _MSC_VER
  cout<<"+ _MSC_VER \tdefined as "<<_MSC_VER<<endl;
#else
  cout<<"- _MSC_VER \tis NOT defined"<<endl;
#endif

#ifdef __CLR_VER
  cout<<"+ __CLR_VER \tdefined as "<<__CLR_VER<<endl;
#else
  cout<<"- __CLR_VER \tis NOT defined"<<endl;
#endif

#ifdef _MT
  cout<<"+ _MT \tdefined (multithread safe)"<<endl;
#else
  cout<<"- _MT \tis NOT defined"<<endl;
#endif

#ifdef _DLL
  cout<<"+ _DLL \tdefined (/MD or /MDd mutithreaded dll MSVCRT usage)"<<endl;
#else
  cout<<"- _DLL \tis NOT defined"<<endl;
#endif

#ifdef _WINDLL
  cout<<"+ _WINDLL \tdefined"<<endl;
#else
  cout<<"- _WINDLL \tis NOT defined"<<endl;
#endif

#ifdef _STATIC_CPPLIB
  cout<<"+ _STATIC_CPPLIB \tdefined "<<endl;
#else
  cout<<"- _STATIC_CPPLIB \tis NOT defined"<<endl;
#endif 

#ifdef _CPPLIB_VER
  cout<<"+ _CPPLIB_VER \tdefined as "<<_CPPLIB_VER<<endl;
#else
  cout<<"- _CPPLIB_VER \tis NOT defined"<<endl;
#endif

#ifdef UNICODE
  cout<<"+ UNICODE \tdefined "<<endl;
#else
  cout<<"- UNICODE \tis NOT defined"<<endl;
#endif 

#ifdef __cplusplus
  cout<<"+ __cplusplus \tdefined "<<endl;
#else
  cout<<"- __cplusplus \tis NOT defined"<<endl;
#endif

#ifdef __cplusplus_cli
  cout<<"+ __cplusplus_cli \tdefined as "<<__cplusplus_cli<<endl;
#else
  cout<<"- __cplusplus_cli \tis NOT defined"<<endl;
#endif

#ifdef __i386__
  cout<<"+ __i386__ \tdefined"<<endl;
#else
  cout<<"- __i386__ \tis NOT defined"<<endl;
#endif 

#ifdef _M_IX86
  cout<<"+ _M_IX86 \tdefined as "<<_M_IX86<<endl;
#else
  cout<<"- _M_IX86 \tis NOT defined"<<endl;
#endif

#ifdef _M_IA64
  cout<<"+ _M_IA64 \tdefined"<<endl;
#else
  cout<<"- _M_IA64 \tis NOT defined"<<endl;
#endif


#ifdef _M_IX86_FP
  cout<<"+ _M_IX86_FP \tdefined as "<<_M_IX86_FP;
#if _M_IX86_FP==1
  cout<<" (/arch:SSE) ";
#endif
#if _M_IX86_FP==2
  cout<<" (/arch:SSE2) ";
#endif
#if _M_IX86_FP==3
  cout<<" (/arch:SSE3) ";
#endif
  cout<<endl;
#else
  cout<<"- _M_IX86_FP \tis NOT defined"<<endl;
#endif

#ifdef _OPENMP
  cout<<"+ _OPENMP \tdefined"<<endl;
#else
  cout<<"- _OPENMP \tis NOT defined"<<endl;
#endif

#ifdef _MSC_EXTENSIONS
  cout<<"+ _MSC_EXTENSIONS \tdefined"<<endl;
#else
  cout<<"- _MSC_EXTENSIONS \tis NOT defined"<<endl;
#endif

#ifdef _CPPRTTI
  cout<<"+ _CPPRTTI \tdefined"<<endl;
#else
  cout<<"- _CPPRTTI \tis NOT defined"<<endl;
#endif

#ifdef _CPPUNWIND
  cout<<"+ _CPPUNWIND \tdefined"<<endl;
#else
  cout<<"- _CPPUNWIND \tis NOT defined"<<endl;
#endif

#ifdef _MANAGED
  cout<<"+ _MANAGED \tdefined"<<endl;
#else
  cout<<"- _MANAGED \tis NOT defined"<<endl;
#endif

#ifdef __GNUG__
  cout<<"+ __GNUG__ \tdefined as "<<__GNUG__<<endl;
#else
  cout<<"- __GNUG__ \tis NOT defined"<<endl;
#endif

#ifdef __GNUC__
  cout<<"+ __GNUC__ \tdefined as "<<__GNUC__<<endl;
#else
  cout<<"- __GNUC__ \tis NOT defined"<<endl;
#endif

#ifdef __GNUC_MINOR__
  cout<<"+ __GNUC_MINOR__ \tdefined as "<<__GNUC_MINOR__<<endl;
#else
  cout<<"- __GNUC_MINOR__ \tis NOT defined"<<endl;
#endif

#ifdef __GNUC_PATCHLEVEL__
  cout<<"+ __GNUC_PATCHLEVEL__ \tdefined as "<<__GNUC_PATCHLEVEL__<<endl;
#else
  cout<<"- __GNUC_PATCHLEVEL__ \tis NOT defined"<<endl;
#endif

#ifdef __VERSION__
  cout<<"+ __VERSION__ \tdefined as "<<__VERSION__<<endl;
#else
  cout<<"- __VERSION__ \tis NOT defined"<<endl;
#endif

#ifdef __OPTIMIZE__
  cout<<"+ __OPTIMIZE__ \tdefined "<<endl;
#else
  cout<<"- __OPTIMIZE__ \tis NOT defined"<<endl;
#endif

#ifdef __OPTIMIZE_SIZE__
  cout<<"+ __OPTIMIZE_SIZE__ \tdefined "<<endl;
#else
  cout<<"- __OPTIMIZE_SIZE__ \tis NOT defined"<<endl;
#endif

#ifdef __NO_INLINE__
  cout<<"+ __NO_INLINE__ \tdefined "<<endl;
#else
  cout<<"- __NO_INLINE__ \tis NOT defined"<<endl;
#endif

#ifdef __STDC__
  cout<<"+ __STDC__ \tdefined "<<endl;
#else
  cout<<"- __STDC__ \tis NOT defined"<<endl;
#endif

#ifdef __STDC_VERSION__
  cout<<"+ __STDC_VERSION__ \tdefined as "<<__STDC_VERSION__<<endl;
#else
  cout<<"- __STDC_VERSION__ \tis NOT defined"<<endl;
#endif

#ifdef __STRICT_ANSI__
  cout<<"+ __STRICT_ANSI__ \tdefined "<<endl;
#else
  cout<<"- __STRICT_ANSI__ \tis NOT defined"<<endl;
#endif

#ifdef __INTEL_COMPILER
  cout<<"+ __INTEL_COMPILER \tdefined "<<endl;
#else
  cout<<"- __INTEL_COMPILER \tis NOT defined"<<endl;
#endif 

  // nvcc test/cuda  compare to __host__/__device__jw
#ifdef __CUDACC__
  cout<<"+ __CUDACC__ \tdefined "<<endl;
#else
  cout<<"- __CUDACC__ \tis NOT defined"<<endl;
#endif 

#ifdef __DEVICE_EMULATION__
  cout<<"+ __DEVICE_EMULATION__ \tdefined "<<endl;
#else
  cout<<"- __DEVICE_EMULATION__ \tis NOT defined"<<endl;
#endif 

  //#ifdef WXUSINGDLL
  //  cout<<"+ WXUSINGDLL \tdefined "<<endl;
  //#else
  //  cout<<"- WXUSINGDLL \tis NOT defined"<<endl;
  //#endif 

#ifdef __FUNCSIG__
  cout<<"+ __FUNCSIG__ (function signature) defined as "<<endl
    <<"\t"<<__FUNCSIG__<<endl;
#else
  cout<<"- __FUNCDNAME__ \tis NOT defined"<<endl;
#endif
#ifdef __FUNCDNAME__
  cout<<"+ __FUNCDNAME__ (decorated function name) defined as "<<endl
    <<"\t"<<__FUNCDNAME__<<endl;
#else
  cout<<"- __FUNCDNAME__ \tis NOT defined"<<endl;
#endif
}


// --- entry point ---
int main(int argc, char ** argv)
{
  if (argc>0)
    cout<<"started "<<argv[0]<<endl;

  //
  // display compilation defines
  //
  print_defines(argc, argv);

  // ctest needs this for >1024 chars output: 
  cout<<endl<<"ctest needs: CTEST_FULL_OUTPUT"<<endl;
  return 0;
}

