/*
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003-2009    (see file CONTACT for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "biasGLviewerWx.hh"
#include <Gui/StringConv.hh>
#include <Utils/IOUtils.hh>
#include <img/miplogochars32x32.xpm>

using namespace std;
using namespace BIAS;

#define GRP_IMAGE 1
#define GRP_GENERAL 2
#define GRP_SEGMENTATION 3
#define GRP_BACKGROUND 4
#define GRP_MODELS 5
#define GRP_VIEW 6

BEGIN_EVENT_TABLE( BIASOpenGLViewerFrame, wxFrame)
EVT_CLOSE ( BIASOpenGLViewerFrame::OnQuit)
EVT_MENU  ( wxID_EXIT,         BIASOpenGLViewerFrame::OnExit)
EVT_MENU  ( wxID_ABOUT,        BIASOpenGLViewerFrame::OnAbout)
EVT_MENU  ( ID_SAVE_PARAMETER, BIASOpenGLViewerFrame::OnSaveParameters)
EVT_MENU  ( ID_SHOW_PARAMETER, BIASOpenGLViewerFrame::OnShowParameters)
EVT_MENU  ( ID_FULLSCREEN,     BIASOpenGLViewerFrame::OnFullscreen)

EVT_MENU  ( ID_SOLID,          BIASOpenGLViewerFrame::OnViewSolid)
EVT_MENU  ( ID_LINES,          BIASOpenGLViewerFrame::OnViewLines)
EVT_MENU  ( ID_POINTS,         BIASOpenGLViewerFrame::OnViewPoints)

EVT_BUTTON( ID_LOADMODEL,      BIASOpenGLViewerFrame::OnAddModel )
EVT_BUTTON( ID_DELETEMODELS,   BIASOpenGLViewerFrame::OnDeleteModels )
EVT_BUTTON( ID_FULLSCREEN,     BIASOpenGLViewerFrame::OnFullscreen)
EVT_BUTTON( ID_SCREENSHOT,     BIASOpenGLViewerFrame::OnScreenshot)

EVT_TIMER ( ID_TIMER,          BIASOpenGLViewerFrame::OnTimer)
EVT_CHECKBOX(ID_CHECK_MEASURE, BIASOpenGLViewerFrame::OnMeasure)
END_EVENT_TABLE()


BIASOpenGLViewerFrame::
BIASOpenGLViewerFrame(wxWindow* parent, wxWindowID id,
		const wxString& title, const wxPoint& pos, const wxSize& size, long style) :
	wxFrame(parent, id, title, pos, size, style)
{
  SetIcon(wxIcon(miplogochars32x32_xpm));

  this->SetSizeHints(wxDefaultSize, wxDefaultSize);

	toolBar_ = this->CreateToolBar(wxTB_DOCKABLE | wxTB_HORIZONTAL, wxID_ANY);
	
	fullscreenButton_ = new wxButton(toolBar_, ID_FULLSCREEN, wxT("Fullscreen"),
			wxDefaultPosition, wxDefaultSize, 0);
	toolBar_->AddControl(fullscreenButton_);
	
	loadModelButton_ = new wxButton(toolBar_, ID_LOADMODEL, wxT("Load Model"),
			wxDefaultPosition, wxDefaultSize, 0);
	toolBar_->AddControl(loadModelButton_);

	deleteModelsButton_ = new wxButton(toolBar_, ID_DELETEMODELS, wxT("Delete Models"),
				wxDefaultPosition, wxDefaultSize, 0);
	toolBar_->AddControl(deleteModelsButton_);

	screenshotButton_ = new wxButton(toolBar_, ID_SCREENSHOT, wxT("Screenshot"),
					wxDefaultPosition, wxDefaultSize, 0);
	toolBar_->AddControl(screenshotButton_);

	
	measureCheckbox_ = new wxCheckBox(toolBar_,ID_CHECK_MEASURE,wxT("Measure")); 
	measureCheckbox_->SetValue(false);
	toolBar_->AddControl(measureCheckbox_);
	toolBar_->Realize();
	//menu
	//file
	menubar_ = new wxMenuBar(0);
	fileMenu_ = new wxMenu();
	fileMenu_->Append(ID_SAVE_PARAMETER, wxT("&Save Parameters") );
	menuItemShowParameter_ = new wxMenuItem(fileMenu_, ID_SHOW_PARAMETER, wxString(
				wxT("Show &Parameters")), wxEmptyString, wxITEM_CHECK);
	fileMenu_->Append(menuItemShowParameter_);	
	fileMenu_->Append( wxID_EXIT, wxT("E&xit") );	  
	menubar_->Append(fileMenu_, wxT("&File"));
	//view
	viewMenu_ = new wxMenu();
	menuItemFullscreen_ = new wxMenuItem(fileMenu_, ID_FULLSCREEN, wxString(
					wxT("&Fullscreen")), wxEmptyString, wxITEM_CHECK);
	viewMenu_->Append(menuItemFullscreen_);
	menuItemSolid_ = new wxMenuItem(fileMenu_, ID_SOLID, wxString(
			wxT("&Solid")), wxEmptyString, wxITEM_CHECK);
	viewMenu_->Append(menuItemSolid_);
	menuItemLines_ = new wxMenuItem(fileMenu_, ID_LINES, wxString(
			wxT("&Lines")), wxEmptyString, wxITEM_CHECK);
	viewMenu_->Append(menuItemLines_);
	menuItemPoints_ = new wxMenuItem(fileMenu_, ID_POINTS, wxString(
			wxT("&Points")), wxEmptyString, wxITEM_CHECK);
	viewMenu_->Append(menuItemPoints_);

	menubar_->Append(viewMenu_, wxT("&View"));
	//help
	helpMenu_ = new wxMenu();
	helpMenu_->Append(wxID_ABOUT, wxT("&About...") );
	menubar_->Append(helpMenu_, wxT("&Help"));

	this->SetMenuBar(menubar_);
	
	const int numFields = 2;
	CreateStatusBar(numFields);



	//*********************  ContextWX  ********************************************************
	const BIAS::Vector4<float> cc(65/255.0, 139/255.0, 212/255.0, 0);
	int ContextAttribs[] = {  WX_GL_RGBA,
			WX_GL_LEVEL, 0,
			WX_GL_MIN_RED, 8,
			WX_GL_MIN_GREEN, 8,
			WX_GL_MIN_BLUE, 8,
			WX_GL_MIN_ALPHA, 8,
			WX_GL_DEPTH_SIZE, 24,
			WX_GL_STENCIL_SIZE, 8,
			WX_GL_DOUBLEBUFFER, 0 };
	contextWX_ = new ContextWX( this, wxID_ANY,wxDefaultPosition, wxSize(640, 480), 0,wxT("GLCanvas"),ContextAttribs);
	contextWX_->SetBlending(false);
	contextWX_->SetBackfaceCulling(false);

	// GLProjectionParametersBase
	virtualCamera_= new BIAS::GLProjectionParametersPerspective();//if (virtualCamera_ == NULL) 
	(dynamic_cast<BIAS::GLProjectionParametersPerspective*> (virtualCamera_))->SetSimplePerspectiveCam(65);
	Vector3<double> C(0,0,-10); Quaternion<double> Q(0,0,0,1);
	(dynamic_cast<BIAS::GLProjectionParametersPerspective*> (virtualCamera_))->SetExtrinsics(C,Q);
	virtualCamera_->SetAutoReshapeBehaviour(BIAS::GLProjectionParametersInterface::AutoRescaleParams);

	contextWX_->SetGLProjectionParametersInterface(virtualCamera_);
	contextWX_->SetControl(&Control_);
	contextWX_->SetAutoClipping(true);
	contextWX_->SetAutoClipMinZNear(0.1);
	contextWX_->SetClearColor(cc);		
	contextWX_->InitializationComplete();
	//********************************************************************************

	InitParameters_();	
	InitModels();	
	this->Layout();
	
	timer_.SetOwner(this, ID_TIMER);
	timer_.Start(5,true);
}


BIASOpenGLViewerFrame::
~BIASOpenGLViewerFrame() {
 
}


void BIASOpenGLViewerFrame::
OnMeasure(wxCommandEvent& event){
	Control_.SetDistanceMeasure(event.IsChecked());
}

void BIASOpenGLViewerFrame::
OnTimer(wxTimerEvent& /*event*/) {
  timer_.Stop();
  contextWX_->Refresh();
  timer_.Start(5, true);
}

int BIASOpenGLViewerFrame::
InitModels() {
	
	 screenShotControl_.Init(contextWX_, *screenShotName_, "png", "", true);
	
	// add a light source
	lightScene_.SetLightAsHeadLight(true);
	lightScene_.SetDrawLightPosition(false);
	lightScene_.SetAmbient(Vector4<double> (0.3, 0.3, 0.3, 1.0));
	lightScene_.SetDiffuse(Vector4<double> (1.0, 1.0, 1.0, 1.0));
	lightScene_.SetSpecular(Vector4<double> (0.1, 0.1, 0.1, 1.0));
  
	contextWX_->AppendScene(&lightScene_);
#ifdef BIAS_HAVE_OPENSCENEGRAPH
	scene_.Init();	
	contextWX_->AppendScene(&scene_);
#endif
	
#ifdef BIAS_HAVE_GLUT	
  contextWX_->AppendScene(&sceneGlut_);
#endif
	contextWX_->AppendScene(&Control_);
	contextWX_->AppendAdditionalListener(&screenShotControl_);
	return 0;
}

int BIASOpenGLViewerFrame::
ResetSceneCenter(Vector3<double>* C,Quaternion<double>* Q) {
	if(*useSceneCenter_) {
		BIAS::Vector3<double> CameraCenter, radius, SceneCenter;
		Quaternion<double> CameraQuaternion;
		if (!contextWX_->GetSceneCenter(SceneCenter, radius))
			BIASERR("could not retrieve bounding box from scene");
		// move 10% outwards
		if(C != NULL && Q != NULL){
			CameraCenter = *C;
			CameraQuaternion = *Q;
			virtualCamera_->GetMyselfAsProjectionParameterBase()->SetC(CameraCenter);
			virtualCamera_->GetMyselfAsProjectionParameterBase()->SetQ(CameraQuaternion);						
			Control_.SetPointOfInterest(SceneCenter);
			return 0;
		}
		else
			CameraCenter = SceneCenter + 1.1 * radius;
		cout<<"using scene center "<<SceneCenter<<" and new camera center "
		<<CameraCenter <<endl;
		// set point of interest only if not equal to camera center
		if ((CameraCenter-SceneCenter).NormL2()>0.001) {
			virtualCamera_->GetMyselfAsProjectionParameterBase()->SetC(CameraCenter);
			Control_.SetPointOfInterest(SceneCenter);
		} else {
			cerr<<"could not set poi, empty scene ?"<<endl;
		}
		return 0;
	}
	else return 1;
}
/****************************************************
  @brief


 *****************************************************/
void BIASOpenGLViewerFrame::
OnSaveParameters(wxCommandEvent& event){

 wxString fileName;
 wxFileDialog dialog(this,wxT("Save Parameters"),wxT(""),wxT(""),
                     wxT("MIP Parameter files (*.param)|*.param"),
                     wxFD_SAVE|wxFD_OVERWRITE_PROMPT);
 int ret = dialog.ShowModal();
 if(ret == wxID_OK){
   fileName = dialog.GetPath();
   WriteParameter(WxToAscii(fileName));
 }
}

/****************************************************
  @brief


 *****************************************************/
void BIASOpenGLViewerFrame::
OnShowParameters(wxCommandEvent& event){
	if(paramFrame_ != NULL){
		if(event.IsChecked() ){
			paramFrame_->Show(true);
		}
		else{
			paramFrame_->Show(false);
		}

	}
}

/****************************************************
  @brief


 *****************************************************/
void BIASOpenGLViewerFrame::
OnAbout(wxCommandEvent& WXUNUSED(event))
{
	wxString desc;
	desc << wxT("The BIAS OpenGL Viewer, part of the\n\n");
	desc << wxT(" BIAS library (Basic ImageAlgorithmS)\n");
	stringstream text;
	text << "built on " << __DATE__ << " " << __TIME__ << "\n"	
	<< "GUI using: " << WxToAscii(wxVERSION_STRING)<< "\n";
	desc<<AsciiToWx(text.str());

	wxString lic = wxT("BIAS is free software; you can redistribute it and/or modify\n \
			it under the terms of the GNU Lesser General Public Licence as published by\n \
			the Free Software Foundation; either version 2.1 of the Licence, or\n \
			(at your option) any later version.\n \
			\n \
			BIAS is distributed in the hope that it will be useful,\n \
			but WITHOUT ANY WARRANTY; without even the implied warranty of\n \
			MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n \
			GNU Lesser General Public Licence for more details.\n \
			\n \
			You should have received a copy of the GNU Lesser General Public Licence\n \
			along with BIAS; if not, write to the Free Software\n \
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA\n");

	wxAboutDialogInfo aboutdlg;
	aboutdlg.SetDescription( desc );
	aboutdlg.SetLicence( lic );
	aboutdlg.SetCopyright( wxT("Copyright (C) 2003-2010\n \
			Multimediale Systeme der Informationsverarbeitung\n \
			Institut fuer Informatik\n \
	Christian-Albrechts-Universitaet Kiel") );
	aboutdlg.SetWebSite( wxT("http://www.mip.informatik.uni-kiel.de/") );

	wxAboutBox( aboutdlg );
}


/****************************************************
  @brief


 *****************************************************/
void BIASOpenGLViewerFrame::
OnDeleteModels(wxCommandEvent& event){
#ifdef BIAS_HAVE_OPENSCENEGRAPH
	scene_.Clear();
#endif
}
/****************************************************
  @brief


 *****************************************************/
void BIASOpenGLViewerFrame::
SetParameterFile(int argc, char** argv,std::string& paramFile) {
	firstUnknownParam_ =0;
	bool ret = IOUtils::ParseCommandLineEvalHelp(*this,argc, argv,firstUnknownParam_);
	if(!ret) BIASWARN("Parsing command line arguments failed. Please check them!");
	
	wxSize panelSize(400,600);
	paramFrame_ = NULL;
	paramFrame_ = GetFrame(this);
	if(paramFrame_ ==NULL ){
		BIASERR("Param Panel can not be created, exiting!");
		return;
	}
	
#ifdef BIAS_HAVE_OPENSCENEGRAPH
	if(argc - firstUnknownParam_ >= 1 && firstUnknownParam_ > 0) {
		for(int i=0;i< (argc - firstUnknownParam_);i++){  	
			cout<<"loading "<<argv[firstUnknownParam_+i]<< "... \n"; cout.flush();
			if(scene_.AppendSubTreeFromFile(argv[firstUnknownParam_+i])!=0)
				cout<<" failed\n";
			else{
				cout<<" success\n";				
			}
		}
		ResetSceneCenter();
	}	
	else
		LoadDefaultModel_();
#endif
#ifdef BIAS_HAVE_GLUT
	sceneGlut_.SetDrawTeapot(*teapot_,0.4);
  sceneGlut_.SetDrawCube(*cube_,0.4);  	  	
  sceneGlut_.SetDrawIcosahedron(*icosahedron_);
  sceneGlut_.SetDrawTetrahedron(*tetrahedron_);
#endif
}


void BIASOpenGLViewerFrame::
LoadDefaultModel_(){
#ifdef BIAS_HAVE_OPENSCENEGRAPH
	string model=BIAS_IMG_DIR;
	model += "BIAS_LOGO_3D.3ds";
	if(scene_.AppendSubTreeFromFile(model)==0){
		Vector3<double> C(-0.186, 1.1, 1.09);
		Quaternion<double> Q(0.07, -0.7, 0.69,0.008);						
		ResetSceneCenter(&C,&Q);
	}
#endif
}

/****************************************************
  @brief


 *****************************************************/
void BIASOpenGLViewerFrame::
InitParameters_() {

	// Animation / Control Params
	int groupID = GetFreeGroupID();
	useFlyMode_ = AddParamBool(
			"fly", "set mouse control to fly mode", false, 0 ,groupID);
	useRotMode_ = AddParamBool(
			"onlyRot", "turn only rotation for mouse control on", false, 0, groupID);
	SetGroupName(groupID, "Animation / Control");

	// View Params
	groupID = GetFreeGroupID();
	zNear_ = AddParamDouble(
			"zNear", "set near clipping plane distance, overrides autoClipping", 0.001, 0.000001, DBL_MAX, 0 ,groupID);
	zFar_ = AddParamDouble(
			"zFar", "set near clipping plane distance, overrides autoClipping", 10000, 0.00001, DBL_MAX, 0, groupID);
	fov_ = AddParamDouble(
			"fov", "field of view in degree", 0, 0, 359.9, 0, groupID);
	autoClipping_ = AddParamBool(
			"autoClipping", "turns auto clipping on/off", true, 0 , groupID);
	backfaceCulling_ = AddParamBool(
			"backfaceCulling", "do not draw back faces of triangles?", false, 0, groupID);
#ifdef BIAS_HAVE_GLUT
	teapot_ = AddParamBool("teapot", "Add a glut teapot to scene", false, 0, groupID);
	RegisterCallbackObject("teapot", this);
		
	cube_ = AddParamBool("cube", "Add a glut cube to scene", false, 0, groupID);
	RegisterCallbackObject("cube", this);
	tetrahedron_ = AddParamBool("tetrahedron", "Add a glut tetrahedron to scene", false, 0, groupID);
	RegisterCallbackObject("tetrahedron", this);
	icosahedron_ = AddParamBool("icosahedron", "Add a glut  icosahedron to scene", false, 0, groupID);
	RegisterCallbackObject("icosahedron", this);
#endif
	SetGroupName(groupID, "View");

	// Camera Params
	groupID = GetFreeGroupID(); 
	projectionFile_ = AddParamString(
			"projection", "load camera parameters from this file, sets findSceneCenter to false if useJustIntrinsics is false (default)", 
			"", 'p', groupID);
	useSceneCenter_ = AddParamBool(
			"findSceneCenter", "sets camera to center of scene", 
			true, 0, groupID);
	useJustIntrinsics_ = AddParamBool(
			"useJustIntrinsics", "uses only intrinsics from loaded projection", 
			false, 0, groupID);

	// create values for reshape behaviour
	std::vector<std::string> reshapeBehaviour;
	reshapeBehaviour.push_back("Deny");
	reshapeBehaviour.push_back("AutoRescaleParams");
	reshapeBehaviour.push_back("AutoCutImage");
	reshapeBehaviour.push_back("AutoSimplePerspectiveCam");

	autoReshapeBehaviour_ = AddParamEnum(
			"reshapeBehaviour", "sets the automatic reshape behaviour of the camera",
			reshapeBehaviour, 1,NULL, 0, groupID);
	SetGroupName(groupID, "Camera");

	// Misc Params
	groupID = GetFreeGroupID();
	winwidth_ = AddParamInt(
			"winwidth", "window width", 0, 0, INT_MAX, 'w', groupID);
	winheight_ = AddParamInt(
			"winheight", "window width", 0, 0, INT_MAX, 'h', groupID);
	screenShotName_ = AddParamString(
			"screenShotName", "name for screenshoys default is 'screenShot'", "screenShot", 0, groupID);
	backGroundColor_ = AddParamVecDbl(
			"bgColor", "set background clear color", "200.0 230.0 230.0", 0, groupID); 
	SetGroupName(groupID, "Window");

}
/****************************************************
  @brief


 *****************************************************/
void BIASOpenGLViewerFrame::
ParameterChanged(const std::string &paramname,const void *data) {
	cout<<"Change:";
#ifdef BIAS_HAVE_GLUT
	if (strcmp((const char*) paramname.c_str(), "teapot") == 0) {		
		sceneGlut_.SetDrawTeapot(*teapot_,0.4);		
		ResetSceneCenter();
	}
	else if (strcmp((const char*) paramname.c_str(), "cube") == 0) {		
		sceneGlut_.SetDrawCube(*cube_,0.4);
		ResetSceneCenter();
	} 
	else if (strcmp((const char*) paramname.c_str(), "icosahedron") == 0) {		
	  sceneGlut_.SetDrawIcosahedron(*icosahedron_);
	  ResetSceneCenter();
	} 	
	else if (strcmp((const char*) paramname.c_str(), "tetrahedron") == 0) {
		sceneGlut_.SetDrawTetrahedron(*tetrahedron_);
		ResetSceneCenter();
	}
#endif
}


/****************************************************
  @brief


 *****************************************************/
void BIASOpenGLViewerFrame::
OnAddModel(wxCommandEvent& event) {
#ifdef BIAS_HAVE_OPENSCENEGRAPH
	wxString fileName;
	wxFileDialog
			dialog(
					this,
					wxT("Load Model"),
					wxT(""),
					wxT(""),
					wxT("Model files (*.wrl;*.dae;*.obj;*.osg;*.3ds;)|*.wrl;*.dae;*.obj;*.osg;*.3ds;|All files (*.*)|*.*"),
					wxFD_OPEN);
	int ret = dialog.ShowModal();
	if (ret == wxID_OK) {	
		if(scene_.AppendSubTreeFromFile(WxToAscii(dialog.GetPath()))!=0)
			cout<<" failed\n";
		else{
			cout<<" success\n";			
 			ResetSceneCenter();
			contextWX_->Refresh();
		}
	}
#else
	stringstream text;
	text<<"OpenSceneGraph is not activated in BIAS, please activate to load 3D Models.";
	wxMessageBox(AsciiToWx(text.str()),wxT("Error"), wxOK | wxICON_ERROR, this);		
#endif
}

/****************************************************
  @brief


 *****************************************************/
void BIASOpenGLViewerFrame::OnQuit(wxCloseEvent& event) {
	wxCommandEvent e;
	OnExit(e);
}
/****************************************************
  @brief


 *****************************************************/
void BIASOpenGLViewerFrame::OnExit(wxCommandEvent& /*event*/) {	
	Destroy();
}
/****************************************************
  @brief


 *****************************************************/
void BIASOpenGLViewerFrame::
OnFullscreen(wxCommandEvent& event) {
	ShowFullScreen(!IsFullScreen(), wxFULLSCREEN_NOCAPTION | wxFULLSCREEN_NOSTATUSBAR);
  menuItemFullscreen_->Check(IsFullScreen());
}

void  BIASOpenGLViewerFrame::
OnScreenshot(wxCommandEvent& event){
	bool ret = screenShotControl_.StoreScreenShot();
	if(ret == false){
		stringstream text;
	  text<<"Storing screenshot failed.";
		wxMessageBox(AsciiToWx(text.str()),wxT("Error"), wxOK | wxICON_ERROR, this);
	}
}

void BIASOpenGLViewerFrame::
OnViewSolid( wxCommandEvent& event ){
	
	glPolygonMode( GL_FRONT_AND_BACK, GL_FILL);
	//cout << "PolygonMode: Fill." << endl;
	menuItemSolid_->Check(true);
	menuItemPoints_->Check(false);
	menuItemLines_->Check(false);
}

void BIASOpenGLViewerFrame::
OnViewLines( wxCommandEvent& event ){
	glPolygonMode( GL_FRONT_AND_BACK, GL_LINE);
	//cout << "PolygonMode: Line." << endl;
	menuItemSolid_->Check(false);
	menuItemPoints_->Check(false);
	menuItemLines_->Check(true);
}

void BIASOpenGLViewerFrame::
OnViewPoints( wxCommandEvent& event ){
	glPolygonMode( GL_FRONT_AND_BACK, GL_POINT);
	//cout << "PolygonMode: Point." << endl;
	menuItemSolid_->Check(false);
	menuItemPoints_->Check(true);
	menuItemLines_->Check(false);
}
////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////
class BIASOpenGLViewerApp: public wxApp
{
  virtual bool OnInit();
};


//declaration
DECLARE_APP(BIASOpenGLViewerApp)

// implementation
IMPLEMENT_APP(BIASOpenGLViewerApp)

bool BIASOpenGLViewerApp::OnInit()
{
  // utility: create a Redirection console for cout, cerr etc.
  //ConsoleRedirectIO("BIASOpenGLViewer");

  string fileName= "BIASOpenGLViewer.param";

  wxSize guiSize(640,480);
  BIASOpenGLViewerFrame *frame_=NULL;
  frame_ = new BIASOpenGLViewerFrame(NULL,-1,_T("BIAS OpenGLViewerWx"),wxPoint(0,0),guiSize);

  frame_->SetParameterFile(argc,WxToAsciiArray(argv,argc),fileName);
  frame_->Show(true);
  return TRUE;
}
