/*
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003-2009    (see file CONTACT for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "ShowCamWxFrame.hh"
#include "ShowCamWxThreads.hh"
#include "ShowCamWxFormat7Frame.hh"

//logos
//#include <img/miplogochars64x64.xpm>
#include <img/miplogochars32x32.xpm>
#include <wx/artprov.h>
#include <wx/aboutdlg.h>
#include <VideoSource/VideoSourceCapabilities.hh>
#include <Tools/BIASShowCamWX/ShowCamWxVideoSourceFactory.hh>
#include <Gui/wxTwoFilesInputDialog.h>

using namespace BIAS;
using namespace std;

#define PARAM_FILE_NAME "ShowCamWx.param"
#define BUFFER_SIZE 50

//#define AUTOSCAN

BEGIN_EVENT_TABLE(ShowCamWxFrame, wxFrame)

  EVT_CLOSE     (                       ShowCamWxFrame::OnWindowClose)
  EVT_KEY_DOWN  (                       ShowCamWxFrame::OnKey)
  EVT_MENU      (wxID_ABOUT,            ShowCamWxFrame::OnAbout)
  EVT_MENU      (wxID_EXIT,               ShowCamWxFrame::OnQuit)
  EVT_MENU      (ID_Grab,               ShowCamWxFrame::OnGrab)
  EVT_MENU      (ID_GrabContinuous,     ShowCamWxFrame::OnGrabContinuous)
  EVT_MENU      (ID_GrabToStream,       ShowCamWxFrame::OnGrabToStream)
  EVT_MENU      (ID_Show1,              ShowCamWxFrame::OnShowImage)
  EVT_MENU      (ID_Show2,              ShowCamWxFrame::OnShowImage)
  EVT_MENU      (ID_Show3,              ShowCamWxFrame::OnShowImage)
  EVT_MENU      (ID_Show4,              ShowCamWxFrame::OnShowImage)
  EVT_MENU      (ID_Show5,              ShowCamWxFrame::OnShowImage)
  EVT_MENU      (ID_Show6,              ShowCamWxFrame::OnShowImage)
  EVT_MENU      (ID_Show7,              ShowCamWxFrame::OnShowImage)
  EVT_MENU      (ID_Show8,              ShowCamWxFrame::OnShowImage)
  EVT_MENU      (ID_Show9,              ShowCamWxFrame::OnShowImage)
  EVT_MENU      (ID_Controller,         ShowCamWxFrame::OnShowController)
  EVT_MENU      (ID_SaveParameters,     ShowCamWxFrame::OnSaveParameters)
  EVT_MENU      (ID_ShowParameters,     ShowCamWxFrame::OnShowParameters)
  EVT_MENU      (ID_FindIEEE1394,       ShowCamWxFrame::OnFindIEEE1394)
  EVT_MENU      (ID_FindUEYE,           ShowCamWxFrame::OnFindUEye)
  EVT_MENU      (ID_FindZess,           ShowCamWxFrame::OnFindPMDZess)
  EVT_MENU      (ID_FindPMDTec,         ShowCamWxFrame::OnFindPMDTec)
  EVT_MENU      (ID_FindSwissranger,    ShowCamWxFrame::OnFindSwissRanger)
  EVT_MENU      (ID_FindKinect,         ShowCamWxFrame::OnFindKinect)
  EVT_MENU      (ID_FindOpenNI,         ShowCamWxFrame::OnFindOpenNI)
  EVT_MENU      (ID_FindKinect2,        ShowCamWxFrame::OnFindKinect2)
  EVT_MENU      (ID_FindWebcams,        ShowCamWxFrame::OnFindWebcams)
  EVT_MENU      (ID_OpenFlirController, ShowCamWxFrame::OnOpenFlirController)
  EVT_MENU      (ID_OpenViscaController, ShowCamWxFrame::OnOpenViscaController)
  EVT_MENU      (ID_OpenDPController,   ShowCamWxFrame::OnOpenDPController)
  EVT_TIMER     (ID_Timer,              ShowCamWxFrame::OnTimer)
  EVT_CHECKBOX  (ID_Fit,                ShowCamWxFrame::OnFit)
  EVT_CHECKBOX  (ID_KeepScroll,         ShowCamWxFrame::OnKeepScrollPosition)
  EVT_BUTTON      (ID_Grab,               ShowCamWxFrame::OnGrab)
  EVT_BUTTON      (ID_GrabContinuous,     ShowCamWxFrame::OnGrabContinuous)
  EVT_BUTTON      (ID_GrabToStream,       ShowCamWxFrame::OnGrabToStream)
  EVT_BUTTON      (ID_FindWebcams,        ShowCamWxFrame::OnFindWebcams)
  EVT_BUTTON      (ID_FindIEEE1394,       ShowCamWxFrame::OnFindIEEE1394)
  EVT_BUTTON      (ID_FindPMDTec,         ShowCamWxFrame::OnFindPMDTec)
  EVT_BUTTON      (ID_FindSwissranger,    ShowCamWxFrame::OnFindSwissRanger)
  EVT_BUTTON      (ID_Controller,         ShowCamWxFrame::OnShowController)
  EVT_BUTTON      (ID_HIDE_CAMERALIST,    ShowCamWxFrame::OnHideCamerasListBox)
  EVT_MENU      (ID_SHOW_CAMERALIST,    ShowCamWxFrame::OnShowCamerasListBox)
  EVT_LISTBOX_DCLICK(ID_CAMERAS_LISTBOX,  ShowCamWxFrame::OnClickCamerasListBox)
  END_EVENT_TABLE()


/****************************************************
*****************************************************/
ShowCamWxFrame::
ShowCamWxFrame(const wxString& title,
               const wxPoint& pos, const wxSize& size)
  : wxFrame((wxFrame*)NULL, -1, title, pos, size) ,
    ParamGUI ()   // disable param desctructor warning
{
  // initialize camera names for params
  cameraNames_ = ShowCamWxVideoSourceFactory::CameraNames();
  SetIcon(wxIcon(miplogochars32x32_xpm));
  // set default value for class variables
  bSingleSaveDirCreated_ = false;
  dSingleDirIndex_ = 0;
  dDirIndex_=0;
  tauControllerFrame_ = NULL;
#ifdef BIAS_HAVE_PANTILT
  viscaControllerFrame_ = NULL;
#endif

#ifdef BIAS_HAVE_PANTILT_DP
  panTiltControl_=NULL;
  panTiltControlDialog_=NULL;
#endif //BIAS_HAVE_PANTILT_DP

  paramFrame_ =NULL;
  OpenedDevices_    = false;
  InitCameras_ = false;
  InitParams_  = false;
  bSaveImages_ = false;
  bWriteBuffered_ = false;
  EmptyDevice_ = false;
  FrameCounter_ = 0;
  GrabCounter_  = 0;
  LastWidth_    = 0;
  LastHeight_   = 0;
  NumCamSel_    = 0;
  Win32DCAMNumber_ = 0;
  ShowXB3_ = 0;
#ifdef BIAS_HAVE_DCAM
  format7Frame_=NULL;
#endif
  FirstCameraID_ = 10000;
  LastCameraID_ = FirstCameraID_;
  TimerActive_ = false;
  ActiveCameras_ = 0;

  NumOfCams_ = 0;
  grabMasterThread_ = NULL;
  menuShow = NULL;

  CleanedUp_ = false;

  // set timer delay to 5 ms
  Delay_ = 25;
  Timer_.SetOwner(this, ID_Timer);

  // init images for cameras
  for (int i = 0; i < NUMBER_CAMERAS; i++)
  {
    // init Images
    CamImg_.push_back(NULL);
    CamImgMiddle_.push_back(NULL);
    CamImgRight_.push_back(NULL);

    CamImgPMDDepth_.push_back(NULL);
    CamImgPMDAmp_.push_back(NULL);
    CamImgPMDInt_.push_back(NULL);
    XB3AllInOne_.push_back(NULL);

    ThreadInited_.push_back(false);
    CameraActive_.push_back(false);
    CameraInitialized_.push_back(false);
    SetFirewireB_.push_back(false);
    SetTrigger_.push_back(false);
    SetUserMode_.push_back(true);
    SelectedFps_.push_back(15.0);
    Format7LeftTop_.push_back(Vector2<int>(0,0));
    Format7WidthHeight_.push_back(Vector2<int>(0,0));
    Format7BpP_.push_back(0);
    Format7ColorMode_.push_back(0);

    grabSingleThread_.push_back(NULL);
    saveThreads_.push_back(NULL);

	  grabMasterThread_ = NULL;

    // Ringbuffer for writing images
    BufferAccess_.push_back(NULL);
    WriteIndex_.push_back(0);
    ReadIndex_.push_back(BUFFER_SIZE-1);

    ImageBuffer_.push_back(new vector<Camera<unsigned char>* > );
    ImageBufferMiddle_.push_back(new vector<Camera<unsigned char>* >);
    ImageBufferRight_.push_back(new vector<Camera<unsigned char>* >);
    ImageBufferPMDDepth_.push_back(new vector<Camera<float>* >);
    ImageBufferPMDAmp_.push_back(new vector<Camera<float>* >);
    ImageBufferPMDInt_.push_back(new vector<Camera<float>* >);

    for(int j=0;j<BUFFER_SIZE;j++){
      ImageBuffer_[i]->push_back(NULL);
      ImageBufferMiddle_[i]->push_back(NULL);
      ImageBufferRight_[i]->push_back(NULL);
      ImageBufferPMDDepth_[i]->push_back(NULL);
      ImageBufferPMDAmp_[i]->push_back(NULL);
      ImageBufferPMDInt_[i]->push_back(NULL);
    }

    ColorModels_.push_back(ImageBase::CM_invalid);
    ViewColorMode_.push_back(ImageBase::CM_invalid);
    VideoControllers_.push_back(NULL);
    std::vector<string> tmp;
    DeviceChannelNames_.push_back(tmp);
    DeviceChannel_.push_back(-1);


  }
 // create tool bar
  wxToolBar* toolBar = CreateToolBar();
  toolBar->SetToolBitmapSize( wxSize( 24,24 ) );
  toolBar->SetToolSeparation( 0 );
  toolBar->SetToolPacking( 0 );
  wxSize smallButtonSize(60,25);
  wxSize mediumButtonSize(80,25);
  wxSize largeButtonSize(110,25);

  pButtonGrab= new wxButton(toolBar, ID_Grab, wxT("Grab"),wxDefaultPosition,smallButtonSize);
  pButtonGrab->SetToolTip(wxT("Grab single image."));
  toolBar->AddControl(pButtonGrab);
  
  pButtonContinuous= new wxButton(toolBar, ID_GrabContinuous, wxT("Grab Cont."),wxDefaultPosition,mediumButtonSize);
  pButtonContinuous->SetToolTip(wxT("Toggle grabbing continuously."));
  toolBar->AddControl(pButtonContinuous);
  
  // toolBar->AddCheckTool( ID_GrabContinuous, wxT("Toggle grabbing continuously."),
  //												wxArtProvider::GetIcon(wxART_ERROR ,wxART_TOOLBAR) );
  
  pButtonStream= new wxButton(toolBar, ID_GrabToStream, wxT("Grab To Stream"),wxDefaultPosition,largeButtonSize);
  pButtonStream->SetToolTip(wxT("Toggle grabbing to stream."));
  toolBar->AddControl(pButtonStream);
  toolBar->AddSeparator();
  
  pButtonIEEE1394= new wxButton(toolBar,ID_FindIEEE1394, wxT("Find IEEE1394"),wxDefaultPosition,largeButtonSize);
  pButtonIEEE1394->SetToolTip(wxT("Find Firewire IEEE1394 cameras."));
  toolBar->AddControl(pButtonIEEE1394);
#ifdef BIAS_HAVE_V4L
  pButtonWebCams= new wxButton(toolBar,ID_FindWebcams, wxT("Find V4L"),wxDefaultPosition,largeButtonSize);
  pButtonWebCams->SetToolTip(wxT("Find V4L Cameras."));
  toolBar->AddControl(pButtonWebCams);
#endif
#ifdef BIAS_HAVE_DSHOW
  pButtonWebCams= new wxButton(toolBar,ID_FindWebcams, wxT("Find DShow"),wxDefaultPosition,largeButtonSize);
  pButtonWebCams->SetToolTip(wxT("Find DSHOW Cameras."));
  toolBar->AddControl(pButtonWebCams);
#endif
#ifdef BIAS_HAVE_SWISSRANGER
  pButtonSR= new wxButton(toolBar,ID_FindSwissranger, wxT("Find SR"),wxDefaultPosition,smallButtonSize);
  pButtonSR->SetToolTip(wxT("Find Swissranger ToF cameras."));
  toolBar->AddControl(pButtonSR);
#endif
#ifdef BIAS_HAVE_PMD_PMDTec
  pButtonPMDTec= new wxButton(toolBar,ID_FindPMDTec, wxT("Find PMDTec"),wxDefaultPosition,largeButtonSize);
  pButtonPMDTec->SetToolTip(wxT("Find PMDTec ToF cameras."));
  toolBar->AddControl(pButtonPMDTec);
#endif
  toolBar->AddSeparator();
  
  pButtonController= new wxButton(toolBar,ID_Controller, wxT("Controller"),wxDefaultPosition,mediumButtonSize);
  pButtonController->SetToolTip(wxT("Show Videosource Controller"));
  toolBar->AddControl(pButtonController);
  toolBar->AddSeparator();

  pCheckBoxFit = new wxCheckBox(toolBar, ID_Fit, wxT("Fit window"));
  toolBar->AddControl(pCheckBoxFit);
  pCheckBoxKeepScrollPosition = new wxCheckBox(toolBar, ID_KeepScroll, wxT("Keep Scroll"));
  toolBar->AddControl(pCheckBoxKeepScrollPosition);
  pCheckBoxKeepScrollPosition->SetValue(true);
  toolBar->Realize();


  CreateCameraMenu();
  // create status bar and add status text
  CreateStatusBar(3);
  wxString text;
  text.Printf(wxT("size: %4dx%4d"),0, 0);
  SetStatusText(text, 1);


  //a splitter window as top level window
  splitterWindow_ = new wxSplitterWindow(this,ID_SPLITTER_WINDOW,wxDefaultPosition,
  		wxDefaultSize,wxSP_3D | wxSP_LIVE_UPDATE);
  splitterWindow_->SetMinimumPaneSize(100);

  // init image canvas as child of splitterwindow
  wxSize imageSize(640,480);
  IC_ = new ImageCanvas(splitterWindow_,GetStatusBar(), 0,-1,wxDefaultPosition,imageSize);
  IC_->SetEvtHandlerEnabled(true);
  IC_->KeepScrollPosition(true);

  cameraListPanel_ = new wxPanel(splitterWindow_);
  wxString choices[10];
  for(unsigned i=0;i<10;i++){
  	wxString str; str.Printf(wxT("Camera %d"),i);
  	choices[i] = str;
  }
  cameraListBox_ = new wxListBox(cameraListPanel_,ID_CAMERAS_LISTBOX,
  		wxDefaultPosition,wxSize(100,500), 10, choices,wxLB_SINGLE);
  wxSize buttonSize(50,35);
  wxButton *hideButton = new wxButton(cameraListPanel_,ID_HIDE_CAMERALIST,
  		wxT("Hide"),wxDefaultPosition,buttonSize);

  wxFlexGridSizer *rightPanelSizer = new wxFlexGridSizer(2,1,2,2);
  rightPanelSizer->AddGrowableRow(0);
  rightPanelSizer->Add(cameraListBox_,1,  wxEXPAND |wxALL , 5);
  rightPanelSizer->Add(hideButton,1,  wxALL , 5);
  cameraListPanel_->SetSizer(rightPanelSizer);

  //split between notebook and ImagelistPanel
  splitterWindow_->SplitVertically(IC_,cameraListPanel_,-1);

  //load test image
  string img = BIAS_IMG_DIR; img += "mipTestImage.gif";
  if(BIAS::ImageIO::Load(img,ShowImage_)!=0)
  {
  	ShowImage_.Init(640,480,3);
  	unsigned char color[3]={255,255,255};
  	ShowImage_.FillImageWithConstValue(color);
  }
  IC_->Show(ShowImage_,"Camera Image");

  // add image canvas to main window
  wxBoxSizer *vbox = new wxBoxSizer(wxVERTICAL);
  vbox->Add(splitterWindow_,1,  wxEXPAND |wxALL , 5); //|wxFIXED_MINSIZE , 5);

  SetSizer(vbox);
  SetAutoLayout(true);
  SetMinSize(size);
  Layout();
  Fit();

  Timer_.Start(Delay_);
}

/****************************************************
*****************************************************/
ShowCamWxFrame::
~ShowCamWxFrame()
{
  // close threads
  for(int i = 0; i < NUMBER_CAMERAS; i++){
    if(grabSingleThread_[i] != NULL){
      delete grabSingleThread_[i];
    }
    if(saveThreads_[i] != NULL){
      delete saveThreads_[i];
    }
  }
}
/****************************************************
*****************************************************/
void ShowCamWxFrame::
InitParams(int &argc, char *argv[])
{
  // initialize color models for cameras
  vector<string> colorModels;
  colorModels.push_back("CM_Default");
  colorModels.push_back("CM_Grey");
  colorModels.push_back("CM_RGB");
  colorModels.push_back("CM_BGR");
  colorModels.push_back("CM_YUYV422");
  colorModels.push_back("CM_UYVY422");
  colorModels.push_back("CM_YUV420P");
  colorModels.push_back("CM_YUV444");
  colorModels.push_back("CM_YUV411");
  colorModels.push_back("CM_HSV");
  colorModels.push_back("CM_HSI_OBS");
  colorModels.push_back("CM_DV");
  colorModels.push_back("CM_RGBA");
  colorModels.push_back("CM_GreyA");
  colorModels.push_back("CM_Bayer_RGGB");
  colorModels.push_back("CM_Bayer_GBRG");
  colorModels.push_back("CM_Bayer_GRBG");
  colorModels.push_back("CM_Bayer_BGGR");
  colorModels.push_back("CM_HSL");
  colorModels.push_back("CM_hsL");
  colorModels.push_back("CM_SymTensor2x2");
  colorModels.push_back("CM_BGRA");
  colorModels.push_back("CM_RGBE");



  for (int i = 0; i < NUMBER_CAMERAS; i++)
    {
      // add nine different cameras to params and set group name
      stringstream number;
      number << i + 1;
      ParamCameras_[i] = AddParamEnum("CameraType" + number.str(),
                                      "Type of camera", cameraNames_, 0, NULL, 0, i);
      ParamColorModel_[i] = AddParamEnum("CameraColorModel" + number.str(),
                                         "Color Model for camera", colorModels,
                                         0, NULL, 0, i);
      ParamFPS_[i]    = AddParamDouble("CameraFPS" + number.str(),
                                       "fps possibilities: 1.875 3.75 7.5 15.0 30.0 60.0 120.0 240.0",
                                       0.0, 0.0, 240.0, 0, i);
      ParamWidth_[i]  = AddParamInt("CameraWidth" + number.str(),
                                    "Width of camera", 0, 0, 10000, 0, i);
      ParamHeight_[i] = AddParamInt("CameraHeight" + number.str(),
                                    "Height of camera", 0, 0, 10000, 0, i);
      //ParamSetTop_[i]  = AddParamInt();
      //ParamSetLeft_[i] = AddParamInt();
      ParamDown_[i]   = AddParamInt("CameraDownsampling" + number.str(),
                                    "Downsampling of camera", 0, 0, 10, 0, i);
      ParamPort_[i]   = AddParamInt("CameraPort" + number.str(),
                                    "Port of NET camera", 0, 0, 100000, 0, i);
      ParamIP_[i]     = AddParamString("CameraIP" + number.str(),
                                       "IP of NET camera", "", 0, i);
      ParamBitFile_[i]= AddParamString("CameraBitFile" + number.str(),
                                       "Bit file for usb pmd cameras", "", 0, i);
      ParamDevice_[i] = AddParamString("CameraDevice" + number.str(),
                                       "Device/Interface name of camera",
                                       "", 0, i);
      // every camera gets own group number
      stringstream text;
      text<<number.str()<<". Camera";
      SetGroupName(i, text.str());
    }
#ifdef BIAS_HAVE_PANTILT_DP
  bUsePanTiltUnit_ = AddParamBool("UsePanTiltUnit","Use the Pan-Tilt-Unit?",false,
                                  'p', NUMBER_CAMERAS);
  stringstream text;
  text<<"General";
  SetGroupName(NUMBER_CAMERAS, text.str());
#endif
  // check if param file exists, else create one
  ifstream ifs(PARAM_FILE_NAME);
  if (!ifs)
    WriteParameter(PARAM_FILE_NAME);
  else
    {
      // read param from file
      ReadParameter(PARAM_FILE_NAME);
      // update parameters from file with arguments from command line
      UpdateParameter(argc, argv, PARAM_FILE_NAME);
      ifs.close();
    }

  // get color models for cameras
  ParseColorModels();

  InitParams_ = true;
}

/****************************************************
*****************************************************/
void ShowCamWxFrame::
OnOpenDPController(wxCommandEvent& event){
#ifdef BIAS_HAVE_PANTILT_DP
	bool ret = true;
	if(panTiltControl_==NULL)
		ret = InitPanTiltUnit();
	if(ret && panTiltControlDialog_!=NULL)
		panTiltControlDialog_->Show(true);
#endif
}


#ifdef BIAS_HAVE_PANTILT_DP
/****************************************************
*****************************************************/
bool ShowCamWxFrame::
InitPanTiltUnit()
{
	bool bRet = false;

	if(panTiltControl_==NULL)
		panTiltControl_= new DPPanTiltControl();

	if(panTiltControlDialog_==NULL)
		panTiltControlDialog_ = new DPPanTiltControlDialog(this, 7651,
			wxT("Pan-Tilt-Unit Control"));
#ifdef WIN32
	bRet = panTiltControl_->InitPanTiltUnit("COM1");
#else
	bRet = panTiltControl_->InitPanTiltUnit("/dev/ttyUSB0");
#endif
	if(bRet){
		panTiltControlDialog_->SetDPControl(panTiltControl_);
		panTiltControlDialog_->Show(true);
		return true;
	}
	else{
		stringstream txt;
		txt << "Could not open port for Directed Perception Control";
		wxMessageDialog errorDialog1(this,AsciiToWx(txt.str().c_str()),
				wxT("ShowCamWx - Error"),wxICON_ERROR|wxOK);
		errorDialog1.ShowModal();
	}
	return false;
}
#endif //BIAS_HAVE_PANTILT_DP

/****************************************************
*****************************************************/
void ShowCamWxFrame::
InitCameras()
{
  if (!InitParams_){
    BIASERR("Initialize params first");
    return;
  }

  bool foundCamera = false;
  for (int i = 0; i < NUMBER_CAMERAS; i++) {
    Cameras_.push_back(NULL);
    if (*ParamCameras_[i] != ShowCamWxVideoSourceFactory::NONE)
      foundCamera = true;
  }


  //if there were no cams found, exit and wait for a manual camera scan.
  if (!foundCamera) {
    return;
  }else{
#ifndef WIN32
#  ifdef BIAS_HAVE_DCAM
    int numDcams = 0;
#  endif //BIAS_HAVE_DCAM
    for(int i = 0; i < NUMBER_CAMERAS; i++){

      stringstream errorText;
      // scan for dcam camera parameters
      if(*ParamCameras_[i] == 1){
#ifdef BIAS_HAVE_DCAM
        BIAS::ScanBus(IEEECameraInfo_);

        if(CameraMenu_ == NULL)
          CameraMenu_ = new wxMenu;
        // search for the cam
        for(unsigned int k=0; k< IEEECameraInfo_.size();k++){
          if(IEEECameraInfo_[k].camid == atoi((*ParamDevice_[i]).c_str())){
            // add to menu
            AddIEEECameraToMenu(i,k,true);
          }
        }
        numDcams++;
#else
        errorText << "DCAM Camera in param file, please enable support in BIAS\n";
        BIASERR("DCAM Camera in param file, please enable support in BIAS");

#endif
      }
#else
      int NumDcams = 0;
      for(int i = 0; i < NUMBER_CAMERAS; i++){
        stringstream errorText;
        // scan for dcam camera parameters
        if(*ParamCameras_[i] == 1){
#ifdef BIAS_HAVE_DCAM
        	BIAS::ScanBus(IEEECameraInfo_);

          if(CameraMenu_ == NULL)
            CameraMenu_ = new wxMenu;
          // search for the cam
          for(unsigned int k=0; k< IEEECameraInfo_.size();k++){
            if(IEEECameraInfo_[k].camid == NumDcams + 1){
              // add to menu
              stringstream ss;
              ss << NumDcams + 1;
              *ParamDevice_[i] = ss.str();
              AddIEEECameraToMenu(i,k,true);

            }
          }
          NumDcams++;
#else
          errorText << "DCAM Camera in param file, please enable support in BIAS\n";
          BIASERR("DCAM Camera in param file, please enable support in BIAS");

#endif
        }
#endif
        if(*ParamCameras_[i] == 2){
#ifdef BIAS_HAVE_PMD_PMDTec
          AddPMDTecCameraToMenu(i,true);
#else
          errorText << "PMDTec Camera in param file, please enable support in BIAS\n";
          BIASERR("PMDTec Camera in param file, please enable support in BIAS");
#endif
        }



        if(*ParamCameras_[i] == 3){
#ifdef BIAS_HAVE_PMD_PMDZess
          AddZessCameraToMenu(i,true,"");
#else
          errorText << "PMD Zess Camera in param file, please enable support in BIAS\n";
          BIASERR("PMD Zess Camera in param file, please enable support in BIAS");
#endif
        }

        if(*ParamCameras_[i] == 5){
#ifdef BIAS_HAVE_UEYE
          AddUEyeCameraToMenu(i,"", true);
#else
          errorText << "UEYE Camera in param file, please enable support in BIAS\n";
          BIASERR("UEYE Camera in param file, please enable support in BIAS");
#endif
        }

        if(*ParamCameras_[i] == 9){
#ifdef BIAS_HAVE_SWISSRANGER
          AddSwissRangerCameraToMenu(i,true);
#else
          errorText << "SwissRanger Camera in param file, please enable support in BIAS\n";
          BIASERR("SwissRanger Camera in param file, please enable support in BIAS");
#endif
        }


      if(*ParamCameras_[i] != 0) {
        cout <<"INIT CAM" << endl;
        InitCamera(i);
      }
    }
  }

  InitCameras_ = true;

  OpenDevices();
}

/****************************************************
 *****************************************************/
void ShowCamWxFrame::InitCamera(int camindex){
    int i = camindex;
    stringstream errorText;

    BufferAccess_[i] = new wxMutex;
    WriteIndex_[i] = 0;
    ReadIndex_[i] = BUFFER_SIZE - 1;

    if(CamImg_[i] == NULL)
      CamImg_[i] = new Camera<unsigned char> ;

    //CamImgCombined_.Release();

    switch(*ParamCameras_[i]){
    case ShowCamWxVideoSourceFactory::NONE:
      // Cameras_.push_back(NULL);
      break;
    case ShowCamWxVideoSourceFactory::DCAM:
      Cameras_[i] = ShowCamWxVideoSourceFactory::CreateDCAM(
          SetFirewireB_[i],
          SetUserMode_[i],
          SelectedModeFromMenu(i),
          SelectedFps_[i],
          Format7BpP_[i],
          Format7ColorMode_[i],
          Format7LeftTop_[i],
          Format7WidthHeight_[i]
      );
      break;
    case ShowCamWxVideoSourceFactory::PMD:
      Cameras_[i] = ShowCamWxVideoSourceFactory::CreatePMD(sourcePlugin_,procPlugin_);
      break;
    case ShowCamWxVideoSourceFactory::PMDSHM:
    case ShowCamWxVideoSourceFactory::SHM:
    case ShowCamWxVideoSourceFactory::NET:
    case ShowCamWxVideoSourceFactory::SR:
    case ShowCamWxVideoSourceFactory::KINECT:
    case ShowCamWxVideoSourceFactory::FREENECT2:
      Cameras_[i] = ShowCamWxVideoSourceFactory::CreateFromTypeIfSupported(*ParamCameras_[i]);
      break;
    case ShowCamWxVideoSourceFactory::PMDUSB:
      if (*ParamBitFile_[i] == "") {
        BIASERR("Specify bit file for usb pmd camera");
        Cameras_[i] = NULL;
      } else {
        Cameras_[i] = ShowCamWxVideoSourceFactory::CreatePMDZess(*ParamBitFile_[i]);
      }
      break;
    case ShowCamWxVideoSourceFactory::UEYE:
      Cameras_[i] = ShowCamWxVideoSourceFactory::CreateUEye();
      break;
    case ShowCamWxVideoSourceFactory::V4L:
      Cameras_[i] = ShowCamWxVideoSourceFactory::CreateV4L();      
      break;
    case ShowCamWxVideoSourceFactory::XB3:
      Cameras_[i] = ShowCamWxVideoSourceFactory::CreateBumbleBee();
#ifdef BIAS_HAVE_DCAM
      dynamic_cast<VideoSource_DCAM_BumbleBee*>(Cameras_[i])->SetStereoDefaultMode(&IEEECameraInfo_[i]);
#endif
      break;
    case ShowCamWxVideoSourceFactory::DSHOW:
      Cameras_[i] = ShowCamWxVideoSourceFactory::CreateV4L();      
      break;
    default: // None
      //Cameras_.push_back(NULL);
      break;
    }

    // create extra images
    if (Cameras_[camindex] != NULL) {
      switch(*ParamCameras_[camindex]) {
      case ShowCamWxVideoSourceFactory::PMD:
        if(CamImgPMDDepth_[camindex] == NULL)
          CamImgPMDDepth_[camindex] = new Camera<float> ;
        if(CamImgPMDAmp_[camindex] == NULL)
          CamImgPMDAmp_[camindex] = new Camera<float> ;
        if(CamImgPMDInt_[camindex] == NULL)
          CamImgPMDInt_[camindex] = new Camera<float> ;
        break;
      case ShowCamWxVideoSourceFactory::SR:
        if(CamImgPMDDepth_[camindex] == NULL)
          CamImgPMDDepth_[camindex] = new Camera<float> ;
        if(CamImgPMDAmp_[camindex] == NULL)
          CamImgPMDAmp_[camindex] = new Camera<float> ;
        break;
      case ShowCamWxVideoSourceFactory::XB3:
        ColorModels_[i] = Cameras_[i]->GetColorModel();
        if(XB3AllInOne_[camindex] == NULL)
          XB3AllInOne_[camindex] = new Camera<unsigned char> ;
        if(CamImgMiddle_[camindex] == NULL)
          CamImgMiddle_[camindex] = new Camera<unsigned char> ;
        if(CamImgRight_[camindex] == NULL)
          CamImgRight_[camindex] = new Camera<unsigned char> ;
        break;
      case ShowCamWxVideoSourceFactory::KINECT:
      case ShowCamWxVideoSourceFactory::OPENNI:
        if(CamImgPMDDepth_[camindex] == NULL)
          CamImgPMDDepth_[camindex] = new Camera<float> ;
        break;
      case ShowCamWxVideoSourceFactory::FREENECT2:
        if(CamImgPMDDepth_[camindex] == NULL)
          CamImgPMDDepth_[camindex] = new Camera<float> ;
        if(CamImgPMDInt_[camindex] == NULL)
          CamImgPMDInt_[camindex] = new Camera<float> ;
        break;
      default: // None
        //Cameras_.push_back(NULL);
        break;
      }
    }

    if (Cameras_[i] == NULL) {
      *ParamCameras_[i] = ShowCamWxVideoSourceFactory::NONE;
      errorText << cameraNames_[i] << " Camera in param file, please enable support in BIAS\n";
      //Cameras_.push_back(NULL);
      BIASERR(cameraNames_[i] << " Camera in param file, please enable support in BIAS");
      wxMessageDialog errorDialog1(this,
                                   AsciiToWx(errorText.str().c_str()),
                                   wxT("ShowCamWx - Error"),
                                   wxICON_ERROR|wxOK);
      errorDialog1.ShowModal();
    }

    if (*ParamHeight_[i] != 0 && *ParamWidth_[i] != 0 && Cameras_[i] != NULL) {
      Cameras_[i]->SetSize(*ParamWidth_[i], *ParamHeight_[i]);
       //cout<<"Setting size to :"<<*ParamWidth_[i]<<", "<< *ParamHeight_[i]<<endl;
    }

    if (*ParamFPS_[i] != 0.0 && Cameras_[i] != NULL) {
      cout<<"Setting "<<i<<". camera fps to "<<*ParamFPS_[i]<<endl;
      Cameras_[i]->SetFPS((float) *ParamFPS_[i]);
    }

    CameraInitialized_[i] = true;
  }//end Init Cameras



/****************************************************
*****************************************************/
int ShowCamWxFrame::OpenDevice(int camindex) {

  // we have to check which camera type we have
  int ret = 0;
  int i = camindex;
  if(*ParamCameras_[i] == ShowCamWxVideoSourceFactory::NONE)
    return -1;
  
  if(!CameraInitialized_[i]){
   InitCamera(i);
  }


  VideoSourceCapabilities caps;
  if (*ParamDevice_[i] != "")  {
  	if (Cameras_[i]->GetCapabilities((*ParamDevice_[i]).c_str(),caps) != 0 ) {
  		BIASERR("Could not get Capabilities for device:"<<*ParamDevice_[i]);
  	}
  }
  else{
  	if ( Cameras_[i]->GetCapabilities(caps) != 0 ) {
  		BIASERR("Could not get Capabilities.");
  	}
  }

  Cameras_[i]->SetDeviceChannel(DeviceChannel_[i]);
  Cameras_[i]->SetExternalTrigger(SetTrigger_[i]);

  ret = 0;

  switch(*ParamCameras_[i]) {
    case  ShowCamWxVideoSourceFactory::NONE:
      break;
    case ShowCamWxVideoSourceFactory::XB3:
    case ShowCamWxVideoSourceFactory::DCAM:
      if (!CameraInitialized_[i]) {
        InitCamera(i);
      }

      ret = ShowCamWxVideoSourceFactory::OpenDevice(*ParamCameras_[i], Cameras_[i], *ParamDevice_[i]);
#ifndef WIN32
      // check if there are multiple cameras with no device name
      if (ret == -2) {
        EmptyDevice_ = true;
        ret = 0;
      }

      if (EmptyDevice_) {
        stringstream text;
        text << "Found more than one camera with no device name!\n";
        //   <<"Available cameras:\n"<<cameras.str().c_str();
        wxMessageDialog errorDialog(this,
                                    AsciiToWx(text.str().c_str()),
                                    wxT("ShowCamWx - Error"),
                                    wxICON_ERROR|wxOK);
        errorDialog.ShowModal();
      }
      if (ret < 0) {
        stringstream text;
        text << "Camera DCAM " << i << " could not be opened";
        wxMessageDialog errorDialog(this,
                                    AsciiToWx(text.str().c_str()),
                                    wxT("ShowCamWx - Error"),
                                    wxICON_ERROR|wxOK);
        errorDialog.ShowModal();
        return -1;
      }
#endif // WIN32
      break;

    default:
      if (!CameraInitialized_[i]) {
        InitCamera(i);
      }
      if (ShowCamWxVideoSourceFactory::OpenDevice(*ParamCameras_[i], Cameras_[i], *ParamDevice_[i], *ParamPort_[i]) < 0)
        {
          delete Cameras_[i];
          Cameras_[i] = NULL;
          stringstream text;
          text << "Camera " << cameraNames_[ *ParamCameras_[i] ] << " " << i << " could not be opened";
          wxMessageDialog errorDialog(this,
                                      AsciiToWx(text.str().c_str()),
                                      wxT("ShowCamWx - Error"),
                                      wxICON_ERROR|wxOK);
          errorDialog.ShowModal();
          *ParamCameras_[i] = ShowCamWxVideoSourceFactory::NONE;
          return -1;
        }
      break;
  }

  // show the correct camera
  NumCamSel_ = camindex;

  /************************************************/
  /******* now init images from cameras *****/
  if (Cameras_[i] != NULL)
  {
    // try to set color model from param file for camera
  	if (ColorModels_[i] != ImageBase::CM_invalid)
  		Cameras_[i]->SetColorModel(ColorModels_[i]);

  	// initialize image for camera and grab
  	Cameras_[i]->InitImage(*CamImg_[i]);
  	
  switch(*ParamCameras_[i]) {
    case  ShowCamWxVideoSourceFactory::NONE:
      break;
    case ShowCamWxVideoSourceFactory::XB3:
      ShowCamWxVideoSourceFactory::InitImages(*ParamCameras_[i], Cameras_[i], (ImageBase*)CamImg_[i], (ImageBase*)CamImgMiddle_[i], (ImageBase*)CamImgRight_[i], (ImageBase*)XB3AllInOne_[i]);
      break;
    case ShowCamWxVideoSourceFactory::PMDUSB:
      ShowCamWxVideoSourceFactory::InitImages(*ParamCameras_[i], Cameras_[i], (ImageBase*)CamImg_[i], (ImageBase*)CamImgPMDDepth_[i]);
      break;
    case ShowCamWxVideoSourceFactory::PMD:
    case ShowCamWxVideoSourceFactory::PMDSHM:
    case ShowCamWxVideoSourceFactory::KINECT:
    case ShowCamWxVideoSourceFactory::OPENNI:
      ShowCamWxVideoSourceFactory::InitImages(*ParamCameras_[i], Cameras_[i], (ImageBase*)CamImgPMDDepth_[i], (ImageBase*)CamImgPMDAmp_[i], (ImageBase*)CamImgPMDInt_[i]);
      break;
    case ShowCamWxVideoSourceFactory::FREENECT2:
      ShowCamWxVideoSourceFactory::InitImages(*ParamCameras_[i], Cameras_[i], (ImageBase*)CamImg_[i], (ImageBase*)CamImgPMDDepth_[i], (ImageBase*)CamImgPMDInt_[i]);
      break;
  }

#ifdef BIAS_DEBUG
  cout<<"Initialized image with size "<<CamImg_[i]->GetWidth()<<" x "<<CamImg_[i]->GetHeight()
      <<" and "<<CamImg_[i]->GetChannelCount()<<" color channels"<<endl;
#endif
  	
  ret = Cameras_[i]->PreGrab();
  if(ret < 0) return ret;
  Cameras_[i]->SetDebugLevel(1);

#ifdef BIAS_HAVE_V4L
  if(dynamic_cast<VideoSource_V4L*>(Cameras_[i]) != NULL)
    Cameras_[i]->SetWaitForNew(true);
#endif

  /******* now start the threads for grabbing *****/
  //check if camera is used
  if(!ThreadInited_[camindex])
    ret = CreateGrabThreadForCamera_(i);
    if(ret < 0) return ret;
  }// end if Cameras_[i] != NULL

  /************************************************/
  // Set CameraActive
  CameraActive_[camindex] = true;
  OpenedDevices_ = true;

  // activate selection of camera
  if(menuShow != NULL)
    menuShow->Enable(ID_Show1 + camindex, true);

  // init and start timer
  if (!TimerActive_) {
    StopWatch_.Start();
    TimerActive_ = true;
  }

  ActiveCameras_++;
  return ret;
}

/****************************************************
*****************************************************/
int ShowCamWxFrame::CloseDevice(int camindex) {
  // close device and reinitialize variables
  int ret =0;
  if(grabSingleThread_[camindex] != NULL){
    ThreadInited_[camindex] = false;
    grabSingleThread_[camindex]->Delete();
    //delete grabSingleThread_[camindex];
    grabSingleThread_[camindex] = NULL;
    biassleep(1);
  }
  bool gmtLocked = false;
  bool noCamActive = false;
  if (grabMasterThread_ != NULL && Cameras_[camindex] != NULL) {
    ServerMutex_[camindex]->Lock();
    gmtLocked = true;
  }

  if(saveThreads_[camindex] != NULL){
    saveThreads_[camindex]->Delete();
    delete saveThreads_[camindex];
    saveThreads_[camindex] = NULL;
  }

  if(BufferAccess_[camindex] != NULL){
    delete BufferAccess_[camindex];
    BufferAccess_[camindex] = NULL;
  }

  WriteIndex_[camindex] = 0;
  ReadIndex_[camindex] = BUFFER_SIZE - 1;

  CameraActive_[camindex] = false;
  CameraInitialized_[camindex] = false;

  if(CamImg_[camindex]!= NULL){
    delete CamImg_[camindex];
    CamImg_[camindex] = NULL;
  }
  if(CamImgPMDDepth_[camindex]!= NULL){
    delete CamImgPMDDepth_[camindex];
    CamImgPMDDepth_[camindex] = NULL;
  }
  if(CamImgPMDAmp_[camindex]!= NULL){
    delete CamImgPMDAmp_[camindex];
    CamImgPMDAmp_[camindex] = NULL;
  }
  if(CamImgPMDInt_[camindex]!= NULL){
    delete CamImgPMDInt_[camindex];
    CamImgPMDInt_[camindex] = NULL;
  }
  if(CamImgMiddle_[camindex]!= NULL){
    delete CamImgMiddle_[camindex];
    CamImgMiddle_[camindex] = NULL;
  }
  if(CamImgRight_[camindex]!= NULL){
    delete CamImgRight_[camindex];
    CamImgRight_[camindex] = NULL;
  }
  if(XB3AllInOne_[camindex]!= NULL){
    delete XB3AllInOne_[camindex];
    XB3AllInOne_[camindex] = NULL;
  }

  if(ViewColorMode_[camindex] != ImageBase::CM_invalid){
    ViewColorMode_[camindex] = ImageBase::CM_invalid;
  }

  menuShow->Enable(ID_Show1 + camindex, false);

  
  if (Cameras_[camindex] != NULL) {
  	Cameras_[camindex]->PostGrab();
  	Cameras_[camindex]->CloseDevice();
  	delete Cameras_[camindex];
  	Cameras_[camindex] = NULL;
  	ActiveCameras_--;
  	noCamActive = (ActiveCameras_ == 0);
  }

  if (gmtLocked) {
    ServerCondition_[camindex]->Signal();
    ServerMutex_[camindex]->Unlock();
  }

  // maybe stop and reset timer
  if(noCamActive) {
    StopWatch_.Stop();
    TimerActive_ = false;
    if(grabMasterThread_ != NULL){
      grabMasterThread_->Delete();
      //delete grabMasterThread_;
      grabMasterThread_=NULL;
    }
    if(ServerCondition_[camindex] !=NULL){
      delete ServerCondition_[camindex];
      ServerCondition_[camindex]=NULL;
    }
    if(ServerMutex_[camindex] !=NULL){
      delete ServerMutex_[camindex];
      ServerMutex_[camindex]=NULL;
    }
    if(ClientsCondition_[camindex] !=NULL){
      delete ClientsCondition_[camindex];
      ClientsCondition_[camindex]=NULL;
    }
    if(ClientsMutex_[camindex] !=NULL){
      delete ClientsMutex_[camindex];
      ClientsMutex_[camindex]=NULL;
    }
  }
  //cout<<"Closed Device :"<<camindex<<endl;
  return ret;
}

/****************************************************
*****************************************************/
void ShowCamWxFrame::OpenDevices() {
  if (!InitCameras_) {
    BIASERR("Initialize cameras first");
    return;
  }

  for (int i = 0; i < NUMBER_CAMERAS; i++) {
    OpenDevice(i);
  }
}

/****************************************************
*****************************************************/
int ShowCamWxFrame::SelectedModeFromMenu(int camindex) {
#ifdef BIAS_HAVE_DCAM
    int menuindex = (camindex + 1) * 10000;
    //    cout<<"Menuindex:"<<menuindex<<endl;
    dc1394video_mode_t mode = DC1394_VIDEO_MODE_640x480_MONO8;
    bool foundItem = false;

    //Never!! commit such things!!
    //return DC1394_VIDEO_MODE_640x480_RGB8;

    while(!foundItem){
      menuindex++;
      wxMenuItem* menuItem = CameraMenu_->FindItem(menuindex);
      if(menuItem!= NULL && menuItem->IsChecked()){
        foundItem = true;
      }
    }

    //cout << "Checked Item: " << (menuindex % 10000 - 1) << endl;
    int modeindex = (menuindex % 10000 - 1);

    for(unsigned int k=0; k< IEEECameraInfo_.size();k++){
      if(IEEECameraInfo_[k].camid == atoi((*ParamDevice_[camindex]).c_str())){
        // add to menu
        mode = IEEECameraInfo_[k].videoModes[modeindex];
      }
    }
    cout << "Found mode: " << mode << endl;
    return (int)mode;
#else
    return -1;
#endif
  }


/****************************************************
*****************************************************/
void ShowCamWxFrame::CreateMenu() {
  // create file menu
  wxMenu *menuFile = new wxMenu;
  menuFile->Append(ID_Grab, wxT("Grab &single image") );
  menuFile->AppendCheckItem(ID_GrabContinuous, wxT("Grab &continuously (Start/Stop)") );
  menuFile->AppendCheckItem(ID_GrabToStream, wxT("Grab to stream (Start/Stop)") );
  menuFile->AppendSeparator();
  menuFile->Append(wxID_EXIT, wxT("E&xit") );

  menuShow = new wxMenu;

  menuShow->Append(ID_Show1, wxT("Camera &1") );
  menuShow->Append(ID_Show2, wxT("Camera &2") );
  menuShow->Append(ID_Show3, wxT("Camera &3") );
  menuShow->Append(ID_Show4, wxT("Camera &4") );
  menuShow->Append(ID_Show5, wxT("Camera &5") );
  menuShow->Append(ID_Show6, wxT("Camera &6") );
  menuShow->Append(ID_Show7, wxT("Camera &7") );
  menuShow->Append(ID_Show8, wxT("Camera &8") );
  menuShow->Append(ID_Show9, wxT("Camera &9") );
  menuShow->AppendSeparator();
  menuShow->Append(ID_Controller, wxT("&Controller") );
  menuShow->Append(ID_SHOW_CAMERALIST, wxT("Camera &List") );

  if(CameraMenu_ == NULL)
    CameraMenu_ = new wxMenu;


  wxMenu *menuSearchCameras = new wxMenu;
  menuSearchCameras->Append(ID_FindIEEE1394, wxT("Scan for IEEE1394 Camera") );
  menuSearchCameras->Append(ID_FindUEYE, wxT("Scan for UEYE Camera") );
#ifdef BIAS_HAVE_V4L
  menuSearchCameras->Append(ID_FindWebcams, wxT("Scan for Video4Linux cameras") );
#endif
#ifdef BIAS_HAVE_SHOW
  menuSearchCameras->Append(ID_FindWebcams, wxT("Scan for DShow cameras") );
#endif
  menuSearchCameras->Append(ID_FindSwissranger, wxT("Scan for Swissranger ToF") );
  menuSearchCameras->Append(ID_FindZess, wxT("Scan for Zess ToF") );
  menuSearchCameras->Append(ID_FindPMDTec, wxT("Scan for PMDTec ToF") );
  menuSearchCameras->Append(ID_FindKinect, wxT("Scan for Kinect") );
  menuSearchCameras->Append(ID_FindOpenNI, wxT("Scan for OpenNI") );
  menuSearchCameras->Append(ID_FindKinect2, wxT("Scan for Kinect2 (freenect2)") );
  //menuSearchCameras->Append(ID_FindV4L, wxT("Scan for Video for Linux Camera") );


  wxMenu *menuParameters = new wxMenu;
  menuParameters->Append(ID_SaveParameters, wxT("Save Parameters") );
  menuParameters->Append(ID_ShowParameters, wxT("Show Parameters") );

  wxMenu *menuHelp = new wxMenu;
  menuHelp->Append(wxID_ABOUT);

  wxMenu *menuSpecial= new wxMenu;
  menuSpecial->Append(ID_OpenFlirController, wxT("Open FLIR Thermal Camera Controller") );
#ifdef BIAS_HAVE_PANTILT
  menuSpecial->Append(ID_OpenViscaController, wxT("Open Sony VISCA PanTilt Controller") );
#endif

#ifdef BIAS_HAVE_PANTILT_DP
  menuSpecial->Append(ID_OpenDPController, wxT("Directed Percpetion PanTilt Controller") );
#endif

  // add menues to menu bar
  wxMenuBar *menuBar = new wxMenuBar;
  menuBar->Append(menuFile, wxT("&File"));
  menuBar->Append(CameraMenu_, wxT("&Cameras"));
  menuBar->Append(menuSearchCameras, wxT("Camera &Scan"));
  menuBar->Append(menuParameters, wxT("&Parameters"));
  menuBar->Append(menuSpecial, wxT("&Extras"));
  menuBar->Append(menuShow, wxT("&Show"));
  menuBar->Append(menuHelp, wxT("&Help"));

  SetMenuBar(menuBar);

  // deactivate uninitialized cameras
  for (int i = 0; i < NUMBER_CAMERAS; i++) {
    if (Cameras_[i] == NULL) {
      menuShow->Enable(ID_Show1 + i, false);
    }
  }

  // fit all components to size of window
  Fit();
}

/****************************************************
*****************************************************/
void ShowCamWxFrame::CreateCameraMenu(){
	CameraMenu_ = new wxMenu;
}

#ifdef BIAS_HAVE_PANTILT_DP

/****************************************************
 *****************************************************/
void ShowCamWxFrame::
GetProjectionFromPanTilt_(BIAS::Projection &projHeader)
{
  double pan=0.0, tilt=0.0;
  //panTiltControl_->GetCurrentPositionRadiansThreaded(pan, tilt);
  panTiltControl_->GetCurrentPositionRadians(pan, tilt);
  pan = -pan; // HACK : Flip sign of pan angle!
  std::cout<<"Current Position: "<<pan*180.0/M_PI<<", "<<tilt*180.0/M_PI<< " degree"<<std::endl;

  // rotation matrix resulting from panning
  BIAS::RMatrix pan_matrix = BIAS::RMatrix(BIAS::Vector3<double>(0,1,0), pan);
  // rotation matrix resulting from tilting
  BIAS::RMatrix tilt_matrix = BIAS::RMatrix(BIAS::Vector3<double>(1,0,0), tilt);

  // new modelling of ptu pose:
  // first, pan ptu
  BIAS::Matrix4x4<double> pan_ptu = BIAS::Matrix4x4<double>(pan_matrix, BIAS::Vector3<double>(0,0,0));

  // tilt ptu
  BIAS::Matrix4x4<double> tilt_ptu = BIAS::Matrix4x4<double>(tilt_matrix, BIAS::Vector3<double>(0,0,0));

  // identity matrix
  BIAS::Matrix3x3<double> id33 = BIAS::Matrix3x3<double>(1,0,0, 0,1,0, 0,0,1);
  // translation from rotation joint to camera center
  BIAS::Vector3<double> translation(0.0, 0.0, 0.0);
  BIAS::Matrix4x4<double> trans_to_camera_center = BIAS::Matrix4x4<double>(id33, translation);

  // calculate complete ptu transformation matrix
  BIAS::Matrix4x4<double> ptu_result = tilt_ptu * pan_ptu * trans_to_camera_center;

  // do some hackish access to the translation part of resulting matrix
  BIAS::Vector4<double> translation_rotated4 = ptu_result.GetCol(3);
  BIAS::Vector3<double> translation_rotated3;
  double *translation_rotated4_ptr = translation_rotated4.GetData();
  for (int i=0;i<3;i++)
    translation_rotated3.GetData()[i] = translation_rotated4_ptr[i];

  // store transformation in dynamic data pool entry
  Pose p;
  Quaternion<double> panq;
  pan_matrix.GetQuaternion(panq);
  Quaternion<double> tiltq;
  tilt_matrix.GetQuaternion(tiltq);
  Quaternion<double> temp = panq * tiltq;
  p.SetC(translation_rotated3);
  p.SetQ(temp);
  KMatrix K;
  projHeader.CreatePerspective(p,K,640,480);
  cout<<"Projection:"<<projHeader<<endl;
}
#endif


/****************************************************
*****************************************************/
void ShowCamWxFrame::
StopSavingImagesToDisk()
{
  bWriteBuffered_=false;

  for (int i = 0; i < NUMBER_CAMERAS; i++){
    if(saveThreads_[i]!=NULL && Cameras_[i] != NULL){
      //      cout<<"Pausing camera :"<<i<<endl;
	    saveThreads_[i]->Delete();
      saveThreads_[i] = NULL;
      //cout << "\n\nSaveThread for cam: " << i << " paused\n\n" << endl;
    }
  }
  //for (int i = 0; i < NUMBER_CAMERAS; i++){
  //  if(saveThreads_[i]!=NULL && Cameras_[i] != NULL){
  //    saveThreads_[i]->FinishSaving();
  //  }
  //}
  cout << "leave StopSavingThreads" << endl;
}

/****************************************************
*****************************************************/
void ShowCamWxFrame::
StartSavingImagesToDisk()
{

  //try to create a new directory, skip of already exists
  int ret =-1;
  while(ret != 0){
    stringstream dir;
    dir <<"Take"<<FileHandling::LeadingZeroString(dDirIndex_,5)<<"/";
    currentSaveDir_ = dir.str().c_str();
    #ifdef WIN32
    ret = _mkdir(dir.str().c_str());
    #else //WIN32
    ret = mkdir(dir.str().c_str(), 0755);
    #endif //WIN32
    dDirIndex_++;
  }

  // important!! This triggers the write to buffer in threads!
  bWriteBuffered_ = true;

  //create a dummy Projection to save Pan Tilt angles in Image headers
  BIAS::Projection projHeader;

  #ifdef BIAS_HAVE_PANTILT_DP
  if(*bUsePanTiltUnit_){
    GetProjectionFromPanTilt_(projHeader);
  }
  #endif //BIAS_HAVE_PANTILT_DP

  for (int i = 0; i < NUMBER_CAMERAS; i++){
    // check if camera is opened
    if(Cameras_[i] != NULL){

      #ifdef BIAS_HAVE_PANTILT_DP
      if(*bUsePanTiltUnit_){
        (*CamImg_[i]).SetProj(projHeader);
        (*CamImg_[i]).UpdateMetaData();
      }
      #endif //BIAS_HAVE_PANTILT_DP

      if(saveThreads_[i] == NULL){
        //Here create ImageSaveThreadForCamera
        CreateImageSaveThreadForCamera_(i);
        saveThreads_[i]->SetSaveDirectory(currentSaveDir_);
      }
      else{
        // signal thread to save a single image
        saveThreads_[i]->SetSaveDirectory(currentSaveDir_);
		    saveThreads_[i]->ResetCounter();
        saveThreads_[i]->Resume();
      }
    }//end else

  }//end for all cameras
}

/****************************************************
*****************************************************/
void ShowCamWxFrame::
ParseColorModels()
{
  for (int i = 0; i < NUMBER_CAMERAS; i++) {
    switch(*ParamColorModel_[i])  {
    case 0:  ColorModels_[i] = ImageBase::CM_invalid; break;
    case 1:  ColorModels_[i] = ImageBase::CM_Grey; break;
    case 2:  ColorModels_[i] = ImageBase::CM_RGB; break;
    case 3:  ColorModels_[i] = ImageBase::CM_BGR; break;
    case 4:  ColorModels_[i] = ImageBase::CM_YUYV422; break;
    case 5:  ColorModels_[i] = ImageBase::CM_UYVY422; break;
    case 6:  ColorModels_[i] = ImageBase::CM_YUV420P; break;
    case 7:  ColorModels_[i] = ImageBase::CM_YUV444; break;
    case 8:  ColorModels_[i] = ImageBase::CM_YUV411; break;
    case 9:  ColorModels_[i] = ImageBase::CM_HSV; break;
    case 10: ColorModels_[i] = ImageBase::CM_HSI_OBS; break;
    case 11: ColorModels_[i] = ImageBase::CM_DV; break;
    case 12: ColorModels_[i] = ImageBase::CM_RGBA; break;
    case 13: ColorModels_[i] = ImageBase::CM_GreyA; break;
    case 14: ColorModels_[i] = ImageBase::CM_Bayer_RGGB; break;
    case 15: ColorModels_[i] = ImageBase::CM_Bayer_GBRG; break;
    case 16: ColorModels_[i] = ImageBase::CM_Bayer_GRBG; break;
    case 17: ColorModels_[i] = ImageBase::CM_Bayer_BGGR; break;
    case 18: ColorModels_[i] = ImageBase::CM_HSL; break;
    case 19: ColorModels_[i] = ImageBase::CM_hsL; break;
    case 20: ColorModels_[i] = ImageBase::CM_SymTensor2x2; break;
    case 21: ColorModels_[i] = ImageBase::CM_BGRA; break;
    case 22: ColorModels_[i] = ImageBase::CM_RGBE; break;
    default: ColorModels_[i] = ImageBase::CM_invalid; break;
    }
  }
}

/****************************************************
 *****************************************************/
void ShowCamWxFrame::
InitCombinedImage()
{
	// kinect, openni
	if (*ParamCameras_[NumCamSel_] == ShowCamWxVideoSourceFactory::KINECT || *ParamCameras_[NumCamSel_] == ShowCamWxVideoSourceFactory::OPENNI) {
    unsigned int w = CamImg_[NumCamSel_]->GetWidth()*2;
    unsigned int h = CamImg_[NumCamSel_]->GetHeight();
    if (!CamImgCombined_.IsEmpty()) {
      if (CamImgCombined_.GetColorModel() != ImageBase::CM_RGB
      		|| CamImgCombined_.GetWidth() != w || CamImgCombined_.GetHeight() != h)
      {
        CamImgCombined_.Release();
      }
    }
    if (CamImgCombined_.IsEmpty()) {
      CamImgCombined_.Init(w,h,3);
      CamImgCombined_.SetColorModel(ImageBase::CM_RGB);
    }
    BIAS::Camera<unsigned char> tmpRgb;
    ImageConvert::ToRGB(*CamImg_[NumCamSel_], tmpRgb);

    CamImgCombined_.SetROICorners(0, 0,
                                  CamImg_[NumCamSel_]->GetWidth(),
                                  CamImg_[NumCamSel_]->GetHeight());

    CamImgCombined_.Paste2ROI(tmpRgb);
    CamImgCombined_.SetROICorners(CamImg_[NumCamSel_]->GetWidth(),
                                  0,
                                  CamImg_[NumCamSel_]->GetWidth() * 2,
                                  CamImg_[NumCamSel_]->GetHeight());
    if(CamImgPMDDepth_[NumCamSel_] != NULL){
      BIAS::Camera<float> tmpImg;
      BIAS::Camera<unsigned char> tmpImgUC;
      tmpImg = *CamImgPMDDepth_[NumCamSel_];
      tmpImg.AboveThresholdToValue(7000,7000);
      tmpImg.BelowThresholdToValue(1000,1000);
      tmpImg.ScaleShiftBetween(0,255);
  
      BIAS::ImageConvert::ConvertST(tmpImg, tmpImgUC, ImageBase::ST_unsignedchar);
      BIAS::ImageConvert::ToRGB(tmpImgUC, tmpRgb);
      CamImgCombined_.Paste2ROI(tmpRgb);
    }
  }
	// kinect2
	else if (*ParamCameras_[NumCamSel_] == ShowCamWxVideoSourceFactory::FREENECT2) {
    unsigned int w = 1920;
    unsigned int h = 1080 + 424;
    if (!CamImgCombined_.IsEmpty()) {
      if (CamImgCombined_.GetColorModel() != ImageBase::CM_RGB
          || CamImgCombined_.GetWidth() != w || CamImgCombined_.GetHeight() != h)
      {
        CamImgCombined_.Release();
      }
    }
    if (CamImgCombined_.IsEmpty()) {
      CamImgCombined_.Init(w,h,3);
      CamImgCombined_.SetColorModel(ImageBase::CM_RGB);
    }
    CamImgCombined_.SetROICorners(0, 0, 1920, 1080);

    CamImgCombined_.Paste2ROI(*CamImg_[NumCamSel_]);

    BIAS::Camera<float> tmpImg;
    BIAS::Camera<unsigned char> tmpImgUC;
    BIAS::Camera<unsigned char> tmpRgb;

    CamImgCombined_.SetROICorners(0, 1080, 512, 1080 + 424);
    if(CamImgPMDDepth_[NumCamSel_] != NULL){
      tmpImg = *CamImgPMDDepth_[NumCamSel_];
      tmpImg.AboveThresholdToValue(7000,7000);
      tmpImg.BelowThresholdToValue(0,0);
      tmpImg.ScaleShiftBetween(0,255);
      BIAS::ImageConvert::ConvertST(tmpImg, tmpImgUC, ImageBase::ST_unsignedchar);
      BIAS::ImageConvert::ToRGB(tmpImgUC, tmpRgb);
      CamImgCombined_.Paste2ROI(tmpRgb);
    }
    CamImgCombined_.SetROICorners(512, 1080, 512+512, 1080 + 424);
    if(CamImgPMDInt_[NumCamSel_] != NULL){
      tmpImg = *CamImgPMDInt_[NumCamSel_];
      tmpImg.AboveThresholdToValue(20000,20000);
      tmpImg.BelowThresholdToValue(0,0);
      tmpImg.ScaleShiftBetween(0,255);
      BIAS::ImageConvert::ConvertST(tmpImg, tmpImgUC, ImageBase::ST_unsignedchar);
      BIAS::ImageConvert::ToRGB(tmpImgUC, tmpRgb);
      CamImgCombined_.Paste2ROI(tmpRgb);
    }
  }
	// pmd tec
  else {
    unsigned wImg,wAmp,wDepth,hImg,hAmp,hDepth;
    ClientsMutex_[NumCamSel_]->Lock();
    BIAS::Camera<float> imgAmp = *CamImgPMDAmp_[NumCamSel_];
    BIAS::Camera<float> imgDepth= *CamImgPMDDepth_[NumCamSel_];
    BIAS::Camera<unsigned char> img = *CamImg_[NumCamSel_];
    ClientsMutex_[NumCamSel_]->Unlock();
    wImg = img.GetWidth();
    hImg =  img.GetHeight();
    wAmp = imgAmp.GetWidth();
    hAmp =  imgAmp.GetHeight();
    wDepth = imgDepth.GetWidth();
    hDepth =  imgDepth.GetHeight();

    if(!CamImgCombined_.IsEmpty()) {
      CamImgCombined_.Release();
    }
    // initialize combined image from pmd usb camera images
    CamImgCombined_.Init(wImg + wAmp + wDepth, hImg);
    CamImgCombined_.SetColorModel(ImageBase::CM_Grey);
    if(wImg > 0 && hImg> 0)
    {
      CamImgCombined_.SetROICorners(0, 0, wImg,hImg);
      CamImgCombined_.Paste2ROI(img);
    }
  
    // add Amp image from pmd usb camera
    BIAS::Camera<unsigned char> tmpImgUC;
    if(!imgAmp.IsEmpty() && wAmp > 0 && hAmp > 0)
    {
      CamImgCombined_.SetROICorners(wImg, 0, wImg + wAmp, hAmp);
  
      imgAmp.ScaleShiftBetween(0,255);
      BIAS::ImageConvert::ConvertST(imgAmp, tmpImgUC, ImageBase::ST_unsignedchar);
      tmpImgUC.SetColorModel(ImageBase::CM_Grey);
      CamImgCombined_.Paste2ROI(tmpImgUC);     
    }
    // add Depth image from pmd usb camera
    if(!imgDepth.IsEmpty() && wDepth > 0 && hDepth > 0)
    {
      CamImgCombined_.SetROICorners(wImg + wAmp, 0,
                                    wImg + wAmp + wDepth, hDepth);
      
      imgDepth.AboveThresholdToValue(7000,7000);
      imgDepth.BelowThresholdToValue(1000,1000);
      imgDepth.ScaleShiftBetween(0,255);
  
      BIAS::ImageConvert::ConvertST(imgDepth, tmpImgUC, ImageBase::ST_unsignedchar);
      tmpImgUC.SetColorModel(ImageBase::CM_Grey);
      CamImgCombined_.Paste2ROI(tmpImgUC);
    }
  }
}

/****************************************************
 *****************************************************/
void ShowCamWxFrame::
OnSaveParameters(wxCommandEvent& event){

	wxString fileName;
	wxFileDialog dialog(this,wxT("Save Parameters"),wxT(""),wxT(""),
			wxT("MIP Parameter files (*.param)|*.param"),
                      wxFD_SAVE|wxFD_OVERWRITE_PROMPT);
  int ret = dialog.ShowModal();
  if(ret == wxID_OK){
    fileName = dialog.GetPath();
    WriteParameter(WxToAscii(fileName));
  }
}
/****************************************************
 *****************************************************/
void ShowCamWxFrame::
OnWindowClose(wxCloseEvent& event){
	//cout << "window close event!!!!!!!!" << endl;
	CleanUpAndExit();
	//Close(true);
	Destroy();
}

/****************************************************
 *****************************************************/
void ShowCamWxFrame::
CleanUpAndExit()
{
	if(!CleanedUp_){
		// stop timer
    Timer_.Stop();

    // close open controller windows
    for (unsigned int i = 0; i < VideoControllers_.size(); i++) {
      if (VideoControllers_[i] != NULL) {
        if (VideoControllers_[i]->IsShown()) {
          VideoControllers_[i]->Close();
          delete VideoControllers_[i];
        }
      }
    }

    // first delete threads while timer is running
    //cout << "start killing threads" << endl;
    for (int i = 0; i < NUMBER_CAMERAS; i++) {
      CloseDevice(i);
    }
    
    for (unsigned int i = 0; i < ServerMutex_.size();i++) {
      if(ServerMutex_[i] != NULL)
        delete ServerMutex_[i];
      ServerMutex_[i]=NULL;
      if(ServerCondition_[i] != NULL)
        delete ServerCondition_[i];
      ServerCondition_[i] = NULL;
      if(ClientsMutex_[i] != NULL)
        delete ClientsMutex_[i];
      ClientsMutex_[i]=NULL;
      if(ClientsCondition_[i] != NULL)
        delete ClientsCondition_[i];
      ClientsCondition_[i] = NULL;
    }

    #ifdef BIAS_HAVE_PANTILT_DP
    if(*bUsePanTiltUnit_){
      if(panTiltControlDialog_->IsShown())
        panTiltControlDialog_->Hide();
      delete panTiltControlDialog_;

      if(panTiltControl_ != NULL){
        panTiltControl_->SetPosition(0,0);
        panTiltControl_->ClosePanTiltUnit();
        delete panTiltControl_;
      }
    }
    #endif //BIAS_HAVE_PANTILT_DP

    CleanedUp_ = true;
  }

  cout << "Cleaned up successfully" << endl;
}

/****************************************************
 *****************************************************/
void ShowCamWxFrame::OnFit(wxCommandEvent& WXUNUSED(event))
{
	if (pCheckBoxKeepScrollPosition != NULL)
	{
		if (IC_!=NULL){
			IC_->KeepScrollPosition(pCheckBoxKeepScrollPosition->IsChecked());
		}         
	}
}

/****************************************************
 *****************************************************/
void ShowCamWxFrame::OnKeepScrollPosition(wxCommandEvent& WXUNUSED(event))
{
	if (pCheckBoxKeepScrollPosition != NULL)
	{
		if (IC_!=NULL){
			IC_->KeepScrollPosition(pCheckBoxKeepScrollPosition->IsChecked());
		}         
	}
}

/****************************************************
 *****************************************************/
void ShowCamWxFrame::
OnQuit(wxCommandEvent& event)
{
	CleanUpAndExit();
	//Close(true);
	Destroy();
}

/****************************************************
 *****************************************************/
void ShowCamWxFrame::
OnGrab(wxCommandEvent& event)
{
	if(!bSingleSaveDirCreated_){
		int ret =-1;

    while(ret != 0){
      stringstream dir;
      dir <<"SingleTake"<<FileHandling::LeadingZeroString(dSingleDirIndex_,5)<<"/";
      currentSingleSaveDir_ = dir.str().c_str();

#ifdef WIN32
      ret = _mkdir(dir.str().c_str());
#else //WIN32
      ret = mkdir(dir.str().c_str(), 0755);
#endif //WIN32

      dSingleDirIndex_++;
    }
    bSingleSaveDirCreated_ = true;
  }

  for (int i = 0; i < NUMBER_CAMERAS; i++){
    if(Cameras_[i] != NULL){
      SaveImageUnthreaded_(i);
    }
  }
  GrabCounter_++;
}

/****************************************************
 *****************************************************/
void ShowCamWxFrame::
OnGrabContinuous(wxCommandEvent& event)
{
  // reset counter and enable/disable continuous saving of images
  FrameCounter_ = 0;
  bSaveImages_ = !bSaveImages_;
  bSaveToStream_ = false;
  if (bSaveImages_){
    StartSavingImagesToDisk();
    pButtonContinuous->SetLabel(_T("Stop"));
  }
  else{
    StopSavingImagesToDisk();
    pButtonContinuous->SetLabel(_T("Grab Cont."));
  }
}
/****************************************************
*****************************************************/
void ShowCamWxFrame::
OnGrabToStream(wxCommandEvent& event)
{
	// reset counter and enable/disable continuous saving of images
	FrameCounter_ = 0;
	bSaveImages_ = !bSaveImages_;
	bSaveToStream_ = true;
	if (bSaveImages_){
		StartSavingImagesToDisk();
		pButtonStream->SetLabel(_T("Stop"));
	}
	else{
		StopSavingImagesToDisk();
		pButtonStream->SetLabel(_T("Grab to Stream"));
	}
}

/****************************************************
*****************************************************/
void ShowCamWxFrame::OnClickCamerasListBox(wxCommandEvent& event){
	int sel = event.GetSelection();
	if(sel < NumOfCams_)
		NumCamSel_ = sel;
}
/****************************************************
*****************************************************/
void ShowCamWxFrame::OnHideCamerasListBox(wxCommandEvent& event){
	splitterWindow_->Unsplit(cameraListPanel_);
}
/****************************************************
*****************************************************/
void ShowCamWxFrame::OnShowCamerasListBox(wxCommandEvent& event){
	//split between notebook and ImagelistPanel
	splitterWindow_->SplitVertically(IC_,cameraListPanel_,-1);
}

/****************************************************
 *****************************************************/
void ShowCamWxFrame::
OnShowImage(wxCommandEvent& event)
{
	NumCamSel_ = event.GetId() - ID_Show1;
}


/****************************************************
 *****************************************************/
void ShowCamWxFrame::
OnShowController(wxCommandEvent& event)
{
	// check if camera is initialized
	if (Cameras_[NumCamSel_] != NULL) {
		if (VideoControllers_[NumCamSel_] == NULL) {
			VideoControllers_[NumCamSel_] =
					ShowCamWxVideoSourceFactory::CreateController((ShowCamWxVideoSourceFactory::CameraType)(*ParamCameras_[NumCamSel_]), Cameras_[NumCamSel_], this);
			if (VideoControllers_[NumCamSel_] == NULL) {
				BIASERR("Controller could not be created");
			} else {
				VideoControllers_[NumCamSel_]->Show(true);
			}
		} else {
			VideoControllers_[NumCamSel_]->Show(true);
		}
	}
	// camera was not initalized
	else {
		wxMessageDialog errorDialog(this,
				wxT("No correct camera was selected"),
				wxT("ShowCamWx - Error"),
				wxICON_ERROR|wxOK);
		errorDialog.ShowModal();
	}
}

/****************************************************
 *****************************************************/
void ShowCamWxFrame::
OnTimer(wxTimerEvent& event)
{
  //check if we opened all devices and started the grabthreads
  if(!OpenedDevices_){
    return;
  }
  if(grabMasterThread_ != NULL){
    double fps = grabMasterThread_->fps_;
    FrameCounter_ = grabMasterThread_->FrameCounter_;
    wxString text;
    text.Printf(wxT("%05d %2.1f fps "),FrameCounter_, fps);
    SetStatusText(text, 2);
    SelectAndShowCameraImage_();
  }
  Timer_.Start(Delay_);
}

/****************************************************
*****************************************************/
void ShowCamWxFrame::
OnOpenFlirController(wxCommandEvent& event){
	if(tauControllerFrame_ == NULL){
		tauControllerFrame_ = new wxTauControlFrame(this);
		tauControllerFrame_->Show();
	}
	else{
		tauControllerFrame_->Show();
	}
}
/****************************************************
*****************************************************/
void ShowCamWxFrame::
OnOpenViscaController(wxCommandEvent& event){

#ifdef BIAS_HAVE_PANTILT
	if(viscaControllerFrame_ == NULL){
		viscaControllerFrame_ = new wxViscaControlFrame(this);
		viscaControllerFrame_->Show();
	}
	else{
		viscaControllerFrame_->Show();
	}
#endif
}


 /****************************************************
  *****************************************************/
void ShowCamWxFrame::OnFindIEEE1394(wxCommandEvent& event) {
#ifdef BIAS_HAVE_DCAM
  NumIEEECams_ = BIAS::ScanBus(IEEECameraInfo_);

  if (NumIEEECams_ > 0) {
    stringstream text;
    text << "Found " << NumIEEECams_ << " IEEE1394 Cameras: " << endl << endl;

    for (int i = 0; i < NumIEEECams_; i++) {
      text << "Camid: " << IEEECameraInfo_[i].camid << " "
           << IEEECameraInfo_[i].vendor << " " << IEEECameraInfo_[i].model
           << endl;
    }
    text<<"\nTrigger them?"<<endl;
    wxMessageDialog messageDialog(this, AsciiToWx(text.str().c_str()), wxT("Trigger- IEEE Cameras?"), 
    		wxYES_NO | wxNO_DEFAULT| wxICON_QUESTION);   
    bool trigger = false;
    if(messageDialog.ShowModal() == wxID_YES){
    	trigger = true;
    }
    
    for (int i = 0; i < NumIEEECams_; i++) {
    	// check if camera already in Menu...
    	bool inUse = false;
    	for(int j = 0; j < NUMBER_CAMERAS; j++){
    		if(IEEECameraInfo_[i].camid == atoi((*ParamDevice_[j]).c_str())){
    			inUse = true;
    		}
    	}
    	SetTrigger_[NumOfCams_] = trigger;
    	// here we have to add the cameras to the menu.
    	if(!inUse)
    		AddIEEECameraToMenu(NumOfCams_,i,false);    		    
    }
    
  } else {
    wxMessageDialog messageDialog(this, wxT("Sorry, no IEEE Cameras found!!"), wxT("ShowCamWx - IEEE Camera Scan"), wxICON_ERROR | wxOK);
    messageDialog.ShowModal();
  }
#else
  wxMessageDialog messageDialog(this, wxT("Sorry, DCAM support not activated in BIAS"), wxT("ShowCamWx - Camera Scan"), wxICON_ERROR | wxOK);
  messageDialog.ShowModal();
#endif

}

/****************************************************
 *****************************************************/
void ShowCamWxFrame::OnFindWebcams(wxCommandEvent& event) {
#ifdef BIAS_HAVE_V4L
  int ret = 0;
  VideoSource* vs = NULL;
	vs = new VideoSource_V4L;
	string device = "";
  int channel =0;

	vector<string> devices;
	((VideoSource_V4L*)vs)->GetAllDevices(devices);

	if (devices.size()==0) {
		//Dialogbox
		wxMessageDialog messageDialog(this, wxT("Sorry, no V4L devices found!!"), wxT("V4L Scan Error"), wxICON_ERROR | wxOK);
		messageDialog.ShowModal();
		delete vs;
		return;
	}
	else if(devices.size()>1)
	{
		device = devices[0];
		// show device selector if more than one entry in devices.
		wxArrayString array;
		stringstream helptext;
		helptext << "Select Device: ";
		for (unsigned int k = 0; k< devices.size(); k++) {
			array.Add(AsciiToWx(devices[k].c_str()));
		}


		wxSingleChoiceDialog choiceDialog(this, wxT("Please choose device"),
#if wxCHECK_VERSION(2, 9, 0)
				wxT("Choose Device"), array, (void**)NULL, wxDEFAULT_DIALOG_STYLE
#else
        wxT("Choose Device"), array, NULL, wxDEFAULT_DIALOG_STYLE
#endif
				| wxRESIZE_BORDER | wxOK | wxCANCEL| wxCENTRE);
		choiceDialog.SetSelection(0);
		ret  = choiceDialog.ShowModal();
		if(ret == wxID_OK) {
			device = WxToAscii(array.Item(choiceDialog.GetSelection()));
		}
		else{
			delete vs;
			return;
		}
	}

	cout <<"Using: "<<device<<endl;
	VideoSourceCapabilities caps;
	if ( vs->GetCapabilities(device.c_str(),caps) == 0 ) {
		// show selector for Input (composite 0-2, s-video)
		//caps.Show();
		vs->GetDeviceChannelNames(DeviceChannelNames_[NumOfCams_]);

		if(DeviceChannelNames_[NumOfCams_].size()>0){
			wxArrayString array;
			stringstream helptext;
			helptext << "Select Input Channel: ";
			for (unsigned int k = 0; k< DeviceChannelNames_[NumOfCams_].size(); k++) {
				array.Add(AsciiToWx(DeviceChannelNames_[NumOfCams_][k].c_str()));
			}

			wxSingleChoiceDialog choiceDialog(this, wxT("Please choose channel"),
#if wxCHECK_VERSION(2, 9, 0)
					wxT("Choose Channel"), array, (void**)NULL, wxDEFAULT_DIALOG_STYLE
#else
          wxT("Choose Channel"), array, NULL, wxDEFAULT_DIALOG_STYLE
#endif
					| wxRESIZE_BORDER | wxOK | wxCANCEL| wxCENTRE);
			choiceDialog.SetSelection(0);
			ret  = choiceDialog.ShowModal();
			if(ret == wxID_OK){
				channel = choiceDialog.GetSelection();
			}
			else{
				delete vs;
				return;
			}
		}
		////////////////////////////////////////////////
		vector<VideoSourceCapabilities::ResolutionEntry> re;
		caps.GetBestForEachResolution(re);
		for (unsigned int i=0;i<re.size();i++){
			cout <<"  Best for "<<re[i].width<<"x"<<re[i].height<<" is "
			<<re[i].colormodel<<" at fps: "<<re[i].fps<<endl;
		}
	}
	else {
		//Dialogbox
		wxMessageDialog messageDialog(this, wxT("V4L device could not be queried!!"),
      wxT("V4L Scan Error"), wxICON_ERROR | wxOK);
		messageDialog.ShowModal();
	}

	//cout <<"Using Device Channel: "<<channel<<endl;

	DeviceChannel_[NumOfCams_] = channel;
	vs->SetDeviceChannel(channel);
	ret = vs->OpenDevice(device.c_str());

	if (ret != 0) {
		//Dialogbox
		wxMessageDialog messageDialog(this, wxT("Sorry, v4l camera could not be opened!!"),
      wxT("V4L Camera Scan"), wxICON_ERROR | wxOK);
		messageDialog.ShowModal();
		delete vs;
		return;
	}
	else
	{
		stringstream text;
		text <<"Found "<<devices.size()<<" V4L Cameras.\n"<<
				"Opening first camera by default.\n\n";

		for (unsigned i = 0; i < devices.size(); i++)
			text << "Id: " << devices[i] << endl;
		text<<"\nTrigger them?"<<endl;

		wxMessageDialog messageDialog(this, AsciiToWx(text.str().c_str()), wxT("Trigger- IEEE Cameras?"),
				wxYES_NO | wxNO_DEFAULT| wxICON_QUESTION);
		bool trigger = false;
		if(messageDialog.ShowModal() == wxID_YES){
			trigger = true;
			SetTrigger_[NumOfCams_] = trigger;
		}

		vs->CloseDevice();
		delete vs;
		// Add camera and open
		AddWebcamToMenu(device,NumOfCams_, false);
	}
#endif

#ifdef BIAS_HAVE_DSHOW
  int ret = 0;
  VideoSource* vs = NULL;
  vs = new VideoSource_DSHOW;
  vector<string> devices;
  vs->GetAllDevices(devices);

  stringstream text;
  if (devices.size() > 0) {
  	text << "Found " << devices.size() << "DirectShow Cameras: " << endl << endl;
  	for (unsigned int i = 0; i < devices.size(); i++) {
  		text << "Identifier: " << devices[i]<<endl;
  	}
  }
  else{
  	text <<"Sorry no DirectShow Cameras found."<<endl;
  }
  wxArrayString devStrings;
  for(unsigned i=0;i<devices.size();i++){
  	cout<<"Found device:"<<devices[i]<<endl;
  	devStrings.Add(AsciiToWx(devices[i]));
  }

  string camera;
  // ask for depth error model
  wxSingleChoiceDialog wcbDlg(this, wxT("Select camera."),
  		wxT("DShow Cameras"), devStrings);
  wcbDlg.SetSelection(0);
  ret  = wcbDlg.ShowModal();
  if(ret == wxID_OK){
  	camera = WxToAscii(devStrings.Item(wcbDlg.GetSelection()));
  }
  else{
  	delete vs;
  	return;
  }

  VideoSourceCapabilities caps;
  vs->GetCapabilities(caps);
  caps.Show();
  vector<VideoSourceCapabilities::ResolutionEntry> re;
  caps.GetBestForEachResolution(re);
  wxArrayString resStrings;
  for (unsigned int i=0;i<re.size();i++){
    stringstream txt;
    txt<<re[i].width<<"x"<<re[i].height<<" with "<<re[i].colormodel;
    resStrings.Add(AsciiToWx(txt.str()));
  }

  // ask for resolution
  int index = -1;
  wxSingleChoiceDialog resDlg(this, wxT("Select resolution."),
		  wxT("Available Configurations"),resStrings);
  resDlg.SetSelection(0);
  ret  = resDlg.ShowModal();
  if(ret == wxID_OK){
	  index = resDlg.GetSelection();
  } 
  else {
  	delete vs;
  	return;
  }

  *ParamWidth_[NumOfCams_] = re[index].width;
  *ParamHeight_[NumOfCams_] = re[index].height;
  *ParamColorModel_[NumOfCams_] = re[index].colormodel;

  vs->SetSize(re[index].width,re[index].height);
  vs->SetColorModel(re[index].colormodel);

  ret = vs->OpenDevice(camera.c_str());
  if (ret!=0) {
    wxMessageDialog messageDialog(this, wxT("Sorry, DSHOW camera could not be opened!!"),
      wxT("DShow Camera Scan"), wxICON_ERROR | wxOK);
		messageDialog.ShowModal();
		delete vs;
    return;
  } 

  vs->CloseDevice();
  for(unsigned i=0;i<devices.size();i++){
    *ParamDevice_[NumOfCams_]=devices[i];
    AddWebcamToMenu(devices[i], NumOfCams_, false);
  }
#endif
}

 /****************************************************
  *****************************************************/
void ShowCamWxFrame::OnFindUEye(wxCommandEvent& event){
  // simply try to open ueye
#ifdef BIAS_HAVE_UEYE

  VideoSource_uEye ueye;
  std::vector<std::string> ret = ueye.GetDevices();

  if (ret.size() == 0) {
    //Dialogbox
    wxMessageDialog messageDialog(this, wxT("Sorry, UEYE not found!!"), wxT("ShowCamWx - Camera Scan"), wxICON_ERROR | wxOK);
    messageDialog.ShowModal();
  } else {
    for (unsigned int i=0;i<ret.size();i++) {
      // Add camera and open
      AddUEyeCameraToMenu(NumOfCams_, ret[i], false);
    }
  }
#else
  wxMessageDialog messageDialog(this, wxT("Sorry, UEYE support not activated in BIAS"), wxT("ShowCamWx - Camera Scan"), wxICON_ERROR | wxOK);
  messageDialog.ShowModal();
#endif

}
/****************************************************
*****************************************************/
void ShowCamWxFrame::OnFindPMDTec(wxCommandEvent& event){
  // simply try to open pmdtec
#ifdef BIAS_HAVE_PMD_PMDTec
  Cameras_[NumOfCams_] = new VideoSource_PMD;
  *ParamCameras_[NumOfCams_] = 2;

  wxTwoFilesInputDialog *dlg = new wxTwoFilesInputDialog(this);
  dlg->SetString1(AsciiToWx(BIAS_PMD_PMDTec_SOURCEPLUGIN));
  dlg->SetString2(AsciiToWx(BIAS_PMD_PMDTec_PROCPLUGIN));
  int ret = dlg->ShowModal();
  if(ret == wxID_OK){
    sourcePlugin_ = WxToAscii(dlg->GetString1());
    procPlugin_ = WxToAscii(dlg->GetString2());
  }
  else{
    sourcePlugin_ =BIAS_PMD_PMDTec_SOURCEPLUGIN;
    procPlugin_= BIAS_PMD_PMDTec_PROCPLUGIN;
    return;
  }

  ret = OpenDevice(NumOfCams_);
  if (ret != 0) {
    //Dialogbox
    wxMessageDialog messageDialog(this, wxT("Sorry, PMDTec Camera not found!!"), wxT("ShowCamWx - Camera Scan"), wxICON_ERROR | wxOK);
    messageDialog.ShowModal();
    delete Cameras_[NumOfCams_];
  } else {
    // Add camera and open
    AddPMDTecCameraToMenu(NumOfCams_, true);
  }
#else
  wxMessageDialog messageDialog(this, wxT("Sorry, PMDTec support not activated in BIAS"), wxT("ShowCamWx - Camera Scan"), wxICON_ERROR | wxOK);
  messageDialog.ShowModal();
#endif

}


 /****************************************************
  *****************************************************/
void ShowCamWxFrame::OnFindPMDZess(wxCommandEvent& event){
#ifdef BIAS_HAVE_PMD_PMDZess

  VideoSource_PMDZess* vs = new VideoSource_PMDZess;
  //ask for bitfile
  wxTextEntryDialog dialog (this,wxT("Please insert path to bitfile,\n else ShowCamWx will try to open default file"),wxT("Searching for PMD Zess"),wxT(""));
  dialog.ShowModal();

  wxString wxStr = dialog.GetValue();
  if(wxStr.empty())
    wxStr = "C:\\ExternLibs\\PMDZess\\Bitfiles\\default.rbt";

  vs->SetBitFile(wxStr.c_str());

  int ret = vs->OpenDevice();

  if(ret != 0){
    wxMessageDialog messageDialog(this, wxT("Sorry, PMD Zess not found!!"), wxT("ShowCamWx - Camera Scan"), wxICON_ERROR | wxOK);
    messageDialog.ShowModal();
    delete vs;
  }else{
    vs->CloseDevice();
    delete vs;

    // Add camera and open
    AddZessCameraToMenu(NumOfCams_, false,wxStr.c_str());
  }
#else
  wxMessageDialog messageDialog(this, wxT("Sorry, PMD Zess support not activated in BIAS"), wxT("ShowCamWx - Camera Scan"), wxICON_ERROR | wxOK);
  messageDialog.ShowModal();
#endif //BIAS_HAVE_PMD_PMDZess
}


 /****************************************************
  *****************************************************/
void ShowCamWxFrame::OnFindSwissRanger(wxCommandEvent& event){
#ifdef BIAS_HAVE_SWISSRANGER
  Cameras_[NumOfCams_] = new VideoSource_SwissRanger;
  *ParamCameras_[NumOfCams_] = 9;
  stringstream message;
  message<<"Trigger Swissranger?";
  wxMessageDialog dlg (this, wxT("Trigger Swissranger?"),
  		AsciiToWx(message.str().c_str()),
  		wxYES_NO | wxNO_DEFAULT| wxICON_QUESTION);
  if(dlg.ShowModal() == wxID_YES){  	
  	SetTrigger_[NumOfCams_] = true;
  }
  else{
  	SetTrigger_[NumOfCams_] = false;
  }
  	
  
  int ret = OpenDevice(NumOfCams_);

  if (ret != 0) {
    //Dialogbox
    wxMessageDialog messageDialog(this, wxT("Sorry, SwissRanger not found!!"), wxT("ShowCamWx - Camera Scan"), wxICON_ERROR | wxOK);
    messageDialog.ShowModal();
    delete Cameras_[NumOfCams_];
  } else {
    // Add camera and open
    AddSwissRangerCameraToMenu(NumOfCams_, true);
  }
#else
  wxMessageDialog messageDialog(this, wxT("Sorry, SwissRanger support not activated in BIAS"), wxT("ShowCamWx - Camera Scan"), wxICON_ERROR | wxOK);
  messageDialog.ShowModal();
#endif

}


/****************************************************
 *****************************************************/
void ShowCamWxFrame::OnFindKinect(wxCommandEvent& event){
#ifdef BIAS_HAVE_KINECT
 Cameras_[NumOfCams_] = new VideoSource_Kinect;
 *ParamCameras_[NumOfCams_] = 12;
 SetTrigger_[NumOfCams_] = false;
 
 int ret = OpenDevice(NumOfCams_);

 if (ret != 0) {
   //Dialogbox
   wxMessageDialog messageDialog(this, wxT("Sorry, Kinect not found!!"), wxT("ShowCamWx - Camera Scan"), wxICON_ERROR | wxOK);
   messageDialog.ShowModal();
   delete Cameras_[NumOfCams_];
 } else {
   // Add camera and open
   AddKinectCameraToMenu(NumOfCams_, true);
 }
#else
 wxMessageDialog messageDialog(this, wxT("Sorry, Kinect support not activated in BIAS"), wxT("ShowCamWx - Camera Scan"), wxICON_ERROR | wxOK);
 messageDialog.ShowModal();
#endif

}
/****************************************************
*****************************************************/
void ShowCamWxFrame::OnFindOpenNI(wxCommandEvent& event){
#ifdef BIAS_HAVE_OPENNI
  VideoSource_OpenNI tester;
  
  for (int i=0;i<tester.GetNumDevices();i++) {
    *ParamCameras_[NumOfCams_+i] = ShowCamWxVideoSourceFactory::OPENNI;
    stringstream ss;
    ss << i;
    *ParamDevice_[NumOfCams_+i] = ss.str();
    Cameras_[NumOfCams_+i] = new VideoSource_OpenNI;
    SetTrigger_[NumOfCams_+i] = false;
  }

  for (int i=0;i<tester.GetNumDevices();i++) {
   int ret = OpenDevice(NumOfCams_);
  
   if (ret != 0) {
     //Dialogbox
     wxMessageDialog messageDialog(this, wxT("Sorry, OpenNI not found!!"), wxT("ShowCamWx - Camera Scan"), wxICON_ERROR | wxOK);
     messageDialog.ShowModal();
     delete Cameras_[NumOfCams_];
   } else {
     // Add camera and open
     AddOpenNICameraToMenu(NumOfCams_, true);
   }
  }
#else
 wxMessageDialog messageDialog(this, wxT("Sorry, OpenNI support not activated in BIAS"), wxT("ShowCamWx - Camera Scan"), wxICON_ERROR | wxOK);
 messageDialog.ShowModal();
#endif

}

void ShowCamWxFrame::OnFindKinect2(wxCommandEvent& event){
#ifdef BIAS_HAVE_KINECT2
  VideoSource_Kinect2 tester;

  for (int i=0;i<tester.GetNumDevices();i++) {
    *ParamCameras_[NumOfCams_+i] = ShowCamWxVideoSourceFactory::FREENECT2;
    stringstream ss;
    ss << i;
    *ParamDevice_[NumOfCams_+i] = ss.str();
    Cameras_[NumOfCams_+i] = new VideoSource_Kinect2;
    SetTrigger_[NumOfCams_+i] = false;
  }

  for (int i=0;i<tester.GetNumDevices();i++) {
   int ret = OpenDevice(NumOfCams_);

   if (ret != 0) {
     //Dialogbox
     wxMessageDialog messageDialog(this, wxT("Sorry, OpenNI not found!!"), wxT("ShowCamWx - Camera Scan"), wxICON_ERROR | wxOK);
     messageDialog.ShowModal();
     delete Cameras_[NumOfCams_];
   } else {
     // Add camera and open
     AddKinect2CameraToMenu(NumOfCams_, true);
   }
  }
#else
 wxMessageDialog messageDialog(this, wxT("Sorry, OpenNI support not activated in BIAS"), wxT("ShowCamWx - Camera Scan"), wxICON_ERROR | wxOK);
 messageDialog.ShowModal();
#endif

}


/****************************************************
*****************************************************/
#ifdef BIAS_HAVE_SWISSRANGER
void ShowCamWxFrame::AddSwissRangerCameraToMenu(int camindex, bool fromFile){
  wxMenu* menuItem = new wxMenu;

  int id = (camindex + 1) * 10000;
  LastCameraID_ = id;

  id++;
  //menuItem->AppendSeparator();
  menuItem->AppendCheckItem(id,wxT("Activate Camera"));
  // connnect with eventhandler
  Connect(id,wxEVT_COMMAND_MENU_SELECTED,
          wxCommandEventHandler(ShowCamWxFrame::OnChangeActivate));
  //menuItem->Append(subMenuItem);
  menuItem->Check(id, true);
  
  id++;
  menuItem->AppendCheckItem(id,wxT("Trigger Camera"));
  Connect(id,wxEVT_COMMAND_MENU_SELECTED,
            wxCommandEventHandler(ShowCamWxFrame::OnChangeTrigger));   
  menuItem->Check(id, SetTrigger_[camindex]);
    
  CameraMenu_->Append(LastCameraID_,wxT("SwissRanger"),menuItem);
  if(!fromFile){
    *ParamCameras_[camindex] = 9;
    //OpenDevice(camindex);
  }

  NumOfCams_++;
}
#endif

/****************************************************
*****************************************************/
#ifdef BIAS_HAVE_KINECT
void ShowCamWxFrame::AddKinectCameraToMenu(int camindex, bool fromFile){
  wxMenu* menuItem = new wxMenu;

  int id = (camindex + 1) * 10000;
  LastCameraID_ = id;

  id++;
  //menuItem->AppendSeparator();
  menuItem->AppendCheckItem(id,wxT("Activate Camera"));
  // connnect with eventhandler
  Connect(id,wxEVT_COMMAND_MENU_SELECTED,
          wxCommandEventHandler(ShowCamWxFrame::OnChangeActivate));
  //menuItem->Append(subMenuItem);
  menuItem->Check(id, true);
  
  CameraMenu_->Append(LastCameraID_,wxT("Kinect"),menuItem);
  if(!fromFile){
    *ParamCameras_[camindex] = 12;
    //OpenDevice(camindex);
  }

  NumOfCams_++;
}
#endif
/****************************************************
*****************************************************/
#ifdef BIAS_HAVE_OPENNI
void ShowCamWxFrame::AddOpenNICameraToMenu(int camindex, bool fromFile){
  wxMenu* menuItem = new wxMenu;

  int id = (camindex + 1) * 10000;
  LastCameraID_ = id;

  id++;
  //menuItem->AppendSeparator();
  menuItem->AppendCheckItem(id,wxT("Activate Camera"));
  // connnect with eventhandler
  Connect(id,wxEVT_COMMAND_MENU_SELECTED,
          wxCommandEventHandler(ShowCamWxFrame::OnChangeActivate));
  //menuItem->Append(subMenuItem);
  menuItem->Check(id, true);
  
  CameraMenu_->Append(LastCameraID_,wxT("OpenNI"),menuItem);
  if(!fromFile){
    *ParamCameras_[camindex] = 13;
    //OpenDevice(camindex);
  }

  NumOfCams_++;
}
#endif

#ifdef BIAS_HAVE_KINECT2
void ShowCamWxFrame::AddKinect2CameraToMenu(int camindex, bool fromFile){
  wxMenu* menuItem = new wxMenu;

  int id = (camindex + 1) * 10000;
  LastCameraID_ = id;

  id++;
  //menuItem->AppendSeparator();
  menuItem->AppendCheckItem(id,wxT("Activate Camera"));
  // connnect with eventhandler
  Connect(id,wxEVT_COMMAND_MENU_SELECTED,
          wxCommandEventHandler(ShowCamWxFrame::OnChangeActivate));
  //menuItem->Append(subMenuItem);
  menuItem->Check(id, true);

  CameraMenu_->Append(LastCameraID_,wxT("Kinect2"),menuItem);
  if(!fromFile){
    *ParamCameras_[camindex] = ShowCamWxVideoSourceFactory::FREENECT2;
    //OpenDevice(camindex);
  }

  NumOfCams_++;
}
#endif


#ifdef BIAS_HAVE_PMD_PMDTec
void ShowCamWxFrame::AddPMDTecCameraToMenu(int camindex, bool fromFile){
  wxMenu* menuItem = new wxMenu;

  int id = (camindex + 1) * 10000;
  LastCameraID_ = id;

  id++;
  //menuItem->AppendSeparator();
  menuItem->AppendCheckItem(id,wxT("Activate Camera"));


  // connnect with eventhandler
  Connect(id,wxEVT_COMMAND_MENU_SELECTED,
          wxCommandEventHandler(ShowCamWxFrame::OnChangeActivate));
  //menuItem->Append(subMenuItem);
  menuItem->Check(id, true);

  id++;
  menuItem->AppendCheckItem(id,wxT("Trigger Camera"));
  Connect(id,wxEVT_COMMAND_MENU_SELECTED,
  		wxCommandEventHandler(ShowCamWxFrame::OnChangeTrigger));   
  menuItem->Check(id, SetTrigger_[camindex]);
  
  CameraMenu_->Append(LastCameraID_,wxT("PMDTec Camera"),menuItem);
  if(!fromFile){
    *ParamCameras_[camindex] = 2;
    //OpenDevice(camindex);
  }

  NumOfCams_++;
}
#endif

/****************************************************
*****************************************************/
#ifdef BIAS_HAVE_PMD_PMDZess
void ShowCamWxFrame::AddZessCameraToMenu(int camindex, bool fromFile, std::string paramFile){
  wxMenu* menuItem = new wxMenu;

  int id = (camindex + 1) * 10000;
  LastCameraID_ = id;

  id++;
  //menuItem->AppendSeparator();
  menuItem->AppendCheckItem(id,wxT("Activate Camera"));


  // connnect with eventhandler
  Connect(id,wxEVT_COMMAND_MENU_SELECTED,
          wxCommandEventHandler(ShowCamWxFrame::OnChangeActivate));
  //menuItem->Append(subMenuItem);
  menuItem->Check(id, true);

  id++;
  menuItem->AppendCheckItem(id,wxT("Trigger Camera"));
  Connect(id,wxEVT_COMMAND_MENU_SELECTED,
  		wxCommandEventHandler(ShowCamWxFrame::OnChangeTrigger));   
  menuItem->Check(id, SetTrigger_[camindex]);
  
  CameraMenu_->Append(LastCameraID_,wxT("PMD Zess"),menuItem);
  if(!fromFile){
    *ParamCameras_[camindex] = 3;
    *ParamBitFile_[camindex] = paramFile;
    OpenDevice(camindex);
  }

  NumOfCams_++;
}
#endif //BIAS_HAVE_PMD_PMDZess


/****************************************************
*****************************************************/
void ShowCamWxFrame::AddUEyeCameraToMenu(int camindex, std::string camId, bool fromFile){
  wxMenu* menuItem = new wxMenu;

  int id = (camindex + 1) * 10000;
  LastCameraID_ = id;
  // int ckid = 0;

  // add videomodes
  for (int i = 0; i < 6; i++) {
    stringstream ss;

    id++;
    switch (i) {
    case 0:
      ss << "1.875 FPS";
      break;

    case 1:
      ss << "3.75 FPS";
      break;

    case 2:
      ss << "7.5 FPS";
      break;

    case 3:
      ss << "15 FPS";
      break;

    case 4:
      ss << "30 FPS";
      break;

    case 5:
      ss << "60 FPS";
      break;

    default:
      break;
    }

    menuItem->AppendRadioItem(id,AsciiToWx(ss.str().c_str()));
    // connnect with eventhandler
    Connect(id, wxEVT_COMMAND_MENU_SELECTED,
            wxCommandEventHandler(ShowCamWxFrame::OnChangeVideoMode));

  }
  menuItem->Check(id, true);

  id++;
  menuItem->AppendSeparator();
  menuItem->AppendCheckItem(id,wxT("Activate Camera"));


  // connnect with eventhandler
  Connect(id,wxEVT_COMMAND_MENU_SELECTED,
          wxCommandEventHandler(ShowCamWxFrame::OnChangeActivate));

  menuItem->Check(id, true);

  id++;
  menuItem->AppendCheckItem(id,wxT("Trigger Camera"));
  Connect(id,wxEVT_COMMAND_MENU_SELECTED,
  		wxCommandEventHandler(ShowCamWxFrame::OnChangeTrigger));   
  menuItem->Check(id, false);
  
  CameraMenu_->Append(LastCameraID_,wxT("UEYE"),menuItem);
  if(!fromFile){
    *ParamCameras_[camindex] = 5;
    *ParamDevice_[camindex] = camId;
    OpenDevice(camindex);
  }

  NumOfCams_++;
}

/****************************************************
*****************************************************/
void ShowCamWxFrame::AddWebcamToMenu(string name, int camindex, bool fromFile){
  wxMenu* menuItem = new wxMenu;
  //cout<<"Adding camera with camindex:"<<camindex<<" to menu."<<endl;
  int id = (camindex + 1) * 10000;
  LastCameraID_ = id;
  int ckid = -1;
  for(int i=0;i<(int)DeviceChannelNames_[camindex].size();i++){
  	//increase
  	id++;
  	if(DeviceChannel_[camindex] == i) ckid = id;

  	string s= DeviceChannelNames_[camindex][i];
  	// check possible framerates
  	stringstream helptext; helptext<<s;
  	menuItem->AppendRadioItem(id,AsciiToWx(s.c_str()),AsciiToWx(helptext.str().c_str()));
  	// connnect with eventhandler
  	Connect(id, wxEVT_COMMAND_MENU_SELECTED,wxCommandEventHandler(ShowCamWxFrame::OnChangeDeviceChannel));
  }
  if(ckid!= -1){
    menuItem->Check(ckid, true);
    menuItem->AppendSeparator();
  }
  //add activate Camera to menu
  id++;
  menuItem->AppendCheckItem(id,wxT("Activate Camera"));
  // connnect with eventhandler
  Connect(id,wxEVT_COMMAND_MENU_SELECTED,
          wxCommandEventHandler(ShowCamWxFrame::OnChangeActivate));
  menuItem->Check(id, false);

  //add Trigger to menu
  id++;
  menuItem->AppendCheckItem(id,wxT("Trigger Camera"));
  Connect(id,wxEVT_COMMAND_MENU_SELECTED,
  		wxCommandEventHandler(ShowCamWxFrame::OnChangeTrigger));   
  menuItem->Check(id, SetTrigger_[camindex]);
  
  //append camera to menu
  CameraMenu_->Append(LastCameraID_,AsciiToWx(name),menuItem);
  // now call opendevice
  
  if(!fromFile){
    *ParamCameras_[camindex] = 8;    
    //OpenDevice(camindex);
  }
  NumOfCams_++;
}


/****************************************************
*****************************************************/
#ifdef BIAS_HAVE_DCAM
void ShowCamWxFrame::AddIEEECameraToMenu(int camindex,int IEEEIndex, bool fromFile) {
  stringstream camtext;
  camtext << IEEECameraInfo_[IEEEIndex].vendor << " " << IEEECameraInfo_[IEEEIndex].model;

  wxMenu* menuItem = new wxMenu;
  int id = (camindex + 1) * 10000;
  LastCameraID_ = id;
  int ckid = 0;

    // add "normal" formats
  for (unsigned int j = 0; j < IEEECameraInfo_[IEEEIndex].videoModes.size(); j++) {
    id++;

    // get all possible modis...
    dc1394video_mode_t mode = IEEECameraInfo_[IEEEIndex].videoModes[j];

    string s;
    s = VideoSource_DCAM::VideoModeToString(mode);

    if (IEEECameraInfo_[IEEEIndex].bestMode == mode) {
      ckid = id;
    }

    // check possible framerates
    stringstream helptext;
    if(mode < DC1394_VIDEO_MODE_EXIF){
    helptext << "Supported Fps: ";
    for(unsigned int k = 0; k < IEEECameraInfo_[IEEEIndex].framerates[j].size(); k++){
      helptext << IEEECameraInfo_[IEEEIndex].framerates[j][k] << ", ";
    }
    }else{
      helptext << "Format 7";
    }

    menuItem->AppendRadioItem(id,AsciiToWx(s.c_str()),AsciiToWx(helptext.str().c_str()));

    // connnect with eventhandler
    Connect(id, wxEVT_COMMAND_MENU_SELECTED,
            wxCommandEventHandler(ShowCamWxFrame::OnChangeVideoMode));
  }

  menuItem->Check(ckid, true);
  menuItem->AppendSeparator();
  if (IEEECameraInfo_[IEEEIndex].b_capable) {
    id++;

    Connect(id,wxEVT_COMMAND_MENU_SELECTED,
            wxCommandEventHandler(ShowCamWxFrame::OnChangeFirewireB));

    menuItem->AppendCheckItem(id,wxT("Activate Firewire B"));
    menuItem->Check(id, true);
    SetFirewireB_[camindex] = true;
  }


  id++;
  menuItem->AppendCheckItem(id,wxT("Activate Camera"));

  // connnect with eventhandler
  Connect(id,wxEVT_COMMAND_MENU_SELECTED,
          wxCommandEventHandler(ShowCamWxFrame::OnChangeActivate));

  if(fromFile)
    menuItem->Check(id, true);

  
  id++;
  menuItem->AppendCheckItem(id,wxT("Trigger Camera"));
  Connect(id,wxEVT_COMMAND_MENU_SELECTED,
  		wxCommandEventHandler(ShowCamWxFrame::OnChangeTrigger));   
  menuItem->Check(id, SetTrigger_[camindex]);

  CameraMenu_->Append(LastCameraID_, AsciiToWx(camtext.str().c_str()), menuItem);
  if(!fromFile){
    *ParamCameras_[camindex] = 1;
    cout << IEEECameraInfo_[IEEEIndex].model << endl;
    if (IEEECameraInfo_[IEEEIndex].model.find("XB3") != string::npos || IEEECameraInfo_[IEEEIndex].model.find("BumbleBee") != string::npos)
      *ParamCameras_[camindex] = 10;
    std::ostringstream buffer;
    buffer << IEEECameraInfo_[IEEEIndex].camid;
    *ParamDevice_[camindex] = buffer.str();
  }

  SelectedFps_[camindex] = IEEECameraInfo_[IEEEIndex].bestFramerate;

  NumOfCams_++;
}
#endif


/****************************************************
*****************************************************/
#ifdef BIAS_HAVE_DCAM
int ShowCamWxFrame::ShowFormat7Selector(int camindex, dc1394video_mode_t mode ){

  //dc1394video_mode_t mode = SelectedModeFromMenu(camindex);
  //int maxWidth, maxHeight,minBpP,maxBpP;
  //cout<<"Cameras_[camindex]:"<<Cameras_[camindex]<<endl;
  cout<<"Camindex:"<<camindex<<endl;

  //if format 7 mode was selected open GUI for parameters
	//if(format7Frame_ == NULL){
    format7Frame_ = new ShowCamWxFormat7Frame(this,
                  wxT("Select Format"),wxDefaultPosition,wxSize(550,400));
  //}

	// first get
	unsigned int IEEEIndex = 0;

    for (unsigned int j = 0; j < IEEECameraInfo_.size(); j++) {
      if (IEEECameraInfo_[j].camid == atoi((*ParamDevice_[camindex]).c_str())) {
        IEEEIndex = j;
      }
    }

    unsigned int modeindex = 0;
	for(unsigned int j = 0; j < IEEECameraInfo_[IEEEIndex].f7_modes.size();j++){
		if(IEEECameraInfo_[IEEEIndex].f7_modes[j] == mode)
        modeindex = j;
    }

    // set min and max settings
  format7Frame_->SetXSize(IEEECameraInfo_[IEEEIndex].f7_unitWidth[modeindex],
                          IEEECameraInfo_[IEEEIndex].f7_maxWidth[modeindex]);
  format7Frame_->SetXOffset(0,IEEECameraInfo_[IEEEIndex].f7_maxWidth[modeindex]);
  format7Frame_->SetYSize(IEEECameraInfo_[IEEEIndex].f7_unitHeight[modeindex],
                          IEEECameraInfo_[IEEEIndex].f7_maxHeight[modeindex]);
  format7Frame_->SetYOffset(0,IEEECameraInfo_[IEEEIndex].f7_maxHeight[modeindex]);
  format7Frame_->SetBpP(IEEECameraInfo_[IEEEIndex].f7_MinBpP[modeindex],
                        IEEECameraInfo_[IEEEIndex].f7_MaxBpP[modeindex],
                        IEEECameraInfo_[IEEEIndex].f7_bestBpP[modeindex]);

  format7Frame_->SetCameraIDandMode(IEEECameraInfo_[IEEEIndex].camid, mode);


  int ret = format7Frame_->ShowModal();
  if (ret == wxID_CANCEL){
      ret = -1;
  }
  if(ret == wxID_OK/* || ret==wxID_CANCEL*/){
    Format7LeftTop_[camindex] = Vector2<int>(format7Frame_->xOffset,format7Frame_->yOffset);
    Format7WidthHeight_[camindex] = Vector2<int>(format7Frame_->xSize,format7Frame_->ySize);
    Format7BpP_[camindex] = format7Frame_->bpp;
    Format7ColorMode_[camindex] = format7Frame_->colorMode;
    ViewColorMode_[camindex] = format7Frame_->colorModeView;
    cout<<"Selection:"<<Format7LeftTop_[camindex]<< " , Colormode: " << ViewColorMode_[camindex] << " " << (int)ViewColorMode_[camindex] << endl;    
    //format7Frame_ = NULL;
  }

  return ret;
}
#endif

/****************************************************
*****************************************************/
void ShowCamWxFrame::OnChangeDeviceChannel(wxCommandEvent& event){

	//cout<<"Event ID:"<<event.GetId()<<endl;
	// probieren den Mode zu setzen und die Kamera neu zu starten...
	int camindex = event.GetId() / 10000 - 1;
	//cout<<"Camindex:"<<camindex<<endl;
	int deviceChannelID = (event.GetId() % 10000) - 1;
	//int modeindex = (event.GetId() % 10000) - 1;
	switch (*ParamCameras_[camindex]) {
	case 8:{
		int index = deviceChannelID;
		//cout<<"Device Channel:"<<index<<endl;
		if (CameraActive_[camindex])
		{
			CloseDevice(camindex);
      biassleep(1);
			DeviceChannel_[camindex] = index;
			OpenDevice(camindex);
		}
		else{
			DeviceChannel_[camindex] = index;
		}
		return;

	}
	break;
	default:
		return;
	}
}

/****************************************************
*****************************************************/
void ShowCamWxFrame::OnChangeVideoMode(wxCommandEvent& event) {

  // probieren den Mode zu setzen und die Kamera neu zu starten...
  int camindex = event.GetId() / 10000 - 1;
  int modeindex = (event.GetId() % 10000) - 1;
  SetUserMode_[camindex] = true;

  switch (*ParamCameras_[camindex]) {
  case 1:{ // DCAM
#ifdef BIAS_HAVE_DCAM
    unsigned int IEEEIndex = 0;

    for (unsigned int j = 0; j < IEEECameraInfo_.size(); j++) {
      if (IEEECameraInfo_[j].camid == atoi((*ParamDevice_[camindex]).c_str())) {
        IEEEIndex = j;
      }
    }

    if(IEEECameraInfo_[IEEEIndex].videoModes[modeindex] < DC1394_VIDEO_MODE_EXIF){

      //user has to select framerate from dropdown menu
      wxArrayString array;
      stringstream helptext;
      helptext << "Supported Fps: ";
      for (unsigned int k = 0; k
             < IEEECameraInfo_[IEEEIndex].framerates[modeindex].size(); k++) {
        stringstream fpss;
        fpss << IEEECameraInfo_[IEEEIndex].framerates[modeindex][k];
        array.Add(AsciiToWx(fpss.str().c_str()));
      }

      wxSingleChoiceDialog choiceDialog((wxWindow*)this, wxT("Please choose framerate"),
                                        wxT("Choose Framerate"), array, (char**)NULL,
                                        wxDEFAULT_DIALOG_STYLE | wxRESIZE_BORDER | wxOK | wxCANCEL| wxCENTRE);


      choiceDialog.ShowModal();
      int selectedFps = choiceDialog.GetSelection();
      SelectedFps_[camindex]
        = IEEECameraInfo_[IEEEIndex].framerates[modeindex][selectedFps];

    }
    else{
      ///////////////////////
      //cout<<"Showing selector"<<endl;
      int ret  = ShowFormat7Selector(camindex,IEEECameraInfo_[IEEEIndex].videoModes[modeindex]);
      if(ret < 0)
        return;
    }

    // if camera is active then restart camera
    if (CameraActive_[camindex]) {
      CloseDevice(camindex);
      // wait some time for the cam to reinitialize
      biassleep(1);
      OpenDevice(camindex);
    } else {
      OnChangeActivate(event);
      int id = IEEECameraInfo_[IEEEIndex].videoModes.size();
      id += (camindex + 1) * 10000 + 1;
      if (IEEECameraInfo_[IEEEIndex].b_capable)
        id++;
      wxMenuItem* item;
      item = CameraMenu_->FindItem(id);
      item->Check(true);
#ifdef WIN32
      CloseDevice(camindex);
      Sleep(1);
      OpenDevice(camindex);
#endif
    }
#endif
  }
    break;

  case 5:{ // UEYE
    // just change framerate
    double fps = 0.0;
    switch (modeindex) {
    case 0:
      fps = 1.875;
      break;
    case 1:
      fps = 3.75;
      break;
    case 2:
      fps = 7.5;
      break;
    case 3:
      fps = 15;
      break;
    case 4:
      fps = 30;
      break;
    case 5:
      fps = 60;
      break;
    default:
      break;
    }
    *ParamFPS_[camindex] = fps;
    //dynamic_cast<VideoSource_uEye*>(Cameras_[camindex])->SetFPS(fps);
    // restart camera
    if (CameraActive_[camindex]) {
      CloseDevice(camindex);
      // wait some time for the cam to reinitialize
       biassleep(1);
       OpenDevice(camindex);
    }
  }
    break;
  default:
    break;
  }
}
/****************************************************
*****************************************************/
void ShowCamWxFrame::OnChangeActivate(wxCommandEvent& event){

  int camindex = event.GetId()/10000 -1;
  if(CameraActive_[camindex])
    CloseDevice(camindex);
  else{
   //wxCommandEvent e;
   // OnChangeVideoMode(e);
   OpenDevice(camindex);
  }

}
/****************************************************
*****************************************************/
void ShowCamWxFrame::OnChangeFirewireB(wxCommandEvent& event) {

  int camindex = event.GetId() / 10000 - 1;

  if (SetFirewireB_[camindex]) {
    cout<<"Setting 1394b false."<<endl;
    SetFirewireB_[camindex] = false;
  } else {
    cout<<"Setting 1394b true."<<endl;
    SetFirewireB_[camindex] = true;
  }

  // if camera is active then restart camera
  if (CameraActive_[camindex]) {
    CloseDevice(camindex);
    // wait some time for the cam to reinitialize
    biassleep(1);
    OpenDevice(camindex);
  }
}

/****************************************************
*****************************************************/
void  ShowCamWxFrame::OnChangeTrigger(wxCommandEvent& event){
	int camindex = event.GetId() / 10000 - 1;

	  if (SetTrigger_[camindex]) {
	    cout<<"Setting Trigger false."<<endl;
	    SetTrigger_[camindex] = false;
	  } else {
	    cout<<"Setting Trigger true."<<endl;
	    SetTrigger_[camindex] = true;
	  }

	  // if camera is active then restart camera
	  if (CameraActive_[camindex]) {
	    CloseDevice(camindex);
	    // wait some time for the cam to reinitialize
	    biassleep(1);
	    OpenDevice(camindex);
	}
}

/****************************************************
*****************************************************/
void ShowCamWxFrame::
SelectAndShowCameraImage_(){
  // show camera image in main window if a camera was selected
  if (Cameras_[NumCamSel_] != NULL && CamImg_[NumCamSel_] != NULL)
  {
    unsigned int width;
    unsigned int height;
    switch (*ParamCameras_[NumCamSel_]) {
      case ShowCamWxVideoSourceFactory::KINECT: // kinect
      case ShowCamWxVideoSourceFactory::OPENNI: // openni
      case ShowCamWxVideoSourceFactory::FREENECT2: // freenect2
      case ShowCamWxVideoSourceFactory::PMDUSB: // pmdzess
      case ShowCamWxVideoSourceFactory::PMD: // pmdtec Camcube
        InitCombinedImage();
        ShowImage_ = CamImgCombined_;
        break;
      case ShowCamWxVideoSourceFactory::XB3: // xb3
        if (VideoControllers_[NumCamSel_]!=NULL) { 
          ShowXB3_ = VideoControllers_[NumCamSel_]->GetSelectedImage();
        }
        switch(ShowXB3_){
        case 1: ShowImage_ = *CamImg_[NumCamSel_]; break;
        case 2: ShowImage_ = *CamImgMiddle_[NumCamSel_]; break;
        case 3: ShowImage_ = *CamImgRight_[NumCamSel_]; break;
        case 4: ShowImage_ = *XB3AllInOne_[NumCamSel_]; break;
        case 0: ShowImage_ = *XB3AllInOne_[NumCamSel_];
          BIAS::ImageBase tmp = ShowImage_;
          ImageConvert::DeinterleaveHorizontal(tmp, ShowImage_);
          break;
        }
        break;
      default:
        ShowImage_ = *CamImg_[NumCamSel_];
        if(ViewColorMode_[NumCamSel_] != ImageBase::CM_invalid){

          //cout << "Integer of ViewMode " <<  (int)ViewColorMode_[NumCamSel_] << endl;
          ShowImage_.SetColorModel((ImageBase::EColorModel)ViewColorMode_[NumCamSel_]);
          //cout << "Colormodel: " << ViewColorMode_[NumCamSel_] << endl;
        }
/*
      // rgb split mode fkeller
        BIAS::Camera<unsigned char> src = *CamImg_[NumCamSel_];
        src.SetColorModel(ImageBase::CM_Bayer_RGGB);
        BIAS::Camera<unsigned char> tmp;
        ImageConvert::ToRGB(src,tmp);
        ShowImage_ = tmp;
        unsigned char **in = tmp.GetImageDataArray();
        unsigned char **out = ShowImage_.GetImageDataArray();
        unsigned int w=tmp.GetWidth();
        unsigned int h=tmp.GetHeight();
        for (unsigned int y=0;y<h;y+=2) {
          for (unsigned int x=0;x<w;x+=2) {
            for (unsigned int c=0;c<3;c++) {
              // RGB lower right corner
              out [h/2+y/2][(w/2+x/2)*3+c] = in[y][x*3+c];
            }
            //out [h/2+y/2][(w/2+x/2)*3+2] = 128;
          }
        }
        for (unsigned int y=0;y<h/2;y++) {
          for (unsigned int x=0;x<w/2;x++) {
            for (unsigned int c=0;c<3;c++) {
              // R channel = upper left corner
              out[y][x*3+c] = out [h/2+y][(w/2+x)*3+0];
              // G channel = upper right corner
              out[y][((w/2)+x)*3+c] = out [h/2+y][(w/2+x)*3+1];
              // B channel = lower left corner
              out[h/2+y][x*3+c] = out [h/2+y][(w/2+x)*3+2];
            }
          }
        }
        //BIAS::ImageBase tmp = ShowImage_;
        //ImageConvert::ToRGB(*CamImg_[NumCamSel_],ShowImage_);
        //cout<<"OT ColorMode:"<<ShowImage_.GetColorModel()<<endl;
        */
        break;
    }

    // downsampling of actual camera image if set by param file
    int downSample = *ParamDown_[NumCamSel_];
    if (downSample != 0) {
      Camera<unsigned char> rescaledImage;
      Rescale<unsigned char, unsigned char> rescale;
      rescale.Downsample(ShowImage_, rescaledImage,
                         (int)(ShowImage_.GetWidth() / downSample),
                           (int)(ShowImage_.GetHeight() / downSample));
      ShowImage_ = rescaledImage;
    }

    /////////////////////////////////////
    // get size available for image canvas
    width = ShowImage_.GetWidth();
    height = ShowImage_.GetHeight();
    int widthClient=0, heightClient=0;
    IC_->GetSize(&widthClient, &heightClient);
    //GetClientSize(&widthClient, &heightClient);
    heightClient -= 30;
    widthClient -= 5;
    if (pCheckBoxFit->IsChecked()) {
    
      // compute scale that keeps the aspect ratio and the image is fully visible
      float xscale = (float)widthClient / width;
      float yscale = (float)heightClient / height;
      float scale = xscale < yscale ? xscale : yscale;

      IC_->SetScaledImageSize(width * scale, height* scale);
      IC_->SetShowScaled(true);
    }//fitcheckbox
    else{
      IC_->SetShowScaled(false);
    }
                    
    // set new height or width if changed
    if (width != LastWidth_ || height != LastHeight_){
      LastWidth_  = width;
      LastHeight_ = height;
    }

    // show actual camera image on main frame
    //cout << "Show Image " << ShowImage_.GetColorModel() << endl;
    if (ShowImage_.GetColorModel() != ImageBase::CM_invalid && ShowImage_.GetStorageType() != ImageBase::ST_invalid) {
      IC_->Show(ShowImage_, "Camera Image");
    }
    wxString text;
    text.Printf(wxT("size: %4dx%4d"),width,height);
    SetStatusText(text, 1);
  }
}

/****************************************************
*****************************************************/
void ShowCamWxFrame::OnKey(wxKeyEvent& event)
{
  int code = event.GetKeyCode();

  //cout << "Recieved keycode: " << code << endl;
  // grab single shot
  wxCommandEvent e;
  if(code == 32){
    OnGrab(e);
  }
  // grab continious
  if(code == 67){
    OnGrabContinuous(e);
  }
}


/****************************************************
*****************************************************/
void  ShowCamWxFrame::
OnShowParameters(wxCommandEvent& event){
  ShowParamGUI();
}

/****************************************************
*****************************************************/
void ShowCamWxFrame::
OnAbout(wxCommandEvent &event)
{
  wxString desc;
  desc << wxT("This tool biasShowCamWX is part of the\n");
  desc << wxT("BIAS library (Basic ImageAlgorithmS)\n");
  desc << wxT("It is used to open cameras and grab images to disk.\n\n");
  
  stringstream text;
  text << "Built on " << __DATE__ << " " << __TIME__ << "\n"
	    // << "from file " << __FILE__ << "\n"
	     << "GUI using: " << WxToAscii(wxVERSION_STRING)<< "\n";
  text<< "Built with support for:\n";
#ifdef BIAS_HAVE_DCAM
  text<< "- Firewire/IEEE1394 (CMU).\n";
#endif
#ifdef BIAS_HAVE_DSHOW
  text<< "- DirectShow.\n";
#endif
#ifdef BIAS_HAVE_V4L
  text<< "- Video4Linux.\n";
#endif
#ifdef  BIAS_HAVE_SWISSRANGER
  text<< "- Swissranger.\n";
#endif
#ifdef  BIAS_HAVE_KINECT
  text<< "- Kinect (libfreenect).\n";
#endif
#ifdef BIAS_HAVE_OPENNI
  text<< "- Kinect (OpenNI).\n";
#endif
#ifdef BIAS_HAVE_KINECT2
  text<< "- Kinect2 (freenect2).\n";
#endif
#ifdef BIAS_HAVE_FFMPEG
text<< "- FFMPEG.\n";
#endif
#ifdef BIAS_HAVE_PMD_PMDTec
text<< "- PMDTec Camcube.\n";
#endif
#ifdef BIAS_HAVE_UEYE
text<< "- IDS Ueye.\n";
#endif
desc<<AsciiToWx(text.str());

  wxString lic = wxT("BIAS is free software; you can redistribute it and/or modify\n \
  it under the terms of the GNU Lesser General Public Licence as published by\n \
  the Free Software Foundation; either version 2.1 of the Licence, or\n \
  (at your option) any later version.\n \
  \n \
  BIAS is distributed in the hope that it will be useful,\n \
  but WITHOUT ANY WARRANTY; without even the implied warranty of\n \
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n \
  GNU Lesser General Public Licence for more details.\n \
  \n \
  You should have received a copy of the GNU Lesser General Public Licence\n \
  along with BIAS; if not, write to the Free Software\n \
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA\n");

  wxAboutDialogInfo aboutdlg;
  aboutdlg.SetDescription( desc );
  aboutdlg.SetLicence( lic );
  aboutdlg.SetCopyright( wxT("Copyright (C) 2003-20011\n \
  Multimediale Systeme der Informationsverarbeitung\n \
  Institut fuer Informatik\n \
  Christian-Albrechts-Universitaet Kiel") );
  aboutdlg.SetWebSite( wxT("http://www.mip.informatik.uni-kiel.de/") );

  wxAboutBox( aboutdlg );
}

/****************************************************
*****************************************************/
void  ShowCamWxFrame::
ShowParamGUI(){
  paramFrame_ = GetFrame(this);
  paramFrame_->Show(true);
  paramFrame_->Move( wxPoint(500,200));
}

/****************************************************
*****************************************************/
void  ShowCamWxFrame::
SaveImageUnthreaded_(int cameraIndex) {
  wxString text;
  switch(*ParamCameras_[cameraIndex]) {
  case ShowCamWxVideoSourceFactory::PMD:
    text.Printf(wxT("SINGLE-CAM%02d-%05d-int.mip"), cameraIndex, GrabCounter_);
    CamImgPMDInt_[cameraIndex]->UpdateMetaData();
    ImageIO::Save(currentSingleSaveDir_ + WxToAscii(text), *CamImgPMDInt_[cameraIndex]);
  case ShowCamWxVideoSourceFactory::SR:
  case ShowCamWxVideoSourceFactory::PMDUSB:
    CamImg_[cameraIndex]->UpdateMetaData();
    text.Printf(wxT("SINGLE-CAM%02d-%05d-grey.mip"), cameraIndex, GrabCounter_);
    ImageIO::Save(currentSingleSaveDir_ + WxToAscii(text), *CamImg_[cameraIndex]);

    text.Printf(wxT("SINGLE-CAM%02d-%05d-depth.mip"), cameraIndex, GrabCounter_);
    CamImgPMDDepth_[cameraIndex]->UpdateMetaData();
    ImageIO::Save(currentSingleSaveDir_ + WxToAscii(text), *CamImgPMDDepth_[cameraIndex]);

    text.Printf(wxT("SINGLE-CAM%02d-%05d-amp.mip"), cameraIndex, GrabCounter_);
    CamImgPMDAmp_[cameraIndex]->UpdateMetaData();
    ImageIO::Save(currentSingleSaveDir_ + WxToAscii(text), *CamImgPMDAmp_[cameraIndex]);

    break;
  case ShowCamWxVideoSourceFactory::XB3:
    text.Printf(wxT("SINGLE-CAM%02d-left-%05d.mip"), cameraIndex, GrabCounter_);
    CamImg_[cameraIndex]->UpdateMetaData();
    ImageIO::Save(currentSingleSaveDir_ + WxToAscii(text), *CamImg_[cameraIndex]);

    text.Printf(wxT("SINGLE-CAM%02d-center-%05d.mip"), cameraIndex, GrabCounter_);
    CamImgMiddle_[cameraIndex]->UpdateMetaData();
    ImageIO::Save(currentSingleSaveDir_ + WxToAscii(text), *CamImgMiddle_[cameraIndex]);

    text.Printf(wxT("SINGLE-CAM%02d-right-%05d.mip"), cameraIndex, GrabCounter_);
    CamImgRight_[cameraIndex]->UpdateMetaData();
    ImageIO::Save(currentSingleSaveDir_ + WxToAscii(text), *CamImgRight_[cameraIndex]);
    break;
  case ShowCamWxVideoSourceFactory::KINECT:
  case ShowCamWxVideoSourceFactory::OPENNI:
    CamImg_[cameraIndex]->UpdateMetaData();
    text.Printf(wxT("SINGLE-CAM%02d-%05d-color.mip"), cameraIndex, GrabCounter_);
    ImageIO::Save(currentSingleSaveDir_ + WxToAscii(text), *CamImg_[cameraIndex]);

    text.Printf(wxT("SINGLE-CAM%02d-%05d-depth.mip"), cameraIndex, GrabCounter_);
    CamImgPMDDepth_[cameraIndex]->UpdateMetaData();
    ImageIO::Save(currentSingleSaveDir_ + WxToAscii(text), *CamImgPMDDepth_[cameraIndex]);
    break;
  case ShowCamWxVideoSourceFactory::FREENECT2:
    CamImg_[cameraIndex]->UpdateMetaData();
    text.Printf(wxT("SINGLE-CAM%02d-%05d-color.mip"), cameraIndex, GrabCounter_);
    ImageIO::Save(currentSingleSaveDir_ + WxToAscii(text), *CamImg_[cameraIndex]);
    text.Printf(wxT("SINGLE-CAM%02d-%05d-depth.mip"), cameraIndex, GrabCounter_);
    CamImgPMDDepth_[cameraIndex]->UpdateMetaData();
    ImageIO::Save(currentSingleSaveDir_ + WxToAscii(text), *CamImgPMDDepth_[cameraIndex]);
    text.Printf(wxT("SINGLE-CAM%02d-%05d-IR.mip"), cameraIndex, GrabCounter_);
    CamImgPMDInt_[cameraIndex]->UpdateMetaData();
    ImageIO::Save(currentSingleSaveDir_ + WxToAscii(text), *CamImgPMDInt_[cameraIndex]);
    break;
  default:
    CamImg_[cameraIndex]->UpdateMetaData();
    text.Printf(wxT("SINGLE-CAM%02d-%05d.mip"), cameraIndex, GrabCounter_);
    ImageIO::Save(currentSingleSaveDir_ + WxToAscii(text), *CamImg_[cameraIndex]);
    break;
  }
}

/****************************************************
*****************************************************/
void  ShowCamWxFrame::
GrabSingleImageUnthreadedFromCamera_(int cameraIndex){
  int i=0;
  i = cameraIndex;
#ifdef BIAS_HAVE_SWISSRANGER
  if(Cameras_[i] != NULL && dynamic_cast<VideoSource_SwissRanger*>(Cameras_[i]) != NULL){
    Cameras_[i]->GrabSingle(*CamImg_[i]);
    dynamic_cast<VideoSource_SwissRanger*>
      (Cameras_[i])->GrabSingleDepth(*CamImgPMDDepth_[i]);
    dynamic_cast<VideoSource_SwissRanger*>
      (Cameras_[i])->GrabSingleModCoeff(*CamImgPMDAmp_[i]);
  }else

#endif // BIAS_HAVE_SWISSRANGER
#ifdef BIAS_HAVE_PMD_PMDTec
  if(Cameras_[i] != NULL && dynamic_cast<VideoSource_PMD*>(Cameras_[i]) != NULL){
    Cameras_[i]->GrabSingle(*CamImg_[i]);
    dynamic_cast<VideoSource_PMD*>
      (Cameras_[i])->GrabSingleDepth(*CamImgPMDDepth_[i]);
    dynamic_cast<VideoSource_PMD*>
      (Cameras_[i])->GrabSingleModCoeff(*CamImgPMDAmp_[i]);
    dynamic_cast<VideoSource_PMD*>
      (Cameras_[i])->GrabSingleIntensity(*CamImgPMDInt_[i]);
  }else

#endif // BIAS_HAVE_SWISSRANGER

    //#ifndef WIN32
#ifdef BIAS_HAVE_DCAM
    if(dynamic_cast<VideoSource_DCAM_BumbleBee*>(Cameras_[i]) != NULL){
      //cout << "Before Casting" << endl;
      (dynamic_cast<VideoSource_DCAM_BumbleBee*>(Cameras_[i]))->
        GrabSingle(*XB3AllInOne_[i], *CamImg_[i],*CamImgMiddle_[i],*CamImgRight_[i]);
    }else
      //#endif
#endif //not WIN32
      {
        Cameras_[i]->GrabSingle(*CamImg_[i]);
      }
}


/****************************************************
*****************************************************/
int ShowCamWxFrame::
CreateMasterThread_(){
	ServerMutex_.resize(NUMBER_CAMERAS);
	ServerCondition_.resize(NUMBER_CAMERAS);
	ClientsMutex_.resize(NUMBER_CAMERAS);
	ClientsCondition_.resize(NUMBER_CAMERAS);
	for (unsigned int i=0;i<NUMBER_CAMERAS;i++) {
		ServerMutex_[i]= new wxMutex();
		ServerCondition_[i] = new wxCondition(*(ServerMutex_[i]));
		ClientsMutex_[i]= new wxMutex();
		ClientsCondition_[i] = new wxCondition(*(ClientsMutex_[i]));
	}

	grabMasterThread_ = new GrabMasterThread(&Cameras_,
			&ClientsCondition_,
			&ServerCondition_,
			&ClientsMutex_,
			&ServerMutex_);
	grabMasterThread_->Create();
	grabMasterThread_->Run();
#ifndef WIN32
	usleep(5);
#else
  Sleep(1);
#endif
	return 0;
}


/****************************************************
*****************************************************/
int ShowCamWxFrame::
CreateGrabThreadForCamera_(int cameraIndex){

	//cout<<"Creating GrabThread for camera:"<<cameraIndex<<endl;
  // create new masterthread if none present
  bool gmtLock = false;
  if (grabMasterThread_ == NULL) {
  	CreateMasterThread_();
  } else {
    gmtLock = true;
    ServerMutex_[cameraIndex]->Lock();
    ClientsMutex_[cameraIndex]->Lock();
  }

  //delete thread before creating new one
  if(grabSingleThread_[cameraIndex] != NULL){
    delete grabSingleThread_[cameraIndex];
    grabSingleThread_[cameraIndex] = NULL;
  }

  //cout<<"Creating new GrabSingleThread:"<<endl<<flush;
  //cout<<"With CamImg="<<CamImg_[cameraIndex]<<endl;
  //create new camera thread
  grabSingleThread_[cameraIndex] =
            new GrabSingleThread(Cameras_[cameraIndex],
                                 ClientsCondition_[cameraIndex],
                                 ServerCondition_[cameraIndex],
                                 ClientsMutex_[cameraIndex],
                                 ServerMutex_[cameraIndex],
                                 cameraIndex,
                                 BufferAccess_[cameraIndex],
                                 bWriteBuffered_,
                                 ReadIndex_[cameraIndex],
                                 WriteIndex_[cameraIndex],
                                 CamImg_[cameraIndex],
                                 CamImgMiddle_[cameraIndex],
                                 CamImgRight_[cameraIndex],
                                 CamImgPMDDepth_[cameraIndex],
                                 CamImgPMDAmp_[cameraIndex],
                                 CamImgPMDInt_[cameraIndex],
                                 XB3AllInOne_[cameraIndex],
                                 ImageBuffer_[cameraIndex],
                                 ImageBufferMiddle_[cameraIndex],
                                 ImageBufferRight_[cameraIndex],
                                 ImageBufferPMDDepth_[cameraIndex],
                                 ImageBufferPMDAmp_[cameraIndex],
                                 ImageBufferPMDInt_[cameraIndex]);



  // try to create and start thread
  if(grabSingleThread_[cameraIndex]!= NULL && !grabSingleThread_[cameraIndex]->IsAlive()){
    if(grabSingleThread_[cameraIndex]->Create()!=wxTHREAD_NO_ERROR) {
      BIASERR("Cannot create GrabSingleThread, saving unthreaded");
    }
    else{
    	grabSingleThread_[cameraIndex]->Run();
      ThreadInited_[cameraIndex] = true;
#ifdef BIAS_DEBUG
      // Thread ID ausgeben
      std::cout<<"Created GrabSingleThread: "<<cameraIndex<<endl;
      cout << "Created Thread with ID: " << grabSingleThread_[cameraIndex]->GetId()
           << " for CAM: " << cameraIndex << endl;
#endif
    }
  }
  else if(grabSingleThread_[cameraIndex]== NULL){
    BIASERR("Thread:"<<cameraIndex<<" is NULL!");
  }
  else if(grabSingleThread_[cameraIndex]->IsAlive()){
    BIASERR("Thread:"<<cameraIndex<<" is already Alive!");
  }

  if (gmtLock) {
    ServerMutex_[cameraIndex]->Unlock();
    ClientsMutex_[cameraIndex]->Unlock();
  }

  return 0;
}



/****************************************************
*****************************************************/
int ShowCamWxFrame::
CreateImageSaveThreadForCamera_(int cameraIndex){
  int i = 0;
  i = cameraIndex;
  stringstream text;
  text<<"CAM"<<FileHandling::LeadingZeroString(i,2);

  // #ifdef BIAS_HAVE_PANTILT_DP
  //     if(*bUsePanTiltUnit_){
  //       (*CamImgPMDDepth_[i]).SetProj(projHeader);
  //       (*CamImgPMDAmp_[i]).SetProj(projHeader);
  //       (*CamImgPMDAmp_[i]).UpdateMetaData();
  //       (*CamImgPMDDepth_[i]).UpdateMetaData();
  //     }
  // #endif

  // create new thread
  saveThreads_[i] = new ImageSaveThread(text.str(),
                                        *ParamDown_[i],
                                        BufferAccess_[i],
                                        bWriteBuffered_,
                                        ReadIndex_[i],
                                        WriteIndex_[i],
                                        CamImg_[i],
                                        CamImgMiddle_[i],
                                        CamImgRight_[i],
                                        CamImgPMDDepth_[i],
                                        CamImgPMDAmp_[i],
                                        CamImgPMDInt_[i],
                                        ImageBuffer_[i],
                                        ImageBufferMiddle_[i],
                                        ImageBufferRight_[i],
                                        ImageBufferPMDDepth_[i],
                                        ImageBufferPMDAmp_[i],
                                        ImageBufferPMDInt_[i],
                                        bSaveToStream_);

  // init the thread or capture single image
  if(saveThreads_[i] != NULL && saveThreads_[i]->Create() != wxTHREAD_NO_ERROR){
    BIASERR("Cannot create ImageSaveThread, saving unthreaded");
  }//saveThreads_[i] != NULL && saveThreads_[i]->Create()
  else{
    cout << "Run Thread" << endl;
    saveThreads_[i]->SetPriority(100);
    saveThreads_[i]->Run();
  }
  return 0;
}
