/*
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003-2009    (see file CONTACT for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifdef WIN32
#include <Base\Common\W32Compat.hh>
#endif
#include "ShowCamWxThreads.hh"
#include <Filter/Rescale.hh>
#include <Base/Common/FileHandling.hh>
#include <wx/wx.h>

#include <Base/Common/W32Compat.hh>
#ifdef BIAS_HAVE_DCAM
#include <VideoSource/VideoSource_DCAM_BumbleBee.hh>
#endif
#ifdef BIAS_HAVE_SWISSRANGER
#include <VideoSource/VideoSource_SwissRanger.hh>
#endif
#include <VideoSource/VideoSource_Kinect_Base.hh>
#ifdef BIAS_HAVE_PMD_PMDTec
#include  <VideoSource/VideoSource_PMD.hh>
#endif
#ifdef BIAS_HAVE_PMD_PMDZess
#include  <VideoSource/VideoSource_PMDZess.hh>
#endif
#ifdef BIAS_HAVE_UEYE
#include  <VideoSource/VideoSource_uEye.hh>
#endif
#ifdef BIAS_HAVE_DSHOW
#include <VideoSource/VideoSource_DSHOW.hh>
#endif
#ifdef BIAS_HAVE_KINECT2
#include <VideoSource/VideoSource_Kinect2.hh>
#endif
#ifdef BIAS_HAVE_V4L
#include <VideoSource/VideoSource_V4L.hh>
#endif

using namespace BIAS;
using namespace std;


#define BUFFER_CAPACITY_MIN  0.8
#define BUFFER_GRAB_SLEEP 500
#define BUFFER_SAVE_SLEEP 2500

/*********************  ImageSaveThread  **********************************
 **************************************************************************/

ImageSaveThread::
ImageSaveThread(std::string                           cameraName,
                const int&                            downsample,
                wxMutex*                              BufferAccessMutex,
                bool&                                 writeToBuffer,
                unsigned int&                         readIndex,
                unsigned int&                         writeIndex,
				        BIAS::Camera<unsigned char>*          img,
		        	  BIAS::Camera<unsigned char>*          imgCenter,
                BIAS::Camera<unsigned char>*          imgRight,
				        BIAS::Camera<float>*                  imgDepth,
                BIAS::Camera<float>*                  imgAmp,
                BIAS::Camera<float>*                  imgInt,
			        	std::vector<Camera<unsigned char>* >* imageBuffer,
				        std::vector<Camera<unsigned char>* >* imageBufferCenter,
                std::vector<Camera<unsigned char>* >* imageBufferRight,
				        std::vector<Camera<float>* >*         imageBufferPmdDepth,
                std::vector<Camera<float>* >*         imageBufferPmdAmp,
                std::vector<Camera<float>* >*         imageBufferPmdInt,
                bool saveToStream)
  : wxThread(wxTHREAD_DETACHED)
{

  cameraName_   = cameraName;
  DownSample_   = downsample;
  BufferAccess_ = BufferAccessMutex;

  //indicates whether we are writing to th ebuffer or not
  WriteToBuffer_= &writeToBuffer;

  //indexs in buffers
  ReadIndex_    = &readIndex;
  WriteIndex_   = &writeIndex;

  Img_          = img;
  ImgCenter_    = imgCenter;
  ImgRight_     = imgRight;
  ImgDepth_     = imgDepth;
  ImgAmp_       = imgAmp;
  ImgInt_       = imgInt;

  ImageBuffer_       = imageBuffer;
  ImageBufferCenter_ = imageBufferCenter;
  ImageBufferRight_  = imageBufferRight;

  ImageBufferPMDDepth_ = imageBufferPmdDepth;
  ImageBufferPMDAmp_   = imageBufferPmdAmp;
  ImageBufferPMDInt_   = imageBufferPmdInt;

  ImageCounter_ = 0;
  saveDirectory_ = "";
  stream_ = saveToStream;
  if (stream_) {
    for (unsigned int i=0;i<10;i++) {
      ips_.push_back(new ImagePackage());
      bStreamOpen_.push_back(false);
    }
  }
}


/**
  @brief
  @author ischiller
*/
void* ImageSaveThread::
Entry() {
  while(!TestDestroy()){
    if(*WriteToBuffer_){
      // save images from buffer
      BufferAccess_->Lock();
      //cout<<"l"<<flush;
      if((*WriteIndex_ != ((*ReadIndex_ + 1)  % ImageBuffer_->size()))){
        //set new readindex
        *ReadIndex_ = (*ReadIndex_ + 1) % ImageBuffer_->size();
        //cout<<"u"<<flush;
        BufferAccess_->Unlock();

		    if((*ImageBuffer_)[*ReadIndex_] != NULL)
			    SaveImage_((*ImageBuffer_)[*ReadIndex_],"");
		    if((*ImageBufferCenter_)[*ReadIndex_] != NULL)
			    SaveImage_((*ImageBufferCenter_)[*ReadIndex_],"Center");
        if((*ImageBufferRight_)[*ReadIndex_] != NULL)
			    SaveImage_((*ImageBufferRight_)[*ReadIndex_],"Right");
		    if((*ImageBufferPMDDepth_)[*ReadIndex_] != NULL)
			    SaveImage_((*ImageBufferPMDDepth_)[*ReadIndex_],"Depth");
        if((*ImageBufferPMDAmp_)[*ReadIndex_] != NULL)
			    SaveImage_((*ImageBufferPMDAmp_)[*ReadIndex_],"Amp");
        if((*ImageBufferPMDInt_)[*ReadIndex_] != NULL)
			    SaveImage_((*ImageBufferPMDInt_)[*ReadIndex_],"Int");

        ImageCounter_++;
      }
      else{
        // wait for images in buffer
        //cout<<"sleepSAVE "<<flush;
        BufferAccess_->Unlock();
#ifndef WIN32
        usleep(BUFFER_SAVE_SLEEP);
#else
        Sleep(BUFFER_SAVE_SLEEP/1000);
#endif
      }
    }else{
      // save single image
	    if(Img_ != NULL)
		    SaveImage_(Img_,"");
	    if(ImgCenter_ != NULL)
		    SaveImage_(ImgCenter_,"Center");
	    if(ImgRight_ != NULL)
		    SaveImage_(ImgRight_,"Right");
	    if(ImgDepth_ != NULL)
		    SaveImage_(ImgDepth_,"Depth");
	    if(ImgAmp_ != NULL)
		    SaveImage_(ImgAmp_,"Amp");
	    if(ImgInt_ != NULL)
		    SaveImage_(ImgInt_,"Int");

      ImageCounter_++;
    }
  }
  FinishSaving();
  return NULL;
}

/**
  @brief
  @author ischiller
*/
void ImageSaveThread::FinishSaving(){
  BufferAccess_->Lock();
  //cout<<"f"<<flush;
  while((*WriteIndex_ != ((*ReadIndex_ + 1)  % ImageBuffer_->size()))){
	  cout<<"finishing:"<<endl<<flush;
    *ReadIndex_ = (*ReadIndex_ + 1) % ImageBuffer_->size();

    if((*ImageBuffer_)[*ReadIndex_] != NULL)
      SaveImage_((*ImageBuffer_)[*ReadIndex_],"");
    if((*ImageBufferCenter_)[*ReadIndex_] != NULL)
      SaveImage_((*ImageBufferCenter_)[*ReadIndex_],"Center");
    if((*ImageBufferRight_)[*ReadIndex_] != NULL)
      SaveImage_((*ImageBufferRight_)[*ReadIndex_],"Right");
    if((*ImageBufferPMDDepth_)[*ReadIndex_] != NULL)
      SaveImage_((*ImageBufferPMDDepth_)[*ReadIndex_],"Depth");
    if((*ImageBufferPMDAmp_)[*ReadIndex_] != NULL)
      SaveImage_((*ImageBufferPMDAmp_)[*ReadIndex_],"Amp");
    if((*ImageBufferPMDInt_)[*ReadIndex_] != NULL)
      SaveImage_((*ImageBufferPMDAmp_)[*ReadIndex_],"Int");

    ImageCounter_++;
  }
  //cout<<"k"<<flush;
  BufferAccess_->Unlock();
  if (stream_) {
    for (unsigned int i=0; i<10;i++) {
      if (bStreamOpen_[i]) {
        bStreamOpen_[i]=false;
        ips_[i]->Close();
      }
    }
  }
}

/**
  @brief
  @author ischiller
*/
void ImageSaveThread::
SaveImage_(Camera <unsigned char>* im, string name) {
  stringstream text;
  text<<"-"<<FileHandling::LeadingZeroString(ImageCounter_,5)<<".mip";
  string FilenameOut = cameraName_ + name  + text.str();
  //cout<<"Write:"<<FilenameOut<<endl;
  int idx = -1;
  if (stream_) {
    idx = mapNameToStreamIdx_(name);
    if (idx!=-1) {
      if (!bStreamOpen_[idx]) {
        string fname = saveDirectory_ + cameraName_ + name;
        ips_[idx]->Create(fname);
        bStreamOpen_[idx] = true;
      }
    }
  }
  if(DownSample_ != 0){
    Camera<unsigned char> rescaledImage;
    Rescale<unsigned char, unsigned char> rescale;
    rescale.Downsample(*im, rescaledImage,
                       (int)(im->GetWidth() / DownSample_),
                       (int)(im->GetHeight() / DownSample_));
    rescaledImage.UpdateMetaData();

    if (stream_) {
      if (ips_[idx]->AddImage(rescaledImage, FilenameOut)!=0) {
        BIASERR("Failed saving image to stream");
      }
    } else {
      //BIAS::ImageIO::Save(saveDirectory_ + FilenameOut, rescaledImage, true);
      BIAS::ImageIO::Save(saveDirectory_ + FilenameOut, 
                          rescaledImage,
                          BIAS::ImageIO::FF_auto,
                          true);
    }

  }else{
    im->UpdateMetaData();
    if (stream_) {
      if (ips_[idx]->AddImage(*im, FilenameOut)!=0) {
        BIASERR("Failed saving image to stream");
      }
    } else {
      //BIAS::ImageIO::Save(saveDirectory_ + FilenameOut, *im, true);
      BIAS::ImageIO::Save(saveDirectory_ + FilenameOut, 
                          *im,  BIAS::ImageIO::FF_auto,
                          true);
    }
  }
}

/**
  @brief
  @author ischiller
*/
void ImageSaveThread::
SaveImage_(Camera <float>* imF, string name){
  stringstream text;
  text<<"-"<<FileHandling::LeadingZeroString(ImageCounter_,5)<<".mip";
  string FilenameOut = cameraName_ + name  + text.str();
  int idx = -1;
  if (stream_) {
    idx = mapNameToStreamIdx_(name);
    if (idx!=-1) {
      if (!bStreamOpen_[idx]) {
        string fname = saveDirectory_ + cameraName_ + name;
        ips_[idx]->Create(fname);
        bStreamOpen_[idx] = true;
      }
    }
  }
  if(DownSample_ != 0){
    Camera<float> rescaledImage;
    Rescale<float, float> rescale;
    rescale.Downsample(*imF, rescaledImage,
                       (int)(imF->GetWidth() / DownSample_),
                       (int)(imF->GetHeight() / DownSample_));
    rescaledImage.UpdateMetaData();

    if (stream_) {
      if (ips_[idx]->AddImage(rescaledImage, FilenameOut)!=0) {
        BIASERR("Failed saving image to stream");
      }
    } else {
      //BIAS::ImageIO::Save(saveDirectory_ + FilenameOut, rescaledImage, true);
      BIAS::ImageIO::Save(saveDirectory_ + FilenameOut,
                          rescaledImage,
                          BIAS::ImageIO::FF_auto,
                          true);
    }

  }else{
    imF->UpdateMetaData();
    if (stream_) {
      if (ips_[idx]->AddImage(*imF, FilenameOut)!=0) {
        BIASERR("Failed saving image to stream");
      }
    } else {
      //BIAS::ImageIO::Save(saveDirectory_ + FilenameOut, *imF, true);
      BIAS::ImageIO::Save(saveDirectory_ + FilenameOut, 
                          *imF, 
                          BIAS::ImageIO::FF_auto,
                          true);
    }
  }
}

int ImageSaveThread::mapNameToStreamIdx_(string name) {
  if (name.length() == 0)
    return 0;
  if (name.find("Center")!=string::npos)
    return 1;
  if (name.find("Right")!=string::npos)
    return 2;
  if (name.find("Amp")!=string::npos)
    return 3;
  if (name.find("Depth")!=string::npos)
    return 4;
  if (name.find("Int")!=string::npos)
    return 5;
  return -1;
}

/************************* GrabSingleThread *******************************
 **************************************************************************/

GrabSingleThread::
GrabSingleThread(BIAS::VideoSource  *cam,
                 wxCondition        *ClientCondition,
                 wxCondition        *ServerCondition,
                 wxMutex            *mutexClientCondition,
                 wxMutex            *mutexServerCondition,
                 unsigned int       threadNumber,
                 wxMutex            *BufferAccessMutex,
                 bool&              writeToBuffer,
                 unsigned int&      readIndex,
                 unsigned int&      writeIndex,

                 BIAS::Camera<unsigned char> *img,
                 BIAS::Camera<unsigned char> *imgCenter,
                 BIAS::Camera<unsigned char> *imgRight,
                 BIAS::Camera<float>         *imgDepth,
                 BIAS::Camera<float>         *imgAmp,
                 BIAS::Camera<float>         *imgInt,
                 BIAS::Camera<unsigned char> *imgAll,

                 std::vector<Camera<unsigned char>* > *imageBuffer,
                 std::vector<Camera<unsigned char>* > *imageBufferCenter,
                 std::vector<Camera<unsigned char>* > *imageBufferRight,
                 std::vector<Camera<float>* >*         imageBufferPMDDepth,
                 std::vector<Camera<float>* >*         imageBufferPMDAmp,
                 std::vector<Camera<float>* >*         imageBufferPMDInt)

	:wxThread(wxTHREAD_DETACHED)
{
  //the images
  Img_           = img;
  ImgCenter_     = imgCenter;
  ImgRight_      = imgRight;
  ImgDepth_      = imgDepth;
  ImgAmp_        = imgAmp;
  ImgInt_        = imgInt;
  AllImages_     = imgAll;
  threadNumber_ = threadNumber;

  //mutex for conditional
  mutexClientCondition_= mutexClientCondition;
  mutexServerCondition_= mutexServerCondition;

  //the conditionals
  ClientCondition_     = ClientCondition;
  ServerCondition_     = ServerCondition;

  //mutex for threaded buffer access
  BufferAccess_  = BufferAccessMutex;

  // The image buffers
  ImageBuffer_         = imageBuffer;
  ImageBufferCenter_   = imageBufferCenter;
  ImageBufferRight_    = imageBufferRight;
  ImageBufferPMDDepth_ = imageBufferPMDDepth;
  ImageBufferPMDAmp_   = imageBufferPMDAmp;
  ImageBufferPMDInt_   = imageBufferPMDInt;

  //shall we write in the buffer?
  WriteToBuffer_ = &writeToBuffer;
  //read an dwrite indexes
  ReadIndex_     = &readIndex;
  WriteIndex_    = &writeIndex;
  //the bnuffersize
  BufferSize_    = ImageBuffer_->size();

  //the camera
  Cam_ = cam;

}

/**
  @brief
  @author ischiller
*/
void* GrabSingleThread::
Entry()
{
//cout << "initing grab thread" << endl;
#ifdef BIAS_HAVE_DCAM
 VideoSource_DCAM_BumbleBee *xb3cam = NULL;
 xb3cam = dynamic_cast<VideoSource_DCAM_BumbleBee*>(Cam_);
 VideoSource_DCAM *dcam = NULL;
 dcam = dynamic_cast<VideoSource_DCAM*>(Cam_);
#endif
#ifdef BIAS_HAVE_SWISSRANGER
 VideoSource_SwissRanger *srcam = NULL;
 srcam = dynamic_cast<VideoSource_SwissRanger*>(Cam_);
#endif
#ifdef BIAS_HAVE_UEYE
 VideoSource_uEye *ueyecam = NULL; 
 ueyecam = dynamic_cast<VideoSource_uEye*>(Cam_);
#endif
#ifdef BIAS_HAVE_PMD_PMDTec
 VideoSource_PMD *pmdcam = NULL;
 pmdcam = dynamic_cast<VideoSource_PMD*>(Cam_);
#endif
#ifdef BIAS_HAVE_PMD_PMDZess
//zess
 VideoSource_PMDZess *usbpmdcam = NULL;
 usbpmdcam = dynamic_cast<VideoSource_PMDZess*>(Cam_);
#endif
#ifdef BIAS_HAVE_V4L
 VideoSource_V4L * webcam=NULL;
 webcam =  dynamic_cast<VideoSource_V4L*>(Cam_);
#endif
#ifdef BIAS_HAVE_DSHOW
 VideoSource_DSHOW * webcam=NULL;
 webcam =  dynamic_cast<VideoSource_DSHOW*>(Cam_);
#endif
 VideoSource_Kinect_Base * kinect=NULL;
 kinect =  dynamic_cast<VideoSource_Kinect_Base*>(Cam_);

#ifdef BIAS_HAVE_DCAM
  //normal
  if(dcam !=NULL){
    camtype_ = DCAM;
  }
  //BumbleBee
  if(xb3cam !=NULL){
    camtype_ = XB3;
  }
#endif

#ifdef BIAS_HAVE_SWISSRANGER
  if(srcam != NULL){
    camtype_ = SR;
  }
#endif
#ifdef BIAS_HAVE_UEYE
  if(ueyecam !=NULL){
    camtype_ = UEYE;
  }
#endif
#ifdef BIAS_HAVE_PMD_PMDTec
//pmdtec
  if(pmdcam !=NULL){
    camtype_ = PMDTEC;
  }
#endif
#ifdef BIAS_HAVE_PMD_PMDZess
//zess
  if(usbpmdcam !=NULL){
    camtype_ = ZESS;
  }
#endif
#ifdef BIAS_HAVE_DSHOW
  if (webcam != NULL) {
    camtype_ = DSHOW;
  }
#endif
#ifdef BIAS_HAVE_V4L
  if (webcam != NULL) {
    camtype_ = V4L;
  }
#endif
#ifdef BIAS_HAVE_KINECT
  if (kinect != NULL) {
    camtype_ = KINECT;
  }
#endif
  
#ifdef BIAS_HAVE_OPENNI
  if (kinect != NULL) {
    camtype_ = KINECT;
  }
#endif
  
#ifdef BIAS_HAVE_KINECT2
  VideoSource_Kinect2 *kinect2 = NULL;
  kinect2 = dynamic_cast<VideoSource_Kinect2*>(Cam_);
  if (kinect2 != NULL) {
    camtype_ = FREENECT2;
  }
#endif

  // lock this clients mutex
  mutexClientCondition_->Lock();

  while(!TestDestroy()){
    if(Cam_ != NULL){
      //cout<<"Grabbing"<<endl;
      if(*WriteToBuffer_){
        double bufferPercent = 0.0;
        if(*WriteIndex_ >= *ReadIndex_){
          bufferPercent = (double)((*WriteIndex_ + 1) -
                                   (*ReadIndex_ + 1) ) / (double)BufferSize_;
        }else{
          bufferPercent = (double)(((*WriteIndex_ + 1)+BufferSize_)-
                                   (*ReadIndex_ + 1)) / (double)BufferSize_;
        }

        while(*WriteToBuffer_ && bufferPercent > BUFFER_CAPACITY_MIN){

          if(*WriteIndex_ >= *ReadIndex_){
            bufferPercent = (double)((*WriteIndex_ + 1) -
                                     (*ReadIndex_ + 1) ) / (double)BufferSize_;
          }else{
            bufferPercent = (double)(((*WriteIndex_ + 1)+BufferSize_)-
                                     (*ReadIndex_ + 1)) / (double)BufferSize_;
          }
          //std::cout << "Thread:"<<threadNumber_<<" Pufferfreiraum: " << bufferPercent << std::endl;
#ifndef WIN32
          usleep(BUFFER_GRAB_SLEEP);
#else
          Sleep(BUFFER_GRAB_SLEEP/1000);
#endif
        }
      }

      timeval t1,t2;
      gettimeofday(&t1,NULL);

      switch (camtype_) {
      #ifdef BIAS_HAVE_DCAM
      case DCAM:
        //cout << "grab dcam" << endl;
        dcam->GrabSingle(*Img_);
        break;
      case XB3:
        //cout << "grab xb3" << endl;
        xb3cam->GrabSingle(*AllImages_,*Img_,*ImgCenter_,*ImgRight_);
        break;
      #endif
      #ifdef BIAS_HAVE_SWISSRANGER
      case SR:
        srcam->GrabSingle(*Img_);
        srcam->GrabSingleDepth(*ImgDepth_);
        srcam->GrabSingleModCoeff(*ImgAmp_);
        break;
      #endif
      case KINECT:
        kinect->GrabSingle(*Img_);
        kinect->GrabSingleDepth(*ImgDepth_);
        break;
      #ifdef BIAS_HAVE_UEYE
      case UEYE:
        ueyecam->GrabSingle(*Img_);
        break;
      #endif
      #ifdef BIAS_HAVE_PMD_PMDTec
      case PMDTEC:
        pmdcam->GrabSingle(*Img_);
      //  cout << Img_->GetWidth() << " " <<Img_->GetHeight() << endl;
        pmdcam->GrabSingleDepth(*ImgDepth_);
        pmdcam->GrabSingleModCoeff(*ImgAmp_);
        pmdcam->GrabSingleIntensity(*ImgInt_);
        break;
      #endif
      #ifdef BIAS_HAVE_PMD_PMDZess
      case ZESS:
        usbpmdcam->GrabSingle(*Img_);
        usbpmdcam->GrabSingleDepth(*ImgDepth_);
        usbpmdcam->GrabSingleModCoeff(*ImgAmp_);
        break;
      #endif
      #ifdef BIAS_HAVE_DSHOW
      case DSHOW:       
        webcam->GrabSingle(*Img_);
        break;
      #endif
      #ifdef BIAS_HAVE_V4L
      case V4L:
        webcam->GrabSingle(*Img_);
        break;
      #endif
      #ifdef BIAS_HAVE_KINECT2
      case FREENECT2:
        kinect2->GrabSingle(*Img_);
        kinect2->GrabSingleDepth(*ImgDepth_);
        kinect2->GrabSingleIR(*ImgInt_);
        break;
      #endif

      case INVALID:
      default:
        BIASWARN("Camera type not supported.");
        break;
      }

      //obtain second time stamp
      gettimeofday(&t2,NULL);
      long duration = ((t2.tv_sec-t1.tv_sec)*1000000) + (t2.tv_usec-t1.tv_usec);
      //long sec=t1.tv_sec + duration/1000000;
      //long usec=t1.tv_usec + duration%1000000;
      int sec=t1.tv_sec + duration/1000000;
      int usec=t1.tv_usec + duration%1000000;
      if (usec > 1000000) {
        sec++;
        usec -= 1000000;
      }
      //set time in image
      Img_->SetTime(sec, usec);

      switch (camtype_) {
        #ifdef BIAS_HAVE_DCAM
        case XB3:
          AllImages_->SetTime(sec, usec);
          ImgCenter_->SetTime(sec, usec);
          ImgRight_->SetTime(sec, usec);
          break;
        #endif
        #ifdef BIAS_HAVE_SWISSRANGER
        case SR:
          ImgAmp_->SetTime(sec, usec);
        case KINECT:
          ImgDepth_->SetTime(sec, usec);
          break;
        #endif
        #ifdef BIAS_HAVE_PMD_PMDTec
        case PMDTEC:
          ImgDepth_->SetTime(sec, usec);
          ImgAmp_->SetTime(sec, usec);
          ImgInt_->SetTime(sec, usec);
          break;
        #endif
        #ifdef BIAS_HAVE_PMD_PMDZess
        case ZESS:
          ImgDepth_->SetTime(sec, usec);
          ImgAmp_->SetTime(sec, usec);
          break;
        #endif
        #ifdef BIAS_HAVE_KINECT2
        case FREENECT2:
          Img_->SetTime(sec, usec);
          ImgDepth_->SetTime(sec, usec);
          ImgInt_->SetTime(sec, usec);
          break;
        #endif
        case INVALID:
        default:
          // nothing
          break;
      }
    }
    // write images to buffer
    if(*WriteToBuffer_){
      //cout<<"L"<<flush;
      BufferAccess_->Lock();

      if((*WriteIndex_ + 1) % ImageBuffer_->size() != *ReadIndex_ ){
        //cout<<"U"<<flush;
        BufferAccess_->Unlock();
        //std::cout << "Write Image to buffer at pos:  " << *WriteIndex_ << std::endl;
        if(Img_ != NULL){
          if((*ImageBuffer_)[*WriteIndex_] == NULL)
            (*ImageBuffer_)[*WriteIndex_] = new BIAS::Camera<unsigned char>;
          *(*ImageBuffer_)[*WriteIndex_]         = *Img_;
        }
        if(ImgCenter_ != NULL){
          if((*ImageBufferCenter_)[*WriteIndex_] == NULL)
            (*ImageBufferCenter_)[*WriteIndex_] = new BIAS::Camera<unsigned char>;
          *(*ImageBufferCenter_)[*WriteIndex_]   = *ImgCenter_;
        }
        if(ImgRight_ != NULL){
          if((*ImageBufferRight_)[*WriteIndex_] == NULL)
            (*ImageBufferRight_)[*WriteIndex_] = new BIAS::Camera<unsigned char>;
          *(*ImageBufferRight_)[*WriteIndex_]   = *ImgRight_;
        }
        if(ImgDepth_ != NULL){
          if((*ImageBufferPMDDepth_)[*WriteIndex_] == NULL)
            (*ImageBufferPMDDepth_)[*WriteIndex_] = new BIAS::Camera<float>;
          *(*ImageBufferPMDDepth_)[*WriteIndex_]   = *ImgDepth_;
        }
        if(ImgAmp_ != NULL){
          if((*ImageBufferPMDAmp_)[*WriteIndex_] == NULL)
            (*ImageBufferPMDAmp_)[*WriteIndex_] = new BIAS::Camera<float>;
          *(*ImageBufferPMDAmp_)[*WriteIndex_]   = *ImgAmp_;
        }
        if(ImgInt_ != NULL){
          if((*ImageBufferPMDInt_)[*WriteIndex_] == NULL)
            (*ImageBufferPMDInt_)[*WriteIndex_] = new BIAS::Camera<float>;
          *(*ImageBufferPMDInt_)[*WriteIndex_]   = *ImgInt_;
        }

        //cout<<"L"<<flush;
        BufferAccess_->Lock();
        *WriteIndex_ = (*WriteIndex_ + 1) % ImageBuffer_->size();
        //cout<<"U"<<flush;
        BufferAccess_->Unlock();
      } else {
        //cout<<"U"<<flush;
        BufferAccess_->Unlock();
        /*std::cout << "Puffer schreiben ueberholt den auslesenden Thread, Frames gehen verloren.\n"
                  << "Thread: " << this->GetId() << " wird kurz schlafen gelegt."
                  << std::endl;*/
        //cout<<"Thread:"<<threadNumber_<<" overtaking!"<<endl;
#ifndef WIN32
        usleep(BUFFER_GRAB_SLEEP);
#else
        Sleep(BUFFER_GRAB_SLEEP/1000);
#endif
      }

    }

#ifdef BIAS_DEBUG
    //cout <<threadNumber_;
#endif   
    //i am done
    //lock server mutex here
    mutexServerCondition_->Lock();
    ServerCondition_->Signal();
    mutexServerCondition_->Unlock();
    ClientCondition_->Wait(); //calls unlock(mutex), waits for signal(condition), calls lock(mutex)
  }
  // on exit, unlock client condition in case grab thread is recreated
  mutexClientCondition_->Unlock();
  return NULL;
}


/************************* GrabMasterThread *******************************
 **************************************************************************/

GrabMasterThread::
GrabMasterThread(std::vector<BIAS::VideoSource*> *Cameras,
                 std::vector<wxCondition*>                     *ClientCondition,
                 std::vector<wxCondition*>        *ServerCondition,
                 std::vector<wxMutex*>                         *mutexClientCondition,
                 std::vector<wxMutex*>            *mutexServerCondition)

  :wxThread(wxTHREAD_DETACHED)
{

  //mutex for conditional
  mutexClientCondition_= mutexClientCondition;
  mutexServerCondition_= mutexServerCondition;

  //the conditionals
  ClientCondition_     = ClientCondition;
  ServerCondition_     = ServerCondition;

  Cameras_ = Cameras;

  FrameCounter_ = 0;
  StopWatch_.Start();


}

/**
  @brief
  @author fkellner
*/
void* GrabMasterThread::
Entry()
{
  // lock all server conditions
  for ( unsigned int i = 0; i < ServerCondition_->size(); i++)
    (*mutexServerCondition_)[i]->Lock();
  
  while(!TestDestroy()){
    StopWatch_.Stop();
    fps_ = 1000000.0 / StopWatch_.GetRealTime();
    StopWatch_.Reset();
    StopWatch_.Start();
    FrameCounter_++;
    //cout << "real fps " << fps_ << " number: " << FrameCounter_ << endl;


#ifdef BIAS_DEBUG
    //cout << "*" <<flush;
#endif

    //send signal to grabthreads to continue
    for ( unsigned int i = 0; i < ServerCondition_->size(); i++) {
      if ((*Cameras_)[i] != NULL) {
        //wait for serverCondition to be signaled from each active camera (unlock, wait, lock)
        (*ServerCondition_)[i]->Wait();
      }
    }
#ifdef BIAS_DEBUG
    //cout << "+" <<flush;
#endif

    // set flags to false for all active cameras
    for ( unsigned int i = 0; i < ServerCondition_->size(); i++) {
      //tell waiting grab threads to continue
      if ((*Cameras_)[i] != NULL) {
        (*mutexClientCondition_)[i]->Lock();
        (*ClientCondition_)[i]->Signal();
        (*mutexClientCondition_)[i]->Unlock();
      }
    }

  }
  return NULL;
}
