/*
 * ShowCamWxVideoSourceFactory.cpp
 *
 *  Created on: 11.02.2011
 *      Author: bravo
 */

#include <Tools/BIASShowCamWX/ShowCamWxVideoSourceFactory.hh>

#  include <Gui/VideoSource_Controller.hh>
#ifdef BIAS_HAVE_V4L
#  include <VideoSource/VideoSource_V4L.hh>
#endif // BIAS_HAVE_V4L

#ifdef BIAS_HAVE_DCAM
#  include <Gui/VideoSource_Controller_XB3.hh>
#  include <VideoSource/VideoSource_DCAM_BumbleBee.hh>
#endif // BIAS_HAVE_DCAM

#ifdef BIAS_HAVE_PMD_PMDTec
#  include <Gui/VideoSource_Controller_PMD.hh>
#  include <VideoSource/VideoSource_PMD.hh>
#endif // BIAS_HAVE_PMD_PMDTec

#ifdef BIAS_HAVE_PMD_PMDZess
#  include <Gui/VideoSource_Controller_PMDZess.hh>
#  include <VideoSource/VideoSource_PMDZess.hh>
#endif // BIAS_HAVE_PMD_PMDZess

#ifdef BIAS_HAVE_SWISSRANGER
#  include <Gui/VideoSource_Controller_SwissRanger.hh>
#  include <VideoSource/VideoSource_SwissRanger.hh>
#endif

#include <Gui/VideoSource_Controller_Kinect.hh>

#ifdef BIAS_HAVE_KINECT
#  include <VideoSource/VideoSource_Kinect.hh>
#endif

#ifdef BIAS_HAVE_OPENNI
#  include <VideoSource/VideoSource_OpenNI.hh>
#endif

#ifdef BIAS_HAVE_KINECT2
#  include <VideoSource/VideoSource_Kinect2.hh>
#endif

#ifdef BIAS_HAVE_DSHOW
#  include <VideoSource/VideoSource_DSHOW.hh>
#endif

#ifdef BIAS_HAVE_UEYE
#  include <VideoSource/VideoSource_uEye.hh>
#endif

#include <sstream>
#include <VideoSource/VideoSource_Shm.hh>
#include <VideoSource/VideoSource_Net.hh>

using namespace std;

//redeclare static members
int BIAS::ShowCamWxVideoSourceFactory::Win32DCAMNumber_;

namespace BIAS {

ShowCamWxVideoSourceFactory::ShowCamWxVideoSourceFactory() {
  Win32DCAMNumber_=0;
}

ShowCamWxVideoSourceFactory::~ShowCamWxVideoSourceFactory() {
}

VideoSource_Controller_Base* ShowCamWxVideoSourceFactory::
CreateController(CameraType type, VideoSource *cam, wxWindow *parent,
      const std::string& title, 
      const wxPoint& pos, 
      const wxSize& size, 
      long style, 
      const wxString& name)
{
  VideoSource_Controller_Base* controller = NULL;
  
  switch(type) {
#ifdef BIAS_HAVE_PMD_PMDZess
    case PMDUSB:
      controller = new VideoSource_Controller_PMDZess(cam, parent, "PMD Controller");
      break;
#endif
#ifdef BIAS_HAVE_PMD_PMDTec
    case PMD:
      controller = new VideoSource_Controller_PMD(cam, parent, "PMD Controller");
      break;
#endif
#ifdef BIAS_HAVE_SWISSRANGER
    case SR:
      controller = new VideoSource_Controller_SwissRanger(cam, parent, "SwissRanger Controller");
      break;
#endif
#ifdef BIAS_HAVE_DCAM
    case XB3:
      controller = new VideoSource_Controller_XB3(cam, parent, "XB3 Controller");
      break;
    case DCAM:
      controller = new VideoSource_Controller(cam, parent, "DCAM Controller");
      break;
#endif
#ifdef BIAS_HAVE_V4L
    case V4L:
      controller = new VideoSource_Controller(cam, parent, "V4L");
      double min, max;
      dynamic_cast<VideoSource_V4L*>(cam)->GetBrightnessRange(min,max);
      controller->SetBrightnessRange(min,max);
      dynamic_cast<VideoSource_V4L*>(cam)->GetContrastRange(min,max);
      controller->SetContrastRange(min,max);
      break;
#endif
    case KINECT:
      controller = new VideoSource_Controller_Kinect(cam, parent, "Kinect (freenect) Controller");
      break;
    case OPENNI:
      controller = new VideoSource_Controller_Kinect(cam, parent, "Kinect (openni) Controller");
      break;
    default: {
      string thetitle = "Camera: "+title;
      controller = new VideoSource_Controller(cam, parent, thetitle);
      double min, max;
      cam->GetBrightnessRange(min,max);
      controller->SetBrightnessRange((unsigned)min,(unsigned)max);
      cam->GetContrastRange(min,max);
      controller->SetContrastRange((unsigned)min,(unsigned)max);
      break;
    }
  }
  return controller;
}

VideoSource* ShowCamWxVideoSourceFactory::
CreateDCAM(bool firewireB, bool user_mode, int dcam_mode,
    float fps, int Format7BpP, int Format7ColorMode, 
    Vector2<int> Format7LeftTop, Vector2<int> Format7WidthHeight)
{
  
#ifdef BIAS_HAVE_DCAM
  VideoSource_DCAM *cam = new VideoSource_DCAM;
  cam = new VideoSource_DCAM;
  cam->SetDebugLevel(D_DCAM);
  cam->SetFirewireB(firewireB);
      
  if(user_mode) {
    dc1394video_mode_t mode = (dc1394video_mode_t)dcam_mode; 

    // choose between normal mode and format 7
    if (mode >= DC1394_VIDEO_MODE_FORMAT7_0) {
      cam->SetFormat7(mode - DC1394_VIDEO_MODE_FORMAT7_MIN, Format7BpP,
          (dc1394color_coding_t) Format7ColorMode);
      cam->SetLeftTop(Format7LeftTop[0], Format7LeftTop[1]);
      cam->SetSize(Format7WidthHeight[0], Format7WidthHeight[1]);
    } else {
      cam->SetModeAndFramerate(mode,fps);
    }
  }
  
  return cam;
#else
  return NULL;
#endif
}

VideoSource* ShowCamWxVideoSourceFactory::
CreatePMDZess(std::string bitfile)
{
#ifdef BIAS_HAVE_PMD_PMDZess
  VideoSource_PMDZess *cam = new VideoSource_PMDZess;
  cam->SetBitFile(bitfile);
  return cam;
#else
  return NULL;
#endif
}

VideoSource* ShowCamWxVideoSourceFactory::
CreateUEye()
{
#ifdef BIAS_HAVE_UEYE
  VideoSource_uEye *cam = new VideoSource_uEye();
  return cam;
#else
  return NULL;
#endif
}

VideoSource* ShowCamWxVideoSourceFactory::
CreateFromTypeIfSupported(int type)
{
  return CreateFromTypeIfSupported((CameraType)type);
}

VideoSource* ShowCamWxVideoSourceFactory::
CreateFromTypeIfSupported(CameraType type)
{
  VideoSource* cam = NULL;
  switch (type) {
   case PMDSHM:
#ifdef BIAS_HAVE_TIFF
//    cam = new VideoSource_ShmPMD;
#endif
  case SHM: // SHM
    cam = new VideoSource_Shm;      
    break;
  case NET: // NET
    cam = new VideoSource_Net;      
    break;
  case SR:
#ifdef BIAS_HAVE_SWISSRANGER
    cam = new VideoSource_SwissRanger;
#endif
    break;
  case KINECT:
#ifdef BIAS_HAVE_KINECT
    cam = new VideoSource_Kinect;
#endif
    break;
  case OPENNI:
#ifdef BIAS_HAVE_OPENNI
    cam = new VideoSource_OpenNI;
#endif
    break;
  case FREENECT2:
#ifdef BIAS_HAVE_KINECT2
    cam = new VideoSource_Kinect2;
#endif
    break;
  default:
    BIASERR("try specialized method for " << type << " camera");
    break;
  }
  return cam;
}

VideoSource* ShowCamWxVideoSourceFactory::
CreateV4L()
{
  VideoSource *cam = NULL;
#ifdef BIAS_HAVE_V4L      
  cam = new VideoSource_V4L;      
#endif
#ifdef BIAS_HAVE_DSHOW
  cam = new VideoSource_DSHOW(false);  
  dynamic_cast<VideoSource_DSHOW*>(cam)->SetWaitForNew(true);
#endif
  return cam;
}

VideoSource* ShowCamWxVideoSourceFactory::
CreateBumbleBee()
{
  VideoSource *cam = NULL;
  
#ifdef BIAS_HAVE_DCAM

  cam = new VideoSource_DCAM_BumbleBee;
  cam->SetDebugLevel(D_DCAM|D_DCAM_F7);

#endif
  
  return cam; 
}

VideoSource* ShowCamWxVideoSourceFactory::
CreatePMD(string sourcePlugin,string procPlugin)
{
   VideoSource* cam = NULL;
#ifdef BIAS_HAVE_PMD_PMDTec
   cam = new VideoSource_PMD;
   (dynamic_cast<VideoSource_PMD*>(cam))->SetSourcePlugin(sourcePlugin);
   (dynamic_cast<VideoSource_PMD*>(cam))->SetProcessingPlugin(procPlugin);
#endif
  return cam;
}

int ShowCamWxVideoSourceFactory::
OpenDevice(int type, VideoSource* cam, std::string deviceName, int param)
{
  
  if (cam == NULL) return -1;
  
  switch (type) {
  
  case ShowCamWxVideoSourceFactory::XB3:
  case ShowCamWxVideoSourceFactory::DCAM:
    if (deviceName != "")  {
      cam->AddDebugLevel(D_DCAM);

      if (cam->OpenDevice(atoi(deviceName.c_str())) < 0)
        return -1;
      else{
        cout<<"Camera number " << deviceName << " successfully opened"<<endl;
        return 0;
      }
    } else {
#ifdef WIN32
      // this does not work if not all cameras are dcam!
      if (cam->OpenDevice(Win32DCAMNumber_++) < 0) {
        cam->CloseDevice();
        delete cam;
        cam = NULL;
      }
#else
      // check if there are multiple cameras with no device name
      if (cam->OpenDevice() < 0) {
        delete cam;
        cam = NULL;
      }
      return -2;
#endif // WIN32
    }
    break;
  case ShowCamWxVideoSourceFactory::PMD: {
#ifdef BIAS_HAVE_PMD_PMDTec
    VideoSource_PMD *pmd = dynamic_cast<VideoSource_PMD*>(cam);
    if (pmd->OpenDevice()==0) {
      pmd->SetFPNCalibrationOn(false);
      return 0;
    } else {
      return -1;
    }
#endif
    break;
  }
  case ShowCamWxVideoSourceFactory::NET: {
    VideoSource_Net *net = dynamic_cast<VideoSource_Net*>(cam);
    if (net!=NULL) {
      if (param == 0) {
        return net->OpenDevice(deviceName.c_str());
      } else {
        return net->OpenDevice(deviceName, param);
      }
    }
    break;
  }
  case ShowCamWxVideoSourceFactory::OPENNI: {
    if (deviceName != "") {
      return cam->OpenDevice(atoi(deviceName.c_str()));
    } else {
      return cam->OpenDevice();
    }
    break;
  }
  case ShowCamWxVideoSourceFactory::FREENECT2: {
    if (deviceName != "") {
      return cam->OpenDevice(atoi(deviceName.c_str()));
    } else {
      return cam->OpenDevice();
    }
    break;
  }
  default:
    if (deviceName != "") {
      cout<<"Opening device:"<<deviceName<<endl;
      return (cam->OpenDevice(deviceName.c_str()));
    } else {
      cout<<"Opening device without device name."<<endl;
      return cam->OpenDevice();
    }
    break;
  }
  
  
  return -1;
}

int ShowCamWxVideoSourceFactory::InitImages(int type, VideoSource *cam, ImageBase *im1, ImageBase *im2, ImageBase *im3, ImageBase *im4) {
  
  switch (type) {
  
  case XB3:
#ifdef BIAS_HAVE_DCAM
    if (dynamic_cast<VideoSource_DCAM_BumbleBee*>(cam) != NULL)
    {
      dynamic_cast<VideoSource_DCAM_BumbleBee*>(cam)->InitSingleImage(*im1);
      dynamic_cast<VideoSource_DCAM_BumbleBee*>(cam)->InitSingleImage(*im2);
      dynamic_cast<VideoSource_DCAM_BumbleBee*>(cam)->InitSingleImage(*im3);
      cam->InitImage(*im4);
    }
#endif
    break;
  case PMDUSB:
#ifdef BIAS_HAVE_PMD_PMDZess
// init 2D image for usb pmd cameras
    if (dynamic_cast<VideoSource_PMDZess*>(cam) != NULL)
    {
      im1->Init((im2->GetWidth()+1)*3, im2->GetHeight()+1);
      im1->SetColorModel(ImageBase::CM_Grey);
    }
#endif // BIAS_HAVE_PMD_PMDZess
    break;
  case PMD:
#ifdef BIAS_HAVE_PMD_PMDTec
      dynamic_cast<VideoSource_PMD*>(cam)->InitDepthImage(*im1);
      dynamic_cast<VideoSource_PMD*>(cam)->InitModCoeffImage(*im2);
      dynamic_cast<VideoSource_PMD*>(cam)->InitIntensityImage(*im3);
#endif
    break;
  case PMDSHM:
#ifdef BIAS_HAVE_PMD_PMDTec
//      dynamic_cast<VideoSource_ShmPMD*>(cam)->GrabSingleDepth(*im1);
//      dynamic_cast<VideoSource_ShmPMD*>(cam)->GrabSingleModCoeff(*im2);
#endif
    break;
  case KINECT:
  case OPENNI:
    dynamic_cast<VideoSource_Kinect_Base*>(cam)->InitDepthImage(*im1);
    break;
  #ifdef BIAS_HAVE_KINECT2
  case FREENECT2:
    dynamic_cast<VideoSource_Kinect2*>(cam)->InitImage(*im1);
    dynamic_cast<VideoSource_Kinect2*>(cam)->InitDepthImage(*im2);
    dynamic_cast<VideoSource_Kinect2*>(cam)->InitIrImage(*im3);
    break;
  #endif
  default:
    // nothing to do
    break;
  }
  return 0;
}


}
