/*
 * ShowCamWxVideoSourceFactory.hh
 *
 *  Created on: 11.02.2011
 *      Author: bravo
 */

#ifndef SHOWCAMWXVIDEOSOURCEFACTORY_HH_
#define SHOWCAMWXVIDEOSOURCEFACTORY_HH_

#include <Base/Common/BIASpragmaStart.hh>
#include <bias_config.h>
#include <Gui/VideoSource_Controller_Base.hh>
#include <VideoSource/VideoSource_Base.hh>
#include <Base/Math/Vector2.hh>

namespace BIAS {

class ShowCamWxVideoSourceFactory {
public:

  enum CameraType {
    NONE=0,
    DCAM,
    PMD,
    PMDUSB,
    PMDSHM,
    UEYE,
    SHM,
    NET,
    V4L,
    SR,
    XB3,
    DSHOW,
    KINECT,
    OPENNI,
    FREENECT2
  };

  static std::vector<std::string> CameraNames() {
    std::vector<std::string> cameras;
    cameras.push_back("NONE");
    cameras.push_back("DCAM");
    cameras.push_back("PMD");
    cameras.push_back("PMDUSB");
    cameras.push_back("PMDSHM");
    cameras.push_back("UEYE");
    cameras.push_back("SHM");
    cameras.push_back("NET");
    cameras.push_back("V4L");
    cameras.push_back("SR");
    cameras.push_back("XB3");
    cameras.push_back("DSHOW");
    cameras.push_back("KINECT");
    cameras.push_back("OPENNI");
    cameras.push_back("FREENECT2");
    return cameras;
  }
  
  ShowCamWxVideoSourceFactory();
  virtual ~ShowCamWxVideoSourceFactory();
  
  static VideoSource_Controller_Base* CreateController(CameraType type, VideoSource *cam, wxWindow *parent, 
      const std::string& title = "", 
      const wxPoint& pos = wxDefaultPosition, 
      const wxSize& size = wxDefaultSize, 
      long style = wxDEFAULT_FRAME_STYLE, 
      const wxString& name = wxFrameNameStr);
  
  static VideoSource* CreateFromTypeIfSupported(CameraType type);
  static VideoSource* CreateFromTypeIfSupported(int type);
  
  static VideoSource* CreateDCAM(bool firewireB, bool user_mode, int dcam_mode=-1, 
      float fps=-1, int Format7BpP=-1, int Format7ColorMode_=-1, 
      Vector2<int> Format7LeftTop_=Vector2<int>(-1,-1), Vector2<int> Format7WidthHeight=Vector2<int>(-1,-1));

  static VideoSource* CreatePMD(std::string sourcePlugin, std::string procPlugin);
  static VideoSource* CreatePMDZess(std::string bitfile);
  static VideoSource* CreateUEye();
  static VideoSource* CreateV4L();
  static VideoSource* CreateBumbleBee();
  
  static int OpenDevice(int type, VideoSource* cam, std::string deviceName, int param=-1);
  static int InitImages(int type, VideoSource* cam, ImageBase *im1, ImageBase *im2 = NULL, ImageBase *im3 = NULL, ImageBase *im4 = NULL);
  
protected:
  static int Win32DCAMNumber_;

};

}

#include <Base/Common/BIASpragmaEnd.hh>

#endif /* SHOWCAMWXVIDEOSOURCEFACTORY_HH_ */
