/*
   This file is part of the BIAS library (Basic ImageAlgorithmS).

   Copyright (C) 2003-2009    (see file CONTACT for details)
   Multimediale Systeme der Informationsverarbeitung
   Institut fuer Informatik
   Christian-Albrechts-Universitaet Kiel


   BIAS is free software; you can redistribute it and/or modify
   it under the terms of the GNU Lesser General Public License as published by
   the Free Software Foundation; either version 2.1 of the License, or
   (at your option) any later version.

   BIAS is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public License
   along with BIAS; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "BVWXMainFrame.hh"
#include <Gui/FilterDialogMean.hh>
#include <Gui/FilterDialogGradientSobel3x3.hh>
#include <Gui/FilterDialogRescale.hh>
#include <Gui/FilterDialogGauss.hh>
#include <Gui/FilterDialogMedian.hh>
#include <Gui/FilterDialogCannyEdge.hh>
#include <Gui/wxVideoSettingsDialog.h>
#include <Utils/StringUtils.hh>
#include <Utils/ImageValueBar.hh>
#ifdef BIAS_HAVE_FFMPEG
#include <VideoSink/VideoSink_FFmpeg.hh>
#endif
#include <wx/printdlg.h>
#include <wx/string.h>
#include <wx/event.h>
#include <wx/artprov.h>
#include <wx/aboutdlg.h>
#include <wx/progdlg.h>
#include <string>
#include <sstream>
#include <limits>

typedef unsigned char UCHAR;

using namespace std;
using namespace BIAS;

// icon
#if !defined(__WXPM__)
#  include <img/biasviewwx.xpm>
#  include <img/play.xpm>
#  include <img/stopplaying.xpm>
#endif

BEGIN_EVENT_TABLE(BVWXMainFrame, wxFrame)
EVT_MENU    (BVWXMainFrame::ID_Exit,       BVWXMainFrame::OnExit)
EVT_MENU    (BVWXMainFrame::ID_Load,       BVWXMainFrame::OnLoad)
EVT_MENU    (BVWXMainFrame::ID_RemoveCurrent, BVWXMainFrame::OnRemoveCurrent)
EVT_MENU    (BVWXMainFrame::ID_RemoveAllButCurrent, BVWXMainFrame::OnRemoveAllButCurrent)
EVT_MENU    (BVWXMainFrame::ID_Info,       BVWXMainFrame::OnInfo)
EVT_MENU    (BVWXMainFrame::ID_MetaData,   BVWXMainFrame::OnMetaData)
EVT_MENU    (BVWXMainFrame::ID_SaveAs,     BVWXMainFrame::OnSaveAs)
EVT_MENU	  (BVWXMainFrame::ID_SaveCurrent, BVWXMainFrame::OnSaveCurrent)
EVT_MENU    (BVWXMainFrame::ID_SaveMovie,     BVWXMainFrame::OnSaveAsMovie)

EVT_MENU	  (BVWXMainFrame::ID_PrintCurrent, BVWXMainFrame::OnPrintCurrent)
EVT_MENU	  (BVWXMainFrame::ID_Print, BVWXMainFrame::OnPrint)
EVT_MENU    (BVWXMainFrame::ID_SaveList,   BVWXMainFrame::OnSaveList)
EVT_MENU    (BVWXMainFrame::ID_ImageValues, BVWXMainFrame::OnImageValues)
EVT_MENU	  (BVWXMainFrame::ID_DrawROI, BVWXMainFrame::OnDrawROI)
EVT_MENU	  (BVWXMainFrame::ID_Histogramm, BVWXMainFrame::OnHistogramm)
EVT_MENU	  (BVWXMainFrame::ID_ImageValueBar, BVWXMainFrame::OnImageValueBar)

EVT_MENU    (BVWXMainFrame::ID_KeyInfo, BVWXMainFrame::OnKeyInfo)
EVT_MENU    (wxID_ABOUT,   BVWXMainFrame::OnAbout)

EVT_BUTTON  (BVWXMainFrame::ID_Next,       BVWXMainFrame::OnNext)
EVT_BUTTON  (BVWXMainFrame::ID_Prev,       BVWXMainFrame::OnPrev)
EVT_BUTTON  (BVWXMainFrame::ID_RunStop,    BVWXMainFrame::OnRunStop)
EVT_BUTTON  (BVWXMainFrame::ID_Run,    BVWXMainFrame::OnRun)
EVT_BUTTON  (BVWXMainFrame::ID_Stop,    BVWXMainFrame::OnStop)
EVT_BUTTON  (BVWXMainFrame::ID_Histogramm, BVWXMainFrame::OnHistogramm)
EVT_BUTTON  (BVWXMainFrame::ID_ImageValueBar, BVWXMainFrame::OnImageValueBar)
EVT_SPINCTRL(BVWXMainFrame::ID_Delay,      BVWXMainFrame::OnDelay)
EVT_CHECKBOX(BVWXMainFrame::ID_Fit,        BVWXMainFrame::OnFit)
EVT_CHECKBOX(BVWXMainFrame::ID_KeepScrollPosition, BVWXMainFrame::OnKeepScrollPosition)

EVT_MENU (BVWXMainFrame::ID_Next,    BVWXMainFrame::OnNext)
EVT_MENU (BVWXMainFrame::ID_Prev,    BVWXMainFrame::OnPrev)
EVT_MENU (BVWXMainFrame::ID_RunStop, BVWXMainFrame::OnRunStop)
EVT_MENU  (BVWXMainFrame::ID_Run,    BVWXMainFrame::OnRun)
EVT_MENU  (BVWXMainFrame::ID_Stop,    BVWXMainFrame::OnStop)
EVT_TOOL_ENTER(BVWXMainFrame::ID_ToolBar, BVWXMainFrame::OnToolEnter)

EVT_MENU (BVWXMainFrame::ID_ChangeChannel_1,   BVWXMainFrame::OnChangeChannel_1)
EVT_MENU (BVWXMainFrame::ID_ChangeChannel_2,   BVWXMainFrame::OnChangeChannel_2)
EVT_MENU (BVWXMainFrame::ID_ChangeChannel_3,   BVWXMainFrame::OnChangeChannel_3)
EVT_MENU (BVWXMainFrame::ID_ChangeChannel_4,   BVWXMainFrame::OnChangeChannel_4)
EVT_MENU (BVWXMainFrame::ID_ChangeChannel_5,   BVWXMainFrame::OnChangeChannel_5)
EVT_MENU (BVWXMainFrame::ID_ChangeChannel_6,   BVWXMainFrame::OnChangeChannel_6)
EVT_MENU (BVWXMainFrame::ID_ChangeChannel_7,   BVWXMainFrame::OnChangeChannel_7)
EVT_MENU (BVWXMainFrame::ID_ChangeChannel_8,   BVWXMainFrame::OnChangeChannel_8)
EVT_MENU (BVWXMainFrame::ID_ChangeChannel_ALL, BVWXMainFrame::OnChangeChannel_ALL)

EVT_MENU (BVWXMainFrame::ID_FilterCannyEdge, BVWXMainFrame::OnFilterCannyEdge)
EVT_MENU (BVWXMainFrame::ID_FilterGauss,     BVWXMainFrame::OnFilterGauss)
EVT_MENU (BVWXMainFrame::ID_FilterGradientSobel3x3, BVWXMainFrame::OnFilterGradientSobel3x3)
EVT_MENU (BVWXMainFrame::ID_FilterMean,      BVWXMainFrame::OnFilterMean)
EVT_MENU (BVWXMainFrame::ID_FilterMedian,    BVWXMainFrame::OnFilterMedian)
EVT_MENU (BVWXMainFrame::ID_FilterRescale,   BVWXMainFrame::OnFilterRescale)

EVT_TIMER   (BVWXMainFrame::ID_Timer, BVWXMainFrame::OnTimer)
EVT_IDLE    (BVWXMainFrame::OnIdle)
EVT_SIZE    (BVWXMainFrame::OnSize)
EVT_COMMAND  (BVWXMainFrame::ID_FilterWindow, wxEVT_FILTER_SAVE, BVWXMainFrame::OnFilterSave)

END_EVENT_TABLE()


// ctor
BVWXMainFrame::BVWXMainFrame(const wxString& title,
                             const wxPoint& pos,
                             const wxSize& size)
: wxFrame((wxFrame*)NULL, BIAS::ID_WIN_BVWX_MAINFRAME, title, pos, size),
  _ImageCanvasUC(NULL),
  _ImageCanvasFloat(NULL),
  _FitCheckBox(NULL),
  _KeepScrollPositionCheckBox(NULL)
{
  SetIcon(wxIcon(biasviewwx_xpm));
  _CurrentImage     = 0;
  _DefaultFile      = wxT("ucim.pgm");
  _DefaultDir       = wxT("");
  _BlockAnimation   = true;
  _SelectedChannel  = 0;
  _ShowImageValueBar = false;
  _ShowHistogramm = false;
  _HistoPosition = wxPoint(10,10);
  _ValueBarPosition = wxPoint(150,10);
  _HistoSize = wxSize(100,100);
  _ValueBarSize = wxSize(200,300);

  // init timer
  _Timer.SetOwner(this, ID_Timer);

  // create file menu
  wxMenu* fileMenu = new wxMenu;
  fileMenu->Append(ID_Load,
                   wxT("&Open/Load...\tCtrl-O"),
                   wxT("Open a file from disk for loading"));
  fileMenu->Append(ID_RemoveCurrent,
                   wxT("&Remove current image...\tCtrl-R"),
                   wxT("Remove current file from viewer list"));
  fileMenu->Append(ID_RemoveAllButCurrent,
                   wxT("Remove &all but current image"),
                   wxT("Remove all files from viewer list "));

  fileMenu->AppendSeparator();

  fileMenu->Append(ID_Info, wxT("Image Info...\tCtrl-I"));
  fileMenu->Append(ID_MetaData, wxT("Meta Data...\tCtrl-M"));
  fileMenu->Append(ID_ImageValues, wxT("Image Values..."));

  fileMenu->AppendSeparator();

  fileMenu->Append(ID_SaveAs,
                   wxT("&Save image as\tAlt-S"),
                   wxT("Save image to disk"));
  fileMenu->Append(ID_SaveCurrent,
									 wxT("&Save current image\tAlt-Ctrl-S"),
                   wxT("Save image as currently displayed with storage type unsigned char to disk."));
  fileMenu->Append(ID_SaveList,
                   wxT("&Save images list to .lst file ...\tCtrl-S"),
                   wxT("save all list of all image filenames to .lst disk file."));

  fileMenu->AppendSeparator();
  fileMenu->Append(ID_SaveMovie,
                   wxT("Save images as movie"),
                   wxT("Save current images in list as movie to disk using ffmpeg."));
  fileMenu->AppendSeparator();

  //  fileMenu->Append(ID_PrintCurrent,
  //                    wxT("&Print current image \tAlt-P"),
  //                    wxT("Print only current image."));
  //   fileMenu->Append(ID_Print,
  // 									 wxT("Print &multiple image \tAlt-m"),
  // 									 wxT("Open Choise dialog and print multiple images."));

  //   fileMenu->AppendSeparator();

  fileMenu->Append(ID_Exit, wxT("E&xit"), wxT("Exit this application"));


  // create help menu
  wxMenu* helpMenu = new wxMenu;
  helpMenu->Append(ID_KeyInfo, wxT("&Keys...\tF1"));
  helpMenu->Append(ID_About, wxT("&About...\tCtrl-F1"));

  // create channel selection menu
  wxMenu* channelMenu = new wxMenu;
  channelMenu->Append(ID_ChangeChannel_ALL, wxT("&0 All Channels\tShift-F9"));

  channelMenu->AppendSeparator();

  channelMenu->Append(ID_ChangeChannel_1, wxT("&1 channel (red?)\tShift-F1"));
  channelMenu->Append(ID_ChangeChannel_2, wxT("&2 channel (green?)\tShift-F2"));
  channelMenu->Append(ID_ChangeChannel_3, wxT("&3 channel (blue?)\tShift-F3"));
  channelMenu->Append(ID_ChangeChannel_4, wxT("&4 channel (alpha?)\tShift-F4"));
  channelMenu->Append(ID_ChangeChannel_5, wxT("&5 channel (H for RGB)\tShift-F5"));
  channelMenu->Append(ID_ChangeChannel_6, wxT("&6 channel (S for RGB)\tShift-F6"));
  channelMenu->Append(ID_ChangeChannel_7, wxT("&7 channel (L for RGB)\tShift-F7"));
  channelMenu->Append(ID_ChangeChannel_8, wxT("&8 channel\tShift-F8"));
  
  wxMenu* viewMenu = new wxMenu;
  _DrawROICheckItem  =  new wxMenuItem(viewMenu, ID_DrawROI, wxString(
			wxT("Draw &ROI")), wxEmptyString, wxITEM_CHECK);
	viewMenu->Append(_DrawROICheckItem);
	_DrawROICheckItem->Check(false);

	_HistogramCheckItem  =  new wxMenuItem(viewMenu, ID_Histogramm, wxString(
			wxT("Show &Histogramm")), wxEmptyString, wxITEM_CHECK);
	viewMenu->Append(_HistogramCheckItem);
	_HistogramCheckItem->Check(false);

	_ValueBarCheckItem  =  new wxMenuItem(viewMenu, ID_ImageValueBar, wxString(
			wxT("Show &Legend")), wxEmptyString, wxITEM_CHECK);
	viewMenu->Append(_ValueBarCheckItem);
	_ValueBarCheckItem->Check(false);
	viewMenu->AppendSeparator();
	viewMenu->Append( ID_Prev, wxT("&Previous \tCTRL-LEFT"));
	viewMenu->Append( ID_Next, wxT("&Next \tCTRL-RIGHT"));
  viewMenu->Append( ID_RunStop, wxT("&Run/Stop \tCTRL-SPACE"));

  // create filter menu
  wxMenu* filterMenu = new wxMenu;
  filterMenu->Append(ID_FilterCannyEdge,
  		               wxT("&Canny Edge\tAlt-Ctrl-C"),
  		               wxT("Canny edge filter"));
  filterMenu->Append(ID_FilterGauss,
										 wxT("&Gauss\tAlt-Ctrl-G"),
										 wxT("Gauss filter"));
  filterMenu->Append(ID_FilterGradientSobel3x3,
  									 wxT("G&radient Sobel 3x3\tAlt-Ctrl-R"),
  									 wxT("Gradient sobel filter"));
  filterMenu->Append(ID_FilterMean,
    								 wxT("&Mean\tAlt-Ctrl-M"),
    								 wxT("Mean filter"));
  filterMenu->Append(ID_FilterMedian,
  		               wxT("Me&dian\tAlt-Ctrl-D"),
  		               wxT("Median filter"));
  filterMenu->Append(ID_FilterRescale,
  		               wxT("Resc&ale\tAlt-Ctrl-A"),
  		               wxT("Rescale"));

  // create menu bar
  wxMenuBar* menuBar = new wxMenuBar;
  menuBar->Append(fileMenu, wxT("&File"));
  menuBar->Append(channelMenu, wxT("&Channel"));
  menuBar->Append(viewMenu, wxT("&View"));
  menuBar->Append(filterMenu, wxT("Fil&ter"));
  menuBar->Append(helpMenu, wxT("&Help"));
  SetMenuBar(menuBar);

  // create status bar
  const int numFields = 3;
  CreateStatusBar(numFields);
  int widths[] = {30 * GetCharWidth(),-1, 60 * GetCharWidth()};// sizes of fields in array, negative means relative *variable* size
  // positive means absolute *fixed* size FRISO
  SetStatusWidths(numFields, widths);
  SetStatusText(wxT("Welcome to BiasViewWX"), 0);

  // create tool bar
  wxToolBar* toolBar = CreateToolBar(wxTB_HORIZONTAL | wxNO_BORDER |wxTB_TEXT,ID_ToolBar);
  //toolBar->CaptureMouse();

  //this tells especially windows that 24x24 px icons are used
  toolBar->SetToolBitmapSize(wxSize(24,24));

  toolBar->AddTool( ID_Load, wxT("Open"),
           wxArtProvider::GetIcon(wxART_FILE_OPEN,wxART_TOOLBAR), wxT("Open image(s)"));
  toolBar->AddTool( ID_SaveCurrent, wxT("Save Current"),
             wxArtProvider::GetIcon(wxART_FILE_SAVE,wxART_TOOLBAR),wxT("Save image as currently displayed with storage type unsigned char to disk.") );
  toolBar->AddTool( ID_SaveAs, wxT("Save As"),
             wxArtProvider::GetIcon(wxART_FILE_SAVE_AS,wxART_TOOLBAR),wxT("Save image with new filename.") );

  toolBar->AddSeparator();
  toolBar->AddTool( ID_Prev, wxT("Previous"),
 		     wxArtProvider::GetIcon(wxART_GO_BACK,wxART_TOOLBAR),wxT("[Ctrl-LEFT] Previous") );
  toolBar->AddTool( ID_Next, wxT("Next"),
  		     wxArtProvider::GetIcon(wxART_GO_FORWARD,wxART_TOOLBAR), wxT("Next [Ctrl-RIGHT]") );

  _RunTool = toolBar->AddTool( ID_RunStop, wxT("Run/Stop"), wxIcon(play_xpm), wxT("Run/Stop [Ctrl-SPACE]"));

  //_HistogrammButton = new wxButton(toolBar, ID_Histogramm, wxT(" Histogramm "));
  //_ValueBarButton = new wxButton(toolBar, ID_ImageValueBar, wxT(" Legend "));
  //toolBar->AddControl(_HistogrammButton);
  //toolBar->AddControl(_ValueBarButton);

  toolBar->AddSeparator();
  _ReverseCheckBox = new wxCheckBox(toolBar, ID_ReverseLoop, wxT("Reverse"));
  toolBar->AddControl(_ReverseCheckBox);

  toolBar->AddSeparator();
  wxSize sizeSize(60,25);
  _DelaySpin = new wxSpinCtrl(toolBar, ID_Delay, wxT("100"),
                              wxDefaultPosition, sizeSize,
                              wxSP_ARROW_KEYS, 1, 50000,100, wxT("Delay"));
  toolBar->AddControl(_DelaySpin);

  toolBar->AddSeparator();

  _FitCheckBox = new wxCheckBox(toolBar, ID_Fit, wxT("Fit Window"));
  toolBar->AddControl(_FitCheckBox);

  toolBar->AddSeparator();

  _KeepScrollPositionCheckBox = new wxCheckBox(toolBar,
                                               ID_KeepScrollPosition,
                                               wxT("Keep Position"));
  toolBar->AddControl(_KeepScrollPositionCheckBox);

  toolBar->Realize();
  SetToolBar(toolBar);

  /************************************************
   * the format of wxwindow layout is as follows:
   *
   *  /------------------------------------\
   *  |              TOOLBAR               |
   *  |------------------------------------|
   *  |               SIZER                |
   *  |------------------------------------|
   *  |             STATUSBAR              |
   *  \------------------------------------/
   *
   * so what we do - we add a _SizerWindow to our frame
   * (the frame handles resizing and refitting of a single
   * window AUTOMATICALLY) and then we add wxBoxSizer
   * _MainSizer which contains both canvases canvases as
   * children (so that the sizer decides what size should
   * the canvases be of)
   ***********************************************/
  _MainSizer = new wxBoxSizer(wxVERTICAL);
  _SizerWindow = new wxWindow(this,-1);

  _SizerWindow->SetSizer(_MainSizer);

  // set the shortcuts, but they only seldomly work
  std::vector<wxAcceleratorEntry> entries;
  entries.push_back(wxAcceleratorEntry(wxACCEL_NORMAL, 'r', ID_RunStop));
  entries.push_back(wxAcceleratorEntry(wxACCEL_NORMAL, 'h', ID_Histogramm));

  wxAcceleratorTable accel(entries.size(), (wxAcceleratorEntry*)& entries[0] );
  this->SetAcceleratorTable(accel);

  //_NewButton->SetFocus();
}


BVWXMainFrame::~BVWXMainFrame() { }


void BVWXMainFrame::SetFileList(const vector<string>& fileNames)
{
  _FileNames = fileNames;
  if (_FileNames.size() > 0)
  {
    LoadImageFromList(0);
  }
}

void BVWXMainFrame::UncompressImage(BIAS::ImageBase & image)
{
  if (image.GetColorModel()==BIAS::ImageBase::CM_PGR_XB3_F7M3_GBRG){
    // store info about raw image which will be lost by conversion
    stringstream ss;
    AddInfo(image, ss);
    infoRaw_ = ss.str();

    // uncompress raw to different size display image
    // the uncompression scheme can be selecetd via menu mode
    // ... or hard coded for now
    BIAS::ImageBase tmp = image;
    ImageConvert::DeinterleaveHorizontal(tmp, image);
  }
}


void BVWXMainFrame::LoadImageFromList(int index)
{
  if (_FileNames.size() == 0)
  {
    //The CANVASES should be emptied and refreshed (somehow they don't
    //do it automatically, so we have to make it M-A-N-U-A-L-L-Y

    //P.S. Somehow even with the following code written:
    if(_ImageCanvasUC!=NULL){
      _ImageCanvasUC->SetSize(5, 5);
      _ImageCanvasUC->Destroy();
      _ImageCanvasUC = NULL;
    }
    if(_ImageCanvasFloat!=NULL){
      _ImageCanvasFloat->SetSize(5, 5);
      _ImageCanvasFloat->Destroy();
      _ImageCanvasFloat = NULL;
    }

    _SizerWindow->SetSize(5, 5);
    _SizerWindow->Refresh();
    _SizerWindow->Update();

    //it doesn't work - the CANVAS remains to be shown.
    //So what we do is a little trick:

    /**
    ImageBase image = new ImageBase();
      Camera<unsigned char> camera(image);
      camera.ParseMetaData();
      camera.PrintAppData(info);
    **/

    return;
  }

  _CurrentImage = index;
  const string& fileName = _FileNames[index];
  //cout<<"Loading image with index:"<<_CurrentImage<<" and filename:"<<fileName<<endl;

  // load image
  ImageBase image;

  // true disk access
  if (ImageIO::Load(fileName.c_str(), image)!=0)
  {
    stringstream msg;
    msg << "Error loading " << fileName;
    BIASERR(msg.str());
    SetStatusText(AsciiToWx(msg.str()));
    wxMessageBox(AsciiToWx(msg.str()),
          wxT("Loading Error"), wxOK | wxICON_EXCLAMATION);
    return;
  }

  // handle compressed image sizes
  // where display image dimension
  // differs form orig image dimension.
  // e..g. X3B multicam images (with three images in one file
  // or Pyramid images, depth/color PMD,...
  UncompressImage(image);

  // save meta data for image info dialog
  stringstream info;
  switch (image.GetStorageType())
  {
  case ImageBase::ST_unsignedchar:
    {
      Camera<unsigned char> camera(image);
      camera.ParseMetaData();
      camera.PrintAppData(info);
    } break;

  case ImageBase::ST_float:
    {
      Camera<float> camera(image);
      camera.ParseMetaData();
      camera.PrintAppData(info);
    } break;
#ifdef BUILD_IMAGE_USHORT
  case ImageBase::ST_unsignedshortint:
    {
      Camera<unsigned short> camera(image);
      camera.ParseMetaData();
      camera.PrintAppData(info);
    } break;
#endif
  default:
    //    BIASERR("unfinished");
    break;
  }
  _MetaData = info.str();

  // create appropriate image canvas based on storage type
  if (image.GetStorageType() == ImageBase::ST_unsignedchar)
  {
    //check if the previous image was with floats
    bool histo_was_shown_for_float = false;
    bool valuebar_was_shown_for_float = false;

    // delete float image canvas
    if (_ImageCanvasFloat != NULL)
    {
      //when recreating Histogramm-window, give it the same position as it had
      //before
      _HistoPosition = _ImageCanvasFloat->GetImageCanvas()->GetHistoPos();
      _HistoSize     = _ImageCanvasFloat->GetImageCanvas()->GetHistoSize();

      _ValueBarSize     = _ImageCanvasFloat->GetImageCanvas()->GetValueBarSize();
      _ValueBarPosition = _ImageCanvasFloat->GetImageCanvas()->GetValueBarPos();

//      cout<<"Got Pos/Size of ValueBar:"<<_ValueBarPosition.x<<","<<_ValueBarPosition.y
//            		<<_ValueBarSize.x<<_ValueBarSize.y<<endl;

      if(_ImageCanvasFloat->GetImageCanvas() != NULL){
        if(_ImageCanvasFloat->GetImageCanvas()->HistogrammIsShown())
          histo_was_shown_for_float = true;
        if(_ImageCanvasFloat->GetImageCanvas()->ImageValueBarIsShown())
        	valuebar_was_shown_for_float = true;

      }

      _MainSizer->Detach(_ImageCanvasFloat);
      _ImageCanvasFloat->Destroy();
      _ImageCanvasFloat = NULL;
    }

    // create uc image canvas
    if (_ImageCanvasUC == NULL)
    {
      _ImageCanvasUC = new ImageCanvas(_SizerWindow, GetStatusBar(), 2, -1, wxPoint(1,1));
      // can safely turn this on, since ImageCanvas now uses double buffering
      _ImageCanvasUC->SetClearDrawingContext(true);
      
      _MainSizer->Add(_ImageCanvasUC,wxEXPAND);
    }
    else{
    	_HistoPosition = _ImageCanvasUC->GetHistoPos();
    	_HistoSize     = _ImageCanvasUC->GetHistoSize();
    	_ValueBarSize     = _ImageCanvasUC->GetValueBarSize();
    	_ValueBarPosition = _ImageCanvasUC->GetValueBarPos();

//    	cout<<"Got Pos/Size of ValueBar:"<<_ValueBarPosition.x<<","<<_ValueBarPosition.y
//    			<<_ValueBarSize.x<<_ValueBarSize.y<<endl;
    }
    //check if the histogramm should be shown and if YES, then update it
    if(_ShowHistogramm){
      if(!histo_was_shown_for_float){
        //cout << "showing histo" << endl;
        if(_ImageCanvasUC->HistogrammIsShown())
          ShowHistogramm(image);
        else
          _ShowHistogramm = false;
      }
      else{
        ShowHistogramm(image);
      }
    }
    //check if legend is requested and draw it (or not)
    if(_ShowImageValueBar){
    	if(!valuebar_was_shown_for_float){
    		//cout << "showing histo" << endl;
    		if(_ImageCanvasUC->ImageValueBarIsShown())
    			ShowImageValueBar(image);
    		else
    			_ShowImageValueBar = false;
    	}
    	else{
    		ShowImageValueBar(image);
    	}
    }

    // convert image
    Image<unsigned char> ucImage;
    ucImage.StealImage(image);
    ucImage.SetUID(image.GetUID());

    if (ucImage.GetChannelCount() < _SelectedChannel) {
      if ( _SelectedChannel>4 && _SelectedChannel<8) {
        // show hsl for rgb-like images
        Image<unsigned char> hsl;
        if (ImageConvert::ToHSV(ucImage, hsl)!=0) {
          BIASERR("Picture conversion to HSL failed.");
        } else {
          ucImage = hsl;
          ImageConvert::GetChannel(ucImage, _SelectedChannel-5);
          ucImage.SetColorModel(ImageBase::CM_RGB);
        }
      } else {
        BIASERR("Can't show selected Channel, because picture has less channels.");
      }
    }
    else
    {
      // Shows only the selected channel of image
      if (_SelectedChannel > 0)
      {
        ImageConvert::GetChannel(ucImage, _SelectedChannel-1);
      }
    }

    if (_FitCheckBox->IsChecked()) {
      // get size available for image canvas
      int width, height;
      GetClientSize(&width, &height);

      // compute scale that keeps the aspect ratio and the image is fully visible
      float xscale = (float)width / image.GetWidth();
      float yscale = (float)height / image.GetHeight();
      float scale = xscale < yscale ? xscale : yscale;

      _ImageCanvasUC->SetScaledImageSize((int)(image.GetWidth() * scale), (int)(image.GetHeight() * scale));
      _ImageCanvasUC->SetShowScaled(true);
    }//fitcheckbox
    else{
      _ImageCanvasUC->SetShowScaled(false);
    }

    // show image in canvas
    //cout<<"Showing image with filename:"<<fileName<<endl;
    _ImageCanvasUC->Show(ucImage, fileName);

    // don't show scrollbars if image is maximized
    if (_FitCheckBox->IsChecked()){
      _ImageCanvasUC->SetScrollbars(0, 0, 0, 0);
    }
  }
  else // image.GetStorageType() != ImageBase::ST_unsignedchar
  {
    //check if the previous image was with unsigned char
    bool histo_was_shown_for_uc = false;
    bool valuebar_was_shown_for_uc = false;

    // delete uc image canvas
    if (_ImageCanvasUC != NULL)
    {
      //when recreating Histogramm-window, give it the same position as it had
      //before
      _HistoPosition = _ImageCanvasUC->GetHistoPos();
      _HistoSize     = _ImageCanvasUC->GetHistoSize();

      _ValueBarSize     = _ImageCanvasUC->GetValueBarSize();
      _ValueBarPosition = _ImageCanvasUC->GetValueBarPos();

//      cout<<"Got Pos/Size of ValueBar:"<<_ValueBarPosition.x<<","<<_ValueBarPosition.y
//      		<<_ValueBarSize.x<<_ValueBarSize.y<<endl;

      if(_ImageCanvasUC->HistogrammIsShown())
        histo_was_shown_for_uc = true;
      if(_ImageCanvasUC->ImageValueBarIsShown())
      	valuebar_was_shown_for_uc = true;

      _MainSizer->Detach(_ImageCanvasUC);
      _ImageCanvasUC->Destroy();
      _ImageCanvasUC = NULL;
    }

    // create float image canvas
    if (_ImageCanvasFloat == NULL)
    {
      _ImageCanvasFloat = new ScaledImageCanvas(_SizerWindow, GetStatusBar(), 2, -1, wxPoint(1,1));
      _MainSizer->Add(_ImageCanvasFloat,wxEXPAND);
    }
    else{
    	_HistoPosition = _ImageCanvasFloat->GetImageCanvas()->GetHistoPos();
    	_HistoSize     = _ImageCanvasFloat->GetImageCanvas()->GetHistoSize();
    	_ValueBarSize     = _ImageCanvasFloat->GetImageCanvas()->GetValueBarSize();
    	_ValueBarPosition = _ImageCanvasFloat->GetImageCanvas()->GetValueBarPos();

//    	cout<<"Got Pos/Size of ValueBar:"<<_ValueBarPosition.x<<","<<_ValueBarPosition.y
//    			<<_ValueBarSize.x<<","<<_ValueBarSize.y<<endl;
    }
    //check if the histogramm should be shown and if YES, then update it
    if(_ShowHistogramm){
      if(!histo_was_shown_for_uc){
        //cout << "showing histo" << endl;
        if(_ImageCanvasFloat->GetImageCanvas() != NULL){
          //cout << "canvas exists" << endl;
          if(_ImageCanvasFloat->GetImageCanvas()->HistogrammIsShown())
            ShowHistogramm(image);
          else
            _ShowHistogramm = false;
        }
      }
      else{
        if(_ImageCanvasFloat->GetImageCanvas() != NULL){
          ShowHistogramm(image);
        }
      }
    }
    //check if legend is requested and draw it (or not)
    if(_ShowImageValueBar){
    	if(!valuebar_was_shown_for_uc){
    		//cout << "showing histo" << endl;
    		if(_ImageCanvasFloat->GetImageCanvas() != NULL){
    			//cout << "canvas exists" << endl;
    			if(_ImageCanvasFloat->GetImageCanvas()->ImageValueBarIsShown())
    				ShowImageValueBar(image);
    			else
    				_ShowImageValueBar = false;
    		}
    	}
    	else{
    		if(_ImageCanvasFloat->GetImageCanvas() != NULL){
    			ShowImageValueBar(image);
    		}
    	}
    }

    /**
    // set appropriate scale and offset range
    switch (image.GetStorageType())
    {
    case ImageBase::ST_unsignedshortint:
      _ImageCanvasFloat->SetScaleRange(1.0/16.0, 1.0);
      _ImageCanvasFloat->SetOffsetRange(-4095.0, 0.0);
      break;

    case ImageBase::ST_char:
      _ImageCanvasFloat->SetScaleRange(0.5, 5.0);
      _ImageCanvasFloat->SetOffsetRange(-128.0, 128.0);
      break;

    case ImageBase::ST_float:
      _ImageCanvasFloat->SetScaleRange(0.5, 50.0);
      _ImageCanvasFloat->SetOffsetRange(-100.0, 100.0);
      break;

    default:
      cout << "Warning: Unknown storage type" << endl;
      break;
    }
    **/

    if (image.GetChannelCount() < _SelectedChannel) {
      BIASERR("Can't show selected Channel, because picture has less channels.")
    }
    else
    {
      // Shows only the selected channel of image
      if (_SelectedChannel > 0)
      {
        ImageConvert::GetChannel(image, _SelectedChannel-1);
      }
    }

    if (_FitCheckBox->IsChecked())
    {
      // get size available for image canvas
      int width, height;
      GetClientSize(&width, &height);
      width = max(1, width - 10); // little fix so the image is really fully visible
      height = max(1, height - 40); // larger fix because of sliders in float canvas

      // compute scale that keeps the aspect ratio and the image is fully visible
      float xscale = (float)width / image.GetWidth();
      float yscale = (float)height / image.GetHeight();
      float scale = xscale < yscale ? xscale : yscale;

      _ImageCanvasFloat->GetImageCanvas()->SetScaledImageSize((int)(image.GetWidth() * scale), (int)(image.GetHeight() * scale));
      _ImageCanvasFloat->GetImageCanvas()->SetShowScaled(true);
    }
    else
    {
      _ImageCanvasFloat->GetImageCanvas()->SetShowScaled(false);
    }

    //cout<<"Showing image with filename:"<<fileName<<endl;
    // show image in canvas
    _ImageCanvasFloat->Show(image, fileName);

    // don't show scrollbars if image is maximized
    if (_FitCheckBox->IsChecked())
    {
      _ImageCanvasFloat->GetImageCanvas()->SetScrollbars(0, 0, 0, 0);
    }
  }

  stringstream msg;
  msg << "File '" << fileName << "' (" << image.GetWidth() << "x" << image.GetHeight() << ") loaded";
  SetStatusText(AsciiToWx(msg.str()),1);

  this->RefitClients();

  // breaks WIN32
  //#ifndef WIN32
  // make the shortcuts work without the need to click the canvas manually (on linux)
  //if (_ImageCanvasUC)
  //  _ImageCanvasUC->SetFocus();
  //if (_ImageCanvasFloat)
  //  _ImageCanvasFloat->SetFocus();
  //#endif
}

void BVWXMainFrame::LoadImages(const wxArrayString &paths)
{
  // add file names to internal std::string list
  bool ok=true;
  unsigned old_size = _FileNames.size();
  for (int i = 0; i < (int)paths.GetCount(); i++) {
    wxString extUP=wxFileName(paths[i]).GetExt().Upper();
    if (wxFileName(paths[i]).IsDir()) {
      ok=false;
    } else if ((extUP==AsciiToWx("LST")) || (extUP==AsciiToWx("TXT"))) {
      // special handling for .lst and .txt files.
      // they are no images but contain a list of line separated image
      //filenames
      vector<string> lstFilenames;
      int result=BIAS::Param::ParseListFile(WxToAscii(paths[i]),
        lstFilenames);
      if (result!=0){
        BIASERR("could not load image list file "<<WxToAscii(paths[i])
          <<" result="<<result<<endl);
      } else {
        // append filenames
        _FileNames.insert(_FileNames.end(), lstFilenames.begin(),
          lstFilenames.end() );
      }
    } else {
      _FileNames.push_back( string(WxToAscii(paths[i])) );
    }
  }

  // convert filenames to right format
  if (!_FileNames.empty()) {
    std::string str;
    for (std::vector<std::string>::iterator it = _FileNames.begin();
         it != _FileNames.end(); it++) {
      // remove whitespaces at beginning and end of filenames
      StringUtils::Trim(*it, str);
      // replace slash by backslash (Windows) or vice versa (Unix)
      *it = FileHandling::CorrectSlashes(str);
    }
  }

  // load last added file and remember file name and directory
  // do NOT jump to file 0 if adding files subsequently.
  // we may discuss to jump to first of currently "added" file,
  // but never jump to first file in list of all files!
  if (!_FileNames.empty()) {
  	LoadImageFromList(old_size);
  	_DefaultFile = AsciiToWx(_FileNames[old_size]);    
  }

  if (!ok) {
    wxMessageBox(wxT("Dropping directories is unsupported. Please drop only filenames!"),
                 wxT("Drag&Drop Error"), wxOK | wxICON_EXCLAMATION);
  }
}

void BVWXMainFrame::FitSizetIfTooSmall()
{

  int w=0, h=0;
  int imgW=0, imgH=0;
  if (_ImageCanvasUC!=NULL){
    if (_ImageCanvasUC->GetOrigImage()==NULL)
      return;
    imgW = _ImageCanvasUC->GetOrigImage()->GetWidth();
    imgH = _ImageCanvasUC->GetOrigImage()->GetHeight();
  } else if (_ImageCanvasFloat!=NULL){
    imgW = _ImageCanvasFloat->GetOrigimg().GetWidth();
    imgH = _ImageCanvasFloat->GetOrigimg().GetHeight();
  } else {
    // no valid canvas, nothing to do.
    return;
  }
  // get current frame size, TODO: query client area of canvas instead.
  this->GetClientSize(&w, &h);

  // query display screen dim
  int screenW=0, screenH=0;
  ::wxDisplaySize(&screenW, &screenH);
  if ( (w<imgW) || (h<imgH)) {
    // expand limited by screen size
    // TODO: adpat Client size, to avoid the FitInside...
    // problem: ImageCanves do not handle sizers correctly...
    // this is an approximation working for now:
    this->SetSize(
      //0,0, /* move win to top-left of screen */
      //-1, -1, /* keep window pos. */
      min(imgW+40, screenW), min(imgH+100, screenH) );
  }
  this->RefitClients();
}


void BVWXMainFrame::RefitClients()
{
  int w, h;

  GetClientSize(&w, &h);

  if(_ImageCanvasUC != NULL || _ImageCanvasFloat != NULL){
    _SizerWindow->SetSize(w,h);
  }
  if (_ImageCanvasUC!=NULL){
    _ImageCanvasUC->SetSize(
      //0,0,
      //-1, -1,
      w, h);//< keep window pos., do not move the window!
  }
  if (_ImageCanvasFloat!=NULL){
    _ImageCanvasFloat->SetSize(
      //0,0,
      //-1, -1,
      w, h); //< keep window pos., do not move the window!
  }
}

void BVWXMainFrame::OnLoad(wxCommandEvent& WXUNUSED(event))
{
  wxFileDialog fileDialog(
    this,                          // parent
    wxT("Choose file(s) to load"), // message
    _DefaultDir,                   // default directory
    _DefaultFile,                  // default file
    wxT("Image files (mip;pgm;ppm;jpg;jpeg;png)|*.mip;*.pgm;*.ppm;*.jpg;*.jpeg;*.png|All files (*.*)|*.*"),                   // wildcard
    wxFD_OPEN | wxFD_MULTIPLE);          // style

  if (fileDialog.ShowModal() == wxID_OK)
  {
    // get file names
    wxArrayString paths;
    fileDialog.GetPaths(paths);
    // load them
    LoadImages(paths);
  } else {
    SetStatusText(wxT("Loading canceled"),0);
  }

  _DefaultDir = fileDialog.GetDirectory(); 
}


void BVWXMainFrame::AddInfo(const BIAS::ImageBase & image, std::stringstream &info){
  info << "Width:    " << image.GetWidth() << endl;
  info << "Height:   " << image.GetHeight() << endl;
  info << "Depth:    " << image.GetDepth() << endl;
  info << "BitDepth: " << image.GetBitDepth() << endl;
  info << "SizeByte: " << image.GetSizeByte() << " bytes" << endl;
  info << "Pixel Count:   " << image.GetPixelCount() << endl;
  info << "Channels:      " << image.GetChannelCount() << endl;
  info << "ColorModel:    " << image.GetColorModel()  << endl;
  info << "StorageType:   " << image.GetStorageType() << endl;
  info << "IsPlanar:      " << image.IsPlanar() << endl;
  info << "IsInterleaved: " << image.IsInterleaved() << endl;
  info << "UUID:          " << image.GetUID() << endl;
  info << "Version:       " << image.GetVersionNumber() << endl;
}

void BVWXMainFrame::OnInfo(wxCommandEvent& WXUNUSED(event))
{
  // get currently displayed image
  ImageBase* image = NULL;
  if (_ImageCanvasUC != NULL)
  {
    image = _ImageCanvasUC->GetOrigImage();
  }
  else if (_ImageCanvasFloat != NULL)
  {
    image = &_ImageCanvasFloat->GetOrigimg();
  }
  if (image == NULL)
  {
    wxMessageBox(wxT("No image loaded"), wxT("Image Info"), wxOK | wxICON_ERROR, this);
    return;
  }

  // collect information
  stringstream info;
  BIASASSERT(image!=NULL);
  AddInfo(*image, info);

  // some info may already be lost through by Uncompress pre-conversion
  if (infoRaw_.size()>0)
    info << endl<<endl<<"RAW info before uncompression: "<<endl<<infoRaw_<<endl;

  // show information in message box
  wxMessageBox(AsciiToWx(info.str()), wxT("Image Info"), wxOK, this);
}

void BVWXMainFrame::OnDrawROI(wxCommandEvent& WXUNUSED(event))
{
  if (_ImageCanvasUC != NULL) {
    _ImageCanvasUC->DrawRoi(_DrawROICheckItem->IsChecked());   
  }
  else if(_ImageCanvasFloat != NULL){
    _ImageCanvasFloat->GetImageCanvas()->DrawRoi(_DrawROICheckItem->IsChecked());
  }
}

void BVWXMainFrame::OnImageValues(wxCommandEvent& WXUNUSED(event))
{
  float dmin=0.0, dmax=0.0;
  // get currently displayed image
  ImageBase* image = NULL;
  if (_ImageCanvasUC != NULL) {
    image = _ImageCanvasUC->GetOrigImage();
    Image<unsigned char> tim(*image);
    unsigned char min, max;
    tim.GetMinMaxPixelValue(min, max);
    dmin = min;
    dmax = max;
  } else if (_ImageCanvasFloat != NULL){
    image = &_ImageCanvasFloat->GetOrigimg();
    Image<float> tim(*image);
    tim.GetMinMaxPixelValue(dmin, dmax);
  }
  if (image == NULL){
    wxMessageBox(wxT("No image loaded"), wxT("Image Values"),
                 wxOK | wxICON_ERROR, this);
    return;
  }

  // collect information
  stringstream info;
  info << "min pixel value:   " << dmin << endl;
  info << "max pixel value:   " << dmax << endl;

  // show information in message box
  wxMessageBox(AsciiToWx(info.str()), wxT("Image Values"), wxOK, this);
}

void BVWXMainFrame::OnMetaData(wxCommandEvent& WXUNUSED(event))
{
  wxFont font(10, wxMODERN, wxNORMAL, wxNORMAL, false, wxT("Courier"));
  wxDialog* dlg = new wxDialog(this, -1, wxT("Meta Data"), wxDefaultPosition, wxDefaultSize,
    wxDEFAULT_DIALOG_STYLE | wxRESIZE_BORDER);
  wxTextCtrl* text = new wxTextCtrl(dlg, -1, wxT(""),
    wxDefaultPosition, wxSize(650, 400),
    wxTE_MULTILINE | wxTE_READONLY | wxTE_DONTWRAP);
#ifdef WIN32
  text->SetFont(font);
#endif
  text->SetDefaultStyle(wxTextAttr(wxNullColour, wxNullColour, font));
  text->AppendText(AsciiToWx(_MetaData));
  text->SetInsertionPoint(0);
  dlg->Fit();
  dlg->ShowModal();
  dlg->Destroy();
}

void BVWXMainFrame::OnExit(wxCommandEvent& WXUNUSED(event))
{
  Destroy();
}

void BVWXMainFrame::OnKeyInfo(wxCommandEvent& WXUNUSED(event))
{
  wxString text =
    wxT("Next Image\tCtrl+[Right]\n")
    wxT("Prev Image\tCtrl+[Left]\n")
    wxT("Run-Stop\tCtrl+[Space]");

  wxMessageBox(text, wxT("Keys"),
    wxOK | wxICON_INFORMATION, this);
}


void BVWXMainFrame::
OnAbout(wxCommandEvent &event)
{

  wxString desc;
  desc << wxT("The BiasViewWX Image Viewer, part of the\n");
  desc << wxT("BIAS library (Basic ImageAlgorithmS)\n\n");

  stringstream text;
  text << "Built on " << __DATE__ << " " << __TIME__ << "\n"
	    // << "from file " << __FILE__ << "\n"
	     << "GUI using: " << WxToAscii(wxVERSION_STRING)<< "\n";
  text<< "Built with support for:\n";
#ifdef BIAS_HAVE_LIBJPEG
  text<< "- jpeg (libJPEG).\n";
#endif
#ifdef BIAS_HAVE_OPENCV
  text<< "- OpenCV.\n";
#endif
#ifdef BIAS_HAVE_IMAGEMAGICKLIB
  text<< "- ImageMagick.\n";
#endif
#ifdef BIAS_HAVE_OPENEXR
  text<< "- OpenEXR.\n";
#endif
  desc<<AsciiToWx(text.str());
   
  wxString lic = wxT("BIAS is free software; you can redistribute it and/or modify\n \
  it under the terms of the GNU Lesser General Public Licence as published by\n \
  the Free Software Foundation; either version 2.1 of the Licence, or\n \
  (at your option) any later version.\n \
  \n \
  BIAS is distributed in the hope that it will be useful,\n \
  but WITHOUT ANY WARRANTY; without even the implied warranty of\n \
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n \
  GNU Lesser General Public Licence for more details.\n \
  \n \
  You should have received a copy of the GNU Lesser General Public Licence\n \
  along with BIAS; if not, write to the Free Software\n \
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA\n");

  wxAboutDialogInfo aboutdlg;
  aboutdlg.SetDescription( desc );
  aboutdlg.SetLicence( lic );
  aboutdlg.SetCopyright( wxT("Copyright (C) 2003-2011\n Multimediale Systeme der Informationsverarbeitung\n Institut fuer Informatik\n Christian-Albrechts-Universitaet Kiel") );
  aboutdlg.SetWebSite( wxT("http://www.mip.informatik.uni-kiel.de/") );

  wxAboutBox( aboutdlg );
}


void BVWXMainFrame::OnRemoveCurrent(wxCommandEvent& WXUNUSED(event)){
	//  cout << "Image NR:" << _CurrentImage << " out of "
	//       << (int)_FileNames.size() << " images is currently removed" << endl;

  if (_CurrentImage < (int)_FileNames.size()){
    _FileNames.erase(_FileNames.begin()+_CurrentImage);
  }
  if(_CurrentImage > (int)_FileNames.size()-1){
    _CurrentImage=0;
  }

  LoadImageFromList(_CurrentImage);
}

void BVWXMainFrame::OnRemoveAllButCurrent(wxCommandEvent& WXUNUSED(event)){
	//  cout << "Image NR:" << _CurrentImage << " out of "
	//       << (int)_FileNames.size() << " images is currently loaded"
	//       << " before erasing all but current" << endl;

  if (_FileNames.size()<1){
    SetStatusText(_T("No images to remove."),0);
    return; // nothing to to
  }
  BIASASSERT( (int)_FileNames.size() > _CurrentImage);
  // make current the first and only image in list
  _FileNames[0] = _FileNames[_CurrentImage];
  _CurrentImage=0;
  _FileNames.resize(1);
  // update display
  LoadImageFromList(_CurrentImage);
  //  cout << "Image NR:" << _CurrentImage << " out of "
  //       << (int)_FileNames.size() << " images is currently loaded"
  //       << " after erasing all but current" << endl;
}


void BVWXMainFrame::PrintSavingError(const int errorNo, wxString fname) 
{
	std::stringstream msg;
  msg << "Error saving "
			<< WxToAscii(fname) 
			<< " with result = " 
			<< errorNo;			
  SetStatusText(AsciiToWx(msg.str()),0);
	wxMessageBox(AsciiToWx(msg.str()),
								         wxT("Error saving file"),
								         wxICON_INFORMATION,
								         this);
	BIASERR(msg.str());
}

void BVWXMainFrame::PrintConversionError(const int errorNo, wxString fname) 
{
	std::stringstream msg;
  msg << "Error converting "
			<< WxToAscii(fname) 
			<< " with result = " 
			<< errorNo;			
  SetStatusText(AsciiToWx(msg.str()),0);
	wxMessageBox(AsciiToWx(msg.str()),
												 wxT("Conversion Error"),
												 wxICON_INFORMATION,
												 this);
	BIASERR(msg.str());
}

void BVWXMainFrame::PrintEmptyNameMsg() 
{
	wxString str(wxT("Please, give this image a name if you want to save it"));
	wxMessageBox(str, wxT("No Filename"), wxICON_INFORMATION, this);
	SetStatusText(str);
}

void BVWXMainFrame::PrintNoImageToFilter()
{
  wxString msg(wxT("No Image Loaded To Filter"));
  SetStatusText(msg);
  wxMessageBox(msg,
               wxT("No Image"),
               wxOK | wxICON_INFORMATION,
               this);
}

int BVWXMainFrame::ConvertFLtoUC(ImageBase& img) {
	const ImageBase imageTemp = img;
	return ImageConvert::ConvertST(imageTemp,
																  img, 
																  ImageBase::ST_unsignedchar);	
}


void BVWXMainFrame::OnSaveAs(wxCommandEvent& event)
{
	if (!_FileNames.empty()) { // check if the vector with filenames is empty
		FileHandling fh = FileHandling();
		std::string absoluteName = _FileNames[_CurrentImage];
		std::string tmpFileName  = fh.Basename(absoluteName) + ".mip";
		
		wxString message   			 = _T("Save current image to");
		wxString defaultDir 		 = AsciiToWx(fh.Directory(absoluteName));
		wxString defaultFileName = AsciiToWx(tmpFileName);
		wxString wildcard        = wxT("Image files (mip;pgm;ppm;jpg;jpeg;png)|*.mip;*.pgm;*.ppm;*.jpg;*.jpeg;*.png|All files (*.*)|*.*");
		
		wxFileDialog wd(this,
										message,
										defaultDir,
										defaultFileName,
										wildcard, 
										wxFD_SAVE|wxFD_OVERWRITE_PROMPT);
		
		if (wxID_OK == wd.ShowModal()) {
			wxString fname = wd.GetPath();
			
			if (!(fh.Basename(WxToAscii(fname))).empty()) {	// images should be named
				int notOK;	// var representing saving to disk success 
				
				if (_ImageCanvasUC) {
					if (ImageIO::FF_exr != ImageIO::GetFileFormat(WxToAscii(fname))) {
						ImageBase* imUC = _ImageCanvasUC->GetOrigImage();
						notOK = ImageIO::Save(WxToAscii(fname), *imUC);
						if (notOK!=0) PrintSavingError(notOK, fname);
						else SetStatusText(wxT("Saving to disk succeeded"));
					} else { // FF is exr
						wxString msg = wxT("We don't support conversion from lesser to higher storagetype");
						wxMessageBox(msg, wxT("Unsupported Conversion"), wxICON_INFORMATION);
						SetStatusText(msg);
						BIASERR(WxToAscii(msg));
					}
				} // end _ImageCanvasUC
				else if(_ImageCanvasFloat)
				{
					ImageBase imFL = _ImageCanvasFloat->GetShownImage();
					ImageIO::TFileFormat tf = ImageIO::GetFileFormat(WxToAscii(fname));
					
					if (ImageIO::FF_exr == tf) { // user want *.exr...just save to disk
						notOK = ImageIO::Save(WxToAscii(fname), imFL);
						if (notOK!=0) PrintSavingError(notOK, fname);
						else SetStatusText(wxT("Saving to disk succeeded"));
					}
					else if (ImageIO::FF_mip == tf)
					{	// user want *.mip...ask him whether to convert to uchar
						wxMessageDialog dl(this,
															 wxT("Convert to UNSIGNED CHAR?"),
															 wxT("Conversion Question"),
															 wxNO_DEFAULT|wxYES_NO|wxCANCEL);
						
						switch(dl.ShowModal()) {
							case wxID_YES:
								notOK = BVWXMainFrame::ConvertFLtoUC(imFL);
								if (notOK!=0) PrintConversionError(notOK, fname);
								break;
							case wxID_NO:
								notOK = ImageIO::Save(WxToAscii(fname), imFL);
								if (notOK!=0) PrintSavingError(notOK, fname);
								else SetStatusText(wxT("Saving to disk succeeded"));
								break;
							case wxID_CANCEL:
								break;
							default:
								SetStatusText(wxT("Unexpected wxMessageDialog return code"));
						}
					} 	// end if mip image
					else
					{ //save normally
					  Image<unsigned char>* ucImg;
					  ucImg = _ImageCanvasFloat->GetImageCanvas()->GetImageShown();
    				notOK = ImageIO::Save(WxToAscii(fname), *ucImg);
						if (notOK) PrintSavingError(notOK, fname);
						else SetStatusText(wxT("Saving to disk succeeded"));
					}
				}//end if _ImageCanvasFloat
				else{
					BIASERR("No image canvas!");
					SetStatusText(wxT("No image canvas!"));
				}
			}	else { // empty filename
				PrintEmptyNameMsg();
			}			
		} // end wd.ShowModal
	} else { // vector is empty
		SetStatusText(wxT("Nothing to Save"));
	}
}

void BVWXMainFrame::OnPrintCurrent(wxCommandEvent& event)
{
  wxPrintDialogData dlgData;
  wxPrintData data;
  
  dlgData.SetPrintData(data);

  wxPrintDialog pDialog(this,&dlgData);
  int ret = pDialog.ShowModal();
  if(ret==wxID_OK){
    
  }
  else if(ret==wxID_CANCEL)
    return;
}


void BVWXMainFrame::OnPrint(wxCommandEvent& event)
{
  wxPrintDialogData dlgData;
  

  wxPrintDialog pDialog(this,&dlgData);
  int ret = pDialog.ShowModal();
  if(ret==wxID_OK){
    
  }
  else if(ret==wxID_CANCEL)
    return;
}

void BVWXMainFrame::OnSaveCurrent(wxCommandEvent& event)
{
	if (!_FileNames.empty()) { // check if the vector with filenames is empty
		FileHandling fh = FileHandling();
		std::string absoluteName = _FileNames[_CurrentImage];
		std::string tmpFileName  = fh.Basename(absoluteName) + ".mip";
		
    wxString message   			 = _T("Save image as currently displayed with storage type unsigned char to:");
		wxString defaultDir 		 = AsciiToWx(fh.Directory(absoluteName));
		wxString defaultFileName = AsciiToWx(tmpFileName);
		wxString wildcard        = wxT("Image files (mip;pgm;ppm;jpg;jpeg;png)|*.mip;*.pgm;*.ppm;*.jpg;*.jpeg;*.png|All files (*.*)|*.*");

		wxFileDialog wd(this,
										message,
										defaultDir,
										defaultFileName,
										wildcard, 
										wxFD_SAVE|wxFD_OVERWRITE_PROMPT);
		
		if (wxID_OK == wd.ShowModal()) {
			wxString fname = wd.GetPath();
			
			if (!(fh.Basename(WxToAscii(fname))).empty()) { // images should be named
				
			  int notOK;
				ImageBase* img = NULL;

				if (_ImageCanvasUC) {
					img = _ImageCanvasUC->GetOrigImage();
					notOK = ImageIO::Save(WxToAscii(fname), *img);
				} else {
					img = &_ImageCanvasFloat->GetShownImage();

          //if we have a storagetype float get current image as displayed and save it
          if(img->GetStorageType()==ImageBase::ST_float)
          {					      
						BIAS::Image<unsigned char> tmpUC;           
            ImageConvert::ConvertST(*img,tmpUC,ImageBase::ST_unsignedchar);
						notOK = ImageIO::Save(WxToAscii(fname), tmpUC);
					}
					else
						notOK = ImageIO::Save(WxToAscii(fname), *img);
				}
				
				if (notOK) { // let us see if saving was successfull
					PrintSavingError(notOK, fname);
				} else {
					SetStatusText(wxT("Saving to disk succeeded"));
				}
				
			}	else { // empty filename
				PrintEmptyNameMsg();
			}			
		} // end wd.ShowModal		
			
	} else { // vector is empty
		SetStatusText(wxT("Nothing to Save"));
	}
}


void BVWXMainFrame::OnSaveAsMovie(wxCommandEvent& WXUNUSED(event)){
#ifdef BIAS_HAVE_FFMPEG
  FileHandling fh = FileHandling();
  std::string absoluteName = _FileNames[_CurrentImage];
  std::string tmpFileName  = fh.Basename(absoluteName) + ".mpeg";

  wxString message   			 = _T("Save movie of images to:");
  wxString defaultDir 		 = AsciiToWx(fh.Directory(absoluteName));
  wxString defaultFileName = AsciiToWx(tmpFileName);
  wxString wildcard        = wxT("Movie files (avi;mpeg;divx;xvid)|*.avi;*.mpeg;*.divx;*.xvid;|All files (*.*)|*.*");
/*
  wxFileDialog wd(this,
                  message,
                  defaultDir,
                  defaultFileName,
                  wildcard,
                  wxSAVE|wxOVERWRITE_PROMPT);

  if (wxID_OK == wd.ShowModal()) {
    wxString fname = wd.GetPath();
    cout<<"Saving movie to:"<<WxToAscii(fname)<<endl;
  }*/

  VideoSink_FFmpeg vs;

  ImageBase tmp;
  ImageIO::Load(_FileNames[_CurrentImage],tmp);

  wxVideoSettingsDialog dialog(this);
  dialog.SetGoPSize(12);
  dialog.SetBitrate(500000);//bitrate in Bit/seconds
  dialog.SetWidth(tmp.GetWidth());
  dialog.SetHeight(tmp.GetHeight());
  dialog.SetFPS(25);
  dialog.SetMaxBFrames(0);
  dialog.SetOutputFilename(wxT("movie.mpeg"));

  int ret = dialog.ShowModal();
  if(ret == wxID_OK){
    wxString fname = dialog.GetOutputFilename();
    vs.SetSize(dialog.GetWidth(),dialog.GetHeight());
    vs.SetBitrate(dialog.GetBitrate());
    vs.SetFPS(dialog.GetFPS());
    vs.SetGopSize(dialog.GetGoPSize());
    vs.SetMaxBFrames(dialog.GetMaxBFrames());
    vs.SetEncoder(CodecID(dialog.GetCodecID()));

    int ret = vs.Open(WxToAscii(fname));
    if( ret < 0) {
      stringstream st;
      st<<"Opening video ouput file failed: "<<vs.GetError();
      wxString msg = AsciiToWx(st.str().c_str());
      wxMessageBox(msg, wxT("Encoding failed."),
                   wxOK | wxICON_ERROR, this);
      return;
    }

    wxProgressDialog progressDlg(wxT("Encoding..."),
                                 wxT("Encoding movie, please wait."),
                                 100,this,
                                 wxPD_ELAPSED_TIME|wxPD_REMAINING_TIME|wxPD_AUTO_HIDE|wxPD_SMOOTH);

    //check direction
    bool reverse = _ReverseCheckBox->GetValue();

    for(unsigned i=0;i<_FileNames.size();i++){
      int index = i;
      if(reverse) index = _FileNames.size()-1-index;
      ret= vs.AddFrame(_FileNames[index]);
      if(ret <0) break;
      progressDlg.Update(((float)i/(float)_FileNames.size()) * 100.0);
    }

    if( ret < 0) {
      stringstream st;
      st<<"Encoding failed: "<<vs.GetError();
      wxString msg = AsciiToWx(st.str().c_str());
      wxMessageBox(msg,
                   wxT("Encoding failed."),
                   wxOK | wxICON_ERROR, this);
      return;
    }
    else
      progressDlg.Update(100);

    SetStatusText(wxT("Saving movie succeeded."));
    stringstream st;
    st<<"Movie successfully saved to: "<<WxToAscii(fname);
    wxString msg = AsciiToWx(st.str().c_str());
    wxMessageBox(msg,
                 wxT("Encoding successful."),
                 wxOK | wxICON_INFORMATION, this);
  }
#else
  wxString msg = wxT("No FFMPEG support found in BIAS. Activate it to save images encoded as videosS");
  wxMessageBox(msg,
               wxT("No FFMPEG support found in BIAS."),
               wxOK | wxICON_ERROR, this);
#endif
}


void BVWXMainFrame::OnSaveList(wxCommandEvent& WXUNUSED(event))
{
  wxString filename = wxFileSelector(_T("Choose file to save the image list to.")
             ,_T("") // default path
             ,_T("out_images.lst") //def. filename
             ,_T(".lst") // def. extension
             ,_T("LST files (*.lst)|*.lst|TXT files (*.txt)|*.txt|All files (*.*)|*.*") // wildcard
             ,wxFD_SAVE|wxFD_OVERWRITE_PROMPT // flags
             );
  if ( !filename.empty() ){
    ofstream f( WxToAscii(filename) ); // ios::out overwrite, not append
      if (!f.good()){
      	stringstream msg;
        msg<<"Could not open "<<filename.c_str()<<" for writing";
				SetStatusText( AsciiToWx( msg.str() ));
				BIASERR(msg.str());
				return;
      }
    vector<string>::const_iterator it = _FileNames.begin();
    int n=0;
    for ( ;it<_FileNames.end(); it++){
      f<<*it<<endl;
      n++;
    }
    f.close();
    stringstream msg; msg<<"Wrote "<<filename.c_str()<<" with "<<n<<" filenames";
    SetStatusText( AsciiToWx(msg.str() ));
  }
}

void BVWXMainFrame::OnToolEnter ( wxCommandEvent &){
  //cout<<"Entering tool with id:"<<event.GetSelection()<<endl;
  //event.skip();
}

void BVWXMainFrame::OnNext(wxCommandEvent& WXUNUSED(event))
{
  _CurrentImage++;
  if (_CurrentImage >= (int)_FileNames.size())
  {
    _CurrentImage = 0;
  }  
  LoadImageFromList(_CurrentImage);
}

void BVWXMainFrame::OnPrev(wxCommandEvent& WXUNUSED(event))
{
  _CurrentImage--;
  if (_CurrentImage < 0)
  {
    _CurrentImage = _FileNames.size() - 1;
  }
  LoadImageFromList(_CurrentImage);
}

void BVWXMainFrame::OnRunStop(wxCommandEvent& WXUNUSED(event))
{
  if (_Timer.IsRunning())
  {
    _Timer.Stop();
    wxToolBar* tb = GetToolBar();
    tb->SetToolNormalBitmap(ID_RunStop,wxIcon(play_xpm));
  }
  else
  {
    _Timer.Start(_DelaySpin->GetValue());
    wxToolBar* tb = GetToolBar();
    tb->SetToolNormalBitmap(ID_RunStop,wxIcon(stopplaying_xpm));
  }
}

void BVWXMainFrame::OnRun(wxCommandEvent& WXUNUSED(event))
{
  if (!_Timer.IsRunning())
  {
    _Timer.Start(_DelaySpin->GetValue());
  }
}

void BVWXMainFrame::OnStop(wxCommandEvent& WXUNUSED(event))
{
  if (_Timer.IsRunning())
  {
    _Timer.Stop();    
  }
}

void BVWXMainFrame::OnHistogramm(wxCommandEvent& WXUNUSED(event))
{
  if (_FileNames.size() > 0){
    const string& fileName = _FileNames[_CurrentImage];

    // load image
    ImageBase image;
    if (ImageIO::Load(fileName.c_str(), image)!=0){
      stringstream msg;
      msg << "Error loading " << fileName;
      BIASERR(msg.str());
      SetStatusText(AsciiToWx(msg.str()));
    }

    if(!_ShowHistogramm)
      _ShowHistogramm = true;

    ShowHistogramm(image);
  }
  else
  	SetStatusText(AsciiToWx("No Images Loaded (No Image = No Histogramm)"));
}

void BVWXMainFrame::ShowHistogramm(ImageBase& image){
  if (_FileNames.size() > 0){
    //Check if float or UC
    switch (image.GetStorageType()){
    case ImageBase::ST_unsignedchar:
      if(_ImageCanvasUC != NULL){
        _ImageCanvasUC->ShowHistogramm(image, _HistoPosition, _HistoSize);
      }
      break;
    case ImageBase::ST_float:
      if(_ImageCanvasFloat != NULL &&
         _ImageCanvasFloat->GetImageCanvas() != NULL){
        _ImageCanvasFloat->GetImageCanvas()->ShowHistogramm(image,
                                                            _HistoPosition,
                                                            _HistoSize);
      }
      break;
    default:
      break;
    }
    this->SetFocus();
  }
  else
    cout << "No Images Loaded (No Image = No Histogramm)" << endl;
}


void BVWXMainFrame::OnImageValueBar(wxCommandEvent& WXUNUSED(event))
{
	_ShowImageValueBar = !_ShowImageValueBar;

	if (_ShowImageValueBar && _FileNames.size() > 0){
		const string& fileName = _FileNames[_CurrentImage];

		// load image
		ImageBase image;
		if (ImageIO::Load(fileName.c_str(), image)!=0){
			stringstream msg;
			msg << "Error loading " << fileName;
			BIASERR(msg.str());
			SetStatusText(AsciiToWx(msg.str()));
		}
		UncompressImage(image);
		ShowImageValueBar(image);
	}
	else{
		if(_ImageCanvasUC != NULL &&	_ImageCanvasUC != NULL)
			_ImageCanvasUC->HideImageValueBar();
		if(_ImageCanvasFloat != NULL && _ImageCanvasFloat->GetImageCanvas() != NULL)
			_ImageCanvasFloat->GetImageCanvas()->HideImageValueBar();
	}
}

void BVWXMainFrame::ShowImageValueBar(ImageBase& image){
	 if (_FileNames.size() > 0){
	    //Check if float or UC
	    switch (image.GetStorageType()){
	    case ImageBase::ST_unsignedchar:
	      if(_ImageCanvasUC != NULL){
	        _ImageCanvasUC->ShowImageValueBar(image, _ValueBarPosition, _ValueBarSize);
	      }
	      break;
	    case ImageBase::ST_float:
	      if(_ImageCanvasFloat != NULL &&
	         _ImageCanvasFloat->GetImageCanvas() != NULL){
	        _ImageCanvasFloat->GetImageCanvas()->ShowImageValueBar(image,
																															_ValueBarPosition,
	                                                            _ValueBarSize);
	      }
	      break;
	    default:
	      break;
	    }
	    this->SetFocus();
	  }
	  else
	    cout << "No Images Loaded (No Image = No Histogramm)" << endl;
}

void BVWXMainFrame::OnDelay(wxSpinEvent& WXUNUSED(event))
{
  if (_Timer.IsRunning())
  {
    _Timer.Start(_DelaySpin->GetValue());
  }
}

void BVWXMainFrame::OnFit(wxCommandEvent& WXUNUSED(event))
{
  LoadImageFromList(_CurrentImage);
}

void BVWXMainFrame::OnKeepScrollPosition(wxCommandEvent& WXUNUSED(event))
{
  if (_KeepScrollPositionCheckBox != NULL)
    {
      if (_KeepScrollPositionCheckBox->IsChecked())
      {
          if (_ImageCanvasUC!=NULL){
            _ImageCanvasUC->KeepScrollPosition(true);
          }
          if (_ImageCanvasFloat!=NULL){
            _ImageCanvasFloat->KeepScrollPosition(true);
          }
      }else{
        if (_ImageCanvasUC!=NULL){
          _ImageCanvasUC->KeepScrollPosition(false);
        }
        if (_ImageCanvasFloat!=NULL){
          _ImageCanvasFloat->KeepScrollPosition(false);
        }
      }
    }
}

void BVWXMainFrame::OnTimer(wxTimerEvent& WXUNUSED(event))
{
  if (!_BlockAnimation)
  {
    // block animation until at least one idle event occurs,
    // so the viewer doesn't get stuck if delay is too low.
    _BlockAnimation = true;
    wxCommandEvent dummyEvent;

    if (_ReverseCheckBox->GetValue())
      OnPrev(dummyEvent);
    else
      OnNext(dummyEvent);
  }
}

void BVWXMainFrame::OnIdle(wxIdleEvent& WXUNUSED(event))
{
  _BlockAnimation = false;
}

void BVWXMainFrame::OnSize(wxSizeEvent& event)
{
  if (_FitCheckBox != NULL)
  {
    if (_FitCheckBox->IsChecked())
    {
      LoadImageFromList(_CurrentImage);
    }
  }
  this->RefitClients();
  event.Skip();
}


void BVWXMainFrame::OnChangeChannel_1(wxCommandEvent& event)
{
  _SelectedChannel = 1;
  stringstream msg;
  msg <<_SelectedChannel<<". channel (red?) selected.";
  SetStatusText(AsciiToWx(msg.str()));
  LoadImageFromList(_CurrentImage);
}

void BVWXMainFrame::OnChangeChannel_2(wxCommandEvent& event)
{
  _SelectedChannel = 2;
  stringstream msg;
  msg <<_SelectedChannel<<". channel (green?) selected.";
  SetStatusText(AsciiToWx(msg.str()));
  LoadImageFromList(_CurrentImage);
}

void BVWXMainFrame::OnChangeChannel_3(wxCommandEvent& event)
{
  _SelectedChannel = 3;
  stringstream msg;
  msg <<_SelectedChannel<<". channel (blue?) selected.";
  SetStatusText(AsciiToWx(msg.str()));
  LoadImageFromList(_CurrentImage);
}


void BVWXMainFrame::OnChangeChannel_4(wxCommandEvent& event)
{
  _SelectedChannel = 4;
  stringstream msg;
  msg <<_SelectedChannel<<". channel selected.";
  SetStatusText(AsciiToWx(msg.str()));
  LoadImageFromList(_CurrentImage);
}

void BVWXMainFrame::OnChangeChannel_5(wxCommandEvent& event)
{
  _SelectedChannel = 5;
  stringstream msg;
  msg <<_SelectedChannel<<". channel selected.";
  SetStatusText(AsciiToWx(msg.str()));
  LoadImageFromList(_CurrentImage);
}

void BVWXMainFrame::OnChangeChannel_6(wxCommandEvent& event)
{
  _SelectedChannel = 6;
  stringstream msg;
  msg <<_SelectedChannel<<". channel selected.";
  SetStatusText(AsciiToWx(msg.str()));
  LoadImageFromList(_CurrentImage);
}

void BVWXMainFrame::OnChangeChannel_7(wxCommandEvent& event)
{
  _SelectedChannel = 7;
  stringstream msg;
  msg <<_SelectedChannel<<". channel selected.";
  SetStatusText(AsciiToWx(msg.str()));
  LoadImageFromList(_CurrentImage);
}

void BVWXMainFrame::OnChangeChannel_8(wxCommandEvent& event)
{
  _SelectedChannel = 8;
  cout<<_SelectedChannel<<". channel selected."<<endl;
  LoadImageFromList(_CurrentImage);
}


void BVWXMainFrame::OnChangeChannel_ALL(wxCommandEvent& event)
{
  stringstream msg;
  msg <<"All channel selected.";
  SetStatusText(AsciiToWx(msg.str()));
  _SelectedChannel = 0;
  LoadImageFromList(_CurrentImage);
}

void BVWXMainFrame::OnFilterCannyEdge(wxCommandEvent& WXUNUSED(event))
{
  if ((NULL == _ImageCanvasUC) && (NULL == _ImageCanvasFloat)) {

    PrintNoImageToFilter();

  } else if (_ImageCanvasUC) {

    Image<UCHAR> imgToFilter = (Image<UCHAR>) *_ImageCanvasUC->GetOrigImage();
    FilterDialogCannyEdge<UCHAR, float>* cannyEdge
       = new FilterDialogCannyEdge<UCHAR, float>(imgToFilter, this, ID_FilterWindow);
    cannyEdge->Show(true);
    
  } else {

    Image<float> imgToFilter = (Image<float>) _ImageCanvasFloat->GetShownImage();
    FilterDialogCannyEdge<float, float>* cannyEdge
      = new FilterDialogCannyEdge<float, float>(imgToFilter, this, ID_FilterWindow);
    cannyEdge->Show(true);

  }
}

void BVWXMainFrame::OnFilterGauss(wxCommandEvent& WXUNUSED(event))
{
  if ((NULL == _ImageCanvasUC) && (NULL == _ImageCanvasFloat)) {

    PrintNoImageToFilter();

  } else if (_ImageCanvasUC) {

    Image<UCHAR> imgToFilter = (Image<UCHAR>) *_ImageCanvasUC->GetOrigImage();

    // ask user about outputst! uchar or float
    if (wxYES == wxMessageBox(wxT("Float Storagetype as output?"),
                              wxT("Storagetype Question"),
                              wxYES_NO | wxNO_DEFAULT | wxICON_QUESTION,
                              this))
    {
      FilterDialogGauss<UCHAR, float>* gauss
         = new FilterDialogGauss<UCHAR, float>(imgToFilter, this, ID_FilterWindow);
      gauss->Show(true);
    } else {
      FilterDialogGauss<UCHAR, UCHAR>* gauss
         = new FilterDialogGauss<UCHAR, UCHAR>(imgToFilter, this, ID_FilterWindow);
      gauss->Show(true);
    }

  } else {

    Image<float> imgToFilter = (Image<float>) _ImageCanvasFloat->GetShownImage();
    FilterDialogGauss<float, float>* gauss
      = new FilterDialogGauss<float, float>(imgToFilter, this, ID_FilterWindow);
    gauss->Show(true);

  }
}

void BVWXMainFrame::OnFilterGradientSobel3x3(wxCommandEvent& WXUNUSED(event))
{
  if ((NULL == _ImageCanvasUC) && (NULL == _ImageCanvasFloat)) {

    PrintNoImageToFilter();

  } else if (_ImageCanvasUC) {

    Image<UCHAR> imgToFilter = (Image<UCHAR>) *_ImageCanvasUC->GetOrigImage();
    FilterDialogGradientSobel3x3<UCHAR, float>* sobel3x3
      = new FilterDialogGradientSobel3x3<UCHAR, float>(imgToFilter, this, ID_FilterWindow);
    sobel3x3->Show(true);

  } else {

    Image<float> imgToFilter = (Image<float>) _ImageCanvasFloat->GetShownImage();
    FilterDialogGradientSobel3x3<float, float>* sobel3x3
      = new FilterDialogGradientSobel3x3<float, float>(imgToFilter, this, ID_FilterWindow);
    sobel3x3->Show(true);

  }

}

void BVWXMainFrame::OnFilterMean(wxCommandEvent& WXUNUSED(event))
{
	if ((NULL == _ImageCanvasUC) && (NULL == _ImageCanvasFloat)) {

	  PrintNoImageToFilter();

	} else if (_ImageCanvasUC) {

	  Image<UCHAR> imgToFilter = (Image<UCHAR>) *_ImageCanvasUC->GetOrigImage();

	  // ask user about outputst! uchar or float
	  if (wxYES == wxMessageBox(wxT("Float Storagetype as output?"),
	                            wxT("Storagetype Question"),
	                            wxYES_NO | wxNO_DEFAULT | wxICON_QUESTION,
	                            this))
	  {
	    FilterDialogMean<UCHAR, float>* mean
	       = new FilterDialogMean<UCHAR, float>(imgToFilter, this, ID_FilterWindow);
	     mean->Show(true);
	  } else {
	    FilterDialogMean<UCHAR, UCHAR>* mean
	       = new FilterDialogMean<UCHAR, UCHAR>(imgToFilter, this, ID_FilterWindow);
	     mean->Show(true);
	  }


	} else {

	  Image<float> imgToFilter = (Image<float>) _ImageCanvasFloat->GetShownImage();
	  FilterDialogMean<float, float>* mean
      = new FilterDialogMean<float, float>(imgToFilter, this, ID_FilterWindow);
	  mean->Show(true);

	}

}

void BVWXMainFrame::OnFilterMedian(wxCommandEvent& WXUNUSED(event))
{
  if ((NULL == _ImageCanvasUC) && (NULL == _ImageCanvasFloat)) {

    PrintNoImageToFilter();

  } else if (_ImageCanvasUC) {

    Image<UCHAR> imgToFilter = (Image<UCHAR>) *_ImageCanvasUC->GetOrigImage();

    // ask user about outputst! uchar or float
    if (wxYES == wxMessageBox(wxT("Float Storagetype as output?"),
                              wxT("Storagetype Question"),
                              wxYES_NO | wxNO_DEFAULT | wxICON_QUESTION,
                              this))
    {
      FilterDialogMedian<UCHAR, float>* median
         = new FilterDialogMedian<UCHAR, float>(imgToFilter, this, ID_FilterWindow);
       median->Show(true);
    } else {
      FilterDialogMedian<UCHAR, UCHAR>* median
         = new FilterDialogMedian<UCHAR, UCHAR>(imgToFilter, this, ID_FilterWindow);
       median->Show(true);
    }


  } else {

    Image<float> imgToFilter = (Image<float>) _ImageCanvasFloat->GetShownImage();
    FilterDialogMedian<float, float>* median
      = new FilterDialogMedian<float, float>(imgToFilter, this, ID_FilterWindow);
    median->Show(true);

  }

}

void BVWXMainFrame::OnFilterRescale(wxCommandEvent& WXUNUSED(event))
{
  if ((NULL == _ImageCanvasUC) && (NULL == _ImageCanvasFloat)) {

    PrintNoImageToFilter();

  } else if (_ImageCanvasUC) {

    Image<UCHAR> imgToFilter = (Image<UCHAR>) *_ImageCanvasUC->GetOrigImage();

    // ask user about outputst! uchar or float
    if (wxYES == wxMessageBox(wxT("Float Storagetype as output?"),
                              wxT("Storagetype Question"),
                              wxYES_NO | wxNO_DEFAULT | wxICON_QUESTION,
                              this))
    {
      FilterDialogRescale<UCHAR, float>* sobel3x3
        = new FilterDialogRescale<UCHAR, float>(imgToFilter, this, ID_FilterWindow);
      sobel3x3->Show(true);
    } else {
      FilterDialogRescale<UCHAR, UCHAR>* sobel3x3
        = new FilterDialogRescale<UCHAR, UCHAR>(imgToFilter, this, ID_FilterWindow);
      sobel3x3->Show(true);
    }

  } else {

    Image<float> imgToFilter = (Image<float>) _ImageCanvasFloat->GetShownImage();
    FilterDialogRescale<float, float>* sobel3x3
      = new FilterDialogRescale<float, float>(imgToFilter, this, ID_FilterWindow);
    sobel3x3->Show(true);

  }
}





void BVWXMainFrame::OnFilterSave(  wxCommandEvent &event ) {
  wxArrayString str = wxArrayString();
  str.Add(event.GetString());

  LoadImages(str);
}
