#include <iostream>
#include <Base/Image/ImageIO.hh>

using namespace std;
using namespace BIAS;
/**
  @file biasSetColormodel.cpp
  @ingroup g_tools
  @brief Simply set the color model of one ore multiple .mip image files WITHOUT
         further adjustments (color conversion, channelcount adaptation, etc.)
  @author jordt
  */
int main(int argc, char* argv[])
{
  if (argc < 3){
    cout << endl << "   biassetcolormodel sets the color model of one ore multiple .mip image files WITHOUT" << endl;
    cout << "   further adjustments (color conversion, channelcount adaptation, etc.)." << endl;
    cout << "     Usage: biassetcolormodel <ColorModel> <File1> [File2] [File3] ..." << endl;
    cout << "     For example: biassetcolormodel CM_Bayer_RGGB *.mip" << endl;
    cout << "     See BIAS::ImageBase::EColorModel documentation for available color models" << endl << endl;
    exit(0);
  }
  ImageBase::EColorModel cm;
  ImageBase::StringToColorModel(string(argv[1]),cm);

  if (cm == ImageBase::CM_invalid){
    cout << "  Error: Invalid Color Model" << endl ;
    cout << "   Usage: biassetcolormodel <ColorModel> <File1> [File2] [File3] ..." << endl;
    cout << "   For example: biassetcolormodel CM_Bayer_RGGB *.mip" << endl;
    cout << "   See BIAS::ImageBase::EColorModel documentation for available color models" << endl << endl;
    exit(0);
  }

  for (int i = 2; i < argc; i++){
    ImageBase loadImg;
    ImageIO::Load(argv[i],loadImg);
    loadImg.SetColorModel(cm);
    ImageIO::Save(argv[i],loadImg);
    cout << "." << flush;
  }
  cout << endl;
}
