/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003-2009    (see file CONTACT for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

//#include <Base/Common/LeakChecking.h> 
#include <Base/Debug/Error.hh>
#include <Base/Image/ImageIO.hh>
#include <Base/Image/ImageConvert.hh>
#include <iostream>

using namespace BIAS;
using namespace std;

/**
 * @file
 * @brief Tool for adding two images pixelwise, see biasadd.cpp
 * @ingroup g_tools
 * @author MIP
**/
int main(int argc, char ** argv)
{
  //system("pwd");
  if (argc<4){
    if (argc>0) cerr << argv[0] <<" <image 1> <image 2> <outfile> \n";
    cerr << "   adds image 1 to image 2 and writes the result to outfile"
      <<" scaling and shifting is done for unsigned char"<<
      endl;    
    return 0; // for Dart testing
  }

  ImageBase im1, im2, result;
  if (ImageIO::Load(argv[1], im1)!=0){
    BIASERR("error loading "<<argv[1]);
    return -1;
  }
  if (ImageIO::Load(argv[2], im2)!=0){
    BIASERR("error loading "<<argv[2]);
    return -1;
  }
  if (!im1.SamePixelAndChannelCount(im2)) {
    BIASERR("image size,depth,or ST does not match");
    exit(-2);
  }
  BIASASSERT(im1.GetWidth()==im2.GetWidth());
  BIASASSERT(im1.GetHeight()==im2.GetHeight());

  Image<float> sumImg;
  Image<unsigned char> ucim1, ucim2,ucSum;
  sumImg.Init(im1.GetWidth(),im1.GetHeight(),im1.GetChannelCount());
  switch (im1.GetStorageType()){
  case ImageBase::ST_unsignedchar: 
    {
      ucim1=im1;
      ucim2=im2;
      unsigned char *p1, *p2, *end;
      float *pSum;
      p1=ucim1.GetImageData();
      p2=ucim2.GetImageData();
      pSum=sumImg.GetImageData();
      end=p1+im1.GetSizeByte();
      //add
      float max=0;
      while (p1!=end) {
        *pSum= (float)(*p1) + (float)(*p2);
        if (*pSum>max) 
          max=*pSum;
        p1++; p2++; pSum++;
      }

      // scale
      if (max>255.0) {
        for (unsigned int i=0;
             i<sumImg.GetPixelCount()*sumImg.GetChannelCount();
             i++)
          sumImg.GetImageData()[i] *= (float)(255.0/max);
      }
      
      ImageConvert::ConvertST(sumImg,ucSum,ImageBase::ST_unsignedchar);
      if (ImageIO::Save(string(argv[3]), ucSum)!=0){
        BIASERR("error writing "<<argv[3]);
      }
     //  ucdiff.GetMinMaxPixelValue(ucmin, ucmax);
//       //     cerr << "differences between "<<(int)ucmin<<" and "<<(int)ucmax<<endl;
//       if (ucdiff.ScaleShiftBetween(0, 255)!=0){
//         BIASERR("error scaling image");
//       }
//       if (ImageIO::Save("diff-scaled.mip", ucdiff)!=0){
//         cerr << "error writing diff-scaled.mip "<<endl;
//       }

    }
    break;
  case ImageBase::ST_float:

  default:
    cerr << "storage type not supported \n";
    break;
  }
  return 0;
}
