/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003-2009    (see file CONTACT for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

//#include <Base/Common/LeakChecking.h> 

#include <Base/Image/ImageIO.hh>
#include "Base/Image/ImageConvert.hh"
#include "Utils/Param.hh"
#include <stdio.h>
#include <iostream>

using namespace BIAS;
using namespace std;



/**
 * @file
 * @ingroup g_tools
 * @brief tool for combining images channel-wise, see biasaddchannel.cpp
 * @author evers
 */
int main(int argc, char *argv[]){
  // Dart Testing
  if (argc>=2) {
    if (strcmp(argv[1],"-darttest") == 0){
      std::cout<<"started dart test: "<<argv[0]<<std::endl; 
      return 0;
    }
  }
  if (argc < 4) {
    cout << "Usage: biasaddchannel <image 1> <image 2> <output image>" << endl;
    return 0;
  }

    
  //bool *planar;
 Param param;
  //param = param.AddParamBool("planar","use planar instead of interleaved",false, 'p');
  int next =1;
  //next = param.ParseCommandLine(argc, argv);
  //if (next<0) exit(1);
  //param.DisableDestructorWarning();

  //if (argc<3){
  //  param.Usage();
  //  return -1;
  //}

  ImageBase im1, im2, diff;
  if (ImageIO::Load(argv[next], im1)!=0){
    BIASERR("error loading "<<argv[next]);
    return -1;
  }
  if (ImageIO::Load(argv[next+1], im2)!=0){
    BIASERR("error loading "<<argv[next+1]);
    return -1;
  }

  if (im1.GetWidth() != im2.GetWidth() ||
      im1.GetHeight() != im2.GetHeight()){
    BIASERR("image sizes do not match");
    return -2;
  }



  string newfilename;
  string fn1,fn2;
  fn1 = argv[next];
  fn2 = argv[next+1];
  newfilename = argv[next+2];

  Image<float> fim1, fim2, fdiff;
  Image<unsigned char> ucim1, ucim2, ucdiff;


  switch (im1.GetStorageType()){
  case ImageBase::ST_unsignedchar:
    ucim1=im1;
    ucim2=im2;
    //fim1.SetInterleaved(!(*planar));
    ucim1.AppendChannel(ucim1);
    ucim1.SetColorModel(ImageBase::CM_Grey);
    //if (ImageIO::Save(newfilename, ucim2)!=0){
    if (ImageIO::Save(newfilename, ucim2)!=0){
      cerr << "error writing diff.mip "<<endl;
    }
    break;
  case ImageBase::ST_float:
    fim1=im1;
    fim2=im2;
    //fim1.SetInterleaved(!(*planar));
    ucim1.SetColorModel(ImageBase::CM_Grey);
    fim1.AppendChannel(fim2);
 
 
    //if (ImageIO::Save(newfilename, fim1)!=0){
    if (ImageIO::Save(newfilename, fim1)!=0){
      cerr << "error writing diff.mip "<<endl;
    }
   
    break;
  default:
    cerr << "unknown storage type\n";
    break;
  }

  return 0;
}
