/*
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003-2009    (see file CONTACT for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/**
	@example ExampleConvertBayerPattern.cpp
	@relates ConvertBayerPattern
	@brief Example for converting Bayer Pattern to RGB images using BIAS::ConvertBayerPattern
	@ingroup g_examples
	@author bangerer
*/


#ifndef WIN32
#include <Image/ConvertBayerPattern.hh>
#else
#error This tool requires LINUX
#endif
#include <Base/Image/ImageIO.hh>
#include <Base/Image/ImageBase.hh>
#include <Base/Image/ImageConvert.hh>
#include <dc1394/dc1394.h>
#include <dc1394/conversions.h>


#include <Utils/IOUtils.hh>
#include <Image/Camera.hh>

using namespace BIAS;
using namespace std;

int main(int argc, char *argv[])
{

  Param params(true);
  vector<string> conversionMethod;
  conversionMethod.push_back("NEAREST");  
  conversionMethod.push_back("SIMPLE");  
  conversionMethod.push_back("BILINEAR");  
  conversionMethod.push_back("HQLINEAR");  
  conversionMethod.push_back("VNG");
  conversionMethod.push_back("AHD");
  int* convMethod = params.AddParamEnum("conversionMethod",
                                      "what method for conversion shall be used?"
                                      " Default :  AHD",
                                      conversionMethod, 5, NULL, 'M');

  int fup = 0;
  if(!IOUtils::ParseCommandLineEvalHelp(params, argc, argv, fup)){
    
    cout<<"Usage:\n"<<argv[0]<<" <srcImg> <targetImg> \n";
    return 0;
  }
  if(argc<2) {
        cout<<"Usage:\n"<<argv[0]<<" <srcImg> <targetImg> \n";
    return 0;
  }  
  
  BayerDemosaicMethod method = (BayerDemosaicMethod)*convMethod;

  unsigned int width=0, height=0, channels=0;
  Projection proj;
	Camera <unsigned char> source;

  
  if(!IOUtils::LoadCamera(argv[fup], source, proj, width, height, channels)) {
    cout<<"error loading "<<argv[fup]<<endl;
    return -1;
  }

  // read the image
	// if (ImageIO::Load(argv[fup], source)!=0){
	// 	BIASERR("error loading image "<<argv[1]);
	// 	//return -2;
	// 	return 0; // for Dart testing
	// }

	// define an image with 3 channels
	Camera<unsigned char> out;
  out.Init(width, height, 3);

	// instantiate ConvertBayerPattern class with correct storage type
	ConvertBayerPattern<unsigned char> converter;

	// demosaic image. you can choose the method for ConvertBayerPattern with the third parameter
	converter.Convert(source,out, method,
                    //BAYER_DEMOSAIC_METHOD_AHD,
										//ImageBase::CM_Bayer_BGGR);
                    //ImageBase::CM_Bayer_RGGB);
                    ImageBase::CM_invalid);
  
  if(source.IsProjValid()) {
     out.SetProj(proj);
     out.UpdateMetaData();
  }
	// write the image
  cout<<"writing "<<argv[fup+1]<<endl;
	if (ImageIO::Save(argv[fup+1], out)!=0){
    cout<<"Error writing result!\n";
		return -1; // for Dart testing
	}

	return 0;
}
