#include <FeatureDetector/CornerDetectorKLT.hh>
#include <Matcher2D/RegionMatcher.hh>
#include <Base/Image/Image.hh>
#include <Base/Image/ImageIO.hh>
#include <Base/Image/ImageConvert.hh>
#include <Base/ImageUtils/ImageDraw.hh>

using namespace BIAS;
using namespace std;
/**
   @file
   @ingroup g_tools
   @brief Match KLT corners, see biascornermatcher.cpp
   @author MIP
*/
int main(int argc, char *argv[])
{
  Image<unsigned char> img[2], grey[2], result;
  if (argc!=3){
    cerr << argv[0] << " <img 1> <img 2>\n";
    return -1;
  }

  vector<vector<HomgPoint2D> > p(2);
  vector<vector<QUAL> > qual(2);
  CornerDetectorKLT<unsigned char, float> cd;
  cd.SetMinCornerness(50);
  cd.SetMinDistance(5);
  cd.SetMaxNumFeatures(1000);

  for (int i=0; i<2; i++){
    int res = ImageIO::Load(argv[1+i], img[i]);
    if (res !=0){
      cerr << "error loading image \""<<argv[1+i]<<"\"\n";
      return -2;
    }
    res = ImageConvert::Convert(img[i], grey[i], ImageBase::CM_Grey);
    if (res !=0){
      cerr << "error converting image \""<<argv[1+i]<<"\"\n";
      return -3;
    }
    res = cd.Detect(grey[i], p[i], qual[i]);
    if (res !=0){
      cerr << "error detecting corners in image \""<<argv[1+i]<<"\"\n";
      return -4;
    }
  }

  // prepare result image
  int height = max(img[0].GetHeight(), img[1].GetHeight());
  int width = img[0].GetWidth() + img[1].GetWidth();
  Vector2<int> offset(img[0].GetWidth(), 0);  
  result.Init(width, height, img[0].GetChannelCount());
  result.SetColorModel(img[0].GetColorModel());
  result.FillImageWithConstValue((unsigned char)0);
  result.GetROI()->SetCorners(0, 0, img[0].GetWidth(), img[0].GetHeight());
  result.Paste2ROI(img[0]);
  result.GetROI()->SetCorners(img[0].GetWidth(), 0, width, img[1].GetHeight());
  result.Paste2ROI(img[1]);

  // now match every corner with every corner from te other image
  unsigned p0[2], p1[2], p1s[2];
  unsigned width0 = img[0].GetWidth();
  unsigned width1 = img[1].GetWidth();
  unsigned height0 = img[0].GetHeight();
  unsigned height1 = img[1].GetHeight();
  unsigned hws = 5;
  double ncc_val;
  const double min_ncc_val = 0.95;
  RegionMatcher rm;
  const unsigned char **ida0 = 
    (const unsigned char **) grey[0].GetImageDataArray();
  const unsigned char **ida1 = 
    (const unsigned char **) grey[1].GetImageDataArray();
  for (int idx0=0; idx0<(int)p[0].size(); idx0++){
    p0[0] = (unsigned)rint(p[0][idx0][0]);
    p0[1] = (unsigned)rint(p[0][idx0][1]);
    if (p0[0]<hws || p0[0]+hws>=width0 || 
        p0[1]<hws || p0[1]+hws>=height0){
      continue;
    }
    
    unsigned char col[] = { 255, 255, 255};
    unsigned radius = 5;
    ImageDraw<unsigned char>::CircleCenter(result, p0[0], p0[1], radius, col);

    for (int idx1=0; idx1<(int)p[1].size(); idx1++){
      p1[0] = (unsigned)rint(p[1][idx1][0]);
      p1[1] = (unsigned)rint(p[1][idx1][1]);
      if (p1[0]<hws || p1[0]+hws>=width1 || 
          p1[1]<hws || p1[1]+hws>=height1){
        continue;
      }
      if (true || idx1==0){
        ImageDraw<unsigned char>::CircleCenter(result, p1[0]+offset[0], 
                                               p1[1]+offset[1], radius, col);
      }
      int search_region = 1;
      for (int y=-search_region; y<=search_region; y++){
         p1s[1] = (int)(p1[1]) + y;
        for (int x=-search_region; x<=search_region; x++){
          p1s[0] = (int)(p1[0]) + x;
          if (p1s[0]<hws || p1s[0]+hws>=width1 || 
              p1s[1]<hws || p1s[1]+hws>=height1){
            continue;
          }
          rm.NCC(p0, p1s, ida0, ida1, hws, ncc_val);
          
          if (ncc_val>min_ncc_val){
            unsigned p1offs[2] = { p1[0]+offset[0], p1[1]+offset[1] };
            ImageDraw<unsigned char>::Line(result, p0, p1offs, col);
            break;
            break;
          } 
        }
      }
    }
    cout << "."<<flush;
  }
  cout << endl;
  ImageIO::Save("matches.mip", result);

  return 0;
}
