/*
 This file is part of the BIAS library (Basic ImageAlgorithmS).

 Copyright (C) 2003-2009    (see file CONTACT for details)
 Multimediale Systeme der Informationsverarbeitung
 Institut fuer Informatik
 Christian-Albrechts-Universitaet Kiel


 BIAS is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 (at your option) any later version.

 BIAS is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with BIAS; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <Image/Camera.hh>
#include <Base/Image/ImageBase.hh>
#include <Base/Image/ImageIO.hh>
#include <Base/Image/ImageConvert.hh>
#include <Base/Image/ImageAttributes.hh>
#include <Geometry/PMatrix.hh>
#include <bias_config.h>
#include <stdio.h>
#include <fstream>
#include <Utils/Param.hh>

#ifdef   WIN32
#  include "Base/Common/getopt_W32.h"
#else  //not WIN32
#  include <getopt.h>
#endif //WIN32
using namespace BIAS;
using namespace std;

/**
 @file
 @ingroup g_tools
 @brief New one channel image from selected channel of a multichannel image, see biasextractchannel.cpp
 @author djung
 */
int main(int argc, char *argv[]) {
  //    pic.SetDebugLevel(D_CAM_IO);
  Param param;
  bool *help = param.AddParamBool("help", "display help", false, 'h');
  int *channel = param.AddParamInt("channel", "channel to extract", 0, 0, INT_MAX, 'c');

  int fup = param.ParseCommandLine(argc, argv);

  if (*help || fup + 1 >= argc) {
    cout << "Usage: " << argv[0] << " [options] infile outfile\n";
    param.Usage();
    return 0;
  }
  string ifile = argv[fup];
  string ofile = argv[fup + 1];

  // after getopt() all recognized options are before non-options
  ifile = argv[argc - 2];
  ofile = argv[argc - 1];

  ImageBase pic;

  if (ImageIO::Load(ifile, pic) < 0) {
    BIASERR("could not load " <<ifile);
    exit(1);
  }
  if (*channel >= (int)(pic.GetChannelCount())) {
    BIASERR("image "<<ifile<<" has only "<<pic.GetChannelCount()<<" channels"<<endl);
    exit(1);
  }
  ImageConvert::GetChannel(pic, *channel, true);
  ImageIO::Save(ofile, pic);
  return 0;
}

