/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004    (see file CONTACTS for details)
Multimediale Systeme der Informationsverarbeitung
Institut fuer Informatik
Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <Base/Image/ImageIO.hh>
#include <Image/HistogramImage.hh>

using namespace BIAS;
using namespace std;


/**
   @file
   @ingroup g_tools
   @brief Detects a color histogram and writes to file hist.txt, see biashistogram.cpp
   @author ischiller
*/
HistogramImage hist;
bool loga = false;
bool ignore_negative = true;
Image<float> imf;

void Draw()
{
  hist.ZeroHist(0);
  if (ignore_negative){
    vector<float> v;
    float *id = imf.GetImageData();
    float *end = id + imf.GetPixelCount();
    while (id<end){
      if (*id>=0.0) v.push_back(*id);
      id++;
    }
    hist.AddHist(v);
  } else {
    hist.AddHist(imf);
  }
  hist.FillImageWithConstValue(255);
  if (loga) hist.DrawLog();
  else hist.Draw();
}

int main(int argc, char *argv[])
{
  const int num_bins = 200;
  const int num_hist = 1;
  hist.InitHist(num_bins, num_hist);
  hist.SetColor(0,0,0,0);
  hist.AddDebugLevel("D_HIST_BIN");


  ImageBase im;
  
  if (argc!=2){
    cerr << argv[0] << " <image>\n";
    return -1;
  }
  
  if (ImageIO::Load(argv[1], im)!=0){
    cerr << "error loading image "<<argv[1]<<endl;
    return -2;
  }

  imf = Image<float>(im);
  if (imf.IsEmpty()){
    cerr << "only for float images"<<endl;
    return -3;
  }
  hist.AddHist(imf);
  Draw();

  hist.WriteASCII("hist.txt");

  return 0;
}
