/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003-2009    (see file CONTACT for details)
Multimediale Systeme der Informationsverarbeitung
Institut fuer Informatik
Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <Base/Common/BIASpragma.hh>
#include <Base/Image/ImageBase.hh>
#include <Base/Image/ImageIO.hh>
#include <Base/Image/ImageConvert.hh>
#include <stdio.h>
#include <string>
#include <Utils/Param.hh>
#include <Utils/IOUtils.hh>

using namespace BIAS;
using namespace std;

void usage(int argc, char* argv[])
{
  cout<<"\n"<<argv[0]<<" --help for usage instructions"<<endl;
}

/**
   @file
   @ingroup g_tools
   @brief scales and shifts image values in given range, see biasscaleshiftimage.cpp
   @author ischiller
*/
int main (int argc, char *argv[])
{
  if(argc<2){
    usage(argc, argv);
    return -1;
  }

  Param *param = new Param(true);
  string *sourceImageName,*sinkImageName;
  double *min, *max;
  bool *convertToUC;
  BIAS::ImageBase origImage;

  sourceImageName = param->AddParamString("SourceImage",
                                           "The source Image",
                                           "",'s');
  sinkImageName = param->AddParamString("TargetImage",
                                           "The target Image",
                                           "",
                                           't');


  min = param->AddParamDouble("Min","Minimum Scaling value",0,0,255,'i');
  max = param->AddParamDouble("Max","Maximum Scaling value",255,0,255,'a');
                        
  convertToUC = param->AddParamBool("ConvertToUC","Convert to unsigned char image",false,'c');

  if(!IOUtils::ParseCommandLineEvalHelp(*param, argc, argv)) {
    cout<<"Tool for Scaling and Shifting images:"<<endl;     
    return -1;
  }
  int res=0;
  if((res = BIAS::ImageIO::Load(*sourceImageName,origImage)) <0)
    {
      cout<<" Loading of source image :"<<*sourceImageName<<"failed!"<<endl;
      return -1;
    }  


  BIAS::ImageBase::EStorageType type=origImage.GetStorageType();

  if(type==BIAS::ImageBase::ST_float){
    BIAS::Image<float> resImage(origImage);    
    resImage.ScaleShiftBetween(*min,*max);
    if(*convertToUC){
      BIAS::Image<unsigned char> imageUC;
      BIAS::ImageConvert::ConvertST(resImage,imageUC,ImageBase::ST_unsignedchar);
      BIAS::ImageIO::Save(*sinkImageName,imageUC);
    }
    else{
      BIAS::ImageIO::Save(*sinkImageName,resImage);
    }
    cout<<"Wrote image to "<<*sinkImageName<<endl;
  }
  else if(type==BIAS::ImageBase::ST_unsignedchar){
    BIAS::Image<unsigned char> resImage(origImage);    
    resImage.ScaleShiftBetween(*min,*max);
    BIAS::ImageIO::Save(*sinkImageName,resImage);
    cout<<"Wrote image to:"<<*sinkImageName<<endl;
  }
  else{
    cout<<"Source Image is of wrong storage type:"<<type<<endl;
    return -1;
  }
 
 
}


