/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003-2009    (see file CONTACT for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <Image/Camera.hh>
#include <iostream>
#include <Base/Image/ImageIO.hh>
#include <Base/Image/ImageConvert.hh>


using namespace BIAS;
using namespace std;

/** 
 *  @file
 *  @ingroup g_tools
 *  @brief loads two images and sets UUID from second image to first image,
 *  if only one image is given, the UUID is printed to the command line, see  biassetrefuuid.cpp
 *  @author MIP
*/
int main(int argc, char *argv[])
{
  if (argc<2) {
    cout<<"Usage: biassetrefuuid <image> [<ref-image>]"<<endl;
    exit(1);
  }

  bool SetID=false;
  if (argc>2) SetID=true;

  ImageBase LoadImg,LoadRefImg;
  BIAS::UUID newrefid,oldrefid;
  if (ImageIO::Load(argv[1],LoadImg)<0) exit(1);
  if (SetID) {
    if (ImageIO::Load(argv[2],LoadRefImg)<0) exit(1);
    newrefid = LoadRefImg.GetUID();
  }

  if (LoadImg.GetStorageType() == ImageBase::ST_float) {
    Camera<float> cam(LoadImg);
    //   cam.SetDebugLevel(D_CAMERA_METADATA);
    cam.ParseMetaData();
    oldrefid = cam.GetReferenceUUID();
    if (SetID) {
      cam.SetReferenceUUID(newrefid);
      cam.UpdateMetaData();
      //ImageIO::Save(argv[1],cam);
      ImageIO::Save(argv[1],cam);
    }
  }
  
  if (LoadImg.GetStorageType() == ImageBase::ST_unsignedchar) {
    Camera<unsigned char> cam(LoadImg);
    cam.SetDebugLevel(D_CAMERA_METADATA);
    cam.ParseMetaData();
    oldrefid = cam.GetReferenceUUID();
    if (SetID) {
      cam.SetReferenceUUID(newrefid);
      cout <<"Check: "<<cam.GetReferenceUUID()<<endl;
      cam.UpdateMetaData();
      //ImageIO::Save(argv[1],cam);
      ImageIO::Save(argv[1],cam);
   }
  }
  
  if (SetID) {
    cout <<"Old RefUUID: "<<oldrefid<<endl;
    cout <<"New RefUUID: "<<newrefid<<endl;
  }
  else
    cout <<"Current RefUUID: "<<oldrefid<<endl;
  
  return 0;
}
