/*
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003-2009    (see file CONTACT for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <stdlib.h>
#include <iostream>
#include <string>

#include <Base/Debug/TimeMeasure.hh>

using namespace std;
using namespace BIAS;

/**
  @file
  @ingroup g_tools
  @brief Helper to emulate the Unix "time" command on WIN32 platforms,
  Measure the duration of the execution of a command=timean application run,
  You may want to use TimeThis.exe from MS Resource Kit, see biastimeit.cpp
  @author Jan Woetzel 2005
*/
int main(int argc, char** argv)
{   
  if (argc<=1){
    cout<<"usage: "<<endl;
    if (argc>0) cout<<argv[0]<<" ";
    cout<<"<application.exe> [<parameters>] "<<endl;
    cout<<"example: timeit mapp.exe -myarg1"<<endl;
    return -1;
  }
  // variables:
  int result = 0;
  string command="";
  TimeMeasure timer1;

  // create command: 
  for (int i=1; i<argc; i++)
  {
    command+=argv[i];
    command+= " "; // delimiter
  }
  //cout<<"overhead: "<<timer1.MeasureOverhead()<<endl;      
  cout<<"timing: "<<command<<endl;

  // run timed application as system call: 
  timer1.Start();
  result = system( command.c_str() );


  //{ // timed code
  //  double d(1);
  //  for (unsigned int a=0; a< (unsigned int)(5.0* 4000./5.); a++){
  //    for (unsigned int b=0; b<10000; b++){
  //      d+=sin(1.0+1.1*d);
  //      d-=cos(2.0+1.2*d);
  //    }
  //  }
  //  Sleep(5000);
  //}

  timer1.Stop();

  if (result!=0){
    cout<<"could not run, result="<<result<<endl;
    return -2;
  } else {
    cout<<"timing successful. "<<endl;
    // display final timing result: 
    timer1.Print(cout);        
  }
  return 0;
}
