#include <Base/Image/ImageIO.hh>
#include <OpenGLFramework/RenderingContext/glfPBuffer.hh>
#include <OpenGLFramework/RenderingContext/glfRenderingContext.hh>
#include <iostream>

#include <VideoSource/VideoSource_Kinect2.hh>
#include <OpenGLFramework/SpecializedBatches/MultipleDepthWarp.hh>

using namespace std;
using namespace BIAS;

int main(int argc, char *argv[]) {


  glfPBuffer context;
  glfRenderingContextConfig config;
  config.width = 5;
  config.height = 5;
  config.redSize = 8;
  config.greenSize = 8;
  config.blueSize = 8;
  config.alphaSize = 8;
  context.Init(config);

  context.MakeCurrent();

  std::string cdir="/home/fkellner/20150415_calibration_kinects_only/";
  std::string idir="/home/fkellner/20150415_test_kinect/";

  Image<float> depthmapmain,depthmapsieg;
  ImageIO::Load(idir+"kinect-main-00000-depth.mip", depthmapmain);
  ImageIO::Load(idir+"kinect-sieg-00000-depth.mip", depthmapsieg);

  Projection projmain, projsieg;
  projmain.Load(cdir+"igor-kinect-calibration-main.xml");
  projsieg.Load(cdir+"igor-kinect-calibration-sieg.xml");
  std::vector<Projection> sourceCams;
  sourceCams.push_back( projmain );
  sourceCams.push_back( projsieg );
  Projection target = projmain;
  Vector3<double> C = projsieg.GetC() + projmain.GetC();
  C.MultIP(0.5);
  target.SetC( C );
  Quaternion<double> Q = projsieg.GetQ().Interpolate(projmain.GetQ(), 0.5);
  target.SetQ( Q );

  ProjectionParametersPerspective *targetCam = dynamic_cast<ProjectionParametersPerspective *>(target.GetParameters());

  MultipleDepthWarp warp;
  warp.SetWarpType(MultipleDepthWarp::DEPTH);
  warp.FlipTextureCoordinates(true);
  warp.Init( *targetCam, sourceCams, true);
  warp.UploadDepthMap( depthmapmain,0 );
  warp.UploadDepthMap( depthmapsieg,1 );
  warp.Warp();
  Image<float> result;
  warp.GetWarpedDisparity(result);
  ImageIO::Save("warp.mip", result);
  return 0;
}
