/*
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003-2009    (see file CONTACT for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "Checksum.hh"

using namespace BIAS;

// CRC-CCITT-16 polynominal:
// x^16+x^12+x^5+1 = (1) 0001 0000 0010 0001 = 0x1021

Checksum::Checksum() {}

Checksum::~Checksum() {}

// calculates CCITT-16 checksum with most significant bit first MSB
bool Checksum::
CalcCRC_CCITT16(unsigned int& crc,unsigned char* input, unsigned length)
{
	crc = 0;
	for(unsigned i=0;i<length;i++)
	{
		crc = crc ^ (input[i] << 8);
		for(unsigned j=0;j<8;j++){
			if(crc & 0x8000) {   // if leftmost (most significant) bit is set
				// CRC-CCITT-16 polynominal:
				// x^16+x^12+x^5+1 = (1) 0001 0000 0010 0001 = 0x1021
				crc = (crc << 1) ^ 0x1021;
			} else {
				crc = (crc << 1);
			}
			crc = crc & 0xffff;      // Trim remainder to 16 bits
		}
	}
	return true;
}
