/*
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003-2009    (see file CONTACT for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#ifndef CHECKSUM_HH_
#define CHECKSUM_HH_

#include <bias_config.h>

namespace BIAS
{
/** \class Checksum
 * \brief Utility class to calculate checksums as CRCs (Cyclic Redundancy Check).
 * Currently Only the CCITT16 is claculated, add functions when necessary.
 * \note The type of checksum is defined by the polynomial used:
 * for the CCITT16 it is x^16+x^12+x^5+1 = (1) 0001 0000 0010 0001 = 0x1021
 * this hex code can be replaced in the function to calculate different
 * types of CRCs
 * see http://en.wikipedia.org/wiki/Cyclic_redundancy_check for a list of polynomials
 * \author ischiller
 * \date 01/2011
 */
class BIASUtils_EXPORT Checksum {
public:
	Checksum();
	virtual ~Checksum();

	/**\brief Calculates CCITT-16 checksum with most significant bit first MSB
	 * \author ischiller
	 * \ingroup g_utils
	 * \date 01/2011*/
	static bool CalcCRC_CCITT16(unsigned int& crc, unsigned char* input, unsigned lenght);

};
}

#endif /* CHECKSUM_HH_ */
