/*
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003-2013    (see file CONTACT for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "ColourMap.hh"

using namespace std;
using namespace BIAS;


void ColourMap::Apply(const Image<float>& imageIn,
                      Image<unsigned char>& imageOut)
{
  // assertions
  BIASASSERT(imageIn.GetChannelCount());

  // copy and normalise input image
  Image<float> imageInNorm(imageIn);
  imageInNorm.ScaleShiftBetween(0, 1);

  // create output image
  imageOut.Init(imageInNorm.GetWidth(), imageInNorm.GetHeight(), 3);

  // get data pointers
  const float*   imageInData  = imageInNorm.GetImageData();
  unsigned char* imageOutData = imageOut.GetImageData();

  // apply colour map
  for (size_t y = 0; y < imageInNorm.GetHeight(); ++y)
  {
    for (size_t x = 0; x < imageInNorm.GetWidth(); ++x)
    {
      const size_t offsetIn  = y * imageInNorm.GetWidth() + x;
      const size_t offsetOut = y * imageInNorm.GetWidth() * imageOut.GetChannelCount()
                               + x * imageOut.GetChannelCount();

      const float curValue = imageInData[offsetIn];

      vector<unsigned char> curColour;
      m_mapGradient.GetColour(curValue, curColour);

      imageOutData[offsetOut]     = curColour[0];
      imageOutData[offsetOut + 1] = curColour[1];
      imageOutData[offsetOut + 2] = curColour[2];
    }
  }
}
