/*
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003-2013    (see file CONTACT for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __COLOUR_MAP_HH__
#define __COLOUR_MAP_HH__

// BIAS
#include <bias_config.h>
#include <Utils/ColourGradient.hh>
#include <Base/Image/Image.hh>

namespace BIAS
{
  /**
   * @brief Applies a colour map to an image.
   *
   * @todo so far only float images are supported
   *
   * @author rwulff
   * @date   07/2013
   */
  class BIASUtils_EXPORT ColourMap
  {

  public:

    /**
     * @brief Constructor. Creates a ColourMap with the given gradient.
     *
     * @param mapGradient
     *        the gradient to use for colouring the image
     */
    inline
    ColourMap(const ColourGradient<unsigned char>& mapGradient)
      : m_mapGradient(ColourGradient<unsigned char>::Jet, 255)
    { }

    /**
     * @brief Constructor. Creates a ColourMap with one of the preset gradients.
     *
     * @param gradientType
     *        the gradient preset type
     */
    inline
    ColourMap(ColourGradient<unsigned char>::ColourGradientType gradientType
              = ColourGradient<unsigned char>::Gray)
      : m_mapGradient(gradientType, 255)
    { }

    /**
     * @brief Applies the current ColourMap to the given image and returns the
     *        result.
     *
     * A copy of the image is normalised to [0, 1] before the map is applied.
     *
     * @param[in]  imageIn
     *             input image to apply colour map on. must be single-channel
     * @param[out] imageOut
     *             the coloured image
     */
    void Apply(const Image<float>& imageIn,
               Image<unsigned char>& imageOut);

  private:

    ColourGradient<unsigned char> m_mapGradient;

  };
}

#endif // __COLOUR_MAP_HH__
