/*
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003-2009    (see file CONTACT for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __DataPlot_hh__
#define __DataPlot_hh__

#include <bias_config.h>
#include <Base/Image/Image.hh>
#include <Base/Image/ColourRGB.hh>
#include <vector>

namespace BIAS {

	/**
	 * \class DataPlot
	 * \brief Class to plot data as lines, crosses etc, as gnuplot does, duplicates functionality in
	 * class GraphPlotter
	 * \author evers
	 * \date April 2009
	 */
  class BIASUtils_EXPORT DataPlot 
  {
    
  public:

    enum  DrawingStyle {
      Points,
      Lines,
      Cross
    };

    // put an complete initialized image here.
    DataPlot(Image<unsigned char> &PlotArea);
    ~DataPlot();

    void SetXRange(float min,float max);
    void SetYRange(float min,float max);
    void SetDrawingStyle(enum DrawingStyle);
    void SetAutoRange(bool x, bool y);
    void SetThickness(unsigned int t){Thickness_ = t;};
    // draw a complete data set at once
    void Plot(const std::vector<float> &data);
    void CutLinePlot();

    void Plot(const std::vector<float> &x,const std::vector<float> &y);

    /** \brief draw single value, no AutoRange possible
				\return 0 for success, -1 for x out of range, -2 for y out of range */
    int Plot(float x, float y);

    void Clear();
    void DrawXAxis();
    void DrawYAxis();
    void SetBGColour(ColourRGB<unsigned char> color)
    {BGColour_ = color;};
    void SetPenColour(ColourRGB<unsigned char> color)
    {PenColour_ = color;}


  protected:
    // convert window coordinates (MinX_<=wx<=MaxX_, MinY_<=wy<=MaxY_) to view-port coordinates
    // 0<=vx<PlotArea_->GetWidht(), 0<=vy<=PlotArea_->GetHeight()
    int WindowToViewport_(float wx,float wy, unsigned int &vx,unsigned int &vy);

    void DrawLine_(unsigned int x,unsigned int y,unsigned int x2, unsigned int y2);

    float MaxX_,MinX_,MaxY_,MinY_;
    bool AutoRangeX_,AutoRangeY_;
    unsigned int BorderX_,BorderY_;
    Image<unsigned char >*PlotArea_;
    unsigned char **DrawIda_; // cache these
    unsigned char ChannelCount_; // cache these
    unsigned int PlotAreaHeight_,PlotAreaWidth_; 

    DrawingStyle DrawStyle_;
    int  Thickness_;

    ColourRGB <unsigned char > BGColour_,PenColour_;
    int LastVX_,LastVY_;
  }; //class

} //namespace



#endif

