/*
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003-2009    (see file CONTACT for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/** @example ExampleMeshFromImage.cpp
 *  @relates TriangleMesh, ThreeDOut
    @ingroup g_examples
    @brief Example for generating a trianglemesh from an image
    @author MIP */

#include <Utils/TriangleMesh.hh>
#include <Utils/ThreeDOut.hh>
#include <Base/Image/ImageIO.hh>
#include <Base/Image/ImageConvert.hh>
#include <Image/HomographyMapping.hh>
#include <iostream>

using namespace std;
using namespace BIAS;

void usage(char* name) 
{
  cout<<"usage:\n";
  cout<<name<<" <depthmap> <texture> [outputFile]\n";
}

int main(int argc, char* argv[]) 
{
  if(argc<2) {
    usage(argv[0]);
    return -1;
  }

  ImageBase depthMap;
  if(ImageIO::Load(argv[1], depthMap)!=0) {
    cout<<"depth map could not be loaded!\n";
    usage(argv[0]);
    return -1;
  }

  Image<float> source, enlarged;
  ImageConvert::ConvertST(depthMap, source, ImageBase::ST_float);
    
  //Unhardcode Scale Factor
  HomographyMapping<float, float> HMapper;
  HMatrix H(MatrixIdentity);
  H[0][0]=H[1][1]=0.5;
  HMapper.SetHomography(H);
  enlarged.Init(source.GetWidth()*2,source.GetHeight()*2,1 );
  HMapper.Map(source, enlarged);
  Image<unsigned char> texture;
  if(ImageIO::Load(argv[2], texture)!=0) {
    cout<<"texture could not be loaded!\n";
    usage(argv[0]);
    return -1;
  }

  cout<<"meshing..."; cout.flush();
  TriangleMesh mesh;
  
  mesh.GenerateDenseMesh(enlarged, texture);  
  cout<<"finished\n";

  ThreeDOut vrmlOut;
  vrmlOut.AddTriangleMesh(mesh, "mesh_from_"+string(argv[1]), 
                          argv[2], false);

  cout<<"Writing VRML to "; 
  if(argc>3) {
    cout<<argv[3]<<"... ";cout.flush();
    vrmlOut.VRMLOut(argv[3]);
  } 
  else {
    cout<<"DenseTriangleMesh.wrl... ";cout.flush();
    vrmlOut.VRMLOut("DenseTriangleMesh.wrl");
  }
  cout<<"finished\n";
  
  return 0;
}
