/*
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003-2009    (see file CONTACT for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __GlutInitWrapper_hh__
#define __GlutInitWrapper_hh__

#include <Base/Common/Singleton.hh>

namespace BIAS {

  /**
   * @class GlutInitWrapper 
     @brief To call glutInit() at the right time and only once is not easy in
     complex applications. This wrapper helps to avoid calling it
     twice or more times.  At the eraliest possible place right after
     the gl-context is ready, just call:

     GlutInitWrapper::GetInstance()->Init();
     No need to protect it agains multiple calls, this is checked internally.
		 @ingroup g_openglframework	
		 @author ischiller, adopted from glewinitwrapper
   */
  class BIASUtils_EXPORT GlutInitWrapper: public Singleton<GlutInitWrapper>
  {
    
  public:
    int Init();
    int Init(int argc, char* argv[]);
    bool IsInitalised() {return IsInitialised_;} ;

    friend class BIAS::Singleton<GlutInitWrapper>;

  protected:

    GlutInitWrapper();

    bool IsInitialised_;
  }; //class


} //namespace


#endif
