/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003-2009    (see file CONTACT for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __GRAPHPLOTTER_hh__
#define __GRAPHPLOTTER_hh__

// includes
#include <bias_config.h>
#include <Base/Image/Image.hh>

#ifndef BIAS_HAVE_OPENCV 	 
#  error BIAS_HAVE_OPENCV not defined. Please recompile BIAS with USE_OPENCV enabled if you want to use GraphPlotter.
#endif
namespace BIAS {
  
  enum GraphPlotter_DrawStyle{
    DS_POINT_1 =1,
    DS_POINT_2,
    DS_POINT_3,
    DS_POINT_4,
    DS_POINT_5,
    DS_LINE
  };

  /** @class GraphPlotter
      @brief Use this class to plot data as gnuplot does
			@ingroup g_utils
      @author ischiller 10/2008 */
  class BIASUtils_EXPORT GraphPlotter 
  {
  public:
    /// constructor
    GraphPlotter();

    /// destructor
    ~GraphPlotter();

    void SetXRange(double minX, double maxX);    
    void SetYRange(double minY, double maxY);    
    
    void SetXTics(unsigned xtics);    
    void SetYTics(unsigned ytics);        

    void SetXLabel(std::string label);
    void SetYLabel(std::string label);
    
    void SetImageSize(unsigned width, unsigned height);

    void SetDrawGrid(bool drawGrid);
    void SetAxesToYLeftAndXBottom(bool fixAxes);
    
    void SetLineThickness(unsigned thickness);
    void SetTextThickness(unsigned thickness);

    void SetLabel(std::string label, BIAS::ColourRGB<unsigned char> color){
    	std::pair<std::string,BIAS::ColourRGB<unsigned char> > lb;
    	lb = std::make_pair(label,color);
    	labels_.push_back(lb);
    }
    
    void Reset();

    /**
       @brief draws xvalues against yvalues
    */
    void Draw(std::vector<double>& xValues,
              std::vector<double>& yValues,
              BIAS::ColourRGB<unsigned char> color,
              BIAS::GraphPlotter_DrawStyle = DS_POINT_1);

    /**
       @brief draws xvalues against yvalues with standard deviations 
              as errorbars
    */
    void Draw(std::vector<double>& xValues,
              std::vector<double>& yValues,
              std::vector<double>& yStdDeviation,
              BIAS::ColourRGB<unsigned char> color,
              BIAS::ColourRGB<unsigned char> colorSTD,
              BIAS::GraphPlotter_DrawStyle = DS_POINT_1,
              std::vector<std::string> *labels = NULL);

		
    void DrawLabels();

    
    BIAS::Image<unsigned char>* GetImage();
    BIAS::Image<unsigned char>  GetCopyOfImage();

  protected:
    void Prepare_();
    std::vector< std::pair<std::string,BIAS::ColourRGB<unsigned char> > > labels_;
    
    BIAS::Image<unsigned char> image_;
    double   xCoeff_, yCoeff_;
    double   xMin_, xMax_, yMin_,yMax_;
    int      xTics_ , yTics_;
    unsigned width_ , height_;
    std::string xLabel_,yLabel_;
    int xOffset_,yOffset_;
    bool bDrawGrid_;
    bool bAutoDetectBordersX_;
    bool bAutoDetectBordersY_;
    bool bPrepared_;
    bool bFixXAxisToBottom_;
    bool bFixYAxisToLeft_;
    unsigned dLineThickness_;
    unsigned dTextThickness_;
  }; // class GraphPlotter

} // namespace

#endif // __GRAPHPLOTTER_hh__
