/*
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003-2009    (see file CONTACT for details)
Multimediale Systeme der Informationsverarbeitung
Institut fuer Informatik
Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef IMAGEVALUEBAR_HH_
#define IMAGEVALUEBAR_HH_

#include <Base/Image/Image.hh>

namespace BIAS
{
	/**
	 * \brief this class represents an image value bar.
	 * This means it dispays the values in an image with description
	 * Usefull for example to illustrate the depth of a scene in a depth map
	 * This class is derived from Image, so you can display it as a normal image
	 * and insert it in other images for example.
	 * When you set the size, part of it will be used to display the ticks and units
	 * Use ImageLegendFrame to display it!
	 * \author ischiller
	 * \ingroup g_utils
	 * \date 03/11/2010
	 */

	template<class StorageType>
  class BIASUtils_EXPORT ImageValueBar : public Image<StorageType>
  {
	public:
		ImageValueBar();
		virtual ~ImageValueBar();

		void SetDisplayRange(double min, double max){minDisplay_ = min; maxDisplay_ = max;};
		void SetValueRange(double min, double max){minValue_ = min; maxValue_ = max;};
		void SetTicks(int num_ticks){ticks_ = num_ticks;};
		void SetUnit(std::string unit){unit_ = unit;};
		void SetColorModel(ImageBase::EColorModel model){model_ = model;};
		void Update();

	protected:
		double minValue_, maxValue_;
		double minDisplay_, maxDisplay_;
		int ticks_;
		std::string unit_;
		ImageBase::EColorModel model_;
	};

}//end namespace
#endif /* IMAGEVALUEBAR_HH_ */
