/*
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003-2009    (see file CONTACT for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __IndexLineSetHandler_hh__
#define __IndexLineSetHandler_hh__

#include <string>
#include <vector>
#include <iostream>
#include <Base/Geometry/HomgPoint3D.hh>


namespace BIAS{
	/**
	 * \class IndexLineSetHandler
	 * \brief This class parses a VRML file containing an indexed line set and
	 * stores the containing lines.
	 * \author Ingo Schiller
	 * \date 05/2007
	 * \ingroup g_utils
	 */
  class BIASUtils_EXPORT IndexLineSetHandler
  {
  public:
    IndexLineSetHandler();
    ~IndexLineSetHandler();
        
    /**
       Parse a IndexLineSet and return lines in it
    */
    std::vector<std::vector<BIAS::HomgPoint3D> > Parse(std::string fileName,
                                                       int &dNumberOfLines);

    /**
       Write a set of lines to disk as a IndexedLineSet
     */
    int Write(std::string fileName,
              std::vector<std::vector<BIAS::HomgPoint3D> > fLines);

  private:
    int GetColumnElement(std::vector< bool > column, int size);

    int FindPoint(BIAS::HomgPoint3D point,
                  std::vector<double > pX,
                  std::vector<double > pY,
                  std::vector<double > pZ, 
                  int size);

 
    std::vector<std::vector<double> >     vDetectedPoints_;
    std::vector<std::vector<int>    >     vConnectingLines_;
    int                                   dNumberOfLinesInSet_;
  };

}

#endif
