/*
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003-2009    (see file CONTACT for details)
Multimediale Systeme der Informationsverarbeitung
Institut fuer Informatik
Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#ifndef MARCHINGCUBES_HH_
#define MARCHINGCUBES_HH_
#include <bias_config.h>
#include <Base/Math/Vector3.hh>

namespace BIAS {
	
	class BIASUtils_EXPORT MarchingCubesTriangle
	{
	public:
		bool IsDifferent(MarchingCubesTriangle& mct);
		BIAS::Vector3<double> p[3];
	};
	
	typedef struct {
		BIAS::Vector3<double> p[8];
		double val[8];
	} MarchingCubesGridCell;
	
	/**
	 * \class MarchingCubes
	 * \brief creates a closed mesh from a set of points. This class only calculates
	 * the intersecting triangles of one cube, call multiple times for all points
	 * in the point cloud. Implements the algorithm in Lornsen1987:Marching Cubes
	 * \author ischiller
	 * \ingroup g_utils
	 * \date 08/2010
	 */
	class BIASUtils_EXPORT MarchingCubes {
	public:
		MarchingCubes();
		virtual ~MarchingCubes();

		/** \brief Given a grid cell and an isolevel, calculate the triangular
		   facets required to represent the isosurface through the cell.
		   \param grid[in]: the 8 grid cell corners which include coordinates and weights
		          the weight of unoccupied corners should be 0, the weight if valid
		          corners should be > 0 and > isolevel (parameter 2). The surface
		          through the cube will be calculated according to these weights.
		   \param isolevel[in]: isolevel of the cube
		   \param triangles[out]: output triangles  will be loaded up with the vertices at most 5 triangular facets.
		   \return the number of triangular facets 0 will be returned if the grid cell is either totally above
		   of totally below the isolevel.			   */
		int PolygoniseCube(MarchingCubesGridCell grid,double isolevel,
				               MarchingCubesTriangle *triangles);

	protected:
		BIAS::Vector3<double> InterpolateVertex_(double isolevel,
				BIAS::Vector3<double> p1,BIAS::Vector3<double> p2,
				double valp1,double valp2);
		
	};

}

#endif /* MARCHINGCUBES_HH_ */
