/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004    (see file CONTACTS for details)
Multimediale Systeme der Informationsverarbeitung
Institut fuer Informatik
Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __MUTEXSINGLETON__
#define __MUTEXSINGLETON__

#include <pthread.h>
#include <Base/Debug/Error.hh>

namespace BIAS {
  /**
     @class MutexSingleton
     @brief Wrapper for pthread mutex for use with 'automatic' 
            initialization. 
		 @ingroup g_utils
     @author apetersen
     @date 12/11/07
   */
  class MutexSingleton {
  public:
    MutexSingleton(): m() {
      //initialize pthread_mutex_t
      #ifdef BIASASSERT_ISACTIVE
        BIASASSERT(pthread_mutex_init(&m, NULL) == 0);
      #else
        pthread_mutex_init(&m, NULL);
      #endif
    }

    ~MutexSingleton() {pthread_mutex_destroy(&m);}


    int Lock() {return pthread_mutex_lock(&m);}
    int Unlock() {return pthread_mutex_unlock(&m);}

  private:
    pthread_mutex_t m;
  };
}

#endif //__MUTEXSINGLETON__
