/*
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003-2009    (see file CONTACT for details)
Multimediale Systeme der Informationsverarbeitung
Institut fuer Informatik
Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef _StatekeeperGL__hh__
#define _StatekeeperGL__hh__

#include <bias_config.h>

#ifndef BIAS_HAVE_OPENGL
#  error Please recompile BIAS with USE_OPENGL
#endif // BIAS_HAVE_OPENGL


#  include <Gui/biasgl.h>

namespace BIAS {

  /** push and pop GL states on constructor/destructor 
  Helper class pop(save) OpenGL states on construction
  and push them on destruction automatically.
  Using matching  push/pop pairs is easy with this class.
  Sample usage:
  \verbatim
  // begin var. scop
  {
  // save state by push
  StatekeeperGL myTmp;   
  // change the state, fire GL calls 
  ...   
  // reload state by destructors pop call automatically
  }
  // end var. scope
  \endverbatim
  There should not be a performance impact if you use display lists. 
  @author Jan Woetzel */
  class BIASUtils_EXPORT StatekeeperGL {
  public:
    /// constructor, pushes major OpenGL states
    StatekeeperGL(){
      glPushAttrib( 0
        //|GL_ALL_ATTRIB_BITS 
        |GL_ENABLE_BIT 
        |GL_COLOR_BUFFER_BIT 
        |GL_CURRENT_BIT 
        //|GL_DEPTH_BUFFER_BIT 
        //|GL_LIGHTING_BIT 
        //|GL_SCISSOR_BIT 
        //|GL_STENCIL_BUFFER_BIT 
        );
    };

    /** constructor, pushes OpenGL state as given by mask
    GLbitfield mask */
    StatekeeperGL(const GLbitfield & mask){
      glPushAttrib(mask);
    }

    /// destructor, pops OpenGL state 
    ~StatekeeperGL(){
      glPopAttrib();
    }

  }; // end of class
} // end namespace


#endif

