/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003-2009    (see file CONTACT for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


/** @example ExampleVideoSink.cpp
    @relates VideoSink
    @ingroup g_examples    
    @ingroup g_videosink
    @brief  Example for using the videosink class to write a ffmpeg encoded video
    @author MIP
*/

#include <iostream>
#include <vector>
#include <string>
#include <VideoSink/VideoSink.hh>
#include <Utils/Param.hh>

using namespace std;
using namespace BIAS;

void Usage(){
	cout<<"\n\nUsage: ExampleVideoSink imagelist.lst\n";
	cout<<"imagelist.lst is a list containing one filename of in image in each row.\n\n";
	
}

int main(int argc, char* argv[])
{
  VideoSink vs;
  if(argc < 2){
  	Usage();
  	return -1;
  }
  std::vector<string> imageList;
  if (Param::ParseListFile(argv[1], imageList) != 0){
  	BIASERR("Could not parse list file."<<argv[1]);
  	return -1;
  }

  for (unsigned i = 0; i < imageList.size(); i++)
  {
    vs.AddImage(imageList[i]); 
  }
  vs.SetOutputVideo("out.mpg");
  vs.SetFPS(25);
  vs.SetBPP(10);

  cout << "Encoding...\n";
  if (!vs.Encode())
    cout << "Error: " << vs.GetErrMsg() << endl;

  cout << "Done!" << endl;
}

