
#include <Utils/Param.hh>
#include <Utils/IOUtils.hh>
#include <VideoSink/VideoSink_FFmpeg.hh>

using namespace std;
using namespace BIAS;

int main(int argc, char** argv) {

    Param params(true);

    string* videoFile = params.AddParamString("resultVideoFile",
                                              "path to the result video file", "result.mkv", 'o');

    string* imageList = params.AddParamString("imageList",
                                              "path to image list", "", 'i');

    bool* losslessCoding = params.AddParamBool("lossLessCoding", "choose loss less video coder", false);


    if (!IOUtils::ParseCommandLineEvalHelp(params, argc, argv))
        return 0;

    vector<string> imageNames;

    if (Param::ParseListFile(*imageList, imageNames) != 0) {
        BIASERR("error: loading list " << *imageList << endl)
                return -1;
    } else {
        cout << "loaded image list: " << *imageList << endl;;
    }

    VideoSink_FFmpeg encoder;

    Image<unsigned char> image;
    if (ImageIO::Load(imageNames[0],image)!=0) {
        BIASERR("could not load first image from list")
                return -1;
    }

    encoder.SetSize(image.GetWidth(),image.GetHeight());

    if (*losslessCoding) {
        encoder.SetEncoder(CODEC_ID_FFV1);
    } else {
        encoder.SetEncoder(CODEC_ID_MPEG4);
    }

    int res = encoder.Open(*videoFile);

    if (res == 0) {

        res = encoder.AddFrames(imageNames);

        if (res!=0) {
            cout << encoder.GetError() << endl;
        }
    } else {
        BIASERR(encoder.GetError())
            }

    return 0;
}
