/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003-2009    (see file CONTACT for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#ifndef _VIDEOSINK_HH_
#define _VIDEOSINK_HH_

#include <bias_config.h>
#include <Base/Debug/Debug.hh>
#include <VideoSource/VideoSource_Disk.hh>
#include <Base/Image/Image.hh>
#include <Base/Image/ImageConvert.hh>
#include <Base/Image/ImageIO.hh>

extern "C"
{
  #include <avcodec.h>
  #include <avformat.h>
  #include <avutil.h>
  #include <swscale.h>
}

#include <vector>
#include <string>


// FIXME
void av_free_packet(AVPacket *pkt) 
{ 
  if (pkt)
  {
    if (pkt->destruct) pkt->destruct(pkt);
    pkt->data = NULL; pkt->size = 0;
  }
}

namespace BIAS
{

/**
  \class VideoSink
  \ingroup g_videosink
  \author Friso Evers
  \brief VideoSink for writing FFMPEG Videos from images 
  \deprecated Is replaced by VideoSink_FFmpeg
 */
class BIASVideoSink_EXPORT VideoSink
{
public:
  VideoSink();
  VideoSink(std::string Video);
  VideoSink(std::vector<std::string> Images);
  VideoSink(std::vector<std::string> Images, std::string Video);
  ~VideoSink();

  /**
   * Append an image to the end of the list.
   */
  void AddImage(std::string Image);
  void AddImages(std::vector<std::string> Images);

  /**
   * Set the file where the video will be written to.
   * This or the according constructor must be called befor encoding.
   */
  void SetOutputVideo(std::string Video);
  /**
   * Set the framerate for the video.
   * Note that the encoder only works with some values (e.g. 25, 50).
   */
  void SetFPS(float FPS);

  /**
   * Set the bitrate of the output video.
   * This overrides a value set by SetBPP.
   */
  void SetBitrate(int Bitrate);
  /**
   * Set the bits per pixel of the output video. The initial value is 4.
   * This overrides a value set by SetBitrate.
   */
  void SetBPP(double BPP);

  /**
   * Encode the video.
   */
  bool Encode();

  /**
   * Return the last produced errormessage.
   * Use the return value of the methods to test for errors though since this
   * message is not reset on success.
   */
  std::string GetErrMsg();

protected:
  bool _SetupVideoSource();
  //bool _SetupSwsContext(PixelFormat fromFmt, int width = 0, int height = 0);

  /**
   * Convert RGB888p data to Y'UV420p.
   *
   * \param rgb Pointer to RGB data.
   * \param yuv Pointer to YUV data, must point to an allocated memory with a
   *            minimum size of (width*height * 3) / 2.
   */
  static void _RGB888pToYUV420p(uint8_t *rgb, uint8_t *yuv,
				int width, int height);

  std::vector<std::string> _ImgFiles;
  std::string _Video;
  float _FPS;
  int _Bitrate;
  int _BPP;
  VideoSource_Disk _VSD;
  bool _IsYUV;

  AVCodec *_Codec;
  AVCodecContext *_CodecCtx;
  //SwsContext *_SwsCtx;

  std::string _ErrMsg;
};

}

#endif
