/*
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003-2009    (see file CONTACT for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


/** @example ExampleVideoCentaurus.cpp
    @relates VideoSource_Centaurus
    @ingroup g_examples
    @ingroup g_videosource
    @brief  Example for using the videosource centaurus
    @author MIP
*/

#include <VideoSource/VideoSource_Centaurus.hh>
#include <Base/Image/ImageIO.hh>

using namespace std;
using namespace BIAS;

int main(int argc, char** argv)
{

    int res=-1;
    Image<unsigned char> image1;
    image1.Init(1920, 1080, 2, ImageBase::ST_unsignedchar);
    image1.SetColorModel(ImageBase::CM_YUYV422);

    Image<unsigned char> image2;
    image2.Init(1920, 1080, 2, ImageBase::ST_unsignedchar);
    image2.SetColorModel(ImageBase::CM_YUYV422);


    char setup[] = "PCI,card=0";
//    char setup[] = "";
    sv_handle* cam_handleP1_= NULL;

    cam_handleP1_ = sv_open(&setup[0]);



     if (cam_handleP1_==NULL) {
         cout << "failed to open cam, camera handle = " << cam_handleP1_ << endl;
         return -1;
     } else {
         cout << "opened cam " << cam_handleP1_ << endl;
     }




     // Configure single-link input B (jack ID 3)
         int mode = SV_MODE_COLOR_YUV422_YUYV
                    | SV_MODE_SMPTE274_25sF | SV_MODE_STORAGE_FRAME;


         //-------------------------------------- A
       res = sv_jack_option_set(cam_handleP1_, 1, SV_OPTION_VIDEOMODE, mode);
       if (res != SV_OK) {
               BIASERR(sv_geterrortext(res) << endl);
       }
       res = sv_jack_option_set(cam_handleP1_, 1, SV_OPTION_IOMODE, SV_IOMODE_YUV422);
       if (res != SV_OK) {
               BIASERR(sv_geterrortext(res) << endl);
       }

             res = sv_option_set(cam_handleP1_,SV_OPTION_MULTICHANNEL,SV_MULTICHANNEL_INPUT);
            if (res != SV_OK) {
                    BIASERR(sv_geterrortext(res) << endl);
            }

       //--------------------------------------- B


       res = sv_jack_option_set(cam_handleP1_, 3, SV_OPTION_VIDEOMODE, mode);
       if (res != SV_OK) {
               BIASERR(sv_geterrortext(res) << endl);
       }
       res = sv_jack_option_set(cam_handleP1_, 3, SV_OPTION_IOMODE, SV_IOMODE_YUV422);
       if (res != SV_OK) {
               BIASERR(sv_geterrortext(res) << endl);
       }


//        res = sv_videomode(cam_handleP1_, mode);
//         if (res != SV_OK) {
//                 BIASERR(sv_geterrortext(res) << endl);
//         }



//    int mode = SV_MODE_COLOR_YUV422_YUYV
//               | SV_MODE_SMPTE274_25sF | SV_MODE_STORAGE_FRAME;

//    res = sv_jack_option_set(cam_handleP1_,port,SV_OPTION_VIDEOMODE,mode);
//    if (res != SV_OK) {
//            BIASERR("sv_jack_option_set : " << sv_geterrortext(res) << endl);
//    }



    sv_fifo* inputFifoP1_;
    res = sv_fifo_init(cam_handleP1_, &inputFifoP1_,1, 0, 1,
                       SV_FIFO_FLAG_VIDEOONLY, 0);
    if (res != SV_OK) {
        BIASERR(sv_geterrortext(res) << endl);
    }


    sv_fifo* inputFifoP2_;
    res = sv_fifo_init(cam_handleP1_, &inputFifoP2_,3, 0, 1,
                       SV_FIFO_FLAG_VIDEOONLY, 0);
    if (res != SV_OK) {
        BIASERR(sv_geterrortext(res) << endl);
    }


    //-----------------------------
    res = sv_fifo_start(cam_handleP1_, inputFifoP1_);
    if (res != SV_OK) {
        BIASERR(sv_geterrortext(res) << endl);
    }

    res = sv_fifo_start(cam_handleP1_, inputFifoP2_);
    if (res != SV_OK) {
        BIASERR(sv_geterrortext(res) << endl);
    }

    int flags = 0;
    flags |= SV_FIFO_FLAG_VSYNCWAIT;
    flags |= SV_FIFO_FLAG_FLUSH;


    //--------------------------- A
    sv_fifo_buffer* sv_buffer1;
    res = sv_fifo_getbuffer(cam_handleP1_, inputFifoP1_, &sv_buffer1,
                    NULL, flags);
    if (res != SV_OK) {
            BIASERR(sv_geterrortext(res) << endl);

    }

    sv_buffer1->dma.addr = (char*) image1.GetImageData();
    sv_buffer1->dma.size = image1.GetWidth() * image1.GetHeight()
                    * image1.GetChannelCount();

    res = sv_fifo_putbuffer(cam_handleP1_, inputFifoP1_, sv_buffer1, NULL);
    if (res != SV_OK) {
            BIASERR(sv_geterrortext(res) << endl);
    }

    // -------------------------- B
    sv_fifo_buffer* sv_buffer2;
    res = sv_fifo_getbuffer(cam_handleP1_, inputFifoP2_, &sv_buffer2,
                    NULL, flags);
    if (res != SV_OK) {
            BIASERR(sv_geterrortext(res) << endl);

    }

    sv_buffer2->dma.addr = (char*) image2.GetImageData();
    sv_buffer2->dma.size = image2.GetWidth() * image2.GetHeight()
                    * image2.GetChannelCount();

    res = sv_fifo_putbuffer(cam_handleP1_, inputFifoP2_, sv_buffer2, NULL);
    if (res != SV_OK) {
            BIASERR(sv_geterrortext(res) << endl);
    }

////    VideoSource_Centaurus vc;
////    int res = vc.OpenDevice(setup, SV_OPENPROGRAM_DEFAULT, SV_OPENTYPE_DEFAULT, 0, 0);
////    cout << "res = " << res << endl;

////    res = vc.SetVideoMode();
////    res = vc.InitFiFo();
////   Camera<unsigned char> image;
////   vc.InitImage(image);

////   vc.GrabSingle(image);

   ImageIO::Save("testGrab1.mip",image1);
   cout << "saved testGrab1.mip" << endl;

   ImageIO::Save("testGrab2.mip",image2);
   cout << "saved testGrab2.mip" << endl;

    res = sv_close(cam_handleP1_);
    if (res != SV_OK) {
            BIASERR("sv_close : " << sv_geterrortext(res) << endl);
    }

    return 0;
}
