/*
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003-2009    (see file CONTACT for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <Utils/Param.hh>
#include <Utils/IOUtils.hh>
#include <VideoSource/VideoSource_FFmpeg.hh>

using namespace std;
using namespace BIAS;

int main(int argc, char** argv) {



    Param params(true);

    string* videoFile = params.AddParamString("videoFile",
                                              "path to the video file", "", 'i');

    int* maxNumberOfFrames = params.AddParamInt("maxNumberOfFrames",
                                              "maximal number of frames",INT_MAX, 0, INT_MAX, 'n');

    if (!IOUtils::ParseCommandLineEvalHelp(params, argc, argv))
        return 0;

    cout << "getting " << *maxNumberOfFrames << endl;


    Camera<unsigned char> image;

    VideoSource_FFmpeg decoder;

    if (decoder.OpenDevice(videoFile->c_str()) < 0 ) {
        cout << decoder.GetError();
        exit(1);
    }
    decoder.InitImage(image);
    decoder.PreGrab();


    string dir, base, suffix;
    FileHandling::SplitName(*videoFile, dir, base, suffix);

    int res = 0;

    unsigned int numberOfImageSaved = 0;


    while (res == 0 && (int)numberOfImageSaved < *maxNumberOfFrames) {

        cout << "got frame " << numberOfImageSaved << endl;
        res = decoder.GrabSingle(image);
        if(res != 0) BIASERR("Error:"<<decoder.GetError());
        string imageName = base+"_"+FileHandling::LeadingZeroString(numberOfImageSaved, 4) + ".png";
        ImageIO::Save(imageName,image);

        numberOfImageSaved ++;
    }


    return 0;
}
