/*
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003-2009    (see file CONTACT for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <VideoSource/VideoSource_Kinect2.hh>
#include <Base/Image/ImageIO.hh>
#include <iostream>

using namespace BIAS;
using namespace std;

int main(int argc, char *argv[])
{
  cout << "ExampleVideoSource_Kinect2 [<devID>] [<imgNum>]" << endl; // " [<devID2>]" << endl;
  int devnum = argc > 1 ? atoi(argv[1]) : -1;
  int imgnum = argc > 2 ? atoi(argv[2]) :  1;
  //int devnum2 = argc > 3 ? atoi(argv[3]) : -1;
  int res = 0;

  VideoSource_Kinect2 cam;
  //VideoSource_Kinect2 cam2;

  Camera<unsigned char> color;
  Camera<float> depth, ir;

  cam.SetMode(VideoSource_Kinect2::all);

  cam.InitImage(color);
  cam.InitDepthImage(depth);
  cam.InitIrImage(ir);

  if (devnum >= 0)
    res = cam.OpenDevice(devnum);
  else
    res = cam.OpenDevice();
  cout << "OpenDevice() ->  " << res << endl;
  if (res < 0) return -1;

  //if (devnum2 >= 0)
  //  cam2.OpenDevice(devnum2);
  //else
  //  cam2.OpenDevice();

  res = cam.PreGrab();
  //cam2.PreGrab();
  cout << "PreGrab() -> " << res << endl;

  if (imgnum > 1)
    cout << "Grabbing " << imgnum << " images..." << endl;
  for (int i = 0; i < imgnum; i++) {
    cam.GrabSingle(color);
    cam.GrabSingleDepth(depth);
    cam.GrabSingleIR(ir);
    //cam2.GrabSingle(color);
    //cam2.GrabSingleDepth(depth);
    //cam2.GrabSingleIR(ir);
    std::cout << "." << std::flush;
  }

  ImageIO::Save("kinect2-color.mip", color);
  ImageIO::Save("kinect2-depth.mip", depth);
  ImageIO::Save("kinect2-ir.mip", ir);

  res = cam.PostGrab();
  cout << "PostGrab() -> " << res << endl;

  cam.CloseDevice();
  cout << "CloseDevice() -> " << res << endl;

  //cam2.PostGrab();
  //cam2.CloseDevice();

  return 0;
}
