/*
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003-2009    (see file CONTACT for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <Base/Debug/TimeMeasure.hh>
#include <VideoSource/VideoSource_Kinect2.hh>
#include <Base/Image/ImageIO.hh>
#include <NetworkComm/CScommClient.hh>
#include <VideoSource/VideoSource_Kinect2Net.hh>
#include <iostream>

using namespace BIAS;
using namespace std;
/*
 * int main(int argc, char *argv[]) {

  VideoSource_Kinect2 cam;

  Camera<unsigned char> color;
  Camera<float> depth, ir;

  cam.InitImage(color);
  cam.InitDepthImage(depth);
  cam.InitIrImage(ir);

  CScommClient client;
  client.SetVerbose(true);

  client.RegisterMsg("img", BIAS::CS_BINARY, 512*424*2*sizeof(float));

  client.ConnectServer("main", 8090);

  float *data = new float[512*424*2];
  std::vector<char> binary;

  int i=0;
  while (i<100) {

    if (client.GetData("img", binary) == 0) {
      memcpy(data, binary.data(), 512*424*2*sizeof(float));
      std::cout << data[100] << " " << flush;
    }

    i++;

  }


  ImageIO::Save("kinect2-color.mip", color);
  ImageIO::Save("kinect2-depth.mip", depth);
  ImageIO::Save("kinect2-ir.mip", ir);
  
  return 0;
}
*/
int main(int argc, char *argv[]) {

  VideoSource_Kinect2Net cam;

  Camera<float> depth, ir;

  cam.InitDepthImage(depth);
  cam.InitIrImage(ir);

  cam.OpenDevice("main");
  cam.PreGrab();

  TimeMeasure tm;
  tm.Start();
  for (int i=0;i<100;i++) {
    cam.GrabSingleDepth(depth);
    cam.GrabSingleIR(ir);
    std::cout << depth.GetImageDataArray()[424/2][512/2] << " " << std::flush;
  }
  std::cout << std::endl;
  tm.Stop();
  tm.Print(std::cout);

  cam.PostGrab();

  ImageIO::Save("kinect2-depth.mip", depth);
  ImageIO::Save("kinect2-ir.mip", ir);

  return 0;
}
