/*
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003-2009    (see file CONTACT for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


/** @example ExampleVideoSource_Shm.cpp
    @relates VideoSource_Shm
    @ingroup g_examples
    @ingroup g_videosource
    @brief  Example for a videosource getting images over shared memory
    @author MIP
*/
#include <Base/Common/W32Compat.hh> // sprintf wrn
#include <VideoSource/VideoSource_Shm.hh>
#include <Base/Image/ImageIO.hh>
#include <Image/Camera.hh>

using namespace BIAS;
using namespace std;

bool done = false;
bool save = false;
int imgNo = 0;


int main( int argc, char *argv[])
{
  VideoSource *cam;
  Camera<unsigned char> pic;
  cam = new VideoSource_Shm;
  cam->SetDebugLevel(D_NET);
  // char c;
  int res;
  if (argc>1) {
	  cout <<"Using shared mem file: "<<argv[1]<<endl;
	  res = cam->OpenDevice(argv[1]);
  }
  else res = cam->OpenDevice();
  if (res <0) exit(1);
  res = cam->PreGrab();
  if (res <0) exit(1);
  cam->InitImage(pic);
 
  while(!done){
    cam->GrabSingle(pic);
    cout <<"Got Image: "<<pic.GetWidth()<<"x"<<pic.GetHeight()<<"x"<<
      pic.GetChannelCount()<<endl;

      char name[255];
      sprintf(name,"image_%04i.mip",imgNo);
      BIAS::ImageIO::Save(name,pic,ImageIO::FF_ppm);
    imgNo++;
    }
  

  cam->PostGrab();
  cam->CloseDevice();

  delete cam;
  return 0;
}

 
