/*
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003-2009    (see file CONTACT for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


/** @example Scan1394Bus.cpp
    @relates MDcam
    @ingroup g_examples
    @ingroup g_videosource
    @brief  Example scanning the IEEE1394 (firewire) bus and looking for cameras
    @author MIP
*/
//#include <unistd.h>

#include <getopt.h>

#include <VideoSource/VideoSource_MDcam.hh>
#include <iostream>
#include <fstream>

using namespace std;
using namespace BIAS;

void usage() 
{
  cout <<"Usage: Scan1394Bus [options] filename"<<endl;
  cout <<"Options: "<<endl;
  cout <<"  -a    : append to file instead of overwrite"<<endl;
  cout <<" -b N : scan bus number N instead of 0"<<endl;
}


int main(int argc, char *argv[])
{
  ofstream ofs;
  char *filename;
  bool append = false;
  //int bus = 0; // Set but not used.
  char c;

  VideoSource_MDcam cams;
  

  while (argc >1)
  {
      c = getopt(argc,argv,"ab:");
      if (c == -1) break;
      switch (c)
        {
	case 'a': 
	  append = true;
	  break;
	case 'b':
          //bus = atoi(optarg); // Set but not used.
	  break;
	default:
	  usage();
	  exit(1);
	}
  }
  
  
  if ( (argc -optind)  >0)
    filename = argv[optind];
  else {
    usage();
    exit(1);
  }
 

  if (append)
    ofs.open(filename,ios::app);
  else
    ofs.open(filename);
  
  if ( !ofs ) {
    perror(filename);
    exit(1);
  }
 

  cams.SetDebugLevel(0);
  
  cams.ScanBus(ofs);
  
  ofs.close();
  

  return 0;
}
