/*
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003-2009    (see file CONTACT for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/



#ifndef FLIRTHERMALCAMERASERIALCONTROL_HH_
#define FLIRTHERMALCAMERASERIALCONTROL_HH_

#include <bias_config.h>
// BIAS
#include <Utils/SerialPortIO.hh>

namespace BIAS{

enum BIASVideoSource_EXPORT FLIR_STATUS_MESSAGE{
	CAM_GENERAL_ERROR=-1,
	CAM_OK=0,
	CAM_BUSY,
	CAM_NOT_READY,
	CAM_RANGE_ERROR,
	CAM_CHECKSUM_ERROR,
	CAM_UNDEFINED_PROCESS_ERROR,
	CAM_UNDEFINED_FUNCTION_ERROR,
	CAM_TIMEOUT_ERROR,
	CAM_BYTE_COUNT_ERROR,
	CAM_FEATURE_ERROR
};

/**
* \class FlirThermalCameraSerialControl
*	\brief this class is used for serial communication with the FLIR Tau 320 Thermal camera
*	\brief Use the Set functions to set values to the camera such as FCCMode, Color etc.
* \date Jan 12, 2011
* \author ischiller
*/
class BIASVideoSource_EXPORT FlirThermalCameraSerialControl {
	public:
		FlirThermalCameraSerialControl();
		virtual ~FlirThermalCameraSerialControl();

		/** \brief connect over serial port */
		FLIR_STATUS_MESSAGE ConnectCamera(std::string port);
		/** \brief disconnnect camera control*/
		FLIR_STATUS_MESSAGE DisconnectCamera();
		/** \brief check whether camera control is connected */
		bool IsConnected(){return connected_;};

		/** \brief set default camera parameters */
		FLIR_STATUS_MESSAGE SetDefaults();

		/** \brief allowed values are 0,1,4,8
		 * 0=real time
		 * 1 = freeze frame
		 * 4 = 2x zoom
		 * 8 = 4x zoom */
		FLIR_STATUS_MESSAGE SetVideoMode(unsigned level);

		/** \brief Set the flat field correction
		 * allowed values 0,1,2 */
		FLIR_STATUS_MESSAGE SetFFCMode(unsigned mode);
    /** \brief Set the flat field correction interval in frames*/
		FLIR_STATUS_MESSAGE SetFFCInterval(unsigned interval, unsigned intervalLowGain);
    /** \brief Set the flat field correction max temp change*/
		FLIR_STATUS_MESSAGE SetFFCTempChange(unsigned tmpChange,unsigned tmpChangeLowGain);
   
    /** \brief Set the algorithm for Automatic Gain Control*/
		FLIR_STATUS_MESSAGE SetAGCAlgorithm(unsigned algo);

		/** \brief set different color shemes
		 * allowed range 0,...,12 */
		FLIR_STATUS_MESSAGE SetColoring(unsigned color);

		/** \brief set orientation of video
		* allowed range 0,...,4
		* 0 = normal
		* 1 = Invert
		* 2 = Revert
		* 3 = Invert and Revert*/
		FLIR_STATUS_MESSAGE SetVideoOrientation(unsigned orientation);

		/** \brief set the video standard
		 * 0=NTSC
		 * 1=PAL*/
		FLIR_STATUS_MESSAGE SetVideoStandard(unsigned std);

		FLIR_STATUS_MESSAGE SetOptimization(unsigned optimization);

		FLIR_STATUS_MESSAGE SetContrast(unsigned contrast);
		FLIR_STATUS_MESSAGE SetBrightness(unsigned brigthness);

    /**\brief sets the gain mode */
    FLIR_STATUS_MESSAGE SetGainMode(unsigned mode);
      /**\brief sets the gain mode */
    FLIR_STATUS_MESSAGE SetExternalSync(unsigned sync);

		/** \brief */
		FLIR_STATUS_MESSAGE SetSpotMeterMode(unsigned spotmeter);
		/** \brief */
		FLIR_STATUS_MESSAGE SetSpotMeterDisplay(unsigned spotmeter);
		/** \brief */
		FLIR_STATUS_MESSAGE GetSpotMeterCelcius(double& degree);
		FLIR_STATUS_MESSAGE GetSensorTemp(double& degree);

		FLIR_STATUS_MESSAGE GetSerialNumber(std::string &serial);
		std::string StatusCodeToString(FLIR_STATUS_MESSAGE code);

	protected:
		FLIR_STATUS_MESSAGE CheckCameraReply_(unsigned char* buf);
		FLIR_STATUS_MESSAGE SendCommand_(unsigned char* buf, int length);
		FLIR_STATUS_MESSAGE GetAnswer_(unsigned char* buf, int length, bool wait=false);

	protected:
		BIAS::SerialPortIO serialIO_;
		bool connected_;
		std::string serialNumber_;
	};
}
#endif /* FLIRTHERMALCAMERASERIALCONTROL_HH_ */
