#ifndef __SAMPLEGRABERCALLBACK_HH__
#define __SAMPLEGRABERCALLBACK_HH__
#include <BIAS_DeclSpec.hh>

#ifdef BIAS_HAVE_DSHOW
#  include <qedit.h>
#endif

namespace BIAS {

  typedef void (*SampleGrabberCBF)(double time_sec);

  /** @brief sample grabber callback class for grabbing single frames only
      @author Marc Grauel 2008 (www.vision-n.de) */
  class BIASVideoSource_EXPORT SampleGrabberCallback 
#ifdef BIAS_HAVE_DSHOW
    : public ISampleGrabberCB 
#endif
  {
  public:
    SampleGrabberCallback();

    ~SampleGrabberCallback();

    /** @brief resets internal trigger
    call this before grabbing a frame or the internal trigger can't signal
    that the SampleCB callback has been called.*/
    void SampleTriggerArm();

    /** @brief waits until one frame ist grabbed.*/
    void SampleTriggerWait();

    /** @brief returns trigger state
        use this if you want to wait for the sample trigger on your own...*/
    bool SampleTriggered();

    void SetTargetBuffer(unsigned char *TargetBuffer, unsigned int size, 
        unsigned width, unsigned height);

    /** use NULL to disable usage of CB function */
    inline void SetCallbackFunction(SampleGrabberCBF function)
    { CallbackFunction_ = function; }

    inline bool HasCallbackFunction() const
    { return (CallbackFunction_!=0); }

#ifdef BIAS_HAVE_DSHOW
    STDMETHODIMP SampleCB(double SampleTime, IMediaSample *pSample);
    STDMETHODIMP BufferCB(double SampleTime, BYTE *pBuffer, long BufferLen);
    STDMETHODIMP QueryInterface(REFIID riid, void ** ppv);
    STDMETHODIMP_(ULONG) AddRef() { return 2; }
    STDMETHODIMP_(ULONG) Release() { return 1; }
#endif

  protected:
    bool SampleTrigger_;
    unsigned char *TargetBuffer_;
    unsigned int TargetSize_;

    unsigned Width_;
    unsigned Height_;

    SampleGrabberCBF CallbackFunction_;
  }; // class SampleGrabberCallBack

} //namespace




#endif
