/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003-2009    (see file CONTACT for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


/**
   @file TestPMDZess.cpp
   @brief Test VideoSource_PMDZess
   @relates VideoSource_PMDZess
   @ingroup g_tests
   @author MIP
*/


#include <VideoSource/VideoSource_PMDZess.hh>
#include <Base/Image/ImageIO.hh>

#include <sstream>

using namespace std;
using namespace BIAS;

int main(int argc, char** argv)
{
  VideoSource_PMDZess cam;

  cam.SetBitFile("C:/Programme/z2D3D/Bitfiles/AchtBitExt.rbt");

  if (cam.OpenDevice() < 0) {
    BIASERR("Error initializing camera!!!");
    abort();
  }
  float shutter = 10.0;
  if (cam.SetShutter(shutter) < 0) {
    BIASERR("Error setting shutter!!!");
    abort();
  }
  else
    cerr<<"SetShutter: "<<shutter<<endl;

  cam.SetShutter2D(30.0);
  cerr<<"SetShutter2D: 30.0"<<endl;

  shutter = cam.GetShutter();
  cerr<<"GetShutter:"<<shutter<<endl;
  shutter = cam.GetShutter2D();
  cerr<<"GetShutter2D:"<<shutter<<endl; 

  Camera<unsigned char> grey;
  Camera<float> depth, coeff;

  stringstream st("");

  int i=0;
  while (i<10) {
    i++;

    //if(cam.CaptureSinglePhase(grey)!=0){
    //  BIASERR("CaptureSinglePhase failed");
    //  return -1;
    //}

    st << "temp" << i << ".mip";
    if (cam.GrabSingle(grey) < 0) {
      BIASERR("Grabbing went wrong!");
      return -1;
    }
    if (cam.GrabSingleDepth(depth) < 0) {
      BIASERR("No depthimage available!!!");
      return -1;
    }
    if (cam.GrabSingleModCoeff(coeff) < 0) {
      BIASERR("No depthimage available!!!");
      return -1;
    }
    //ImageIO::Save(st.str(), grey);
    ImageIO::Save(st.str(), grey);
    st.str(string(""));
    st << "tempDepth" << i << ".mip";
    //ImageIO::Save(st.str(), depth);
    ImageIO::Save(st.str(), depth);
    st.str(string(""));
    st << "tempCoeff" << i << ".mip";
    //ImageIO::Save(st.str(), coeff);
    ImageIO::Save(st.str(), coeff);
    st.str(string(""));
  }

  return 0;
}
