/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003-2009    (see file CONTACT for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


/**
   @file TestVideoSource_FFmpeg.cpp
   @brief Test VideoSource_FFmpeg
   @relates VideoSource_FFmpeg
   @ingroup g_tests
   @author MIP
*/


#include <iostream>
#include <VideoSource/VideoSource_FFmpeg.hh>
#include <Base/Image/ImageBase.hh>
#include <Base/Image/ImageIO.hh>
#include <Base/Common/FileHandling.hh>

using namespace std;
using namespace BIAS;


int main(int argc, char* argv[])
{
  VideoSource_FFmpeg vid;

  Camera<unsigned char> img;
  string videoFile;
  if(argc < 2) videoFile = "test.avi";
  else videoFile = argv[1];
  cout << "Opening video: "<<videoFile<<endl;
  if (vid.OpenDevice(videoFile.c_str())!=0)
  {
    cout << "Error opening file:" << vid.GetError()<<endl;
    return -1;
  }
  cout << "Initializing image..." << endl;
  vid.InitImage(img);
  vid.PreGrab();
  

  int ret = 0;
  int i = 0;
  while (ret==0)
  {
    cout << "Getting frame " << i << "..." << endl;
    if ((ret = vid.GrabSingle(img)) && i % 1 == 0)
    {
      cout << "Writing frame "<<i<<" to disk..." << endl;    
      if(!ImageIO::Save("frame-"+BIAS::FileHandling::LeadingZeroString(i,5)+".mip", img))      
      	cout << "Error writing frame!" << endl;
    }
    i++;
  }
  

  cout << "Closing video..." << endl;
  vid.CloseDevice();
  
  return 0;
}
