#ifndef __VIDEOSOURCECPABILITIES_H__
#define __VIDEOSOURCECPABILITIES_H__

#include <Base/Image/ImageBase.hh>
#include <vector>
#include <utility>

namespace BIAS {

/**
  \class VideoSourceCapabilities
  \ingroup g_videosource
  \relates VideoSource, VideoSource_V4L, VideoSource_DSHOW
  \author MIP
  \brief Checks for VideoSource capabilities
 */
class BIASVideoSource_EXPORT VideoSourceCapabilities {
 public:
  VideoSourceCapabilities();
  ~VideoSourceCapabilities();

    enum VSType {
      TypeCamera, TypeTuner
    };

  struct ResolutionEntry {
    unsigned int width;
    unsigned int height;
    float fps;
    ImageBase::EColorModel colormodel;
    bool interlaced;
  };

  struct InputDescriptor {
    unsigned int number;
    VSType type;
    std::string name;
    bool NoSignal;
    bool NoColor;
    bool NoPower;
    bool Active;
    
  };


  void Reset();
    void AddMode(unsigned int width, unsigned int height, float fps,  
		 ImageBase::EColorModel colormodel, bool interlaced=false);

    bool HasMode(unsigned int width, unsigned int height, float fps,  
		 ImageBase::EColorModel colormodel,bool interlaced);

    std::vector<ResolutionEntry> GetAllResolutionsRef() const
    {return AllResolutions_;}

    void GetBestForEachResolution(std::vector<ResolutionEntry> &res);


    void AddInput(InputDescriptor id);


    void Show();

    void SetName(const std::string &name) {Name_ = name;};
    std::string GetName(){return Name_;}
    
 protected:
    std::string Name_;

  std::vector<ResolutionEntry> AllResolutions_;

  std::vector<InputDescriptor> Inputs_;

}; //class

} //namespace

#endif
