/*
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003-2009    (see file CONTACT for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __BIASVIDEOSOURCE_DCAM_BUMBLEBEE_HH__
#define __BIASVIDEOSOURCE_DCAM_BUMBLEBEE_HH__

#include <VideoSource/VideoSource_Base.hh>
#include <VideoSource/VideoSource_DCAM.hh>

namespace BIAS {

/**
  \class VideoSource_DCAM_BUMBLEBEE
  \ingroup g_videosource
  \author djung 08, fkellner 11
  \brief VideoSource PtGrey Bumblebee cameras  
 */
  class BIASVideoSource_EXPORT VideoSource_DCAM_BumbleBee : public VideoSource_DCAM{


  public:
    VideoSource_DCAM_BumbleBee(int Card = 0);
    
    /**
     * @brief Set camera to stereo/tri mode. Call this before OpenDevice(). Else only one camera image is grabbed
     * @author fkellner 
    */
    int SetStereoDefaultMode(CameraInfo *camInfo = NULL);
    
    /**
     * @brief Init image according to bumblebee type and stereo mode setting
     * @author fkellner
     */
    virtual int InitImage(BIAS::ImageBase &Image);

    /**
     * @brief Init single image according to bumblebee type (used for demultiplexing)
     * @author fkellner
     */
    virtual int InitSingleImage(BIAS::ImageBase &Image);

    /**
     * @brief Get number of images for this bumblebee (returns 2 for xb2, 3 for xb3)
     * @author fkellner 
    */
    virtual int GetNumImages();

    /**
      @brief Grab single images. Calls 2 image version if xb2 cam is used. 
      @author Daniel Jung 08, fkellner 11
    */
    virtual int GrabSingle(BIAS::Camera <unsigned char> &allImages,
                   BIAS::Camera <unsigned char> &imageLeft,
                   BIAS::Camera <unsigned char> &imageMiddle,
                   BIAS::Camera <unsigned char> &imageRight
                   );

    /**
      @brief Grab single images. Preferred method for xb2. XB2 ONLY!
      @author fkellner 
    **/
    virtual int GrabSingle(BIAS::Camera <unsigned char> &allImages,
                   BIAS::Camera <unsigned char> &imageLeft,
                   BIAS::Camera <unsigned char> &imageRight
                   );

    /**
      @brief Grab single for grey value images for left and right cam. XB3 ONLY! 
             !!!Images have to initialized with 640x480-gray!!!
      @author apetersen
    **/
    int GrabSingleLRGrey(BIAS::Camera <unsigned char> &allImages,
                         BIAS::Camera <unsigned char> &imageLeft,
                         BIAS::Camera <unsigned char> &imageRight);

    /**
      @brief XB3 ONLY! Grab single images as RGB. No interpolation on bayer pattern. 50% of green values are dropped. Left, middle and right images HAVE to be 640x480, 3 channels, interleaved, color model RGB and unsigned char or must be omitted (pass empty image)!
      @author Daniel Jung
      @date 08/2008
    **/
    virtual int GrabSingleAsSmallRGB(BIAS::Camera <unsigned char> &allImages,
        BIAS::Camera <unsigned char> &imageLeft,
        BIAS::Camera <unsigned char> &imageMiddle,
        BIAS::Camera <unsigned char> &imageRight);
    
  private:
    bool stereo_;
  };


} // namespace BIAS

#endif // __MIPVIDEOSOURCE_DCAM_BUMBLEBEE_HH__
