#ifndef __BIASVIDEOSOURCE_DV_HH__
#define __BIASVIDEOSOURCE_DV_HH__

#include <bias_config.h>
#ifndef BIAS_HAVE_DV
#  error Please recompile BIAS with DV to use VideoSource_DV
#endif // BUILD_VideoSource


#include "VideoSource_Base.hh"
#include <sys/time.h>
#include <libraw1394/raw1394.h>



#ifndef MAX_1394_CARDS
#define MAX_1394_CARDS 2
#endif

#ifndef MAX_1394_CHANNELS
#define MAX_1394_CHANNELS 100
#endif

// 300 packets/frame, each of 480 bytes
#define DV_PIC_SIZE 144000 

namespace BIAS{

/** 
    \class VideoSource_DV
    \ingroup g_videosource
    \author evers
    \brief This class  extends VideoSource for the use of IEEE1394-connected 
    DV-camcorders.
    
    It uses the old raw1394 interface to transfer packets without DMA.

*/
  class VideoSource_DV : public VideoSource{
  
 
  public: 
    VideoSource_DV(int Card);
    VideoSource_DV();
    ~VideoSource_DV();

    int OpenDevice();
    int CloseDevice();

    int OpenDevice(const char *device);

    int UseChannel(int channel);
  
    int GrabSingle(BIAS::Camera <unsigned char>  &image);
  
    int PreGrab();
  
    int PostGrab();
  


  protected:
    void VideoSource_DV_Internal_Init_(int card);
    static raw1394handle_t IEEE1394Handle_[MAX_1394_CARDS];
    static int Instances_[MAX_1394_CARDS];
    static int InstancesTotal_;
  

    // this is the card to serve for
    int IEEE1394Card_;
    int Channel_;
    int MaxTrials_;
    bool  FrameFinished_;
    bool  FrameStarted_;
    bool  DroppedFrame_;
 

    unsigned char *GrabBuffer_;
  

    friend int raw_dv_handler(raw1394handle_t handle, int channel, 
			      size_t length, quadlet_t *data);
  

  
  };
} // namespace BIAS

  /** This array is used to associate DV-sources identified via channel witch 
   * videosource-objects since raw_dv_handler() only knows the handle and 
   * channel, not the instance-object of videosource().
   * It would be better to use handle instead of channel, to avoid conflicts 
   * with cameras connected to different cards using the same channel (63)
   */
  extern BIAS::VideoSource_DV* videosourcefromchannel[MAX_1394_CHANNELS];



#endif
