/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003-2009    (see file CONTACT for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#ifndef __BIASVIDEOSOURCE_DCAMPMD_HH__
#define __BIASVIDEOSOURCE_DCAMPMD_HH__

#ifndef WIN32
#  error The PMD-camera can only be run under windows.
#endif

#include <pmdmsdk.h>
#include <VideoSource/VideoSource_Base.hh>
#include <Image/Camera.hh>
#include <bias_config.h>
#include <vector>
#include <Image/PMDImageProc.hh>

/* no longer in use -- timemeasure instead
#include <sys/time.h>
*/

namespace BIAS {

  /** @class VideoSource_DcamPMD
      @ingroup g_videosource
      @brief reads PMD-tiffs with depth images and modulation coefficients
      @author streckel 05/2006 
  */
  class BIASVideoSource_EXPORT VideoSource_DcamPMD : public VideoSource {

  public:
    VideoSource_DcamPMD();

    ~VideoSource_DcamPMD();

    virtual int OpenDevice();
    virtual int CloseDevice();

    virtual int PreGrab();
    virtual int PostGrab();

 
	/** Returns current framerate
	*/
    double GetFPS();

    virtual int InitImage(ImageBase &img);


    /** Returns the 2D image, and reads depth and modulation coefficients */
    virtual int GrabSingle(Camera<unsigned char> &image);

     /// Set shutter (exposure time) to exptime in seconds
    virtual int SetShutter(float exptime);
  
    /// Set gain to g with g[dB] 
    virtual int SetGain(float g);
  
    virtual float GetShutter();
    virtual float GetGain();

    /* Set the integration time in us */
    int SetIntegrationTime(int dIntegrationTime);  
    /* Get the integration time in us */
    int GetIntegrationTime();
    /* Set the Modulation frequency*/
    int SetModulationFrequency(int dModulationFrequency);
    /* Get the Modulation frequency*/
    int GetModulationFrequency();

    /** Returns the depth image. Must be called after GrabSingle() */
    inline int GrabSingleDepth(Camera<float> &image)
    { 
      if (!DepthImg_.IsEmpty()) {
        image = DepthImg_; 
        return 0;
      } 
      return -1;
    }

    /** Returns the amplitude image. Must be called after GrabSingle() */
    inline int GrabSingleModCoeff(Camera<float> &image)
    {       
      if (!ModCoeffImg_.IsEmpty()) {
        image = ModCoeffImg_; 
        return 0;
      } 
      return -1;
    }

    void SetGenerateNewUIDs(bool s) {  
      BIASERR("Always generating new UUIDS for PMD images");
      BIASABORT;
    }

  protected:
    PMDHandle handle_;

    //BIAS::Camera<unsigned char> 2DImg_;
    BIAS::Camera<float> DepthImg_, ModCoeffImg_;
    PMDPixelOrigin pixori_;

    void ImageUcharFromDouble_(double* data, Image<unsigned char> &dst);
    void ImageFloatFromDouble_(double* data, Image<float> &dst);

  };

} // namespace BIAS

#endif
