/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003-2009    (see file CONTACT for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#ifndef __BIASVIDEOSOURCE_DISKZESSPMD_HH__
#define __BIASVIDEOSOURCE_DISKZESSPMD_HH__

#include <VideoSource/VideoSource_Disk.hh>
#include <Image/Camera.hh>
#include <bias_config.h>
#include <vector>


namespace BIAS {

  /** @class VideoSource_DiskZessPMD
      @ingroup g_videosource
      @brief reads images from three image lists with 2d images, depth images and modulation coefficients
      @author kellner 10/2007 */
  class BIASVideoSource_EXPORT VideoSource_DiskZessPMD : public VideoSource_Disk {

  public:
    VideoSource_DiskZessPMD();

    virtual ~VideoSource_DiskZessPMD();

    /** The filenames vector should contain the names of PMD-TIFFS
    */
    virtual int OpenDevice(const std::vector<std::string> &FileNames);

    virtual int OpenDevice(); 

    /** Returns the 2D image, and reads depth and modulation coefficients */
    virtual int GrabSingle(Camera<unsigned char> &image);

    /** Returns the depth image. Must be called after GrabSingle() */
    inline int GrabSingleDepth(Camera<float> &image)
    { 
      if (!DepthImg_.IsEmpty()) {
        image = DepthImg_; 
        return 0;
      } 
      return -1;
    }

    /** Returns the depth image. Must be called after GrabSingle() */
    inline int GrabSingleModCoeff(Camera<float> &image)
    {       
      if (!ModCoeffImg_.IsEmpty()) {
        image = ModCoeffImg_; 
        return 0;
      } 
      return -1;
    }

    void SetGenerateNewUIDs(bool s) {  
      BIASERR("Always generating new UUIDS for PMD images");
      BIASABORT;
    }

  protected:
    VideoSource_Disk *vsd_;
    BIAS::Camera<float> DepthImg_, ModCoeffImg_;
  };

} // namespace BIAS

#endif
