/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003-2009    (see file CONTACT for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef _VideoSource_FFMPEGVideo_H_
#define _VideoSource_FFMPEGVideo_H_

#include <string>
#include <vector>
extern "C"
{
  #include <avcodec.h>
  #include <avformat.h>
  #include <avutil.h>
  #include <swscale.h>
}
#include <Base/Image/ImageBase.hh>

namespace BIAS
{
  /** \class VideoSource_FFMPEGVideo, formerly FFMPEGVideo
   *  \test tested with TestVideoSource_FFMPEGVideo.cpp
   *  \ingroup g_videosource  
   *  \author Max hollmann
   *  \brief Class for extracting frames from video files using FFMpeg, analog to class DShowVideo 
   *  \deprecated replaced by VideoSource_FFmpeg, to be removed (ischiller)
   */
  class VideoSource_FFMPEGVideo
  {
  public:
    VideoSource_FFMPEGVideo();
    ~VideoSource_FFMPEGVideo();

    /** Open file and retreive info. */
    bool Open(const char *file);

    /** Select video stream, initialize stuff and set destination size
     *  and pixel format. Only call after successfully calling Open().
     *  If width and height both are zero, the video is not resized.
     *  If either is zero, the other is calculated to maintain the aspect ratio.
     */
    bool Init(int width = 0, int height = 0,
	      PixelFormat pixFmt = PIX_FMT_RGB24, unsigned int streamIndex = 0);

    /** Close file and free buffers. */
    void Close();


    /** Return number of video streams. */
    unsigned int CountVideoStreams();
    

    /** Return width of selected video stream. */
    int Width();

    /** Return height of selected video stream. */
    int Height();


    /** Retreive the next frame from the video. */
    bool GetFrame(BIAS::ImageBase &img);

    /** Seeks to \p time which is given in TimeBase units. */
    void Seek(int64_t time);

    /** Returns the TimeBase of the video stream. */
    AVRational TimeBase();

    /** Returns the current decoding timestamp. */
    unsigned int GetCurrentDTS() const;

    /** Returns the length of the video in TimeBase units. */
    unsigned int GetLength() const;


    /** Returns last error message. */
    std::string ErrMsg();

    /** Maps BIAS colormodels to ffmpeg pixelformats.
     *  Supported so far:
     *  - CM_RGB
     *  - CM_BGR
     *  - CM_GRAY
     */
    static PixelFormat Colormodel2PixFmt(ImageBase::EColorModel cm);

  protected:
    std::string _errMsg;

    std::vector<int> _vidStreams;
    unsigned int _streamIndex;
    AVFormatContext *_fmtCtx;
    AVCodecContext *_codecCtx;
    SwsContext *_swsCtx;
    AVCodec *_codec;
    int _vidStr;
    AVFrame *_frame;
    AVFrame *_frameRGB;
    uint8_t *_buffer;
  };
}

#endif
