#include "VideoSource_Kinect2.hh"
#include <iostream>
#include <Base/Image/ImageIO.hh>

#include <libfreenect2/frame_listener.hpp>
#include <libfreenect2/packet_pipeline.h>
#include <libfreenect2/async_packet_processor.h>
#include <libfreenect2/response.h>
#include <libfreenect2/resource.h>
#include <iostream>
#include <OpenCLFramework/Algorithm/clfAlgorithm.hh>

#ifdef FREENECT_HAVE_TURBOJPEG
#include <turbojpeg.h>
#else
#ifdef BIAS_HAVE_LIBJPEG
#include <jpeglib.h>
#else
// ImageMagick support
#ifdef BIAS_HAVE_IMAGEMAGICKLIB
#  include <Base/Common/BIASpragmaStart.hh>
#  include <Magick++.h>
#  include <Base/Common/BIASpragmaEnd.hh>
#  if (MagickLibVersion >= 0x650)
#    define MagickLib MagickCore
#  endif
#endif
#endif
#endif

using namespace std;
using namespace BIAS;
using namespace libfreenect2;

///////////////////////FREENECT INTERFACES/////////////////////////////////

#ifdef FREENECT_HAVE_TURBOJPEG
class FreenectJpegRgbPacketProcessor : public RgbPacketProcessor
{
public:
  FreenectJpegRgbPacketProcessor() {
    frame_ = new Frame(1920,1080,tjPixelSize[TJPF_RGB]);
    flippedframe_ = new Frame(1920,1080,tjPixelSize[TJPF_RGB]);
    decompressor_ = tjInitDecompress();
    if(decompressor_ == 0)
    {
      BIASERR("[TurboJpegRgbPacketProcessor] Failed to initialize TurboJPEG decompressor! TurboJPEG error: '" << tjGetErrorStr() << "'");
    }
  }
  virtual ~FreenectJpegRgbPacketProcessor() {
    delete frame_;
    delete flippedframe_;
    if (decompressor_ != 0) {
      tjDestroy(decompressor_);
    }
  }
protected:
  virtual void process(const libfreenect2::RgbPacket &packet) {
    if (listener_ != 0 && decompressor_ != 0) {
      tjDecompress2(decompressor_, packet.jpeg_buffer, packet.jpeg_buffer_length, flippedframe_->data, 1920, 1920 * tjPixelSize[TJPF_RGB], 1080, TJPF_RGB, 0);
      unsigned char *out = frame_->data;
      unsigned char *in = flippedframe_->data;
      unsigned int stride = 1920*3;
      register int istart = 0;
      register int ostart = 0;
      for (unsigned int y=0;y<1080;y++) {
        istart = y*stride;
        ostart = y*stride+1919*3;
        for (unsigned int x=0;x<1920;x++) {
          memcpy( out+ostart, in+istart, 3);
          istart += 3;
          ostart -= 3;
        }
      }
      if(listener_->onNewFrame(Frame::Color, frame_))
      {
        // yay new frame
      }
    }
  }
private:
  Frame *frame_;
  Frame *flippedframe_;
  tjhandle decompressor_;
};
#else
#ifdef BIAS_HAVE_LIBJPEG
class FreenectJpegRgbPacketProcessor : public RgbPacketProcessor
{
public:
  FreenectJpegRgbPacketProcessor() {
    frame_ = new Frame(1920,1080,3);
    flippedframe_ = new Frame(1920,1080,3);
    jpeg_create_decompress(&cinfo_);
    cinfo_.err = jpeg_std_error(&err_);
  }
  virtual ~FreenectJpegRgbPacketProcessor() {
    delete frame_;
    delete flippedframe_;
    jpeg_destroy_decompress(&cinfo_);
  }
protected:
  virtual void process(const libfreenect2::RgbPacket &packet) {
    if (listener_ != 0) {
      jpeg_mem_src(&cinfo_, packet.jpeg_buffer, packet.jpeg_buffer_length);
      jpeg_read_header(&cinfo_, true);
      jpeg_start_decompress(&cinfo_);
      int row_stride = cinfo_.output_width * cinfo_.output_components;
      unsigned char *data = flippedframe_->data;
      while (cinfo_.output_scanline < cinfo_.output_height) {
        jpeg_read_scanlines(&cinfo_, &data, 1);
        data+=row_stride;
      }
      jpeg_finish_decompress(&cinfo_);
      unsigned char *out = frame_->data;
      unsigned char *in = flippedframe_->data;
      unsigned int stride = 1920*3;
      register int istart = 0;
      register int ostart = 0;
      for (unsigned int y=0;y<1080;y++) {
        istart = y*stride;
        ostart = y*stride+1919*3;
        for (unsigned int x=0;x<1920;x++) {
          memcpy( out+ostart, in+istart, 3);
          istart += 3;
          ostart -= 3;
        }
      }
      if(listener_->onNewFrame(Frame::Color, frame_))
      {
        // yay new frame
      }
    }
  }
private:
  Frame *frame_;
  Frame *flippedframe_;
  jpeg_decompress_struct cinfo_;
  jpeg_error_mgr err_;
};
#else
class FreenectJpegRgbPacketProcessor : public RgbPacketProcessor
{
public:
  FreenectJpegRgbPacketProcessor() {
    frame_ = new Frame(1920,1080,3);
    flippedframe_ = new Frame(1920,1080,3);
  }
  virtual ~FreenectJpegRgbPacketProcessor() {
    delete frame_;
    delete flippedframe_;
  }
protected:
  virtual void process(const libfreenect2::RgbPacket &packet) {
    if (listener_ != 0) {
#ifdef BIAS_HAVE_IMAGEMAGICKLIB
      Magick::Image image;
      Magick::Blob data(packet.jpeg_buffer, packet.jpeg_buffer_length);
      Magick::Geometry size(1920,1080);
      image.read( data, size, 1, "JPEG" );
      image.write(0, 0, 1920, 1080, "RGB", MagickLib::CharPixel, flippedframe_->data);
      unsigned char *out = frame_->data;
      unsigned char *in = flippedframe_->data;
      unsigned int stride = 1920*3;
      register int istart = 0;
      register int ostart = 0;
      for (unsigned int y=0;y<1080;y++) {
        istart = y*stride;
        ostart = y*stride+1919*3;
        for (unsigned int x=0;x<1920;x++) {
          memcpy( out+ostart, in+istart, 3);
          istart += 3;
          ostart -= 3;
        }
      }
#else
      BIASERR("could not find any suitable lib to decompress jpeg!");
#endif
      if(listener_->onNewFrame(Frame::Color, frame_))
      {
        //std::cout << "YAY" << std::endl;
      }
    }
  }
private:
  Frame *frame_;
  Frame *flippedframe_;
};
#endif
#endif

class FreenectDepthPacketProcessor : public DepthPacketProcessor, public BIAS::clfAlgorithm
{
public:
  FreenectDepthPacketProcessor(BIAS::clfContext *context);
  virtual ~FreenectDepthPacketProcessor();
  virtual void setConfiguration(const libfreenect2::DepthPacketProcessor::Config &config);

  virtual void loadP0TablesFromCommandResponse(unsigned char* buffer, size_t buffer_length);

  void loadXTableFromFile(const std::string &filename);
  void loadZTableFromFile(const std::string &filename);
  void load11To16LutFromFile(const std::string &filename);

  virtual void process(const DepthPacket &packet);

protected:
  void initialize_();
  void generateOptions_(std::string &options);
  bool loadBufferFromResources_(const std::string &filename, unsigned char *buffer, const size_t n);
private:
  Frame *ir_frame_;
  Frame *depth_frame_;

  cl_short lut11to16[2048];
  cl_float x_table[512 * 424];
  cl_float z_table[512 * 424];
  cl_float3 p0_table[512 * 424];
  libfreenect2::DepthPacketProcessor::Config config_;
  DepthPacketProcessor::Parameters params_;
  size_t image_size;
  bool initialized_;

private: // opencl
  // Read only buffers
  BIAS::clfBuffer *buf_lut11to16;
  BIAS::clfBuffer *buf_p0_table;
  BIAS::clfBuffer *buf_x_table;
  BIAS::clfBuffer *buf_z_table;
  BIAS::clfBuffer *buf_packet;

  // Read-Write buffers
  BIAS::clfBuffer *buf_a;
  BIAS::clfBuffer *buf_b;
  BIAS::clfBuffer *buf_n;
  BIAS::clfBuffer *buf_ir;
  BIAS::clfBuffer *buf_a_filtered;
  BIAS::clfBuffer *buf_b_filtered;
  BIAS::clfBuffer *buf_edge_test;
  BIAS::clfBuffer *buf_depth;
  BIAS::clfBuffer *buf_ir_sum;
  BIAS::clfBuffer *buf_filtered;
};

void FreenectDepthPacketProcessor::generateOptions_(std::string &options) {
    std::ostringstream oss;
    oss.precision(16);
    oss << std::scientific;

    oss << " -D BFI_BITMASK=" << "0x180";

    oss << " -D AB_MULTIPLIER=" << params_.ab_multiplier << "f";
    oss << " -D AB_MULTIPLIER_PER_FRQ0=" << params_.ab_multiplier_per_frq[0] << "f";
    oss << " -D AB_MULTIPLIER_PER_FRQ1=" << params_.ab_multiplier_per_frq[1] << "f";
    oss << " -D AB_MULTIPLIER_PER_FRQ2=" << params_.ab_multiplier_per_frq[2] << "f";
    oss << " -D AB_OUTPUT_MULTIPLIER=" << params_.ab_output_multiplier << "f";

    oss << " -D PHASE_IN_RAD0=" << params_.phase_in_rad[0] << "f";
    oss << " -D PHASE_IN_RAD1=" << params_.phase_in_rad[1] << "f";
    oss << " -D PHASE_IN_RAD2=" << params_.phase_in_rad[2] << "f";

    oss << " -D JOINT_BILATERAL_AB_THRESHOLD=" << params_.joint_bilateral_ab_threshold << "f";
    oss << " -D JOINT_BILATERAL_MAX_EDGE=" << params_.joint_bilateral_max_edge << "f";
    oss << " -D JOINT_BILATERAL_EXP=" << params_.joint_bilateral_exp << "f";
    oss << " -D JOINT_BILATERAL_THRESHOLD=" << (params_.joint_bilateral_ab_threshold * params_.joint_bilateral_ab_threshold) / (params_.ab_multiplier * params_.ab_multiplier) << "f";
    oss << " -D GAUSSIAN_KERNEL_0=" << params_.gaussian_kernel[0] << "f";
    oss << " -D GAUSSIAN_KERNEL_1=" << params_.gaussian_kernel[1] << "f";
    oss << " -D GAUSSIAN_KERNEL_2=" << params_.gaussian_kernel[2] << "f";
    oss << " -D GAUSSIAN_KERNEL_3=" << params_.gaussian_kernel[3] << "f";
    oss << " -D GAUSSIAN_KERNEL_4=" << params_.gaussian_kernel[4] << "f";
    oss << " -D GAUSSIAN_KERNEL_5=" << params_.gaussian_kernel[5] << "f";
    oss << " -D GAUSSIAN_KERNEL_6=" << params_.gaussian_kernel[6] << "f";
    oss << " -D GAUSSIAN_KERNEL_7=" << params_.gaussian_kernel[7] << "f";
    oss << " -D GAUSSIAN_KERNEL_8=" << params_.gaussian_kernel[8] << "f";

    oss << " -D PHASE_OFFSET=" << params_.phase_offset << "f";
    oss << " -D UNAMBIGIOUS_DIST=" << params_.unambigious_dist << "f";
    oss << " -D INDIVIDUAL_AB_THRESHOLD=" << params_.individual_ab_threshold << "f";
    oss << " -D AB_THRESHOLD=" << params_.ab_threshold << "f";
    oss << " -D AB_CONFIDENCE_SLOPE=" << params_.ab_confidence_slope << "f";
    oss << " -D AB_CONFIDENCE_OFFSET=" << params_.ab_confidence_offset << "f";
    oss << " -D MIN_DEALIAS_CONFIDENCE=" << params_.min_dealias_confidence << "f";
    oss << " -D MAX_DEALIAS_CONFIDENCE=" << params_.max_dealias_confidence << "f";

    oss << " -D EDGE_AB_AVG_MIN_VALUE=" << params_.edge_ab_avg_min_value << "f";
    oss << " -D EDGE_AB_STD_DEV_THRESHOLD=" << params_.edge_ab_std_dev_threshold << "f";
    oss << " -D EDGE_CLOSE_DELTA_THRESHOLD=" << params_.edge_close_delta_threshold << "f";
    oss << " -D EDGE_FAR_DELTA_THRESHOLD=" << params_.edge_far_delta_threshold << "f";
    oss << " -D EDGE_MAX_DELTA_THRESHOLD=" << params_.edge_max_delta_threshold << "f";
    oss << " -D EDGE_AVG_DELTA_THRESHOLD=" << params_.edge_avg_delta_threshold << "f";
    oss << " -D MAX_EDGE_COUNT=" << params_.max_edge_count << "f";

    oss << " -D MIN_DEPTH=" << config_.MinDepth * 1000.0f << "f";
    oss << " -D MAX_DEPTH=" << config_.MaxDepth * 1000.0f << "f";
    options = oss.str();
}

BIAS::clfBuffer* createbuffer( BIAS::clfContext *ctx, bool readonly, bool writeonly, size_t size) {
  BIAS::clfBuffer *ret = ctx->CreateBuffer();
  ret->SetCached(true);
  ret->Allocate(size, readonly, writeonly);
  return ret;
}


void FreenectDepthPacketProcessor::initialize_() {
  if (initialized_) return;
  std::string options;
  generateOptions_(options);
  programCL_->AddSource("/OpenCLFramework/cl/opencl_freenect2.cl");
  programCL_->Build(0, options);
  programCL_->AddKernel("processPixelStage1");
  programCL_->AddKernel("filterPixelStage1");
  programCL_->AddKernel("processPixelStage2");
  programCL_->AddKernel("filterPixelStage2");
  programCL_->AddKernel("flipImage");
  // read only

  buf_lut11to16 = createbuffer( context_, true, false, 2048 * sizeof(cl_short));
  buf_p0_table = createbuffer( context_, true, false, image_size * sizeof(cl_float3));
  buf_x_table = createbuffer( context_, true, false, image_size * sizeof(cl_float));
  buf_z_table = createbuffer( context_, true, false, image_size * sizeof(cl_float));
  buf_packet = createbuffer( context_, true, false, ((image_size * 11) / 16) * 10 * sizeof(cl_ushort));

  // read write
  buf_a = createbuffer( context_, false, false, image_size * sizeof(cl_float3));
  buf_b = createbuffer( context_, false, false, image_size * sizeof(cl_float3));
  buf_n = createbuffer( context_, false, false, image_size * sizeof(cl_float3));
  buf_ir = createbuffer( context_, false, false, image_size * sizeof(cl_float));
  buf_a_filtered = createbuffer( context_, false, false, image_size * sizeof(cl_float3));
  buf_b_filtered = createbuffer( context_, false, false, image_size * sizeof(cl_float3));
  buf_edge_test = createbuffer( context_, false, false, image_size * sizeof(cl_uchar));
  buf_depth = createbuffer( context_, false, false, image_size * sizeof(cl_float));
  buf_ir_sum = createbuffer( context_, false, false, image_size * sizeof(cl_float));
  buf_filtered = createbuffer( context_, false, false, image_size * sizeof(cl_float));

  programCL_->KernelSetArgument("processPixelStage1", 0, *buf_lut11to16);
  programCL_->KernelSetArgument("processPixelStage1", 1, *buf_z_table);
  programCL_->KernelSetArgument("processPixelStage1", 2, *buf_p0_table);
  programCL_->KernelSetArgument("processPixelStage1", 3, *buf_packet);
  programCL_->KernelSetArgument("processPixelStage1", 4, *buf_a);
  programCL_->KernelSetArgument("processPixelStage1", 5, *buf_b);
  programCL_->KernelSetArgument("processPixelStage1", 6, *buf_n);
  programCL_->KernelSetArgument("processPixelStage1", 7, *buf_ir);

  programCL_->KernelSetArgument("filterPixelStage1", 0, *buf_a);
  programCL_->KernelSetArgument("filterPixelStage1", 1, *buf_b);
  programCL_->KernelSetArgument("filterPixelStage1", 2, *buf_n);
  programCL_->KernelSetArgument("filterPixelStage1", 3, *buf_a_filtered);
  programCL_->KernelSetArgument("filterPixelStage1", 4, *buf_b_filtered);
  programCL_->KernelSetArgument("filterPixelStage1", 5, *buf_edge_test);

  if (config_.EnableBilateralFilter) {
    programCL_->KernelSetArgument("processPixelStage2", 0, *buf_a_filtered);
    programCL_->KernelSetArgument("processPixelStage2", 1, *buf_b_filtered);
  } else {
    programCL_->KernelSetArgument("processPixelStage2", 0, *buf_a);
    programCL_->KernelSetArgument("processPixelStage2", 1, *buf_b);
  }
  programCL_->KernelSetArgument("processPixelStage2", 2, *buf_x_table);
  programCL_->KernelSetArgument("processPixelStage2", 3, *buf_z_table);
  programCL_->KernelSetArgument("processPixelStage2", 4, *buf_depth);
  programCL_->KernelSetArgument("processPixelStage2", 5, *buf_ir_sum);

  programCL_->KernelSetArgument("filterPixelStage2", 0, *buf_depth);
  programCL_->KernelSetArgument("filterPixelStage2", 1, *buf_ir_sum);
  programCL_->KernelSetArgument("filterPixelStage2", 2, *buf_edge_test);
  programCL_->KernelSetArgument("filterPixelStage2", 3, *buf_filtered);

  // reuse buffer for flipping
  programCL_->KernelSetArgument("flipImage", 1, *buf_ir_sum);

  buf_lut11to16->WriteToBuffer( lut11to16 );
  buf_p0_table->WriteToBuffer( p0_table );
  buf_x_table->WriteToBuffer( x_table );
  buf_z_table->WriteToBuffer( z_table );

  initialized_ = true;
}

FreenectDepthPacketProcessor::FreenectDepthPacketProcessor(BIAS::clfContext *context) : BIAS::clfAlgorithm(context, true) {
  image_size = 512 * 424;
  load11To16LutFromFile("11to16.bin");
  loadXTableFromFile("xTable.bin");
  loadZTableFromFile("zTable.bin");
  ir_frame_ = new Frame(526,424,4);
  depth_frame_ = new Frame(526,424,4);
  initialized_ = false;
  config_.MaxDepth = 9.0f;
}

#define OUT_NAME(FUNCTION) "[OpenCLDepthPacketProcessor::" FUNCTION "] "

FreenectDepthPacketProcessor::~FreenectDepthPacketProcessor() {}
void FreenectDepthPacketProcessor::setConfiguration(const libfreenect2::DepthPacketProcessor::Config &config) {
  DepthPacketProcessor::setConfiguration(config);
  config_ = config;
}

void FreenectDepthPacketProcessor::loadP0TablesFromCommandResponse(unsigned char* buffer, size_t buffer_length) {
 libfreenect2::protocol::P0TablesResponse *p0table = (libfreenect2::protocol::P0TablesResponse *)buffer;

  if(buffer_length < sizeof(libfreenect2::protocol::P0TablesResponse))
  {
    std::cerr << OUT_NAME("loadP0TablesFromCommandResponse") "P0Table response too short!" << std::endl;
    return;
  }
  for(int r = 0; r < 424; ++r)
  {
    cl_float3 *it = &p0_table[r * 512];
    const uint16_t *it0 = &p0table->p0table0[r * 512];
    const uint16_t *it1 = &p0table->p0table1[r * 512];
    const uint16_t *it2 = &p0table->p0table2[r * 512];
    for(int c = 0; c < 512; ++c, ++it, ++it0, ++it1, ++it2)
    {
      it->s[0] = -((float) * it0) * 0.000031 * M_PI;
      it->s[1] = -((float) * it1) * 0.000031 * M_PI;
      it->s[2] = -((float) * it2) * 0.000031 * M_PI;
      it->s[3] = 0.0f;
    }
  }
}



void FreenectDepthPacketProcessor::loadXTableFromFile(const std::string &filename) {
  if(!loadBufferFromResources_(filename, (unsigned char *)x_table, image_size * sizeof(float)))
  {
    std::cerr << OUT_NAME("loadXTableFromFile") "could not load x table from: " << filename << std::endl;
  }
}
void FreenectDepthPacketProcessor::loadZTableFromFile(const std::string &filename) {
  if(!loadBufferFromResources_(filename, (unsigned char *)z_table, image_size * sizeof(float)))
  {
    std::cerr << OUT_NAME("loadZTableFromFile") "could not load z table from: " << filename << std::endl;
  }
}
void FreenectDepthPacketProcessor::load11To16LutFromFile(const std::string &filename) {
  if(!loadBufferFromResources_(filename, (unsigned char *)lut11to16, 2048 * sizeof(cl_ushort)))
  {
    std::cerr << OUT_NAME("load11To16LutFromFile") "could not load lut table from: " << filename << std::endl;
  }
}

void FreenectDepthPacketProcessor::process(const DepthPacket &packet) {
  bool has_listener = this->listener_ != 0;

  initialize_();

  buf_packet->WriteToBuffer( packet.buffer );
  context_->RunOn1DRange(*programCL_, "processPixelStage1", image_size, 0);
  if (config_.EnableBilateralFilter) {
    context_->RunOn1DRange(*programCL_, "filterPixelStage1", image_size, 0);
  }
  context_->RunOn1DRange(*programCL_, "processPixelStage2", image_size, 0);
  if (config_.EnableEdgeAwareFilter) {
    context_->RunOn1DRange(*programCL_, "filterPixelStage2", image_size, 0);
    programCL_->KernelSetArgument("flipImage", 0, *buf_filtered);
  } else {
    programCL_->KernelSetArgument("flipImage", 0, *buf_depth);
  }
  context_->RunOn1DRange(*programCL_, "flipImage", image_size, 0);
  buf_ir_sum->ReadFromBuffer( depth_frame_->data );
  programCL_->KernelSetArgument("flipImage", 0, *buf_ir);
  context_->RunOn1DRange(*programCL_, "flipImage", image_size, 0);
  buf_ir_sum->ReadFromBuffer( ir_frame_->data );
  if(has_listener)
  {
    if (listener_->onNewFrame(Frame::Ir, ir_frame_))
    {
      //std::cout << "yay, ir frame!" << std::endl;
    }

    if (listener_->onNewFrame(Frame::Depth, depth_frame_))
    {
      //std::cout << "yay, depth frame!" << std::endl;
    }
  }
}

bool FreenectDepthPacketProcessor::loadBufferFromResources_(const std::string &filename, unsigned char *buffer, const size_t n)
{
  size_t length = 0;
  const unsigned char *data = NULL;

  if(!loadResource(filename, &data, &length))
  {
    std::cerr << OUT_NAME("loadBufferFromResources") "failed to load resource: " << filename << std::endl;
    return false;
  }

  if(length != n)
  {
    std::cerr << OUT_NAME("loadBufferFromResources") "wrong size of resource: " << filename << std::endl;
    return false;
  }

  memcpy(buffer, data, length);
  return true;
}




class FreenectPacketPipeline : public PacketPipeline
{
public:
  FreenectPacketPipeline();
  virtual ~FreenectPacketPipeline();
  virtual void initialize();
  virtual PacketParser *getRgbPacketParser() const;
  virtual PacketParser *getIrPacketParser() const;

  virtual RgbPacketProcessor *getRgbPacketProcessor() const;
  virtual DepthPacketProcessor *getDepthPacketProcessor() const;
protected:
  RgbPacketStreamParser *rgb_parser_;
  DepthPacketStreamParser *depth_parser_;

  RgbPacketProcessor *rgb_processor_;
  BaseRgbPacketProcessor *async_rgb_processor_;
  DepthPacketProcessor *depth_processor_;
  BaseDepthPacketProcessor *async_depth_processor_;
  BIAS::clfContext ctx_;

};

FreenectPacketPipeline::FreenectPacketPipeline() {
  rgb_parser_ = 0;
  depth_parser_ = 0;
  rgb_processor_ = 0;
  async_rgb_processor_ = 0;
  depth_processor_ = 0;
  async_depth_processor_ = 0;
  initialize();
}

FreenectPacketPipeline::~FreenectPacketPipeline() {
  delete rgb_parser_;
  delete depth_parser_;
  delete rgb_processor_;
  delete async_rgb_processor_;
  delete depth_processor_;
  delete async_depth_processor_;
}

void FreenectPacketPipeline::initialize() {
  rgb_parser_ = new RgbPacketStreamParser();
  depth_parser_ = new DepthPacketStreamParser();

  rgb_processor_ = new FreenectJpegRgbPacketProcessor();
  depth_processor_ = new FreenectDepthPacketProcessor(&ctx_);

  async_rgb_processor_ = new AsyncPacketProcessor<RgbPacket>(rgb_processor_);
  async_depth_processor_ = new AsyncPacketProcessor<DepthPacket>(depth_processor_);

  rgb_parser_->setPacketProcessor(async_rgb_processor_);
  depth_parser_->setPacketProcessor(async_depth_processor_);
}

PacketPipeline::PacketParser* FreenectPacketPipeline::getRgbPacketParser() const {
  return rgb_parser_;
}

PacketPipeline::PacketParser* FreenectPacketPipeline::getIrPacketParser() const {
  return depth_parser_;
}

RgbPacketProcessor *FreenectPacketPipeline::getRgbPacketProcessor() const {
  return rgb_processor_;
}

DepthPacketProcessor *FreenectPacketPipeline::getDepthPacketProcessor() const {
  return depth_processor_;
}



///////////////////////VideoSource_Kinect2////////////////////////////////
/////////////////////////////////////////////////////////////////////////

VideoSource_Kinect2::VideoSource_Kinect2(){
  dev_ = NULL;
  pipeline_ = NULL;
  grabMode_ = Kinect2Mode::all;
  numDevices_ = 0;
}

VideoSource_Kinect2::~VideoSource_Kinect2(){
  delete pipeline_;
  delete dev_;
  delete freenect2_;
}

void VideoSource_Kinect2::SetMode(Kinect2Mode mode) {
  grabMode_ = mode;
}

bool VideoSource_Kinect2::onNewFrame(Frame::Type type, Frame *frame) {
  timespec ts;
  clock_gettime(CLOCK_REALTIME, &ts);
  ts.tv_sec += 2;
  if (type == Frame::Color) {
    int semOk = sem_timedwait(&colorbuffer_.sProd, &ts);
    if (semOk != 0) {
      std::cout << "dropping frames!" << std::endl;
      return true;
    }
    colorbuffer_.buffer[ colorbuffer_.curProd ].CopyIn_NoInit ( frame->data );
    colorbuffer_.curProd = (colorbuffer_.curProd+1) % buffersize_;
    sem_post(&colorbuffer_.sCons);
  }
  if (type == Frame::Depth) {
    int semOk = sem_timedwait(&depthbuffer_.sProd, &ts);
    if (semOk != 0) {
      std::cout << "dropping frames!" << std::endl;
      return true;
    }
    depthbuffer_.buffer[ depthbuffer_.curProd ].CopyIn_NoInit ( frame->data );
    depthbuffer_.curProd = (depthbuffer_.curProd+1) % buffersize_;
    sem_post(&depthbuffer_.sCons);
  }
  if (type == Frame::Ir) {
    int semOk = sem_timedwait(&irbuffer_.sProd, &ts);
    if (semOk != 0) {
      std::cout << "dropping frames!" << std::endl;
      return true;
    }
    irbuffer_.buffer[ irbuffer_.curProd ].CopyIn_NoInit ( frame->data );
    irbuffer_.curProd = (irbuffer_.curProd+1) % buffersize_;
    sem_post(&irbuffer_.sCons);
  }
  return true;
}

int VideoSource_Kinect2::OpenDevice(){
  return OpenDevice(0);
}

libfreenect2::Freenect2 *VideoSource_Kinect2::freenect2_ = NULL;

int VideoSource_Kinect2::OpenDevice(int device){
  pipeline_ = new FreenectPacketPipeline();
  if (freenect2_ == NULL) {
    freenect2_ = new Freenect2;
  }
  GetNumDevices(); // sets numDevices_
  if (numDevices_ <= device) {
    BIASERR("could not find device with index " << device);
    return -1;
  }
  dev_ = freenect2_->openDevice(device, pipeline_);
  return 0;
}

int VideoSource_Kinect2::CloseDevice(){
  BIASWARN("todo: CloseDevice() free resources here!");
//  delete pipeline_;
//  delete freenect2_;
//  delete dev_;
//  freenect2_ = NULL;
  dev_ = NULL;
  pipeline_ = NULL;
  return 0;
}


int VideoSource_Kinect2::PreGrab(){
  if (grabMode_ == Kinect2Mode::all || grabMode_ == Kinect2Mode::color) {
    dev_->setColorFrameListener(this);
  }
  if (grabMode_ == Kinect2Mode::all || grabMode_ == Kinect2Mode::depthAndIr) {
    dev_->setIrAndDepthFrameListener(this);
  }
  dev_->start();
  return 0;
}

int VideoSource_Kinect2::PostGrab(){
  dev_->stop();
  dev_->close();
  return 0;
}

int VideoSource_Kinect2::InitImage(ImageBase &Image){
  Image.Init(1920,1080,3,ImageBase::ST_unsignedchar);
  return 0;
}

int VideoSource_Kinect2::InitDepthImage(ImageBase &Image){
  Image.Init(512,424,1,ImageBase::ST_float);
  return 0;
}

int VideoSource_Kinect2::InitIrImage(ImageBase &Image){
  Image.Init(512,424,1,ImageBase::ST_float);
  return 0;
}

int VideoSource_Kinect2::InitIrFullImage(ImageBase &Image){
  Image.Init(512,424,9,ImageBase::ST_float);
  return 0;
}


int VideoSource_Kinect2::GrabSingle(Camera <unsigned char> &image){
  sem_wait(&colorbuffer_.sCons);
  image.CopyIn_NoInit( colorbuffer_.buffer[ colorbuffer_.curCons ].GetImageData() );
  colorbuffer_.curCons = (colorbuffer_.curCons+1) % buffersize_;
  sem_post(&colorbuffer_.sProd);
  return 0;
}


int VideoSource_Kinect2::GrabSingleIR(Camera <float> &image){
  sem_wait(&irbuffer_.sCons);
  image.CopyIn_NoInit( irbuffer_.buffer[ irbuffer_.curCons ].GetImageData() );
  irbuffer_.curCons = (irbuffer_.curCons+1) % buffersize_;
  sem_post(&irbuffer_.sProd);
  return 0;
}


int VideoSource_Kinect2::GrabSingleIRFull(Camera <float> &image){
  // todo
  return 0;
}

int VideoSource_Kinect2::GrabSingleDepth(Camera <float> &image){
  sem_wait(&depthbuffer_.sCons);
  image.CopyIn_NoInit( depthbuffer_.buffer[ depthbuffer_.curCons ].GetImageData() );
  depthbuffer_.curCons = (depthbuffer_.curCons+1) % buffersize_;
  sem_post(&depthbuffer_.sProd);
  return 0;
}


int VideoSource_Kinect2::GetNumDevices(){
  if (freenect2_ == NULL) {
    freenect2_ = new Freenect2;
  }
  numDevices_ = freenect2_->enumerateDevices();
  return numDevices_;
}

int VideoSource_Kinect2::GetCapabilities(VideoSourceCapabilities &caps){
  return 0;
}


int VideoSource_Kinect2::GetCapabilities(const char *device, VideoSourceCapabilities &caps){
  return 0;
}
