/* 
 This file is part of the BIAS library (Basic ImageAlgorithmS).

 Copyright (C) 2003, 2004    (see file CONTACTS for details)
 Multimediale Systeme der Informationsverarbeitung
 Institut fuer Informatik
 Christian-Albrechts-Universitaet Kiel


 BIAS is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 (at your option) any later version.

 BIAS is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with BIAS; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef VIDEOSOURCE_KINECT_BASE_HH_
#define VIDEOSOURCE_KINECT_BASE_HH_

#include <VideoSource/VideoSource_Base.hh>
#include <Image/Camera.hh>

namespace BIAS {

  /**
   * Use Kinect just like the ToF Cameras. There is no parameters to control the camera yet.
   * Functionally might be added later depending on evolution of libfreenect
   * @author fkellner 02/11
   */
  class BIASVideoSource_EXPORT VideoSource_Kinect_Base : public VideoSource {
    public:

      enum CaptureMode {
        DepthRay=0,
        DepthEuclidean,
        DepthRaw
      };
      
      /// Opens first device found, OpenDevice(0)
      virtual int OpenDevice() = 0;

      /// Opens device by id. On first call inits the context and looks for available cameras (i.e. scan bus)
      virtual int OpenDevice(int device) = 0;
      
      /// Close device, if no other kinect cameras are in use, context is deleted (i.e. close usb bus connection)
      virtual int CloseDevice() = 0;

      /// set appropriate operation mode for kinect and start grabbing thread (see callbacks in VideoSource_Kinect_Callback)
      virtual int PreGrab() = 0;
      /// stop grabbing thread 
      virtual int PostGrab() = 0;
      
      /// kinect color image will be 640x480, RGB. 
      virtual int InitImage(BIAS::ImageBase &Image) = 0;
      /// kinect depth image will be 640x480, float in Millimeters 
      virtual int InitDepthImage(BIAS::ImageBase &Image) = 0;

      /// grab single color image
      virtual int GrabSingle(BIAS::Camera <unsigned char> &image) = 0;
      /// depth according to formula  by Stephane Magnenat, see http://openkinect.org/wiki/Imaging_Information, todo: needs validation!
      virtual int GrabSingleDepth(BIAS::Camera <float> &image) = 0;
      
      virtual int CreateDepthLookupTable(double px, double py, double fx, double fy) = 0;
      
      virtual int SetVideoModeColor() = 0;
      virtual int SetVideoModeIR() = 0;
      
      virtual int SetDepthCaptureMode(CaptureMode mode) = 0;

      virtual int SetSwitchVideoModeAfterGrab() = 0;
      
      virtual int SetTilt(double degrees) = 0;
      virtual int SetLed(int mode) = 0;
      virtual int GetAccelerometer(double &tilt, double &x, double &y, double &z) = 0;

      virtual int GetNumDevices() = 0;

      virtual int GetCapabilities(VideoSourceCapabilities &caps) =0;
      virtual int GetCapabilities(const char *device, VideoSourceCapabilities &caps) =0;

      virtual int ToggleDepthStream() = 0;
  };

}

#endif /* VIDEOSOURCE_KINECT_BASE_HH_ */
