/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003-2009    (see file CONTACT for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifndef __BIASVIDEOSOURCE_SHM_HH__
#define __BIASVIDEOSOURCE_SHM_HH__



#include <iostream>
#include <sstream>
#include <VideoSource/VideoSource_Base.hh>
#include <Geometry/RMatrix.hh>

#ifdef WIN32
#  define WIN32_LEAN_AND_MEAN		// Exclude rarely-used stuff from Windows headers
#  include <windows.h>
#  ifdef BIAS_HAVE_WXWIDGETS
#    include <wx/msw/winundef.h>
#  endif
#endif


namespace BIAS{



  /**
     \class VideoSource_Shm
     \ingroup g_videosource
     \author evers
     \brief This class implements a video streaming client using shared memory to receive images from another application.
  */
  class BIASVideoSource_EXPORT VideoSource_Shm : public VideoSource {

  public: 
    struct ShmImageHeader {
      //transmission size
      unsigned int Size_;
      //images timestamp
      double Time_;
      //images rotation matrix
      double AxisX_, AxisY_, AxisZ_, Angle_;
      //images CVector
      double C1_,C2_,C3_;
    };

    /**
       @author Jan-Friso Evers
    **/
    VideoSource_Shm();
    ~VideoSource_Shm();

    int OpenDevice(const char *name);

    int OpenDevice(); 

    int PreGrab();
    int GrabSingle(BIAS::Camera <unsigned char> &image);
    int PostGrab();

    void SetSize(int w, int h, int bytesperpixel);

    /**
       In blocking-mode each call to GrabSingle waits for a NEW (evaluated 
       via images timestamp) image to arrive in shm. That means the program
       will block until a new image is available (!this is standard!).

       In nonblocking-mode allways the last received image will be returned!
       The program won't block in GrabSingle-calls.
     **/
    inline void SetBlockingMode(bool block) {bSync_ = block;}

    int CloseDevice();

  protected:
    int Resize_(int newsize);
#ifdef WIN32
    HANDLE Context_; // _hFile
    HANDLE Mapping_;//! header describing the memory layout
    long int Granularity_;
#else
    int Context_;
#endif

    bool IsCreator_;
    bool bSync_;
    std::string FileName_;
    void *Shm_;
    unsigned int DataLength_;
    unsigned int TotalLength_;
    std::stringstream *pImgBuffer_; // pointer needed to free mem
    ShmImageHeader *Header_;
    char *Data_;
    double LastTime_;

 protected:
    /*!\brief prohibit default function (move to 'public' if needed) */
  VideoSource_Shm(const VideoSource_Shm &source);
  /*!\brief prohibit default function (move to 'public' if needed) */
  VideoSource_Shm& operator =(const VideoSource_Shm &source);

 
  };


} // namespace MIP

#endif
