/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003-2009    (see file CONTACT for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include <string>
#include <sstream>
#include <iostream>
#include <sstream>

#include <Image/Camera.hh>
#include <VideoSource/VideoSource_ShmPMD.hh>

#ifdef WIN32
#  define WIN32_LEAN_AND_MEAN		// Exclude rarely-used stuff from Windows headers
#  include <windows.h>
#else
# include <sys/types.h>
# include <sys/mman.h>
#include <fcntl.h>
#endif


#include <Base/Common/BIASpragma.hh>


using namespace BIAS;
using namespace std;

VideoSource_ShmPMD::
VideoSource_ShmPMD() : VideoSource_Shm() 
{} 

VideoSource_ShmPMD::
~VideoSource_ShmPMD()
{} 


int VideoSource_ShmPMD::GrabSingle(Camera <unsigned char> &image) 
{
  bool gotit = false;
  do {

    if (Header_->Time_ >LastTime_) {
//   cout <<"VideoShmFeeder: Header is: "<<setw(20)<<setprecision(20)
//        <<Header_->Time_<<" , "<<Header_->Size_<<endl;
      if (Header_->Size_ >DataLength_) Resize_(Header_->Size_);
      
      pImgBuffer_->str("");
      if ((int)pImgBuffer_->rdbuf()->sputn(Data_,Header_->Size_) < (int) Header_->Size_) {
        BIASERR("Could not write data to streambuffer!");
        BIASABORT;
      }
      image.Release();
      *pImgBuffer_>>image;    
      image.InvalidateUID();
      image.SetTime(Header_->Time_);
      *pImgBuffer_>>DepthImg_;    
      DepthImg_.InvalidateUID();
      DepthImg_.SetTime(Header_->Time_);
      *pImgBuffer_>>ModCoeffImg_;    
      ModCoeffImg_.InvalidateUID();
      ModCoeffImg_.SetTime(Header_->Time_);
      LastTime_ =  Header_->Time_;
      gotit = true;

      //cout << "shm: cm: "<<boolalpha<< (image.GetColorModel()==ImageBase::CM_Bayer_RGGB)<<endl;
    }
    else biasusleep(1000);
  } while (!gotit);
  image.SetUID(UUID::GenerateUUID());
  DepthImg_.SetUID(image.GetUID());
  ModCoeffImg_.SetUID(image.GetUID());
  return 0;  
} 

