#ifndef __BIASVIDEOSOURCE_STEREO_hh_
#define __BIASVIDEOSOURCE_STEREO_hh_

#include <VideoSource/VideoSource_Base.hh>
#include <Image/Camera.hh>
#include <Base/Image/ImageBase.hh>

namespace BIAS {

  /** @class VideoSource_Stereo
      @ingroup g_videosource
      @brief bundles two video sources for stereo
      @author woelk 04/2006 */
  class BIASVideoSource_EXPORT VideoSource_Stereo
    : public VideoSource
  {
  public:
    VideoSource_Stereo(VideoSource *left, VideoSource *right);

    virtual ~VideoSource_Stereo();

    virtual int OpenDevice(const char *name);

    virtual int OpenDevice();

    virtual int PreGrab();
    /** Returns the left image and captures the right image from the camera */
    virtual int GrabSingle(Camera<unsigned char> &image);

    /** Returns the right image. Must be called after GrabSingle() */
    inline int GrabSingleRight(Camera<unsigned char> &image)
    { image = RightIm_; return 0; }

    virtual int PostGrab();

    virtual void SetSize(int w, int h, int bytesperpixel);

    virtual int CloseDevice();

    virtual int InitImage(ImageBase & img);

  protected:
    VideoSource *Left_;
    VideoSource *Right_;

    BIAS::Camera<unsigned char> LeftIm_, RightIm_;

    /*!\brief prohibit default function (move to 'public' if needed) */
    VideoSource_Stereo();
    /*!\brief prohibit default function (move to 'public' if needed) */
    VideoSource_Stereo(const VideoSource_Stereo &source);
    /*!\brief prohibit default function (move to 'public' if needed) */
    VideoSource_Stereo& operator =(const VideoSource_Stereo &source);
  };

}

#endif // _VideSource_Stereo_hh_
