#ifdef WIN32
# pragma warning (disable: 4251) // STL MAP DLL warning noise (JW)
#endif

#include "VideoTransmitter.hh"

using namespace BIAS;
using namespace std;

VideoTransmitter::VideoTransmitter() : server_(false)
{
  port_ = D_CS_DEFAULT_PORT;
  targetAddress_ = "";
 
  jpeg_ = -1; // no compression
  compressionColor_=true;
  useUDP_ = false;

  UDPServerThrottle_= 1;
  useUDP_ = false;

  msgSize_=0;
  Initialized_ = false;
}

VideoTransmitter::~VideoTransmitter()
{
}


int VideoTransmitter::Init(int port)  {
  if (port!=port_)
    port_=port;
  cout <<"Initializing VideoTransmitter on port:"<<port_<<endl;
  
  #ifdef BIAS_HAVE_LIBJPEG
  if (jpeg_ > 0) {
    if (Compressor_.Init() < 0) {
      BIASERR("Could not initialize JPEG-compression!!!");
      return -1;
    }
  }
#else //BIAS_HAVE_LIBJPEG
  if (jpeg_ > 0) {
    BIASERR("Compression mode not available without LIBJPEG!");
  }
#endif //BIAS_HAVE_LIBJPEG
  if (useUDP_){
  //  Transmitter_.Connect(TargetAddress_->c_str(),*SendPort_);
  } else {
    server_.WaitForConnections(port_);
  }
  
  server_.SetKeepAll(false);
  Initialized_ = true;
  return 0;
}


int VideoTransmitter::Process(BIAS::Image<unsigned char> & image)  {
  if (! Initialized_) {
    BIASERR("VideoTransmitter not initialized");
    return -1;
  }
  if (server_.GetConnections()<=0)
  {
    server_.WaitForConnections(port_);
    return 1;
  }


  //if (useUDP_) Transmitter_.SetThrottle(*UDPServerThrottle_);
  if (jpeg_ > 0) {
#ifdef BIAS_HAVE_LIBJPEG
    // compress here --->> add parameter for jpeg-quality ???
    if (compressionColor_) {
      if (image.GetColorModel() != ImageBase::CM_RGB) {
        Image<unsigned char> temp;
        ImageConvert::ToRGB(image, temp);
        Compressor_.Compress(temp,jpeg_);
      } else {
        Compressor_.Compress(image,jpeg_);
      }
    } else {
      if (image.GetColorModel() != ImageBase::CM_Grey) {
        Image<unsigned char> temp;
        ImageConvert::ToGrey(image, temp);
        Compressor_.Compress(temp,jpeg_);
      } else {
        Compressor_.Compress(image,jpeg_);
      }
    }
    void *data = NULL;
    long length;
    Compressor_.GetCompressedData(data,length);
    if (useUDP_){
      //      Transmitter_.Send(&Header_, (unsigned char*) data);
    } else {
      server_.SendMsg("JPEG_IMAGE", (char*)data, length);
    }
    msgSize_=length;
#else //BIAS_HAVE_LIBJPEG
    BIASERR("LIBJPEG not configured --> no compression available !!!");
    return -1;
#endif //BIAS_HAVE_LIBJPEG
  } else {
    if (useUDP_){
      // Transmitter_.Send(&Header_, SendImage_.GetImageData());
    } else {
      stringstream dataToSend;
      dataToSend<<image;
      msgSize_= dataToSend.str().length();
      server_.SendMsg("BIAS_IMAGE",(char*)&(dataToSend.str()[0]), msgSize_);
    }
  }

  return 0;
}


