#ifndef __BIAS_VideoTransmitter_HH__
#define __BIAS_VideoTransmitter_HH__

#include <iostream>
#include <sstream>
#include <string>

#include <bias_config.h>
// BIAS
#include <Base/Debug/Debug.hh>
#include <Base/Image/ImageIO.hh>
#include <Filter/Rescale.hh>
#include <Image/Camera.hh>
#include <VideoSource/VideoSource_Base.hh>
#include <NetworkComm/CScommServer.hh>


#ifdef  BIAS_HAVE_LIBJPEG
#include <Base/Image/CompressJpeg.hh>
#include <Base/Image/ImageConvert.hh>
#endif // BIAS_HAVE_LIBJPEG


namespace BIAS {

/** @class VideoTransmitter
    @ingroup g_videotransmission
    @ingroup network
    @brief class to send images to a client    
    uses TCP or UDPServer, 
    @author Ingo Schilller
    @date 2009
*/
class BIASVideoSource_EXPORT VideoTransmitter :public BIAS::Debug {
public:
  VideoTransmitter();
  ~VideoTransmitter();
  
  int Init(int port=D_CS_DEFAULT_PORT);

  int Process(BIAS::Image<unsigned char> & image);  

 void SetCompression(int compressionLevel=60){
  	jpeg_ = compressionLevel;  	
  }
  inline unsigned int GetMessageSize() { return msgSize_; }

protected:
  int port_;
  std::string targetAddress_;
 
  int jpeg_;
  double resizeFactor_;
  int UDPServerThrottle_;
  bool useUDP_;

  bool Initialized_;
  std::ostringstream SendBuffer_;
 
  unsigned int msgSize_;
  bool *CompressedMode_;
  bool compressionColor_;
  int *CompressionQuality_;

  CScommServer server_;

#ifdef BIAS_HAVE_LIBJPEG
  JpegHandler Compressor_;
#endif //BIAS_HAVE_LIBJPEG

private:
  // disabled copy-constructor
  VideoTransmitter(const VideoTransmitter &){};
   VideoTransmitter &operator=(const VideoTransmitter &) {return *this;};

}; // class VideoTransmitter

} // namespace BIAS


#endif

 
